import flags
import numpy
import typing
from typing import Any, Callable, ClassVar, overload

AGT_H: AnsatzGateType
AGT_RX: AnsatzGateType
AGT_RY: AnsatzGateType
AGT_RZ: AnsatzGateType
AGT_X: AnsatzGateType
ARBITRARY_ROTATION: SingleGateTransferType
BACKEND_CALC_ERROR: ErrorCode
BARRIER_GATE: GateType
CLUSTER_BASE: ErrorCode
CLUSTER_SIMULATE_CALC_ERR: ErrorCode
CNOT_GATE: LATEX_GATE_TYPE
CPHASE_GATE: GateType
CPU: BackendType
CPU_SINGLE_THREAD: BackendType
CP_GATE: GateType
CU_GATE: GateType
CZ_GATE: GateType
DATABASE_ERROR: ErrorCode
DOUBLE_BIT_GATE: DoubleGateTransferType
DOUBLE_CONTINUOUS: SingleGateTransferType
DOUBLE_DISCRETE: SingleGateTransferType
DOUBLE_GATE_INVALID: DoubleGateTransferType
ERR_BACKEND_CHIP_TASK_SOCKET_WRONG: ErrorCode
ERR_CHIP_OFFLINE: ErrorCode
ERR_EMPTY_PROG: ErrorCode
ERR_FIDELITY_MATRIX: ErrorCode
ERR_INVALID_URL: ErrorCode
ERR_MATE_GATE_CONFIG: ErrorCode
ERR_NOT_FOUND_APP_ID: ErrorCode
ERR_NOT_FOUND_TASK_ID: ErrorCode
ERR_OPERATOR_DB: ErrorCode
ERR_PARAMETER: ErrorCode
ERR_PARSER_SUB_TASK_RESULT: ErrorCode
ERR_PRE_ESTIMATE: ErrorCode
ERR_QCOMPILER_FAILED: ErrorCode
ERR_QPROG_LENGTH: ErrorCode
ERR_QST_PROG: ErrorCode
ERR_QUANTUM_CHIP_PROG: ErrorCode
ERR_QUBIT_SIZE: ErrorCode
ERR_QUBIT_TOPO: ErrorCode
ERR_QVM_INIT_FAILED: ErrorCode
ERR_REPEAT_MEASURE: ErrorCode
ERR_SCHEDULE_CHIP_TOPOLOGY_SUPPORTED: ErrorCode
ERR_SUB_GRAPH_OUT_OF_RANGE: ErrorCode
ERR_SYS_CALL_TIME_OUT: ErrorCode
ERR_TASK_BUF_OVERFLOW: ErrorCode
ERR_TASK_CONFIG: ErrorCode
ERR_TASK_STATUS_BUF_OVERFLOW: ErrorCode
ERR_TASK_TERMINATED: ErrorCode
ERR_TCP_INIT_FATLT: ErrorCode
ERR_TCP_SERVER_HALT: ErrorCode
ERR_UNKNOW_TASK_TYPE: ErrorCode
ERR_UNSUPPORT_BACKEND_TYPE: ErrorCode
EXCEED_MAX_CLOCK: ErrorCode
EXCEED_MAX_QUBIT: ErrorCode
GATE_NOP: GateType
GATE_UNDEFINED: GateType
GD_DIRECTION: UpdateMode
GD_VALUE: UpdateMode
GENERAL_GATE: LATEX_GATE_TYPE
GPU: BackendType
GRADIENT: OptimizerType
HADAMARD_GATE: GateType
ISWAP_GATE: GateType
ISWAP_THETA_GATE: GateType
I_GATE: GateType
JSON_FIELD_ERROR: ErrorCode
LINEAR: ComplexVertexSplitMethod
METHOD_UNDEFINED: ComplexVertexSplitMethod
MPS: BackendType
MS_GATE: GateType
NELDER_MEAD: OptimizerType
NOISE: BackendType
NO_ERROR_FOUND: ErrorCode
ORACLE_GATE: GateType
ORIGINIR_ERROR: ErrorCode
P00_GATE: GateType
P0_GATE: GateType
P11_GATE: GateType
P1_GATE: GateType
PAULI_X_GATE: GateType
PAULI_Y_GATE: GateType
PAULI_Z_GATE: GateType
PEC: em_method
POWELL: OptimizerType
P_GATE: GateType
READ_OUT: em_method
RING: ComplexVertexSplitMethod
RPHI_GATE: GateType
RXX_GATE: GateType
RX_GATE: GateType
RYY_GATE: GateType
RY_GATE: GateType
RZX_GATE: GateType
RZZ_GATE: GateType
RZ_GATE: GateType
SINGLE_CONTINUOUS_DISCRETE: SingleGateTransferType
SINGLE_GATE_INVALID: SingleGateTransferType
SQISWAP_GATE: GateType
SWAP_GATE: LATEX_GATE_TYPE
S_GATE: GateType
Simulation: ChipID
TOFFOLI_GATE: GateType
TWO_QUBIT_GATE: GateType
T_GATE: GateType
U1_GATE: GateType
U2_GATE: GateType
U3_GATE: GateType
U4_GATE: GateType
UNDEFINED_ERROR: ErrorCode
WUYUAN_1: ChipID
WUYUAN_2: ChipID
WUYUAN_3: ChipID
X_HALF_PI: GateType
Y_HALF_PI: GateType
ZNE: em_method
Z_HALF_PI: GateType
origin_72: real_chip_type
origin_wuyuan_d3: real_chip_type
origin_wuyuan_d4: real_chip_type
origin_wuyuan_d5: real_chip_type

class AbstractOptimizer:
    def __init__(self, *args, **kwargs) -> None:
        """Initialize self.  See help(type(self)) for accurate signature."""
    def exec(self) -> None:
        """exec(self: pyQPanda.AbstractOptimizer) -> None

        Execute the optimization process.

        Args:
             None: This method takes no parameters.

        Returns:
             result: The result of the optimization process.
        """
    def getResult(self, *args, **kwargs):
        """getResult(self: pyQPanda.AbstractOptimizer) -> QPanda::QOptimizationResult

        Retrieve the result of the last optimization.

        Args:
             None: This method takes no parameters.

        Returns:
             result: The result of the last optimization.
        """
    def registerFunc(self, arg0: Callable[[list[float], list[float], int, int], tuple[str, float]], arg1: list[float]) -> None:
        """registerFunc(self: pyQPanda.AbstractOptimizer, arg0: Callable[[List[float], List[float], int, int], Tuple[str, float]], arg1: List[float]) -> None

        Register an optimization function.

        Args:
             func: The optimization function to be registered.

        Returns:
             None
        """
    def setAdaptive(self, arg0: bool) -> None:
        """setAdaptive(self: pyQPanda.AbstractOptimizer, arg0: bool) -> None

        Set whether the optimizer should use adaptive methods.

        Args:
             adaptive: A boolean indicating whether to enable adaptive optimization.

        Returns:
             None
        """
    def setCacheFile(self, arg0: str) -> None:
        """setCacheFile(self: pyQPanda.AbstractOptimizer, arg0: str) -> None

        Set the path for the cache file used in optimization.

        Args:
             cache_file: A string representing the path to the cache file.

        Returns:
             None

        """
    def setDisp(self, arg0: bool) -> None:
        """setDisp(self: pyQPanda.AbstractOptimizer, arg0: bool) -> None

        Set the display flag for the optimizer.

        Args:
             disp: A boolean indicating whether to display optimization progress.

        Returns:
             None
        """
    def setFatol(self, arg0: float) -> None:
        """setFatol(self: pyQPanda.AbstractOptimizer, arg0: float) -> None

        Set the function absolute tolerance for optimization.

        Args:
             fatol: The function absolute tolerance value to be set.

        Returns:
             None
        """
    def setMaxFCalls(self, arg0: int) -> None:
        """setMaxFCalls(self: pyQPanda.AbstractOptimizer, arg0: int) -> None

        Set the maximum number of function calls allowed during optimization.

        Args:
             max_calls: The maximum number of function calls to be set.

        Returns:
             None
        """
    def setMaxIter(self, arg0: int) -> None:
        """setMaxIter(self: pyQPanda.AbstractOptimizer, arg0: int) -> None

        Set the maximum number of iterations allowed during optimization.

        Args:
             max_iter: The maximum number of iterations to be set.

        Returns:
             None
        """
    def setRestoreFromCacheFile(self, arg0: bool) -> None:
        """setRestoreFromCacheFile(self: pyQPanda.AbstractOptimizer, arg0: bool) -> None

        Set whether to restore the optimization state from a cache file.
        Args:

             cache_file: A string representing the path to the cache file.

        Returns:
             None

        """
    def setXatol(self, arg0: float) -> None:
        """setXatol(self: pyQPanda.AbstractOptimizer, arg0: float) -> None

        Set the absolute tolerance for optimization.

        Args:
             atol: The absolute tolerance value to be set.

        Returns:
             None
        """

class AdaGradOptimizer:
    def __init__(self, arg0: var, arg1: float, arg2: float, arg3: float) -> None:
        """__init__(self: pyQPanda.AdaGradOptimizer, arg0: pyQPanda.var, arg1: float, arg2: float, arg3: float) -> None"""
    def get_loss(self) -> float:
        """get_loss(self: pyQPanda.AdaGradOptimizer) -> float"""
    def get_variables(self) -> list[var]:
        """get_variables(self: pyQPanda.AdaGradOptimizer) -> List[pyQPanda.var]"""
    def minimize(self, arg0: float, arg1: float, arg2: float) -> Optimizer:
        """minimize(self: pyQPanda.var, arg0: float, arg1: float, arg2: float) -> pyQPanda.Optimizer"""
    def run(self, arg0: list[var], arg1: int) -> bool:
        """run(self: pyQPanda.AdaGradOptimizer, arg0: List[pyQPanda.var], arg1: int) -> bool"""

class AdamOptimizer:
    def __init__(self, arg0: var, arg1: float, arg2: float, arg3: float, arg4: float) -> None:
        """__init__(self: pyQPanda.AdamOptimizer, arg0: pyQPanda.var, arg1: float, arg2: float, arg3: float, arg4: float) -> None"""
    def get_loss(self) -> float:
        """get_loss(self: pyQPanda.AdamOptimizer) -> float"""
    def get_variables(self) -> list[var]:
        """get_variables(self: pyQPanda.AdamOptimizer) -> List[pyQPanda.var]"""
    def minimize(self, arg0: float, arg1: float, arg2: float, arg3: float) -> Optimizer:
        """minimize(self: pyQPanda.var, arg0: float, arg1: float, arg2: float, arg3: float) -> pyQPanda.Optimizer"""
    def run(self, arg0: list[var], arg1: int) -> bool:
        """run(self: pyQPanda.AdamOptimizer, arg0: List[pyQPanda.var], arg1: int) -> bool"""

class Ansatz:
    @overload
    def __init__(self) -> None:
        """__init__(*args, **kwargs)
        Overloaded function.

        1. __init__(self: pyQPanda.Ansatz) -> None

        2. __init__(self: pyQPanda.Ansatz, arg0: pyQPanda.QGate) -> None

        3. __init__(self: pyQPanda.Ansatz, arg0: pyQPanda.AnsatzGate) -> None

        4. __init__(self: pyQPanda.Ansatz, ansatz: List[pyQPanda.AnsatzGate], thetas: List[float] = []) -> None

        5. __init__(self: pyQPanda.Ansatz, ansatz_circuit: pyQPanda.Ansatz, thetas: List[float] = []) -> None

        6. __init__(self: pyQPanda.Ansatz, circuit: pyQPanda.QCircuit, thetas: List[float] = []) -> None
        """
    @overload
    def __init__(self, arg0: QGate) -> None:
        """__init__(*args, **kwargs)
        Overloaded function.

        1. __init__(self: pyQPanda.Ansatz) -> None

        2. __init__(self: pyQPanda.Ansatz, arg0: pyQPanda.QGate) -> None

        3. __init__(self: pyQPanda.Ansatz, arg0: pyQPanda.AnsatzGate) -> None

        4. __init__(self: pyQPanda.Ansatz, ansatz: List[pyQPanda.AnsatzGate], thetas: List[float] = []) -> None

        5. __init__(self: pyQPanda.Ansatz, ansatz_circuit: pyQPanda.Ansatz, thetas: List[float] = []) -> None

        6. __init__(self: pyQPanda.Ansatz, circuit: pyQPanda.QCircuit, thetas: List[float] = []) -> None
        """
    @overload
    def __init__(self, arg0: AnsatzGate) -> None:
        """__init__(*args, **kwargs)
        Overloaded function.

        1. __init__(self: pyQPanda.Ansatz) -> None

        2. __init__(self: pyQPanda.Ansatz, arg0: pyQPanda.QGate) -> None

        3. __init__(self: pyQPanda.Ansatz, arg0: pyQPanda.AnsatzGate) -> None

        4. __init__(self: pyQPanda.Ansatz, ansatz: List[pyQPanda.AnsatzGate], thetas: List[float] = []) -> None

        5. __init__(self: pyQPanda.Ansatz, ansatz_circuit: pyQPanda.Ansatz, thetas: List[float] = []) -> None

        6. __init__(self: pyQPanda.Ansatz, circuit: pyQPanda.QCircuit, thetas: List[float] = []) -> None
        """
    @overload
    def __init__(self, ansatz: list[AnsatzGate], thetas: list[float] = ...) -> None:
        """__init__(*args, **kwargs)
        Overloaded function.

        1. __init__(self: pyQPanda.Ansatz) -> None

        2. __init__(self: pyQPanda.Ansatz, arg0: pyQPanda.QGate) -> None

        3. __init__(self: pyQPanda.Ansatz, arg0: pyQPanda.AnsatzGate) -> None

        4. __init__(self: pyQPanda.Ansatz, ansatz: List[pyQPanda.AnsatzGate], thetas: List[float] = []) -> None

        5. __init__(self: pyQPanda.Ansatz, ansatz_circuit: pyQPanda.Ansatz, thetas: List[float] = []) -> None

        6. __init__(self: pyQPanda.Ansatz, circuit: pyQPanda.QCircuit, thetas: List[float] = []) -> None
        """
    @overload
    def __init__(self, ansatz_circuit: Ansatz, thetas: list[float] = ...) -> None:
        """__init__(*args, **kwargs)
        Overloaded function.

        1. __init__(self: pyQPanda.Ansatz) -> None

        2. __init__(self: pyQPanda.Ansatz, arg0: pyQPanda.QGate) -> None

        3. __init__(self: pyQPanda.Ansatz, arg0: pyQPanda.AnsatzGate) -> None

        4. __init__(self: pyQPanda.Ansatz, ansatz: List[pyQPanda.AnsatzGate], thetas: List[float] = []) -> None

        5. __init__(self: pyQPanda.Ansatz, ansatz_circuit: pyQPanda.Ansatz, thetas: List[float] = []) -> None

        6. __init__(self: pyQPanda.Ansatz, circuit: pyQPanda.QCircuit, thetas: List[float] = []) -> None
        """
    @overload
    def __init__(self, circuit: QCircuit, thetas: list[float] = ...) -> None:
        """__init__(*args, **kwargs)
        Overloaded function.

        1. __init__(self: pyQPanda.Ansatz) -> None

        2. __init__(self: pyQPanda.Ansatz, arg0: pyQPanda.QGate) -> None

        3. __init__(self: pyQPanda.Ansatz, arg0: pyQPanda.AnsatzGate) -> None

        4. __init__(self: pyQPanda.Ansatz, ansatz: List[pyQPanda.AnsatzGate], thetas: List[float] = []) -> None

        5. __init__(self: pyQPanda.Ansatz, ansatz_circuit: pyQPanda.Ansatz, thetas: List[float] = []) -> None

        6. __init__(self: pyQPanda.Ansatz, circuit: pyQPanda.QCircuit, thetas: List[float] = []) -> None
        """
    def get_ansatz_list(self) -> list[AnsatzGate]:
        """get_ansatz_list(self: pyQPanda.Ansatz) -> List[pyQPanda.AnsatzGate]"""
    def get_thetas_list(self) -> list[float]:
        """get_thetas_list(self: pyQPanda.Ansatz) -> List[float]"""
    @overload
    def insert(self, gate: QGate) -> None:
        """insert(*args, **kwargs)
        Overloaded function.

        1. insert(self: pyQPanda.Ansatz, gate: pyQPanda.QGate) -> None

        2. insert(self: pyQPanda.Ansatz, gate: pyQPanda.AnsatzGate) -> None

        3. insert(self: pyQPanda.Ansatz, gate: List[pyQPanda.AnsatzGate]) -> None

        4. insert(self: pyQPanda.Ansatz, gate: pyQPanda.QCircuit) -> None

        5. insert(self: pyQPanda.Ansatz, gate: pyQPanda.Ansatz, thetas: List[float] = []) -> None
        """
    @overload
    def insert(self, gate: AnsatzGate) -> None:
        """insert(*args, **kwargs)
        Overloaded function.

        1. insert(self: pyQPanda.Ansatz, gate: pyQPanda.QGate) -> None

        2. insert(self: pyQPanda.Ansatz, gate: pyQPanda.AnsatzGate) -> None

        3. insert(self: pyQPanda.Ansatz, gate: List[pyQPanda.AnsatzGate]) -> None

        4. insert(self: pyQPanda.Ansatz, gate: pyQPanda.QCircuit) -> None

        5. insert(self: pyQPanda.Ansatz, gate: pyQPanda.Ansatz, thetas: List[float] = []) -> None
        """
    @overload
    def insert(self, gate: list[AnsatzGate]) -> None:
        """insert(*args, **kwargs)
        Overloaded function.

        1. insert(self: pyQPanda.Ansatz, gate: pyQPanda.QGate) -> None

        2. insert(self: pyQPanda.Ansatz, gate: pyQPanda.AnsatzGate) -> None

        3. insert(self: pyQPanda.Ansatz, gate: List[pyQPanda.AnsatzGate]) -> None

        4. insert(self: pyQPanda.Ansatz, gate: pyQPanda.QCircuit) -> None

        5. insert(self: pyQPanda.Ansatz, gate: pyQPanda.Ansatz, thetas: List[float] = []) -> None
        """
    @overload
    def insert(self, gate: QCircuit) -> None:
        """insert(*args, **kwargs)
        Overloaded function.

        1. insert(self: pyQPanda.Ansatz, gate: pyQPanda.QGate) -> None

        2. insert(self: pyQPanda.Ansatz, gate: pyQPanda.AnsatzGate) -> None

        3. insert(self: pyQPanda.Ansatz, gate: List[pyQPanda.AnsatzGate]) -> None

        4. insert(self: pyQPanda.Ansatz, gate: pyQPanda.QCircuit) -> None

        5. insert(self: pyQPanda.Ansatz, gate: pyQPanda.Ansatz, thetas: List[float] = []) -> None
        """
    @overload
    def insert(self, gate: Ansatz, thetas: list[float] = ...) -> None:
        """insert(*args, **kwargs)
        Overloaded function.

        1. insert(self: pyQPanda.Ansatz, gate: pyQPanda.QGate) -> None

        2. insert(self: pyQPanda.Ansatz, gate: pyQPanda.AnsatzGate) -> None

        3. insert(self: pyQPanda.Ansatz, gate: List[pyQPanda.AnsatzGate]) -> None

        4. insert(self: pyQPanda.Ansatz, gate: pyQPanda.QCircuit) -> None

        5. insert(self: pyQPanda.Ansatz, gate: pyQPanda.Ansatz, thetas: List[float] = []) -> None
        """
    def set_thetas(self, thetas: list[float]) -> None:
        """set_thetas(self: pyQPanda.Ansatz, thetas: List[float]) -> None"""
    @overload
    def __lshift__(self, arg0: QGate) -> Ansatz:
        """__lshift__(*args, **kwargs)
        Overloaded function.

        1. __lshift__(self: pyQPanda.Ansatz, arg0: pyQPanda.QGate) -> pyQPanda.Ansatz

        2. __lshift__(self: pyQPanda.Ansatz, arg0: List[pyQPanda.AnsatzGate]) -> pyQPanda.Ansatz

        3. __lshift__(self: pyQPanda.Ansatz, arg0: pyQPanda.QCircuit) -> pyQPanda.Ansatz

        4. __lshift__(self: pyQPanda.Ansatz, arg0: pyQPanda.AnsatzGate) -> pyQPanda.Ansatz
        """
    @overload
    def __lshift__(self, arg0: list[AnsatzGate]) -> Ansatz:
        """__lshift__(*args, **kwargs)
        Overloaded function.

        1. __lshift__(self: pyQPanda.Ansatz, arg0: pyQPanda.QGate) -> pyQPanda.Ansatz

        2. __lshift__(self: pyQPanda.Ansatz, arg0: List[pyQPanda.AnsatzGate]) -> pyQPanda.Ansatz

        3. __lshift__(self: pyQPanda.Ansatz, arg0: pyQPanda.QCircuit) -> pyQPanda.Ansatz

        4. __lshift__(self: pyQPanda.Ansatz, arg0: pyQPanda.AnsatzGate) -> pyQPanda.Ansatz
        """
    @overload
    def __lshift__(self, arg0: QCircuit) -> Ansatz:
        """__lshift__(*args, **kwargs)
        Overloaded function.

        1. __lshift__(self: pyQPanda.Ansatz, arg0: pyQPanda.QGate) -> pyQPanda.Ansatz

        2. __lshift__(self: pyQPanda.Ansatz, arg0: List[pyQPanda.AnsatzGate]) -> pyQPanda.Ansatz

        3. __lshift__(self: pyQPanda.Ansatz, arg0: pyQPanda.QCircuit) -> pyQPanda.Ansatz

        4. __lshift__(self: pyQPanda.Ansatz, arg0: pyQPanda.AnsatzGate) -> pyQPanda.Ansatz
        """
    @overload
    def __lshift__(self, arg0: AnsatzGate) -> Ansatz:
        """__lshift__(*args, **kwargs)
        Overloaded function.

        1. __lshift__(self: pyQPanda.Ansatz, arg0: pyQPanda.QGate) -> pyQPanda.Ansatz

        2. __lshift__(self: pyQPanda.Ansatz, arg0: List[pyQPanda.AnsatzGate]) -> pyQPanda.Ansatz

        3. __lshift__(self: pyQPanda.Ansatz, arg0: pyQPanda.QCircuit) -> pyQPanda.Ansatz

        4. __lshift__(self: pyQPanda.Ansatz, arg0: pyQPanda.AnsatzGate) -> pyQPanda.Ansatz
        """

class AnsatzGate:
    control: int
    target: int
    theta: float
    type: AnsatzGateType
    @overload
    def __init__(self, arg0: AnsatzGateType, arg1: int) -> None:
        """__init__(*args, **kwargs)
        Overloaded function.

        1. __init__(self: pyQPanda.AnsatzGate, arg0: pyQPanda.AnsatzGateType, arg1: int) -> None

        2. __init__(self: pyQPanda.AnsatzGate, arg0: pyQPanda.AnsatzGateType, arg1: int, arg2: float) -> None

        3. __init__(self: pyQPanda.AnsatzGate, arg0: pyQPanda.AnsatzGateType, arg1: int, arg2: float, arg3: int) -> None
        """
    @overload
    def __init__(self, arg0: AnsatzGateType, arg1: int, arg2: float) -> None:
        """__init__(*args, **kwargs)
        Overloaded function.

        1. __init__(self: pyQPanda.AnsatzGate, arg0: pyQPanda.AnsatzGateType, arg1: int) -> None

        2. __init__(self: pyQPanda.AnsatzGate, arg0: pyQPanda.AnsatzGateType, arg1: int, arg2: float) -> None

        3. __init__(self: pyQPanda.AnsatzGate, arg0: pyQPanda.AnsatzGateType, arg1: int, arg2: float, arg3: int) -> None
        """
    @overload
    def __init__(self, arg0: AnsatzGateType, arg1: int, arg2: float, arg3: int) -> None:
        """__init__(*args, **kwargs)
        Overloaded function.

        1. __init__(self: pyQPanda.AnsatzGate, arg0: pyQPanda.AnsatzGateType, arg1: int) -> None

        2. __init__(self: pyQPanda.AnsatzGate, arg0: pyQPanda.AnsatzGateType, arg1: int, arg2: float) -> None

        3. __init__(self: pyQPanda.AnsatzGate, arg0: pyQPanda.AnsatzGateType, arg1: int, arg2: float, arg3: int) -> None
        """

class AnsatzGateType:
    __members__: ClassVar[dict] = ...  # read-only
    AGT_H: ClassVar[AnsatzGateType] = ...
    AGT_RX: ClassVar[AnsatzGateType] = ...
    AGT_RY: ClassVar[AnsatzGateType] = ...
    AGT_RZ: ClassVar[AnsatzGateType] = ...
    AGT_X: ClassVar[AnsatzGateType] = ...
    __entries: ClassVar[dict] = ...
    def __init__(self, value: int) -> None:
        """__init__(self: pyQPanda.AnsatzGateType, value: int) -> None"""
    def __eq__(self, other: object) -> bool:
        """__eq__(self: object, other: object) -> bool"""
    def __hash__(self) -> int:
        """__hash__(self: object) -> int"""
    def __index__(self) -> int:
        """__index__(self: pyQPanda.AnsatzGateType) -> int"""
    def __int__(self) -> int:
        """__int__(self: pyQPanda.AnsatzGateType) -> int"""
    def __ne__(self, other: object) -> bool:
        """__ne__(self: object, other: object) -> bool"""
    @property
    def name(self) -> str: ...
    @property
    def value(self) -> int: ...

class BackendType:
    __members__: ClassVar[dict] = ...  # read-only
    CPU: ClassVar[BackendType] = ...
    CPU_SINGLE_THREAD: ClassVar[BackendType] = ...
    GPU: ClassVar[BackendType] = ...
    MPS: ClassVar[BackendType] = ...
    NOISE: ClassVar[BackendType] = ...
    __entries: ClassVar[dict] = ...
    def __init__(self, value: int) -> None:
        """__init__(self: pyQPanda.BackendType, value: int) -> None"""
    def __eq__(self, other: object) -> bool:
        """__eq__(self: object, other: object) -> bool"""
    def __hash__(self) -> int:
        """__hash__(self: object) -> int"""
    def __index__(self) -> int:
        """__index__(self: pyQPanda.BackendType) -> int"""
    def __int__(self) -> int:
        """__int__(self: pyQPanda.BackendType) -> int"""
    def __ne__(self, other: object) -> bool:
        """__ne__(self: object, other: object) -> bool"""
    @property
    def name(self) -> str: ...
    @property
    def value(self) -> int: ...

class CBit:
    def __init__(self, *args, **kwargs) -> None:
        """Initialize self.  See help(type(self)) for accurate signature."""
    def getName(self) -> str:
        """getName(self: pyQPanda.CBit) -> str

        Retrieve the name of the classical bit.

        Args:
             None

        Returns:
             The name of the CBit as a string.

        """

class CPUQVM(QuantumMachine):
    def __init__(self) -> None:
        """__init__(self: pyQPanda.CPUQVM) -> None"""
    def get_prob_dict(self, qubit_list: QVec, select_max: int = ...) -> dict[str, float]:
        """get_prob_dict(self: pyQPanda.CPUQVM, qubit_list: pyQPanda.QVec, select_max: int = -1) -> Dict[str, float]

        Get a dictionary of probabilities for the specified qubits.

        Args:
             qubit_list: List of qubits to measure.

             select_max: int, optional, maximum number of entries to return (default: -1).

        Returns:
             Dictionary of probabilities as a reference.

        """
    def get_prob_list(self, qubit_list: QVec, select_max: int = ...) -> list[float]:
        """get_prob_list(self: pyQPanda.CPUQVM, qubit_list: pyQPanda.QVec, select_max: int = -1) -> List[float]

        Get a list of probabilities for the specified qubits.

        Args:
             qubit_list: List of qubits to measure.

             select_max: int, optional, maximum number of probabilities to return (default: -1).

        Returns:
             List of probabilities as a reference.

        """
    def get_prob_tuple_list(self, qubit_list: QVec, select_max: int = ...) -> list[tuple[int, float]]:
        """get_prob_tuple_list(self: pyQPanda.CPUQVM, qubit_list: pyQPanda.QVec, select_max: int = -1) -> List[Tuple[int, float]]

        Get a list of probability tuples for the specified qubits.

        Args:
             qubit_list: List of qubits to measure.

             select_max: int, optional, maximum number of tuples to return (default: -1).

        Returns:
             List of probability tuples as a reference.

        """
    @overload
    def init_qvm(self, arg0: bool) -> None:
        """init_qvm(*args, **kwargs)
        Overloaded function.

        1. init_qvm(self: pyQPanda.CPUQVM, arg0: bool) -> None

        2. init_qvm(self: pyQPanda.CPUQVM) -> None
        """
    @overload
    def init_qvm(self) -> None:
        """init_qvm(*args, **kwargs)
        Overloaded function.

        1. init_qvm(self: pyQPanda.CPUQVM, arg0: bool) -> None

        2. init_qvm(self: pyQPanda.CPUQVM) -> None
        """
    def pmeasure(self, qubit_list: QVec, select_max: int = ...) -> list[tuple[int, float]]:
        """pmeasure(self: pyQPanda.CPUQVM, qubit_list: pyQPanda.QVec, select_max: int = -1) -> List[Tuple[int, float]]

        Get the probability distribution over qubits.

        Args:
             qubit_list: List of qubits to measure.

             select_max: int, optional, maximum number of results to select (default: -1).

        Returns:
             Probability distribution as a reference.

        """
    def pmeasure_no_index(self, qubit_list: QVec) -> list[float]:
        """pmeasure_no_index(self: pyQPanda.CPUQVM, qubit_list: pyQPanda.QVec) -> List[float]

        Get the probability distribution over qubits without index.

        Args:
             qubit_list: List of qubits to measure.

        Returns:
             Probability distribution as a reference.

        """
    @overload
    def prob_run_dict(self, program: QProg, qubit_list: QVec, select_max: int = ...) -> dict[str, float]:
        """prob_run_dict(*args, **kwargs)
        Overloaded function.

        1. prob_run_dict(self: pyQPanda.CPUQVM, program: pyQPanda.QProg, qubit_list: pyQPanda.QVec, select_max: int = -1) -> Dict[str, float]

        Execute a quantum program and retrieve a dictionary of probabilities for the specified qubits.

        Args:
             program: The quantum program to execute.

             qubit_list: List of qubits to measure.

             select_max: int, optional, maximum number of entries in the dictionary to return (default: -1).

        Returns:
             Dictionary of probabilities.


        2. prob_run_dict(self: pyQPanda.CPUQVM, program: pyQPanda.QProg, qubit_addr_list: List[int], select_max: int = -1) -> Dict[str, float]

        Execute a quantum program and retrieve a dictionary of probabilities using qubit addresses.

        Args:
             program: The quantum program to execute.

             qubit_addr_list: List of qubit addresses to measure.

             select_max: int, optional, maximum number of entries in the dictionary to return (default: -1).

        Returns:
             Dictionary of probabilities.

        """
    @overload
    def prob_run_dict(self, program: QProg, qubit_addr_list: list[int], select_max: int = ...) -> dict[str, float]:
        """prob_run_dict(*args, **kwargs)
        Overloaded function.

        1. prob_run_dict(self: pyQPanda.CPUQVM, program: pyQPanda.QProg, qubit_list: pyQPanda.QVec, select_max: int = -1) -> Dict[str, float]

        Execute a quantum program and retrieve a dictionary of probabilities for the specified qubits.

        Args:
             program: The quantum program to execute.

             qubit_list: List of qubits to measure.

             select_max: int, optional, maximum number of entries in the dictionary to return (default: -1).

        Returns:
             Dictionary of probabilities.


        2. prob_run_dict(self: pyQPanda.CPUQVM, program: pyQPanda.QProg, qubit_addr_list: List[int], select_max: int = -1) -> Dict[str, float]

        Execute a quantum program and retrieve a dictionary of probabilities using qubit addresses.

        Args:
             program: The quantum program to execute.

             qubit_addr_list: List of qubit addresses to measure.

             select_max: int, optional, maximum number of entries in the dictionary to return (default: -1).

        Returns:
             Dictionary of probabilities.

        """
    @overload
    def prob_run_list(self, program: QProg, qubit_list: QVec, select_max: int = ...) -> list[float]:
        """prob_run_list(*args, **kwargs)
        Overloaded function.

        1. prob_run_list(self: pyQPanda.CPUQVM, program: pyQPanda.QProg, qubit_list: pyQPanda.QVec, select_max: int = -1) -> List[float]

        Execute a quantum program and retrieve a list of probabilities for the specified qubits.

        Args:
             program: The quantum program to execute.

             qubit_list: List of qubits to measure.

             select_max: int, optional, maximum number of probabilities to return (default: -1).

        Returns:
             List of probabilities.


        2. prob_run_list(self: pyQPanda.CPUQVM, program: pyQPanda.QProg, qubit_addr_list: List[int], select_max: int = -1) -> List[float]

        Execute a quantum program and retrieve a list of probabilities using qubit addresses.

        Args:
             program: The quantum program to execute.

             qubit_addr_list: List of qubit addresses to measure.

             select_max: int, optional, maximum number of probabilities to return (default: -1).

        Returns:
             List of probabilities.

        """
    @overload
    def prob_run_list(self, program: QProg, qubit_addr_list: list[int], select_max: int = ...) -> list[float]:
        """prob_run_list(*args, **kwargs)
        Overloaded function.

        1. prob_run_list(self: pyQPanda.CPUQVM, program: pyQPanda.QProg, qubit_list: pyQPanda.QVec, select_max: int = -1) -> List[float]

        Execute a quantum program and retrieve a list of probabilities for the specified qubits.

        Args:
             program: The quantum program to execute.

             qubit_list: List of qubits to measure.

             select_max: int, optional, maximum number of probabilities to return (default: -1).

        Returns:
             List of probabilities.


        2. prob_run_list(self: pyQPanda.CPUQVM, program: pyQPanda.QProg, qubit_addr_list: List[int], select_max: int = -1) -> List[float]

        Execute a quantum program and retrieve a list of probabilities using qubit addresses.

        Args:
             program: The quantum program to execute.

             qubit_addr_list: List of qubit addresses to measure.

             select_max: int, optional, maximum number of probabilities to return (default: -1).

        Returns:
             List of probabilities.

        """
    @overload
    def prob_run_tuple_list(self, program: QProg, qubit_list: QVec, select_max: int = ...) -> list[tuple[int, float]]:
        """prob_run_tuple_list(*args, **kwargs)
        Overloaded function.

        1. prob_run_tuple_list(self: pyQPanda.CPUQVM, program: pyQPanda.QProg, qubit_list: pyQPanda.QVec, select_max: int = -1) -> List[Tuple[int, float]]

        Execute a quantum program and get a list of probability tuples.

        Args:
             program: The quantum program to execute.

             qubit_list: List of qubits to measure.

             select_max: int, optional, maximum number of tuples to return (default: -1).

        Returns:
             List of probability tuples.


        2. prob_run_tuple_list(self: pyQPanda.CPUQVM, program: pyQPanda.QProg, qubit_addr_list: List[int], select_max: int = -1) -> List[Tuple[int, float]]

        Execute a quantum program and get a list of probability tuples using qubit addresses.

        Args:
             program: The quantum program to execute.

             qubit_addr_list: List of qubit addresses to measure.

             select_max: int, optional, maximum number of tuples to return (default: -1).

        Returns:
             List of probability tuples.

        """
    @overload
    def prob_run_tuple_list(self, program: QProg, qubit_addr_list: list[int], select_max: int = ...) -> list[tuple[int, float]]:
        """prob_run_tuple_list(*args, **kwargs)
        Overloaded function.

        1. prob_run_tuple_list(self: pyQPanda.CPUQVM, program: pyQPanda.QProg, qubit_list: pyQPanda.QVec, select_max: int = -1) -> List[Tuple[int, float]]

        Execute a quantum program and get a list of probability tuples.

        Args:
             program: The quantum program to execute.

             qubit_list: List of qubits to measure.

             select_max: int, optional, maximum number of tuples to return (default: -1).

        Returns:
             List of probability tuples.


        2. prob_run_tuple_list(self: pyQPanda.CPUQVM, program: pyQPanda.QProg, qubit_addr_list: List[int], select_max: int = -1) -> List[Tuple[int, float]]

        Execute a quantum program and get a list of probability tuples using qubit addresses.

        Args:
             program: The quantum program to execute.

             qubit_addr_list: List of qubit addresses to measure.

             select_max: int, optional, maximum number of tuples to return (default: -1).

        Returns:
             List of probability tuples.

        """
    def quick_measure(self, qubit_list: QVec, shots: int) -> dict[str, int]:
        """quick_measure(self: pyQPanda.CPUQVM, qubit_list: pyQPanda.QVec, shots: int) -> Dict[str, int]

        Perform a quick measurement on the specified qubits.

        Args:
             qubit_list: List of qubits to measure.

             shots: Number of measurement shots to perform.

        Returns:
             Reference to the measurement results.

        """
    def set_max_threads(self, size: int) -> None:
        """set_max_threads(self: pyQPanda.CPUQVM, size: int) -> None

        Set the maximum number of threads for the CPU quantum virtual machine (QVM).

        Args:
             size: The maximum number of threads to use.

        Returns:
             None: This method does not return a value.

        """

class CPUSingleThreadQVM(QuantumMachine):
    def __init__(self) -> None:
        """__init__(self: pyQPanda.CPUSingleThreadQVM) -> None"""
    def get_prob_dict(self, qubit_list: QVec, select_max: int = ...) -> dict[str, float]:
        """get_prob_dict(self: pyQPanda.CPUSingleThreadQVM, qubit_list: pyQPanda.QVec, select_max: int = -1) -> Dict[str, float]

        Get a dictionary of probabilities for the specified qubits.

        Args:
             qubit_list: List of qubits to measure.

             select_max: int, optional, maximum number of entries to return (default: -1).

        Returns:
             Dictionary of probabilities as a reference.

        """
    def get_prob_list(self, qubit_list: QVec, select_max: int = ...) -> list[float]:
        """get_prob_list(self: pyQPanda.CPUSingleThreadQVM, qubit_list: pyQPanda.QVec, select_max: int = -1) -> List[float]

        Get a list of probabilities for the specified qubits.

        Args:
             qubit_list: List of qubits to measure.

             select_max: int, optional, maximum number of probabilities to return (default: -1).

        Returns:
             List of probabilities as a reference.

        """
    def get_prob_tuple_list(self, qubit_list: QVec, select_max: int = ...) -> list[tuple[int, float]]:
        """get_prob_tuple_list(self: pyQPanda.CPUSingleThreadQVM, qubit_list: pyQPanda.QVec, select_max: int = -1) -> List[Tuple[int, float]]

        Get a list of probability tuples for the specified qubits.

        Args:
             qubit_list: List of qubits to measure.

             select_max: int, optional, maximum number of tuples to return (default: -1).

        Returns:
             List of probability tuples as a reference.

        """
    def pmeasure(self, qubit_list: QVec, select_max: int = ...) -> list[tuple[int, float]]:
        """pmeasure(self: pyQPanda.CPUSingleThreadQVM, qubit_list: pyQPanda.QVec, select_max: int = -1) -> List[Tuple[int, float]]

        Get the probability distribution over qubits.

        Args:
             qubit_list: List of qubits to measure.

             select_max: int, optional, maximum number of results to select (default: -1).

        Returns:
             Probability distribution as a reference.

        """
    def pmeasure_no_index(self, qubit_list: QVec) -> list[float]:
        """pmeasure_no_index(self: pyQPanda.CPUSingleThreadQVM, qubit_list: pyQPanda.QVec) -> List[float]

        Get the probability distribution over qubits without index.

        Args:
             qubit_list: List of qubits to measure.

        Returns:
             Probability distribution as a reference.

        """
    @overload
    def prob_run_dict(self, program: QProg, qubit_list: QVec, select_max: int = ...) -> dict[str, float]:
        """prob_run_dict(*args, **kwargs)
        Overloaded function.

        1. prob_run_dict(self: pyQPanda.CPUSingleThreadQVM, program: pyQPanda.QProg, qubit_list: pyQPanda.QVec, select_max: int = -1) -> Dict[str, float]

        Execute a quantum program and retrieve a dictionary of probabilities for the specified qubits.

        Args:
             program: The quantum program to execute.

             qubit_list: List of qubits to measure.

             select_max: int, optional, maximum number of entries in the dictionary to return (default: -1).

        Returns:
             Dictionary of probabilities.


        2. prob_run_dict(self: pyQPanda.CPUSingleThreadQVM, program: pyQPanda.QProg, qubit_addr_list: List[int], select_max: int = -1) -> Dict[str, float]

        Execute a quantum program and retrieve a dictionary of probabilities using qubit addresses.

        Args:
             program: The quantum program to execute.

             qubit_addr_list: List of qubit addresses to measure.

             select_max: int, optional, maximum number of entries in the dictionary to return (default: -1).

        Returns:
             Dictionary of probabilities.

        """
    @overload
    def prob_run_dict(self, program: QProg, qubit_addr_list: list[int], select_max: int = ...) -> dict[str, float]:
        """prob_run_dict(*args, **kwargs)
        Overloaded function.

        1. prob_run_dict(self: pyQPanda.CPUSingleThreadQVM, program: pyQPanda.QProg, qubit_list: pyQPanda.QVec, select_max: int = -1) -> Dict[str, float]

        Execute a quantum program and retrieve a dictionary of probabilities for the specified qubits.

        Args:
             program: The quantum program to execute.

             qubit_list: List of qubits to measure.

             select_max: int, optional, maximum number of entries in the dictionary to return (default: -1).

        Returns:
             Dictionary of probabilities.


        2. prob_run_dict(self: pyQPanda.CPUSingleThreadQVM, program: pyQPanda.QProg, qubit_addr_list: List[int], select_max: int = -1) -> Dict[str, float]

        Execute a quantum program and retrieve a dictionary of probabilities using qubit addresses.

        Args:
             program: The quantum program to execute.

             qubit_addr_list: List of qubit addresses to measure.

             select_max: int, optional, maximum number of entries in the dictionary to return (default: -1).

        Returns:
             Dictionary of probabilities.

        """
    @overload
    def prob_run_list(self, program: QProg, qubit_list: QVec, select_max: int = ...) -> list[float]:
        """prob_run_list(*args, **kwargs)
        Overloaded function.

        1. prob_run_list(self: pyQPanda.CPUSingleThreadQVM, program: pyQPanda.QProg, qubit_list: pyQPanda.QVec, select_max: int = -1) -> List[float]

        Execute a quantum program and retrieve a list of probabilities for the specified qubits.

        Args:
             program: The quantum program to execute.

             qubit_list: List of qubits to measure.

             select_max: int, optional, maximum number of probabilities to return (default: -1).

        Returns:
             List of probabilities.


        2. prob_run_list(self: pyQPanda.CPUSingleThreadQVM, program: pyQPanda.QProg, qubit_addr_list: List[int], select_max: int = -1) -> List[float]

        Execute a quantum program and retrieve a list of probabilities using qubit addresses.

        Args:
             program: The quantum program to execute.

             qubit_addr_list: List of qubit addresses to measure.

             select_max: int, optional, maximum number of probabilities to return (default: -1).

        Returns:
             List of probabilities.

        """
    @overload
    def prob_run_list(self, program: QProg, qubit_addr_list: list[int], select_max: int = ...) -> list[float]:
        """prob_run_list(*args, **kwargs)
        Overloaded function.

        1. prob_run_list(self: pyQPanda.CPUSingleThreadQVM, program: pyQPanda.QProg, qubit_list: pyQPanda.QVec, select_max: int = -1) -> List[float]

        Execute a quantum program and retrieve a list of probabilities for the specified qubits.

        Args:
             program: The quantum program to execute.

             qubit_list: List of qubits to measure.

             select_max: int, optional, maximum number of probabilities to return (default: -1).

        Returns:
             List of probabilities.


        2. prob_run_list(self: pyQPanda.CPUSingleThreadQVM, program: pyQPanda.QProg, qubit_addr_list: List[int], select_max: int = -1) -> List[float]

        Execute a quantum program and retrieve a list of probabilities using qubit addresses.

        Args:
             program: The quantum program to execute.

             qubit_addr_list: List of qubit addresses to measure.

             select_max: int, optional, maximum number of probabilities to return (default: -1).

        Returns:
             List of probabilities.

        """
    @overload
    def prob_run_tuple_list(self, program: QProg, qubit_list: QVec, select_max: int = ...) -> list[tuple[int, float]]:
        """prob_run_tuple_list(*args, **kwargs)
        Overloaded function.

        1. prob_run_tuple_list(self: pyQPanda.CPUSingleThreadQVM, program: pyQPanda.QProg, qubit_list: pyQPanda.QVec, select_max: int = -1) -> List[Tuple[int, float]]

        Execute a quantum program and get a list of probability tuples.

        Args:
             program: The quantum program to execute.

             qubit_list: List of qubits to measure.

             select_max: int, optional, maximum number of tuples to return (default: -1).

        Returns:
             List of probability tuples.


        2. prob_run_tuple_list(self: pyQPanda.CPUSingleThreadQVM, program: pyQPanda.QProg, qubit_addr_list: List[int], select_max: int = -1) -> List[Tuple[int, float]]

        Execute a quantum program and get a list of probability tuples using qubit addresses.

        Args:
             program: The quantum program to execute.

             qubit_addr_list: List of qubit addresses to measure.

             select_max: int, optional, maximum number of tuples to return (default: -1).

        Returns:
             List of probability tuples.

        """
    @overload
    def prob_run_tuple_list(self, program: QProg, qubit_addr_list: list[int], select_max: int = ...) -> list[tuple[int, float]]:
        """prob_run_tuple_list(*args, **kwargs)
        Overloaded function.

        1. prob_run_tuple_list(self: pyQPanda.CPUSingleThreadQVM, program: pyQPanda.QProg, qubit_list: pyQPanda.QVec, select_max: int = -1) -> List[Tuple[int, float]]

        Execute a quantum program and get a list of probability tuples.

        Args:
             program: The quantum program to execute.

             qubit_list: List of qubits to measure.

             select_max: int, optional, maximum number of tuples to return (default: -1).

        Returns:
             List of probability tuples.


        2. prob_run_tuple_list(self: pyQPanda.CPUSingleThreadQVM, program: pyQPanda.QProg, qubit_addr_list: List[int], select_max: int = -1) -> List[Tuple[int, float]]

        Execute a quantum program and get a list of probability tuples using qubit addresses.

        Args:
             program: The quantum program to execute.

             qubit_addr_list: List of qubit addresses to measure.

             select_max: int, optional, maximum number of tuples to return (default: -1).

        Returns:
             List of probability tuples.

        """
    def quick_measure(self, qubit_list: QVec, shots: int) -> dict[str, int]:
        """quick_measure(self: pyQPanda.CPUSingleThreadQVM, qubit_list: pyQPanda.QVec, shots: int) -> Dict[str, int]

        Perform a quick measurement on the specified qubits.

        Args:
             qubit_list: List of qubits to measure.

             shots: Number of measurement shots to perform.

        Returns:
             Reference to the measurement results.

        """

class ChipID:
    __members__: ClassVar[dict] = ...  # read-only
    Simulation: ClassVar[ChipID] = ...
    WUYUAN_1: ClassVar[ChipID] = ...
    WUYUAN_2: ClassVar[ChipID] = ...
    WUYUAN_3: ClassVar[ChipID] = ...
    __entries: ClassVar[dict] = ...
    def __init__(self, value: int) -> None:
        """__init__(self: pyQPanda.ChipID, value: int) -> None"""
    def __eq__(self, other: object) -> bool:
        """__eq__(self: object, other: object) -> bool"""
    def __hash__(self) -> int:
        """__hash__(self: object) -> int"""
    def __index__(self) -> int:
        """__index__(self: pyQPanda.ChipID) -> int"""
    def __int__(self) -> int:
        """__int__(self: pyQPanda.ChipID) -> int"""
    def __ne__(self, other: object) -> bool:
        """__ne__(self: object, other: object) -> bool"""
    @property
    def name(self) -> str: ...
    @property
    def value(self) -> int: ...

class ClassicalCondition:
    def __init__(self, *args, **kwargs) -> None:
        """Initialize self.  See help(type(self)) for accurate signature."""
    @overload
    def c_and(self, arg0: int) -> ClassicalCondition:
        """c_and(*args, **kwargs)
        Overloaded function.

        1. c_and(self: pyQPanda.ClassicalCondition, arg0: int) -> pyQPanda.ClassicalCondition

        Perform a logical AND operation with a classical bit.

        Args:
             cbit: The classical bit size to perform AND with.

        Returns:
             The result of the AND operation.


        2. c_and(self: pyQPanda.ClassicalCondition, arg0: pyQPanda.ClassicalCondition) -> pyQPanda.ClassicalCondition

        Perform a logical AND operation with another ClassicalCondition.

        Args:
             other: Another ClassicalCondition to perform AND with.

        Returns:
             The result of the AND operation.

        """
    @overload
    def c_and(self, arg0: ClassicalCondition) -> ClassicalCondition:
        """c_and(*args, **kwargs)
        Overloaded function.

        1. c_and(self: pyQPanda.ClassicalCondition, arg0: int) -> pyQPanda.ClassicalCondition

        Perform a logical AND operation with a classical bit.

        Args:
             cbit: The classical bit size to perform AND with.

        Returns:
             The result of the AND operation.


        2. c_and(self: pyQPanda.ClassicalCondition, arg0: pyQPanda.ClassicalCondition) -> pyQPanda.ClassicalCondition

        Perform a logical AND operation with another ClassicalCondition.

        Args:
             other: Another ClassicalCondition to perform AND with.

        Returns:
             The result of the AND operation.

        """
    def c_not(self) -> ClassicalCondition:
        """c_not(self: pyQPanda.ClassicalCondition) -> pyQPanda.ClassicalCondition

        Perform a logical NOT operation on the classical condition.

        Args:
             None

        Returns:
             The result of the NOT operation.

        """
    @overload
    def c_or(self, arg0: int) -> ClassicalCondition:
        """c_or(*args, **kwargs)
        Overloaded function.

        1. c_or(self: pyQPanda.ClassicalCondition, arg0: int) -> pyQPanda.ClassicalCondition

        Perform a logical OR operation with a classical bit.

        Args:
             cbit: The classical bit size to perform OR with.

        Returns:
             The result of the OR operation.


        2. c_or(self: pyQPanda.ClassicalCondition, arg0: pyQPanda.ClassicalCondition) -> pyQPanda.ClassicalCondition

        Perform a logical OR operation with another ClassicalCondition.

        Args:
             other: Another ClassicalCondition to perform OR with.

        Returns:
             The result of the OR operation.

        """
    @overload
    def c_or(self, arg0: ClassicalCondition) -> ClassicalCondition:
        """c_or(*args, **kwargs)
        Overloaded function.

        1. c_or(self: pyQPanda.ClassicalCondition, arg0: int) -> pyQPanda.ClassicalCondition

        Perform a logical OR operation with a classical bit.

        Args:
             cbit: The classical bit size to perform OR with.

        Returns:
             The result of the OR operation.


        2. c_or(self: pyQPanda.ClassicalCondition, arg0: pyQPanda.ClassicalCondition) -> pyQPanda.ClassicalCondition

        Perform a logical OR operation with another ClassicalCondition.

        Args:
             other: Another ClassicalCondition to perform OR with.

        Returns:
             The result of the OR operation.

        """
    def get_val(self) -> int:
        """get_val(self: pyQPanda.ClassicalCondition) -> int

        Retrieve the current value of the classical condition.

        Args:
             None

        Returns:
             The value of the ClassicalCondition.

        """
    def set_val(self, arg0: int) -> None:
        """set_val(self: pyQPanda.ClassicalCondition, arg0: int) -> None

        Set a new value for the classical condition.

        Args:
             value: The new value to set.

        Returns:
             None

        """
    @overload
    def __add__(self, arg0: ClassicalCondition) -> ClassicalCondition:
        """__add__(*args, **kwargs)
        Overloaded function.

        1. __add__(self: pyQPanda.ClassicalCondition, arg0: pyQPanda.ClassicalCondition) -> pyQPanda.ClassicalCondition

        2. __add__(self: pyQPanda.ClassicalCondition, arg0: int) -> pyQPanda.ClassicalCondition
        """
    @overload
    def __add__(self, arg0: int) -> ClassicalCondition:
        """__add__(*args, **kwargs)
        Overloaded function.

        1. __add__(self: pyQPanda.ClassicalCondition, arg0: pyQPanda.ClassicalCondition) -> pyQPanda.ClassicalCondition

        2. __add__(self: pyQPanda.ClassicalCondition, arg0: int) -> pyQPanda.ClassicalCondition
        """
    @overload
    def __eq__(self, arg0: ClassicalCondition) -> ClassicalCondition:
        """__eq__(*args, **kwargs)
        Overloaded function.

        1. __eq__(self: pyQPanda.ClassicalCondition, arg0: pyQPanda.ClassicalCondition) -> pyQPanda.ClassicalCondition

        2. __eq__(self: pyQPanda.ClassicalCondition, arg0: int) -> pyQPanda.ClassicalCondition

        3. __eq__(self: pyQPanda.ClassicalCondition, arg0: int) -> pyQPanda.ClassicalCondition
        """
    @overload
    def __eq__(self, arg0: int) -> ClassicalCondition:
        """__eq__(*args, **kwargs)
        Overloaded function.

        1. __eq__(self: pyQPanda.ClassicalCondition, arg0: pyQPanda.ClassicalCondition) -> pyQPanda.ClassicalCondition

        2. __eq__(self: pyQPanda.ClassicalCondition, arg0: int) -> pyQPanda.ClassicalCondition

        3. __eq__(self: pyQPanda.ClassicalCondition, arg0: int) -> pyQPanda.ClassicalCondition
        """
    @overload
    def __eq__(self, arg0: int) -> ClassicalCondition:
        """__eq__(*args, **kwargs)
        Overloaded function.

        1. __eq__(self: pyQPanda.ClassicalCondition, arg0: pyQPanda.ClassicalCondition) -> pyQPanda.ClassicalCondition

        2. __eq__(self: pyQPanda.ClassicalCondition, arg0: int) -> pyQPanda.ClassicalCondition

        3. __eq__(self: pyQPanda.ClassicalCondition, arg0: int) -> pyQPanda.ClassicalCondition
        """
    @overload
    def __ge__(self, arg0: int) -> ClassicalCondition:
        """__ge__(*args, **kwargs)
        Overloaded function.

        1. __ge__(self: pyQPanda.ClassicalCondition, arg0: int) -> pyQPanda.ClassicalCondition

        2. __ge__(self: pyQPanda.ClassicalCondition, arg0: pyQPanda.ClassicalCondition) -> pyQPanda.ClassicalCondition

        3. __ge__(self: pyQPanda.ClassicalCondition, arg0: int) -> pyQPanda.ClassicalCondition
        """
    @overload
    def __ge__(self, arg0: ClassicalCondition) -> ClassicalCondition:
        """__ge__(*args, **kwargs)
        Overloaded function.

        1. __ge__(self: pyQPanda.ClassicalCondition, arg0: int) -> pyQPanda.ClassicalCondition

        2. __ge__(self: pyQPanda.ClassicalCondition, arg0: pyQPanda.ClassicalCondition) -> pyQPanda.ClassicalCondition

        3. __ge__(self: pyQPanda.ClassicalCondition, arg0: int) -> pyQPanda.ClassicalCondition
        """
    @overload
    def __ge__(self, arg0: int) -> ClassicalCondition:
        """__ge__(*args, **kwargs)
        Overloaded function.

        1. __ge__(self: pyQPanda.ClassicalCondition, arg0: int) -> pyQPanda.ClassicalCondition

        2. __ge__(self: pyQPanda.ClassicalCondition, arg0: pyQPanda.ClassicalCondition) -> pyQPanda.ClassicalCondition

        3. __ge__(self: pyQPanda.ClassicalCondition, arg0: int) -> pyQPanda.ClassicalCondition
        """
    @overload
    def __gt__(self, arg0: int) -> ClassicalCondition:
        """__gt__(*args, **kwargs)
        Overloaded function.

        1. __gt__(self: pyQPanda.ClassicalCondition, arg0: int) -> pyQPanda.ClassicalCondition

        2. __gt__(self: pyQPanda.ClassicalCondition, arg0: pyQPanda.ClassicalCondition) -> pyQPanda.ClassicalCondition

        3. __gt__(self: pyQPanda.ClassicalCondition, arg0: int) -> pyQPanda.ClassicalCondition
        """
    @overload
    def __gt__(self, arg0: ClassicalCondition) -> ClassicalCondition:
        """__gt__(*args, **kwargs)
        Overloaded function.

        1. __gt__(self: pyQPanda.ClassicalCondition, arg0: int) -> pyQPanda.ClassicalCondition

        2. __gt__(self: pyQPanda.ClassicalCondition, arg0: pyQPanda.ClassicalCondition) -> pyQPanda.ClassicalCondition

        3. __gt__(self: pyQPanda.ClassicalCondition, arg0: int) -> pyQPanda.ClassicalCondition
        """
    @overload
    def __gt__(self, arg0: int) -> ClassicalCondition:
        """__gt__(*args, **kwargs)
        Overloaded function.

        1. __gt__(self: pyQPanda.ClassicalCondition, arg0: int) -> pyQPanda.ClassicalCondition

        2. __gt__(self: pyQPanda.ClassicalCondition, arg0: pyQPanda.ClassicalCondition) -> pyQPanda.ClassicalCondition

        3. __gt__(self: pyQPanda.ClassicalCondition, arg0: int) -> pyQPanda.ClassicalCondition
        """
    @overload
    def __le__(self, arg0: ClassicalCondition) -> ClassicalCondition:
        """__le__(*args, **kwargs)
        Overloaded function.

        1. __le__(self: pyQPanda.ClassicalCondition, arg0: pyQPanda.ClassicalCondition) -> pyQPanda.ClassicalCondition

        2. __le__(self: pyQPanda.ClassicalCondition, arg0: int) -> pyQPanda.ClassicalCondition

        3. __le__(self: pyQPanda.ClassicalCondition, arg0: int) -> pyQPanda.ClassicalCondition
        """
    @overload
    def __le__(self, arg0: int) -> ClassicalCondition:
        """__le__(*args, **kwargs)
        Overloaded function.

        1. __le__(self: pyQPanda.ClassicalCondition, arg0: pyQPanda.ClassicalCondition) -> pyQPanda.ClassicalCondition

        2. __le__(self: pyQPanda.ClassicalCondition, arg0: int) -> pyQPanda.ClassicalCondition

        3. __le__(self: pyQPanda.ClassicalCondition, arg0: int) -> pyQPanda.ClassicalCondition
        """
    @overload
    def __le__(self, arg0: int) -> ClassicalCondition:
        """__le__(*args, **kwargs)
        Overloaded function.

        1. __le__(self: pyQPanda.ClassicalCondition, arg0: pyQPanda.ClassicalCondition) -> pyQPanda.ClassicalCondition

        2. __le__(self: pyQPanda.ClassicalCondition, arg0: int) -> pyQPanda.ClassicalCondition

        3. __le__(self: pyQPanda.ClassicalCondition, arg0: int) -> pyQPanda.ClassicalCondition
        """
    @overload
    def __lt__(self, arg0: ClassicalCondition) -> ClassicalCondition:
        """__lt__(*args, **kwargs)
        Overloaded function.

        1. __lt__(self: pyQPanda.ClassicalCondition, arg0: pyQPanda.ClassicalCondition) -> pyQPanda.ClassicalCondition

        2. __lt__(self: pyQPanda.ClassicalCondition, arg0: int) -> pyQPanda.ClassicalCondition

        3. __lt__(self: pyQPanda.ClassicalCondition, arg0: int) -> pyQPanda.ClassicalCondition
        """
    @overload
    def __lt__(self, arg0: int) -> ClassicalCondition:
        """__lt__(*args, **kwargs)
        Overloaded function.

        1. __lt__(self: pyQPanda.ClassicalCondition, arg0: pyQPanda.ClassicalCondition) -> pyQPanda.ClassicalCondition

        2. __lt__(self: pyQPanda.ClassicalCondition, arg0: int) -> pyQPanda.ClassicalCondition

        3. __lt__(self: pyQPanda.ClassicalCondition, arg0: int) -> pyQPanda.ClassicalCondition
        """
    @overload
    def __lt__(self, arg0: int) -> ClassicalCondition:
        """__lt__(*args, **kwargs)
        Overloaded function.

        1. __lt__(self: pyQPanda.ClassicalCondition, arg0: pyQPanda.ClassicalCondition) -> pyQPanda.ClassicalCondition

        2. __lt__(self: pyQPanda.ClassicalCondition, arg0: int) -> pyQPanda.ClassicalCondition

        3. __lt__(self: pyQPanda.ClassicalCondition, arg0: int) -> pyQPanda.ClassicalCondition
        """
    @overload
    def __mul__(self, arg0: ClassicalCondition) -> ClassicalCondition:
        """__mul__(*args, **kwargs)
        Overloaded function.

        1. __mul__(self: pyQPanda.ClassicalCondition, arg0: pyQPanda.ClassicalCondition) -> pyQPanda.ClassicalCondition

        2. __mul__(self: pyQPanda.ClassicalCondition, arg0: int) -> pyQPanda.ClassicalCondition
        """
    @overload
    def __mul__(self, arg0: int) -> ClassicalCondition:
        """__mul__(*args, **kwargs)
        Overloaded function.

        1. __mul__(self: pyQPanda.ClassicalCondition, arg0: pyQPanda.ClassicalCondition) -> pyQPanda.ClassicalCondition

        2. __mul__(self: pyQPanda.ClassicalCondition, arg0: int) -> pyQPanda.ClassicalCondition
        """
    def __radd__(self, arg0: int) -> ClassicalCondition:
        """__radd__(self: pyQPanda.ClassicalCondition, arg0: int) -> pyQPanda.ClassicalCondition"""
    def __rmul__(self, arg0: int) -> ClassicalCondition:
        """__rmul__(self: pyQPanda.ClassicalCondition, arg0: int) -> pyQPanda.ClassicalCondition"""
    def __rsub__(self, arg0: int) -> ClassicalCondition:
        """__rsub__(self: pyQPanda.ClassicalCondition, arg0: int) -> pyQPanda.ClassicalCondition"""
    def __rtruediv__(self, arg0: int) -> ClassicalCondition:
        """__rtruediv__(self: pyQPanda.ClassicalCondition, arg0: int) -> pyQPanda.ClassicalCondition"""
    @overload
    def __sub__(self, arg0: ClassicalCondition) -> ClassicalCondition:
        """__sub__(*args, **kwargs)
        Overloaded function.

        1. __sub__(self: pyQPanda.ClassicalCondition, arg0: pyQPanda.ClassicalCondition) -> pyQPanda.ClassicalCondition

        2. __sub__(self: pyQPanda.ClassicalCondition, arg0: int) -> pyQPanda.ClassicalCondition
        """
    @overload
    def __sub__(self, arg0: int) -> ClassicalCondition:
        """__sub__(*args, **kwargs)
        Overloaded function.

        1. __sub__(self: pyQPanda.ClassicalCondition, arg0: pyQPanda.ClassicalCondition) -> pyQPanda.ClassicalCondition

        2. __sub__(self: pyQPanda.ClassicalCondition, arg0: int) -> pyQPanda.ClassicalCondition
        """
    @overload
    def __truediv__(self, arg0: ClassicalCondition) -> ClassicalCondition:
        """__truediv__(*args, **kwargs)
        Overloaded function.

        1. __truediv__(self: pyQPanda.ClassicalCondition, arg0: pyQPanda.ClassicalCondition) -> pyQPanda.ClassicalCondition

        2. __truediv__(self: pyQPanda.ClassicalCondition, arg0: int) -> pyQPanda.ClassicalCondition
        """
    @overload
    def __truediv__(self, arg0: int) -> ClassicalCondition:
        """__truediv__(*args, **kwargs)
        Overloaded function.

        1. __truediv__(self: pyQPanda.ClassicalCondition, arg0: pyQPanda.ClassicalCondition) -> pyQPanda.ClassicalCondition

        2. __truediv__(self: pyQPanda.ClassicalCondition, arg0: int) -> pyQPanda.ClassicalCondition
        """

class ClassicalProg:
    def __init__(self, arg0: ClassicalCondition) -> None:
        """__init__(self: pyQPanda.ClassicalProg, arg0: pyQPanda.ClassicalCondition) -> None"""

class CommProtocolConfig:
    circuits_num: int
    open_error_mitigation: bool
    open_mapping: bool
    optimization_level: int
    shots: int
    def __init__(self) -> None:
        """__init__(self: pyQPanda.CommProtocolConfig) -> None"""

class ComplexVertexSplitMethod:
    __members__: ClassVar[dict] = ...  # read-only
    LINEAR: ClassVar[ComplexVertexSplitMethod] = ...
    METHOD_UNDEFINED: ClassVar[ComplexVertexSplitMethod] = ...
    RING: ClassVar[ComplexVertexSplitMethod] = ...
    __entries: ClassVar[dict] = ...
    def __init__(self, value: int) -> None:
        """__init__(self: pyQPanda.ComplexVertexSplitMethod, value: int) -> None"""
    def __eq__(self, other: object) -> bool:
        """__eq__(self: object, other: object) -> bool"""
    def __hash__(self) -> int:
        """__hash__(self: object) -> int"""
    def __index__(self) -> int:
        """__index__(self: pyQPanda.ComplexVertexSplitMethod) -> int"""
    def __int__(self) -> int:
        """__int__(self: pyQPanda.ComplexVertexSplitMethod) -> int"""
    def __ne__(self, other: object) -> bool:
        """__ne__(self: object, other: object) -> bool"""
    @property
    def name(self) -> str: ...
    @property
    def value(self) -> int: ...

class DAGNodeType:
    __members__: ClassVar[dict] = ...  # read-only
    MAX_GATE_TYPE: ClassVar[DAGNodeType] = ...
    MEASURE: ClassVar[DAGNodeType] = ...
    NUKNOW_SEQ_NODE_TYPE: ClassVar[DAGNodeType] = ...
    QUBIT: ClassVar[DAGNodeType] = ...
    RESET: ClassVar[DAGNodeType] = ...
    __entries: ClassVar[dict] = ...
    def __init__(self, value: int) -> None:
        """__init__(self: pyQPanda.DAGNodeType, value: int) -> None"""
    def __eq__(self, other: object) -> bool:
        """__eq__(self: object, other: object) -> bool"""
    def __hash__(self) -> int:
        """__hash__(self: object) -> int"""
    def __index__(self) -> int:
        """__index__(self: pyQPanda.DAGNodeType) -> int"""
    def __int__(self) -> int:
        """__int__(self: pyQPanda.DAGNodeType) -> int"""
    def __ne__(self, other: object) -> bool:
        """__ne__(self: object, other: object) -> bool"""
    @property
    def name(self) -> str: ...
    @property
    def value(self) -> int: ...

class DecompositionMode:
    __members__: ClassVar[dict] = ...  # read-only
    CSDecomposition: ClassVar[DecompositionMode] = ...
    HOUSEHOLDER_QR: ClassVar[DecompositionMode] = ...
    QR: ClassVar[DecompositionMode] = ...
    QSDecomposition: ClassVar[DecompositionMode] = ...
    __entries: ClassVar[dict] = ...
    def __init__(self, value: int) -> None:
        """__init__(self: pyQPanda.DecompositionMode, value: int) -> None"""
    def __eq__(self, other: object) -> bool:
        """__eq__(self: object, other: object) -> bool"""
    def __hash__(self) -> int:
        """__hash__(self: object) -> int"""
    def __index__(self) -> int:
        """__index__(self: pyQPanda.DecompositionMode) -> int"""
    def __int__(self) -> int:
        """__int__(self: pyQPanda.DecompositionMode) -> int"""
    def __ne__(self, other: object) -> bool:
        """__ne__(self: object, other: object) -> bool"""
    @property
    def name(self) -> str: ...
    @property
    def value(self) -> int: ...

class DensityMatrixSimulator(QuantumMachine):
    def __init__(self) -> None:
        """__init__(self: pyQPanda.DensityMatrixSimulator) -> None"""
    def get_density_matrix(self, prog: QProg) -> numpy.ndarray[numpy.complex128[m, n]]:
        """get_density_matrix(self: pyQPanda.DensityMatrixSimulator, prog: pyQPanda.QProg) -> numpy.ndarray[numpy.complex128[m, n]]

        Run quantum program and get the full density matrix.

        Args:
             prog: The quantum program to execute.

        Returns:
             The full density matrix.

        """
    @overload
    def get_expectation(self, prog: QProg, hamiltonian: list[tuple[dict[int, str], float]], qubits: QVec) -> float:
        """get_expectation(*args, **kwargs)
        Overloaded function.

        1. get_expectation(self: pyQPanda.DensityMatrixSimulator, prog: pyQPanda.QProg, hamiltonian: List[Tuple[Dict[int, str], float]], qubits: pyQPanda.QVec) -> float

        Run the quantum program and calculate the Hamiltonian expectation for the specified qubits.

        Args:
             prog: The quantum program to execute.

             hamiltonian: The QHamiltonian to use for the expectation value.

             qubits: The selected qubits for measurement.

        Returns:
             The Hamiltonian expectation for the specified qubits.


        2. get_expectation(self: pyQPanda.DensityMatrixSimulator, prog: pyQPanda.QProg, hamiltonian: List[Tuple[Dict[int, str], float]], qubits: List[int]) -> float

        Run the quantum program and calculate the Hamiltonian expectation for the specified qubits.

        Args:
             prog: The quantum program to execute.

             hamiltonian: The QHamiltonian to use for the expectation value.

             qubits: The selected qubits for measurement.

        Returns:
             The Hamiltonian expectation for the specified qubits.

        """
    @overload
    def get_expectation(self, prog: QProg, hamiltonian: list[tuple[dict[int, str], float]], qubits: list[int]) -> float:
        """get_expectation(*args, **kwargs)
        Overloaded function.

        1. get_expectation(self: pyQPanda.DensityMatrixSimulator, prog: pyQPanda.QProg, hamiltonian: List[Tuple[Dict[int, str], float]], qubits: pyQPanda.QVec) -> float

        Run the quantum program and calculate the Hamiltonian expectation for the specified qubits.

        Args:
             prog: The quantum program to execute.

             hamiltonian: The QHamiltonian to use for the expectation value.

             qubits: The selected qubits for measurement.

        Returns:
             The Hamiltonian expectation for the specified qubits.


        2. get_expectation(self: pyQPanda.DensityMatrixSimulator, prog: pyQPanda.QProg, hamiltonian: List[Tuple[Dict[int, str], float]], qubits: List[int]) -> float

        Run the quantum program and calculate the Hamiltonian expectation for the specified qubits.

        Args:
             prog: The quantum program to execute.

             hamiltonian: The QHamiltonian to use for the expectation value.

             qubits: The selected qubits for measurement.

        Returns:
             The Hamiltonian expectation for the specified qubits.

        """
    @overload
    def get_probabilities(self, prog: QProg) -> list[float]:
        """get_probabilities(*args, **kwargs)
        Overloaded function.

        1. get_probabilities(self: pyQPanda.DensityMatrixSimulator, prog: pyQPanda.QProg) -> List[float]

        Run the quantum program and get the probabilities for all indices.

        Args:
             prog: The quantum program to execute.

        Returns:
             The probabilities result of the quantum program.


        2. get_probabilities(self: pyQPanda.DensityMatrixSimulator, prog: pyQPanda.QProg, qubits: pyQPanda.QVec) -> List[float]

        Run the quantum program and get the probabilities for all indices for the specified qubits.

        Args:
             prog: The quantum program to execute.

             qubits: The selected qubits for measurement.

        Returns:
             The probabilities result of the quantum program.


        3. get_probabilities(self: pyQPanda.DensityMatrixSimulator, prog: pyQPanda.QProg, qubits: List[int]) -> List[float]

        Run the quantum program and get the probabilities for all indices for the specified qubits.

        Args:
             prog: The quantum program to execute.

             qubits: The selected qubits for measurement.

        Returns:
             The probabilities result of the quantum program.


        4. get_probabilities(self: pyQPanda.DensityMatrixSimulator, prog: pyQPanda.QProg, indices: List[str]) -> List[float]

        Run the quantum program and get the probabilities for the specified binary indices.

        Args:
             prog: The quantum program to execute.

             indices: The selected binary indices for measurement.

        Returns:
             The probabilities result of the quantum program.

        """
    @overload
    def get_probabilities(self, prog: QProg, qubits: QVec) -> list[float]:
        """get_probabilities(*args, **kwargs)
        Overloaded function.

        1. get_probabilities(self: pyQPanda.DensityMatrixSimulator, prog: pyQPanda.QProg) -> List[float]

        Run the quantum program and get the probabilities for all indices.

        Args:
             prog: The quantum program to execute.

        Returns:
             The probabilities result of the quantum program.


        2. get_probabilities(self: pyQPanda.DensityMatrixSimulator, prog: pyQPanda.QProg, qubits: pyQPanda.QVec) -> List[float]

        Run the quantum program and get the probabilities for all indices for the specified qubits.

        Args:
             prog: The quantum program to execute.

             qubits: The selected qubits for measurement.

        Returns:
             The probabilities result of the quantum program.


        3. get_probabilities(self: pyQPanda.DensityMatrixSimulator, prog: pyQPanda.QProg, qubits: List[int]) -> List[float]

        Run the quantum program and get the probabilities for all indices for the specified qubits.

        Args:
             prog: The quantum program to execute.

             qubits: The selected qubits for measurement.

        Returns:
             The probabilities result of the quantum program.


        4. get_probabilities(self: pyQPanda.DensityMatrixSimulator, prog: pyQPanda.QProg, indices: List[str]) -> List[float]

        Run the quantum program and get the probabilities for the specified binary indices.

        Args:
             prog: The quantum program to execute.

             indices: The selected binary indices for measurement.

        Returns:
             The probabilities result of the quantum program.

        """
    @overload
    def get_probabilities(self, prog: QProg, qubits: list[int]) -> list[float]:
        """get_probabilities(*args, **kwargs)
        Overloaded function.

        1. get_probabilities(self: pyQPanda.DensityMatrixSimulator, prog: pyQPanda.QProg) -> List[float]

        Run the quantum program and get the probabilities for all indices.

        Args:
             prog: The quantum program to execute.

        Returns:
             The probabilities result of the quantum program.


        2. get_probabilities(self: pyQPanda.DensityMatrixSimulator, prog: pyQPanda.QProg, qubits: pyQPanda.QVec) -> List[float]

        Run the quantum program and get the probabilities for all indices for the specified qubits.

        Args:
             prog: The quantum program to execute.

             qubits: The selected qubits for measurement.

        Returns:
             The probabilities result of the quantum program.


        3. get_probabilities(self: pyQPanda.DensityMatrixSimulator, prog: pyQPanda.QProg, qubits: List[int]) -> List[float]

        Run the quantum program and get the probabilities for all indices for the specified qubits.

        Args:
             prog: The quantum program to execute.

             qubits: The selected qubits for measurement.

        Returns:
             The probabilities result of the quantum program.


        4. get_probabilities(self: pyQPanda.DensityMatrixSimulator, prog: pyQPanda.QProg, indices: List[str]) -> List[float]

        Run the quantum program and get the probabilities for the specified binary indices.

        Args:
             prog: The quantum program to execute.

             indices: The selected binary indices for measurement.

        Returns:
             The probabilities result of the quantum program.

        """
    @overload
    def get_probabilities(self, prog: QProg, indices: list[str]) -> list[float]:
        """get_probabilities(*args, **kwargs)
        Overloaded function.

        1. get_probabilities(self: pyQPanda.DensityMatrixSimulator, prog: pyQPanda.QProg) -> List[float]

        Run the quantum program and get the probabilities for all indices.

        Args:
             prog: The quantum program to execute.

        Returns:
             The probabilities result of the quantum program.


        2. get_probabilities(self: pyQPanda.DensityMatrixSimulator, prog: pyQPanda.QProg, qubits: pyQPanda.QVec) -> List[float]

        Run the quantum program and get the probabilities for all indices for the specified qubits.

        Args:
             prog: The quantum program to execute.

             qubits: The selected qubits for measurement.

        Returns:
             The probabilities result of the quantum program.


        3. get_probabilities(self: pyQPanda.DensityMatrixSimulator, prog: pyQPanda.QProg, qubits: List[int]) -> List[float]

        Run the quantum program and get the probabilities for all indices for the specified qubits.

        Args:
             prog: The quantum program to execute.

             qubits: The selected qubits for measurement.

        Returns:
             The probabilities result of the quantum program.


        4. get_probabilities(self: pyQPanda.DensityMatrixSimulator, prog: pyQPanda.QProg, indices: List[str]) -> List[float]

        Run the quantum program and get the probabilities for the specified binary indices.

        Args:
             prog: The quantum program to execute.

             indices: The selected binary indices for measurement.

        Returns:
             The probabilities result of the quantum program.

        """
    @overload
    def get_probability(self, prog: QProg, index: int) -> float:
        """get_probability(*args, **kwargs)
        Overloaded function.

        1. get_probability(self: pyQPanda.DensityMatrixSimulator, prog: pyQPanda.QProg, index: int) -> float

        Run the quantum program and get the probability for the specified index.

        Args:
             prog: The quantum program to execute.

             index: The measurement index in [0, 2^N      1].

        Returns:
             The probability result of the quantum program.


        2. get_probability(self: pyQPanda.DensityMatrixSimulator, prog: pyQPanda.QProg, index: str) -> float

        Run the quantum program and get the probability for the specified index.

        Args:
             prog: The quantum program to execute.

             index: The measurement index in [0, 2^N      1].

        Returns:
             The probability result of the quantum program.
        """
    @overload
    def get_probability(self, prog: QProg, index: str) -> float:
        """get_probability(*args, **kwargs)
        Overloaded function.

        1. get_probability(self: pyQPanda.DensityMatrixSimulator, prog: pyQPanda.QProg, index: int) -> float

        Run the quantum program and get the probability for the specified index.

        Args:
             prog: The quantum program to execute.

             index: The measurement index in [0, 2^N      1].

        Returns:
             The probability result of the quantum program.


        2. get_probability(self: pyQPanda.DensityMatrixSimulator, prog: pyQPanda.QProg, index: str) -> float

        Run the quantum program and get the probability for the specified index.

        Args:
             prog: The quantum program to execute.

             index: The measurement index in [0, 2^N      1].

        Returns:
             The probability result of the quantum program.
        """
    @overload
    def get_reduced_density_matrix(self, prog: QProg, qubits: QVec) -> numpy.ndarray[numpy.complex128[m, n]]:
        """get_reduced_density_matrix(*args, **kwargs)
        Overloaded function.

        1. get_reduced_density_matrix(self: pyQPanda.DensityMatrixSimulator, prog: pyQPanda.QProg, qubits: pyQPanda.QVec) -> numpy.ndarray[numpy.complex128[m, n]]

        Run quantum program and get the density matrix for current qubits.

        Args:
             prog: The quantum program to execute.

             qubits: The selected qubits from the quantum program.

        Returns:
             The density matrix for the specified qubits.


        2. get_reduced_density_matrix(self: pyQPanda.DensityMatrixSimulator, prog: pyQPanda.QProg, qubits: List[int]) -> numpy.ndarray[numpy.complex128[m, n]]

        Run quantum program and get the density matrix for current qubits.

        Args:
             prog: The quantum program to execute.

             qubits: The selected qubits from the quantum program.

        Returns:
             The density matrix for the specified qubits.

        """
    @overload
    def get_reduced_density_matrix(self, prog: QProg, qubits: list[int]) -> numpy.ndarray[numpy.complex128[m, n]]:
        """get_reduced_density_matrix(*args, **kwargs)
        Overloaded function.

        1. get_reduced_density_matrix(self: pyQPanda.DensityMatrixSimulator, prog: pyQPanda.QProg, qubits: pyQPanda.QVec) -> numpy.ndarray[numpy.complex128[m, n]]

        Run quantum program and get the density matrix for current qubits.

        Args:
             prog: The quantum program to execute.

             qubits: The selected qubits from the quantum program.

        Returns:
             The density matrix for the specified qubits.


        2. get_reduced_density_matrix(self: pyQPanda.DensityMatrixSimulator, prog: pyQPanda.QProg, qubits: List[int]) -> numpy.ndarray[numpy.complex128[m, n]]

        Run quantum program and get the density matrix for current qubits.

        Args:
             prog: The quantum program to execute.

             qubits: The selected qubits from the quantum program.

        Returns:
             The density matrix for the specified qubits.

        """
    def init_qvm(self, is_double_precision: bool = ...) -> None:
        """init_qvm(self: pyQPanda.DensityMatrixSimulator, is_double_precision: bool = True) -> None"""
    @overload
    def set_noise_model(self, arg0: numpy.ndarray[numpy.complex128[m, n]]) -> None:
        """set_noise_model(*args, **kwargs)
        Overloaded function.

        1. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: numpy.ndarray[numpy.complex128[m, n]]) -> None

        Set the noise model for the density matrix simulator.

        Args:
             noise_model: The noise model represented as a complex matrix.

        Returns:
             None.


        2. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: numpy.ndarray[numpy.complex128[m, n]], arg1: List[pyQPanda.GateType]) -> None

        Set the noise model for the density matrix simulator with specific gate types.

        Args:
             noise_model: The noise model represented as a complex matrix.

             gate_types: A vector of gate types to which the noise model applies.

        Returns:
             None.


        3. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: List[numpy.ndarray[numpy.complex128[m, n]]]) -> None

        Set multiple noise models for the density matrix simulator.

        Args:
             noise_models: A vector of noise models, each represented as a complex matrix.

        Returns:
             None.


        4. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: List[numpy.ndarray[numpy.complex128[m, n]]], arg1: List[pyQPanda.GateType]) -> None

        Set multiple noise models for the density matrix simulator with specific gate types.

        Args:
             noise_models: A vector of noise models, each represented as a complex matrix.

             gate_types: A vector of gate types to which the noise models apply.

        Returns:
             None.


        5. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: pyQPanda.NoiseModel, arg1: pyQPanda.GateType, arg2: float) -> None

        Set a specific noise model for the density matrix simulator with a given gate type and probability.

        Args:
             noise_model: The noise model to apply.

             gate_type: The specific gate type associated with the noise model.

             probability: The probability of the noise occurring.

        Returns:
             None.


        6. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: pyQPanda.NoiseModel, arg1: List[pyQPanda.GateType], arg2: float) -> None

        Set a specific noise model for the density matrix simulator with multiple gate types and a given probability.

        Args:
             noise_model: The noise model to apply.

             gate_types: A vector of gate types associated with the noise model.

             probability: The probability of the noise occurring.

        Returns:
             None.


        7. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: pyQPanda.NoiseModel, arg1: pyQPanda.GateType, arg2: float, arg3: pyQPanda.QVec) -> None

        Set a specific noise model for the density matrix simulator with a given gate type, probability, and target qubits.

        Args:
             noise_model: The noise model to apply.

             gate_type: The specific gate type associated with the noise model.

             probability: The probability of the noise occurring.

             qubits: The target qubits affected by the noise model.

        Returns:
             None.


        8. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: pyQPanda.NoiseModel, arg1: List[pyQPanda.GateType], arg2: float, arg3: pyQPanda.QVec) -> None

        Set a specific noise model for the density matrix simulator with multiple gate types, a given probability, and target qubits.

        Args:
             noise_model: The noise model to apply.

             gate_types: A vector of gate types associated with the noise model.

             probability: The probability of the noise occurring.

             qubits: The target qubits affected by the noise model.

        Returns:
             None.


        9. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: pyQPanda.NoiseModel, arg1: pyQPanda.GateType, arg2: float, arg3: List[pyQPanda.QVec]) -> None

        Set a specific noise model for the density matrix simulator with a given gate type, probability, and groups of target qubits.

        Args:
             noise_model: The noise model to apply.

             gate_type: The specific gate type associated with the noise model.

             probability: The probability of the noise occurring.

             qubit_groups: A vector of QVecs representing groups of target qubits affected by the noise model.

        Returns:
             None.


        10. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: pyQPanda.NoiseModel, arg1: pyQPanda.GateType, arg2: float, arg3: float, arg4: float) -> None

        Set a specific noise model for the density matrix simulator with a given gate type, probability, duration, and temperature.

        Args:
             noise_model: The noise model to apply.

             gate_type: The specific gate type associated with the noise model.

             probability: The probability of the noise occurring.

             duration: The duration for which the noise model is applied.

             temperature: The temperature affecting the noise characteristics.

        Returns:
             None.


        11. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: pyQPanda.NoiseModel, arg1: List[pyQPanda.GateType], arg2: float, arg3: float, arg4: float) -> None

        Set a specific noise model for the density matrix simulator with multiple gate types, a given probability, duration, and temperature.

        Args:
             noise_model: The noise model to apply.

             gate_types: A vector of gate types associated with the noise model.

             probability: The probability of the noise occurring.

             duration: The duration for which the noise model is applied.

             temperature: The temperature affecting the noise characteristics.

        Returns:
             None.


        12. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: pyQPanda.NoiseModel, arg1: pyQPanda.GateType, arg2: float, arg3: float, arg4: float, arg5: pyQPanda.QVec) -> None

        Set a specific noise model for the density matrix simulator with a given gate type, probability, duration, temperature, and a target qubit.

        Args:
             noise_model: The noise model to apply.

             gate_type: The specific gate type associated with the noise model.

             probability: The probability of the noise occurring.

             duration: The duration for which the noise model is applied.

             temperature: The temperature affecting the noise characteristics.

             target_qubit: The specific qubit targeted by the noise model.

        Returns:
             None.


        13. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: pyQPanda.NoiseModel, arg1: List[pyQPanda.GateType], arg2: float, arg3: float, arg4: float, arg5: pyQPanda.QVec) -> None

        Set a specific noise model for the density matrix simulator with multiple gate types, probability, duration, temperature, and a target qubit.

        Args:
             noise_model: The noise model to apply.

             gate_types: A vector of gate types associated with the noise model.

             probability: The probability of the noise occurring.

             duration: The duration for which the noise model is applied.

             temperature: The temperature affecting the noise characteristics.

             target_qubit: The specific qubit targeted by the noise model.

        Returns:
             None.


        14. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: pyQPanda.NoiseModel, arg1: pyQPanda.GateType, arg2: float, arg3: float, arg4: float, arg5: List[pyQPanda.QVec]) -> None

        Set a specific noise model for the density matrix simulator with a given gate type, probability, duration, temperature, and multiple target qubits.

        Args:
             noise_model: The noise model to apply.

             gate_type: The specific gate type associated with the noise model.

             probability: The probability of the noise occurring.

             duration: The duration for which the noise model is applied.

             temperature: The temperature affecting the noise characteristics.

             target_qubits: A vector of qubits targeted by the noise model.

        Returns:
             None.

        """
    @overload
    def set_noise_model(self, arg0: numpy.ndarray[numpy.complex128[m, n]], arg1: list[GateType]) -> None:
        """set_noise_model(*args, **kwargs)
        Overloaded function.

        1. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: numpy.ndarray[numpy.complex128[m, n]]) -> None

        Set the noise model for the density matrix simulator.

        Args:
             noise_model: The noise model represented as a complex matrix.

        Returns:
             None.


        2. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: numpy.ndarray[numpy.complex128[m, n]], arg1: List[pyQPanda.GateType]) -> None

        Set the noise model for the density matrix simulator with specific gate types.

        Args:
             noise_model: The noise model represented as a complex matrix.

             gate_types: A vector of gate types to which the noise model applies.

        Returns:
             None.


        3. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: List[numpy.ndarray[numpy.complex128[m, n]]]) -> None

        Set multiple noise models for the density matrix simulator.

        Args:
             noise_models: A vector of noise models, each represented as a complex matrix.

        Returns:
             None.


        4. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: List[numpy.ndarray[numpy.complex128[m, n]]], arg1: List[pyQPanda.GateType]) -> None

        Set multiple noise models for the density matrix simulator with specific gate types.

        Args:
             noise_models: A vector of noise models, each represented as a complex matrix.

             gate_types: A vector of gate types to which the noise models apply.

        Returns:
             None.


        5. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: pyQPanda.NoiseModel, arg1: pyQPanda.GateType, arg2: float) -> None

        Set a specific noise model for the density matrix simulator with a given gate type and probability.

        Args:
             noise_model: The noise model to apply.

             gate_type: The specific gate type associated with the noise model.

             probability: The probability of the noise occurring.

        Returns:
             None.


        6. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: pyQPanda.NoiseModel, arg1: List[pyQPanda.GateType], arg2: float) -> None

        Set a specific noise model for the density matrix simulator with multiple gate types and a given probability.

        Args:
             noise_model: The noise model to apply.

             gate_types: A vector of gate types associated with the noise model.

             probability: The probability of the noise occurring.

        Returns:
             None.


        7. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: pyQPanda.NoiseModel, arg1: pyQPanda.GateType, arg2: float, arg3: pyQPanda.QVec) -> None

        Set a specific noise model for the density matrix simulator with a given gate type, probability, and target qubits.

        Args:
             noise_model: The noise model to apply.

             gate_type: The specific gate type associated with the noise model.

             probability: The probability of the noise occurring.

             qubits: The target qubits affected by the noise model.

        Returns:
             None.


        8. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: pyQPanda.NoiseModel, arg1: List[pyQPanda.GateType], arg2: float, arg3: pyQPanda.QVec) -> None

        Set a specific noise model for the density matrix simulator with multiple gate types, a given probability, and target qubits.

        Args:
             noise_model: The noise model to apply.

             gate_types: A vector of gate types associated with the noise model.

             probability: The probability of the noise occurring.

             qubits: The target qubits affected by the noise model.

        Returns:
             None.


        9. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: pyQPanda.NoiseModel, arg1: pyQPanda.GateType, arg2: float, arg3: List[pyQPanda.QVec]) -> None

        Set a specific noise model for the density matrix simulator with a given gate type, probability, and groups of target qubits.

        Args:
             noise_model: The noise model to apply.

             gate_type: The specific gate type associated with the noise model.

             probability: The probability of the noise occurring.

             qubit_groups: A vector of QVecs representing groups of target qubits affected by the noise model.

        Returns:
             None.


        10. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: pyQPanda.NoiseModel, arg1: pyQPanda.GateType, arg2: float, arg3: float, arg4: float) -> None

        Set a specific noise model for the density matrix simulator with a given gate type, probability, duration, and temperature.

        Args:
             noise_model: The noise model to apply.

             gate_type: The specific gate type associated with the noise model.

             probability: The probability of the noise occurring.

             duration: The duration for which the noise model is applied.

             temperature: The temperature affecting the noise characteristics.

        Returns:
             None.


        11. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: pyQPanda.NoiseModel, arg1: List[pyQPanda.GateType], arg2: float, arg3: float, arg4: float) -> None

        Set a specific noise model for the density matrix simulator with multiple gate types, a given probability, duration, and temperature.

        Args:
             noise_model: The noise model to apply.

             gate_types: A vector of gate types associated with the noise model.

             probability: The probability of the noise occurring.

             duration: The duration for which the noise model is applied.

             temperature: The temperature affecting the noise characteristics.

        Returns:
             None.


        12. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: pyQPanda.NoiseModel, arg1: pyQPanda.GateType, arg2: float, arg3: float, arg4: float, arg5: pyQPanda.QVec) -> None

        Set a specific noise model for the density matrix simulator with a given gate type, probability, duration, temperature, and a target qubit.

        Args:
             noise_model: The noise model to apply.

             gate_type: The specific gate type associated with the noise model.

             probability: The probability of the noise occurring.

             duration: The duration for which the noise model is applied.

             temperature: The temperature affecting the noise characteristics.

             target_qubit: The specific qubit targeted by the noise model.

        Returns:
             None.


        13. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: pyQPanda.NoiseModel, arg1: List[pyQPanda.GateType], arg2: float, arg3: float, arg4: float, arg5: pyQPanda.QVec) -> None

        Set a specific noise model for the density matrix simulator with multiple gate types, probability, duration, temperature, and a target qubit.

        Args:
             noise_model: The noise model to apply.

             gate_types: A vector of gate types associated with the noise model.

             probability: The probability of the noise occurring.

             duration: The duration for which the noise model is applied.

             temperature: The temperature affecting the noise characteristics.

             target_qubit: The specific qubit targeted by the noise model.

        Returns:
             None.


        14. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: pyQPanda.NoiseModel, arg1: pyQPanda.GateType, arg2: float, arg3: float, arg4: float, arg5: List[pyQPanda.QVec]) -> None

        Set a specific noise model for the density matrix simulator with a given gate type, probability, duration, temperature, and multiple target qubits.

        Args:
             noise_model: The noise model to apply.

             gate_type: The specific gate type associated with the noise model.

             probability: The probability of the noise occurring.

             duration: The duration for which the noise model is applied.

             temperature: The temperature affecting the noise characteristics.

             target_qubits: A vector of qubits targeted by the noise model.

        Returns:
             None.

        """
    @overload
    def set_noise_model(self, arg0: list[numpy.ndarray[numpy.complex128[m, n]]]) -> None:
        """set_noise_model(*args, **kwargs)
        Overloaded function.

        1. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: numpy.ndarray[numpy.complex128[m, n]]) -> None

        Set the noise model for the density matrix simulator.

        Args:
             noise_model: The noise model represented as a complex matrix.

        Returns:
             None.


        2. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: numpy.ndarray[numpy.complex128[m, n]], arg1: List[pyQPanda.GateType]) -> None

        Set the noise model for the density matrix simulator with specific gate types.

        Args:
             noise_model: The noise model represented as a complex matrix.

             gate_types: A vector of gate types to which the noise model applies.

        Returns:
             None.


        3. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: List[numpy.ndarray[numpy.complex128[m, n]]]) -> None

        Set multiple noise models for the density matrix simulator.

        Args:
             noise_models: A vector of noise models, each represented as a complex matrix.

        Returns:
             None.


        4. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: List[numpy.ndarray[numpy.complex128[m, n]]], arg1: List[pyQPanda.GateType]) -> None

        Set multiple noise models for the density matrix simulator with specific gate types.

        Args:
             noise_models: A vector of noise models, each represented as a complex matrix.

             gate_types: A vector of gate types to which the noise models apply.

        Returns:
             None.


        5. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: pyQPanda.NoiseModel, arg1: pyQPanda.GateType, arg2: float) -> None

        Set a specific noise model for the density matrix simulator with a given gate type and probability.

        Args:
             noise_model: The noise model to apply.

             gate_type: The specific gate type associated with the noise model.

             probability: The probability of the noise occurring.

        Returns:
             None.


        6. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: pyQPanda.NoiseModel, arg1: List[pyQPanda.GateType], arg2: float) -> None

        Set a specific noise model for the density matrix simulator with multiple gate types and a given probability.

        Args:
             noise_model: The noise model to apply.

             gate_types: A vector of gate types associated with the noise model.

             probability: The probability of the noise occurring.

        Returns:
             None.


        7. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: pyQPanda.NoiseModel, arg1: pyQPanda.GateType, arg2: float, arg3: pyQPanda.QVec) -> None

        Set a specific noise model for the density matrix simulator with a given gate type, probability, and target qubits.

        Args:
             noise_model: The noise model to apply.

             gate_type: The specific gate type associated with the noise model.

             probability: The probability of the noise occurring.

             qubits: The target qubits affected by the noise model.

        Returns:
             None.


        8. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: pyQPanda.NoiseModel, arg1: List[pyQPanda.GateType], arg2: float, arg3: pyQPanda.QVec) -> None

        Set a specific noise model for the density matrix simulator with multiple gate types, a given probability, and target qubits.

        Args:
             noise_model: The noise model to apply.

             gate_types: A vector of gate types associated with the noise model.

             probability: The probability of the noise occurring.

             qubits: The target qubits affected by the noise model.

        Returns:
             None.


        9. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: pyQPanda.NoiseModel, arg1: pyQPanda.GateType, arg2: float, arg3: List[pyQPanda.QVec]) -> None

        Set a specific noise model for the density matrix simulator with a given gate type, probability, and groups of target qubits.

        Args:
             noise_model: The noise model to apply.

             gate_type: The specific gate type associated with the noise model.

             probability: The probability of the noise occurring.

             qubit_groups: A vector of QVecs representing groups of target qubits affected by the noise model.

        Returns:
             None.


        10. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: pyQPanda.NoiseModel, arg1: pyQPanda.GateType, arg2: float, arg3: float, arg4: float) -> None

        Set a specific noise model for the density matrix simulator with a given gate type, probability, duration, and temperature.

        Args:
             noise_model: The noise model to apply.

             gate_type: The specific gate type associated with the noise model.

             probability: The probability of the noise occurring.

             duration: The duration for which the noise model is applied.

             temperature: The temperature affecting the noise characteristics.

        Returns:
             None.


        11. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: pyQPanda.NoiseModel, arg1: List[pyQPanda.GateType], arg2: float, arg3: float, arg4: float) -> None

        Set a specific noise model for the density matrix simulator with multiple gate types, a given probability, duration, and temperature.

        Args:
             noise_model: The noise model to apply.

             gate_types: A vector of gate types associated with the noise model.

             probability: The probability of the noise occurring.

             duration: The duration for which the noise model is applied.

             temperature: The temperature affecting the noise characteristics.

        Returns:
             None.


        12. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: pyQPanda.NoiseModel, arg1: pyQPanda.GateType, arg2: float, arg3: float, arg4: float, arg5: pyQPanda.QVec) -> None

        Set a specific noise model for the density matrix simulator with a given gate type, probability, duration, temperature, and a target qubit.

        Args:
             noise_model: The noise model to apply.

             gate_type: The specific gate type associated with the noise model.

             probability: The probability of the noise occurring.

             duration: The duration for which the noise model is applied.

             temperature: The temperature affecting the noise characteristics.

             target_qubit: The specific qubit targeted by the noise model.

        Returns:
             None.


        13. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: pyQPanda.NoiseModel, arg1: List[pyQPanda.GateType], arg2: float, arg3: float, arg4: float, arg5: pyQPanda.QVec) -> None

        Set a specific noise model for the density matrix simulator with multiple gate types, probability, duration, temperature, and a target qubit.

        Args:
             noise_model: The noise model to apply.

             gate_types: A vector of gate types associated with the noise model.

             probability: The probability of the noise occurring.

             duration: The duration for which the noise model is applied.

             temperature: The temperature affecting the noise characteristics.

             target_qubit: The specific qubit targeted by the noise model.

        Returns:
             None.


        14. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: pyQPanda.NoiseModel, arg1: pyQPanda.GateType, arg2: float, arg3: float, arg4: float, arg5: List[pyQPanda.QVec]) -> None

        Set a specific noise model for the density matrix simulator with a given gate type, probability, duration, temperature, and multiple target qubits.

        Args:
             noise_model: The noise model to apply.

             gate_type: The specific gate type associated with the noise model.

             probability: The probability of the noise occurring.

             duration: The duration for which the noise model is applied.

             temperature: The temperature affecting the noise characteristics.

             target_qubits: A vector of qubits targeted by the noise model.

        Returns:
             None.

        """
    @overload
    def set_noise_model(self, arg0: list[numpy.ndarray[numpy.complex128[m, n]]], arg1: list[GateType]) -> None:
        """set_noise_model(*args, **kwargs)
        Overloaded function.

        1. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: numpy.ndarray[numpy.complex128[m, n]]) -> None

        Set the noise model for the density matrix simulator.

        Args:
             noise_model: The noise model represented as a complex matrix.

        Returns:
             None.


        2. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: numpy.ndarray[numpy.complex128[m, n]], arg1: List[pyQPanda.GateType]) -> None

        Set the noise model for the density matrix simulator with specific gate types.

        Args:
             noise_model: The noise model represented as a complex matrix.

             gate_types: A vector of gate types to which the noise model applies.

        Returns:
             None.


        3. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: List[numpy.ndarray[numpy.complex128[m, n]]]) -> None

        Set multiple noise models for the density matrix simulator.

        Args:
             noise_models: A vector of noise models, each represented as a complex matrix.

        Returns:
             None.


        4. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: List[numpy.ndarray[numpy.complex128[m, n]]], arg1: List[pyQPanda.GateType]) -> None

        Set multiple noise models for the density matrix simulator with specific gate types.

        Args:
             noise_models: A vector of noise models, each represented as a complex matrix.

             gate_types: A vector of gate types to which the noise models apply.

        Returns:
             None.


        5. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: pyQPanda.NoiseModel, arg1: pyQPanda.GateType, arg2: float) -> None

        Set a specific noise model for the density matrix simulator with a given gate type and probability.

        Args:
             noise_model: The noise model to apply.

             gate_type: The specific gate type associated with the noise model.

             probability: The probability of the noise occurring.

        Returns:
             None.


        6. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: pyQPanda.NoiseModel, arg1: List[pyQPanda.GateType], arg2: float) -> None

        Set a specific noise model for the density matrix simulator with multiple gate types and a given probability.

        Args:
             noise_model: The noise model to apply.

             gate_types: A vector of gate types associated with the noise model.

             probability: The probability of the noise occurring.

        Returns:
             None.


        7. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: pyQPanda.NoiseModel, arg1: pyQPanda.GateType, arg2: float, arg3: pyQPanda.QVec) -> None

        Set a specific noise model for the density matrix simulator with a given gate type, probability, and target qubits.

        Args:
             noise_model: The noise model to apply.

             gate_type: The specific gate type associated with the noise model.

             probability: The probability of the noise occurring.

             qubits: The target qubits affected by the noise model.

        Returns:
             None.


        8. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: pyQPanda.NoiseModel, arg1: List[pyQPanda.GateType], arg2: float, arg3: pyQPanda.QVec) -> None

        Set a specific noise model for the density matrix simulator with multiple gate types, a given probability, and target qubits.

        Args:
             noise_model: The noise model to apply.

             gate_types: A vector of gate types associated with the noise model.

             probability: The probability of the noise occurring.

             qubits: The target qubits affected by the noise model.

        Returns:
             None.


        9. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: pyQPanda.NoiseModel, arg1: pyQPanda.GateType, arg2: float, arg3: List[pyQPanda.QVec]) -> None

        Set a specific noise model for the density matrix simulator with a given gate type, probability, and groups of target qubits.

        Args:
             noise_model: The noise model to apply.

             gate_type: The specific gate type associated with the noise model.

             probability: The probability of the noise occurring.

             qubit_groups: A vector of QVecs representing groups of target qubits affected by the noise model.

        Returns:
             None.


        10. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: pyQPanda.NoiseModel, arg1: pyQPanda.GateType, arg2: float, arg3: float, arg4: float) -> None

        Set a specific noise model for the density matrix simulator with a given gate type, probability, duration, and temperature.

        Args:
             noise_model: The noise model to apply.

             gate_type: The specific gate type associated with the noise model.

             probability: The probability of the noise occurring.

             duration: The duration for which the noise model is applied.

             temperature: The temperature affecting the noise characteristics.

        Returns:
             None.


        11. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: pyQPanda.NoiseModel, arg1: List[pyQPanda.GateType], arg2: float, arg3: float, arg4: float) -> None

        Set a specific noise model for the density matrix simulator with multiple gate types, a given probability, duration, and temperature.

        Args:
             noise_model: The noise model to apply.

             gate_types: A vector of gate types associated with the noise model.

             probability: The probability of the noise occurring.

             duration: The duration for which the noise model is applied.

             temperature: The temperature affecting the noise characteristics.

        Returns:
             None.


        12. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: pyQPanda.NoiseModel, arg1: pyQPanda.GateType, arg2: float, arg3: float, arg4: float, arg5: pyQPanda.QVec) -> None

        Set a specific noise model for the density matrix simulator with a given gate type, probability, duration, temperature, and a target qubit.

        Args:
             noise_model: The noise model to apply.

             gate_type: The specific gate type associated with the noise model.

             probability: The probability of the noise occurring.

             duration: The duration for which the noise model is applied.

             temperature: The temperature affecting the noise characteristics.

             target_qubit: The specific qubit targeted by the noise model.

        Returns:
             None.


        13. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: pyQPanda.NoiseModel, arg1: List[pyQPanda.GateType], arg2: float, arg3: float, arg4: float, arg5: pyQPanda.QVec) -> None

        Set a specific noise model for the density matrix simulator with multiple gate types, probability, duration, temperature, and a target qubit.

        Args:
             noise_model: The noise model to apply.

             gate_types: A vector of gate types associated with the noise model.

             probability: The probability of the noise occurring.

             duration: The duration for which the noise model is applied.

             temperature: The temperature affecting the noise characteristics.

             target_qubit: The specific qubit targeted by the noise model.

        Returns:
             None.


        14. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: pyQPanda.NoiseModel, arg1: pyQPanda.GateType, arg2: float, arg3: float, arg4: float, arg5: List[pyQPanda.QVec]) -> None

        Set a specific noise model for the density matrix simulator with a given gate type, probability, duration, temperature, and multiple target qubits.

        Args:
             noise_model: The noise model to apply.

             gate_type: The specific gate type associated with the noise model.

             probability: The probability of the noise occurring.

             duration: The duration for which the noise model is applied.

             temperature: The temperature affecting the noise characteristics.

             target_qubits: A vector of qubits targeted by the noise model.

        Returns:
             None.

        """
    @overload
    def set_noise_model(self, arg0: NoiseModel, arg1: GateType, arg2: float) -> None:
        """set_noise_model(*args, **kwargs)
        Overloaded function.

        1. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: numpy.ndarray[numpy.complex128[m, n]]) -> None

        Set the noise model for the density matrix simulator.

        Args:
             noise_model: The noise model represented as a complex matrix.

        Returns:
             None.


        2. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: numpy.ndarray[numpy.complex128[m, n]], arg1: List[pyQPanda.GateType]) -> None

        Set the noise model for the density matrix simulator with specific gate types.

        Args:
             noise_model: The noise model represented as a complex matrix.

             gate_types: A vector of gate types to which the noise model applies.

        Returns:
             None.


        3. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: List[numpy.ndarray[numpy.complex128[m, n]]]) -> None

        Set multiple noise models for the density matrix simulator.

        Args:
             noise_models: A vector of noise models, each represented as a complex matrix.

        Returns:
             None.


        4. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: List[numpy.ndarray[numpy.complex128[m, n]]], arg1: List[pyQPanda.GateType]) -> None

        Set multiple noise models for the density matrix simulator with specific gate types.

        Args:
             noise_models: A vector of noise models, each represented as a complex matrix.

             gate_types: A vector of gate types to which the noise models apply.

        Returns:
             None.


        5. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: pyQPanda.NoiseModel, arg1: pyQPanda.GateType, arg2: float) -> None

        Set a specific noise model for the density matrix simulator with a given gate type and probability.

        Args:
             noise_model: The noise model to apply.

             gate_type: The specific gate type associated with the noise model.

             probability: The probability of the noise occurring.

        Returns:
             None.


        6. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: pyQPanda.NoiseModel, arg1: List[pyQPanda.GateType], arg2: float) -> None

        Set a specific noise model for the density matrix simulator with multiple gate types and a given probability.

        Args:
             noise_model: The noise model to apply.

             gate_types: A vector of gate types associated with the noise model.

             probability: The probability of the noise occurring.

        Returns:
             None.


        7. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: pyQPanda.NoiseModel, arg1: pyQPanda.GateType, arg2: float, arg3: pyQPanda.QVec) -> None

        Set a specific noise model for the density matrix simulator with a given gate type, probability, and target qubits.

        Args:
             noise_model: The noise model to apply.

             gate_type: The specific gate type associated with the noise model.

             probability: The probability of the noise occurring.

             qubits: The target qubits affected by the noise model.

        Returns:
             None.


        8. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: pyQPanda.NoiseModel, arg1: List[pyQPanda.GateType], arg2: float, arg3: pyQPanda.QVec) -> None

        Set a specific noise model for the density matrix simulator with multiple gate types, a given probability, and target qubits.

        Args:
             noise_model: The noise model to apply.

             gate_types: A vector of gate types associated with the noise model.

             probability: The probability of the noise occurring.

             qubits: The target qubits affected by the noise model.

        Returns:
             None.


        9. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: pyQPanda.NoiseModel, arg1: pyQPanda.GateType, arg2: float, arg3: List[pyQPanda.QVec]) -> None

        Set a specific noise model for the density matrix simulator with a given gate type, probability, and groups of target qubits.

        Args:
             noise_model: The noise model to apply.

             gate_type: The specific gate type associated with the noise model.

             probability: The probability of the noise occurring.

             qubit_groups: A vector of QVecs representing groups of target qubits affected by the noise model.

        Returns:
             None.


        10. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: pyQPanda.NoiseModel, arg1: pyQPanda.GateType, arg2: float, arg3: float, arg4: float) -> None

        Set a specific noise model for the density matrix simulator with a given gate type, probability, duration, and temperature.

        Args:
             noise_model: The noise model to apply.

             gate_type: The specific gate type associated with the noise model.

             probability: The probability of the noise occurring.

             duration: The duration for which the noise model is applied.

             temperature: The temperature affecting the noise characteristics.

        Returns:
             None.


        11. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: pyQPanda.NoiseModel, arg1: List[pyQPanda.GateType], arg2: float, arg3: float, arg4: float) -> None

        Set a specific noise model for the density matrix simulator with multiple gate types, a given probability, duration, and temperature.

        Args:
             noise_model: The noise model to apply.

             gate_types: A vector of gate types associated with the noise model.

             probability: The probability of the noise occurring.

             duration: The duration for which the noise model is applied.

             temperature: The temperature affecting the noise characteristics.

        Returns:
             None.


        12. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: pyQPanda.NoiseModel, arg1: pyQPanda.GateType, arg2: float, arg3: float, arg4: float, arg5: pyQPanda.QVec) -> None

        Set a specific noise model for the density matrix simulator with a given gate type, probability, duration, temperature, and a target qubit.

        Args:
             noise_model: The noise model to apply.

             gate_type: The specific gate type associated with the noise model.

             probability: The probability of the noise occurring.

             duration: The duration for which the noise model is applied.

             temperature: The temperature affecting the noise characteristics.

             target_qubit: The specific qubit targeted by the noise model.

        Returns:
             None.


        13. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: pyQPanda.NoiseModel, arg1: List[pyQPanda.GateType], arg2: float, arg3: float, arg4: float, arg5: pyQPanda.QVec) -> None

        Set a specific noise model for the density matrix simulator with multiple gate types, probability, duration, temperature, and a target qubit.

        Args:
             noise_model: The noise model to apply.

             gate_types: A vector of gate types associated with the noise model.

             probability: The probability of the noise occurring.

             duration: The duration for which the noise model is applied.

             temperature: The temperature affecting the noise characteristics.

             target_qubit: The specific qubit targeted by the noise model.

        Returns:
             None.


        14. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: pyQPanda.NoiseModel, arg1: pyQPanda.GateType, arg2: float, arg3: float, arg4: float, arg5: List[pyQPanda.QVec]) -> None

        Set a specific noise model for the density matrix simulator with a given gate type, probability, duration, temperature, and multiple target qubits.

        Args:
             noise_model: The noise model to apply.

             gate_type: The specific gate type associated with the noise model.

             probability: The probability of the noise occurring.

             duration: The duration for which the noise model is applied.

             temperature: The temperature affecting the noise characteristics.

             target_qubits: A vector of qubits targeted by the noise model.

        Returns:
             None.

        """
    @overload
    def set_noise_model(self, arg0: NoiseModel, arg1: list[GateType], arg2: float) -> None:
        """set_noise_model(*args, **kwargs)
        Overloaded function.

        1. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: numpy.ndarray[numpy.complex128[m, n]]) -> None

        Set the noise model for the density matrix simulator.

        Args:
             noise_model: The noise model represented as a complex matrix.

        Returns:
             None.


        2. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: numpy.ndarray[numpy.complex128[m, n]], arg1: List[pyQPanda.GateType]) -> None

        Set the noise model for the density matrix simulator with specific gate types.

        Args:
             noise_model: The noise model represented as a complex matrix.

             gate_types: A vector of gate types to which the noise model applies.

        Returns:
             None.


        3. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: List[numpy.ndarray[numpy.complex128[m, n]]]) -> None

        Set multiple noise models for the density matrix simulator.

        Args:
             noise_models: A vector of noise models, each represented as a complex matrix.

        Returns:
             None.


        4. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: List[numpy.ndarray[numpy.complex128[m, n]]], arg1: List[pyQPanda.GateType]) -> None

        Set multiple noise models for the density matrix simulator with specific gate types.

        Args:
             noise_models: A vector of noise models, each represented as a complex matrix.

             gate_types: A vector of gate types to which the noise models apply.

        Returns:
             None.


        5. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: pyQPanda.NoiseModel, arg1: pyQPanda.GateType, arg2: float) -> None

        Set a specific noise model for the density matrix simulator with a given gate type and probability.

        Args:
             noise_model: The noise model to apply.

             gate_type: The specific gate type associated with the noise model.

             probability: The probability of the noise occurring.

        Returns:
             None.


        6. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: pyQPanda.NoiseModel, arg1: List[pyQPanda.GateType], arg2: float) -> None

        Set a specific noise model for the density matrix simulator with multiple gate types and a given probability.

        Args:
             noise_model: The noise model to apply.

             gate_types: A vector of gate types associated with the noise model.

             probability: The probability of the noise occurring.

        Returns:
             None.


        7. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: pyQPanda.NoiseModel, arg1: pyQPanda.GateType, arg2: float, arg3: pyQPanda.QVec) -> None

        Set a specific noise model for the density matrix simulator with a given gate type, probability, and target qubits.

        Args:
             noise_model: The noise model to apply.

             gate_type: The specific gate type associated with the noise model.

             probability: The probability of the noise occurring.

             qubits: The target qubits affected by the noise model.

        Returns:
             None.


        8. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: pyQPanda.NoiseModel, arg1: List[pyQPanda.GateType], arg2: float, arg3: pyQPanda.QVec) -> None

        Set a specific noise model for the density matrix simulator with multiple gate types, a given probability, and target qubits.

        Args:
             noise_model: The noise model to apply.

             gate_types: A vector of gate types associated with the noise model.

             probability: The probability of the noise occurring.

             qubits: The target qubits affected by the noise model.

        Returns:
             None.


        9. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: pyQPanda.NoiseModel, arg1: pyQPanda.GateType, arg2: float, arg3: List[pyQPanda.QVec]) -> None

        Set a specific noise model for the density matrix simulator with a given gate type, probability, and groups of target qubits.

        Args:
             noise_model: The noise model to apply.

             gate_type: The specific gate type associated with the noise model.

             probability: The probability of the noise occurring.

             qubit_groups: A vector of QVecs representing groups of target qubits affected by the noise model.

        Returns:
             None.


        10. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: pyQPanda.NoiseModel, arg1: pyQPanda.GateType, arg2: float, arg3: float, arg4: float) -> None

        Set a specific noise model for the density matrix simulator with a given gate type, probability, duration, and temperature.

        Args:
             noise_model: The noise model to apply.

             gate_type: The specific gate type associated with the noise model.

             probability: The probability of the noise occurring.

             duration: The duration for which the noise model is applied.

             temperature: The temperature affecting the noise characteristics.

        Returns:
             None.


        11. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: pyQPanda.NoiseModel, arg1: List[pyQPanda.GateType], arg2: float, arg3: float, arg4: float) -> None

        Set a specific noise model for the density matrix simulator with multiple gate types, a given probability, duration, and temperature.

        Args:
             noise_model: The noise model to apply.

             gate_types: A vector of gate types associated with the noise model.

             probability: The probability of the noise occurring.

             duration: The duration for which the noise model is applied.

             temperature: The temperature affecting the noise characteristics.

        Returns:
             None.


        12. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: pyQPanda.NoiseModel, arg1: pyQPanda.GateType, arg2: float, arg3: float, arg4: float, arg5: pyQPanda.QVec) -> None

        Set a specific noise model for the density matrix simulator with a given gate type, probability, duration, temperature, and a target qubit.

        Args:
             noise_model: The noise model to apply.

             gate_type: The specific gate type associated with the noise model.

             probability: The probability of the noise occurring.

             duration: The duration for which the noise model is applied.

             temperature: The temperature affecting the noise characteristics.

             target_qubit: The specific qubit targeted by the noise model.

        Returns:
             None.


        13. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: pyQPanda.NoiseModel, arg1: List[pyQPanda.GateType], arg2: float, arg3: float, arg4: float, arg5: pyQPanda.QVec) -> None

        Set a specific noise model for the density matrix simulator with multiple gate types, probability, duration, temperature, and a target qubit.

        Args:
             noise_model: The noise model to apply.

             gate_types: A vector of gate types associated with the noise model.

             probability: The probability of the noise occurring.

             duration: The duration for which the noise model is applied.

             temperature: The temperature affecting the noise characteristics.

             target_qubit: The specific qubit targeted by the noise model.

        Returns:
             None.


        14. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: pyQPanda.NoiseModel, arg1: pyQPanda.GateType, arg2: float, arg3: float, arg4: float, arg5: List[pyQPanda.QVec]) -> None

        Set a specific noise model for the density matrix simulator with a given gate type, probability, duration, temperature, and multiple target qubits.

        Args:
             noise_model: The noise model to apply.

             gate_type: The specific gate type associated with the noise model.

             probability: The probability of the noise occurring.

             duration: The duration for which the noise model is applied.

             temperature: The temperature affecting the noise characteristics.

             target_qubits: A vector of qubits targeted by the noise model.

        Returns:
             None.

        """
    @overload
    def set_noise_model(self, arg0: NoiseModel, arg1: GateType, arg2: float, arg3: QVec) -> None:
        """set_noise_model(*args, **kwargs)
        Overloaded function.

        1. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: numpy.ndarray[numpy.complex128[m, n]]) -> None

        Set the noise model for the density matrix simulator.

        Args:
             noise_model: The noise model represented as a complex matrix.

        Returns:
             None.


        2. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: numpy.ndarray[numpy.complex128[m, n]], arg1: List[pyQPanda.GateType]) -> None

        Set the noise model for the density matrix simulator with specific gate types.

        Args:
             noise_model: The noise model represented as a complex matrix.

             gate_types: A vector of gate types to which the noise model applies.

        Returns:
             None.


        3. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: List[numpy.ndarray[numpy.complex128[m, n]]]) -> None

        Set multiple noise models for the density matrix simulator.

        Args:
             noise_models: A vector of noise models, each represented as a complex matrix.

        Returns:
             None.


        4. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: List[numpy.ndarray[numpy.complex128[m, n]]], arg1: List[pyQPanda.GateType]) -> None

        Set multiple noise models for the density matrix simulator with specific gate types.

        Args:
             noise_models: A vector of noise models, each represented as a complex matrix.

             gate_types: A vector of gate types to which the noise models apply.

        Returns:
             None.


        5. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: pyQPanda.NoiseModel, arg1: pyQPanda.GateType, arg2: float) -> None

        Set a specific noise model for the density matrix simulator with a given gate type and probability.

        Args:
             noise_model: The noise model to apply.

             gate_type: The specific gate type associated with the noise model.

             probability: The probability of the noise occurring.

        Returns:
             None.


        6. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: pyQPanda.NoiseModel, arg1: List[pyQPanda.GateType], arg2: float) -> None

        Set a specific noise model for the density matrix simulator with multiple gate types and a given probability.

        Args:
             noise_model: The noise model to apply.

             gate_types: A vector of gate types associated with the noise model.

             probability: The probability of the noise occurring.

        Returns:
             None.


        7. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: pyQPanda.NoiseModel, arg1: pyQPanda.GateType, arg2: float, arg3: pyQPanda.QVec) -> None

        Set a specific noise model for the density matrix simulator with a given gate type, probability, and target qubits.

        Args:
             noise_model: The noise model to apply.

             gate_type: The specific gate type associated with the noise model.

             probability: The probability of the noise occurring.

             qubits: The target qubits affected by the noise model.

        Returns:
             None.


        8. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: pyQPanda.NoiseModel, arg1: List[pyQPanda.GateType], arg2: float, arg3: pyQPanda.QVec) -> None

        Set a specific noise model for the density matrix simulator with multiple gate types, a given probability, and target qubits.

        Args:
             noise_model: The noise model to apply.

             gate_types: A vector of gate types associated with the noise model.

             probability: The probability of the noise occurring.

             qubits: The target qubits affected by the noise model.

        Returns:
             None.


        9. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: pyQPanda.NoiseModel, arg1: pyQPanda.GateType, arg2: float, arg3: List[pyQPanda.QVec]) -> None

        Set a specific noise model for the density matrix simulator with a given gate type, probability, and groups of target qubits.

        Args:
             noise_model: The noise model to apply.

             gate_type: The specific gate type associated with the noise model.

             probability: The probability of the noise occurring.

             qubit_groups: A vector of QVecs representing groups of target qubits affected by the noise model.

        Returns:
             None.


        10. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: pyQPanda.NoiseModel, arg1: pyQPanda.GateType, arg2: float, arg3: float, arg4: float) -> None

        Set a specific noise model for the density matrix simulator with a given gate type, probability, duration, and temperature.

        Args:
             noise_model: The noise model to apply.

             gate_type: The specific gate type associated with the noise model.

             probability: The probability of the noise occurring.

             duration: The duration for which the noise model is applied.

             temperature: The temperature affecting the noise characteristics.

        Returns:
             None.


        11. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: pyQPanda.NoiseModel, arg1: List[pyQPanda.GateType], arg2: float, arg3: float, arg4: float) -> None

        Set a specific noise model for the density matrix simulator with multiple gate types, a given probability, duration, and temperature.

        Args:
             noise_model: The noise model to apply.

             gate_types: A vector of gate types associated with the noise model.

             probability: The probability of the noise occurring.

             duration: The duration for which the noise model is applied.

             temperature: The temperature affecting the noise characteristics.

        Returns:
             None.


        12. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: pyQPanda.NoiseModel, arg1: pyQPanda.GateType, arg2: float, arg3: float, arg4: float, arg5: pyQPanda.QVec) -> None

        Set a specific noise model for the density matrix simulator with a given gate type, probability, duration, temperature, and a target qubit.

        Args:
             noise_model: The noise model to apply.

             gate_type: The specific gate type associated with the noise model.

             probability: The probability of the noise occurring.

             duration: The duration for which the noise model is applied.

             temperature: The temperature affecting the noise characteristics.

             target_qubit: The specific qubit targeted by the noise model.

        Returns:
             None.


        13. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: pyQPanda.NoiseModel, arg1: List[pyQPanda.GateType], arg2: float, arg3: float, arg4: float, arg5: pyQPanda.QVec) -> None

        Set a specific noise model for the density matrix simulator with multiple gate types, probability, duration, temperature, and a target qubit.

        Args:
             noise_model: The noise model to apply.

             gate_types: A vector of gate types associated with the noise model.

             probability: The probability of the noise occurring.

             duration: The duration for which the noise model is applied.

             temperature: The temperature affecting the noise characteristics.

             target_qubit: The specific qubit targeted by the noise model.

        Returns:
             None.


        14. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: pyQPanda.NoiseModel, arg1: pyQPanda.GateType, arg2: float, arg3: float, arg4: float, arg5: List[pyQPanda.QVec]) -> None

        Set a specific noise model for the density matrix simulator with a given gate type, probability, duration, temperature, and multiple target qubits.

        Args:
             noise_model: The noise model to apply.

             gate_type: The specific gate type associated with the noise model.

             probability: The probability of the noise occurring.

             duration: The duration for which the noise model is applied.

             temperature: The temperature affecting the noise characteristics.

             target_qubits: A vector of qubits targeted by the noise model.

        Returns:
             None.

        """
    @overload
    def set_noise_model(self, arg0: NoiseModel, arg1: list[GateType], arg2: float, arg3: QVec) -> None:
        """set_noise_model(*args, **kwargs)
        Overloaded function.

        1. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: numpy.ndarray[numpy.complex128[m, n]]) -> None

        Set the noise model for the density matrix simulator.

        Args:
             noise_model: The noise model represented as a complex matrix.

        Returns:
             None.


        2. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: numpy.ndarray[numpy.complex128[m, n]], arg1: List[pyQPanda.GateType]) -> None

        Set the noise model for the density matrix simulator with specific gate types.

        Args:
             noise_model: The noise model represented as a complex matrix.

             gate_types: A vector of gate types to which the noise model applies.

        Returns:
             None.


        3. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: List[numpy.ndarray[numpy.complex128[m, n]]]) -> None

        Set multiple noise models for the density matrix simulator.

        Args:
             noise_models: A vector of noise models, each represented as a complex matrix.

        Returns:
             None.


        4. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: List[numpy.ndarray[numpy.complex128[m, n]]], arg1: List[pyQPanda.GateType]) -> None

        Set multiple noise models for the density matrix simulator with specific gate types.

        Args:
             noise_models: A vector of noise models, each represented as a complex matrix.

             gate_types: A vector of gate types to which the noise models apply.

        Returns:
             None.


        5. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: pyQPanda.NoiseModel, arg1: pyQPanda.GateType, arg2: float) -> None

        Set a specific noise model for the density matrix simulator with a given gate type and probability.

        Args:
             noise_model: The noise model to apply.

             gate_type: The specific gate type associated with the noise model.

             probability: The probability of the noise occurring.

        Returns:
             None.


        6. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: pyQPanda.NoiseModel, arg1: List[pyQPanda.GateType], arg2: float) -> None

        Set a specific noise model for the density matrix simulator with multiple gate types and a given probability.

        Args:
             noise_model: The noise model to apply.

             gate_types: A vector of gate types associated with the noise model.

             probability: The probability of the noise occurring.

        Returns:
             None.


        7. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: pyQPanda.NoiseModel, arg1: pyQPanda.GateType, arg2: float, arg3: pyQPanda.QVec) -> None

        Set a specific noise model for the density matrix simulator with a given gate type, probability, and target qubits.

        Args:
             noise_model: The noise model to apply.

             gate_type: The specific gate type associated with the noise model.

             probability: The probability of the noise occurring.

             qubits: The target qubits affected by the noise model.

        Returns:
             None.


        8. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: pyQPanda.NoiseModel, arg1: List[pyQPanda.GateType], arg2: float, arg3: pyQPanda.QVec) -> None

        Set a specific noise model for the density matrix simulator with multiple gate types, a given probability, and target qubits.

        Args:
             noise_model: The noise model to apply.

             gate_types: A vector of gate types associated with the noise model.

             probability: The probability of the noise occurring.

             qubits: The target qubits affected by the noise model.

        Returns:
             None.


        9. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: pyQPanda.NoiseModel, arg1: pyQPanda.GateType, arg2: float, arg3: List[pyQPanda.QVec]) -> None

        Set a specific noise model for the density matrix simulator with a given gate type, probability, and groups of target qubits.

        Args:
             noise_model: The noise model to apply.

             gate_type: The specific gate type associated with the noise model.

             probability: The probability of the noise occurring.

             qubit_groups: A vector of QVecs representing groups of target qubits affected by the noise model.

        Returns:
             None.


        10. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: pyQPanda.NoiseModel, arg1: pyQPanda.GateType, arg2: float, arg3: float, arg4: float) -> None

        Set a specific noise model for the density matrix simulator with a given gate type, probability, duration, and temperature.

        Args:
             noise_model: The noise model to apply.

             gate_type: The specific gate type associated with the noise model.

             probability: The probability of the noise occurring.

             duration: The duration for which the noise model is applied.

             temperature: The temperature affecting the noise characteristics.

        Returns:
             None.


        11. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: pyQPanda.NoiseModel, arg1: List[pyQPanda.GateType], arg2: float, arg3: float, arg4: float) -> None

        Set a specific noise model for the density matrix simulator with multiple gate types, a given probability, duration, and temperature.

        Args:
             noise_model: The noise model to apply.

             gate_types: A vector of gate types associated with the noise model.

             probability: The probability of the noise occurring.

             duration: The duration for which the noise model is applied.

             temperature: The temperature affecting the noise characteristics.

        Returns:
             None.


        12. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: pyQPanda.NoiseModel, arg1: pyQPanda.GateType, arg2: float, arg3: float, arg4: float, arg5: pyQPanda.QVec) -> None

        Set a specific noise model for the density matrix simulator with a given gate type, probability, duration, temperature, and a target qubit.

        Args:
             noise_model: The noise model to apply.

             gate_type: The specific gate type associated with the noise model.

             probability: The probability of the noise occurring.

             duration: The duration for which the noise model is applied.

             temperature: The temperature affecting the noise characteristics.

             target_qubit: The specific qubit targeted by the noise model.

        Returns:
             None.


        13. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: pyQPanda.NoiseModel, arg1: List[pyQPanda.GateType], arg2: float, arg3: float, arg4: float, arg5: pyQPanda.QVec) -> None

        Set a specific noise model for the density matrix simulator with multiple gate types, probability, duration, temperature, and a target qubit.

        Args:
             noise_model: The noise model to apply.

             gate_types: A vector of gate types associated with the noise model.

             probability: The probability of the noise occurring.

             duration: The duration for which the noise model is applied.

             temperature: The temperature affecting the noise characteristics.

             target_qubit: The specific qubit targeted by the noise model.

        Returns:
             None.


        14. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: pyQPanda.NoiseModel, arg1: pyQPanda.GateType, arg2: float, arg3: float, arg4: float, arg5: List[pyQPanda.QVec]) -> None

        Set a specific noise model for the density matrix simulator with a given gate type, probability, duration, temperature, and multiple target qubits.

        Args:
             noise_model: The noise model to apply.

             gate_type: The specific gate type associated with the noise model.

             probability: The probability of the noise occurring.

             duration: The duration for which the noise model is applied.

             temperature: The temperature affecting the noise characteristics.

             target_qubits: A vector of qubits targeted by the noise model.

        Returns:
             None.

        """
    @overload
    def set_noise_model(self, arg0: NoiseModel, arg1: GateType, arg2: float, arg3: list[QVec]) -> None:
        """set_noise_model(*args, **kwargs)
        Overloaded function.

        1. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: numpy.ndarray[numpy.complex128[m, n]]) -> None

        Set the noise model for the density matrix simulator.

        Args:
             noise_model: The noise model represented as a complex matrix.

        Returns:
             None.


        2. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: numpy.ndarray[numpy.complex128[m, n]], arg1: List[pyQPanda.GateType]) -> None

        Set the noise model for the density matrix simulator with specific gate types.

        Args:
             noise_model: The noise model represented as a complex matrix.

             gate_types: A vector of gate types to which the noise model applies.

        Returns:
             None.


        3. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: List[numpy.ndarray[numpy.complex128[m, n]]]) -> None

        Set multiple noise models for the density matrix simulator.

        Args:
             noise_models: A vector of noise models, each represented as a complex matrix.

        Returns:
             None.


        4. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: List[numpy.ndarray[numpy.complex128[m, n]]], arg1: List[pyQPanda.GateType]) -> None

        Set multiple noise models for the density matrix simulator with specific gate types.

        Args:
             noise_models: A vector of noise models, each represented as a complex matrix.

             gate_types: A vector of gate types to which the noise models apply.

        Returns:
             None.


        5. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: pyQPanda.NoiseModel, arg1: pyQPanda.GateType, arg2: float) -> None

        Set a specific noise model for the density matrix simulator with a given gate type and probability.

        Args:
             noise_model: The noise model to apply.

             gate_type: The specific gate type associated with the noise model.

             probability: The probability of the noise occurring.

        Returns:
             None.


        6. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: pyQPanda.NoiseModel, arg1: List[pyQPanda.GateType], arg2: float) -> None

        Set a specific noise model for the density matrix simulator with multiple gate types and a given probability.

        Args:
             noise_model: The noise model to apply.

             gate_types: A vector of gate types associated with the noise model.

             probability: The probability of the noise occurring.

        Returns:
             None.


        7. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: pyQPanda.NoiseModel, arg1: pyQPanda.GateType, arg2: float, arg3: pyQPanda.QVec) -> None

        Set a specific noise model for the density matrix simulator with a given gate type, probability, and target qubits.

        Args:
             noise_model: The noise model to apply.

             gate_type: The specific gate type associated with the noise model.

             probability: The probability of the noise occurring.

             qubits: The target qubits affected by the noise model.

        Returns:
             None.


        8. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: pyQPanda.NoiseModel, arg1: List[pyQPanda.GateType], arg2: float, arg3: pyQPanda.QVec) -> None

        Set a specific noise model for the density matrix simulator with multiple gate types, a given probability, and target qubits.

        Args:
             noise_model: The noise model to apply.

             gate_types: A vector of gate types associated with the noise model.

             probability: The probability of the noise occurring.

             qubits: The target qubits affected by the noise model.

        Returns:
             None.


        9. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: pyQPanda.NoiseModel, arg1: pyQPanda.GateType, arg2: float, arg3: List[pyQPanda.QVec]) -> None

        Set a specific noise model for the density matrix simulator with a given gate type, probability, and groups of target qubits.

        Args:
             noise_model: The noise model to apply.

             gate_type: The specific gate type associated with the noise model.

             probability: The probability of the noise occurring.

             qubit_groups: A vector of QVecs representing groups of target qubits affected by the noise model.

        Returns:
             None.


        10. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: pyQPanda.NoiseModel, arg1: pyQPanda.GateType, arg2: float, arg3: float, arg4: float) -> None

        Set a specific noise model for the density matrix simulator with a given gate type, probability, duration, and temperature.

        Args:
             noise_model: The noise model to apply.

             gate_type: The specific gate type associated with the noise model.

             probability: The probability of the noise occurring.

             duration: The duration for which the noise model is applied.

             temperature: The temperature affecting the noise characteristics.

        Returns:
             None.


        11. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: pyQPanda.NoiseModel, arg1: List[pyQPanda.GateType], arg2: float, arg3: float, arg4: float) -> None

        Set a specific noise model for the density matrix simulator with multiple gate types, a given probability, duration, and temperature.

        Args:
             noise_model: The noise model to apply.

             gate_types: A vector of gate types associated with the noise model.

             probability: The probability of the noise occurring.

             duration: The duration for which the noise model is applied.

             temperature: The temperature affecting the noise characteristics.

        Returns:
             None.


        12. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: pyQPanda.NoiseModel, arg1: pyQPanda.GateType, arg2: float, arg3: float, arg4: float, arg5: pyQPanda.QVec) -> None

        Set a specific noise model for the density matrix simulator with a given gate type, probability, duration, temperature, and a target qubit.

        Args:
             noise_model: The noise model to apply.

             gate_type: The specific gate type associated with the noise model.

             probability: The probability of the noise occurring.

             duration: The duration for which the noise model is applied.

             temperature: The temperature affecting the noise characteristics.

             target_qubit: The specific qubit targeted by the noise model.

        Returns:
             None.


        13. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: pyQPanda.NoiseModel, arg1: List[pyQPanda.GateType], arg2: float, arg3: float, arg4: float, arg5: pyQPanda.QVec) -> None

        Set a specific noise model for the density matrix simulator with multiple gate types, probability, duration, temperature, and a target qubit.

        Args:
             noise_model: The noise model to apply.

             gate_types: A vector of gate types associated with the noise model.

             probability: The probability of the noise occurring.

             duration: The duration for which the noise model is applied.

             temperature: The temperature affecting the noise characteristics.

             target_qubit: The specific qubit targeted by the noise model.

        Returns:
             None.


        14. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: pyQPanda.NoiseModel, arg1: pyQPanda.GateType, arg2: float, arg3: float, arg4: float, arg5: List[pyQPanda.QVec]) -> None

        Set a specific noise model for the density matrix simulator with a given gate type, probability, duration, temperature, and multiple target qubits.

        Args:
             noise_model: The noise model to apply.

             gate_type: The specific gate type associated with the noise model.

             probability: The probability of the noise occurring.

             duration: The duration for which the noise model is applied.

             temperature: The temperature affecting the noise characteristics.

             target_qubits: A vector of qubits targeted by the noise model.

        Returns:
             None.

        """
    @overload
    def set_noise_model(self, arg0: NoiseModel, arg1: GateType, arg2: float, arg3: float, arg4: float) -> None:
        """set_noise_model(*args, **kwargs)
        Overloaded function.

        1. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: numpy.ndarray[numpy.complex128[m, n]]) -> None

        Set the noise model for the density matrix simulator.

        Args:
             noise_model: The noise model represented as a complex matrix.

        Returns:
             None.


        2. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: numpy.ndarray[numpy.complex128[m, n]], arg1: List[pyQPanda.GateType]) -> None

        Set the noise model for the density matrix simulator with specific gate types.

        Args:
             noise_model: The noise model represented as a complex matrix.

             gate_types: A vector of gate types to which the noise model applies.

        Returns:
             None.


        3. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: List[numpy.ndarray[numpy.complex128[m, n]]]) -> None

        Set multiple noise models for the density matrix simulator.

        Args:
             noise_models: A vector of noise models, each represented as a complex matrix.

        Returns:
             None.


        4. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: List[numpy.ndarray[numpy.complex128[m, n]]], arg1: List[pyQPanda.GateType]) -> None

        Set multiple noise models for the density matrix simulator with specific gate types.

        Args:
             noise_models: A vector of noise models, each represented as a complex matrix.

             gate_types: A vector of gate types to which the noise models apply.

        Returns:
             None.


        5. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: pyQPanda.NoiseModel, arg1: pyQPanda.GateType, arg2: float) -> None

        Set a specific noise model for the density matrix simulator with a given gate type and probability.

        Args:
             noise_model: The noise model to apply.

             gate_type: The specific gate type associated with the noise model.

             probability: The probability of the noise occurring.

        Returns:
             None.


        6. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: pyQPanda.NoiseModel, arg1: List[pyQPanda.GateType], arg2: float) -> None

        Set a specific noise model for the density matrix simulator with multiple gate types and a given probability.

        Args:
             noise_model: The noise model to apply.

             gate_types: A vector of gate types associated with the noise model.

             probability: The probability of the noise occurring.

        Returns:
             None.


        7. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: pyQPanda.NoiseModel, arg1: pyQPanda.GateType, arg2: float, arg3: pyQPanda.QVec) -> None

        Set a specific noise model for the density matrix simulator with a given gate type, probability, and target qubits.

        Args:
             noise_model: The noise model to apply.

             gate_type: The specific gate type associated with the noise model.

             probability: The probability of the noise occurring.

             qubits: The target qubits affected by the noise model.

        Returns:
             None.


        8. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: pyQPanda.NoiseModel, arg1: List[pyQPanda.GateType], arg2: float, arg3: pyQPanda.QVec) -> None

        Set a specific noise model for the density matrix simulator with multiple gate types, a given probability, and target qubits.

        Args:
             noise_model: The noise model to apply.

             gate_types: A vector of gate types associated with the noise model.

             probability: The probability of the noise occurring.

             qubits: The target qubits affected by the noise model.

        Returns:
             None.


        9. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: pyQPanda.NoiseModel, arg1: pyQPanda.GateType, arg2: float, arg3: List[pyQPanda.QVec]) -> None

        Set a specific noise model for the density matrix simulator with a given gate type, probability, and groups of target qubits.

        Args:
             noise_model: The noise model to apply.

             gate_type: The specific gate type associated with the noise model.

             probability: The probability of the noise occurring.

             qubit_groups: A vector of QVecs representing groups of target qubits affected by the noise model.

        Returns:
             None.


        10. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: pyQPanda.NoiseModel, arg1: pyQPanda.GateType, arg2: float, arg3: float, arg4: float) -> None

        Set a specific noise model for the density matrix simulator with a given gate type, probability, duration, and temperature.

        Args:
             noise_model: The noise model to apply.

             gate_type: The specific gate type associated with the noise model.

             probability: The probability of the noise occurring.

             duration: The duration for which the noise model is applied.

             temperature: The temperature affecting the noise characteristics.

        Returns:
             None.


        11. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: pyQPanda.NoiseModel, arg1: List[pyQPanda.GateType], arg2: float, arg3: float, arg4: float) -> None

        Set a specific noise model for the density matrix simulator with multiple gate types, a given probability, duration, and temperature.

        Args:
             noise_model: The noise model to apply.

             gate_types: A vector of gate types associated with the noise model.

             probability: The probability of the noise occurring.

             duration: The duration for which the noise model is applied.

             temperature: The temperature affecting the noise characteristics.

        Returns:
             None.


        12. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: pyQPanda.NoiseModel, arg1: pyQPanda.GateType, arg2: float, arg3: float, arg4: float, arg5: pyQPanda.QVec) -> None

        Set a specific noise model for the density matrix simulator with a given gate type, probability, duration, temperature, and a target qubit.

        Args:
             noise_model: The noise model to apply.

             gate_type: The specific gate type associated with the noise model.

             probability: The probability of the noise occurring.

             duration: The duration for which the noise model is applied.

             temperature: The temperature affecting the noise characteristics.

             target_qubit: The specific qubit targeted by the noise model.

        Returns:
             None.


        13. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: pyQPanda.NoiseModel, arg1: List[pyQPanda.GateType], arg2: float, arg3: float, arg4: float, arg5: pyQPanda.QVec) -> None

        Set a specific noise model for the density matrix simulator with multiple gate types, probability, duration, temperature, and a target qubit.

        Args:
             noise_model: The noise model to apply.

             gate_types: A vector of gate types associated with the noise model.

             probability: The probability of the noise occurring.

             duration: The duration for which the noise model is applied.

             temperature: The temperature affecting the noise characteristics.

             target_qubit: The specific qubit targeted by the noise model.

        Returns:
             None.


        14. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: pyQPanda.NoiseModel, arg1: pyQPanda.GateType, arg2: float, arg3: float, arg4: float, arg5: List[pyQPanda.QVec]) -> None

        Set a specific noise model for the density matrix simulator with a given gate type, probability, duration, temperature, and multiple target qubits.

        Args:
             noise_model: The noise model to apply.

             gate_type: The specific gate type associated with the noise model.

             probability: The probability of the noise occurring.

             duration: The duration for which the noise model is applied.

             temperature: The temperature affecting the noise characteristics.

             target_qubits: A vector of qubits targeted by the noise model.

        Returns:
             None.

        """
    @overload
    def set_noise_model(self, arg0: NoiseModel, arg1: list[GateType], arg2: float, arg3: float, arg4: float) -> None:
        """set_noise_model(*args, **kwargs)
        Overloaded function.

        1. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: numpy.ndarray[numpy.complex128[m, n]]) -> None

        Set the noise model for the density matrix simulator.

        Args:
             noise_model: The noise model represented as a complex matrix.

        Returns:
             None.


        2. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: numpy.ndarray[numpy.complex128[m, n]], arg1: List[pyQPanda.GateType]) -> None

        Set the noise model for the density matrix simulator with specific gate types.

        Args:
             noise_model: The noise model represented as a complex matrix.

             gate_types: A vector of gate types to which the noise model applies.

        Returns:
             None.


        3. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: List[numpy.ndarray[numpy.complex128[m, n]]]) -> None

        Set multiple noise models for the density matrix simulator.

        Args:
             noise_models: A vector of noise models, each represented as a complex matrix.

        Returns:
             None.


        4. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: List[numpy.ndarray[numpy.complex128[m, n]]], arg1: List[pyQPanda.GateType]) -> None

        Set multiple noise models for the density matrix simulator with specific gate types.

        Args:
             noise_models: A vector of noise models, each represented as a complex matrix.

             gate_types: A vector of gate types to which the noise models apply.

        Returns:
             None.


        5. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: pyQPanda.NoiseModel, arg1: pyQPanda.GateType, arg2: float) -> None

        Set a specific noise model for the density matrix simulator with a given gate type and probability.

        Args:
             noise_model: The noise model to apply.

             gate_type: The specific gate type associated with the noise model.

             probability: The probability of the noise occurring.

        Returns:
             None.


        6. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: pyQPanda.NoiseModel, arg1: List[pyQPanda.GateType], arg2: float) -> None

        Set a specific noise model for the density matrix simulator with multiple gate types and a given probability.

        Args:
             noise_model: The noise model to apply.

             gate_types: A vector of gate types associated with the noise model.

             probability: The probability of the noise occurring.

        Returns:
             None.


        7. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: pyQPanda.NoiseModel, arg1: pyQPanda.GateType, arg2: float, arg3: pyQPanda.QVec) -> None

        Set a specific noise model for the density matrix simulator with a given gate type, probability, and target qubits.

        Args:
             noise_model: The noise model to apply.

             gate_type: The specific gate type associated with the noise model.

             probability: The probability of the noise occurring.

             qubits: The target qubits affected by the noise model.

        Returns:
             None.


        8. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: pyQPanda.NoiseModel, arg1: List[pyQPanda.GateType], arg2: float, arg3: pyQPanda.QVec) -> None

        Set a specific noise model for the density matrix simulator with multiple gate types, a given probability, and target qubits.

        Args:
             noise_model: The noise model to apply.

             gate_types: A vector of gate types associated with the noise model.

             probability: The probability of the noise occurring.

             qubits: The target qubits affected by the noise model.

        Returns:
             None.


        9. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: pyQPanda.NoiseModel, arg1: pyQPanda.GateType, arg2: float, arg3: List[pyQPanda.QVec]) -> None

        Set a specific noise model for the density matrix simulator with a given gate type, probability, and groups of target qubits.

        Args:
             noise_model: The noise model to apply.

             gate_type: The specific gate type associated with the noise model.

             probability: The probability of the noise occurring.

             qubit_groups: A vector of QVecs representing groups of target qubits affected by the noise model.

        Returns:
             None.


        10. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: pyQPanda.NoiseModel, arg1: pyQPanda.GateType, arg2: float, arg3: float, arg4: float) -> None

        Set a specific noise model for the density matrix simulator with a given gate type, probability, duration, and temperature.

        Args:
             noise_model: The noise model to apply.

             gate_type: The specific gate type associated with the noise model.

             probability: The probability of the noise occurring.

             duration: The duration for which the noise model is applied.

             temperature: The temperature affecting the noise characteristics.

        Returns:
             None.


        11. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: pyQPanda.NoiseModel, arg1: List[pyQPanda.GateType], arg2: float, arg3: float, arg4: float) -> None

        Set a specific noise model for the density matrix simulator with multiple gate types, a given probability, duration, and temperature.

        Args:
             noise_model: The noise model to apply.

             gate_types: A vector of gate types associated with the noise model.

             probability: The probability of the noise occurring.

             duration: The duration for which the noise model is applied.

             temperature: The temperature affecting the noise characteristics.

        Returns:
             None.


        12. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: pyQPanda.NoiseModel, arg1: pyQPanda.GateType, arg2: float, arg3: float, arg4: float, arg5: pyQPanda.QVec) -> None

        Set a specific noise model for the density matrix simulator with a given gate type, probability, duration, temperature, and a target qubit.

        Args:
             noise_model: The noise model to apply.

             gate_type: The specific gate type associated with the noise model.

             probability: The probability of the noise occurring.

             duration: The duration for which the noise model is applied.

             temperature: The temperature affecting the noise characteristics.

             target_qubit: The specific qubit targeted by the noise model.

        Returns:
             None.


        13. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: pyQPanda.NoiseModel, arg1: List[pyQPanda.GateType], arg2: float, arg3: float, arg4: float, arg5: pyQPanda.QVec) -> None

        Set a specific noise model for the density matrix simulator with multiple gate types, probability, duration, temperature, and a target qubit.

        Args:
             noise_model: The noise model to apply.

             gate_types: A vector of gate types associated with the noise model.

             probability: The probability of the noise occurring.

             duration: The duration for which the noise model is applied.

             temperature: The temperature affecting the noise characteristics.

             target_qubit: The specific qubit targeted by the noise model.

        Returns:
             None.


        14. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: pyQPanda.NoiseModel, arg1: pyQPanda.GateType, arg2: float, arg3: float, arg4: float, arg5: List[pyQPanda.QVec]) -> None

        Set a specific noise model for the density matrix simulator with a given gate type, probability, duration, temperature, and multiple target qubits.

        Args:
             noise_model: The noise model to apply.

             gate_type: The specific gate type associated with the noise model.

             probability: The probability of the noise occurring.

             duration: The duration for which the noise model is applied.

             temperature: The temperature affecting the noise characteristics.

             target_qubits: A vector of qubits targeted by the noise model.

        Returns:
             None.

        """
    @overload
    def set_noise_model(self, arg0: NoiseModel, arg1: GateType, arg2: float, arg3: float, arg4: float, arg5: QVec) -> None:
        """set_noise_model(*args, **kwargs)
        Overloaded function.

        1. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: numpy.ndarray[numpy.complex128[m, n]]) -> None

        Set the noise model for the density matrix simulator.

        Args:
             noise_model: The noise model represented as a complex matrix.

        Returns:
             None.


        2. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: numpy.ndarray[numpy.complex128[m, n]], arg1: List[pyQPanda.GateType]) -> None

        Set the noise model for the density matrix simulator with specific gate types.

        Args:
             noise_model: The noise model represented as a complex matrix.

             gate_types: A vector of gate types to which the noise model applies.

        Returns:
             None.


        3. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: List[numpy.ndarray[numpy.complex128[m, n]]]) -> None

        Set multiple noise models for the density matrix simulator.

        Args:
             noise_models: A vector of noise models, each represented as a complex matrix.

        Returns:
             None.


        4. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: List[numpy.ndarray[numpy.complex128[m, n]]], arg1: List[pyQPanda.GateType]) -> None

        Set multiple noise models for the density matrix simulator with specific gate types.

        Args:
             noise_models: A vector of noise models, each represented as a complex matrix.

             gate_types: A vector of gate types to which the noise models apply.

        Returns:
             None.


        5. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: pyQPanda.NoiseModel, arg1: pyQPanda.GateType, arg2: float) -> None

        Set a specific noise model for the density matrix simulator with a given gate type and probability.

        Args:
             noise_model: The noise model to apply.

             gate_type: The specific gate type associated with the noise model.

             probability: The probability of the noise occurring.

        Returns:
             None.


        6. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: pyQPanda.NoiseModel, arg1: List[pyQPanda.GateType], arg2: float) -> None

        Set a specific noise model for the density matrix simulator with multiple gate types and a given probability.

        Args:
             noise_model: The noise model to apply.

             gate_types: A vector of gate types associated with the noise model.

             probability: The probability of the noise occurring.

        Returns:
             None.


        7. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: pyQPanda.NoiseModel, arg1: pyQPanda.GateType, arg2: float, arg3: pyQPanda.QVec) -> None

        Set a specific noise model for the density matrix simulator with a given gate type, probability, and target qubits.

        Args:
             noise_model: The noise model to apply.

             gate_type: The specific gate type associated with the noise model.

             probability: The probability of the noise occurring.

             qubits: The target qubits affected by the noise model.

        Returns:
             None.


        8. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: pyQPanda.NoiseModel, arg1: List[pyQPanda.GateType], arg2: float, arg3: pyQPanda.QVec) -> None

        Set a specific noise model for the density matrix simulator with multiple gate types, a given probability, and target qubits.

        Args:
             noise_model: The noise model to apply.

             gate_types: A vector of gate types associated with the noise model.

             probability: The probability of the noise occurring.

             qubits: The target qubits affected by the noise model.

        Returns:
             None.


        9. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: pyQPanda.NoiseModel, arg1: pyQPanda.GateType, arg2: float, arg3: List[pyQPanda.QVec]) -> None

        Set a specific noise model for the density matrix simulator with a given gate type, probability, and groups of target qubits.

        Args:
             noise_model: The noise model to apply.

             gate_type: The specific gate type associated with the noise model.

             probability: The probability of the noise occurring.

             qubit_groups: A vector of QVecs representing groups of target qubits affected by the noise model.

        Returns:
             None.


        10. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: pyQPanda.NoiseModel, arg1: pyQPanda.GateType, arg2: float, arg3: float, arg4: float) -> None

        Set a specific noise model for the density matrix simulator with a given gate type, probability, duration, and temperature.

        Args:
             noise_model: The noise model to apply.

             gate_type: The specific gate type associated with the noise model.

             probability: The probability of the noise occurring.

             duration: The duration for which the noise model is applied.

             temperature: The temperature affecting the noise characteristics.

        Returns:
             None.


        11. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: pyQPanda.NoiseModel, arg1: List[pyQPanda.GateType], arg2: float, arg3: float, arg4: float) -> None

        Set a specific noise model for the density matrix simulator with multiple gate types, a given probability, duration, and temperature.

        Args:
             noise_model: The noise model to apply.

             gate_types: A vector of gate types associated with the noise model.

             probability: The probability of the noise occurring.

             duration: The duration for which the noise model is applied.

             temperature: The temperature affecting the noise characteristics.

        Returns:
             None.


        12. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: pyQPanda.NoiseModel, arg1: pyQPanda.GateType, arg2: float, arg3: float, arg4: float, arg5: pyQPanda.QVec) -> None

        Set a specific noise model for the density matrix simulator with a given gate type, probability, duration, temperature, and a target qubit.

        Args:
             noise_model: The noise model to apply.

             gate_type: The specific gate type associated with the noise model.

             probability: The probability of the noise occurring.

             duration: The duration for which the noise model is applied.

             temperature: The temperature affecting the noise characteristics.

             target_qubit: The specific qubit targeted by the noise model.

        Returns:
             None.


        13. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: pyQPanda.NoiseModel, arg1: List[pyQPanda.GateType], arg2: float, arg3: float, arg4: float, arg5: pyQPanda.QVec) -> None

        Set a specific noise model for the density matrix simulator with multiple gate types, probability, duration, temperature, and a target qubit.

        Args:
             noise_model: The noise model to apply.

             gate_types: A vector of gate types associated with the noise model.

             probability: The probability of the noise occurring.

             duration: The duration for which the noise model is applied.

             temperature: The temperature affecting the noise characteristics.

             target_qubit: The specific qubit targeted by the noise model.

        Returns:
             None.


        14. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: pyQPanda.NoiseModel, arg1: pyQPanda.GateType, arg2: float, arg3: float, arg4: float, arg5: List[pyQPanda.QVec]) -> None

        Set a specific noise model for the density matrix simulator with a given gate type, probability, duration, temperature, and multiple target qubits.

        Args:
             noise_model: The noise model to apply.

             gate_type: The specific gate type associated with the noise model.

             probability: The probability of the noise occurring.

             duration: The duration for which the noise model is applied.

             temperature: The temperature affecting the noise characteristics.

             target_qubits: A vector of qubits targeted by the noise model.

        Returns:
             None.

        """
    @overload
    def set_noise_model(self, arg0: NoiseModel, arg1: list[GateType], arg2: float, arg3: float, arg4: float, arg5: QVec) -> None:
        """set_noise_model(*args, **kwargs)
        Overloaded function.

        1. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: numpy.ndarray[numpy.complex128[m, n]]) -> None

        Set the noise model for the density matrix simulator.

        Args:
             noise_model: The noise model represented as a complex matrix.

        Returns:
             None.


        2. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: numpy.ndarray[numpy.complex128[m, n]], arg1: List[pyQPanda.GateType]) -> None

        Set the noise model for the density matrix simulator with specific gate types.

        Args:
             noise_model: The noise model represented as a complex matrix.

             gate_types: A vector of gate types to which the noise model applies.

        Returns:
             None.


        3. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: List[numpy.ndarray[numpy.complex128[m, n]]]) -> None

        Set multiple noise models for the density matrix simulator.

        Args:
             noise_models: A vector of noise models, each represented as a complex matrix.

        Returns:
             None.


        4. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: List[numpy.ndarray[numpy.complex128[m, n]]], arg1: List[pyQPanda.GateType]) -> None

        Set multiple noise models for the density matrix simulator with specific gate types.

        Args:
             noise_models: A vector of noise models, each represented as a complex matrix.

             gate_types: A vector of gate types to which the noise models apply.

        Returns:
             None.


        5. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: pyQPanda.NoiseModel, arg1: pyQPanda.GateType, arg2: float) -> None

        Set a specific noise model for the density matrix simulator with a given gate type and probability.

        Args:
             noise_model: The noise model to apply.

             gate_type: The specific gate type associated with the noise model.

             probability: The probability of the noise occurring.

        Returns:
             None.


        6. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: pyQPanda.NoiseModel, arg1: List[pyQPanda.GateType], arg2: float) -> None

        Set a specific noise model for the density matrix simulator with multiple gate types and a given probability.

        Args:
             noise_model: The noise model to apply.

             gate_types: A vector of gate types associated with the noise model.

             probability: The probability of the noise occurring.

        Returns:
             None.


        7. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: pyQPanda.NoiseModel, arg1: pyQPanda.GateType, arg2: float, arg3: pyQPanda.QVec) -> None

        Set a specific noise model for the density matrix simulator with a given gate type, probability, and target qubits.

        Args:
             noise_model: The noise model to apply.

             gate_type: The specific gate type associated with the noise model.

             probability: The probability of the noise occurring.

             qubits: The target qubits affected by the noise model.

        Returns:
             None.


        8. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: pyQPanda.NoiseModel, arg1: List[pyQPanda.GateType], arg2: float, arg3: pyQPanda.QVec) -> None

        Set a specific noise model for the density matrix simulator with multiple gate types, a given probability, and target qubits.

        Args:
             noise_model: The noise model to apply.

             gate_types: A vector of gate types associated with the noise model.

             probability: The probability of the noise occurring.

             qubits: The target qubits affected by the noise model.

        Returns:
             None.


        9. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: pyQPanda.NoiseModel, arg1: pyQPanda.GateType, arg2: float, arg3: List[pyQPanda.QVec]) -> None

        Set a specific noise model for the density matrix simulator with a given gate type, probability, and groups of target qubits.

        Args:
             noise_model: The noise model to apply.

             gate_type: The specific gate type associated with the noise model.

             probability: The probability of the noise occurring.

             qubit_groups: A vector of QVecs representing groups of target qubits affected by the noise model.

        Returns:
             None.


        10. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: pyQPanda.NoiseModel, arg1: pyQPanda.GateType, arg2: float, arg3: float, arg4: float) -> None

        Set a specific noise model for the density matrix simulator with a given gate type, probability, duration, and temperature.

        Args:
             noise_model: The noise model to apply.

             gate_type: The specific gate type associated with the noise model.

             probability: The probability of the noise occurring.

             duration: The duration for which the noise model is applied.

             temperature: The temperature affecting the noise characteristics.

        Returns:
             None.


        11. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: pyQPanda.NoiseModel, arg1: List[pyQPanda.GateType], arg2: float, arg3: float, arg4: float) -> None

        Set a specific noise model for the density matrix simulator with multiple gate types, a given probability, duration, and temperature.

        Args:
             noise_model: The noise model to apply.

             gate_types: A vector of gate types associated with the noise model.

             probability: The probability of the noise occurring.

             duration: The duration for which the noise model is applied.

             temperature: The temperature affecting the noise characteristics.

        Returns:
             None.


        12. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: pyQPanda.NoiseModel, arg1: pyQPanda.GateType, arg2: float, arg3: float, arg4: float, arg5: pyQPanda.QVec) -> None

        Set a specific noise model for the density matrix simulator with a given gate type, probability, duration, temperature, and a target qubit.

        Args:
             noise_model: The noise model to apply.

             gate_type: The specific gate type associated with the noise model.

             probability: The probability of the noise occurring.

             duration: The duration for which the noise model is applied.

             temperature: The temperature affecting the noise characteristics.

             target_qubit: The specific qubit targeted by the noise model.

        Returns:
             None.


        13. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: pyQPanda.NoiseModel, arg1: List[pyQPanda.GateType], arg2: float, arg3: float, arg4: float, arg5: pyQPanda.QVec) -> None

        Set a specific noise model for the density matrix simulator with multiple gate types, probability, duration, temperature, and a target qubit.

        Args:
             noise_model: The noise model to apply.

             gate_types: A vector of gate types associated with the noise model.

             probability: The probability of the noise occurring.

             duration: The duration for which the noise model is applied.

             temperature: The temperature affecting the noise characteristics.

             target_qubit: The specific qubit targeted by the noise model.

        Returns:
             None.


        14. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: pyQPanda.NoiseModel, arg1: pyQPanda.GateType, arg2: float, arg3: float, arg4: float, arg5: List[pyQPanda.QVec]) -> None

        Set a specific noise model for the density matrix simulator with a given gate type, probability, duration, temperature, and multiple target qubits.

        Args:
             noise_model: The noise model to apply.

             gate_type: The specific gate type associated with the noise model.

             probability: The probability of the noise occurring.

             duration: The duration for which the noise model is applied.

             temperature: The temperature affecting the noise characteristics.

             target_qubits: A vector of qubits targeted by the noise model.

        Returns:
             None.

        """
    @overload
    def set_noise_model(self, arg0: NoiseModel, arg1: GateType, arg2: float, arg3: float, arg4: float, arg5: list[QVec]) -> None:
        """set_noise_model(*args, **kwargs)
        Overloaded function.

        1. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: numpy.ndarray[numpy.complex128[m, n]]) -> None

        Set the noise model for the density matrix simulator.

        Args:
             noise_model: The noise model represented as a complex matrix.

        Returns:
             None.


        2. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: numpy.ndarray[numpy.complex128[m, n]], arg1: List[pyQPanda.GateType]) -> None

        Set the noise model for the density matrix simulator with specific gate types.

        Args:
             noise_model: The noise model represented as a complex matrix.

             gate_types: A vector of gate types to which the noise model applies.

        Returns:
             None.


        3. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: List[numpy.ndarray[numpy.complex128[m, n]]]) -> None

        Set multiple noise models for the density matrix simulator.

        Args:
             noise_models: A vector of noise models, each represented as a complex matrix.

        Returns:
             None.


        4. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: List[numpy.ndarray[numpy.complex128[m, n]]], arg1: List[pyQPanda.GateType]) -> None

        Set multiple noise models for the density matrix simulator with specific gate types.

        Args:
             noise_models: A vector of noise models, each represented as a complex matrix.

             gate_types: A vector of gate types to which the noise models apply.

        Returns:
             None.


        5. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: pyQPanda.NoiseModel, arg1: pyQPanda.GateType, arg2: float) -> None

        Set a specific noise model for the density matrix simulator with a given gate type and probability.

        Args:
             noise_model: The noise model to apply.

             gate_type: The specific gate type associated with the noise model.

             probability: The probability of the noise occurring.

        Returns:
             None.


        6. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: pyQPanda.NoiseModel, arg1: List[pyQPanda.GateType], arg2: float) -> None

        Set a specific noise model for the density matrix simulator with multiple gate types and a given probability.

        Args:
             noise_model: The noise model to apply.

             gate_types: A vector of gate types associated with the noise model.

             probability: The probability of the noise occurring.

        Returns:
             None.


        7. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: pyQPanda.NoiseModel, arg1: pyQPanda.GateType, arg2: float, arg3: pyQPanda.QVec) -> None

        Set a specific noise model for the density matrix simulator with a given gate type, probability, and target qubits.

        Args:
             noise_model: The noise model to apply.

             gate_type: The specific gate type associated with the noise model.

             probability: The probability of the noise occurring.

             qubits: The target qubits affected by the noise model.

        Returns:
             None.


        8. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: pyQPanda.NoiseModel, arg1: List[pyQPanda.GateType], arg2: float, arg3: pyQPanda.QVec) -> None

        Set a specific noise model for the density matrix simulator with multiple gate types, a given probability, and target qubits.

        Args:
             noise_model: The noise model to apply.

             gate_types: A vector of gate types associated with the noise model.

             probability: The probability of the noise occurring.

             qubits: The target qubits affected by the noise model.

        Returns:
             None.


        9. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: pyQPanda.NoiseModel, arg1: pyQPanda.GateType, arg2: float, arg3: List[pyQPanda.QVec]) -> None

        Set a specific noise model for the density matrix simulator with a given gate type, probability, and groups of target qubits.

        Args:
             noise_model: The noise model to apply.

             gate_type: The specific gate type associated with the noise model.

             probability: The probability of the noise occurring.

             qubit_groups: A vector of QVecs representing groups of target qubits affected by the noise model.

        Returns:
             None.


        10. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: pyQPanda.NoiseModel, arg1: pyQPanda.GateType, arg2: float, arg3: float, arg4: float) -> None

        Set a specific noise model for the density matrix simulator with a given gate type, probability, duration, and temperature.

        Args:
             noise_model: The noise model to apply.

             gate_type: The specific gate type associated with the noise model.

             probability: The probability of the noise occurring.

             duration: The duration for which the noise model is applied.

             temperature: The temperature affecting the noise characteristics.

        Returns:
             None.


        11. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: pyQPanda.NoiseModel, arg1: List[pyQPanda.GateType], arg2: float, arg3: float, arg4: float) -> None

        Set a specific noise model for the density matrix simulator with multiple gate types, a given probability, duration, and temperature.

        Args:
             noise_model: The noise model to apply.

             gate_types: A vector of gate types associated with the noise model.

             probability: The probability of the noise occurring.

             duration: The duration for which the noise model is applied.

             temperature: The temperature affecting the noise characteristics.

        Returns:
             None.


        12. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: pyQPanda.NoiseModel, arg1: pyQPanda.GateType, arg2: float, arg3: float, arg4: float, arg5: pyQPanda.QVec) -> None

        Set a specific noise model for the density matrix simulator with a given gate type, probability, duration, temperature, and a target qubit.

        Args:
             noise_model: The noise model to apply.

             gate_type: The specific gate type associated with the noise model.

             probability: The probability of the noise occurring.

             duration: The duration for which the noise model is applied.

             temperature: The temperature affecting the noise characteristics.

             target_qubit: The specific qubit targeted by the noise model.

        Returns:
             None.


        13. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: pyQPanda.NoiseModel, arg1: List[pyQPanda.GateType], arg2: float, arg3: float, arg4: float, arg5: pyQPanda.QVec) -> None

        Set a specific noise model for the density matrix simulator with multiple gate types, probability, duration, temperature, and a target qubit.

        Args:
             noise_model: The noise model to apply.

             gate_types: A vector of gate types associated with the noise model.

             probability: The probability of the noise occurring.

             duration: The duration for which the noise model is applied.

             temperature: The temperature affecting the noise characteristics.

             target_qubit: The specific qubit targeted by the noise model.

        Returns:
             None.


        14. set_noise_model(self: pyQPanda.DensityMatrixSimulator, arg0: pyQPanda.NoiseModel, arg1: pyQPanda.GateType, arg2: float, arg3: float, arg4: float, arg5: List[pyQPanda.QVec]) -> None

        Set a specific noise model for the density matrix simulator with a given gate type, probability, duration, temperature, and multiple target qubits.

        Args:
             noise_model: The noise model to apply.

             gate_type: The specific gate type associated with the noise model.

             probability: The probability of the noise occurring.

             duration: The duration for which the noise model is applied.

             temperature: The temperature affecting the noise characteristics.

             target_qubits: A vector of qubits targeted by the noise model.

        Returns:
             None.

        """

class DoubleGateTransferType:
    __members__: ClassVar[dict] = ...  # read-only
    DOUBLE_BIT_GATE: ClassVar[DoubleGateTransferType] = ...
    DOUBLE_GATE_INVALID: ClassVar[DoubleGateTransferType] = ...
    __entries: ClassVar[dict] = ...
    def __init__(self, value: int) -> None:
        """__init__(self: pyQPanda.DoubleGateTransferType, value: int) -> None"""
    def __eq__(self, other: object) -> bool:
        """__eq__(self: object, other: object) -> bool"""
    def __hash__(self) -> int:
        """__hash__(self: object) -> int"""
    def __index__(self) -> int:
        """__index__(self: pyQPanda.DoubleGateTransferType) -> int"""
    def __int__(self) -> int:
        """__int__(self: pyQPanda.DoubleGateTransferType) -> int"""
    def __ne__(self, other: object) -> bool:
        """__ne__(self: object, other: object) -> bool"""
    @property
    def name(self) -> str: ...
    @property
    def value(self) -> int: ...

class Encode:
    def __init__(self) -> None:
        """__init__(self: pyQPanda.Encode) -> None"""
    @overload
    def amplitude_encode(self, qubit: QVec, data: list[float]) -> None:
        """amplitude_encode(*args, **kwargs)
        Overloaded function.

        1. amplitude_encode(self: pyQPanda.Encode, qubit: pyQPanda.QVec, data: List[float]) -> None

        Perform amplitude encoding on the given qubits.

        Args:
             qubit: The quantum vector to be encoded.

             data: The classical data to be encoded.

        Returns:
             An encoded quantum state.


        2. amplitude_encode(self: pyQPanda.Encode, qubit: pyQPanda.QVec, data: List[complex]) -> None

        Perform amplitude encoding using complex numbers on the given qubits.

        Args:
             qubit: The quantum vector to be encoded.

             data: The classical complex data to be encoded.

        Returns:
             An encoded quantum state.

        """
    @overload
    def amplitude_encode(self, qubit: QVec, data: list[complex]) -> None:
        """amplitude_encode(*args, **kwargs)
        Overloaded function.

        1. amplitude_encode(self: pyQPanda.Encode, qubit: pyQPanda.QVec, data: List[float]) -> None

        Perform amplitude encoding on the given qubits.

        Args:
             qubit: The quantum vector to be encoded.

             data: The classical data to be encoded.

        Returns:
             An encoded quantum state.


        2. amplitude_encode(self: pyQPanda.Encode, qubit: pyQPanda.QVec, data: List[complex]) -> None

        Perform amplitude encoding using complex numbers on the given qubits.

        Args:
             qubit: The quantum vector to be encoded.

             data: The classical complex data to be encoded.

        Returns:
             An encoded quantum state.

        """
    @overload
    def amplitude_encode_recursive(self, qubit: QVec, data: list[float]) -> None:
        """amplitude_encode_recursive(*args, **kwargs)
        Overloaded function.

        1. amplitude_encode_recursive(self: pyQPanda.Encode, qubit: pyQPanda.QVec, data: List[float]) -> None

        Perform recursive amplitude encoding on the given qubits.

        Args:
             qubit: The quantum vector to be encoded.

             data: The classical data to be encoded.

        Returns:
             An encoded quantum state.


        2. amplitude_encode_recursive(self: pyQPanda.Encode, qubit: pyQPanda.QVec, data: List[complex]) -> None

        Encode by amplitude recursively.

        Args:
             QVec: qubits

             QStat: amplitude

        Returns:
             circuit

        """
    @overload
    def amplitude_encode_recursive(self, qubit: QVec, data: list[complex]) -> None:
        """amplitude_encode_recursive(*args, **kwargs)
        Overloaded function.

        1. amplitude_encode_recursive(self: pyQPanda.Encode, qubit: pyQPanda.QVec, data: List[float]) -> None

        Perform recursive amplitude encoding on the given qubits.

        Args:
             qubit: The quantum vector to be encoded.

             data: The classical data to be encoded.

        Returns:
             An encoded quantum state.


        2. amplitude_encode_recursive(self: pyQPanda.Encode, qubit: pyQPanda.QVec, data: List[complex]) -> None

        Encode by amplitude recursively.

        Args:
             QVec: qubits

             QStat: amplitude

        Returns:
             circuit

        """
    def angle_encode(self, qubit: QVec, data: list[float], gate_type: GateType = ...) -> None:
        """angle_encode(self: pyQPanda.Encode, qubit: pyQPanda.QVec, data: List[float], gate_type: pyQPanda.GateType = GateType.RY_GATE) -> None

        Encode by angle.

        Args:
             QVec: qubits 

             prob_vec: data 

        Returns:
             circuit.

        """
    @overload
    def approx_mps(self, qubit: QVec, data: list[float], layers: int = ..., sweeps: int = ..., double2float: bool = ...) -> None:
        """approx_mps(*args, **kwargs)
        Overloaded function.

        1. approx_mps(self: pyQPanda.Encode, qubit: pyQPanda.QVec, data: List[float], layers: int = 3, sweeps: int = 100, double2float: bool = False) -> None

        Approximate Matrix Product State encoding.

        Args:
             QVec: qubits 

             std::vector<double>: input data 

             int: number of layers for encoding (default: 3)

             int: number of sweeps for optimization (default: 100)

             bool: flag to convert double data to float (default: false)

        Returns:
             Encoded circuit based on input parameters.

        Raises:
             run_fail: An error occurred during the encoding process.


        2. approx_mps(self: pyQPanda.Encode, qubit: pyQPanda.QVec, data: List[complex], layers: int = 3, sweeps: int = 100) -> None

        Approximate Matrix Product State encoding.

        Args:
             QVec: qubits 

             std::vector<qcomplex_t>: input data 

             int: number of layers (default: 3)

             int: number of steps (default: 100)

        Returns:
             Encoded circuit.

        """
    @overload
    def approx_mps(self, qubit: QVec, data: list[complex], layers: int = ..., sweeps: int = ...) -> None:
        """approx_mps(*args, **kwargs)
        Overloaded function.

        1. approx_mps(self: pyQPanda.Encode, qubit: pyQPanda.QVec, data: List[float], layers: int = 3, sweeps: int = 100, double2float: bool = False) -> None

        Approximate Matrix Product State encoding.

        Args:
             QVec: qubits 

             std::vector<double>: input data 

             int: number of layers for encoding (default: 3)

             int: number of sweeps for optimization (default: 100)

             bool: flag to convert double data to float (default: false)

        Returns:
             Encoded circuit based on input parameters.

        Raises:
             run_fail: An error occurred during the encoding process.


        2. approx_mps(self: pyQPanda.Encode, qubit: pyQPanda.QVec, data: List[complex], layers: int = 3, sweeps: int = 100) -> None

        Approximate Matrix Product State encoding.

        Args:
             QVec: qubits 

             std::vector<qcomplex_t>: input data 

             int: number of layers (default: 3)

             int: number of steps (default: 100)

        Returns:
             Encoded circuit.

        """
    def basic_encode(self, qubit: QVec, data: str) -> None:
        """basic_encode(self: pyQPanda.Encode, qubit: pyQPanda.QVec, data: str) -> None

        Basic encoding.

        Args:
             QVec: qubits 

             string: data 

        Returns:
             circuit

        """
    def bid_amplitude_encode(self, qubit: QVec, data: list[float], split: int = ...) -> None:
        """bid_amplitude_encode(self: pyQPanda.Encode, qubit: pyQPanda.QVec, data: List[float], split: int = 0) -> None

        Encode by bid.

        Args:
             QVec: qubits 

             QStat: amplitude 

             split: int 

        Returns:
             circuit

        """
    def dc_amplitude_encode(self, qubit: QVec, data: list[float]) -> None:
        """dc_amplitude_encode(self: pyQPanda.Encode, qubit: pyQPanda.QVec, data: List[float]) -> None

        Encode by DC amplitude.

        Args:
             QVec: qubits 

             QStat: amplitude 

        Returns:
             circuit

        """
    def dense_angle_encode(self, qubit: QVec, data: list[float]) -> None:
        """dense_angle_encode(self: pyQPanda.Encode, qubit: pyQPanda.QVec, data: List[float]) -> None

        Encode by dense angle.

        Args:
             QVec: qubits 

             prob_vec: data 

        Returns:
             circuit

        """
    @overload
    def ds_quantum_state_preparation(self, qubit: QVec, data: dict[str, float]) -> None:
        """ds_quantum_state_preparation(*args, **kwargs)
        Overloaded function.

        1. ds_quantum_state_preparation(self: pyQPanda.Encode, qubit: pyQPanda.QVec, data: Dict[str, float]) -> None

        Prepare a quantum state.

        Args:
             QVec: qubits 

             std::map<std::string, double>: state parameters 

        Returns:
             circuit


        2. ds_quantum_state_preparation(self: pyQPanda.Encode, qubit: pyQPanda.QVec, data: Dict[str, complex]) -> None

        Prepare a quantum state.

        Args:
             QVec: qubits 

             std::map<std::string, std::complex<double>>: state parameters 

        Returns:
             circuit


        3. ds_quantum_state_preparation(self: pyQPanda.Encode, qubit: pyQPanda.QVec, data: List[float]) -> None

        Prepare a quantum state.

        Args:
             QVec: qubits 

             std::vector<double>: state parameters 

        Returns:
             circuit


        4. ds_quantum_state_preparation(self: pyQPanda.Encode, qubit: pyQPanda.QVec, data: List[complex]) -> None

        Prepare a quantum state.

        Args:
             QVec: qubits 

             std::vector<std::complex<double>>: state parameters 

        Returns:
             circuit

        """
    @overload
    def ds_quantum_state_preparation(self, qubit: QVec, data: dict[str, complex]) -> None:
        """ds_quantum_state_preparation(*args, **kwargs)
        Overloaded function.

        1. ds_quantum_state_preparation(self: pyQPanda.Encode, qubit: pyQPanda.QVec, data: Dict[str, float]) -> None

        Prepare a quantum state.

        Args:
             QVec: qubits 

             std::map<std::string, double>: state parameters 

        Returns:
             circuit


        2. ds_quantum_state_preparation(self: pyQPanda.Encode, qubit: pyQPanda.QVec, data: Dict[str, complex]) -> None

        Prepare a quantum state.

        Args:
             QVec: qubits 

             std::map<std::string, std::complex<double>>: state parameters 

        Returns:
             circuit


        3. ds_quantum_state_preparation(self: pyQPanda.Encode, qubit: pyQPanda.QVec, data: List[float]) -> None

        Prepare a quantum state.

        Args:
             QVec: qubits 

             std::vector<double>: state parameters 

        Returns:
             circuit


        4. ds_quantum_state_preparation(self: pyQPanda.Encode, qubit: pyQPanda.QVec, data: List[complex]) -> None

        Prepare a quantum state.

        Args:
             QVec: qubits 

             std::vector<std::complex<double>>: state parameters 

        Returns:
             circuit

        """
    @overload
    def ds_quantum_state_preparation(self, qubit: QVec, data: list[float]) -> None:
        """ds_quantum_state_preparation(*args, **kwargs)
        Overloaded function.

        1. ds_quantum_state_preparation(self: pyQPanda.Encode, qubit: pyQPanda.QVec, data: Dict[str, float]) -> None

        Prepare a quantum state.

        Args:
             QVec: qubits 

             std::map<std::string, double>: state parameters 

        Returns:
             circuit


        2. ds_quantum_state_preparation(self: pyQPanda.Encode, qubit: pyQPanda.QVec, data: Dict[str, complex]) -> None

        Prepare a quantum state.

        Args:
             QVec: qubits 

             std::map<std::string, std::complex<double>>: state parameters 

        Returns:
             circuit


        3. ds_quantum_state_preparation(self: pyQPanda.Encode, qubit: pyQPanda.QVec, data: List[float]) -> None

        Prepare a quantum state.

        Args:
             QVec: qubits 

             std::vector<double>: state parameters 

        Returns:
             circuit


        4. ds_quantum_state_preparation(self: pyQPanda.Encode, qubit: pyQPanda.QVec, data: List[complex]) -> None

        Prepare a quantum state.

        Args:
             QVec: qubits 

             std::vector<std::complex<double>>: state parameters 

        Returns:
             circuit

        """
    @overload
    def ds_quantum_state_preparation(self, qubit: QVec, data: list[complex]) -> None:
        """ds_quantum_state_preparation(*args, **kwargs)
        Overloaded function.

        1. ds_quantum_state_preparation(self: pyQPanda.Encode, qubit: pyQPanda.QVec, data: Dict[str, float]) -> None

        Prepare a quantum state.

        Args:
             QVec: qubits 

             std::map<std::string, double>: state parameters 

        Returns:
             circuit


        2. ds_quantum_state_preparation(self: pyQPanda.Encode, qubit: pyQPanda.QVec, data: Dict[str, complex]) -> None

        Prepare a quantum state.

        Args:
             QVec: qubits 

             std::map<std::string, std::complex<double>>: state parameters 

        Returns:
             circuit


        3. ds_quantum_state_preparation(self: pyQPanda.Encode, qubit: pyQPanda.QVec, data: List[float]) -> None

        Prepare a quantum state.

        Args:
             QVec: qubits 

             std::vector<double>: state parameters 

        Returns:
             circuit


        4. ds_quantum_state_preparation(self: pyQPanda.Encode, qubit: pyQPanda.QVec, data: List[complex]) -> None

        Prepare a quantum state.

        Args:
             QVec: qubits 

             std::vector<std::complex<double>>: state parameters 

        Returns:
             circuit

        """
    @overload
    def efficient_sparse(self, qubit: QVec, data: dict[str, float]) -> None:
        """efficient_sparse(*args, **kwargs)
        Overloaded function.

        1. efficient_sparse(self: pyQPanda.Encode, qubit: pyQPanda.QVec, data: Dict[str, float]) -> None

        Perform an efficient sparse operation.

        Args:
             QVec: qubits 

             std::map<std::string, double>: parameters for the operation 

        Returns:
             circuit


        2. efficient_sparse(self: pyQPanda.Encode, qubit: pyQPanda.QVec, data: Dict[str, complex]) -> None

        Perform an efficient sparse operation.

        Args:
             QVec: qubits 

             std::map<std::string, std::complex<double>>: parameters for the operation 

        Returns:
             circuit


        3. efficient_sparse(self: pyQPanda.Encode, qubit: pyQPanda.QVec, data: List[float]) -> None

        Perform an efficient sparse operation.

        Args:
             QVec: qubits 

             std::vector<double>: parameters for the operation 

        Returns:
             circuit


        4. efficient_sparse(self: pyQPanda.Encode, qubit: pyQPanda.QVec, data: List[complex]) -> None

        Perform an efficient sparse operation.

        Args:
             QVec: qubits 

             std::vector<std::complex<double>>: parameters for the operation 

        Returns:
             circuit

        """
    @overload
    def efficient_sparse(self, qubit: QVec, data: dict[str, complex]) -> None:
        """efficient_sparse(*args, **kwargs)
        Overloaded function.

        1. efficient_sparse(self: pyQPanda.Encode, qubit: pyQPanda.QVec, data: Dict[str, float]) -> None

        Perform an efficient sparse operation.

        Args:
             QVec: qubits 

             std::map<std::string, double>: parameters for the operation 

        Returns:
             circuit


        2. efficient_sparse(self: pyQPanda.Encode, qubit: pyQPanda.QVec, data: Dict[str, complex]) -> None

        Perform an efficient sparse operation.

        Args:
             QVec: qubits 

             std::map<std::string, std::complex<double>>: parameters for the operation 

        Returns:
             circuit


        3. efficient_sparse(self: pyQPanda.Encode, qubit: pyQPanda.QVec, data: List[float]) -> None

        Perform an efficient sparse operation.

        Args:
             QVec: qubits 

             std::vector<double>: parameters for the operation 

        Returns:
             circuit


        4. efficient_sparse(self: pyQPanda.Encode, qubit: pyQPanda.QVec, data: List[complex]) -> None

        Perform an efficient sparse operation.

        Args:
             QVec: qubits 

             std::vector<std::complex<double>>: parameters for the operation 

        Returns:
             circuit

        """
    @overload
    def efficient_sparse(self, qubit: QVec, data: list[float]) -> None:
        """efficient_sparse(*args, **kwargs)
        Overloaded function.

        1. efficient_sparse(self: pyQPanda.Encode, qubit: pyQPanda.QVec, data: Dict[str, float]) -> None

        Perform an efficient sparse operation.

        Args:
             QVec: qubits 

             std::map<std::string, double>: parameters for the operation 

        Returns:
             circuit


        2. efficient_sparse(self: pyQPanda.Encode, qubit: pyQPanda.QVec, data: Dict[str, complex]) -> None

        Perform an efficient sparse operation.

        Args:
             QVec: qubits 

             std::map<std::string, std::complex<double>>: parameters for the operation 

        Returns:
             circuit


        3. efficient_sparse(self: pyQPanda.Encode, qubit: pyQPanda.QVec, data: List[float]) -> None

        Perform an efficient sparse operation.

        Args:
             QVec: qubits 

             std::vector<double>: parameters for the operation 

        Returns:
             circuit


        4. efficient_sparse(self: pyQPanda.Encode, qubit: pyQPanda.QVec, data: List[complex]) -> None

        Perform an efficient sparse operation.

        Args:
             QVec: qubits 

             std::vector<std::complex<double>>: parameters for the operation 

        Returns:
             circuit

        """
    @overload
    def efficient_sparse(self, qubit: QVec, data: list[complex]) -> None:
        """efficient_sparse(*args, **kwargs)
        Overloaded function.

        1. efficient_sparse(self: pyQPanda.Encode, qubit: pyQPanda.QVec, data: Dict[str, float]) -> None

        Perform an efficient sparse operation.

        Args:
             QVec: qubits 

             std::map<std::string, double>: parameters for the operation 

        Returns:
             circuit


        2. efficient_sparse(self: pyQPanda.Encode, qubit: pyQPanda.QVec, data: Dict[str, complex]) -> None

        Perform an efficient sparse operation.

        Args:
             QVec: qubits 

             std::map<std::string, std::complex<double>>: parameters for the operation 

        Returns:
             circuit


        3. efficient_sparse(self: pyQPanda.Encode, qubit: pyQPanda.QVec, data: List[float]) -> None

        Perform an efficient sparse operation.

        Args:
             QVec: qubits 

             std::vector<double>: parameters for the operation 

        Returns:
             circuit


        4. efficient_sparse(self: pyQPanda.Encode, qubit: pyQPanda.QVec, data: List[complex]) -> None

        Perform an efficient sparse operation.

        Args:
             QVec: qubits 

             std::vector<std::complex<double>>: parameters for the operation 

        Returns:
             circuit

        """
    def get_circuit(self) -> QCircuit:
        """get_circuit(self: pyQPanda.Encode) -> pyQPanda.QCircuit

        Retrieve the circuit from the encoder.

        Returns:
             The corresponding circuit object.

        """
    @overload
    def get_fidelity(self, data: list[float]) -> float:
        """get_fidelity(*args, **kwargs)
        Overloaded function.

        1. get_fidelity(self: pyQPanda.Encode, data: List[float]) -> float

        Calculate the fidelity based on the provided data.

        Args:
             data: A vector of doubles representing the input data.

        Returns:
             The calculated fidelity value.


        2. get_fidelity(self: pyQPanda.Encode, data: List[complex]) -> float

        Calculate the fidelity based on the provided complex data.

        Args:
             data: A vector of qcomplex_t representing the input data.

        Returns:
             The calculated fidelity value.


        3. get_fidelity(self: pyQPanda.Encode, data: List[float]) -> float

        Calculate the fidelity based on the provided float data.

        Args:
             data: A vector of floats representing the input data.

        Returns:
             The calculated fidelity value.

        """
    @overload
    def get_fidelity(self, data: list[complex]) -> float:
        """get_fidelity(*args, **kwargs)
        Overloaded function.

        1. get_fidelity(self: pyQPanda.Encode, data: List[float]) -> float

        Calculate the fidelity based on the provided data.

        Args:
             data: A vector of doubles representing the input data.

        Returns:
             The calculated fidelity value.


        2. get_fidelity(self: pyQPanda.Encode, data: List[complex]) -> float

        Calculate the fidelity based on the provided complex data.

        Args:
             data: A vector of qcomplex_t representing the input data.

        Returns:
             The calculated fidelity value.


        3. get_fidelity(self: pyQPanda.Encode, data: List[float]) -> float

        Calculate the fidelity based on the provided float data.

        Args:
             data: A vector of floats representing the input data.

        Returns:
             The calculated fidelity value.

        """
    @overload
    def get_fidelity(self, data: list[float]) -> float:
        """get_fidelity(*args, **kwargs)
        Overloaded function.

        1. get_fidelity(self: pyQPanda.Encode, data: List[float]) -> float

        Calculate the fidelity based on the provided data.

        Args:
             data: A vector of doubles representing the input data.

        Returns:
             The calculated fidelity value.


        2. get_fidelity(self: pyQPanda.Encode, data: List[complex]) -> float

        Calculate the fidelity based on the provided complex data.

        Args:
             data: A vector of qcomplex_t representing the input data.

        Returns:
             The calculated fidelity value.


        3. get_fidelity(self: pyQPanda.Encode, data: List[float]) -> float

        Calculate the fidelity based on the provided float data.

        Args:
             data: A vector of floats representing the input data.

        Returns:
             The calculated fidelity value.

        """
    def get_out_qubits(self) -> QVec:
        """get_out_qubits(self: pyQPanda.Encode) -> pyQPanda.QVec

        Retrieve the output qubits from the encoder.

        Returns:
             A vector of output qubits.

        """
    def iqp_encode(self, qubit: QVec, data: list[float], control_list: list[tuple[int, int]] = ..., bool_inverse: bool = ..., repeats: int = ...) -> None:
        """iqp_encode(self: pyQPanda.Encode, qubit: pyQPanda.QVec, data: List[float], control_list: List[Tuple[int, int]] = [], bool_inverse: bool = False, repeats: int = 1) -> None

        Encode by IQP.

        Args:
             QVec: qubits 

             prob_vec: data 

             list: control_list 

             bool: bool_inverse 

             int: repeats 

        Returns:
             circuit.

        """
    def schmidt_encode(self, qubit: QVec, data: list[float], cutoff: float) -> None:
        """schmidt_encode(self: pyQPanda.Encode, qubit: pyQPanda.QVec, data: List[float], cutoff: float) -> None

        Encode by schmidt.

        Args:
             QVec: qubits 

             QStat: amplitude 

             double: cutoff 

        Returns:
             circuit

        """
    @overload
    def sparse_isometry(self, qubit: QVec, data: dict[str, float]) -> None:
        """sparse_isometry(*args, **kwargs)
        Overloaded function.

        1. sparse_isometry(self: pyQPanda.Encode, qubit: pyQPanda.QVec, data: Dict[str, float]) -> None

        Perform a sparse isometry operation.

        Args:
             QVec: qubits 

             std::map<std::string, double>: parameters for the isometry 

        Returns:
             circuit


        2. sparse_isometry(self: pyQPanda.Encode, qubit: pyQPanda.QVec, data: Dict[str, complex]) -> None

        Perform a sparse isometry operation.

        Args:
             QVec: qubits 

             std::map<std::string, std::complex<double>>: parameters for the isometry 

        Returns:
             circuit


        3. sparse_isometry(self: pyQPanda.Encode, qubit: pyQPanda.QVec, data: List[float]) -> None

        Perform a sparse isometry operation.

        Args:
             QVec: qubits 

             std::vector<double>: parameters for the isometry 

        Returns:
             circuit


        4. sparse_isometry(self: pyQPanda.Encode, qubit: pyQPanda.QVec, data: List[complex]) -> None

        Perform a sparse isometry operation.

        Args:
             QVec: qubits 

             std::vector<std::complex<double>>: parameters for the isometry 

        Returns:
             circuit

        """
    @overload
    def sparse_isometry(self, qubit: QVec, data: dict[str, complex]) -> None:
        """sparse_isometry(*args, **kwargs)
        Overloaded function.

        1. sparse_isometry(self: pyQPanda.Encode, qubit: pyQPanda.QVec, data: Dict[str, float]) -> None

        Perform a sparse isometry operation.

        Args:
             QVec: qubits 

             std::map<std::string, double>: parameters for the isometry 

        Returns:
             circuit


        2. sparse_isometry(self: pyQPanda.Encode, qubit: pyQPanda.QVec, data: Dict[str, complex]) -> None

        Perform a sparse isometry operation.

        Args:
             QVec: qubits 

             std::map<std::string, std::complex<double>>: parameters for the isometry 

        Returns:
             circuit


        3. sparse_isometry(self: pyQPanda.Encode, qubit: pyQPanda.QVec, data: List[float]) -> None

        Perform a sparse isometry operation.

        Args:
             QVec: qubits 

             std::vector<double>: parameters for the isometry 

        Returns:
             circuit


        4. sparse_isometry(self: pyQPanda.Encode, qubit: pyQPanda.QVec, data: List[complex]) -> None

        Perform a sparse isometry operation.

        Args:
             QVec: qubits 

             std::vector<std::complex<double>>: parameters for the isometry 

        Returns:
             circuit

        """
    @overload
    def sparse_isometry(self, qubit: QVec, data: list[float]) -> None:
        """sparse_isometry(*args, **kwargs)
        Overloaded function.

        1. sparse_isometry(self: pyQPanda.Encode, qubit: pyQPanda.QVec, data: Dict[str, float]) -> None

        Perform a sparse isometry operation.

        Args:
             QVec: qubits 

             std::map<std::string, double>: parameters for the isometry 

        Returns:
             circuit


        2. sparse_isometry(self: pyQPanda.Encode, qubit: pyQPanda.QVec, data: Dict[str, complex]) -> None

        Perform a sparse isometry operation.

        Args:
             QVec: qubits 

             std::map<std::string, std::complex<double>>: parameters for the isometry 

        Returns:
             circuit


        3. sparse_isometry(self: pyQPanda.Encode, qubit: pyQPanda.QVec, data: List[float]) -> None

        Perform a sparse isometry operation.

        Args:
             QVec: qubits 

             std::vector<double>: parameters for the isometry 

        Returns:
             circuit


        4. sparse_isometry(self: pyQPanda.Encode, qubit: pyQPanda.QVec, data: List[complex]) -> None

        Perform a sparse isometry operation.

        Args:
             QVec: qubits 

             std::vector<std::complex<double>>: parameters for the isometry 

        Returns:
             circuit

        """
    @overload
    def sparse_isometry(self, qubit: QVec, data: list[complex]) -> None:
        """sparse_isometry(*args, **kwargs)
        Overloaded function.

        1. sparse_isometry(self: pyQPanda.Encode, qubit: pyQPanda.QVec, data: Dict[str, float]) -> None

        Perform a sparse isometry operation.

        Args:
             QVec: qubits 

             std::map<std::string, double>: parameters for the isometry 

        Returns:
             circuit


        2. sparse_isometry(self: pyQPanda.Encode, qubit: pyQPanda.QVec, data: Dict[str, complex]) -> None

        Perform a sparse isometry operation.

        Args:
             QVec: qubits 

             std::map<std::string, std::complex<double>>: parameters for the isometry 

        Returns:
             circuit


        3. sparse_isometry(self: pyQPanda.Encode, qubit: pyQPanda.QVec, data: List[float]) -> None

        Perform a sparse isometry operation.

        Args:
             QVec: qubits 

             std::vector<double>: parameters for the isometry 

        Returns:
             circuit


        4. sparse_isometry(self: pyQPanda.Encode, qubit: pyQPanda.QVec, data: List[complex]) -> None

        Perform a sparse isometry operation.

        Args:
             QVec: qubits 

             std::vector<std::complex<double>>: parameters for the isometry 

        Returns:
             circuit

        """

class ErrorCode:
    __members__: ClassVar[dict] = ...  # read-only
    BACKEND_CALC_ERROR: ClassVar[ErrorCode] = ...
    CLUSTER_BASE: ClassVar[ErrorCode] = ...
    CLUSTER_SIMULATE_CALC_ERR: ClassVar[ErrorCode] = ...
    DATABASE_ERROR: ClassVar[ErrorCode] = ...
    ERR_BACKEND_CHIP_TASK_SOCKET_WRONG: ClassVar[ErrorCode] = ...
    ERR_CHIP_OFFLINE: ClassVar[ErrorCode] = ...
    ERR_EMPTY_PROG: ClassVar[ErrorCode] = ...
    ERR_FIDELITY_MATRIX: ClassVar[ErrorCode] = ...
    ERR_INVALID_URL: ClassVar[ErrorCode] = ...
    ERR_MATE_GATE_CONFIG: ClassVar[ErrorCode] = ...
    ERR_NOT_FOUND_APP_ID: ClassVar[ErrorCode] = ...
    ERR_NOT_FOUND_TASK_ID: ClassVar[ErrorCode] = ...
    ERR_OPERATOR_DB: ClassVar[ErrorCode] = ...
    ERR_PARAMETER: ClassVar[ErrorCode] = ...
    ERR_PARSER_SUB_TASK_RESULT: ClassVar[ErrorCode] = ...
    ERR_PRE_ESTIMATE: ClassVar[ErrorCode] = ...
    ERR_QCOMPILER_FAILED: ClassVar[ErrorCode] = ...
    ERR_QPROG_LENGTH: ClassVar[ErrorCode] = ...
    ERR_QST_PROG: ClassVar[ErrorCode] = ...
    ERR_QUANTUM_CHIP_PROG: ClassVar[ErrorCode] = ...
    ERR_QUBIT_SIZE: ClassVar[ErrorCode] = ...
    ERR_QUBIT_TOPO: ClassVar[ErrorCode] = ...
    ERR_QVM_INIT_FAILED: ClassVar[ErrorCode] = ...
    ERR_REPEAT_MEASURE: ClassVar[ErrorCode] = ...
    ERR_SCHEDULE_CHIP_TOPOLOGY_SUPPORTED: ClassVar[ErrorCode] = ...
    ERR_SUB_GRAPH_OUT_OF_RANGE: ClassVar[ErrorCode] = ...
    ERR_SYS_CALL_TIME_OUT: ClassVar[ErrorCode] = ...
    ERR_TASK_BUF_OVERFLOW: ClassVar[ErrorCode] = ...
    ERR_TASK_CONFIG: ClassVar[ErrorCode] = ...
    ERR_TASK_STATUS_BUF_OVERFLOW: ClassVar[ErrorCode] = ...
    ERR_TASK_TERMINATED: ClassVar[ErrorCode] = ...
    ERR_TCP_INIT_FATLT: ClassVar[ErrorCode] = ...
    ERR_TCP_SERVER_HALT: ClassVar[ErrorCode] = ...
    ERR_UNKNOW_TASK_TYPE: ClassVar[ErrorCode] = ...
    ERR_UNSUPPORT_BACKEND_TYPE: ClassVar[ErrorCode] = ...
    EXCEED_MAX_CLOCK: ClassVar[ErrorCode] = ...
    EXCEED_MAX_QUBIT: ClassVar[ErrorCode] = ...
    JSON_FIELD_ERROR: ClassVar[ErrorCode] = ...
    NO_ERROR_FOUND: ClassVar[ErrorCode] = ...
    ORIGINIR_ERROR: ClassVar[ErrorCode] = ...
    UNDEFINED_ERROR: ClassVar[ErrorCode] = ...
    __entries: ClassVar[dict] = ...
    def __init__(self, value: int) -> None:
        """__init__(self: pyQPanda.ErrorCode, value: int) -> None"""
    def __eq__(self, other: object) -> bool:
        """__eq__(self: object, other: object) -> bool"""
    def __hash__(self) -> int:
        """__hash__(self: object) -> int"""
    def __index__(self) -> int:
        """__index__(self: pyQPanda.ErrorCode) -> int"""
    def __int__(self) -> int:
        """__int__(self: pyQPanda.ErrorCode) -> int"""
    def __ne__(self, other: object) -> bool:
        """__ne__(self: object, other: object) -> bool"""
    @property
    def name(self) -> str: ...
    @property
    def value(self) -> int: ...

class FullAmplitudeQVM(QuantumMachine):
    def __init__(self) -> None:
        """__init__(self: pyQPanda.FullAmplitudeQVM) -> None"""
    def get_prob_dict(self, qubit_list: QVec, select_max: int = ...) -> dict[str, float]:
        """get_prob_dict(self: pyQPanda.FullAmplitudeQVM, qubit_list: pyQPanda.QVec, select_max: int = -1) -> Dict[str, float]

        Get a dictionary of probabilities for the specified qubits.

        Args:
             qubit_list: List of qubits to measure.

             select_max: int, optional, maximum number of entries to return (default: -1).

        Returns:
             Dictionary of probabilities as a reference.

        """
    def get_prob_list(self, qubit_list: QVec, select_max: int = ...) -> list[float]:
        """get_prob_list(self: pyQPanda.FullAmplitudeQVM, qubit_list: pyQPanda.QVec, select_max: int = -1) -> List[float]

        Get a list of probabilities for the specified qubits.

        Args:
             qubit_list: List of qubits to measure.

             select_max: int, optional, maximum number of probabilities to return (default: -1).

        Returns:
             List of probabilities as a reference.

        """
    def get_prob_tuple_list(self, qubit_list: QVec, select_max: int = ...) -> list[tuple[int, float]]:
        """get_prob_tuple_list(self: pyQPanda.FullAmplitudeQVM, qubit_list: pyQPanda.QVec, select_max: int = -1) -> List[Tuple[int, float]]

        Get a list of probability tuples for the specified qubits.

        Args:
             qubit_list: List of qubits to measure.

             select_max: int, optional, maximum number of tuples to return (default: -1).

        Returns:
             List of probability tuples as a reference.

        """
    @overload
    def init_qvm(self, arg0: str) -> None:
        """init_qvm(*args, **kwargs)
        Overloaded function.

        1. init_qvm(self: pyQPanda.FullAmplitudeQVM, arg0: str) -> None

        2. init_qvm(self: pyQPanda.FullAmplitudeQVM) -> None
        """
    @overload
    def init_qvm(self) -> None:
        """init_qvm(*args, **kwargs)
        Overloaded function.

        1. init_qvm(self: pyQPanda.FullAmplitudeQVM, arg0: str) -> None

        2. init_qvm(self: pyQPanda.FullAmplitudeQVM) -> None
        """
    def pmeasure(self, qubit_list: QVec, select_max: int = ...) -> list[tuple[int, float]]:
        """pmeasure(self: pyQPanda.FullAmplitudeQVM, qubit_list: pyQPanda.QVec, select_max: int = -1) -> List[Tuple[int, float]]

        Get the probability distribution over qubits.

        Args:
             qubit_list: List of qubits to measure.

             select_max: int, optional, maximum number of results to select (default: -1).

        Returns:
             Probability distribution as a reference.

        """
    def pmeasure_no_index(self, qubit_list: QVec) -> list[float]:
        """pmeasure_no_index(self: pyQPanda.FullAmplitudeQVM, qubit_list: pyQPanda.QVec) -> List[float]

        Get the probability distribution over qubits without index.

        Args:
             qubit_list: List of qubits to measure.

        Returns:
             Probability distribution as a reference.

        """
    @overload
    def prob_run_dict(self, program: QProg, qubit_list: QVec, select_max: int = ...) -> dict[str, float]:
        """prob_run_dict(*args, **kwargs)
        Overloaded function.

        1. prob_run_dict(self: pyQPanda.FullAmplitudeQVM, program: pyQPanda.QProg, qubit_list: pyQPanda.QVec, select_max: int = -1) -> Dict[str, float]

        Execute a quantum program and retrieve a dictionary of probabilities for the specified qubits.

        Args:
             program: The quantum program to execute.

             qubit_list: List of qubits to measure.

             select_max: int, optional, maximum number of entries in the dictionary to return (default: -1).

        Returns:
             Dictionary of probabilities.


        2. prob_run_dict(self: pyQPanda.FullAmplitudeQVM, program: pyQPanda.QProg, qubit_addr_list: List[int], select_max: int = -1) -> Dict[str, float]

        Execute a quantum program and retrieve a dictionary of probabilities using qubit addresses.

        Args:
             program: The quantum program to execute.

             qubit_addr_list: List of qubit addresses to measure.

             select_max: int, optional, maximum number of entries in the dictionary to return (default: -1).

        Returns:
             Dictionary of probabilities.

        """
    @overload
    def prob_run_dict(self, program: QProg, qubit_addr_list: list[int], select_max: int = ...) -> dict[str, float]:
        """prob_run_dict(*args, **kwargs)
        Overloaded function.

        1. prob_run_dict(self: pyQPanda.FullAmplitudeQVM, program: pyQPanda.QProg, qubit_list: pyQPanda.QVec, select_max: int = -1) -> Dict[str, float]

        Execute a quantum program and retrieve a dictionary of probabilities for the specified qubits.

        Args:
             program: The quantum program to execute.

             qubit_list: List of qubits to measure.

             select_max: int, optional, maximum number of entries in the dictionary to return (default: -1).

        Returns:
             Dictionary of probabilities.


        2. prob_run_dict(self: pyQPanda.FullAmplitudeQVM, program: pyQPanda.QProg, qubit_addr_list: List[int], select_max: int = -1) -> Dict[str, float]

        Execute a quantum program and retrieve a dictionary of probabilities using qubit addresses.

        Args:
             program: The quantum program to execute.

             qubit_addr_list: List of qubit addresses to measure.

             select_max: int, optional, maximum number of entries in the dictionary to return (default: -1).

        Returns:
             Dictionary of probabilities.

        """
    @overload
    def prob_run_list(self, program: QProg, qubit_list: QVec, select_max: int = ...) -> list[float]:
        """prob_run_list(*args, **kwargs)
        Overloaded function.

        1. prob_run_list(self: pyQPanda.FullAmplitudeQVM, program: pyQPanda.QProg, qubit_list: pyQPanda.QVec, select_max: int = -1) -> List[float]

        Execute a quantum program and retrieve a list of probabilities for the specified qubits.

        Args:
             program: The quantum program to execute.

             qubit_list: List of qubits to measure.

             select_max: int, optional, maximum number of probabilities to return (default: -1).

        Returns:
             List of probabilities.


        2. prob_run_list(self: pyQPanda.FullAmplitudeQVM, program: pyQPanda.QProg, qubit_addr_list: List[int], select_max: int = -1) -> List[float]

        Execute a quantum program and retrieve a list of probabilities using qubit addresses.

        Args:
             program: The quantum program to execute.

             qubit_addr_list: List of qubit addresses to measure.

             select_max: int, optional, maximum number of probabilities to return (default: -1).

        Returns:
             List of probabilities.

        """
    @overload
    def prob_run_list(self, program: QProg, qubit_addr_list: list[int], select_max: int = ...) -> list[float]:
        """prob_run_list(*args, **kwargs)
        Overloaded function.

        1. prob_run_list(self: pyQPanda.FullAmplitudeQVM, program: pyQPanda.QProg, qubit_list: pyQPanda.QVec, select_max: int = -1) -> List[float]

        Execute a quantum program and retrieve a list of probabilities for the specified qubits.

        Args:
             program: The quantum program to execute.

             qubit_list: List of qubits to measure.

             select_max: int, optional, maximum number of probabilities to return (default: -1).

        Returns:
             List of probabilities.


        2. prob_run_list(self: pyQPanda.FullAmplitudeQVM, program: pyQPanda.QProg, qubit_addr_list: List[int], select_max: int = -1) -> List[float]

        Execute a quantum program and retrieve a list of probabilities using qubit addresses.

        Args:
             program: The quantum program to execute.

             qubit_addr_list: List of qubit addresses to measure.

             select_max: int, optional, maximum number of probabilities to return (default: -1).

        Returns:
             List of probabilities.

        """
    @overload
    def prob_run_tuple_list(self, program: QProg, qubit_list: QVec, select_max: int = ...) -> list[tuple[int, float]]:
        """prob_run_tuple_list(*args, **kwargs)
        Overloaded function.

        1. prob_run_tuple_list(self: pyQPanda.FullAmplitudeQVM, program: pyQPanda.QProg, qubit_list: pyQPanda.QVec, select_max: int = -1) -> List[Tuple[int, float]]

        Execute a quantum program and get a list of probability tuples.

        Args:
             program: The quantum program to execute.

             qubit_list: List of qubits to measure.

             select_max: int, optional, maximum number of tuples to return (default: -1).

        Returns:
             List of probability tuples.


        2. prob_run_tuple_list(self: pyQPanda.FullAmplitudeQVM, program: pyQPanda.QProg, qubit_addr_list: List[int], select_max: int = -1) -> List[Tuple[int, float]]

        Execute a quantum program and get a list of probability tuples using qubit addresses.

        Args:
             program: The quantum program to execute.

             qubit_addr_list: List of qubit addresses to measure.

             select_max: int, optional, maximum number of tuples to return (default: -1).

        Returns:
             List of probability tuples.

        """
    @overload
    def prob_run_tuple_list(self, program: QProg, qubit_addr_list: list[int], select_max: int = ...) -> list[tuple[int, float]]:
        """prob_run_tuple_list(*args, **kwargs)
        Overloaded function.

        1. prob_run_tuple_list(self: pyQPanda.FullAmplitudeQVM, program: pyQPanda.QProg, qubit_list: pyQPanda.QVec, select_max: int = -1) -> List[Tuple[int, float]]

        Execute a quantum program and get a list of probability tuples.

        Args:
             program: The quantum program to execute.

             qubit_list: List of qubits to measure.

             select_max: int, optional, maximum number of tuples to return (default: -1).

        Returns:
             List of probability tuples.


        2. prob_run_tuple_list(self: pyQPanda.FullAmplitudeQVM, program: pyQPanda.QProg, qubit_addr_list: List[int], select_max: int = -1) -> List[Tuple[int, float]]

        Execute a quantum program and get a list of probability tuples using qubit addresses.

        Args:
             program: The quantum program to execute.

             qubit_addr_list: List of qubit addresses to measure.

             select_max: int, optional, maximum number of tuples to return (default: -1).

        Returns:
             List of probability tuples.

        """
    def quick_measure(self, qubit_list: QVec, shots: int) -> dict[str, int]:
        """quick_measure(self: pyQPanda.FullAmplitudeQVM, qubit_list: pyQPanda.QVec, shots: int) -> Dict[str, int]

        Perform a quick measurement on the specified qubits.

        Args:
             qubit_list: List of qubits to measure.

             shots: Number of measurement shots to perform.

        Returns:
             Reference to the measurement results.

        """

class Fusion:
    def __init__(self) -> None:
        """__init__(self: pyQPanda.Fusion) -> None"""
    @overload
    def aggregate_operations(self, circuit: QCircuit) -> None:
        """aggregate_operations(*args, **kwargs)
        Overloaded function.

        1. aggregate_operations(self: pyQPanda.Fusion, circuit: pyQPanda.QCircuit) -> None

        Aggregate operations into the provided quantum circuit.

        Args:
             circuit: The quantum circuit to which operations will be added.

        Returns:
             A reference to the modified circuit.


        2. aggregate_operations(self: pyQPanda.Fusion, qprog: pyQPanda.QProg) -> None

        Aggregate operations into the provided quantum program.

        Args:
             qprog: The quantum program to which operations will be added.

        Returns:
             A reference to the modified program.

        """
    @overload
    def aggregate_operations(self, qprog: QProg) -> None:
        """aggregate_operations(*args, **kwargs)
        Overloaded function.

        1. aggregate_operations(self: pyQPanda.Fusion, circuit: pyQPanda.QCircuit) -> None

        Aggregate operations into the provided quantum circuit.

        Args:
             circuit: The quantum circuit to which operations will be added.

        Returns:
             A reference to the modified circuit.


        2. aggregate_operations(self: pyQPanda.Fusion, qprog: pyQPanda.QProg) -> None

        Aggregate operations into the provided quantum program.

        Args:
             qprog: The quantum program to which operations will be added.

        Returns:
             A reference to the modified program.

        """

class GPUQVM(QuantumMachine):
    def __init__(self) -> None:
        """__init__(self: pyQPanda.GPUQVM) -> None"""
    def get_prob_dict(self, qubit_list: QVec, select_max: int = ...) -> dict[str, float]:
        """get_prob_dict(self: pyQPanda.GPUQVM, qubit_list: pyQPanda.QVec, select_max: int = -1) -> Dict[str, float]

        Get a dictionary of probabilities for the specified qubits.

        Args:
             qubit_list: List of qubits to measure.

             select_max: int, optional, maximum number of entries to return (default: -1).

        Returns:
             Dictionary of probabilities as a reference.

        """
    def get_prob_list(self, qubit_list: QVec, select_max: int = ...) -> list[float]:
        """get_prob_list(self: pyQPanda.GPUQVM, qubit_list: pyQPanda.QVec, select_max: int = -1) -> List[float]

        Get a list of probabilities for the specified qubits.

        Args:
             qubit_list: List of qubits to measure.

             select_max: int, optional, maximum number of probabilities to return (default: -1).

        Returns:
             List of probabilities as a reference.

        """
    def get_prob_tuple_list(self, qubit_list: QVec, select_max: int = ...) -> list[tuple[int, float]]:
        """get_prob_tuple_list(self: pyQPanda.GPUQVM, qubit_list: pyQPanda.QVec, select_max: int = -1) -> List[Tuple[int, float]]

        Get a list of probability tuples for the specified qubits.

        Args:
             qubit_list: List of qubits to measure.

             select_max: int, optional, maximum number of tuples to return (default: -1).

        Returns:
             List of probability tuples as a reference.

        """
    def pmeasure(self, qubit_list: QVec, select_max: int = ...) -> list[tuple[int, float]]:
        """pmeasure(self: pyQPanda.GPUQVM, qubit_list: pyQPanda.QVec, select_max: int = -1) -> List[Tuple[int, float]]

        Get the probability distribution over qubits.

        Args:
             qubit_list: List of qubits to measure.

             select_max: int, optional, maximum number of results to select (default: -1).

        Returns:
             Probability distribution as a reference.

        """
    def pmeasure_no_index(self, qubit_list: QVec) -> list[float]:
        """pmeasure_no_index(self: pyQPanda.GPUQVM, qubit_list: pyQPanda.QVec) -> List[float]

        Get the probability distribution over qubits without index.

        Args:
             qubit_list: List of qubits to measure.

        Returns:
             Probability distribution as a reference.

        """
    @overload
    def prob_run_dict(self, program: QProg, qubit_list: QVec, select_max: int = ...) -> dict[str, float]:
        """prob_run_dict(*args, **kwargs)
        Overloaded function.

        1. prob_run_dict(self: pyQPanda.GPUQVM, program: pyQPanda.QProg, qubit_list: pyQPanda.QVec, select_max: int = -1) -> Dict[str, float]

        Execute a quantum program and retrieve a dictionary of probabilities for the specified qubits.

        Args:
             program: The quantum program to execute.

             qubit_list: List of qubits to measure.

             select_max: int, optional, maximum number of entries in the dictionary to return (default: -1).

        Returns:
             Dictionary of probabilities.


        2. prob_run_dict(self: pyQPanda.GPUQVM, program: pyQPanda.QProg, qubit_addr_list: List[int], select_max: int = -1) -> Dict[str, float]

        Execute a quantum program and retrieve a dictionary of probabilities using qubit addresses.

        Args:
             program: The quantum program to execute.

             qubit_addr_list: List of qubit addresses to measure.

             select_max: int, optional, maximum number of entries in the dictionary to return (default: -1).

        Returns:
             Dictionary of probabilities.

        """
    @overload
    def prob_run_dict(self, program: QProg, qubit_addr_list: list[int], select_max: int = ...) -> dict[str, float]:
        """prob_run_dict(*args, **kwargs)
        Overloaded function.

        1. prob_run_dict(self: pyQPanda.GPUQVM, program: pyQPanda.QProg, qubit_list: pyQPanda.QVec, select_max: int = -1) -> Dict[str, float]

        Execute a quantum program and retrieve a dictionary of probabilities for the specified qubits.

        Args:
             program: The quantum program to execute.

             qubit_list: List of qubits to measure.

             select_max: int, optional, maximum number of entries in the dictionary to return (default: -1).

        Returns:
             Dictionary of probabilities.


        2. prob_run_dict(self: pyQPanda.GPUQVM, program: pyQPanda.QProg, qubit_addr_list: List[int], select_max: int = -1) -> Dict[str, float]

        Execute a quantum program and retrieve a dictionary of probabilities using qubit addresses.

        Args:
             program: The quantum program to execute.

             qubit_addr_list: List of qubit addresses to measure.

             select_max: int, optional, maximum number of entries in the dictionary to return (default: -1).

        Returns:
             Dictionary of probabilities.

        """
    @overload
    def prob_run_list(self, program: QProg, qubit_list: QVec, select_max: int = ...) -> list[float]:
        """prob_run_list(*args, **kwargs)
        Overloaded function.

        1. prob_run_list(self: pyQPanda.GPUQVM, program: pyQPanda.QProg, qubit_list: pyQPanda.QVec, select_max: int = -1) -> List[float]

        Execute a quantum program and retrieve a list of probabilities for the specified qubits.

        Args:
             program: The quantum program to execute.

             qubit_list: List of qubits to measure.

             select_max: int, optional, maximum number of probabilities to return (default: -1).

        Returns:
             List of probabilities.


        2. prob_run_list(self: pyQPanda.GPUQVM, program: pyQPanda.QProg, qubit_addr_list: List[int], select_max: int = -1) -> List[float]

        Execute a quantum program and retrieve a list of probabilities using qubit addresses.

        Args:
             program: The quantum program to execute.

             qubit_addr_list: List of qubit addresses to measure.

             select_max: int, optional, maximum number of probabilities to return (default: -1).

        Returns:
             List of probabilities.

        """
    @overload
    def prob_run_list(self, program: QProg, qubit_addr_list: list[int], select_max: int = ...) -> list[float]:
        """prob_run_list(*args, **kwargs)
        Overloaded function.

        1. prob_run_list(self: pyQPanda.GPUQVM, program: pyQPanda.QProg, qubit_list: pyQPanda.QVec, select_max: int = -1) -> List[float]

        Execute a quantum program and retrieve a list of probabilities for the specified qubits.

        Args:
             program: The quantum program to execute.

             qubit_list: List of qubits to measure.

             select_max: int, optional, maximum number of probabilities to return (default: -1).

        Returns:
             List of probabilities.


        2. prob_run_list(self: pyQPanda.GPUQVM, program: pyQPanda.QProg, qubit_addr_list: List[int], select_max: int = -1) -> List[float]

        Execute a quantum program and retrieve a list of probabilities using qubit addresses.

        Args:
             program: The quantum program to execute.

             qubit_addr_list: List of qubit addresses to measure.

             select_max: int, optional, maximum number of probabilities to return (default: -1).

        Returns:
             List of probabilities.

        """
    @overload
    def prob_run_tuple_list(self, program: QProg, qubit_list: QVec, select_max: int = ...) -> list[tuple[int, float]]:
        """prob_run_tuple_list(*args, **kwargs)
        Overloaded function.

        1. prob_run_tuple_list(self: pyQPanda.GPUQVM, program: pyQPanda.QProg, qubit_list: pyQPanda.QVec, select_max: int = -1) -> List[Tuple[int, float]]

        Execute a quantum program and get a list of probability tuples.

        Args:
             program: The quantum program to execute.

             qubit_list: List of qubits to measure.

             select_max: int, optional, maximum number of tuples to return (default: -1).

        Returns:
             List of probability tuples.


        2. prob_run_tuple_list(self: pyQPanda.GPUQVM, program: pyQPanda.QProg, qubit_addr_list: List[int], select_max: int = -1) -> List[Tuple[int, float]]

        Execute a quantum program and get a list of probability tuples using qubit addresses.

        Args:
             program: The quantum program to execute.

             qubit_addr_list: List of qubit addresses to measure.

             select_max: int, optional, maximum number of tuples to return (default: -1).

        Returns:
             List of probability tuples.

        """
    @overload
    def prob_run_tuple_list(self, program: QProg, qubit_addr_list: list[int], select_max: int = ...) -> list[tuple[int, float]]:
        """prob_run_tuple_list(*args, **kwargs)
        Overloaded function.

        1. prob_run_tuple_list(self: pyQPanda.GPUQVM, program: pyQPanda.QProg, qubit_list: pyQPanda.QVec, select_max: int = -1) -> List[Tuple[int, float]]

        Execute a quantum program and get a list of probability tuples.

        Args:
             program: The quantum program to execute.

             qubit_list: List of qubits to measure.

             select_max: int, optional, maximum number of tuples to return (default: -1).

        Returns:
             List of probability tuples.


        2. prob_run_tuple_list(self: pyQPanda.GPUQVM, program: pyQPanda.QProg, qubit_addr_list: List[int], select_max: int = -1) -> List[Tuple[int, float]]

        Execute a quantum program and get a list of probability tuples using qubit addresses.

        Args:
             program: The quantum program to execute.

             qubit_addr_list: List of qubit addresses to measure.

             select_max: int, optional, maximum number of tuples to return (default: -1).

        Returns:
             List of probability tuples.

        """
    def quick_measure(self, qubit_list: QVec, shots: int) -> dict[str, int]:
        """quick_measure(self: pyQPanda.GPUQVM, qubit_list: pyQPanda.QVec, shots: int) -> Dict[str, int]

        Perform a quick measurement on the specified qubits.

        Args:
             qubit_list: List of qubits to measure.

             shots: Number of measurement shots to perform.

        Returns:
             Reference to the measurement results.

        """

class GateType:
    __members__: ClassVar[dict] = ...  # read-only
    BARRIER_GATE: ClassVar[GateType] = ...
    CNOT_GATE: ClassVar[GateType] = ...
    CPHASE_GATE: ClassVar[GateType] = ...
    CP_GATE: ClassVar[GateType] = ...
    CU_GATE: ClassVar[GateType] = ...
    CZ_GATE: ClassVar[GateType] = ...
    GATE_NOP: ClassVar[GateType] = ...
    GATE_UNDEFINED: ClassVar[GateType] = ...
    HADAMARD_GATE: ClassVar[GateType] = ...
    ISWAP_GATE: ClassVar[GateType] = ...
    ISWAP_THETA_GATE: ClassVar[GateType] = ...
    I_GATE: ClassVar[GateType] = ...
    MS_GATE: ClassVar[GateType] = ...
    ORACLE_GATE: ClassVar[GateType] = ...
    P00_GATE: ClassVar[GateType] = ...
    P0_GATE: ClassVar[GateType] = ...
    P11_GATE: ClassVar[GateType] = ...
    P1_GATE: ClassVar[GateType] = ...
    PAULI_X_GATE: ClassVar[GateType] = ...
    PAULI_Y_GATE: ClassVar[GateType] = ...
    PAULI_Z_GATE: ClassVar[GateType] = ...
    P_GATE: ClassVar[GateType] = ...
    RPHI_GATE: ClassVar[GateType] = ...
    RXX_GATE: ClassVar[GateType] = ...
    RX_GATE: ClassVar[GateType] = ...
    RYY_GATE: ClassVar[GateType] = ...
    RY_GATE: ClassVar[GateType] = ...
    RZX_GATE: ClassVar[GateType] = ...
    RZZ_GATE: ClassVar[GateType] = ...
    RZ_GATE: ClassVar[GateType] = ...
    SQISWAP_GATE: ClassVar[GateType] = ...
    SWAP_GATE: ClassVar[GateType] = ...
    S_GATE: ClassVar[GateType] = ...
    TOFFOLI_GATE: ClassVar[GateType] = ...
    TWO_QUBIT_GATE: ClassVar[GateType] = ...
    T_GATE: ClassVar[GateType] = ...
    U1_GATE: ClassVar[GateType] = ...
    U2_GATE: ClassVar[GateType] = ...
    U3_GATE: ClassVar[GateType] = ...
    U4_GATE: ClassVar[GateType] = ...
    X_HALF_PI: ClassVar[GateType] = ...
    Y_HALF_PI: ClassVar[GateType] = ...
    Z_HALF_PI: ClassVar[GateType] = ...
    __entries: ClassVar[dict] = ...
    def __init__(self, value: int) -> None:
        """__init__(self: pyQPanda.GateType, value: int) -> None"""
    def __eq__(self, other: object) -> bool:
        """__eq__(self: object, other: object) -> bool"""
    def __hash__(self) -> int:
        """__hash__(self: object) -> int"""
    def __index__(self) -> int:
        """__index__(self: pyQPanda.GateType) -> int"""
    def __int__(self) -> int:
        """__int__(self: pyQPanda.GateType) -> int"""
    def __ne__(self, other: object) -> bool:
        """__ne__(self: object, other: object) -> bool"""
    @property
    def name(self) -> str: ...
    @property
    def value(self) -> int: ...

class HHLAlg:
    def __init__(self, arg0: QuantumMachine) -> None:
        """__init__(self: pyQPanda.HHLAlg, arg0: pyQPanda.QuantumMachine) -> None"""
    def check_QPE_result(self) -> str:
        """check_QPE_result(self: pyQPanda.HHLAlg) -> str

        check QPE result
        """
    def get_amplification_factor(self) -> float:
        """get_amplification_factor(self: pyQPanda.HHLAlg) -> float

        get_amplification_factor
        """
    def get_ancillary_qubit(self) -> QVec:
        """get_ancillary_qubit(self: pyQPanda.HHLAlg) -> pyQPanda.QVec

        get_ancillary_qubit
        """
    def get_hhl_circuit(self, matrix_A: list[complex], data_b: list[float], precision_cnt: int = ...) -> QCircuit:
        """get_hhl_circuit(self: pyQPanda.HHLAlg, matrix_A: List[complex], data_b: List[float], precision_cnt: int = 0) -> pyQPanda.QCircuit"""
    def get_qubit_for_QFT(self) -> list[Qubit]:
        """get_qubit_for_QFT(self: pyQPanda.HHLAlg) -> List[pyQPanda.Qubit]

        get_qubit_for_QFT
        """
    def get_qubit_for_b(self) -> list[Qubit]:
        """get_qubit_for_b(self: pyQPanda.HHLAlg) -> List[pyQPanda.Qubit]

        get_qubit_for_b
        """
    def query_uesed_qubit_num(self) -> int:
        """query_uesed_qubit_num(self: pyQPanda.HHLAlg) -> int

        query_uesed_qubit_num
        """

class LATEX_GATE_TYPE:
    __members__: ClassVar[dict] = ...  # read-only
    CNOT_GATE: ClassVar[LATEX_GATE_TYPE] = ...
    GENERAL_GATE: ClassVar[LATEX_GATE_TYPE] = ...
    SWAP_GATE: ClassVar[LATEX_GATE_TYPE] = ...
    __entries: ClassVar[dict] = ...
    def __init__(self, value: int) -> None:
        """__init__(self: pyQPanda.LATEX_GATE_TYPE, value: int) -> None"""
    def __eq__(self, other: object) -> bool:
        """__eq__(self: object, other: object) -> bool"""
    def __hash__(self) -> int:
        """__hash__(self: object) -> int"""
    def __index__(self) -> int:
        """__index__(self: pyQPanda.LATEX_GATE_TYPE) -> int"""
    def __int__(self) -> int:
        """__int__(self: pyQPanda.LATEX_GATE_TYPE) -> int"""
    def __ne__(self, other: object) -> bool:
        """__ne__(self: object, other: object) -> bool"""
    @property
    def name(self) -> str: ...
    @property
    def value(self) -> int: ...

class LatexMatrix:
    def __init__(self) -> None:
        """__init__(self: pyQPanda.LatexMatrix) -> None"""
    def insert_barrier(self, rows: list[int], from_col: int) -> int:
        """insert_barrier(self: pyQPanda.LatexMatrix, rows: List[int], from_col: int) -> int

        Insert a barrier into the circuit.

        Args:
             rows: The rows of the LaTeX matrix where the barrier is applied.

             from_col: Desired column position for the barrier; if space is insufficient, a suitable column will be found.

        Returns:
             int: Actual column number where the barrier is placed.

        """
    def insert_gate(self, target_rows: list[int], ctrl_rows: list[int], from_col: int, gate_type: LATEX_GATE_TYPE, gate_name: str = ..., dagger: bool = ..., param: str = ...) -> int:
        """insert_gate(self: pyQPanda.LatexMatrix, target_rows: List[int], ctrl_rows: List[int], from_col: int, gate_type: pyQPanda.LATEX_GATE_TYPE, gate_name: str = '', dagger: bool = False, param: str = '') -> int

        Insert a gate into the circuit.

        Args:
             target_rows: Gate target rows of the LaTeX matrix.

             ctrl_rows: Control rows for the gate.

             from_col: Desired column position for the gate; if space is insufficient, a suitable column will be found.

             gate_type: Enum type of LATEX_GATE_TYPE.

             gate_name: Name of the gate (default: '').

             dagger: Flag indicating if the gate is a dagger (default: false).

             param: Parameter string for the gate (default: '').

        Returns:
             int: Actual column number where the gate is placed.

        """
    def insert_measure(self, q_row: int, c_row: int, from_col: int, cbit_id: int) -> int:
        """insert_measure(self: pyQPanda.LatexMatrix, q_row: int, c_row: int, from_col: int, cbit_id: int) -> int

        Insert a measurement operation into the circuit.

        Args:
             q_row: The row of the qubit being measured.

             c_row: The row of the classical bit that will store the measurement result.

             from_col: The desired column position for the measurement.

        Returns:
             None, as the function modifies the matrix in place.

        """
    def insert_reset(self, q_row: int, from_col: int) -> int:
        """insert_reset(self: pyQPanda.LatexMatrix, q_row: int, from_col: int) -> int

        Insert a reset operation into the circuit.

        Args:
             q_row: The row of the qubit to be reset.

             from_col: The desired column position for the reset.

        Returns:
             None, as the function modifies the matrix in place.

        """
    def insert_timeseq(self, t_col: int, time_seq: int) -> None:
        """insert_timeseq(self: pyQPanda.LatexMatrix, t_col: int, time_seq: int) -> None

        Insert a time sequence into the circuit.

        Args:
             t_col: The column position where the time sequence will be inserted.

             time_seq: The time sequence data to be inserted.

        Warning:
             This function does not check for column number validity, which may cause overwriting.

             Users must ensure the column number is managed correctly to avoid conflicts.

        """
    def set_label(self, qubit_label: dict[int, str], cbit_label: dict[int, str] = ..., time_seq_label: str = ..., head: bool = ...) -> None:
        """set_label(self: pyQPanda.LatexMatrix, qubit_label: Dict[int, str], cbit_label: Dict[int, str] = {}, time_seq_label: str = '', head: bool = True) -> None

        Set label at the leftmost head column or rightmost tail column.
        Labels can be reset at any time.

        Args:
             qubit_label: Label for the qubit wire's leftmost head label, specified in LaTeX syntax.If not given, the row will remain empty (e.g., {0: 'q_{1}', 2:'q_{2}'}).

             cbit_label: Classic label string, supports LaTeX formatting.

             time_seq_label: If given, sets the time sequence label.

             head: If true, appends the label at the head; if false, appends at the tail.

        Returns:
             None, as the function modifies the matrix in place.

        """
    def set_logo(self, logo: str = ...) -> None:
        """set_logo(self: pyQPanda.LatexMatrix, logo: str = '') -> None

        Add a logo string.

        Args:
             logo: The logo string to be added. If not provided, the logo will be set to an empty string.

        Returns:
             None, as the function modifies the matrix in place.

        """
    def str(self, with_time: bool = ...) -> str:
        """str(self: pyQPanda.LatexMatrix, with_time: bool = False) -> str

        Return the final LaTeX source code representation of the matrix.

        Args:
             with_time: A boolean flag indicating whether to include timing information in the output.

        Returns:
             str: The LaTeX source code as a string. This method can be called at any time to obtain the current state of the matrix.

        """

class MPSQVM(QuantumMachine):
    def __init__(self) -> None:
        """__init__(self: pyQPanda.MPSQVM) -> None"""
    @overload
    def add_single_noise_model(self, arg0: NoiseModel, arg1: GateType, arg2: float) -> None:
        """add_single_noise_model(*args, **kwargs)
        Overloaded function.

        1. add_single_noise_model(self: pyQPanda.MPSQVM, arg0: pyQPanda.NoiseModel, arg1: pyQPanda.GateType, arg2: float) -> None

        Add a noise model to a specific gate.

        Args:
             noise_model: NOISE_MODEL, the type of noise model to apply.

             gate_type: GateType, the type of gate affected by the noise.

             error_rate: float, the rate of noise occurrence.

        Returns:
             None.


        2. add_single_noise_model(self: pyQPanda.MPSQVM, arg0: pyQPanda.NoiseModel, arg1: pyQPanda.GateType, arg2: float, arg3: float, arg4: float) -> None

        Add a noise model to a specific gate with multiple error rates.

        Args:
             noise_model: NOISE_MODEL, the type of noise model to apply.

             gate_type: GateType, the type of gate affected by the noise.

             error_rate_1: float, the first error rate.

             error_rate_2: float, the second error rate.

             error_rate_3: float, the third error rate.

        Returns:
             None.

        """
    @overload
    def add_single_noise_model(self, arg0: NoiseModel, arg1: GateType, arg2: float, arg3: float, arg4: float) -> None:
        """add_single_noise_model(*args, **kwargs)
        Overloaded function.

        1. add_single_noise_model(self: pyQPanda.MPSQVM, arg0: pyQPanda.NoiseModel, arg1: pyQPanda.GateType, arg2: float) -> None

        Add a noise model to a specific gate.

        Args:
             noise_model: NOISE_MODEL, the type of noise model to apply.

             gate_type: GateType, the type of gate affected by the noise.

             error_rate: float, the rate of noise occurrence.

        Returns:
             None.


        2. add_single_noise_model(self: pyQPanda.MPSQVM, arg0: pyQPanda.NoiseModel, arg1: pyQPanda.GateType, arg2: float, arg3: float, arg4: float) -> None

        Add a noise model to a specific gate with multiple error rates.

        Args:
             noise_model: NOISE_MODEL, the type of noise model to apply.

             gate_type: GateType, the type of gate affected by the noise.

             error_rate_1: float, the first error rate.

             error_rate_2: float, the second error rate.

             error_rate_3: float, the third error rate.

        Returns:
             None.

        """
    def get_prob_dict(self, qubit_list: QVec, select_max: int = ...) -> dict[str, float]:
        """get_prob_dict(self: pyQPanda.MPSQVM, qubit_list: pyQPanda.QVec, select_max: int = -1) -> Dict[str, float]

        Get pmeasure result as dict.

        Args:
             qubit_list: List of qubits for pmeasure.

             select_max: Maximum number of returned elements in the result tuple; should be in [-1, 1<<len(qubit_list)]. Default is -1, meaning no limit.

        Returns:
             Measure result of the quantum machine.

        """
    def get_prob_list(self, qubit_list: QVec, select_max: int = ...) -> list[float]:
        """get_prob_list(self: pyQPanda.MPSQVM, qubit_list: pyQPanda.QVec, select_max: int = -1) -> List[float]

        Get pmeasure result as list.

        Args:
             qubit_list: List of qubits for pmeasure.

             select_max: Maximum number of returned elements in the result tuple; should be in [-1, 1<<len(qubit_list)]. Default is -1, meaning no limit.

        Returns:
             Measure result of the quantum machine.

        """
    def get_prob_tuple_list(self, qubit_list: QVec, select_max: int = ...) -> list[tuple[int, float]]:
        """get_prob_tuple_list(self: pyQPanda.MPSQVM, qubit_list: pyQPanda.QVec, select_max: int = -1) -> List[Tuple[int, float]]

        Get pmeasure result as list.

        Args:
             qubit_list: List of qubits for pmeasure.

             select_max: Maximum number of returned elements in the result tuple; should be in [-1, 1<<len(qubit_list)]. Default is -1, meaning no limit.

        Returns:
             Measure result of the quantum machine.

        """
    def pmeasure(self, qubit_list: QVec, select_max: int = ...) -> list[tuple[int, float]]:
        """pmeasure(self: pyQPanda.MPSQVM, qubit_list: pyQPanda.QVec, select_max: int = -1) -> List[Tuple[int, float]]

        Get the probability distribution over qubits.

        Args:
             qubit_list: List of qubits to measure.

             select_max: Maximum number of returned elements in the result tuple; should be in [-1, 1<<len(qubit_list)]. Default is -1, which means no limit.

        Returns:
             Measure result of the quantum machine in tuple form.

        """
    def pmeasure_bin_index(self, program: QProg, string: str) -> complex:
        """pmeasure_bin_index(self: pyQPanda.MPSQVM, program: pyQPanda.QProg, string: str) -> complex

        Get pmeasure bin index quantum state amplitude.

        Args:
             string: Bin string.

        Returns:
             Complex: Bin amplitude.

        """
    def pmeasure_bin_subset(self, program: QProg, string_list: list[str]) -> list[complex]:
        """pmeasure_bin_subset(self: pyQPanda.MPSQVM, program: pyQPanda.QProg, string_list: List[str]) -> List[complex]

        Get pmeasure quantum state amplitude subset.

        Args:
             list: List of bin state strings.

        Returns:
             List: Bin amplitude result list.

        """
    def pmeasure_dec_index(self, program: QProg, string: str) -> complex:
        """pmeasure_dec_index(self: pyQPanda.MPSQVM, program: pyQPanda.QProg, string: str) -> complex

        Get pmeasure decimal index quantum state amplitude.

        Args:
             string: Decimal string.

        Returns:
             Complex: Decimal amplitude.

        """
    def pmeasure_dec_subset(self, program: QProg, string_list: list[str]) -> list[complex]:
        """pmeasure_dec_subset(self: pyQPanda.MPSQVM, program: pyQPanda.QProg, string_list: List[str]) -> List[complex]

        Get pmeasure quantum state amplitude subset.

        Args:
             list: List of decimal state strings.

        Returns:
             List: Decimal amplitude result list.

        """
    def pmeasure_no_index(self, qubit_list: QVec) -> list[float]:
        """pmeasure_no_index(self: pyQPanda.MPSQVM, qubit_list: pyQPanda.QVec) -> List[float]

        Get the probability distribution over qubits.

        Args:
             qubit_list: List of qubits to measure.

        Returns:
             Measure result of the quantum machine in list form.

        """
    def prob_run_dict(self, program: QProg, qubit_list: QVec, select_max: int = ...) -> dict[str, float]:
        """prob_run_dict(self: pyQPanda.MPSQVM, program: pyQPanda.QProg, qubit_list: pyQPanda.QVec, select_max: int = -1) -> Dict[str, float]

        Run quantum program and get pmeasure result as dict.
        Args:
             program: Quantum program to run.

             qubit_list: List of qubits for pmeasure.

             select_max: Maximum number of returned elements in the result; should be in [-1, 1<<len(qubit_list)]. Default is -1, meaning no limit.
        Returns:
             Measure result of the quantum machine.

        """
    def prob_run_list(self, program: QProg, qubit_list: QVec, select_max: int = ...) -> list[float]:
        """prob_run_list(self: pyQPanda.MPSQVM, program: pyQPanda.QProg, qubit_list: pyQPanda.QVec, select_max: int = -1) -> List[float]

        Run quantum program and get pmeasure result as list.

        Args:
             program: Quantum program to run.

             qubit_list: List of qubits for pmeasure.

             select_max: Maximum number of returned elements in the result; should be in [-1, 1<<len(qubit_list)]. Default is -1, meaning no limit.

        Returns:
             Measure result of the quantum machine.

        """
    def prob_run_tuple_list(self, program: QProg, qubit_list: QVec, select_max: int = ...) -> list[tuple[int, float]]:
        """prob_run_tuple_list(self: pyQPanda.MPSQVM, program: pyQPanda.QProg, qubit_list: pyQPanda.QVec, select_max: int = -1) -> List[Tuple[int, float]]

        Run quantum program and get pmeasure result as tuple list.

        Args:
             program: Quantum program to run.

             qubit_list: List of qubits for pmeasure.

             select_max: Maximum number of returned elements in the result tuple; should be in [-1, 1<<len(qubit_list)]. Default is -1, meaning no limit.

        Returns:
             Measure result of the quantum machine.

        """
    def quick_measure(self, qubit_list: QVec, shots: int) -> dict[str, int]:
        """quick_measure(self: pyQPanda.MPSQVM, qubit_list: pyQPanda.QVec, shots: int) -> Dict[str, int]

        Quick measure.

        Args:
             qubit_list: List of qubits to measure.

             shots: The number of repetitions for the measurement operation.

        Returns:
             Result of the quantum program.

        """
    @overload
    def set_measure_error(self, arg0: NoiseModel, arg1: float) -> None:
        """set_measure_error(*args, **kwargs)
        Overloaded function.

        1. set_measure_error(self: pyQPanda.MPSQVM, arg0: pyQPanda.NoiseModel, arg1: float) -> None

        Set the measurement error based on the specified noise model.

        Args:
             noise_model: The type of noise model to apply.

             error_rate: The rate of measurement error to be set.


        2. set_measure_error(self: pyQPanda.MPSQVM, arg0: pyQPanda.NoiseModel, arg1: float, arg2: float, arg3: float) -> None

        Set the measurement error with multiple error rates for the specified noise model.

        Args:
             noise_model: The type of noise model to apply.

             error_rate1: First error rate.

             error_rate2: Second error rate.

             error_rate3: Third error rate.

        """
    @overload
    def set_measure_error(self, arg0: NoiseModel, arg1: float, arg2: float, arg3: float) -> None:
        """set_measure_error(*args, **kwargs)
        Overloaded function.

        1. set_measure_error(self: pyQPanda.MPSQVM, arg0: pyQPanda.NoiseModel, arg1: float) -> None

        Set the measurement error based on the specified noise model.

        Args:
             noise_model: The type of noise model to apply.

             error_rate: The rate of measurement error to be set.


        2. set_measure_error(self: pyQPanda.MPSQVM, arg0: pyQPanda.NoiseModel, arg1: float, arg2: float, arg3: float) -> None

        Set the measurement error with multiple error rates for the specified noise model.

        Args:
             noise_model: The type of noise model to apply.

             error_rate1: First error rate.

             error_rate2: Second error rate.

             error_rate3: Third error rate.

        """
    @overload
    def set_mixed_unitary_error(self, arg0: GateType, arg1: list[list[complex]], arg2: list[QVec]) -> None:
        """set_mixed_unitary_error(*args, **kwargs)
        Overloaded function.

        1. set_mixed_unitary_error(self: pyQPanda.MPSQVM, arg0: pyQPanda.GateType, arg1: List[List[complex]], arg2: List[pyQPanda.QVec]) -> None

        Set mixed unitary errors for the specified gate type.

        Args:
             gate_type: Type of gate affected by the error.

             unitary_errors: List of unitary error matrices.

             qubits: List of qubits where the errors will be applied.


        2. set_mixed_unitary_error(self: pyQPanda.MPSQVM, arg0: pyQPanda.GateType, arg1: List[List[complex]], arg2: List[float], arg3: List[pyQPanda.QVec]) -> None

        Set mixed unitary errors with associated probabilities for the specified gate type.

        Args:
             gate_type: Type of gate affected by the error.

             unitary_errors: List of unitary error matrices.

             probabilities: Probabilities associated with each unitary error.

             qubits: List of qubits where the errors will be applied.


        3. set_mixed_unitary_error(self: pyQPanda.MPSQVM, arg0: pyQPanda.GateType, arg1: List[List[complex]]) -> None

        Set mixed unitary errors for the specified gate type.

        Args:
             gate_type: Type of gate affected by the error.

             unitary_errors: List of unitary error matrices to apply for the gate type.


        4. set_mixed_unitary_error(self: pyQPanda.MPSQVM, arg0: pyQPanda.GateType, arg1: List[List[complex]], arg2: List[float]) -> None

        Set mixed unitary errors with associated probabilities for the specified gate type.

        Args:
             gate_type: Type of gate affected by the error.

             unitary_errors: List of unitary error matrices.

             probabilities: Probabilities associated with each unitary error.

        """
    @overload
    def set_mixed_unitary_error(self, arg0: GateType, arg1: list[list[complex]], arg2: list[float], arg3: list[QVec]) -> None:
        """set_mixed_unitary_error(*args, **kwargs)
        Overloaded function.

        1. set_mixed_unitary_error(self: pyQPanda.MPSQVM, arg0: pyQPanda.GateType, arg1: List[List[complex]], arg2: List[pyQPanda.QVec]) -> None

        Set mixed unitary errors for the specified gate type.

        Args:
             gate_type: Type of gate affected by the error.

             unitary_errors: List of unitary error matrices.

             qubits: List of qubits where the errors will be applied.


        2. set_mixed_unitary_error(self: pyQPanda.MPSQVM, arg0: pyQPanda.GateType, arg1: List[List[complex]], arg2: List[float], arg3: List[pyQPanda.QVec]) -> None

        Set mixed unitary errors with associated probabilities for the specified gate type.

        Args:
             gate_type: Type of gate affected by the error.

             unitary_errors: List of unitary error matrices.

             probabilities: Probabilities associated with each unitary error.

             qubits: List of qubits where the errors will be applied.


        3. set_mixed_unitary_error(self: pyQPanda.MPSQVM, arg0: pyQPanda.GateType, arg1: List[List[complex]]) -> None

        Set mixed unitary errors for the specified gate type.

        Args:
             gate_type: Type of gate affected by the error.

             unitary_errors: List of unitary error matrices to apply for the gate type.


        4. set_mixed_unitary_error(self: pyQPanda.MPSQVM, arg0: pyQPanda.GateType, arg1: List[List[complex]], arg2: List[float]) -> None

        Set mixed unitary errors with associated probabilities for the specified gate type.

        Args:
             gate_type: Type of gate affected by the error.

             unitary_errors: List of unitary error matrices.

             probabilities: Probabilities associated with each unitary error.

        """
    @overload
    def set_mixed_unitary_error(self, arg0: GateType, arg1: list[list[complex]]) -> None:
        """set_mixed_unitary_error(*args, **kwargs)
        Overloaded function.

        1. set_mixed_unitary_error(self: pyQPanda.MPSQVM, arg0: pyQPanda.GateType, arg1: List[List[complex]], arg2: List[pyQPanda.QVec]) -> None

        Set mixed unitary errors for the specified gate type.

        Args:
             gate_type: Type of gate affected by the error.

             unitary_errors: List of unitary error matrices.

             qubits: List of qubits where the errors will be applied.


        2. set_mixed_unitary_error(self: pyQPanda.MPSQVM, arg0: pyQPanda.GateType, arg1: List[List[complex]], arg2: List[float], arg3: List[pyQPanda.QVec]) -> None

        Set mixed unitary errors with associated probabilities for the specified gate type.

        Args:
             gate_type: Type of gate affected by the error.

             unitary_errors: List of unitary error matrices.

             probabilities: Probabilities associated with each unitary error.

             qubits: List of qubits where the errors will be applied.


        3. set_mixed_unitary_error(self: pyQPanda.MPSQVM, arg0: pyQPanda.GateType, arg1: List[List[complex]]) -> None

        Set mixed unitary errors for the specified gate type.

        Args:
             gate_type: Type of gate affected by the error.

             unitary_errors: List of unitary error matrices to apply for the gate type.


        4. set_mixed_unitary_error(self: pyQPanda.MPSQVM, arg0: pyQPanda.GateType, arg1: List[List[complex]], arg2: List[float]) -> None

        Set mixed unitary errors with associated probabilities for the specified gate type.

        Args:
             gate_type: Type of gate affected by the error.

             unitary_errors: List of unitary error matrices.

             probabilities: Probabilities associated with each unitary error.

        """
    @overload
    def set_mixed_unitary_error(self, arg0: GateType, arg1: list[list[complex]], arg2: list[float]) -> None:
        """set_mixed_unitary_error(*args, **kwargs)
        Overloaded function.

        1. set_mixed_unitary_error(self: pyQPanda.MPSQVM, arg0: pyQPanda.GateType, arg1: List[List[complex]], arg2: List[pyQPanda.QVec]) -> None

        Set mixed unitary errors for the specified gate type.

        Args:
             gate_type: Type of gate affected by the error.

             unitary_errors: List of unitary error matrices.

             qubits: List of qubits where the errors will be applied.


        2. set_mixed_unitary_error(self: pyQPanda.MPSQVM, arg0: pyQPanda.GateType, arg1: List[List[complex]], arg2: List[float], arg3: List[pyQPanda.QVec]) -> None

        Set mixed unitary errors with associated probabilities for the specified gate type.

        Args:
             gate_type: Type of gate affected by the error.

             unitary_errors: List of unitary error matrices.

             probabilities: Probabilities associated with each unitary error.

             qubits: List of qubits where the errors will be applied.


        3. set_mixed_unitary_error(self: pyQPanda.MPSQVM, arg0: pyQPanda.GateType, arg1: List[List[complex]]) -> None

        Set mixed unitary errors for the specified gate type.

        Args:
             gate_type: Type of gate affected by the error.

             unitary_errors: List of unitary error matrices to apply for the gate type.


        4. set_mixed_unitary_error(self: pyQPanda.MPSQVM, arg0: pyQPanda.GateType, arg1: List[List[complex]], arg2: List[float]) -> None

        Set mixed unitary errors with associated probabilities for the specified gate type.

        Args:
             gate_type: Type of gate affected by the error.

             unitary_errors: List of unitary error matrices.

             probabilities: Probabilities associated with each unitary error.

        """
    @overload
    def set_noise_model(self, arg0: NoiseModel, arg1: GateType, arg2: float) -> None:
        """set_noise_model(*args, **kwargs)
        Overloaded function.

        1. set_noise_model(self: pyQPanda.MPSQVM, arg0: pyQPanda.NoiseModel, arg1: pyQPanda.GateType, arg2: float) -> None

        Set the noise model for the quantum simulation.

        Args:
             noise_model: Type of noise model (bit-flip, phase-flip, etc.).

             gate_type: Type of gate affected by the noise.

             noise_level: Level of noise to apply.


        2. set_noise_model(self: pyQPanda.MPSQVM, arg0: pyQPanda.NoiseModel, arg1: pyQPanda.GateType, arg2: float, arg3: List[pyQPanda.QVec]) -> None

        Set the noise model for the quantum simulation with specific qubits.

        Args:
             noise_model: Type of noise model (bit-flip, phase-flip, etc.).

             gate_type: Type of gate affected by the noise.

             noise_level: Level of noise to apply.

             qubits: List of qubits to which the noise model will be applied.


        3. set_noise_model(self: pyQPanda.MPSQVM, arg0: pyQPanda.NoiseModel, arg1: pyQPanda.GateType, arg2: float, arg3: float, arg4: float) -> None

        Set the noise model for the quantum simulation with multiple noise levels.

        Args:
             noise_model: Type of noise model (bit-flip, phase-flip, etc.).

             gate_type: Type of gate affected by the noise.

             noise_level_1: First noise level to apply.

             noise_level_2: Second noise level to apply.

             noise_level_3: Third noise level to apply.


        4. set_noise_model(self: pyQPanda.MPSQVM, arg0: pyQPanda.NoiseModel, arg1: pyQPanda.GateType, arg2: float, arg3: float, arg4: float, arg5: List[pyQPanda.QVec]) -> None

        Set the noise model for the quantum simulation with specific noise levels and qubits.

        Args:
             noise_model: Type of noise model (bit-flip, phase-flip, etc.).

             gate_type: Type of gate affected by the noise.

             noise_level_1: First noise level to apply.

             noise_level_2: Second noise level to apply.

             noise_level_3: Third noise level to apply.

             qubits: List of qubits to which the noise model will be applied.

        """
    @overload
    def set_noise_model(self, arg0: NoiseModel, arg1: GateType, arg2: float, arg3: list[QVec]) -> None:
        """set_noise_model(*args, **kwargs)
        Overloaded function.

        1. set_noise_model(self: pyQPanda.MPSQVM, arg0: pyQPanda.NoiseModel, arg1: pyQPanda.GateType, arg2: float) -> None

        Set the noise model for the quantum simulation.

        Args:
             noise_model: Type of noise model (bit-flip, phase-flip, etc.).

             gate_type: Type of gate affected by the noise.

             noise_level: Level of noise to apply.


        2. set_noise_model(self: pyQPanda.MPSQVM, arg0: pyQPanda.NoiseModel, arg1: pyQPanda.GateType, arg2: float, arg3: List[pyQPanda.QVec]) -> None

        Set the noise model for the quantum simulation with specific qubits.

        Args:
             noise_model: Type of noise model (bit-flip, phase-flip, etc.).

             gate_type: Type of gate affected by the noise.

             noise_level: Level of noise to apply.

             qubits: List of qubits to which the noise model will be applied.


        3. set_noise_model(self: pyQPanda.MPSQVM, arg0: pyQPanda.NoiseModel, arg1: pyQPanda.GateType, arg2: float, arg3: float, arg4: float) -> None

        Set the noise model for the quantum simulation with multiple noise levels.

        Args:
             noise_model: Type of noise model (bit-flip, phase-flip, etc.).

             gate_type: Type of gate affected by the noise.

             noise_level_1: First noise level to apply.

             noise_level_2: Second noise level to apply.

             noise_level_3: Third noise level to apply.


        4. set_noise_model(self: pyQPanda.MPSQVM, arg0: pyQPanda.NoiseModel, arg1: pyQPanda.GateType, arg2: float, arg3: float, arg4: float, arg5: List[pyQPanda.QVec]) -> None

        Set the noise model for the quantum simulation with specific noise levels and qubits.

        Args:
             noise_model: Type of noise model (bit-flip, phase-flip, etc.).

             gate_type: Type of gate affected by the noise.

             noise_level_1: First noise level to apply.

             noise_level_2: Second noise level to apply.

             noise_level_3: Third noise level to apply.

             qubits: List of qubits to which the noise model will be applied.

        """
    @overload
    def set_noise_model(self, arg0: NoiseModel, arg1: GateType, arg2: float, arg3: float, arg4: float) -> None:
        """set_noise_model(*args, **kwargs)
        Overloaded function.

        1. set_noise_model(self: pyQPanda.MPSQVM, arg0: pyQPanda.NoiseModel, arg1: pyQPanda.GateType, arg2: float) -> None

        Set the noise model for the quantum simulation.

        Args:
             noise_model: Type of noise model (bit-flip, phase-flip, etc.).

             gate_type: Type of gate affected by the noise.

             noise_level: Level of noise to apply.


        2. set_noise_model(self: pyQPanda.MPSQVM, arg0: pyQPanda.NoiseModel, arg1: pyQPanda.GateType, arg2: float, arg3: List[pyQPanda.QVec]) -> None

        Set the noise model for the quantum simulation with specific qubits.

        Args:
             noise_model: Type of noise model (bit-flip, phase-flip, etc.).

             gate_type: Type of gate affected by the noise.

             noise_level: Level of noise to apply.

             qubits: List of qubits to which the noise model will be applied.


        3. set_noise_model(self: pyQPanda.MPSQVM, arg0: pyQPanda.NoiseModel, arg1: pyQPanda.GateType, arg2: float, arg3: float, arg4: float) -> None

        Set the noise model for the quantum simulation with multiple noise levels.

        Args:
             noise_model: Type of noise model (bit-flip, phase-flip, etc.).

             gate_type: Type of gate affected by the noise.

             noise_level_1: First noise level to apply.

             noise_level_2: Second noise level to apply.

             noise_level_3: Third noise level to apply.


        4. set_noise_model(self: pyQPanda.MPSQVM, arg0: pyQPanda.NoiseModel, arg1: pyQPanda.GateType, arg2: float, arg3: float, arg4: float, arg5: List[pyQPanda.QVec]) -> None

        Set the noise model for the quantum simulation with specific noise levels and qubits.

        Args:
             noise_model: Type of noise model (bit-flip, phase-flip, etc.).

             gate_type: Type of gate affected by the noise.

             noise_level_1: First noise level to apply.

             noise_level_2: Second noise level to apply.

             noise_level_3: Third noise level to apply.

             qubits: List of qubits to which the noise model will be applied.

        """
    @overload
    def set_noise_model(self, arg0: NoiseModel, arg1: GateType, arg2: float, arg3: float, arg4: float, arg5: list[QVec]) -> None:
        """set_noise_model(*args, **kwargs)
        Overloaded function.

        1. set_noise_model(self: pyQPanda.MPSQVM, arg0: pyQPanda.NoiseModel, arg1: pyQPanda.GateType, arg2: float) -> None

        Set the noise model for the quantum simulation.

        Args:
             noise_model: Type of noise model (bit-flip, phase-flip, etc.).

             gate_type: Type of gate affected by the noise.

             noise_level: Level of noise to apply.


        2. set_noise_model(self: pyQPanda.MPSQVM, arg0: pyQPanda.NoiseModel, arg1: pyQPanda.GateType, arg2: float, arg3: List[pyQPanda.QVec]) -> None

        Set the noise model for the quantum simulation with specific qubits.

        Args:
             noise_model: Type of noise model (bit-flip, phase-flip, etc.).

             gate_type: Type of gate affected by the noise.

             noise_level: Level of noise to apply.

             qubits: List of qubits to which the noise model will be applied.


        3. set_noise_model(self: pyQPanda.MPSQVM, arg0: pyQPanda.NoiseModel, arg1: pyQPanda.GateType, arg2: float, arg3: float, arg4: float) -> None

        Set the noise model for the quantum simulation with multiple noise levels.

        Args:
             noise_model: Type of noise model (bit-flip, phase-flip, etc.).

             gate_type: Type of gate affected by the noise.

             noise_level_1: First noise level to apply.

             noise_level_2: Second noise level to apply.

             noise_level_3: Third noise level to apply.


        4. set_noise_model(self: pyQPanda.MPSQVM, arg0: pyQPanda.NoiseModel, arg1: pyQPanda.GateType, arg2: float, arg3: float, arg4: float, arg5: List[pyQPanda.QVec]) -> None

        Set the noise model for the quantum simulation with specific noise levels and qubits.

        Args:
             noise_model: Type of noise model (bit-flip, phase-flip, etc.).

             gate_type: Type of gate affected by the noise.

             noise_level_1: First noise level to apply.

             noise_level_2: Second noise level to apply.

             noise_level_3: Third noise level to apply.

             qubits: List of qubits to which the noise model will be applied.

        """
    def set_readout_error(self, readout_params: list[list[float]], qubits: QVec) -> None:
        """set_readout_error(self: pyQPanda.MPSQVM, readout_params: List[List[float]], qubits: pyQPanda.QVec) -> None

        Set readout error parameters for the specified qubits.

        Args:
             readout_params: Parameters defining the readout errors.

             qubits: List of qubits to which the readout errors apply.

        """
    def set_reset_error(self, reset_0_param: float, reset_1_param: float) -> None:
        """set_reset_error(self: pyQPanda.MPSQVM, reset_0_param: float, reset_1_param: float) -> None

        Set the reset error for the quantum state.

        Args:
             reset_0_param: float, error probability for resetting qubit to 0.

             reset_1_param: float, error probability for resetting qubit to 1.

        Returns:
             None.

        """
    def set_rotation_error(self, param: float) -> None:
        """set_rotation_error(self: pyQPanda.MPSQVM, param: float) -> None

        Set the rotation error parameters.

        Args:
             param: The parameters defining the rotation error.

        Returns:
             A reference to the updated instance of the class.

        """

class MomentumOptimizer:
    def __init__(self, arg0: var, arg1: float, arg2: float) -> None:
        """__init__(self: pyQPanda.MomentumOptimizer, arg0: pyQPanda.var, arg1: float, arg2: float) -> None"""
    def get_loss(self) -> float:
        """get_loss(self: pyQPanda.MomentumOptimizer) -> float"""
    def get_variables(self) -> list[var]:
        """get_variables(self: pyQPanda.MomentumOptimizer) -> List[pyQPanda.var]"""
    def minimize(self, arg0: float, arg1: float) -> Optimizer:
        """minimize(self: pyQPanda.var, arg0: float, arg1: float) -> pyQPanda.Optimizer"""
    def run(self, arg0: list[var], arg1: int) -> bool:
        """run(self: pyQPanda.MomentumOptimizer, arg0: List[pyQPanda.var], arg1: int) -> bool"""

class NodeInfo:
    m_cbits: list[int]
    m_control_qubits: QVec
    m_gate_type: GateType
    m_is_dagger: bool
    m_iter: NodeIter
    m_name: str
    m_node_type: NodeType
    m_params: list[float]
    m_target_qubits: QVec
    @overload
    def __init__(self) -> None:
        """__init__(*args, **kwargs)
        Overloaded function.

        1. __init__(self: pyQPanda.NodeInfo) -> None

        2. __init__(self: pyQPanda.NodeInfo, iter: pyQPanda.NodeIter, target_qubits: pyQPanda.QVec, control_qubits: pyQPanda.QVec, type: int, dagger: bool) -> None

        Initialize a quantum operation with the specified parameters.

        Args:
             iter: The node iterator for the quantum operation.

             target_qubits: The target qubits involved in the operation.

             control_qubits: The control qubits for the operation.

             type: An integer representing the type of operation.

             dagger: A boolean indicating whether the operation is a dagger (adjoint) operation.

        Returns:
             None: This function initializes the quantum operation.

        """
    @overload
    def __init__(self, iter: NodeIter, target_qubits: QVec, control_qubits: QVec, type: int, dagger: bool) -> None:
        """__init__(*args, **kwargs)
        Overloaded function.

        1. __init__(self: pyQPanda.NodeInfo) -> None

        2. __init__(self: pyQPanda.NodeInfo, iter: pyQPanda.NodeIter, target_qubits: pyQPanda.QVec, control_qubits: pyQPanda.QVec, type: int, dagger: bool) -> None

        Initialize a quantum operation with the specified parameters.

        Args:
             iter: The node iterator for the quantum operation.

             target_qubits: The target qubits involved in the operation.

             control_qubits: The control qubits for the operation.

             type: An integer representing the type of operation.

             dagger: A boolean indicating whether the operation is a dagger (adjoint) operation.

        Returns:
             None: This function initializes the quantum operation.

        """
    def reset(self) -> None:
        """reset(self: pyQPanda.NodeInfo) -> None

        Reset the NodeInfo instance to its initial state.
        This function clears any current data in the NodeInfo and prepares it for reuse.

        Args:
             None

        Returns:
             None

        """

class NodeIter:
    def __init__(self) -> None:
        """__init__(self: pyQPanda.NodeIter) -> None

        Initialize a new NodeIter instance.

        Args:
             None

        Returns:
             A new instance of NodeIter.

        """
    def get_next(self) -> NodeIter:
        """get_next(self: pyQPanda.NodeIter) -> pyQPanda.NodeIter

        Get the next node iterator.

        Args:
             None

        Returns:
             The next NodeIter instance.

        """
    def get_node_type(self) -> NodeType:
        """get_node_type(self: pyQPanda.NodeIter) -> pyQPanda.NodeType

        Retrieve the type of the current node.

        Args:
             None

        Returns:
             The type of the node.

        """
    def get_pre(self) -> NodeIter:
        """get_pre(self: pyQPanda.NodeIter) -> pyQPanda.NodeIter

        Get the previous node iterator.

        Args:
             None

        Returns:
             The previous NodeIter instance.

        """
    def __eq__(self, arg0: NodeIter) -> bool:
        """__eq__(self: pyQPanda.NodeIter, arg0: pyQPanda.NodeIter) -> bool

        Check equality between two NodeIter instances.

        Args:
             other: Another NodeIter instance.

        Returns:
             True if equal, false otherwise.

        """
    def __ne__(self, arg0: NodeIter) -> bool:
        """__ne__(self: pyQPanda.NodeIter, arg0: pyQPanda.NodeIter) -> bool

        Check inequality between two NodeIter instances.

        Args:
             other: Another NodeIter instance.

        Returns:
             True if not equal, false otherwise.

        """

class NodeType:
    __members__: ClassVar[dict] = ...  # read-only
    CIRCUIT_NODE: ClassVar[NodeType] = ...
    CLASS_COND_NODE: ClassVar[NodeType] = ...
    GATE_NODE: ClassVar[NodeType] = ...
    MEASURE_GATE: ClassVar[NodeType] = ...
    NODE_UNDEFINED: ClassVar[NodeType] = ...
    PROG_NODE: ClassVar[NodeType] = ...
    QIF_START_NODE: ClassVar[NodeType] = ...
    RESET_NODE: ClassVar[NodeType] = ...
    WHILE_START_NODE: ClassVar[NodeType] = ...
    __entries: ClassVar[dict] = ...
    def __init__(self, value: int) -> None:
        """__init__(self: pyQPanda.NodeType, value: int) -> None"""
    def __eq__(self, other: object) -> bool:
        """__eq__(self: object, other: object) -> bool"""
    def __hash__(self) -> int:
        """__hash__(self: object) -> int"""
    def __index__(self) -> int:
        """__index__(self: pyQPanda.NodeType) -> int"""
    def __int__(self) -> int:
        """__int__(self: pyQPanda.NodeType) -> int"""
    def __ne__(self, other: object) -> bool:
        """__ne__(self: object, other: object) -> bool"""
    @property
    def name(self) -> str: ...
    @property
    def value(self) -> int: ...

class Noise:
    def __init__(self) -> None:
        """__init__(self: pyQPanda.Noise) -> None

        Initialize a new NoiseModel instance.

        Args:
             None

        Returns:
             A new NoiseModel instance.

        """
    @overload
    def add_mixed_unitary_error(self, gate_types: GateType, unitary_matrices: list[list[complex]], probs: list[float]) -> None:
        """add_mixed_unitary_error(*args, **kwargs)
        Overloaded function.

        1. add_mixed_unitary_error(self: pyQPanda.Noise, gate_types: pyQPanda.GateType, unitary_matrices: List[List[complex]], probs: List[float]) -> None

        Add mixed unitary errors to specified gate types.

        Args:
             gate_types: The type of gates to which the mixed unitary errors apply.

             unitary_matrices: A vector of unitary matrices representing the errors.

             probs: A vector of probabilities corresponding to each unitary matrix.

        Returns:
             None.


        2. add_mixed_unitary_error(self: pyQPanda.Noise, gate_types: pyQPanda.GateType, unitary_matrices: List[List[complex]], probs: List[float], qubits: pyQPanda.QVec) -> None

        Add mixed unitary errors to specified gate types with targeted qubits.

        Args:
             gate_types: The type of gates to which the mixed unitary errors apply.

             unitary_matrices: A vector of unitary matrices representing the errors.

             probs: A vector of probabilities corresponding to each unitary matrix.

             qubits: A vector of qubit indices that the mixed unitary errors affect.

        Returns:
             None.


        3. add_mixed_unitary_error(self: pyQPanda.Noise, gate_types: pyQPanda.GateType, unitary_matrices: List[List[complex]], probs: List[float], qubits: List[pyQPanda.QVec]) -> None

        Add mixed unitary errors to specified gate types for multiple qubits.

        Args:
             gate_types: The type of gates to which the mixed unitary errors apply.

             unitary_matrices: A vector of unitary matrices representing the errors.

             probs: A vector of probabilities corresponding to each unitary matrix.

             qubits: A vector of QVec instances indicating which qubits the errors affect.

        Returns:
             None.

        """
    @overload
    def add_mixed_unitary_error(self, gate_types: GateType, unitary_matrices: list[list[complex]], probs: list[float], qubits: QVec) -> None:
        """add_mixed_unitary_error(*args, **kwargs)
        Overloaded function.

        1. add_mixed_unitary_error(self: pyQPanda.Noise, gate_types: pyQPanda.GateType, unitary_matrices: List[List[complex]], probs: List[float]) -> None

        Add mixed unitary errors to specified gate types.

        Args:
             gate_types: The type of gates to which the mixed unitary errors apply.

             unitary_matrices: A vector of unitary matrices representing the errors.

             probs: A vector of probabilities corresponding to each unitary matrix.

        Returns:
             None.


        2. add_mixed_unitary_error(self: pyQPanda.Noise, gate_types: pyQPanda.GateType, unitary_matrices: List[List[complex]], probs: List[float], qubits: pyQPanda.QVec) -> None

        Add mixed unitary errors to specified gate types with targeted qubits.

        Args:
             gate_types: The type of gates to which the mixed unitary errors apply.

             unitary_matrices: A vector of unitary matrices representing the errors.

             probs: A vector of probabilities corresponding to each unitary matrix.

             qubits: A vector of qubit indices that the mixed unitary errors affect.

        Returns:
             None.


        3. add_mixed_unitary_error(self: pyQPanda.Noise, gate_types: pyQPanda.GateType, unitary_matrices: List[List[complex]], probs: List[float], qubits: List[pyQPanda.QVec]) -> None

        Add mixed unitary errors to specified gate types for multiple qubits.

        Args:
             gate_types: The type of gates to which the mixed unitary errors apply.

             unitary_matrices: A vector of unitary matrices representing the errors.

             probs: A vector of probabilities corresponding to each unitary matrix.

             qubits: A vector of QVec instances indicating which qubits the errors affect.

        Returns:
             None.

        """
    @overload
    def add_mixed_unitary_error(self, gate_types: GateType, unitary_matrices: list[list[complex]], probs: list[float], qubits: list[QVec]) -> None:
        """add_mixed_unitary_error(*args, **kwargs)
        Overloaded function.

        1. add_mixed_unitary_error(self: pyQPanda.Noise, gate_types: pyQPanda.GateType, unitary_matrices: List[List[complex]], probs: List[float]) -> None

        Add mixed unitary errors to specified gate types.

        Args:
             gate_types: The type of gates to which the mixed unitary errors apply.

             unitary_matrices: A vector of unitary matrices representing the errors.

             probs: A vector of probabilities corresponding to each unitary matrix.

        Returns:
             None.


        2. add_mixed_unitary_error(self: pyQPanda.Noise, gate_types: pyQPanda.GateType, unitary_matrices: List[List[complex]], probs: List[float], qubits: pyQPanda.QVec) -> None

        Add mixed unitary errors to specified gate types with targeted qubits.

        Args:
             gate_types: The type of gates to which the mixed unitary errors apply.

             unitary_matrices: A vector of unitary matrices representing the errors.

             probs: A vector of probabilities corresponding to each unitary matrix.

             qubits: A vector of qubit indices that the mixed unitary errors affect.

        Returns:
             None.


        3. add_mixed_unitary_error(self: pyQPanda.Noise, gate_types: pyQPanda.GateType, unitary_matrices: List[List[complex]], probs: List[float], qubits: List[pyQPanda.QVec]) -> None

        Add mixed unitary errors to specified gate types for multiple qubits.

        Args:
             gate_types: The type of gates to which the mixed unitary errors apply.

             unitary_matrices: A vector of unitary matrices representing the errors.

             probs: A vector of probabilities corresponding to each unitary matrix.

             qubits: A vector of QVec instances indicating which qubits the errors affect.

        Returns:
             None.

        """
    @overload
    def add_noise_model(self, noise_model: NoiseModel, gate_type: GateType, prob: float) -> None:
        """add_noise_model(*args, **kwargs)
        Overloaded function.

        1. add_noise_model(self: pyQPanda.Noise, noise_model: pyQPanda.NoiseModel, gate_type: pyQPanda.GateType, prob: float) -> None

        Add a noise model to the noise simulation.

        Args:
             noise_model: An instance of NOISE_MODEL to be added.

             gate_type: The type of gate to which the noise model applies.

             prob: The probability of the noise occurring.

        Returns:
             None.


        2. add_noise_model(self: pyQPanda.Noise, noise_model: pyQPanda.NoiseModel, gate_types: List[pyQPanda.GateType], prob: float) -> None

        Add a noise model to multiple gate types.

        Args:
             noise_model: An instance of NOISE_MODEL to be added.

             gate_types: A vector of gate types to which the noise model applies.

             prob: The probability of the noise occurring.

        Returns:
             None.


        3. add_noise_model(self: pyQPanda.Noise, noise_model: pyQPanda.NoiseModel, gate_type: pyQPanda.GateType, prob: float, qubits: pyQPanda.QVec) -> None

        Add a noise model to a specific gate with targeted qubits.

        Args:
             noise_model: An instance of NOISE_MODEL to be added.

             gate_type: The type of gate to which the noise model applies.

             prob: The probability of the noise occurring.

             qubits: A vector of qubit indices that the noise affects.

        Returns:
             None.


        4. add_noise_model(self: pyQPanda.Noise, noise_model: pyQPanda.NoiseModel, gate_types: List[pyQPanda.GateType], prob: float, qubits: pyQPanda.QVec) -> None

        Add a noise model to multiple gate types with targeted qubits.

        Args:
             noise_model: An instance of NOISE_MODEL to be added.

             gate_types: A vector of gate types to which the noise model applies.

             prob: The probability of the noise occurring.

             qubits: A vector of qubit indices that the noise affects.

        Returns:
             None.


        5. add_noise_model(self: pyQPanda.Noise, noise_model: pyQPanda.NoiseModel, gate_type: pyQPanda.GateType, prob: float, qubits: List[pyQPanda.QVec]) -> None

        Add a noise model to a specific gate with targeted qubits.

        Args:
             noise_model: An instance of NOISE_MODEL to be added.

             gate_type: The type of gate to which the noise model applies.

             prob: The probability of the noise occurring.

             qubits: A vector of qubit indices that the noise affects.

        Returns:
             None.


        6. add_noise_model(self: pyQPanda.Noise, noise_model: pyQPanda.NoiseModel, gate_type: pyQPanda.GateType, t1: float, t2: float, t_gate: float) -> None

        Add a noise model to a specific gate with time parameters.

        Args:
             noise_model: An instance of NOISE_MODEL to be added.

             gate_type: The type of gate to which the noise model applies.

             t1: The time constant for relaxation (T1).

             t2: The time constant for dephasing (T2).

             t_gate: The duration of the gate operation.

        Returns:
             None.


        7. add_noise_model(self: pyQPanda.Noise, noise_model: pyQPanda.NoiseModel, gate_types: List[pyQPanda.GateType], t1: float, t2: float, t_gate: float) -> None

        Add a noise model to multiple gate types with time parameters.

        Args:
             noise_model: An instance of NOISE_MODEL to be added.

             gate_types: A vector of gate types to which the noise model applies.

             t1: The time constant for relaxation (T1).

             t2: The time constant for dephasing (T2).

             t_gate: The duration of the gate operation.

        Returns:
             None.


        8. add_noise_model(self: pyQPanda.Noise, noise_model: pyQPanda.NoiseModel, gate_type: pyQPanda.GateType, t1: float, t2: float, t_gate: float, qubits: pyQPanda.QVec) -> None

        Add a noise model to a specific gate with time parameters and targeted qubits.

        Args:
             noise_model: An instance of NOISE_MODEL to be added.

             gate_type: The type of gate to which the noise model applies.

             t1: The time constant for relaxation (T1).

             t2: The time constant for dephasing (T2).

             t_gate: The duration of the gate operation.

             qubits: A vector of qubit indices that the noise affects.

        Returns:
             None.


        9. add_noise_model(self: pyQPanda.Noise, noise_model: pyQPanda.NoiseModel, gate_types: List[pyQPanda.GateType], t1: float, t2: float, t_gate: float, qubits: pyQPanda.QVec) -> None

        Add a noise model to multiple gate types with specified time parameters and targeted qubits.

        Args:
             noise_model: An instance of NOISE_MODEL to be added.

             gate_types: A vector of gate types to which the noise model applies.

             t1: The time constant for relaxation (T1).

             t2: The time constant for dephasing (T2).

             t_gate: The duration of the gate operation.

             qubits: A vector of qubit indices that the noise affects.

        Returns:
             None.


        10. add_noise_model(self: pyQPanda.Noise, noise_model: pyQPanda.NoiseModel, gate_type: pyQPanda.GateType, t1: float, t2: float, t_gate: float, qubits: List[pyQPanda.QVec]) -> None

        Add a noise model to a specific gate with specified time parameters and targeted qubits.

        Args:
             noise_model: An instance of NOISE_MODEL to be added.

             gate_type: The type of gate to which the noise model applies.

             t1: The time constant for relaxation (T1).

             t2: The time constant for dephasing (T2).

             t_gate: The duration of the gate operation.

             qubits: A vector of vectors of qubit indices that the noise affects.

        Returns:
             None.

        """
    @overload
    def add_noise_model(self, noise_model: NoiseModel, gate_types: list[GateType], prob: float) -> None:
        """add_noise_model(*args, **kwargs)
        Overloaded function.

        1. add_noise_model(self: pyQPanda.Noise, noise_model: pyQPanda.NoiseModel, gate_type: pyQPanda.GateType, prob: float) -> None

        Add a noise model to the noise simulation.

        Args:
             noise_model: An instance of NOISE_MODEL to be added.

             gate_type: The type of gate to which the noise model applies.

             prob: The probability of the noise occurring.

        Returns:
             None.


        2. add_noise_model(self: pyQPanda.Noise, noise_model: pyQPanda.NoiseModel, gate_types: List[pyQPanda.GateType], prob: float) -> None

        Add a noise model to multiple gate types.

        Args:
             noise_model: An instance of NOISE_MODEL to be added.

             gate_types: A vector of gate types to which the noise model applies.

             prob: The probability of the noise occurring.

        Returns:
             None.


        3. add_noise_model(self: pyQPanda.Noise, noise_model: pyQPanda.NoiseModel, gate_type: pyQPanda.GateType, prob: float, qubits: pyQPanda.QVec) -> None

        Add a noise model to a specific gate with targeted qubits.

        Args:
             noise_model: An instance of NOISE_MODEL to be added.

             gate_type: The type of gate to which the noise model applies.

             prob: The probability of the noise occurring.

             qubits: A vector of qubit indices that the noise affects.

        Returns:
             None.


        4. add_noise_model(self: pyQPanda.Noise, noise_model: pyQPanda.NoiseModel, gate_types: List[pyQPanda.GateType], prob: float, qubits: pyQPanda.QVec) -> None

        Add a noise model to multiple gate types with targeted qubits.

        Args:
             noise_model: An instance of NOISE_MODEL to be added.

             gate_types: A vector of gate types to which the noise model applies.

             prob: The probability of the noise occurring.

             qubits: A vector of qubit indices that the noise affects.

        Returns:
             None.


        5. add_noise_model(self: pyQPanda.Noise, noise_model: pyQPanda.NoiseModel, gate_type: pyQPanda.GateType, prob: float, qubits: List[pyQPanda.QVec]) -> None

        Add a noise model to a specific gate with targeted qubits.

        Args:
             noise_model: An instance of NOISE_MODEL to be added.

             gate_type: The type of gate to which the noise model applies.

             prob: The probability of the noise occurring.

             qubits: A vector of qubit indices that the noise affects.

        Returns:
             None.


        6. add_noise_model(self: pyQPanda.Noise, noise_model: pyQPanda.NoiseModel, gate_type: pyQPanda.GateType, t1: float, t2: float, t_gate: float) -> None

        Add a noise model to a specific gate with time parameters.

        Args:
             noise_model: An instance of NOISE_MODEL to be added.

             gate_type: The type of gate to which the noise model applies.

             t1: The time constant for relaxation (T1).

             t2: The time constant for dephasing (T2).

             t_gate: The duration of the gate operation.

        Returns:
             None.


        7. add_noise_model(self: pyQPanda.Noise, noise_model: pyQPanda.NoiseModel, gate_types: List[pyQPanda.GateType], t1: float, t2: float, t_gate: float) -> None

        Add a noise model to multiple gate types with time parameters.

        Args:
             noise_model: An instance of NOISE_MODEL to be added.

             gate_types: A vector of gate types to which the noise model applies.

             t1: The time constant for relaxation (T1).

             t2: The time constant for dephasing (T2).

             t_gate: The duration of the gate operation.

        Returns:
             None.


        8. add_noise_model(self: pyQPanda.Noise, noise_model: pyQPanda.NoiseModel, gate_type: pyQPanda.GateType, t1: float, t2: float, t_gate: float, qubits: pyQPanda.QVec) -> None

        Add a noise model to a specific gate with time parameters and targeted qubits.

        Args:
             noise_model: An instance of NOISE_MODEL to be added.

             gate_type: The type of gate to which the noise model applies.

             t1: The time constant for relaxation (T1).

             t2: The time constant for dephasing (T2).

             t_gate: The duration of the gate operation.

             qubits: A vector of qubit indices that the noise affects.

        Returns:
             None.


        9. add_noise_model(self: pyQPanda.Noise, noise_model: pyQPanda.NoiseModel, gate_types: List[pyQPanda.GateType], t1: float, t2: float, t_gate: float, qubits: pyQPanda.QVec) -> None

        Add a noise model to multiple gate types with specified time parameters and targeted qubits.

        Args:
             noise_model: An instance of NOISE_MODEL to be added.

             gate_types: A vector of gate types to which the noise model applies.

             t1: The time constant for relaxation (T1).

             t2: The time constant for dephasing (T2).

             t_gate: The duration of the gate operation.

             qubits: A vector of qubit indices that the noise affects.

        Returns:
             None.


        10. add_noise_model(self: pyQPanda.Noise, noise_model: pyQPanda.NoiseModel, gate_type: pyQPanda.GateType, t1: float, t2: float, t_gate: float, qubits: List[pyQPanda.QVec]) -> None

        Add a noise model to a specific gate with specified time parameters and targeted qubits.

        Args:
             noise_model: An instance of NOISE_MODEL to be added.

             gate_type: The type of gate to which the noise model applies.

             t1: The time constant for relaxation (T1).

             t2: The time constant for dephasing (T2).

             t_gate: The duration of the gate operation.

             qubits: A vector of vectors of qubit indices that the noise affects.

        Returns:
             None.

        """
    @overload
    def add_noise_model(self, noise_model: NoiseModel, gate_type: GateType, prob: float, qubits: QVec) -> None:
        """add_noise_model(*args, **kwargs)
        Overloaded function.

        1. add_noise_model(self: pyQPanda.Noise, noise_model: pyQPanda.NoiseModel, gate_type: pyQPanda.GateType, prob: float) -> None

        Add a noise model to the noise simulation.

        Args:
             noise_model: An instance of NOISE_MODEL to be added.

             gate_type: The type of gate to which the noise model applies.

             prob: The probability of the noise occurring.

        Returns:
             None.


        2. add_noise_model(self: pyQPanda.Noise, noise_model: pyQPanda.NoiseModel, gate_types: List[pyQPanda.GateType], prob: float) -> None

        Add a noise model to multiple gate types.

        Args:
             noise_model: An instance of NOISE_MODEL to be added.

             gate_types: A vector of gate types to which the noise model applies.

             prob: The probability of the noise occurring.

        Returns:
             None.


        3. add_noise_model(self: pyQPanda.Noise, noise_model: pyQPanda.NoiseModel, gate_type: pyQPanda.GateType, prob: float, qubits: pyQPanda.QVec) -> None

        Add a noise model to a specific gate with targeted qubits.

        Args:
             noise_model: An instance of NOISE_MODEL to be added.

             gate_type: The type of gate to which the noise model applies.

             prob: The probability of the noise occurring.

             qubits: A vector of qubit indices that the noise affects.

        Returns:
             None.


        4. add_noise_model(self: pyQPanda.Noise, noise_model: pyQPanda.NoiseModel, gate_types: List[pyQPanda.GateType], prob: float, qubits: pyQPanda.QVec) -> None

        Add a noise model to multiple gate types with targeted qubits.

        Args:
             noise_model: An instance of NOISE_MODEL to be added.

             gate_types: A vector of gate types to which the noise model applies.

             prob: The probability of the noise occurring.

             qubits: A vector of qubit indices that the noise affects.

        Returns:
             None.


        5. add_noise_model(self: pyQPanda.Noise, noise_model: pyQPanda.NoiseModel, gate_type: pyQPanda.GateType, prob: float, qubits: List[pyQPanda.QVec]) -> None

        Add a noise model to a specific gate with targeted qubits.

        Args:
             noise_model: An instance of NOISE_MODEL to be added.

             gate_type: The type of gate to which the noise model applies.

             prob: The probability of the noise occurring.

             qubits: A vector of qubit indices that the noise affects.

        Returns:
             None.


        6. add_noise_model(self: pyQPanda.Noise, noise_model: pyQPanda.NoiseModel, gate_type: pyQPanda.GateType, t1: float, t2: float, t_gate: float) -> None

        Add a noise model to a specific gate with time parameters.

        Args:
             noise_model: An instance of NOISE_MODEL to be added.

             gate_type: The type of gate to which the noise model applies.

             t1: The time constant for relaxation (T1).

             t2: The time constant for dephasing (T2).

             t_gate: The duration of the gate operation.

        Returns:
             None.


        7. add_noise_model(self: pyQPanda.Noise, noise_model: pyQPanda.NoiseModel, gate_types: List[pyQPanda.GateType], t1: float, t2: float, t_gate: float) -> None

        Add a noise model to multiple gate types with time parameters.

        Args:
             noise_model: An instance of NOISE_MODEL to be added.

             gate_types: A vector of gate types to which the noise model applies.

             t1: The time constant for relaxation (T1).

             t2: The time constant for dephasing (T2).

             t_gate: The duration of the gate operation.

        Returns:
             None.


        8. add_noise_model(self: pyQPanda.Noise, noise_model: pyQPanda.NoiseModel, gate_type: pyQPanda.GateType, t1: float, t2: float, t_gate: float, qubits: pyQPanda.QVec) -> None

        Add a noise model to a specific gate with time parameters and targeted qubits.

        Args:
             noise_model: An instance of NOISE_MODEL to be added.

             gate_type: The type of gate to which the noise model applies.

             t1: The time constant for relaxation (T1).

             t2: The time constant for dephasing (T2).

             t_gate: The duration of the gate operation.

             qubits: A vector of qubit indices that the noise affects.

        Returns:
             None.


        9. add_noise_model(self: pyQPanda.Noise, noise_model: pyQPanda.NoiseModel, gate_types: List[pyQPanda.GateType], t1: float, t2: float, t_gate: float, qubits: pyQPanda.QVec) -> None

        Add a noise model to multiple gate types with specified time parameters and targeted qubits.

        Args:
             noise_model: An instance of NOISE_MODEL to be added.

             gate_types: A vector of gate types to which the noise model applies.

             t1: The time constant for relaxation (T1).

             t2: The time constant for dephasing (T2).

             t_gate: The duration of the gate operation.

             qubits: A vector of qubit indices that the noise affects.

        Returns:
             None.


        10. add_noise_model(self: pyQPanda.Noise, noise_model: pyQPanda.NoiseModel, gate_type: pyQPanda.GateType, t1: float, t2: float, t_gate: float, qubits: List[pyQPanda.QVec]) -> None

        Add a noise model to a specific gate with specified time parameters and targeted qubits.

        Args:
             noise_model: An instance of NOISE_MODEL to be added.

             gate_type: The type of gate to which the noise model applies.

             t1: The time constant for relaxation (T1).

             t2: The time constant for dephasing (T2).

             t_gate: The duration of the gate operation.

             qubits: A vector of vectors of qubit indices that the noise affects.

        Returns:
             None.

        """
    @overload
    def add_noise_model(self, noise_model: NoiseModel, gate_types: list[GateType], prob: float, qubits: QVec) -> None:
        """add_noise_model(*args, **kwargs)
        Overloaded function.

        1. add_noise_model(self: pyQPanda.Noise, noise_model: pyQPanda.NoiseModel, gate_type: pyQPanda.GateType, prob: float) -> None

        Add a noise model to the noise simulation.

        Args:
             noise_model: An instance of NOISE_MODEL to be added.

             gate_type: The type of gate to which the noise model applies.

             prob: The probability of the noise occurring.

        Returns:
             None.


        2. add_noise_model(self: pyQPanda.Noise, noise_model: pyQPanda.NoiseModel, gate_types: List[pyQPanda.GateType], prob: float) -> None

        Add a noise model to multiple gate types.

        Args:
             noise_model: An instance of NOISE_MODEL to be added.

             gate_types: A vector of gate types to which the noise model applies.

             prob: The probability of the noise occurring.

        Returns:
             None.


        3. add_noise_model(self: pyQPanda.Noise, noise_model: pyQPanda.NoiseModel, gate_type: pyQPanda.GateType, prob: float, qubits: pyQPanda.QVec) -> None

        Add a noise model to a specific gate with targeted qubits.

        Args:
             noise_model: An instance of NOISE_MODEL to be added.

             gate_type: The type of gate to which the noise model applies.

             prob: The probability of the noise occurring.

             qubits: A vector of qubit indices that the noise affects.

        Returns:
             None.


        4. add_noise_model(self: pyQPanda.Noise, noise_model: pyQPanda.NoiseModel, gate_types: List[pyQPanda.GateType], prob: float, qubits: pyQPanda.QVec) -> None

        Add a noise model to multiple gate types with targeted qubits.

        Args:
             noise_model: An instance of NOISE_MODEL to be added.

             gate_types: A vector of gate types to which the noise model applies.

             prob: The probability of the noise occurring.

             qubits: A vector of qubit indices that the noise affects.

        Returns:
             None.


        5. add_noise_model(self: pyQPanda.Noise, noise_model: pyQPanda.NoiseModel, gate_type: pyQPanda.GateType, prob: float, qubits: List[pyQPanda.QVec]) -> None

        Add a noise model to a specific gate with targeted qubits.

        Args:
             noise_model: An instance of NOISE_MODEL to be added.

             gate_type: The type of gate to which the noise model applies.

             prob: The probability of the noise occurring.

             qubits: A vector of qubit indices that the noise affects.

        Returns:
             None.


        6. add_noise_model(self: pyQPanda.Noise, noise_model: pyQPanda.NoiseModel, gate_type: pyQPanda.GateType, t1: float, t2: float, t_gate: float) -> None

        Add a noise model to a specific gate with time parameters.

        Args:
             noise_model: An instance of NOISE_MODEL to be added.

             gate_type: The type of gate to which the noise model applies.

             t1: The time constant for relaxation (T1).

             t2: The time constant for dephasing (T2).

             t_gate: The duration of the gate operation.

        Returns:
             None.


        7. add_noise_model(self: pyQPanda.Noise, noise_model: pyQPanda.NoiseModel, gate_types: List[pyQPanda.GateType], t1: float, t2: float, t_gate: float) -> None

        Add a noise model to multiple gate types with time parameters.

        Args:
             noise_model: An instance of NOISE_MODEL to be added.

             gate_types: A vector of gate types to which the noise model applies.

             t1: The time constant for relaxation (T1).

             t2: The time constant for dephasing (T2).

             t_gate: The duration of the gate operation.

        Returns:
             None.


        8. add_noise_model(self: pyQPanda.Noise, noise_model: pyQPanda.NoiseModel, gate_type: pyQPanda.GateType, t1: float, t2: float, t_gate: float, qubits: pyQPanda.QVec) -> None

        Add a noise model to a specific gate with time parameters and targeted qubits.

        Args:
             noise_model: An instance of NOISE_MODEL to be added.

             gate_type: The type of gate to which the noise model applies.

             t1: The time constant for relaxation (T1).

             t2: The time constant for dephasing (T2).

             t_gate: The duration of the gate operation.

             qubits: A vector of qubit indices that the noise affects.

        Returns:
             None.


        9. add_noise_model(self: pyQPanda.Noise, noise_model: pyQPanda.NoiseModel, gate_types: List[pyQPanda.GateType], t1: float, t2: float, t_gate: float, qubits: pyQPanda.QVec) -> None

        Add a noise model to multiple gate types with specified time parameters and targeted qubits.

        Args:
             noise_model: An instance of NOISE_MODEL to be added.

             gate_types: A vector of gate types to which the noise model applies.

             t1: The time constant for relaxation (T1).

             t2: The time constant for dephasing (T2).

             t_gate: The duration of the gate operation.

             qubits: A vector of qubit indices that the noise affects.

        Returns:
             None.


        10. add_noise_model(self: pyQPanda.Noise, noise_model: pyQPanda.NoiseModel, gate_type: pyQPanda.GateType, t1: float, t2: float, t_gate: float, qubits: List[pyQPanda.QVec]) -> None

        Add a noise model to a specific gate with specified time parameters and targeted qubits.

        Args:
             noise_model: An instance of NOISE_MODEL to be added.

             gate_type: The type of gate to which the noise model applies.

             t1: The time constant for relaxation (T1).

             t2: The time constant for dephasing (T2).

             t_gate: The duration of the gate operation.

             qubits: A vector of vectors of qubit indices that the noise affects.

        Returns:
             None.

        """
    @overload
    def add_noise_model(self, noise_model: NoiseModel, gate_type: GateType, prob: float, qubits: list[QVec]) -> None:
        """add_noise_model(*args, **kwargs)
        Overloaded function.

        1. add_noise_model(self: pyQPanda.Noise, noise_model: pyQPanda.NoiseModel, gate_type: pyQPanda.GateType, prob: float) -> None

        Add a noise model to the noise simulation.

        Args:
             noise_model: An instance of NOISE_MODEL to be added.

             gate_type: The type of gate to which the noise model applies.

             prob: The probability of the noise occurring.

        Returns:
             None.


        2. add_noise_model(self: pyQPanda.Noise, noise_model: pyQPanda.NoiseModel, gate_types: List[pyQPanda.GateType], prob: float) -> None

        Add a noise model to multiple gate types.

        Args:
             noise_model: An instance of NOISE_MODEL to be added.

             gate_types: A vector of gate types to which the noise model applies.

             prob: The probability of the noise occurring.

        Returns:
             None.


        3. add_noise_model(self: pyQPanda.Noise, noise_model: pyQPanda.NoiseModel, gate_type: pyQPanda.GateType, prob: float, qubits: pyQPanda.QVec) -> None

        Add a noise model to a specific gate with targeted qubits.

        Args:
             noise_model: An instance of NOISE_MODEL to be added.

             gate_type: The type of gate to which the noise model applies.

             prob: The probability of the noise occurring.

             qubits: A vector of qubit indices that the noise affects.

        Returns:
             None.


        4. add_noise_model(self: pyQPanda.Noise, noise_model: pyQPanda.NoiseModel, gate_types: List[pyQPanda.GateType], prob: float, qubits: pyQPanda.QVec) -> None

        Add a noise model to multiple gate types with targeted qubits.

        Args:
             noise_model: An instance of NOISE_MODEL to be added.

             gate_types: A vector of gate types to which the noise model applies.

             prob: The probability of the noise occurring.

             qubits: A vector of qubit indices that the noise affects.

        Returns:
             None.


        5. add_noise_model(self: pyQPanda.Noise, noise_model: pyQPanda.NoiseModel, gate_type: pyQPanda.GateType, prob: float, qubits: List[pyQPanda.QVec]) -> None

        Add a noise model to a specific gate with targeted qubits.

        Args:
             noise_model: An instance of NOISE_MODEL to be added.

             gate_type: The type of gate to which the noise model applies.

             prob: The probability of the noise occurring.

             qubits: A vector of qubit indices that the noise affects.

        Returns:
             None.


        6. add_noise_model(self: pyQPanda.Noise, noise_model: pyQPanda.NoiseModel, gate_type: pyQPanda.GateType, t1: float, t2: float, t_gate: float) -> None

        Add a noise model to a specific gate with time parameters.

        Args:
             noise_model: An instance of NOISE_MODEL to be added.

             gate_type: The type of gate to which the noise model applies.

             t1: The time constant for relaxation (T1).

             t2: The time constant for dephasing (T2).

             t_gate: The duration of the gate operation.

        Returns:
             None.


        7. add_noise_model(self: pyQPanda.Noise, noise_model: pyQPanda.NoiseModel, gate_types: List[pyQPanda.GateType], t1: float, t2: float, t_gate: float) -> None

        Add a noise model to multiple gate types with time parameters.

        Args:
             noise_model: An instance of NOISE_MODEL to be added.

             gate_types: A vector of gate types to which the noise model applies.

             t1: The time constant for relaxation (T1).

             t2: The time constant for dephasing (T2).

             t_gate: The duration of the gate operation.

        Returns:
             None.


        8. add_noise_model(self: pyQPanda.Noise, noise_model: pyQPanda.NoiseModel, gate_type: pyQPanda.GateType, t1: float, t2: float, t_gate: float, qubits: pyQPanda.QVec) -> None

        Add a noise model to a specific gate with time parameters and targeted qubits.

        Args:
             noise_model: An instance of NOISE_MODEL to be added.

             gate_type: The type of gate to which the noise model applies.

             t1: The time constant for relaxation (T1).

             t2: The time constant for dephasing (T2).

             t_gate: The duration of the gate operation.

             qubits: A vector of qubit indices that the noise affects.

        Returns:
             None.


        9. add_noise_model(self: pyQPanda.Noise, noise_model: pyQPanda.NoiseModel, gate_types: List[pyQPanda.GateType], t1: float, t2: float, t_gate: float, qubits: pyQPanda.QVec) -> None

        Add a noise model to multiple gate types with specified time parameters and targeted qubits.

        Args:
             noise_model: An instance of NOISE_MODEL to be added.

             gate_types: A vector of gate types to which the noise model applies.

             t1: The time constant for relaxation (T1).

             t2: The time constant for dephasing (T2).

             t_gate: The duration of the gate operation.

             qubits: A vector of qubit indices that the noise affects.

        Returns:
             None.


        10. add_noise_model(self: pyQPanda.Noise, noise_model: pyQPanda.NoiseModel, gate_type: pyQPanda.GateType, t1: float, t2: float, t_gate: float, qubits: List[pyQPanda.QVec]) -> None

        Add a noise model to a specific gate with specified time parameters and targeted qubits.

        Args:
             noise_model: An instance of NOISE_MODEL to be added.

             gate_type: The type of gate to which the noise model applies.

             t1: The time constant for relaxation (T1).

             t2: The time constant for dephasing (T2).

             t_gate: The duration of the gate operation.

             qubits: A vector of vectors of qubit indices that the noise affects.

        Returns:
             None.

        """
    @overload
    def add_noise_model(self, noise_model: NoiseModel, gate_type: GateType, t1: float, t2: float, t_gate: float) -> None:
        """add_noise_model(*args, **kwargs)
        Overloaded function.

        1. add_noise_model(self: pyQPanda.Noise, noise_model: pyQPanda.NoiseModel, gate_type: pyQPanda.GateType, prob: float) -> None

        Add a noise model to the noise simulation.

        Args:
             noise_model: An instance of NOISE_MODEL to be added.

             gate_type: The type of gate to which the noise model applies.

             prob: The probability of the noise occurring.

        Returns:
             None.


        2. add_noise_model(self: pyQPanda.Noise, noise_model: pyQPanda.NoiseModel, gate_types: List[pyQPanda.GateType], prob: float) -> None

        Add a noise model to multiple gate types.

        Args:
             noise_model: An instance of NOISE_MODEL to be added.

             gate_types: A vector of gate types to which the noise model applies.

             prob: The probability of the noise occurring.

        Returns:
             None.


        3. add_noise_model(self: pyQPanda.Noise, noise_model: pyQPanda.NoiseModel, gate_type: pyQPanda.GateType, prob: float, qubits: pyQPanda.QVec) -> None

        Add a noise model to a specific gate with targeted qubits.

        Args:
             noise_model: An instance of NOISE_MODEL to be added.

             gate_type: The type of gate to which the noise model applies.

             prob: The probability of the noise occurring.

             qubits: A vector of qubit indices that the noise affects.

        Returns:
             None.


        4. add_noise_model(self: pyQPanda.Noise, noise_model: pyQPanda.NoiseModel, gate_types: List[pyQPanda.GateType], prob: float, qubits: pyQPanda.QVec) -> None

        Add a noise model to multiple gate types with targeted qubits.

        Args:
             noise_model: An instance of NOISE_MODEL to be added.

             gate_types: A vector of gate types to which the noise model applies.

             prob: The probability of the noise occurring.

             qubits: A vector of qubit indices that the noise affects.

        Returns:
             None.


        5. add_noise_model(self: pyQPanda.Noise, noise_model: pyQPanda.NoiseModel, gate_type: pyQPanda.GateType, prob: float, qubits: List[pyQPanda.QVec]) -> None

        Add a noise model to a specific gate with targeted qubits.

        Args:
             noise_model: An instance of NOISE_MODEL to be added.

             gate_type: The type of gate to which the noise model applies.

             prob: The probability of the noise occurring.

             qubits: A vector of qubit indices that the noise affects.

        Returns:
             None.


        6. add_noise_model(self: pyQPanda.Noise, noise_model: pyQPanda.NoiseModel, gate_type: pyQPanda.GateType, t1: float, t2: float, t_gate: float) -> None

        Add a noise model to a specific gate with time parameters.

        Args:
             noise_model: An instance of NOISE_MODEL to be added.

             gate_type: The type of gate to which the noise model applies.

             t1: The time constant for relaxation (T1).

             t2: The time constant for dephasing (T2).

             t_gate: The duration of the gate operation.

        Returns:
             None.


        7. add_noise_model(self: pyQPanda.Noise, noise_model: pyQPanda.NoiseModel, gate_types: List[pyQPanda.GateType], t1: float, t2: float, t_gate: float) -> None

        Add a noise model to multiple gate types with time parameters.

        Args:
             noise_model: An instance of NOISE_MODEL to be added.

             gate_types: A vector of gate types to which the noise model applies.

             t1: The time constant for relaxation (T1).

             t2: The time constant for dephasing (T2).

             t_gate: The duration of the gate operation.

        Returns:
             None.


        8. add_noise_model(self: pyQPanda.Noise, noise_model: pyQPanda.NoiseModel, gate_type: pyQPanda.GateType, t1: float, t2: float, t_gate: float, qubits: pyQPanda.QVec) -> None

        Add a noise model to a specific gate with time parameters and targeted qubits.

        Args:
             noise_model: An instance of NOISE_MODEL to be added.

             gate_type: The type of gate to which the noise model applies.

             t1: The time constant for relaxation (T1).

             t2: The time constant for dephasing (T2).

             t_gate: The duration of the gate operation.

             qubits: A vector of qubit indices that the noise affects.

        Returns:
             None.


        9. add_noise_model(self: pyQPanda.Noise, noise_model: pyQPanda.NoiseModel, gate_types: List[pyQPanda.GateType], t1: float, t2: float, t_gate: float, qubits: pyQPanda.QVec) -> None

        Add a noise model to multiple gate types with specified time parameters and targeted qubits.

        Args:
             noise_model: An instance of NOISE_MODEL to be added.

             gate_types: A vector of gate types to which the noise model applies.

             t1: The time constant for relaxation (T1).

             t2: The time constant for dephasing (T2).

             t_gate: The duration of the gate operation.

             qubits: A vector of qubit indices that the noise affects.

        Returns:
             None.


        10. add_noise_model(self: pyQPanda.Noise, noise_model: pyQPanda.NoiseModel, gate_type: pyQPanda.GateType, t1: float, t2: float, t_gate: float, qubits: List[pyQPanda.QVec]) -> None

        Add a noise model to a specific gate with specified time parameters and targeted qubits.

        Args:
             noise_model: An instance of NOISE_MODEL to be added.

             gate_type: The type of gate to which the noise model applies.

             t1: The time constant for relaxation (T1).

             t2: The time constant for dephasing (T2).

             t_gate: The duration of the gate operation.

             qubits: A vector of vectors of qubit indices that the noise affects.

        Returns:
             None.

        """
    @overload
    def add_noise_model(self, noise_model: NoiseModel, gate_types: list[GateType], t1: float, t2: float, t_gate: float) -> None:
        """add_noise_model(*args, **kwargs)
        Overloaded function.

        1. add_noise_model(self: pyQPanda.Noise, noise_model: pyQPanda.NoiseModel, gate_type: pyQPanda.GateType, prob: float) -> None

        Add a noise model to the noise simulation.

        Args:
             noise_model: An instance of NOISE_MODEL to be added.

             gate_type: The type of gate to which the noise model applies.

             prob: The probability of the noise occurring.

        Returns:
             None.


        2. add_noise_model(self: pyQPanda.Noise, noise_model: pyQPanda.NoiseModel, gate_types: List[pyQPanda.GateType], prob: float) -> None

        Add a noise model to multiple gate types.

        Args:
             noise_model: An instance of NOISE_MODEL to be added.

             gate_types: A vector of gate types to which the noise model applies.

             prob: The probability of the noise occurring.

        Returns:
             None.


        3. add_noise_model(self: pyQPanda.Noise, noise_model: pyQPanda.NoiseModel, gate_type: pyQPanda.GateType, prob: float, qubits: pyQPanda.QVec) -> None

        Add a noise model to a specific gate with targeted qubits.

        Args:
             noise_model: An instance of NOISE_MODEL to be added.

             gate_type: The type of gate to which the noise model applies.

             prob: The probability of the noise occurring.

             qubits: A vector of qubit indices that the noise affects.

        Returns:
             None.


        4. add_noise_model(self: pyQPanda.Noise, noise_model: pyQPanda.NoiseModel, gate_types: List[pyQPanda.GateType], prob: float, qubits: pyQPanda.QVec) -> None

        Add a noise model to multiple gate types with targeted qubits.

        Args:
             noise_model: An instance of NOISE_MODEL to be added.

             gate_types: A vector of gate types to which the noise model applies.

             prob: The probability of the noise occurring.

             qubits: A vector of qubit indices that the noise affects.

        Returns:
             None.


        5. add_noise_model(self: pyQPanda.Noise, noise_model: pyQPanda.NoiseModel, gate_type: pyQPanda.GateType, prob: float, qubits: List[pyQPanda.QVec]) -> None

        Add a noise model to a specific gate with targeted qubits.

        Args:
             noise_model: An instance of NOISE_MODEL to be added.

             gate_type: The type of gate to which the noise model applies.

             prob: The probability of the noise occurring.

             qubits: A vector of qubit indices that the noise affects.

        Returns:
             None.


        6. add_noise_model(self: pyQPanda.Noise, noise_model: pyQPanda.NoiseModel, gate_type: pyQPanda.GateType, t1: float, t2: float, t_gate: float) -> None

        Add a noise model to a specific gate with time parameters.

        Args:
             noise_model: An instance of NOISE_MODEL to be added.

             gate_type: The type of gate to which the noise model applies.

             t1: The time constant for relaxation (T1).

             t2: The time constant for dephasing (T2).

             t_gate: The duration of the gate operation.

        Returns:
             None.


        7. add_noise_model(self: pyQPanda.Noise, noise_model: pyQPanda.NoiseModel, gate_types: List[pyQPanda.GateType], t1: float, t2: float, t_gate: float) -> None

        Add a noise model to multiple gate types with time parameters.

        Args:
             noise_model: An instance of NOISE_MODEL to be added.

             gate_types: A vector of gate types to which the noise model applies.

             t1: The time constant for relaxation (T1).

             t2: The time constant for dephasing (T2).

             t_gate: The duration of the gate operation.

        Returns:
             None.


        8. add_noise_model(self: pyQPanda.Noise, noise_model: pyQPanda.NoiseModel, gate_type: pyQPanda.GateType, t1: float, t2: float, t_gate: float, qubits: pyQPanda.QVec) -> None

        Add a noise model to a specific gate with time parameters and targeted qubits.

        Args:
             noise_model: An instance of NOISE_MODEL to be added.

             gate_type: The type of gate to which the noise model applies.

             t1: The time constant for relaxation (T1).

             t2: The time constant for dephasing (T2).

             t_gate: The duration of the gate operation.

             qubits: A vector of qubit indices that the noise affects.

        Returns:
             None.


        9. add_noise_model(self: pyQPanda.Noise, noise_model: pyQPanda.NoiseModel, gate_types: List[pyQPanda.GateType], t1: float, t2: float, t_gate: float, qubits: pyQPanda.QVec) -> None

        Add a noise model to multiple gate types with specified time parameters and targeted qubits.

        Args:
             noise_model: An instance of NOISE_MODEL to be added.

             gate_types: A vector of gate types to which the noise model applies.

             t1: The time constant for relaxation (T1).

             t2: The time constant for dephasing (T2).

             t_gate: The duration of the gate operation.

             qubits: A vector of qubit indices that the noise affects.

        Returns:
             None.


        10. add_noise_model(self: pyQPanda.Noise, noise_model: pyQPanda.NoiseModel, gate_type: pyQPanda.GateType, t1: float, t2: float, t_gate: float, qubits: List[pyQPanda.QVec]) -> None

        Add a noise model to a specific gate with specified time parameters and targeted qubits.

        Args:
             noise_model: An instance of NOISE_MODEL to be added.

             gate_type: The type of gate to which the noise model applies.

             t1: The time constant for relaxation (T1).

             t2: The time constant for dephasing (T2).

             t_gate: The duration of the gate operation.

             qubits: A vector of vectors of qubit indices that the noise affects.

        Returns:
             None.

        """
    @overload
    def add_noise_model(self, noise_model: NoiseModel, gate_type: GateType, t1: float, t2: float, t_gate: float, qubits: QVec) -> None:
        """add_noise_model(*args, **kwargs)
        Overloaded function.

        1. add_noise_model(self: pyQPanda.Noise, noise_model: pyQPanda.NoiseModel, gate_type: pyQPanda.GateType, prob: float) -> None

        Add a noise model to the noise simulation.

        Args:
             noise_model: An instance of NOISE_MODEL to be added.

             gate_type: The type of gate to which the noise model applies.

             prob: The probability of the noise occurring.

        Returns:
             None.


        2. add_noise_model(self: pyQPanda.Noise, noise_model: pyQPanda.NoiseModel, gate_types: List[pyQPanda.GateType], prob: float) -> None

        Add a noise model to multiple gate types.

        Args:
             noise_model: An instance of NOISE_MODEL to be added.

             gate_types: A vector of gate types to which the noise model applies.

             prob: The probability of the noise occurring.

        Returns:
             None.


        3. add_noise_model(self: pyQPanda.Noise, noise_model: pyQPanda.NoiseModel, gate_type: pyQPanda.GateType, prob: float, qubits: pyQPanda.QVec) -> None

        Add a noise model to a specific gate with targeted qubits.

        Args:
             noise_model: An instance of NOISE_MODEL to be added.

             gate_type: The type of gate to which the noise model applies.

             prob: The probability of the noise occurring.

             qubits: A vector of qubit indices that the noise affects.

        Returns:
             None.


        4. add_noise_model(self: pyQPanda.Noise, noise_model: pyQPanda.NoiseModel, gate_types: List[pyQPanda.GateType], prob: float, qubits: pyQPanda.QVec) -> None

        Add a noise model to multiple gate types with targeted qubits.

        Args:
             noise_model: An instance of NOISE_MODEL to be added.

             gate_types: A vector of gate types to which the noise model applies.

             prob: The probability of the noise occurring.

             qubits: A vector of qubit indices that the noise affects.

        Returns:
             None.


        5. add_noise_model(self: pyQPanda.Noise, noise_model: pyQPanda.NoiseModel, gate_type: pyQPanda.GateType, prob: float, qubits: List[pyQPanda.QVec]) -> None

        Add a noise model to a specific gate with targeted qubits.

        Args:
             noise_model: An instance of NOISE_MODEL to be added.

             gate_type: The type of gate to which the noise model applies.

             prob: The probability of the noise occurring.

             qubits: A vector of qubit indices that the noise affects.

        Returns:
             None.


        6. add_noise_model(self: pyQPanda.Noise, noise_model: pyQPanda.NoiseModel, gate_type: pyQPanda.GateType, t1: float, t2: float, t_gate: float) -> None

        Add a noise model to a specific gate with time parameters.

        Args:
             noise_model: An instance of NOISE_MODEL to be added.

             gate_type: The type of gate to which the noise model applies.

             t1: The time constant for relaxation (T1).

             t2: The time constant for dephasing (T2).

             t_gate: The duration of the gate operation.

        Returns:
             None.


        7. add_noise_model(self: pyQPanda.Noise, noise_model: pyQPanda.NoiseModel, gate_types: List[pyQPanda.GateType], t1: float, t2: float, t_gate: float) -> None

        Add a noise model to multiple gate types with time parameters.

        Args:
             noise_model: An instance of NOISE_MODEL to be added.

             gate_types: A vector of gate types to which the noise model applies.

             t1: The time constant for relaxation (T1).

             t2: The time constant for dephasing (T2).

             t_gate: The duration of the gate operation.

        Returns:
             None.


        8. add_noise_model(self: pyQPanda.Noise, noise_model: pyQPanda.NoiseModel, gate_type: pyQPanda.GateType, t1: float, t2: float, t_gate: float, qubits: pyQPanda.QVec) -> None

        Add a noise model to a specific gate with time parameters and targeted qubits.

        Args:
             noise_model: An instance of NOISE_MODEL to be added.

             gate_type: The type of gate to which the noise model applies.

             t1: The time constant for relaxation (T1).

             t2: The time constant for dephasing (T2).

             t_gate: The duration of the gate operation.

             qubits: A vector of qubit indices that the noise affects.

        Returns:
             None.


        9. add_noise_model(self: pyQPanda.Noise, noise_model: pyQPanda.NoiseModel, gate_types: List[pyQPanda.GateType], t1: float, t2: float, t_gate: float, qubits: pyQPanda.QVec) -> None

        Add a noise model to multiple gate types with specified time parameters and targeted qubits.

        Args:
             noise_model: An instance of NOISE_MODEL to be added.

             gate_types: A vector of gate types to which the noise model applies.

             t1: The time constant for relaxation (T1).

             t2: The time constant for dephasing (T2).

             t_gate: The duration of the gate operation.

             qubits: A vector of qubit indices that the noise affects.

        Returns:
             None.


        10. add_noise_model(self: pyQPanda.Noise, noise_model: pyQPanda.NoiseModel, gate_type: pyQPanda.GateType, t1: float, t2: float, t_gate: float, qubits: List[pyQPanda.QVec]) -> None

        Add a noise model to a specific gate with specified time parameters and targeted qubits.

        Args:
             noise_model: An instance of NOISE_MODEL to be added.

             gate_type: The type of gate to which the noise model applies.

             t1: The time constant for relaxation (T1).

             t2: The time constant for dephasing (T2).

             t_gate: The duration of the gate operation.

             qubits: A vector of vectors of qubit indices that the noise affects.

        Returns:
             None.

        """
    @overload
    def add_noise_model(self, noise_model: NoiseModel, gate_types: list[GateType], t1: float, t2: float, t_gate: float, qubits: QVec) -> None:
        """add_noise_model(*args, **kwargs)
        Overloaded function.

        1. add_noise_model(self: pyQPanda.Noise, noise_model: pyQPanda.NoiseModel, gate_type: pyQPanda.GateType, prob: float) -> None

        Add a noise model to the noise simulation.

        Args:
             noise_model: An instance of NOISE_MODEL to be added.

             gate_type: The type of gate to which the noise model applies.

             prob: The probability of the noise occurring.

        Returns:
             None.


        2. add_noise_model(self: pyQPanda.Noise, noise_model: pyQPanda.NoiseModel, gate_types: List[pyQPanda.GateType], prob: float) -> None

        Add a noise model to multiple gate types.

        Args:
             noise_model: An instance of NOISE_MODEL to be added.

             gate_types: A vector of gate types to which the noise model applies.

             prob: The probability of the noise occurring.

        Returns:
             None.


        3. add_noise_model(self: pyQPanda.Noise, noise_model: pyQPanda.NoiseModel, gate_type: pyQPanda.GateType, prob: float, qubits: pyQPanda.QVec) -> None

        Add a noise model to a specific gate with targeted qubits.

        Args:
             noise_model: An instance of NOISE_MODEL to be added.

             gate_type: The type of gate to which the noise model applies.

             prob: The probability of the noise occurring.

             qubits: A vector of qubit indices that the noise affects.

        Returns:
             None.


        4. add_noise_model(self: pyQPanda.Noise, noise_model: pyQPanda.NoiseModel, gate_types: List[pyQPanda.GateType], prob: float, qubits: pyQPanda.QVec) -> None

        Add a noise model to multiple gate types with targeted qubits.

        Args:
             noise_model: An instance of NOISE_MODEL to be added.

             gate_types: A vector of gate types to which the noise model applies.

             prob: The probability of the noise occurring.

             qubits: A vector of qubit indices that the noise affects.

        Returns:
             None.


        5. add_noise_model(self: pyQPanda.Noise, noise_model: pyQPanda.NoiseModel, gate_type: pyQPanda.GateType, prob: float, qubits: List[pyQPanda.QVec]) -> None

        Add a noise model to a specific gate with targeted qubits.

        Args:
             noise_model: An instance of NOISE_MODEL to be added.

             gate_type: The type of gate to which the noise model applies.

             prob: The probability of the noise occurring.

             qubits: A vector of qubit indices that the noise affects.

        Returns:
             None.


        6. add_noise_model(self: pyQPanda.Noise, noise_model: pyQPanda.NoiseModel, gate_type: pyQPanda.GateType, t1: float, t2: float, t_gate: float) -> None

        Add a noise model to a specific gate with time parameters.

        Args:
             noise_model: An instance of NOISE_MODEL to be added.

             gate_type: The type of gate to which the noise model applies.

             t1: The time constant for relaxation (T1).

             t2: The time constant for dephasing (T2).

             t_gate: The duration of the gate operation.

        Returns:
             None.


        7. add_noise_model(self: pyQPanda.Noise, noise_model: pyQPanda.NoiseModel, gate_types: List[pyQPanda.GateType], t1: float, t2: float, t_gate: float) -> None

        Add a noise model to multiple gate types with time parameters.

        Args:
             noise_model: An instance of NOISE_MODEL to be added.

             gate_types: A vector of gate types to which the noise model applies.

             t1: The time constant for relaxation (T1).

             t2: The time constant for dephasing (T2).

             t_gate: The duration of the gate operation.

        Returns:
             None.


        8. add_noise_model(self: pyQPanda.Noise, noise_model: pyQPanda.NoiseModel, gate_type: pyQPanda.GateType, t1: float, t2: float, t_gate: float, qubits: pyQPanda.QVec) -> None

        Add a noise model to a specific gate with time parameters and targeted qubits.

        Args:
             noise_model: An instance of NOISE_MODEL to be added.

             gate_type: The type of gate to which the noise model applies.

             t1: The time constant for relaxation (T1).

             t2: The time constant for dephasing (T2).

             t_gate: The duration of the gate operation.

             qubits: A vector of qubit indices that the noise affects.

        Returns:
             None.


        9. add_noise_model(self: pyQPanda.Noise, noise_model: pyQPanda.NoiseModel, gate_types: List[pyQPanda.GateType], t1: float, t2: float, t_gate: float, qubits: pyQPanda.QVec) -> None

        Add a noise model to multiple gate types with specified time parameters and targeted qubits.

        Args:
             noise_model: An instance of NOISE_MODEL to be added.

             gate_types: A vector of gate types to which the noise model applies.

             t1: The time constant for relaxation (T1).

             t2: The time constant for dephasing (T2).

             t_gate: The duration of the gate operation.

             qubits: A vector of qubit indices that the noise affects.

        Returns:
             None.


        10. add_noise_model(self: pyQPanda.Noise, noise_model: pyQPanda.NoiseModel, gate_type: pyQPanda.GateType, t1: float, t2: float, t_gate: float, qubits: List[pyQPanda.QVec]) -> None

        Add a noise model to a specific gate with specified time parameters and targeted qubits.

        Args:
             noise_model: An instance of NOISE_MODEL to be added.

             gate_type: The type of gate to which the noise model applies.

             t1: The time constant for relaxation (T1).

             t2: The time constant for dephasing (T2).

             t_gate: The duration of the gate operation.

             qubits: A vector of vectors of qubit indices that the noise affects.

        Returns:
             None.

        """
    @overload
    def add_noise_model(self, noise_model: NoiseModel, gate_type: GateType, t1: float, t2: float, t_gate: float, qubits: list[QVec]) -> None:
        """add_noise_model(*args, **kwargs)
        Overloaded function.

        1. add_noise_model(self: pyQPanda.Noise, noise_model: pyQPanda.NoiseModel, gate_type: pyQPanda.GateType, prob: float) -> None

        Add a noise model to the noise simulation.

        Args:
             noise_model: An instance of NOISE_MODEL to be added.

             gate_type: The type of gate to which the noise model applies.

             prob: The probability of the noise occurring.

        Returns:
             None.


        2. add_noise_model(self: pyQPanda.Noise, noise_model: pyQPanda.NoiseModel, gate_types: List[pyQPanda.GateType], prob: float) -> None

        Add a noise model to multiple gate types.

        Args:
             noise_model: An instance of NOISE_MODEL to be added.

             gate_types: A vector of gate types to which the noise model applies.

             prob: The probability of the noise occurring.

        Returns:
             None.


        3. add_noise_model(self: pyQPanda.Noise, noise_model: pyQPanda.NoiseModel, gate_type: pyQPanda.GateType, prob: float, qubits: pyQPanda.QVec) -> None

        Add a noise model to a specific gate with targeted qubits.

        Args:
             noise_model: An instance of NOISE_MODEL to be added.

             gate_type: The type of gate to which the noise model applies.

             prob: The probability of the noise occurring.

             qubits: A vector of qubit indices that the noise affects.

        Returns:
             None.


        4. add_noise_model(self: pyQPanda.Noise, noise_model: pyQPanda.NoiseModel, gate_types: List[pyQPanda.GateType], prob: float, qubits: pyQPanda.QVec) -> None

        Add a noise model to multiple gate types with targeted qubits.

        Args:
             noise_model: An instance of NOISE_MODEL to be added.

             gate_types: A vector of gate types to which the noise model applies.

             prob: The probability of the noise occurring.

             qubits: A vector of qubit indices that the noise affects.

        Returns:
             None.


        5. add_noise_model(self: pyQPanda.Noise, noise_model: pyQPanda.NoiseModel, gate_type: pyQPanda.GateType, prob: float, qubits: List[pyQPanda.QVec]) -> None

        Add a noise model to a specific gate with targeted qubits.

        Args:
             noise_model: An instance of NOISE_MODEL to be added.

             gate_type: The type of gate to which the noise model applies.

             prob: The probability of the noise occurring.

             qubits: A vector of qubit indices that the noise affects.

        Returns:
             None.


        6. add_noise_model(self: pyQPanda.Noise, noise_model: pyQPanda.NoiseModel, gate_type: pyQPanda.GateType, t1: float, t2: float, t_gate: float) -> None

        Add a noise model to a specific gate with time parameters.

        Args:
             noise_model: An instance of NOISE_MODEL to be added.

             gate_type: The type of gate to which the noise model applies.

             t1: The time constant for relaxation (T1).

             t2: The time constant for dephasing (T2).

             t_gate: The duration of the gate operation.

        Returns:
             None.


        7. add_noise_model(self: pyQPanda.Noise, noise_model: pyQPanda.NoiseModel, gate_types: List[pyQPanda.GateType], t1: float, t2: float, t_gate: float) -> None

        Add a noise model to multiple gate types with time parameters.

        Args:
             noise_model: An instance of NOISE_MODEL to be added.

             gate_types: A vector of gate types to which the noise model applies.

             t1: The time constant for relaxation (T1).

             t2: The time constant for dephasing (T2).

             t_gate: The duration of the gate operation.

        Returns:
             None.


        8. add_noise_model(self: pyQPanda.Noise, noise_model: pyQPanda.NoiseModel, gate_type: pyQPanda.GateType, t1: float, t2: float, t_gate: float, qubits: pyQPanda.QVec) -> None

        Add a noise model to a specific gate with time parameters and targeted qubits.

        Args:
             noise_model: An instance of NOISE_MODEL to be added.

             gate_type: The type of gate to which the noise model applies.

             t1: The time constant for relaxation (T1).

             t2: The time constant for dephasing (T2).

             t_gate: The duration of the gate operation.

             qubits: A vector of qubit indices that the noise affects.

        Returns:
             None.


        9. add_noise_model(self: pyQPanda.Noise, noise_model: pyQPanda.NoiseModel, gate_types: List[pyQPanda.GateType], t1: float, t2: float, t_gate: float, qubits: pyQPanda.QVec) -> None

        Add a noise model to multiple gate types with specified time parameters and targeted qubits.

        Args:
             noise_model: An instance of NOISE_MODEL to be added.

             gate_types: A vector of gate types to which the noise model applies.

             t1: The time constant for relaxation (T1).

             t2: The time constant for dephasing (T2).

             t_gate: The duration of the gate operation.

             qubits: A vector of qubit indices that the noise affects.

        Returns:
             None.


        10. add_noise_model(self: pyQPanda.Noise, noise_model: pyQPanda.NoiseModel, gate_type: pyQPanda.GateType, t1: float, t2: float, t_gate: float, qubits: List[pyQPanda.QVec]) -> None

        Add a noise model to a specific gate with specified time parameters and targeted qubits.

        Args:
             noise_model: An instance of NOISE_MODEL to be added.

             gate_type: The type of gate to which the noise model applies.

             t1: The time constant for relaxation (T1).

             t2: The time constant for dephasing (T2).

             t_gate: The duration of the gate operation.

             qubits: A vector of vectors of qubit indices that the noise affects.

        Returns:
             None.

        """
    @overload
    def set_measure_error(self, noise_model: NoiseModel, prob: float, qubits: QVec = ...) -> None:
        """set_measure_error(*args, **kwargs)
        Overloaded function.

        1. set_measure_error(self: pyQPanda.Noise, noise_model: pyQPanda.NoiseModel, prob: float, qubits: pyQPanda.QVec = <pyQPanda.QVec object at 0x000002856827DAB0>) -> None

        Set the measurement error for specified qubits.

        Args:
             noise_model: An instance of NOISE_MODEL to be used.

             prob: The probability of measurement error.

             qubits: A vector of qubit indices to which the measurement error applies. Defaults to an empty QVec.

        Returns:
             None.


        2. set_measure_error(self: pyQPanda.Noise, noise_model: pyQPanda.NoiseModel, t1: float, t2: float, t_gate: float, qubits: pyQPanda.QVec = <pyQPanda.QVec object at 0x0000028567B75430>) -> None

        Set the measurement error using time parameters for the specified qubits.

        Args:
             noise_model: An instance of NOISE_MODEL to be used.

             t1: The time constant for relaxation (T1).

             t2: The time constant for dephasing (T2).

             t_gate: The duration of the gate operation.

             qubits: A vector of qubit indices to which the measurement error applies. Defaults to an empty QVec.

        Returns:
             None.

        """
    @overload
    def set_measure_error(self, noise_model: NoiseModel, t1: float, t2: float, t_gate: float, qubits: QVec = ...) -> None:
        """set_measure_error(*args, **kwargs)
        Overloaded function.

        1. set_measure_error(self: pyQPanda.Noise, noise_model: pyQPanda.NoiseModel, prob: float, qubits: pyQPanda.QVec = <pyQPanda.QVec object at 0x000002856827DAB0>) -> None

        Set the measurement error for specified qubits.

        Args:
             noise_model: An instance of NOISE_MODEL to be used.

             prob: The probability of measurement error.

             qubits: A vector of qubit indices to which the measurement error applies. Defaults to an empty QVec.

        Returns:
             None.


        2. set_measure_error(self: pyQPanda.Noise, noise_model: pyQPanda.NoiseModel, t1: float, t2: float, t_gate: float, qubits: pyQPanda.QVec = <pyQPanda.QVec object at 0x0000028567B75430>) -> None

        Set the measurement error using time parameters for the specified qubits.

        Args:
             noise_model: An instance of NOISE_MODEL to be used.

             t1: The time constant for relaxation (T1).

             t2: The time constant for dephasing (T2).

             t_gate: The duration of the gate operation.

             qubits: A vector of qubit indices to which the measurement error applies. Defaults to an empty QVec.

        Returns:
             None.

        """
    def set_readout_error(self, prob_list: list[list[float]], qubits: QVec = ...) -> None:
        """set_readout_error(self: pyQPanda.Noise, prob_list: List[List[float]], qubits: pyQPanda.QVec = <pyQPanda.QVec object at 0x000002856825E0B0>) -> None

        Set readout errors for specified qubits.

        Args:
             prob_list: A list of probabilities for readout errors.

             qubits: A vector of qubit indices that the readout errors apply to (default is all qubits).

        Returns:
             None.

        """
    def set_reset_error(self, p0: float, p1: float, qubits: QVec) -> None:
        """set_reset_error(self: pyQPanda.Noise, p0: float, p1: float, qubits: pyQPanda.QVec) -> None

        Set reset errors for specified qubits.

        Args:
             p0: The probability of resetting to state |0>.

             p1: The probability of resetting to state |1>.

             qubits: A vector of qubit indices that the reset errors apply to.

        Returns:
             None.

        """
    def set_rotation_error(self, error: float) -> None:
        """set_rotation_error(self: pyQPanda.Noise, error: float) -> None

        Set rotation error for gates.

        Args:
             error: The error model for rotation operations.

        Returns:
             None.

        """

class NoiseModel:
    __members__: ClassVar[dict] = ...  # read-only
    BITFLIP_KRAUS_OPERATOR: ClassVar[NoiseModel] = ...
    BIT_PHASE_FLIP_OPRATOR: ClassVar[NoiseModel] = ...
    DAMPING_KRAUS_OPERATOR: ClassVar[NoiseModel] = ...
    DECOHERENCE_KRAUS_OPERATOR: ClassVar[NoiseModel] = ...
    DEPHASING_KRAUS_OPERATOR: ClassVar[NoiseModel] = ...
    DEPOLARIZING_KRAUS_OPERATOR: ClassVar[NoiseModel] = ...
    PAULI_KRAUS_MAP: ClassVar[NoiseModel] = ...
    PHASE_DAMPING_OPRATOR: ClassVar[NoiseModel] = ...
    __entries: ClassVar[dict] = ...
    def __init__(self, value: int) -> None:
        """__init__(self: pyQPanda.NoiseModel, value: int) -> None"""
    def __eq__(self, other: object) -> bool:
        """__eq__(self: object, other: object) -> bool"""
    def __hash__(self) -> int:
        """__hash__(self: object) -> int"""
    def __index__(self) -> int:
        """__index__(self: pyQPanda.NoiseModel) -> int"""
    def __int__(self) -> int:
        """__int__(self: pyQPanda.NoiseModel) -> int"""
    def __ne__(self, other: object) -> bool:
        """__ne__(self: object, other: object) -> bool"""
    @property
    def name(self) -> str: ...
    @property
    def value(self) -> int: ...

class NoiseQVM(QuantumMachine):
    def __init__(self) -> None:
        """__init__(self: pyQPanda.NoiseQVM) -> None"""
    def initQVM(self, arg0: dict) -> None:
        """initQVM(self: pyQPanda.NoiseQVM, arg0: dict) -> None

        init quantum virtual machine
        """
    @overload
    def init_qvm(self, json_config: dict) -> None:
        """init_qvm(*args, **kwargs)
        Overloaded function.

        1. init_qvm(self: pyQPanda.NoiseQVM, json_config: dict) -> None

        init quantum virtual machine

        2. init_qvm(self: pyQPanda.NoiseQVM) -> None

        init quantum virtual machine
        """
    @overload
    def init_qvm(self) -> None:
        """init_qvm(*args, **kwargs)
        Overloaded function.

        1. init_qvm(self: pyQPanda.NoiseQVM, json_config: dict) -> None

        init quantum virtual machine

        2. init_qvm(self: pyQPanda.NoiseQVM) -> None

        init quantum virtual machine
        """
    def set_max_threads(self, size: int) -> None:
        """set_max_threads(self: pyQPanda.NoiseQVM, size: int) -> None

        Set the maximum number of threads for the noise quantum virtual machine (NoiseQVM).

        Args:
             size: The maximum number of threads to utilize.

        Returns:
             None: This method does not return a value.

        """
    @overload
    def set_measure_error(self, model: NoiseModel, prob: float, qubits: QVec = ...) -> None:
        """set_measure_error(*args, **kwargs)
        Overloaded function.

        1. set_measure_error(self: pyQPanda.NoiseQVM, model: pyQPanda.NoiseModel, prob: float, qubits: pyQPanda.QVec = <pyQPanda.QVec object at 0x000002856827F470>) -> None

        Set the measurement error model in the quantum virtual machine.

        Args:
             model: The noise model to be applied for measurement errors.

             prob: A double representing the probability of measurement error.

             qubits: A specific qubit vector (QVec) for which the measurement error applies (default is an empty QVec).

        Returns:
             None, as the function configures the measurement error model in place for the specified qubit vector.


        2. set_measure_error(self: pyQPanda.NoiseQVM, model: pyQPanda.NoiseModel, T1: float, T2: float, t_gate: float, qubits: pyQPanda.QVec = <pyQPanda.QVec object at 0x0000028568271AB0>) -> None

        Set the measurement error model in the quantum virtual machine with specific error parameters.

        Args:
             model: The noise model to be applied for measurement errors.

             T1: A double representing the relaxation time constant for the qubits.

             T2: A double representing the dephasing time constant for the qubits.

             t_gate: A double representing the time duration of the gate operation.

             qubits: A specific qubit vector (QVec) for which the measurement error applies (default is an empty QVec).

        Returns:
             None, as the function configures the measurement error model in place for the specified qubit vector.

        """
    @overload
    def set_measure_error(self, model: NoiseModel, T1: float, T2: float, t_gate: float, qubits: QVec = ...) -> None:
        """set_measure_error(*args, **kwargs)
        Overloaded function.

        1. set_measure_error(self: pyQPanda.NoiseQVM, model: pyQPanda.NoiseModel, prob: float, qubits: pyQPanda.QVec = <pyQPanda.QVec object at 0x000002856827F470>) -> None

        Set the measurement error model in the quantum virtual machine.

        Args:
             model: The noise model to be applied for measurement errors.

             prob: A double representing the probability of measurement error.

             qubits: A specific qubit vector (QVec) for which the measurement error applies (default is an empty QVec).

        Returns:
             None, as the function configures the measurement error model in place for the specified qubit vector.


        2. set_measure_error(self: pyQPanda.NoiseQVM, model: pyQPanda.NoiseModel, T1: float, T2: float, t_gate: float, qubits: pyQPanda.QVec = <pyQPanda.QVec object at 0x0000028568271AB0>) -> None

        Set the measurement error model in the quantum virtual machine with specific error parameters.

        Args:
             model: The noise model to be applied for measurement errors.

             T1: A double representing the relaxation time constant for the qubits.

             T2: A double representing the dephasing time constant for the qubits.

             t_gate: A double representing the time duration of the gate operation.

             qubits: A specific qubit vector (QVec) for which the measurement error applies (default is an empty QVec).

        Returns:
             None, as the function configures the measurement error model in place for the specified qubit vector.

        """
    @overload
    def set_mixed_unitary_error(self, arg0: GateType, arg1: list[list[complex]], arg2: list[float]) -> None:
        """set_mixed_unitary_error(*args, **kwargs)
        Overloaded function.

        1. set_mixed_unitary_error(self: pyQPanda.NoiseQVM, arg0: pyQPanda.GateType, arg1: List[List[complex]], arg2: List[float]) -> None

        Set a mixed unitary error model for a specific gate type in the quantum virtual machine.

        Args:
             gate_type: The type of gate for which the mixed unitary error model applies.

             unitary_ops: A vector of unitary operations (QStat) representing the error model.

             probabilities: A vector of doubles representing the probabilities associated with each unitary operation.

        Returns:
             None, as the function configures the mixed unitary error model in place for the specified gate type.


        2. set_mixed_unitary_error(self: pyQPanda.NoiseQVM, arg0: pyQPanda.GateType, arg1: List[List[complex]], arg2: List[float], arg3: pyQPanda.QVec) -> None

        Set a mixed unitary error model for a specific gate type in the quantum virtual machine with specific qubits.

        Args:
             gate_type: The type of gate for which the mixed unitary error model applies.

             unitary_ops: A vector of unitary operations (QStat) representing the error model.

             probabilities: A vector of doubles representing the probabilities associated with each unitary operation.

             qubits: A specific qubit vector (QVec) for which the mixed unitary error applies.

        Returns:
             None, as the function configures the mixed unitary error model in place for the specified gate type and qubits.


        3. set_mixed_unitary_error(self: pyQPanda.NoiseQVM, arg0: pyQPanda.GateType, arg1: List[List[complex]], arg2: List[float], arg3: List[pyQPanda.QVec]) -> None

        Set a mixed unitary error model for a specific gate type in the quantum virtual machine, targeting multiple qubits.

        Args:
             gate_type: The type of gate for which the mixed unitary error model applies.

             unitary_ops: A vector of unitary operations (QStat) representing the error model.

             probabilities: A vector of doubles representing the probabilities associated with each unitary operation.

             qubit_groups: A vector of qubit vectors (QVec) specifying the qubits affected by the error model.

        Returns:
             None, as the function configures the mixed unitary error model in place for the specified gate type and qubit groups.

        """
    @overload
    def set_mixed_unitary_error(self, arg0: GateType, arg1: list[list[complex]], arg2: list[float], arg3: QVec) -> None:
        """set_mixed_unitary_error(*args, **kwargs)
        Overloaded function.

        1. set_mixed_unitary_error(self: pyQPanda.NoiseQVM, arg0: pyQPanda.GateType, arg1: List[List[complex]], arg2: List[float]) -> None

        Set a mixed unitary error model for a specific gate type in the quantum virtual machine.

        Args:
             gate_type: The type of gate for which the mixed unitary error model applies.

             unitary_ops: A vector of unitary operations (QStat) representing the error model.

             probabilities: A vector of doubles representing the probabilities associated with each unitary operation.

        Returns:
             None, as the function configures the mixed unitary error model in place for the specified gate type.


        2. set_mixed_unitary_error(self: pyQPanda.NoiseQVM, arg0: pyQPanda.GateType, arg1: List[List[complex]], arg2: List[float], arg3: pyQPanda.QVec) -> None

        Set a mixed unitary error model for a specific gate type in the quantum virtual machine with specific qubits.

        Args:
             gate_type: The type of gate for which the mixed unitary error model applies.

             unitary_ops: A vector of unitary operations (QStat) representing the error model.

             probabilities: A vector of doubles representing the probabilities associated with each unitary operation.

             qubits: A specific qubit vector (QVec) for which the mixed unitary error applies.

        Returns:
             None, as the function configures the mixed unitary error model in place for the specified gate type and qubits.


        3. set_mixed_unitary_error(self: pyQPanda.NoiseQVM, arg0: pyQPanda.GateType, arg1: List[List[complex]], arg2: List[float], arg3: List[pyQPanda.QVec]) -> None

        Set a mixed unitary error model for a specific gate type in the quantum virtual machine, targeting multiple qubits.

        Args:
             gate_type: The type of gate for which the mixed unitary error model applies.

             unitary_ops: A vector of unitary operations (QStat) representing the error model.

             probabilities: A vector of doubles representing the probabilities associated with each unitary operation.

             qubit_groups: A vector of qubit vectors (QVec) specifying the qubits affected by the error model.

        Returns:
             None, as the function configures the mixed unitary error model in place for the specified gate type and qubit groups.

        """
    @overload
    def set_mixed_unitary_error(self, arg0: GateType, arg1: list[list[complex]], arg2: list[float], arg3: list[QVec]) -> None:
        """set_mixed_unitary_error(*args, **kwargs)
        Overloaded function.

        1. set_mixed_unitary_error(self: pyQPanda.NoiseQVM, arg0: pyQPanda.GateType, arg1: List[List[complex]], arg2: List[float]) -> None

        Set a mixed unitary error model for a specific gate type in the quantum virtual machine.

        Args:
             gate_type: The type of gate for which the mixed unitary error model applies.

             unitary_ops: A vector of unitary operations (QStat) representing the error model.

             probabilities: A vector of doubles representing the probabilities associated with each unitary operation.

        Returns:
             None, as the function configures the mixed unitary error model in place for the specified gate type.


        2. set_mixed_unitary_error(self: pyQPanda.NoiseQVM, arg0: pyQPanda.GateType, arg1: List[List[complex]], arg2: List[float], arg3: pyQPanda.QVec) -> None

        Set a mixed unitary error model for a specific gate type in the quantum virtual machine with specific qubits.

        Args:
             gate_type: The type of gate for which the mixed unitary error model applies.

             unitary_ops: A vector of unitary operations (QStat) representing the error model.

             probabilities: A vector of doubles representing the probabilities associated with each unitary operation.

             qubits: A specific qubit vector (QVec) for which the mixed unitary error applies.

        Returns:
             None, as the function configures the mixed unitary error model in place for the specified gate type and qubits.


        3. set_mixed_unitary_error(self: pyQPanda.NoiseQVM, arg0: pyQPanda.GateType, arg1: List[List[complex]], arg2: List[float], arg3: List[pyQPanda.QVec]) -> None

        Set a mixed unitary error model for a specific gate type in the quantum virtual machine, targeting multiple qubits.

        Args:
             gate_type: The type of gate for which the mixed unitary error model applies.

             unitary_ops: A vector of unitary operations (QStat) representing the error model.

             probabilities: A vector of doubles representing the probabilities associated with each unitary operation.

             qubit_groups: A vector of qubit vectors (QVec) specifying the qubits affected by the error model.

        Returns:
             None, as the function configures the mixed unitary error model in place for the specified gate type and qubit groups.

        """
    @overload
    def set_noise_model(self, arg0: NoiseModel, arg1: GateType, arg2: float) -> None:
        """set_noise_model(*args, **kwargs)
        Overloaded function.

        1. set_noise_model(self: pyQPanda.NoiseQVM, arg0: pyQPanda.NoiseModel, arg1: pyQPanda.GateType, arg2: float) -> None

        Set the noise model for the quantum virtual machine.

        Args:
             noise_model: The noise model to be applied.

             gate_type: The type of gate for which the noise model is relevant.

             noise_level: A double representing the level of noise to apply.

        Returns:
             None, as the function configures the noise model in place.


        2. set_noise_model(self: pyQPanda.NoiseQVM, arg0: pyQPanda.NoiseModel, arg1: List[pyQPanda.GateType], arg2: float) -> None

        Set the noise model for multiple gate types in the quantum virtual machine.

        Args:
             noise_model: The noise model to be applied.

             gate_types: A vector of gate types for which the noise model is relevant.

             noise_level: A double representing the level of noise to apply.

        Returns:
             None, as the function configures the noise model in place for the specified gate types.


        3. set_noise_model(self: pyQPanda.NoiseQVM, arg0: pyQPanda.NoiseModel, arg1: pyQPanda.GateType, arg2: float, arg3: pyQPanda.QVec) -> None

        Set the noise model for a specific gate type and qubit vector in the quantum virtual machine.

        Args:
             noise_model: The noise model to be applied.

             gate_type: The type of gate for which the noise model is relevant.

             noise_level: A double representing the level of noise to apply.

             qubits: A vector of qubits (QVec) affected by the noise model.

        Returns:
             None, as the function configures the noise model in place for the specified gate type and qubits.


        4. set_noise_model(self: pyQPanda.NoiseQVM, arg0: pyQPanda.NoiseModel, arg1: List[pyQPanda.GateType], arg2: float, arg3: pyQPanda.QVec) -> None

        Set the noise model for multiple gate types and a specific qubit vector in the quantum virtual machine.

        Args:
             noise_model: The noise model to be applied.

             gate_types: A vector of gate types for which the noise model is relevant.

             noise_level: A double representing the level of noise to apply.

             qubits: A vector of qubits (QVec) affected by the noise model.

        Returns:
             None, as the function configures the noise model in place for the specified gate types and qubits.


        5. set_noise_model(self: pyQPanda.NoiseQVM, arg0: pyQPanda.NoiseModel, arg1: pyQPanda.GateType, arg2: float, arg3: List[pyQPanda.QVec]) -> None

        Set the noise model for a specific gate type and multiple qubit vectors in the quantum virtual machine.

        Args:
             noise_model: The noise model to be applied.

             gate_type: The type of gate for which the noise model is relevant.

             noise_level: A double representing the level of noise to apply.

             qubits: A vector of qubit vectors (std::vector<QVec>) affected by the noise model.

        Returns:
             None, as the function configures the noise model in place for the specified gate type and qubit vectors.


        6. set_noise_model(self: pyQPanda.NoiseQVM, arg0: pyQPanda.NoiseModel, arg1: pyQPanda.GateType, arg2: float, arg3: float, arg4: float) -> None

        Set the noise model for a specific gate type with multiple noise parameters in the quantum virtual machine.

        Args:
             noise_model: The noise model to be applied.

             gate_type: The type of gate for which the noise model is relevant.

             noise_level1: A double representing the first level of noise to apply.

             noise_level2: A double representing the second level of noise to apply.

             noise_level3: A double representing the third level of noise to apply.

        Returns:
             None, as the function configures the noise model in place for the specified gate type with the given noise parameters.


        7. set_noise_model(self: pyQPanda.NoiseQVM, arg0: pyQPanda.NoiseModel, arg1: List[pyQPanda.GateType], arg2: float, arg3: float, arg4: float) -> None

        Set the noise model for multiple gate types with various noise parameters in the quantum virtual machine.

        Args:
             noise_model: The noise model to be applied.

             gate_types: A vector of gate types for which the noise model is relevant.

             noise_level1: A double representing the first level of noise to apply.

             noise_level2: A double representing the second level of noise to apply.

             noise_level3: A double representing the third level of noise to apply.

        Returns:
             None, as the function configures the noise model in place for the specified gate types with the given noise parameters.


        8. set_noise_model(self: pyQPanda.NoiseQVM, arg0: pyQPanda.NoiseModel, arg1: pyQPanda.GateType, arg2: float, arg3: float, arg4: float, arg5: pyQPanda.QVec) -> None

        Set the noise model for a specific gate type with multiple noise parameters affecting a specific qubit vector in the quantum virtual machine.

        Args:
             noise_model: The noise model to be applied.

             gate_type: The type of gate for which the noise model is relevant.

             noise_level1: A double representing the first level of noise to apply.

             noise_level2: A double representing the second level of noise to apply.

             noise_level3: A double representing the third level of noise to apply.

             qubits: A specific qubit vector (QVec) affected by the noise model.

        Returns:
             None, as the function configures the noise model in place for the specified gate type and qubit vector.


        9. set_noise_model(self: pyQPanda.NoiseQVM, arg0: pyQPanda.NoiseModel, arg1: List[pyQPanda.GateType], arg2: float, arg3: float, arg4: float, arg5: pyQPanda.QVec) -> None

        Set the noise model for multiple gate types with various noise parameters affecting a specific qubit vector in the quantum virtual machine.

        Args:
             noise_model: The noise model to be applied.

             gate_types: A vector of gate types for which the noise model is relevant.

             noise_level1: A double representing the first level of noise to apply.

             noise_level2: A double representing the second level of noise to apply.

             noise_level3: A double representing the third level of noise to apply.

             qubits: A specific qubit vector (QVec) affected by the noise model.

        Returns:
             None, as the function configures the noise model in place for the specified gate types and qubit vector.


        10. set_noise_model(self: pyQPanda.NoiseQVM, arg0: pyQPanda.NoiseModel, arg1: pyQPanda.GateType, arg2: float, arg3: float, arg4: float, arg5: List[pyQPanda.QVec]) -> None

        Set the noise model for a specific gate type with multiple noise parameters affecting a vector of qubit vectors in the quantum virtual machine.

        Args:
             noise_model: The noise model to be applied.

             gate_type: The type of gate for which the noise model is relevant.

             noise_level1: A double representing the first level of noise to apply.

             noise_level2: A double representing the second level of noise to apply.

             noise_level3: A double representing the third level of noise to apply.

             qubits_list: A vector of qubit vectors (QVec) affected by the noise model.

        Returns:
             None, as the function configures the noise model in place for the specified gate type and qubit vectors.

        """
    @overload
    def set_noise_model(self, arg0: NoiseModel, arg1: list[GateType], arg2: float) -> None:
        """set_noise_model(*args, **kwargs)
        Overloaded function.

        1. set_noise_model(self: pyQPanda.NoiseQVM, arg0: pyQPanda.NoiseModel, arg1: pyQPanda.GateType, arg2: float) -> None

        Set the noise model for the quantum virtual machine.

        Args:
             noise_model: The noise model to be applied.

             gate_type: The type of gate for which the noise model is relevant.

             noise_level: A double representing the level of noise to apply.

        Returns:
             None, as the function configures the noise model in place.


        2. set_noise_model(self: pyQPanda.NoiseQVM, arg0: pyQPanda.NoiseModel, arg1: List[pyQPanda.GateType], arg2: float) -> None

        Set the noise model for multiple gate types in the quantum virtual machine.

        Args:
             noise_model: The noise model to be applied.

             gate_types: A vector of gate types for which the noise model is relevant.

             noise_level: A double representing the level of noise to apply.

        Returns:
             None, as the function configures the noise model in place for the specified gate types.


        3. set_noise_model(self: pyQPanda.NoiseQVM, arg0: pyQPanda.NoiseModel, arg1: pyQPanda.GateType, arg2: float, arg3: pyQPanda.QVec) -> None

        Set the noise model for a specific gate type and qubit vector in the quantum virtual machine.

        Args:
             noise_model: The noise model to be applied.

             gate_type: The type of gate for which the noise model is relevant.

             noise_level: A double representing the level of noise to apply.

             qubits: A vector of qubits (QVec) affected by the noise model.

        Returns:
             None, as the function configures the noise model in place for the specified gate type and qubits.


        4. set_noise_model(self: pyQPanda.NoiseQVM, arg0: pyQPanda.NoiseModel, arg1: List[pyQPanda.GateType], arg2: float, arg3: pyQPanda.QVec) -> None

        Set the noise model for multiple gate types and a specific qubit vector in the quantum virtual machine.

        Args:
             noise_model: The noise model to be applied.

             gate_types: A vector of gate types for which the noise model is relevant.

             noise_level: A double representing the level of noise to apply.

             qubits: A vector of qubits (QVec) affected by the noise model.

        Returns:
             None, as the function configures the noise model in place for the specified gate types and qubits.


        5. set_noise_model(self: pyQPanda.NoiseQVM, arg0: pyQPanda.NoiseModel, arg1: pyQPanda.GateType, arg2: float, arg3: List[pyQPanda.QVec]) -> None

        Set the noise model for a specific gate type and multiple qubit vectors in the quantum virtual machine.

        Args:
             noise_model: The noise model to be applied.

             gate_type: The type of gate for which the noise model is relevant.

             noise_level: A double representing the level of noise to apply.

             qubits: A vector of qubit vectors (std::vector<QVec>) affected by the noise model.

        Returns:
             None, as the function configures the noise model in place for the specified gate type and qubit vectors.


        6. set_noise_model(self: pyQPanda.NoiseQVM, arg0: pyQPanda.NoiseModel, arg1: pyQPanda.GateType, arg2: float, arg3: float, arg4: float) -> None

        Set the noise model for a specific gate type with multiple noise parameters in the quantum virtual machine.

        Args:
             noise_model: The noise model to be applied.

             gate_type: The type of gate for which the noise model is relevant.

             noise_level1: A double representing the first level of noise to apply.

             noise_level2: A double representing the second level of noise to apply.

             noise_level3: A double representing the third level of noise to apply.

        Returns:
             None, as the function configures the noise model in place for the specified gate type with the given noise parameters.


        7. set_noise_model(self: pyQPanda.NoiseQVM, arg0: pyQPanda.NoiseModel, arg1: List[pyQPanda.GateType], arg2: float, arg3: float, arg4: float) -> None

        Set the noise model for multiple gate types with various noise parameters in the quantum virtual machine.

        Args:
             noise_model: The noise model to be applied.

             gate_types: A vector of gate types for which the noise model is relevant.

             noise_level1: A double representing the first level of noise to apply.

             noise_level2: A double representing the second level of noise to apply.

             noise_level3: A double representing the third level of noise to apply.

        Returns:
             None, as the function configures the noise model in place for the specified gate types with the given noise parameters.


        8. set_noise_model(self: pyQPanda.NoiseQVM, arg0: pyQPanda.NoiseModel, arg1: pyQPanda.GateType, arg2: float, arg3: float, arg4: float, arg5: pyQPanda.QVec) -> None

        Set the noise model for a specific gate type with multiple noise parameters affecting a specific qubit vector in the quantum virtual machine.

        Args:
             noise_model: The noise model to be applied.

             gate_type: The type of gate for which the noise model is relevant.

             noise_level1: A double representing the first level of noise to apply.

             noise_level2: A double representing the second level of noise to apply.

             noise_level3: A double representing the third level of noise to apply.

             qubits: A specific qubit vector (QVec) affected by the noise model.

        Returns:
             None, as the function configures the noise model in place for the specified gate type and qubit vector.


        9. set_noise_model(self: pyQPanda.NoiseQVM, arg0: pyQPanda.NoiseModel, arg1: List[pyQPanda.GateType], arg2: float, arg3: float, arg4: float, arg5: pyQPanda.QVec) -> None

        Set the noise model for multiple gate types with various noise parameters affecting a specific qubit vector in the quantum virtual machine.

        Args:
             noise_model: The noise model to be applied.

             gate_types: A vector of gate types for which the noise model is relevant.

             noise_level1: A double representing the first level of noise to apply.

             noise_level2: A double representing the second level of noise to apply.

             noise_level3: A double representing the third level of noise to apply.

             qubits: A specific qubit vector (QVec) affected by the noise model.

        Returns:
             None, as the function configures the noise model in place for the specified gate types and qubit vector.


        10. set_noise_model(self: pyQPanda.NoiseQVM, arg0: pyQPanda.NoiseModel, arg1: pyQPanda.GateType, arg2: float, arg3: float, arg4: float, arg5: List[pyQPanda.QVec]) -> None

        Set the noise model for a specific gate type with multiple noise parameters affecting a vector of qubit vectors in the quantum virtual machine.

        Args:
             noise_model: The noise model to be applied.

             gate_type: The type of gate for which the noise model is relevant.

             noise_level1: A double representing the first level of noise to apply.

             noise_level2: A double representing the second level of noise to apply.

             noise_level3: A double representing the third level of noise to apply.

             qubits_list: A vector of qubit vectors (QVec) affected by the noise model.

        Returns:
             None, as the function configures the noise model in place for the specified gate type and qubit vectors.

        """
    @overload
    def set_noise_model(self, arg0: NoiseModel, arg1: GateType, arg2: float, arg3: QVec) -> None:
        """set_noise_model(*args, **kwargs)
        Overloaded function.

        1. set_noise_model(self: pyQPanda.NoiseQVM, arg0: pyQPanda.NoiseModel, arg1: pyQPanda.GateType, arg2: float) -> None

        Set the noise model for the quantum virtual machine.

        Args:
             noise_model: The noise model to be applied.

             gate_type: The type of gate for which the noise model is relevant.

             noise_level: A double representing the level of noise to apply.

        Returns:
             None, as the function configures the noise model in place.


        2. set_noise_model(self: pyQPanda.NoiseQVM, arg0: pyQPanda.NoiseModel, arg1: List[pyQPanda.GateType], arg2: float) -> None

        Set the noise model for multiple gate types in the quantum virtual machine.

        Args:
             noise_model: The noise model to be applied.

             gate_types: A vector of gate types for which the noise model is relevant.

             noise_level: A double representing the level of noise to apply.

        Returns:
             None, as the function configures the noise model in place for the specified gate types.


        3. set_noise_model(self: pyQPanda.NoiseQVM, arg0: pyQPanda.NoiseModel, arg1: pyQPanda.GateType, arg2: float, arg3: pyQPanda.QVec) -> None

        Set the noise model for a specific gate type and qubit vector in the quantum virtual machine.

        Args:
             noise_model: The noise model to be applied.

             gate_type: The type of gate for which the noise model is relevant.

             noise_level: A double representing the level of noise to apply.

             qubits: A vector of qubits (QVec) affected by the noise model.

        Returns:
             None, as the function configures the noise model in place for the specified gate type and qubits.


        4. set_noise_model(self: pyQPanda.NoiseQVM, arg0: pyQPanda.NoiseModel, arg1: List[pyQPanda.GateType], arg2: float, arg3: pyQPanda.QVec) -> None

        Set the noise model for multiple gate types and a specific qubit vector in the quantum virtual machine.

        Args:
             noise_model: The noise model to be applied.

             gate_types: A vector of gate types for which the noise model is relevant.

             noise_level: A double representing the level of noise to apply.

             qubits: A vector of qubits (QVec) affected by the noise model.

        Returns:
             None, as the function configures the noise model in place for the specified gate types and qubits.


        5. set_noise_model(self: pyQPanda.NoiseQVM, arg0: pyQPanda.NoiseModel, arg1: pyQPanda.GateType, arg2: float, arg3: List[pyQPanda.QVec]) -> None

        Set the noise model for a specific gate type and multiple qubit vectors in the quantum virtual machine.

        Args:
             noise_model: The noise model to be applied.

             gate_type: The type of gate for which the noise model is relevant.

             noise_level: A double representing the level of noise to apply.

             qubits: A vector of qubit vectors (std::vector<QVec>) affected by the noise model.

        Returns:
             None, as the function configures the noise model in place for the specified gate type and qubit vectors.


        6. set_noise_model(self: pyQPanda.NoiseQVM, arg0: pyQPanda.NoiseModel, arg1: pyQPanda.GateType, arg2: float, arg3: float, arg4: float) -> None

        Set the noise model for a specific gate type with multiple noise parameters in the quantum virtual machine.

        Args:
             noise_model: The noise model to be applied.

             gate_type: The type of gate for which the noise model is relevant.

             noise_level1: A double representing the first level of noise to apply.

             noise_level2: A double representing the second level of noise to apply.

             noise_level3: A double representing the third level of noise to apply.

        Returns:
             None, as the function configures the noise model in place for the specified gate type with the given noise parameters.


        7. set_noise_model(self: pyQPanda.NoiseQVM, arg0: pyQPanda.NoiseModel, arg1: List[pyQPanda.GateType], arg2: float, arg3: float, arg4: float) -> None

        Set the noise model for multiple gate types with various noise parameters in the quantum virtual machine.

        Args:
             noise_model: The noise model to be applied.

             gate_types: A vector of gate types for which the noise model is relevant.

             noise_level1: A double representing the first level of noise to apply.

             noise_level2: A double representing the second level of noise to apply.

             noise_level3: A double representing the third level of noise to apply.

        Returns:
             None, as the function configures the noise model in place for the specified gate types with the given noise parameters.


        8. set_noise_model(self: pyQPanda.NoiseQVM, arg0: pyQPanda.NoiseModel, arg1: pyQPanda.GateType, arg2: float, arg3: float, arg4: float, arg5: pyQPanda.QVec) -> None

        Set the noise model for a specific gate type with multiple noise parameters affecting a specific qubit vector in the quantum virtual machine.

        Args:
             noise_model: The noise model to be applied.

             gate_type: The type of gate for which the noise model is relevant.

             noise_level1: A double representing the first level of noise to apply.

             noise_level2: A double representing the second level of noise to apply.

             noise_level3: A double representing the third level of noise to apply.

             qubits: A specific qubit vector (QVec) affected by the noise model.

        Returns:
             None, as the function configures the noise model in place for the specified gate type and qubit vector.


        9. set_noise_model(self: pyQPanda.NoiseQVM, arg0: pyQPanda.NoiseModel, arg1: List[pyQPanda.GateType], arg2: float, arg3: float, arg4: float, arg5: pyQPanda.QVec) -> None

        Set the noise model for multiple gate types with various noise parameters affecting a specific qubit vector in the quantum virtual machine.

        Args:
             noise_model: The noise model to be applied.

             gate_types: A vector of gate types for which the noise model is relevant.

             noise_level1: A double representing the first level of noise to apply.

             noise_level2: A double representing the second level of noise to apply.

             noise_level3: A double representing the third level of noise to apply.

             qubits: A specific qubit vector (QVec) affected by the noise model.

        Returns:
             None, as the function configures the noise model in place for the specified gate types and qubit vector.


        10. set_noise_model(self: pyQPanda.NoiseQVM, arg0: pyQPanda.NoiseModel, arg1: pyQPanda.GateType, arg2: float, arg3: float, arg4: float, arg5: List[pyQPanda.QVec]) -> None

        Set the noise model for a specific gate type with multiple noise parameters affecting a vector of qubit vectors in the quantum virtual machine.

        Args:
             noise_model: The noise model to be applied.

             gate_type: The type of gate for which the noise model is relevant.

             noise_level1: A double representing the first level of noise to apply.

             noise_level2: A double representing the second level of noise to apply.

             noise_level3: A double representing the third level of noise to apply.

             qubits_list: A vector of qubit vectors (QVec) affected by the noise model.

        Returns:
             None, as the function configures the noise model in place for the specified gate type and qubit vectors.

        """
    @overload
    def set_noise_model(self, arg0: NoiseModel, arg1: list[GateType], arg2: float, arg3: QVec) -> None:
        """set_noise_model(*args, **kwargs)
        Overloaded function.

        1. set_noise_model(self: pyQPanda.NoiseQVM, arg0: pyQPanda.NoiseModel, arg1: pyQPanda.GateType, arg2: float) -> None

        Set the noise model for the quantum virtual machine.

        Args:
             noise_model: The noise model to be applied.

             gate_type: The type of gate for which the noise model is relevant.

             noise_level: A double representing the level of noise to apply.

        Returns:
             None, as the function configures the noise model in place.


        2. set_noise_model(self: pyQPanda.NoiseQVM, arg0: pyQPanda.NoiseModel, arg1: List[pyQPanda.GateType], arg2: float) -> None

        Set the noise model for multiple gate types in the quantum virtual machine.

        Args:
             noise_model: The noise model to be applied.

             gate_types: A vector of gate types for which the noise model is relevant.

             noise_level: A double representing the level of noise to apply.

        Returns:
             None, as the function configures the noise model in place for the specified gate types.


        3. set_noise_model(self: pyQPanda.NoiseQVM, arg0: pyQPanda.NoiseModel, arg1: pyQPanda.GateType, arg2: float, arg3: pyQPanda.QVec) -> None

        Set the noise model for a specific gate type and qubit vector in the quantum virtual machine.

        Args:
             noise_model: The noise model to be applied.

             gate_type: The type of gate for which the noise model is relevant.

             noise_level: A double representing the level of noise to apply.

             qubits: A vector of qubits (QVec) affected by the noise model.

        Returns:
             None, as the function configures the noise model in place for the specified gate type and qubits.


        4. set_noise_model(self: pyQPanda.NoiseQVM, arg0: pyQPanda.NoiseModel, arg1: List[pyQPanda.GateType], arg2: float, arg3: pyQPanda.QVec) -> None

        Set the noise model for multiple gate types and a specific qubit vector in the quantum virtual machine.

        Args:
             noise_model: The noise model to be applied.

             gate_types: A vector of gate types for which the noise model is relevant.

             noise_level: A double representing the level of noise to apply.

             qubits: A vector of qubits (QVec) affected by the noise model.

        Returns:
             None, as the function configures the noise model in place for the specified gate types and qubits.


        5. set_noise_model(self: pyQPanda.NoiseQVM, arg0: pyQPanda.NoiseModel, arg1: pyQPanda.GateType, arg2: float, arg3: List[pyQPanda.QVec]) -> None

        Set the noise model for a specific gate type and multiple qubit vectors in the quantum virtual machine.

        Args:
             noise_model: The noise model to be applied.

             gate_type: The type of gate for which the noise model is relevant.

             noise_level: A double representing the level of noise to apply.

             qubits: A vector of qubit vectors (std::vector<QVec>) affected by the noise model.

        Returns:
             None, as the function configures the noise model in place for the specified gate type and qubit vectors.


        6. set_noise_model(self: pyQPanda.NoiseQVM, arg0: pyQPanda.NoiseModel, arg1: pyQPanda.GateType, arg2: float, arg3: float, arg4: float) -> None

        Set the noise model for a specific gate type with multiple noise parameters in the quantum virtual machine.

        Args:
             noise_model: The noise model to be applied.

             gate_type: The type of gate for which the noise model is relevant.

             noise_level1: A double representing the first level of noise to apply.

             noise_level2: A double representing the second level of noise to apply.

             noise_level3: A double representing the third level of noise to apply.

        Returns:
             None, as the function configures the noise model in place for the specified gate type with the given noise parameters.


        7. set_noise_model(self: pyQPanda.NoiseQVM, arg0: pyQPanda.NoiseModel, arg1: List[pyQPanda.GateType], arg2: float, arg3: float, arg4: float) -> None

        Set the noise model for multiple gate types with various noise parameters in the quantum virtual machine.

        Args:
             noise_model: The noise model to be applied.

             gate_types: A vector of gate types for which the noise model is relevant.

             noise_level1: A double representing the first level of noise to apply.

             noise_level2: A double representing the second level of noise to apply.

             noise_level3: A double representing the third level of noise to apply.

        Returns:
             None, as the function configures the noise model in place for the specified gate types with the given noise parameters.


        8. set_noise_model(self: pyQPanda.NoiseQVM, arg0: pyQPanda.NoiseModel, arg1: pyQPanda.GateType, arg2: float, arg3: float, arg4: float, arg5: pyQPanda.QVec) -> None

        Set the noise model for a specific gate type with multiple noise parameters affecting a specific qubit vector in the quantum virtual machine.

        Args:
             noise_model: The noise model to be applied.

             gate_type: The type of gate for which the noise model is relevant.

             noise_level1: A double representing the first level of noise to apply.

             noise_level2: A double representing the second level of noise to apply.

             noise_level3: A double representing the third level of noise to apply.

             qubits: A specific qubit vector (QVec) affected by the noise model.

        Returns:
             None, as the function configures the noise model in place for the specified gate type and qubit vector.


        9. set_noise_model(self: pyQPanda.NoiseQVM, arg0: pyQPanda.NoiseModel, arg1: List[pyQPanda.GateType], arg2: float, arg3: float, arg4: float, arg5: pyQPanda.QVec) -> None

        Set the noise model for multiple gate types with various noise parameters affecting a specific qubit vector in the quantum virtual machine.

        Args:
             noise_model: The noise model to be applied.

             gate_types: A vector of gate types for which the noise model is relevant.

             noise_level1: A double representing the first level of noise to apply.

             noise_level2: A double representing the second level of noise to apply.

             noise_level3: A double representing the third level of noise to apply.

             qubits: A specific qubit vector (QVec) affected by the noise model.

        Returns:
             None, as the function configures the noise model in place for the specified gate types and qubit vector.


        10. set_noise_model(self: pyQPanda.NoiseQVM, arg0: pyQPanda.NoiseModel, arg1: pyQPanda.GateType, arg2: float, arg3: float, arg4: float, arg5: List[pyQPanda.QVec]) -> None

        Set the noise model for a specific gate type with multiple noise parameters affecting a vector of qubit vectors in the quantum virtual machine.

        Args:
             noise_model: The noise model to be applied.

             gate_type: The type of gate for which the noise model is relevant.

             noise_level1: A double representing the first level of noise to apply.

             noise_level2: A double representing the second level of noise to apply.

             noise_level3: A double representing the third level of noise to apply.

             qubits_list: A vector of qubit vectors (QVec) affected by the noise model.

        Returns:
             None, as the function configures the noise model in place for the specified gate type and qubit vectors.

        """
    @overload
    def set_noise_model(self, arg0: NoiseModel, arg1: GateType, arg2: float, arg3: list[QVec]) -> None:
        """set_noise_model(*args, **kwargs)
        Overloaded function.

        1. set_noise_model(self: pyQPanda.NoiseQVM, arg0: pyQPanda.NoiseModel, arg1: pyQPanda.GateType, arg2: float) -> None

        Set the noise model for the quantum virtual machine.

        Args:
             noise_model: The noise model to be applied.

             gate_type: The type of gate for which the noise model is relevant.

             noise_level: A double representing the level of noise to apply.

        Returns:
             None, as the function configures the noise model in place.


        2. set_noise_model(self: pyQPanda.NoiseQVM, arg0: pyQPanda.NoiseModel, arg1: List[pyQPanda.GateType], arg2: float) -> None

        Set the noise model for multiple gate types in the quantum virtual machine.

        Args:
             noise_model: The noise model to be applied.

             gate_types: A vector of gate types for which the noise model is relevant.

             noise_level: A double representing the level of noise to apply.

        Returns:
             None, as the function configures the noise model in place for the specified gate types.


        3. set_noise_model(self: pyQPanda.NoiseQVM, arg0: pyQPanda.NoiseModel, arg1: pyQPanda.GateType, arg2: float, arg3: pyQPanda.QVec) -> None

        Set the noise model for a specific gate type and qubit vector in the quantum virtual machine.

        Args:
             noise_model: The noise model to be applied.

             gate_type: The type of gate for which the noise model is relevant.

             noise_level: A double representing the level of noise to apply.

             qubits: A vector of qubits (QVec) affected by the noise model.

        Returns:
             None, as the function configures the noise model in place for the specified gate type and qubits.


        4. set_noise_model(self: pyQPanda.NoiseQVM, arg0: pyQPanda.NoiseModel, arg1: List[pyQPanda.GateType], arg2: float, arg3: pyQPanda.QVec) -> None

        Set the noise model for multiple gate types and a specific qubit vector in the quantum virtual machine.

        Args:
             noise_model: The noise model to be applied.

             gate_types: A vector of gate types for which the noise model is relevant.

             noise_level: A double representing the level of noise to apply.

             qubits: A vector of qubits (QVec) affected by the noise model.

        Returns:
             None, as the function configures the noise model in place for the specified gate types and qubits.


        5. set_noise_model(self: pyQPanda.NoiseQVM, arg0: pyQPanda.NoiseModel, arg1: pyQPanda.GateType, arg2: float, arg3: List[pyQPanda.QVec]) -> None

        Set the noise model for a specific gate type and multiple qubit vectors in the quantum virtual machine.

        Args:
             noise_model: The noise model to be applied.

             gate_type: The type of gate for which the noise model is relevant.

             noise_level: A double representing the level of noise to apply.

             qubits: A vector of qubit vectors (std::vector<QVec>) affected by the noise model.

        Returns:
             None, as the function configures the noise model in place for the specified gate type and qubit vectors.


        6. set_noise_model(self: pyQPanda.NoiseQVM, arg0: pyQPanda.NoiseModel, arg1: pyQPanda.GateType, arg2: float, arg3: float, arg4: float) -> None

        Set the noise model for a specific gate type with multiple noise parameters in the quantum virtual machine.

        Args:
             noise_model: The noise model to be applied.

             gate_type: The type of gate for which the noise model is relevant.

             noise_level1: A double representing the first level of noise to apply.

             noise_level2: A double representing the second level of noise to apply.

             noise_level3: A double representing the third level of noise to apply.

        Returns:
             None, as the function configures the noise model in place for the specified gate type with the given noise parameters.


        7. set_noise_model(self: pyQPanda.NoiseQVM, arg0: pyQPanda.NoiseModel, arg1: List[pyQPanda.GateType], arg2: float, arg3: float, arg4: float) -> None

        Set the noise model for multiple gate types with various noise parameters in the quantum virtual machine.

        Args:
             noise_model: The noise model to be applied.

             gate_types: A vector of gate types for which the noise model is relevant.

             noise_level1: A double representing the first level of noise to apply.

             noise_level2: A double representing the second level of noise to apply.

             noise_level3: A double representing the third level of noise to apply.

        Returns:
             None, as the function configures the noise model in place for the specified gate types with the given noise parameters.


        8. set_noise_model(self: pyQPanda.NoiseQVM, arg0: pyQPanda.NoiseModel, arg1: pyQPanda.GateType, arg2: float, arg3: float, arg4: float, arg5: pyQPanda.QVec) -> None

        Set the noise model for a specific gate type with multiple noise parameters affecting a specific qubit vector in the quantum virtual machine.

        Args:
             noise_model: The noise model to be applied.

             gate_type: The type of gate for which the noise model is relevant.

             noise_level1: A double representing the first level of noise to apply.

             noise_level2: A double representing the second level of noise to apply.

             noise_level3: A double representing the third level of noise to apply.

             qubits: A specific qubit vector (QVec) affected by the noise model.

        Returns:
             None, as the function configures the noise model in place for the specified gate type and qubit vector.


        9. set_noise_model(self: pyQPanda.NoiseQVM, arg0: pyQPanda.NoiseModel, arg1: List[pyQPanda.GateType], arg2: float, arg3: float, arg4: float, arg5: pyQPanda.QVec) -> None

        Set the noise model for multiple gate types with various noise parameters affecting a specific qubit vector in the quantum virtual machine.

        Args:
             noise_model: The noise model to be applied.

             gate_types: A vector of gate types for which the noise model is relevant.

             noise_level1: A double representing the first level of noise to apply.

             noise_level2: A double representing the second level of noise to apply.

             noise_level3: A double representing the third level of noise to apply.

             qubits: A specific qubit vector (QVec) affected by the noise model.

        Returns:
             None, as the function configures the noise model in place for the specified gate types and qubit vector.


        10. set_noise_model(self: pyQPanda.NoiseQVM, arg0: pyQPanda.NoiseModel, arg1: pyQPanda.GateType, arg2: float, arg3: float, arg4: float, arg5: List[pyQPanda.QVec]) -> None

        Set the noise model for a specific gate type with multiple noise parameters affecting a vector of qubit vectors in the quantum virtual machine.

        Args:
             noise_model: The noise model to be applied.

             gate_type: The type of gate for which the noise model is relevant.

             noise_level1: A double representing the first level of noise to apply.

             noise_level2: A double representing the second level of noise to apply.

             noise_level3: A double representing the third level of noise to apply.

             qubits_list: A vector of qubit vectors (QVec) affected by the noise model.

        Returns:
             None, as the function configures the noise model in place for the specified gate type and qubit vectors.

        """
    @overload
    def set_noise_model(self, arg0: NoiseModel, arg1: GateType, arg2: float, arg3: float, arg4: float) -> None:
        """set_noise_model(*args, **kwargs)
        Overloaded function.

        1. set_noise_model(self: pyQPanda.NoiseQVM, arg0: pyQPanda.NoiseModel, arg1: pyQPanda.GateType, arg2: float) -> None

        Set the noise model for the quantum virtual machine.

        Args:
             noise_model: The noise model to be applied.

             gate_type: The type of gate for which the noise model is relevant.

             noise_level: A double representing the level of noise to apply.

        Returns:
             None, as the function configures the noise model in place.


        2. set_noise_model(self: pyQPanda.NoiseQVM, arg0: pyQPanda.NoiseModel, arg1: List[pyQPanda.GateType], arg2: float) -> None

        Set the noise model for multiple gate types in the quantum virtual machine.

        Args:
             noise_model: The noise model to be applied.

             gate_types: A vector of gate types for which the noise model is relevant.

             noise_level: A double representing the level of noise to apply.

        Returns:
             None, as the function configures the noise model in place for the specified gate types.


        3. set_noise_model(self: pyQPanda.NoiseQVM, arg0: pyQPanda.NoiseModel, arg1: pyQPanda.GateType, arg2: float, arg3: pyQPanda.QVec) -> None

        Set the noise model for a specific gate type and qubit vector in the quantum virtual machine.

        Args:
             noise_model: The noise model to be applied.

             gate_type: The type of gate for which the noise model is relevant.

             noise_level: A double representing the level of noise to apply.

             qubits: A vector of qubits (QVec) affected by the noise model.

        Returns:
             None, as the function configures the noise model in place for the specified gate type and qubits.


        4. set_noise_model(self: pyQPanda.NoiseQVM, arg0: pyQPanda.NoiseModel, arg1: List[pyQPanda.GateType], arg2: float, arg3: pyQPanda.QVec) -> None

        Set the noise model for multiple gate types and a specific qubit vector in the quantum virtual machine.

        Args:
             noise_model: The noise model to be applied.

             gate_types: A vector of gate types for which the noise model is relevant.

             noise_level: A double representing the level of noise to apply.

             qubits: A vector of qubits (QVec) affected by the noise model.

        Returns:
             None, as the function configures the noise model in place for the specified gate types and qubits.


        5. set_noise_model(self: pyQPanda.NoiseQVM, arg0: pyQPanda.NoiseModel, arg1: pyQPanda.GateType, arg2: float, arg3: List[pyQPanda.QVec]) -> None

        Set the noise model for a specific gate type and multiple qubit vectors in the quantum virtual machine.

        Args:
             noise_model: The noise model to be applied.

             gate_type: The type of gate for which the noise model is relevant.

             noise_level: A double representing the level of noise to apply.

             qubits: A vector of qubit vectors (std::vector<QVec>) affected by the noise model.

        Returns:
             None, as the function configures the noise model in place for the specified gate type and qubit vectors.


        6. set_noise_model(self: pyQPanda.NoiseQVM, arg0: pyQPanda.NoiseModel, arg1: pyQPanda.GateType, arg2: float, arg3: float, arg4: float) -> None

        Set the noise model for a specific gate type with multiple noise parameters in the quantum virtual machine.

        Args:
             noise_model: The noise model to be applied.

             gate_type: The type of gate for which the noise model is relevant.

             noise_level1: A double representing the first level of noise to apply.

             noise_level2: A double representing the second level of noise to apply.

             noise_level3: A double representing the third level of noise to apply.

        Returns:
             None, as the function configures the noise model in place for the specified gate type with the given noise parameters.


        7. set_noise_model(self: pyQPanda.NoiseQVM, arg0: pyQPanda.NoiseModel, arg1: List[pyQPanda.GateType], arg2: float, arg3: float, arg4: float) -> None

        Set the noise model for multiple gate types with various noise parameters in the quantum virtual machine.

        Args:
             noise_model: The noise model to be applied.

             gate_types: A vector of gate types for which the noise model is relevant.

             noise_level1: A double representing the first level of noise to apply.

             noise_level2: A double representing the second level of noise to apply.

             noise_level3: A double representing the third level of noise to apply.

        Returns:
             None, as the function configures the noise model in place for the specified gate types with the given noise parameters.


        8. set_noise_model(self: pyQPanda.NoiseQVM, arg0: pyQPanda.NoiseModel, arg1: pyQPanda.GateType, arg2: float, arg3: float, arg4: float, arg5: pyQPanda.QVec) -> None

        Set the noise model for a specific gate type with multiple noise parameters affecting a specific qubit vector in the quantum virtual machine.

        Args:
             noise_model: The noise model to be applied.

             gate_type: The type of gate for which the noise model is relevant.

             noise_level1: A double representing the first level of noise to apply.

             noise_level2: A double representing the second level of noise to apply.

             noise_level3: A double representing the third level of noise to apply.

             qubits: A specific qubit vector (QVec) affected by the noise model.

        Returns:
             None, as the function configures the noise model in place for the specified gate type and qubit vector.


        9. set_noise_model(self: pyQPanda.NoiseQVM, arg0: pyQPanda.NoiseModel, arg1: List[pyQPanda.GateType], arg2: float, arg3: float, arg4: float, arg5: pyQPanda.QVec) -> None

        Set the noise model for multiple gate types with various noise parameters affecting a specific qubit vector in the quantum virtual machine.

        Args:
             noise_model: The noise model to be applied.

             gate_types: A vector of gate types for which the noise model is relevant.

             noise_level1: A double representing the first level of noise to apply.

             noise_level2: A double representing the second level of noise to apply.

             noise_level3: A double representing the third level of noise to apply.

             qubits: A specific qubit vector (QVec) affected by the noise model.

        Returns:
             None, as the function configures the noise model in place for the specified gate types and qubit vector.


        10. set_noise_model(self: pyQPanda.NoiseQVM, arg0: pyQPanda.NoiseModel, arg1: pyQPanda.GateType, arg2: float, arg3: float, arg4: float, arg5: List[pyQPanda.QVec]) -> None

        Set the noise model for a specific gate type with multiple noise parameters affecting a vector of qubit vectors in the quantum virtual machine.

        Args:
             noise_model: The noise model to be applied.

             gate_type: The type of gate for which the noise model is relevant.

             noise_level1: A double representing the first level of noise to apply.

             noise_level2: A double representing the second level of noise to apply.

             noise_level3: A double representing the third level of noise to apply.

             qubits_list: A vector of qubit vectors (QVec) affected by the noise model.

        Returns:
             None, as the function configures the noise model in place for the specified gate type and qubit vectors.

        """
    @overload
    def set_noise_model(self, arg0: NoiseModel, arg1: list[GateType], arg2: float, arg3: float, arg4: float) -> None:
        """set_noise_model(*args, **kwargs)
        Overloaded function.

        1. set_noise_model(self: pyQPanda.NoiseQVM, arg0: pyQPanda.NoiseModel, arg1: pyQPanda.GateType, arg2: float) -> None

        Set the noise model for the quantum virtual machine.

        Args:
             noise_model: The noise model to be applied.

             gate_type: The type of gate for which the noise model is relevant.

             noise_level: A double representing the level of noise to apply.

        Returns:
             None, as the function configures the noise model in place.


        2. set_noise_model(self: pyQPanda.NoiseQVM, arg0: pyQPanda.NoiseModel, arg1: List[pyQPanda.GateType], arg2: float) -> None

        Set the noise model for multiple gate types in the quantum virtual machine.

        Args:
             noise_model: The noise model to be applied.

             gate_types: A vector of gate types for which the noise model is relevant.

             noise_level: A double representing the level of noise to apply.

        Returns:
             None, as the function configures the noise model in place for the specified gate types.


        3. set_noise_model(self: pyQPanda.NoiseQVM, arg0: pyQPanda.NoiseModel, arg1: pyQPanda.GateType, arg2: float, arg3: pyQPanda.QVec) -> None

        Set the noise model for a specific gate type and qubit vector in the quantum virtual machine.

        Args:
             noise_model: The noise model to be applied.

             gate_type: The type of gate for which the noise model is relevant.

             noise_level: A double representing the level of noise to apply.

             qubits: A vector of qubits (QVec) affected by the noise model.

        Returns:
             None, as the function configures the noise model in place for the specified gate type and qubits.


        4. set_noise_model(self: pyQPanda.NoiseQVM, arg0: pyQPanda.NoiseModel, arg1: List[pyQPanda.GateType], arg2: float, arg3: pyQPanda.QVec) -> None

        Set the noise model for multiple gate types and a specific qubit vector in the quantum virtual machine.

        Args:
             noise_model: The noise model to be applied.

             gate_types: A vector of gate types for which the noise model is relevant.

             noise_level: A double representing the level of noise to apply.

             qubits: A vector of qubits (QVec) affected by the noise model.

        Returns:
             None, as the function configures the noise model in place for the specified gate types and qubits.


        5. set_noise_model(self: pyQPanda.NoiseQVM, arg0: pyQPanda.NoiseModel, arg1: pyQPanda.GateType, arg2: float, arg3: List[pyQPanda.QVec]) -> None

        Set the noise model for a specific gate type and multiple qubit vectors in the quantum virtual machine.

        Args:
             noise_model: The noise model to be applied.

             gate_type: The type of gate for which the noise model is relevant.

             noise_level: A double representing the level of noise to apply.

             qubits: A vector of qubit vectors (std::vector<QVec>) affected by the noise model.

        Returns:
             None, as the function configures the noise model in place for the specified gate type and qubit vectors.


        6. set_noise_model(self: pyQPanda.NoiseQVM, arg0: pyQPanda.NoiseModel, arg1: pyQPanda.GateType, arg2: float, arg3: float, arg4: float) -> None

        Set the noise model for a specific gate type with multiple noise parameters in the quantum virtual machine.

        Args:
             noise_model: The noise model to be applied.

             gate_type: The type of gate for which the noise model is relevant.

             noise_level1: A double representing the first level of noise to apply.

             noise_level2: A double representing the second level of noise to apply.

             noise_level3: A double representing the third level of noise to apply.

        Returns:
             None, as the function configures the noise model in place for the specified gate type with the given noise parameters.


        7. set_noise_model(self: pyQPanda.NoiseQVM, arg0: pyQPanda.NoiseModel, arg1: List[pyQPanda.GateType], arg2: float, arg3: float, arg4: float) -> None

        Set the noise model for multiple gate types with various noise parameters in the quantum virtual machine.

        Args:
             noise_model: The noise model to be applied.

             gate_types: A vector of gate types for which the noise model is relevant.

             noise_level1: A double representing the first level of noise to apply.

             noise_level2: A double representing the second level of noise to apply.

             noise_level3: A double representing the third level of noise to apply.

        Returns:
             None, as the function configures the noise model in place for the specified gate types with the given noise parameters.


        8. set_noise_model(self: pyQPanda.NoiseQVM, arg0: pyQPanda.NoiseModel, arg1: pyQPanda.GateType, arg2: float, arg3: float, arg4: float, arg5: pyQPanda.QVec) -> None

        Set the noise model for a specific gate type with multiple noise parameters affecting a specific qubit vector in the quantum virtual machine.

        Args:
             noise_model: The noise model to be applied.

             gate_type: The type of gate for which the noise model is relevant.

             noise_level1: A double representing the first level of noise to apply.

             noise_level2: A double representing the second level of noise to apply.

             noise_level3: A double representing the third level of noise to apply.

             qubits: A specific qubit vector (QVec) affected by the noise model.

        Returns:
             None, as the function configures the noise model in place for the specified gate type and qubit vector.


        9. set_noise_model(self: pyQPanda.NoiseQVM, arg0: pyQPanda.NoiseModel, arg1: List[pyQPanda.GateType], arg2: float, arg3: float, arg4: float, arg5: pyQPanda.QVec) -> None

        Set the noise model for multiple gate types with various noise parameters affecting a specific qubit vector in the quantum virtual machine.

        Args:
             noise_model: The noise model to be applied.

             gate_types: A vector of gate types for which the noise model is relevant.

             noise_level1: A double representing the first level of noise to apply.

             noise_level2: A double representing the second level of noise to apply.

             noise_level3: A double representing the third level of noise to apply.

             qubits: A specific qubit vector (QVec) affected by the noise model.

        Returns:
             None, as the function configures the noise model in place for the specified gate types and qubit vector.


        10. set_noise_model(self: pyQPanda.NoiseQVM, arg0: pyQPanda.NoiseModel, arg1: pyQPanda.GateType, arg2: float, arg3: float, arg4: float, arg5: List[pyQPanda.QVec]) -> None

        Set the noise model for a specific gate type with multiple noise parameters affecting a vector of qubit vectors in the quantum virtual machine.

        Args:
             noise_model: The noise model to be applied.

             gate_type: The type of gate for which the noise model is relevant.

             noise_level1: A double representing the first level of noise to apply.

             noise_level2: A double representing the second level of noise to apply.

             noise_level3: A double representing the third level of noise to apply.

             qubits_list: A vector of qubit vectors (QVec) affected by the noise model.

        Returns:
             None, as the function configures the noise model in place for the specified gate type and qubit vectors.

        """
    @overload
    def set_noise_model(self, arg0: NoiseModel, arg1: GateType, arg2: float, arg3: float, arg4: float, arg5: QVec) -> None:
        """set_noise_model(*args, **kwargs)
        Overloaded function.

        1. set_noise_model(self: pyQPanda.NoiseQVM, arg0: pyQPanda.NoiseModel, arg1: pyQPanda.GateType, arg2: float) -> None

        Set the noise model for the quantum virtual machine.

        Args:
             noise_model: The noise model to be applied.

             gate_type: The type of gate for which the noise model is relevant.

             noise_level: A double representing the level of noise to apply.

        Returns:
             None, as the function configures the noise model in place.


        2. set_noise_model(self: pyQPanda.NoiseQVM, arg0: pyQPanda.NoiseModel, arg1: List[pyQPanda.GateType], arg2: float) -> None

        Set the noise model for multiple gate types in the quantum virtual machine.

        Args:
             noise_model: The noise model to be applied.

             gate_types: A vector of gate types for which the noise model is relevant.

             noise_level: A double representing the level of noise to apply.

        Returns:
             None, as the function configures the noise model in place for the specified gate types.


        3. set_noise_model(self: pyQPanda.NoiseQVM, arg0: pyQPanda.NoiseModel, arg1: pyQPanda.GateType, arg2: float, arg3: pyQPanda.QVec) -> None

        Set the noise model for a specific gate type and qubit vector in the quantum virtual machine.

        Args:
             noise_model: The noise model to be applied.

             gate_type: The type of gate for which the noise model is relevant.

             noise_level: A double representing the level of noise to apply.

             qubits: A vector of qubits (QVec) affected by the noise model.

        Returns:
             None, as the function configures the noise model in place for the specified gate type and qubits.


        4. set_noise_model(self: pyQPanda.NoiseQVM, arg0: pyQPanda.NoiseModel, arg1: List[pyQPanda.GateType], arg2: float, arg3: pyQPanda.QVec) -> None

        Set the noise model for multiple gate types and a specific qubit vector in the quantum virtual machine.

        Args:
             noise_model: The noise model to be applied.

             gate_types: A vector of gate types for which the noise model is relevant.

             noise_level: A double representing the level of noise to apply.

             qubits: A vector of qubits (QVec) affected by the noise model.

        Returns:
             None, as the function configures the noise model in place for the specified gate types and qubits.


        5. set_noise_model(self: pyQPanda.NoiseQVM, arg0: pyQPanda.NoiseModel, arg1: pyQPanda.GateType, arg2: float, arg3: List[pyQPanda.QVec]) -> None

        Set the noise model for a specific gate type and multiple qubit vectors in the quantum virtual machine.

        Args:
             noise_model: The noise model to be applied.

             gate_type: The type of gate for which the noise model is relevant.

             noise_level: A double representing the level of noise to apply.

             qubits: A vector of qubit vectors (std::vector<QVec>) affected by the noise model.

        Returns:
             None, as the function configures the noise model in place for the specified gate type and qubit vectors.


        6. set_noise_model(self: pyQPanda.NoiseQVM, arg0: pyQPanda.NoiseModel, arg1: pyQPanda.GateType, arg2: float, arg3: float, arg4: float) -> None

        Set the noise model for a specific gate type with multiple noise parameters in the quantum virtual machine.

        Args:
             noise_model: The noise model to be applied.

             gate_type: The type of gate for which the noise model is relevant.

             noise_level1: A double representing the first level of noise to apply.

             noise_level2: A double representing the second level of noise to apply.

             noise_level3: A double representing the third level of noise to apply.

        Returns:
             None, as the function configures the noise model in place for the specified gate type with the given noise parameters.


        7. set_noise_model(self: pyQPanda.NoiseQVM, arg0: pyQPanda.NoiseModel, arg1: List[pyQPanda.GateType], arg2: float, arg3: float, arg4: float) -> None

        Set the noise model for multiple gate types with various noise parameters in the quantum virtual machine.

        Args:
             noise_model: The noise model to be applied.

             gate_types: A vector of gate types for which the noise model is relevant.

             noise_level1: A double representing the first level of noise to apply.

             noise_level2: A double representing the second level of noise to apply.

             noise_level3: A double representing the third level of noise to apply.

        Returns:
             None, as the function configures the noise model in place for the specified gate types with the given noise parameters.


        8. set_noise_model(self: pyQPanda.NoiseQVM, arg0: pyQPanda.NoiseModel, arg1: pyQPanda.GateType, arg2: float, arg3: float, arg4: float, arg5: pyQPanda.QVec) -> None

        Set the noise model for a specific gate type with multiple noise parameters affecting a specific qubit vector in the quantum virtual machine.

        Args:
             noise_model: The noise model to be applied.

             gate_type: The type of gate for which the noise model is relevant.

             noise_level1: A double representing the first level of noise to apply.

             noise_level2: A double representing the second level of noise to apply.

             noise_level3: A double representing the third level of noise to apply.

             qubits: A specific qubit vector (QVec) affected by the noise model.

        Returns:
             None, as the function configures the noise model in place for the specified gate type and qubit vector.


        9. set_noise_model(self: pyQPanda.NoiseQVM, arg0: pyQPanda.NoiseModel, arg1: List[pyQPanda.GateType], arg2: float, arg3: float, arg4: float, arg5: pyQPanda.QVec) -> None

        Set the noise model for multiple gate types with various noise parameters affecting a specific qubit vector in the quantum virtual machine.

        Args:
             noise_model: The noise model to be applied.

             gate_types: A vector of gate types for which the noise model is relevant.

             noise_level1: A double representing the first level of noise to apply.

             noise_level2: A double representing the second level of noise to apply.

             noise_level3: A double representing the third level of noise to apply.

             qubits: A specific qubit vector (QVec) affected by the noise model.

        Returns:
             None, as the function configures the noise model in place for the specified gate types and qubit vector.


        10. set_noise_model(self: pyQPanda.NoiseQVM, arg0: pyQPanda.NoiseModel, arg1: pyQPanda.GateType, arg2: float, arg3: float, arg4: float, arg5: List[pyQPanda.QVec]) -> None

        Set the noise model for a specific gate type with multiple noise parameters affecting a vector of qubit vectors in the quantum virtual machine.

        Args:
             noise_model: The noise model to be applied.

             gate_type: The type of gate for which the noise model is relevant.

             noise_level1: A double representing the first level of noise to apply.

             noise_level2: A double representing the second level of noise to apply.

             noise_level3: A double representing the third level of noise to apply.

             qubits_list: A vector of qubit vectors (QVec) affected by the noise model.

        Returns:
             None, as the function configures the noise model in place for the specified gate type and qubit vectors.

        """
    @overload
    def set_noise_model(self, arg0: NoiseModel, arg1: list[GateType], arg2: float, arg3: float, arg4: float, arg5: QVec) -> None:
        """set_noise_model(*args, **kwargs)
        Overloaded function.

        1. set_noise_model(self: pyQPanda.NoiseQVM, arg0: pyQPanda.NoiseModel, arg1: pyQPanda.GateType, arg2: float) -> None

        Set the noise model for the quantum virtual machine.

        Args:
             noise_model: The noise model to be applied.

             gate_type: The type of gate for which the noise model is relevant.

             noise_level: A double representing the level of noise to apply.

        Returns:
             None, as the function configures the noise model in place.


        2. set_noise_model(self: pyQPanda.NoiseQVM, arg0: pyQPanda.NoiseModel, arg1: List[pyQPanda.GateType], arg2: float) -> None

        Set the noise model for multiple gate types in the quantum virtual machine.

        Args:
             noise_model: The noise model to be applied.

             gate_types: A vector of gate types for which the noise model is relevant.

             noise_level: A double representing the level of noise to apply.

        Returns:
             None, as the function configures the noise model in place for the specified gate types.


        3. set_noise_model(self: pyQPanda.NoiseQVM, arg0: pyQPanda.NoiseModel, arg1: pyQPanda.GateType, arg2: float, arg3: pyQPanda.QVec) -> None

        Set the noise model for a specific gate type and qubit vector in the quantum virtual machine.

        Args:
             noise_model: The noise model to be applied.

             gate_type: The type of gate for which the noise model is relevant.

             noise_level: A double representing the level of noise to apply.

             qubits: A vector of qubits (QVec) affected by the noise model.

        Returns:
             None, as the function configures the noise model in place for the specified gate type and qubits.


        4. set_noise_model(self: pyQPanda.NoiseQVM, arg0: pyQPanda.NoiseModel, arg1: List[pyQPanda.GateType], arg2: float, arg3: pyQPanda.QVec) -> None

        Set the noise model for multiple gate types and a specific qubit vector in the quantum virtual machine.

        Args:
             noise_model: The noise model to be applied.

             gate_types: A vector of gate types for which the noise model is relevant.

             noise_level: A double representing the level of noise to apply.

             qubits: A vector of qubits (QVec) affected by the noise model.

        Returns:
             None, as the function configures the noise model in place for the specified gate types and qubits.


        5. set_noise_model(self: pyQPanda.NoiseQVM, arg0: pyQPanda.NoiseModel, arg1: pyQPanda.GateType, arg2: float, arg3: List[pyQPanda.QVec]) -> None

        Set the noise model for a specific gate type and multiple qubit vectors in the quantum virtual machine.

        Args:
             noise_model: The noise model to be applied.

             gate_type: The type of gate for which the noise model is relevant.

             noise_level: A double representing the level of noise to apply.

             qubits: A vector of qubit vectors (std::vector<QVec>) affected by the noise model.

        Returns:
             None, as the function configures the noise model in place for the specified gate type and qubit vectors.


        6. set_noise_model(self: pyQPanda.NoiseQVM, arg0: pyQPanda.NoiseModel, arg1: pyQPanda.GateType, arg2: float, arg3: float, arg4: float) -> None

        Set the noise model for a specific gate type with multiple noise parameters in the quantum virtual machine.

        Args:
             noise_model: The noise model to be applied.

             gate_type: The type of gate for which the noise model is relevant.

             noise_level1: A double representing the first level of noise to apply.

             noise_level2: A double representing the second level of noise to apply.

             noise_level3: A double representing the third level of noise to apply.

        Returns:
             None, as the function configures the noise model in place for the specified gate type with the given noise parameters.


        7. set_noise_model(self: pyQPanda.NoiseQVM, arg0: pyQPanda.NoiseModel, arg1: List[pyQPanda.GateType], arg2: float, arg3: float, arg4: float) -> None

        Set the noise model for multiple gate types with various noise parameters in the quantum virtual machine.

        Args:
             noise_model: The noise model to be applied.

             gate_types: A vector of gate types for which the noise model is relevant.

             noise_level1: A double representing the first level of noise to apply.

             noise_level2: A double representing the second level of noise to apply.

             noise_level3: A double representing the third level of noise to apply.

        Returns:
             None, as the function configures the noise model in place for the specified gate types with the given noise parameters.


        8. set_noise_model(self: pyQPanda.NoiseQVM, arg0: pyQPanda.NoiseModel, arg1: pyQPanda.GateType, arg2: float, arg3: float, arg4: float, arg5: pyQPanda.QVec) -> None

        Set the noise model for a specific gate type with multiple noise parameters affecting a specific qubit vector in the quantum virtual machine.

        Args:
             noise_model: The noise model to be applied.

             gate_type: The type of gate for which the noise model is relevant.

             noise_level1: A double representing the first level of noise to apply.

             noise_level2: A double representing the second level of noise to apply.

             noise_level3: A double representing the third level of noise to apply.

             qubits: A specific qubit vector (QVec) affected by the noise model.

        Returns:
             None, as the function configures the noise model in place for the specified gate type and qubit vector.


        9. set_noise_model(self: pyQPanda.NoiseQVM, arg0: pyQPanda.NoiseModel, arg1: List[pyQPanda.GateType], arg2: float, arg3: float, arg4: float, arg5: pyQPanda.QVec) -> None

        Set the noise model for multiple gate types with various noise parameters affecting a specific qubit vector in the quantum virtual machine.

        Args:
             noise_model: The noise model to be applied.

             gate_types: A vector of gate types for which the noise model is relevant.

             noise_level1: A double representing the first level of noise to apply.

             noise_level2: A double representing the second level of noise to apply.

             noise_level3: A double representing the third level of noise to apply.

             qubits: A specific qubit vector (QVec) affected by the noise model.

        Returns:
             None, as the function configures the noise model in place for the specified gate types and qubit vector.


        10. set_noise_model(self: pyQPanda.NoiseQVM, arg0: pyQPanda.NoiseModel, arg1: pyQPanda.GateType, arg2: float, arg3: float, arg4: float, arg5: List[pyQPanda.QVec]) -> None

        Set the noise model for a specific gate type with multiple noise parameters affecting a vector of qubit vectors in the quantum virtual machine.

        Args:
             noise_model: The noise model to be applied.

             gate_type: The type of gate for which the noise model is relevant.

             noise_level1: A double representing the first level of noise to apply.

             noise_level2: A double representing the second level of noise to apply.

             noise_level3: A double representing the third level of noise to apply.

             qubits_list: A vector of qubit vectors (QVec) affected by the noise model.

        Returns:
             None, as the function configures the noise model in place for the specified gate type and qubit vectors.

        """
    @overload
    def set_noise_model(self, arg0: NoiseModel, arg1: GateType, arg2: float, arg3: float, arg4: float, arg5: list[QVec]) -> None:
        """set_noise_model(*args, **kwargs)
        Overloaded function.

        1. set_noise_model(self: pyQPanda.NoiseQVM, arg0: pyQPanda.NoiseModel, arg1: pyQPanda.GateType, arg2: float) -> None

        Set the noise model for the quantum virtual machine.

        Args:
             noise_model: The noise model to be applied.

             gate_type: The type of gate for which the noise model is relevant.

             noise_level: A double representing the level of noise to apply.

        Returns:
             None, as the function configures the noise model in place.


        2. set_noise_model(self: pyQPanda.NoiseQVM, arg0: pyQPanda.NoiseModel, arg1: List[pyQPanda.GateType], arg2: float) -> None

        Set the noise model for multiple gate types in the quantum virtual machine.

        Args:
             noise_model: The noise model to be applied.

             gate_types: A vector of gate types for which the noise model is relevant.

             noise_level: A double representing the level of noise to apply.

        Returns:
             None, as the function configures the noise model in place for the specified gate types.


        3. set_noise_model(self: pyQPanda.NoiseQVM, arg0: pyQPanda.NoiseModel, arg1: pyQPanda.GateType, arg2: float, arg3: pyQPanda.QVec) -> None

        Set the noise model for a specific gate type and qubit vector in the quantum virtual machine.

        Args:
             noise_model: The noise model to be applied.

             gate_type: The type of gate for which the noise model is relevant.

             noise_level: A double representing the level of noise to apply.

             qubits: A vector of qubits (QVec) affected by the noise model.

        Returns:
             None, as the function configures the noise model in place for the specified gate type and qubits.


        4. set_noise_model(self: pyQPanda.NoiseQVM, arg0: pyQPanda.NoiseModel, arg1: List[pyQPanda.GateType], arg2: float, arg3: pyQPanda.QVec) -> None

        Set the noise model for multiple gate types and a specific qubit vector in the quantum virtual machine.

        Args:
             noise_model: The noise model to be applied.

             gate_types: A vector of gate types for which the noise model is relevant.

             noise_level: A double representing the level of noise to apply.

             qubits: A vector of qubits (QVec) affected by the noise model.

        Returns:
             None, as the function configures the noise model in place for the specified gate types and qubits.


        5. set_noise_model(self: pyQPanda.NoiseQVM, arg0: pyQPanda.NoiseModel, arg1: pyQPanda.GateType, arg2: float, arg3: List[pyQPanda.QVec]) -> None

        Set the noise model for a specific gate type and multiple qubit vectors in the quantum virtual machine.

        Args:
             noise_model: The noise model to be applied.

             gate_type: The type of gate for which the noise model is relevant.

             noise_level: A double representing the level of noise to apply.

             qubits: A vector of qubit vectors (std::vector<QVec>) affected by the noise model.

        Returns:
             None, as the function configures the noise model in place for the specified gate type and qubit vectors.


        6. set_noise_model(self: pyQPanda.NoiseQVM, arg0: pyQPanda.NoiseModel, arg1: pyQPanda.GateType, arg2: float, arg3: float, arg4: float) -> None

        Set the noise model for a specific gate type with multiple noise parameters in the quantum virtual machine.

        Args:
             noise_model: The noise model to be applied.

             gate_type: The type of gate for which the noise model is relevant.

             noise_level1: A double representing the first level of noise to apply.

             noise_level2: A double representing the second level of noise to apply.

             noise_level3: A double representing the third level of noise to apply.

        Returns:
             None, as the function configures the noise model in place for the specified gate type with the given noise parameters.


        7. set_noise_model(self: pyQPanda.NoiseQVM, arg0: pyQPanda.NoiseModel, arg1: List[pyQPanda.GateType], arg2: float, arg3: float, arg4: float) -> None

        Set the noise model for multiple gate types with various noise parameters in the quantum virtual machine.

        Args:
             noise_model: The noise model to be applied.

             gate_types: A vector of gate types for which the noise model is relevant.

             noise_level1: A double representing the first level of noise to apply.

             noise_level2: A double representing the second level of noise to apply.

             noise_level3: A double representing the third level of noise to apply.

        Returns:
             None, as the function configures the noise model in place for the specified gate types with the given noise parameters.


        8. set_noise_model(self: pyQPanda.NoiseQVM, arg0: pyQPanda.NoiseModel, arg1: pyQPanda.GateType, arg2: float, arg3: float, arg4: float, arg5: pyQPanda.QVec) -> None

        Set the noise model for a specific gate type with multiple noise parameters affecting a specific qubit vector in the quantum virtual machine.

        Args:
             noise_model: The noise model to be applied.

             gate_type: The type of gate for which the noise model is relevant.

             noise_level1: A double representing the first level of noise to apply.

             noise_level2: A double representing the second level of noise to apply.

             noise_level3: A double representing the third level of noise to apply.

             qubits: A specific qubit vector (QVec) affected by the noise model.

        Returns:
             None, as the function configures the noise model in place for the specified gate type and qubit vector.


        9. set_noise_model(self: pyQPanda.NoiseQVM, arg0: pyQPanda.NoiseModel, arg1: List[pyQPanda.GateType], arg2: float, arg3: float, arg4: float, arg5: pyQPanda.QVec) -> None

        Set the noise model for multiple gate types with various noise parameters affecting a specific qubit vector in the quantum virtual machine.

        Args:
             noise_model: The noise model to be applied.

             gate_types: A vector of gate types for which the noise model is relevant.

             noise_level1: A double representing the first level of noise to apply.

             noise_level2: A double representing the second level of noise to apply.

             noise_level3: A double representing the third level of noise to apply.

             qubits: A specific qubit vector (QVec) affected by the noise model.

        Returns:
             None, as the function configures the noise model in place for the specified gate types and qubit vector.


        10. set_noise_model(self: pyQPanda.NoiseQVM, arg0: pyQPanda.NoiseModel, arg1: pyQPanda.GateType, arg2: float, arg3: float, arg4: float, arg5: List[pyQPanda.QVec]) -> None

        Set the noise model for a specific gate type with multiple noise parameters affecting a vector of qubit vectors in the quantum virtual machine.

        Args:
             noise_model: The noise model to be applied.

             gate_type: The type of gate for which the noise model is relevant.

             noise_level1: A double representing the first level of noise to apply.

             noise_level2: A double representing the second level of noise to apply.

             noise_level3: A double representing the third level of noise to apply.

             qubits_list: A vector of qubit vectors (QVec) affected by the noise model.

        Returns:
             None, as the function configures the noise model in place for the specified gate type and qubit vectors.

        """
    def set_readout_error(self, probs_list: list[list[float]], qubits: QVec = ...) -> None:
        """set_readout_error(self: pyQPanda.NoiseQVM, probs_list: List[List[float]], qubits: pyQPanda.QVec = <pyQPanda.QVec object at 0x0000028568271B30>) -> None

        Set a readout error model for the quantum virtual machine.

        Args:
             probs_list: A list of probabilities for readout errors associated with each qubit.

             qubits: A vector of qubits (QVec) for which the readout error model applies. Defaults to all qubits if not specified.

        Returns:
             None, as this function configures the readout error model in place for the specified qubits.

        """
    def set_reset_error(self, p0: float, p1: float, qubits: QVec = ...) -> None:
        """set_reset_error(self: pyQPanda.NoiseQVM, p0: float, p1: float, qubits: pyQPanda.QVec = <pyQPanda.QVec object at 0x0000028568270EB0>) -> None

        Set a reset error model for the quantum virtual machine.

        Args:
             p0: Probability of the qubit resetting to state 0.

             p1: Probability of the qubit resetting to state 1.

             qubits: A vector of qubits (QVec) for which the reset error model applies. Defaults to all qubits if not specified.

        Returns:
             None, as this function configures the reset error model in place for the specified qubits.

        """
    def set_rotation_error(self, arg0: float) -> None:
        """set_rotation_error(self: pyQPanda.NoiseQVM, arg0: float) -> None

        Set a rotation error model for the quantum virtual machine.

        Args:
             None specified in the function signature, but typically would include error parameters for the rotation.

        Returns:
             None, as this function configures the rotation error model in place for the quantum operations.

        """

class Optimizer:
    def __init__(self, *args, **kwargs) -> None:
        """Initialize self.  See help(type(self)) for accurate signature."""
    def get_loss(self) -> float:
        """get_loss(self: pyQPanda.Optimizer) -> float"""
    def get_variables(self) -> list[var]:
        """get_variables(self: pyQPanda.Optimizer) -> List[pyQPanda.var]"""
    def run(self, arg0: list[var], arg1: int) -> bool:
        """run(self: pyQPanda.Optimizer, arg0: List[pyQPanda.var], arg1: int) -> bool"""

class OptimizerFactory:
    def __init__(self) -> None:
        """__init__(self: pyQPanda.OptimizerFactory) -> None"""
    @overload
    def makeOptimizer(self) -> AbstractOptimizer:
        """makeOptimizer(*args, **kwargs)
        Overloaded function.

        1. makeOptimizer(self: pyQPanda.OptimizerType) -> pyQPanda.AbstractOptimizer

        Create an optimizer of the specified type.

        Args:
             optimizer_type: An instance of OptimizerType indicating the desired optimizer.

        Returns:
             An instance of the created optimizer.


        2. makeOptimizer(self: str) -> pyQPanda.AbstractOptimizer

        Create an optimizer using its name.

        Args:
             optimizer_name: A string representing the name of the desired optimizer.

        Returns:
             An instance of the created optimizer.

        """
    @overload
    def makeOptimizer(self) -> AbstractOptimizer:
        """makeOptimizer(*args, **kwargs)
        Overloaded function.

        1. makeOptimizer(self: pyQPanda.OptimizerType) -> pyQPanda.AbstractOptimizer

        Create an optimizer of the specified type.

        Args:
             optimizer_type: An instance of OptimizerType indicating the desired optimizer.

        Returns:
             An instance of the created optimizer.


        2. makeOptimizer(self: str) -> pyQPanda.AbstractOptimizer

        Create an optimizer using its name.

        Args:
             optimizer_name: A string representing the name of the desired optimizer.

        Returns:
             An instance of the created optimizer.

        """

class OptimizerMode:
    __members__: ClassVar[dict] = ...  # read-only
    __entries: ClassVar[dict] = ...
    def __init__(self, value: int) -> None:
        """__init__(self: pyQPanda.OptimizerMode, value: int) -> None"""
    def __eq__(self, other: object) -> bool:
        """__eq__(self: object, other: object) -> bool"""
    def __hash__(self) -> int:
        """__hash__(self: object) -> int"""
    def __index__(self) -> int:
        """__index__(self: pyQPanda.OptimizerMode) -> int"""
    def __int__(self) -> int:
        """__int__(self: pyQPanda.OptimizerMode) -> int"""
    def __ne__(self, other: object) -> bool:
        """__ne__(self: object, other: object) -> bool"""
    @property
    def name(self) -> str: ...
    @property
    def value(self) -> int: ...

class OptimizerType:
    __members__: ClassVar[dict] = ...  # read-only
    GRADIENT: ClassVar[OptimizerType] = ...
    NELDER_MEAD: ClassVar[OptimizerType] = ...
    POWELL: ClassVar[OptimizerType] = ...
    __entries: ClassVar[dict] = ...
    def __init__(self, value: int) -> None:
        """__init__(self: pyQPanda.OptimizerType, value: int) -> None"""
    def __eq__(self, other: object) -> bool:
        """__eq__(self: object, other: object) -> bool"""
    def __ge__(self, other: object) -> bool:
        """__ge__(self: object, other: object) -> bool"""
    def __gt__(self, other: object) -> bool:
        """__gt__(self: object, other: object) -> bool"""
    def __hash__(self) -> int:
        """__hash__(self: object) -> int"""
    def __index__(self) -> int:
        """__index__(self: pyQPanda.OptimizerType) -> int"""
    def __int__(self) -> int:
        """__int__(self: pyQPanda.OptimizerType) -> int"""
    def __le__(self, other: object) -> bool:
        """__le__(self: object, other: object) -> bool"""
    def __lt__(self, other: object) -> bool:
        """__lt__(self: object, other: object) -> bool"""
    def __ne__(self, other: object) -> bool:
        """__ne__(self: object, other: object) -> bool"""
    @property
    def name(self) -> str: ...
    @property
    def value(self) -> int: ...

class OriginCMem:
    def __init__(self) -> None:
        """__init__(self: pyQPanda.OriginCMem) -> None

        Create an instance of OriginCMem.
        This constructor returns a singleton instance of OriginCMem.

        """
    @overload
    def Allocate_CBit(self) -> CBit:
        """Allocate_CBit(*args, **kwargs)
        Overloaded function.

        1. Allocate_CBit(self: pyQPanda.OriginCMem) -> pyQPanda.CBit

        Allocate a classical bit.
        This method allocates a new classical bit and returns a reference to it.


        2. Allocate_CBit(self: pyQPanda.OriginCMem, cbit_num: int) -> pyQPanda.CBit

        Allocate a specified number of classical bits.

        Args:
             cbit_num: The number of classical bits to allocate.

        Returns:
             A reference to the allocated classical bits.

        """
    @overload
    def Allocate_CBit(self, cbit_num: int) -> CBit:
        """Allocate_CBit(*args, **kwargs)
        Overloaded function.

        1. Allocate_CBit(self: pyQPanda.OriginCMem) -> pyQPanda.CBit

        Allocate a classical bit.
        This method allocates a new classical bit and returns a reference to it.


        2. Allocate_CBit(self: pyQPanda.OriginCMem, cbit_num: int) -> pyQPanda.CBit

        Allocate a specified number of classical bits.

        Args:
             cbit_num: The number of classical bits to allocate.

        Returns:
             A reference to the allocated classical bits.

        """
    def Free_CBit(self, cbit: CBit) -> None:
        """Free_CBit(self: pyQPanda.OriginCMem, cbit: pyQPanda.CBit) -> None

        Free a previously allocated classical bit.

        Args:
             cbit: The classical bit to be freed.
        """
    @overload
    def cAlloc(self) -> CBit:
        """cAlloc(*args, **kwargs)
        Overloaded function.

        1. cAlloc(self: pyQPanda.OriginCMem) -> pyQPanda.CBit

        Allocate memory for classical bits.
        This method initializes or resets the memory allocation for classical bits.


        2. cAlloc(self: pyQPanda.OriginCMem, arg0: int) -> pyQPanda.CBit

        Allocate memory for classical bits.
        This method initializes or resets the memory allocation for classical bits.

        """
    @overload
    def cAlloc(self, arg0: int) -> CBit:
        """cAlloc(*args, **kwargs)
        Overloaded function.

        1. cAlloc(self: pyQPanda.OriginCMem) -> pyQPanda.CBit

        Allocate memory for classical bits.
        This method initializes or resets the memory allocation for classical bits.


        2. cAlloc(self: pyQPanda.OriginCMem, arg0: int) -> pyQPanda.CBit

        Allocate memory for classical bits.
        This method initializes or resets the memory allocation for classical bits.

        """
    def cAlloc_many(self, count: int) -> list[ClassicalCondition]:
        """cAlloc_many(self: pyQPanda.OriginCMem, count: int) -> List[pyQPanda.ClassicalCondition]

        Allocate memory for multiple classical bits.

        Args:
             count: The number of classical bits to allocate.

        """
    def cFree(self, classical_cond: ClassicalCondition) -> None:
        """cFree(self: pyQPanda.OriginCMem, classical_cond: pyQPanda.ClassicalCondition) -> None

        Free the allocated memory for a classical condition.

        Args:
             classical_cond: The classical condition to be freed.

        """
    @overload
    def cFree_all(self, classical_cond_list: list[ClassicalCondition]) -> None:
        """cFree_all(*args, **kwargs)
        Overloaded function.

        1. cFree_all(self: pyQPanda.OriginCMem, classical_cond_list: List[pyQPanda.ClassicalCondition]) -> None

        Free memory for a list of classical conditions.

        Args:
             classical_cond_list: A vector of classical conditions to be freed.


        2. cFree_all(self: pyQPanda.OriginCMem) -> None

        Free all allocated classical memory.
        This method releases all memory associated with classical conditions.

        """
    @overload
    def cFree_all(self) -> None:
        """cFree_all(*args, **kwargs)
        Overloaded function.

        1. cFree_all(self: pyQPanda.OriginCMem, classical_cond_list: List[pyQPanda.ClassicalCondition]) -> None

        Free memory for a list of classical conditions.

        Args:
             classical_cond_list: A vector of classical conditions to be freed.


        2. cFree_all(self: pyQPanda.OriginCMem) -> None

        Free all allocated classical memory.
        This method releases all memory associated with classical conditions.

        """
    def clearAll(self) -> None:
        """clearAll(self: pyQPanda.OriginCMem) -> None

        Clear all allocated classical bits.
        This method releases all resources associated with classical bits.

        """
    def getIdleMem(self) -> int:
        """getIdleMem(self: pyQPanda.OriginCMem) -> int

        Get the amount of idle memory currently available.

        Returns:
             The amount of idle memory in terms of qubits.

        """
    def getMaxMem(self) -> int:
        """getMaxMem(self: pyQPanda.OriginCMem) -> int

        Get the maximum memory capacity.

        Returns:
             The maximum memory capacity in terms of qubits.
        """
    def get_allocate_cbits(self) -> list[ClassicalCondition]:
        """get_allocate_cbits(self: pyQPanda.OriginCMem) -> List[pyQPanda.ClassicalCondition]

        Retrieve allocated classical bits.
        Returns a vector of ClassicalCondition representing the allocated cbits.

        """
    def get_capacity(self) -> int:
        """get_capacity(self: pyQPanda.OriginCMem) -> int

        Get the capacity of the memory.

        Returns:
             The total capacity of the memory in terms of qubits.

        """
    def get_cbit_by_addr(self, cbit_addr: int) -> CBit:
        """get_cbit_by_addr(self: pyQPanda.OriginCMem, cbit_addr: int) -> pyQPanda.CBit

        Get a classical bit by its address.

        Args:
             cbit_addr: The address of the classical bit.

        Returns:
             A reference to the classical bit associated with the given address
        .
        """
    def set_capacity(self, arg0: int) -> None:
        """set_capacity(self: pyQPanda.OriginCMem, arg0: int) -> None

        Set the capacity of the memory.

        Args:
             capacity: The new capacity for the memory in terms of qubits.

        """

class OriginCollection:
    @overload
    def __init__(self) -> None:
        """__init__(*args, **kwargs)
        Overloaded function.

        1. __init__(self: pyQPanda.OriginCollection) -> None

        2. __init__(self: pyQPanda.OriginCollection, file_name: str) -> None

        Construct a new OriginCollection by reading a JSON file.
        This function initializes the OriginCollection with data from the specified JSON file.

        Args:
             file_name: The path to the JSON file to read.

        Returns:
             An instance of OriginCollection.


        3. __init__(self: pyQPanda.OriginCollection, arg0: pyQPanda.OriginCollection) -> None

        Construct a new OriginCollection by copying an existing instance.
        This function creates a new OriginCollection as a copy of the provided instance.

        Args:
             other: The existing OriginCollection instance to copy.

        Returns:
             An instance of OriginCollection.

        """
    @overload
    def __init__(self, file_name: str) -> None:
        """__init__(*args, **kwargs)
        Overloaded function.

        1. __init__(self: pyQPanda.OriginCollection) -> None

        2. __init__(self: pyQPanda.OriginCollection, file_name: str) -> None

        Construct a new OriginCollection by reading a JSON file.
        This function initializes the OriginCollection with data from the specified JSON file.

        Args:
             file_name: The path to the JSON file to read.

        Returns:
             An instance of OriginCollection.


        3. __init__(self: pyQPanda.OriginCollection, arg0: pyQPanda.OriginCollection) -> None

        Construct a new OriginCollection by copying an existing instance.
        This function creates a new OriginCollection as a copy of the provided instance.

        Args:
             other: The existing OriginCollection instance to copy.

        Returns:
             An instance of OriginCollection.

        """
    @overload
    def __init__(self, arg0: OriginCollection) -> None:
        """__init__(*args, **kwargs)
        Overloaded function.

        1. __init__(self: pyQPanda.OriginCollection) -> None

        2. __init__(self: pyQPanda.OriginCollection, file_name: str) -> None

        Construct a new OriginCollection by reading a JSON file.
        This function initializes the OriginCollection with data from the specified JSON file.

        Args:
             file_name: The path to the JSON file to read.

        Returns:
             An instance of OriginCollection.


        3. __init__(self: pyQPanda.OriginCollection, arg0: pyQPanda.OriginCollection) -> None

        Construct a new OriginCollection by copying an existing instance.
        This function creates a new OriginCollection as a copy of the provided instance.

        Args:
             other: The existing OriginCollection instance to copy.

        Returns:
             An instance of OriginCollection.

        """
    def getFilePath(self) -> str:
        """getFilePath(self: pyQPanda.OriginCollection) -> str

        Retrieve the file path associated with the OriginCollection.
        This function returns the path to the file linked to the collection.

        Returns:
             A string containing the file path.

        """
    def getJsonString(self) -> str:
        """getJsonString(self: pyQPanda.OriginCollection) -> str

        Retrieve the JSON string representation of the OriginCollection.
        This function converts the collection's data into a JSON format string.

        Returns:
             A string containing the JSON representation of the collection.

        """
    def getKeyVector(self) -> list[str]:
        """getKeyVector(self: pyQPanda.OriginCollection) -> List[str]

        Retrieve the vector of keys associated with the OriginCollection.
        This function returns a vector containing all the keys in the collection.

        Returns:
             A vector of keys.

        """
    def getValue(self, key_name: str) -> list[str]:
        """getValue(self: pyQPanda.OriginCollection, key_name: str) -> List[str]

        Get the value associated with the specified key name.
        This function retrieves the value stored in the OriginCollection for the given key.

        Args:
             key_name: The name of the key whose value is to be retrieved.

        Returns:
             The value associated with the specified key.

        """
    @overload
    def getValueByKey(self, key_value: str) -> str:
        """getValueByKey(*args, **kwargs)
        Overloaded function.

        1. getValueByKey(self: pyQPanda.OriginCollection, key_value: str) -> str

        Get the value associated with the specified key value.
        This function retrieves the value from the OriginCollection based on the provided key.

        Args:
             key_value: The key whose corresponding value is to be retrieved.

        Returns:
             The value associated with the specified key value.


        2. getValueByKey(self: pyQPanda.OriginCollection, key_value: int) -> str

        Retrieve the value associated with a specified key.
        This function returns the value that corresponds to the given key.

        Args:
             key_value: The key for which to retrieve the associated value.

        Returns:
             The value associated with the specified key.

        """
    @overload
    def getValueByKey(self, key_value: int) -> str:
        """getValueByKey(*args, **kwargs)
        Overloaded function.

        1. getValueByKey(self: pyQPanda.OriginCollection, key_value: str) -> str

        Get the value associated with the specified key value.
        This function retrieves the value from the OriginCollection based on the provided key.

        Args:
             key_value: The key whose corresponding value is to be retrieved.

        Returns:
             The value associated with the specified key value.


        2. getValueByKey(self: pyQPanda.OriginCollection, key_value: int) -> str

        Retrieve the value associated with a specified key.
        This function returns the value that corresponds to the given key.

        Args:
             key_value: The key for which to retrieve the associated value.

        Returns:
             The value associated with the specified key.

        """
    def insertValue(self, key: str, *args) -> None:
        """insertValue(self: pyQPanda.OriginCollection, key: str, *args) -> None

        Insert values into the OriginCollection under the specified key.
        This function adds the first value associated with the provided key
        and then inserts additional values from the provided arguments.

        Args:
             key: The key under which to insert the values.

             args: A variable number of values to be inserted.

        Returns:
             None.

        """
    def open(self, file_name: str) -> bool:
        """open(self: pyQPanda.OriginCollection, file_name: str) -> bool

        Open and read the JSON file at the specified path.
        This function reads the contents of the JSON file provided.

        Args:
             file_name: The path to the JSON file to be read.

        Returns:
             None.

        """
    def setNames(self, *args) -> None:
        """setNames(self: pyQPanda.OriginCollection, *args) -> None

        Set names in the OriginCollection.
        This function accepts a variable number of name arguments and sets them in the OriginCollection.

        Args:
             args: A variable number of name strings to be set.

        Returns:
             None.

        """
    def write(self) -> bool:
        """write(self: pyQPanda.OriginCollection) -> bool

        Write the current data to a JSON file.
        This function saves the current contents to a specified JSON file.

        Args:
             None.

        Returns:
             None.

        """

class OriginQubitPool:
    def __init__(self) -> None:
        """__init__(self: pyQPanda.OriginQubitPool) -> None

        Initialize the OriginQubitPool singleton instance.

        Returns:
             A reference to the existing OriginQubitPool instance.

        """
    def allocateQubitThroughPhyAddress(self, qubit_addr: int) -> Qubit:
        """allocateQubitThroughPhyAddress(self: pyQPanda.OriginQubitPool, qubit_addr: int) -> pyQPanda.Qubit

        Allocate a qubit using its physical address.

        Args:
             qubit_addr: The physical address of the qubit to allocate.

        Returns:
             A reference to the allocated qubit.

        """
    def allocateQubitThroughVirAddress(self, qubit_num: int) -> Qubit:
        """allocateQubitThroughVirAddress(self: pyQPanda.OriginQubitPool, qubit_num: int) -> pyQPanda.Qubit

        Allocate a qubit using its virtual address.

        Args:
             qubit_num: The virtual address of the qubit to allocate.

        Returns:
             A reference to the allocated qubit.

        """
    def clearAll(self) -> None:
        """clearAll(self: pyQPanda.OriginQubitPool) -> None

        Clear all qubits from the OriginQubitPool.
        This method removes all qubits, resetting the pool to its initial state.

        """
    def getIdleQubit(self) -> int:
        """getIdleQubit(self: pyQPanda.OriginQubitPool) -> int

        Retrieve an idle qubit from the OriginQubitPool.

        Returns:
             An idle qubit if available, otherwise may return a null reference or indicate no idle qubits.

        """
    def getMaxQubit(self) -> int:
        """getMaxQubit(self: pyQPanda.OriginQubitPool) -> int

        Retrieve the maximum qubit from the OriginQubitPool.

        Returns:
             The maximum qubit available in the pool.

        """
    def getPhysicalQubitAddr(self, qubit: Qubit) -> int:
        """getPhysicalQubitAddr(self: pyQPanda.OriginQubitPool, qubit: pyQPanda.Qubit) -> int

        Retrieve the physical address of a specified qubit.

        Args:
             qubit: The qubit for which to retrieve the physical address.

        Returns:
             The physical address of the specified qubit.

        """
    def getVirtualQubitAddress(self, qubit: Qubit) -> int:
        """getVirtualQubitAddress(self: pyQPanda.OriginQubitPool, qubit: pyQPanda.Qubit) -> int

        Retrieve the virtual address of a specified qubit.

        Args:
             qubit: The qubit for which to retrieve the virtual address.

        Returns:
             The virtual address of the specified qubit.

        """
    def get_allocate_qubits(self) -> QVec:
        """get_allocate_qubits(self: pyQPanda.OriginQubitPool) -> pyQPanda.QVec

        Retrieve currently allocated qubits.

        Returns:
             A reference to the vector of currently allocated qubits.

        """
    def get_capacity(self) -> int:
        """get_capacity(self: pyQPanda.OriginQubitPool) -> int

        Get the capacity of the OriginQubitPool.

        Returns:
             An integer representing the capacity of the pool.

        """
    def get_max_usedqubit_addr(self) -> int:
        """get_max_usedqubit_addr(self: pyQPanda.OriginQubitPool) -> int

        Retrieve the address of the maximum used qubit in the OriginQubitPool.

        Returns:
             The address of the maximum used qubit, or an indication if no qubits are in use.

        """
    def get_qubit_by_addr(self, qubit_addr: int) -> Qubit:
        """get_qubit_by_addr(self: pyQPanda.OriginQubitPool, qubit_addr: int) -> pyQPanda.Qubit

        Retrieve a qubit from the pool using its address.

        Args:
             qubit_addr: The address of the qubit to retrieve.
        Returns:
             A reference to the requested qubit.

        """
    def qAlloc(self) -> Qubit:
        """qAlloc(self: pyQPanda.OriginQubitPool) -> pyQPanda.Qubit

        Allocate a qubit.

        Returns:
             A reference to the allocated qubit.

        """
    def qAlloc_many(self, qubit_num: int) -> list[Qubit]:
        """qAlloc_many(self: pyQPanda.OriginQubitPool, qubit_num: int) -> List[pyQPanda.Qubit]

        Allocate a list of qubits.

        Args:
             qubit_num: The number of qubits to allocate.

        Returns:
             A reference to the vector of allocated qubits.

        """
    def qFree(self, arg0: Qubit) -> None:
        """qFree(self: pyQPanda.OriginQubitPool, arg0: pyQPanda.Qubit) -> None

        Free a previously allocated qubit.

        Args:
             qubit: The qubit to be freed.
        """
    @overload
    def qFree_all(self, arg0: QVec) -> None:
        """qFree_all(*args, **kwargs)
        Overloaded function.

        1. qFree_all(self: pyQPanda.OriginQubitPool, arg0: pyQPanda.QVec) -> None

        Free all qubits in the specified vector.

        Args:
             qubits: A vector of qubits to be freed.


        2. qFree_all(self: pyQPanda.OriginQubitPool) -> None

        Free all allocated qubits in the pool.
        This method releases all qubits that have been allocated previously.

        """
    @overload
    def qFree_all(self) -> None:
        """qFree_all(*args, **kwargs)
        Overloaded function.

        1. qFree_all(self: pyQPanda.OriginQubitPool, arg0: pyQPanda.QVec) -> None

        Free all qubits in the specified vector.

        Args:
             qubits: A vector of qubits to be freed.


        2. qFree_all(self: pyQPanda.OriginQubitPool) -> None

        Free all allocated qubits in the pool.
        This method releases all qubits that have been allocated previously.

        """
    def set_capacity(self, arg0: int) -> None:
        """set_capacity(self: pyQPanda.OriginQubitPool, arg0: int) -> None

        Set the capacity of the OriginQubitPool.

        Args:
             capacity: An integer representing the new capacity to be set.

        """

class PartialAmpQVM(QuantumMachine):
    def __init__(self) -> None:
        """__init__(self: pyQPanda.PartialAmpQVM) -> None"""
    def get_prob_dict(self, arg0: QVec) -> dict[str, float]:
        """get_prob_dict(self: pyQPanda.PartialAmpQVM, arg0: pyQPanda.QVec) -> Dict[str, float]

        Get the measurement results as a dictionary.

        Args:
             qubit_list: A list of qubits to measure.

        Returns:
             A dictionary containing the measurement results of the quantum machine.

        """
    def init_qvm(self, backend_type: int = ...) -> None:
        """init_qvm(self: pyQPanda.PartialAmpQVM, backend_type: int = 0) -> None"""
    def pmeasure_bin_index(self, bin_index: str) -> complex:
        """pmeasure_bin_index(self: pyQPanda.PartialAmpQVM, bin_index: str) -> complex

        Get the amplitude of the quantum state for the specified bin index.

        Args:
             bin_index: A string representing the bin.

        Returns:
             A complex number representing the amplitude of the bin.

        """
    def pmeasure_dec_index(self, dec_index: str) -> complex:
        """pmeasure_dec_index(self: pyQPanda.PartialAmpQVM, dec_index: str) -> complex

        Get the amplitude of the quantum state for the specified decimal index.

        Args:
             dec_index: A string representing the decimal.

        Returns:
             A complex number representing the amplitude of the decimal.

        """
    def pmeasure_subset(self, index_list: list[str]) -> dict[str, complex]:
        """pmeasure_subset(self: pyQPanda.PartialAmpQVM, index_list: List[str]) -> Dict[str, complex]

        Get the amplitudes of the quantum state for a subset of indices.

        Args:
             index_list: A list of strings representing decimal states.

        Returns:
             A list of complex numbers representing the amplitude results.

        """
    def prob_run_dict(self, arg0: QProg, arg1: QVec) -> dict[str, float]:
        """prob_run_dict(self: pyQPanda.PartialAmpQVM, arg0: pyQPanda.QProg, arg1: pyQPanda.QVec) -> Dict[str, float]

        Run the quantum program and get the measurement results as a dictionary.

        Args:
             qprog: The quantum program to execute.

             qubit_list: A list of qubits to measure.

        Returns:
             A dictionary containing the measurement results of the quantum machine.

        """
    @overload
    def run(self, qprog: QProg, noise_model: Noise = ...) -> None:
        """run(*args, **kwargs)
        Overloaded function.

        1. run(self: pyQPanda.PartialAmpQVM, qprog: pyQPanda.QProg, noise_model: pyQPanda.Noise = NoiseModel()) -> None

        Run the quantum program.

        Args:
             qprog: The quantum program to execute.

             noise_model: An optional noise model (default is NoiseModel()).

        Returns:
             None.


        2. run(self: pyQPanda.PartialAmpQVM, qprog: pyQPanda.QCircuit, noise_model: pyQPanda.Noise = NoiseModel()) -> None

        Run the quantum program.

        Args:
             qprog: The quantum circuit to execute.

             noise_model: An optional noise model (default is NoiseModel()).

        Returns:
             None.

        """
    @overload
    def run(self, qprog: QCircuit, noise_model: Noise = ...) -> None:
        """run(*args, **kwargs)
        Overloaded function.

        1. run(self: pyQPanda.PartialAmpQVM, qprog: pyQPanda.QProg, noise_model: pyQPanda.Noise = NoiseModel()) -> None

        Run the quantum program.

        Args:
             qprog: The quantum program to execute.

             noise_model: An optional noise model (default is NoiseModel()).

        Returns:
             None.


        2. run(self: pyQPanda.PartialAmpQVM, qprog: pyQPanda.QCircuit, noise_model: pyQPanda.Noise = NoiseModel()) -> None

        Run the quantum program.

        Args:
             qprog: The quantum circuit to execute.

             noise_model: An optional noise model (default is NoiseModel()).

        Returns:
             None.

        """

class PhysicalQubit:
    def __init__(self, *args, **kwargs) -> None:
        """Initialize self.  See help(type(self)) for accurate signature."""
    def getQubitAddr(self) -> int:
        """getQubitAddr(self: pyQPanda.PhysicalQubit) -> int

        Retrieve the address of the physical qubit.

        This function returns the address of the physical qubit associated with the instance.

        Args:
             None

        Returns:
             The address of the physical qubit.

        """

class PilotNoiseParams:
    double_gate_param: float
    double_p2: float
    double_pgate: float
    noise_model: str
    single_gate_param: float
    single_p2: float
    single_pgate: float
    def __init__(self, *args, **kwargs) -> None:
        """Initialize self.  See help(type(self)) for accurate signature."""

class ProgCount:
    double_gate_layer_num: int
    double_gate_num: int
    gate_num: int
    layer_num: int
    multi_control_gate_num: int
    node_num: int
    selected_gate_nums: dict[GateType, int]
    single_gate_layer_num: int
    single_gate_num: int
    def __init__(self) -> None:
        """__init__(self: pyQPanda.ProgCount) -> None"""

class QCircuit:
    @overload
    def __init__(self) -> None:
        """__init__(*args, **kwargs)
        Overloaded function.

        1. __init__(self: pyQPanda.QCircuit) -> None

        2. __init__(self: pyQPanda.QCircuit, arg0: pyQPanda.NodeIter) -> None

        Initialize QCircuit from a node iterator.

        Args:
             iter (NodeIter&): The iterator pointing to the node.

        Returns:
             QCircuit: The initialized quantum circuit object.

        Raises:
             runtime_error: If the iterator is null or the node type is incorrect.

        """
    @overload
    def __init__(self, arg0: NodeIter) -> None:
        """__init__(*args, **kwargs)
        Overloaded function.

        1. __init__(self: pyQPanda.QCircuit) -> None

        2. __init__(self: pyQPanda.QCircuit, arg0: pyQPanda.NodeIter) -> None

        Initialize QCircuit from a node iterator.

        Args:
             iter (NodeIter&): The iterator pointing to the node.

        Returns:
             QCircuit: The initialized quantum circuit object.

        Raises:
             runtime_error: If the iterator is null or the node type is incorrect.

        """
    def begin(self) -> NodeIter:
        """begin(self: pyQPanda.QCircuit) -> pyQPanda.NodeIter

        Get an iterator to the first node in the circuit.

        Returns:
             Iterator: An iterator pointing to the first node.

        """
    def control(self, control_qubits: QVec) -> QCircuit:
        """control(self: pyQPanda.QCircuit, control_qubits: pyQPanda.QVec) -> pyQPanda.QCircuit

        Apply a control operation to the circuit.

        Args:
             control_qubits (list): A list of qubits that will act as control qubits.

        Returns:
             QCircuit: The circuit with the control operation applied.

        """
    def dagger(self) -> QCircuit:
        """dagger(self: pyQPanda.QCircuit) -> pyQPanda.QCircuit

        Compute the adjoint (dagger) of the circuit.

        Returns:
             QCircuit: The adjoint of this circuit.

        """
    def end(self) -> NodeIter:
        """end(self: pyQPanda.QCircuit) -> pyQPanda.NodeIter

        Get an iterator to the end of the circuit.

        Returns:
             Iterator: An iterator pointing to the end of the nodes.

        """
    def head(self) -> NodeIter:
        """head(self: pyQPanda.QCircuit) -> pyQPanda.NodeIter

        Get an iterator to the head of the circuit.

        Returns:
             Iterator: An iterator pointing to the head node.

        """
    @overload
    def insert(self, arg0: QCircuit) -> QCircuit:
        """insert(*args, **kwargs)
        Overloaded function.

        1. insert(self: pyQPanda.QCircuit, arg0: pyQPanda.QCircuit) -> pyQPanda.QCircuit

        Insert another QCircuit into this circuit.

        Args:
             other (QCircuit): The circuit to be inserted.

        Returns:
             QCircuit: A reference to this circuit after insertion.


        2. insert(self: pyQPanda.QCircuit, arg0: pyQPanda.QGate) -> pyQPanda.QCircuit

        Insert a QGate into this circuit.

        Args:
             gate (QGate): The gate to be inserted.

        Returns:
             QCircuit: A reference to this circuit after the gate insertion.

        """
    @overload
    def insert(self, arg0: QGate) -> QCircuit:
        """insert(*args, **kwargs)
        Overloaded function.

        1. insert(self: pyQPanda.QCircuit, arg0: pyQPanda.QCircuit) -> pyQPanda.QCircuit

        Insert another QCircuit into this circuit.

        Args:
             other (QCircuit): The circuit to be inserted.

        Returns:
             QCircuit: A reference to this circuit after insertion.


        2. insert(self: pyQPanda.QCircuit, arg0: pyQPanda.QGate) -> pyQPanda.QCircuit

        Insert a QGate into this circuit.

        Args:
             gate (QGate): The gate to be inserted.

        Returns:
             QCircuit: A reference to this circuit after the gate insertion.

        """
    def is_empty(self) -> bool:
        """is_empty(self: pyQPanda.QCircuit) -> bool

        Check if the circuit is empty.

        Returns:
             bool: True if the circuit has no gates; otherwise, False.

        """
    def last(self) -> NodeIter:
        """last(self: pyQPanda.QCircuit) -> pyQPanda.NodeIter

        Get an iterator to the last node in the circuit.

        Returns:
             Iterator: An iterator pointing to the last node.

        """
    def set_control(self, control_qubits: QVec) -> None:
        """set_control(self: pyQPanda.QCircuit, control_qubits: pyQPanda.QVec) -> None

        Set control qubits for the circuit.

        Args:
             control_qubits (list): A list of qubits to be set as control qubits.

        """
    def set_dagger(self, arg0: bool) -> None:
        """set_dagger(self: pyQPanda.QCircuit, arg0: bool) -> None

        Set the dagger property of the circuit.

        This method modifies the circuit to represent its adjoint.

        """
    @overload
    def __lshift__(self, arg0: QCircuit) -> QCircuit:
        """__lshift__(*args, **kwargs)
        Overloaded function.

        1. __lshift__(self: pyQPanda.QCircuit, arg0: pyQPanda.QCircuit) -> pyQPanda.QCircuit

        Left shift operator for QCircuit.

        Args:
             other (QCircuit): The circuit to be combined with this circuit.

        Returns:
             QCircuit: A new circuit resulting from the left shift operation.


        2. __lshift__(self: pyQPanda.QCircuit, arg0: pyQPanda.QGate) -> pyQPanda.QCircuit

        Left shift operator for QCircuit with a QGate.

        Args:
             other (QGate): The gate to be added to this circuit.

        Returns:
             QCircuit: A new circuit resulting from the left shift operation with the gate.

        """
    @overload
    def __lshift__(self, arg0: QGate) -> QCircuit:
        """__lshift__(*args, **kwargs)
        Overloaded function.

        1. __lshift__(self: pyQPanda.QCircuit, arg0: pyQPanda.QCircuit) -> pyQPanda.QCircuit

        Left shift operator for QCircuit.

        Args:
             other (QCircuit): The circuit to be combined with this circuit.

        Returns:
             QCircuit: A new circuit resulting from the left shift operation.


        2. __lshift__(self: pyQPanda.QCircuit, arg0: pyQPanda.QGate) -> pyQPanda.QCircuit

        Left shift operator for QCircuit with a QGate.

        Args:
             other (QGate): The gate to be added to this circuit.

        Returns:
             QCircuit: A new circuit resulting from the left shift operation with the gate.

        """

class QCircuitOPtimizerMode:
    __members__: ClassVar[dict] = ...  # read-only
    Merge_H_X: ClassVar[QCircuitOPtimizerMode] = ...
    Merge_RX: ClassVar[QCircuitOPtimizerMode] = ...
    Merge_RY: ClassVar[QCircuitOPtimizerMode] = ...
    Merge_RZ: ClassVar[QCircuitOPtimizerMode] = ...
    Merge_U3: ClassVar[QCircuitOPtimizerMode] = ...
    __entries: ClassVar[dict] = ...
    def __init__(self, value: int) -> None:
        """__init__(self: pyQPanda.QCircuitOPtimizerMode, value: int) -> None"""
    def __eq__(self, other: object) -> bool:
        """__eq__(self: object, other: object) -> bool"""
    def __hash__(self) -> int:
        """__hash__(self: object) -> int"""
    def __index__(self) -> int:
        """__index__(self: pyQPanda.QCircuitOPtimizerMode) -> int"""
    def __int__(self) -> int:
        """__int__(self: pyQPanda.QCircuitOPtimizerMode) -> int"""
    def __ne__(self, other: object) -> bool:
        """__ne__(self: object, other: object) -> bool"""
    def __or__(self, arg0: QCircuitOPtimizerMode) -> int:
        """__or__(self: pyQPanda.QCircuitOPtimizerMode, arg0: pyQPanda.QCircuitOPtimizerMode) -> int

        bitwise or
        """
    @property
    def name(self) -> str: ...
    @property
    def value(self) -> int: ...

class QCloudService(QuantumMachine):
    batch_compute_url: str
    batch_inquire_url: str
    big_data_batch_compute_url: str
    compute_url: str
    configuration_header_data: str
    estimate_url: str
    inquire_url: str
    measure_qubits_num: list[int]
    pqc_batch_compute_url: str
    pqc_batch_inquire_url: str
    pqc_compute_url: str
    pqc_init_url: str
    pqc_inquire_url: str
    pqc_keys_url: str
    use_compress_data: bool
    user_token: str
    def __init__(self) -> None:
        """__init__(self: pyQPanda.QCloudService) -> None"""
    def batch_cyclic_query(self, arg0: str) -> tuple[bool, list[str]]:
        """batch_cyclic_query(self: pyQPanda.QCloudService, arg0: str) -> Tuple[bool, List[str]]"""
    def build_error_mitigation(self, shots: int, chip_id: int, expectations: list[str], noise_strength: list[float], qemMethod: em_method) -> str:
        """build_error_mitigation(self: pyQPanda.QCloudService, shots: int, chip_id: int, expectations: List[str], noise_strength: List[float], qemMethod: pyQPanda.em_method) -> str"""
    def build_full_amplitude_measure(self, shots: int) -> str:
        """build_full_amplitude_measure(self: pyQPanda.QCloudService, shots: int) -> str"""
    def build_full_amplitude_pmeasure(self, qubit_vec: list[int]) -> str:
        """build_full_amplitude_pmeasure(self: pyQPanda.QCloudService, qubit_vec: List[int]) -> str"""
    def build_get_expectation(self, hamiltonian: list[tuple[dict[int, str], float]], qubits: list[int]) -> str:
        """build_get_expectation(self: pyQPanda.QCloudService, hamiltonian: List[Tuple[Dict[int, str], float]], qubits: List[int]) -> str"""
    def build_get_state_fidelity(self, shot: int, chip_id: int = ..., is_amend: bool = ..., is_mapping: bool = ..., is_optimization: bool = ...) -> str:
        """build_get_state_fidelity(self: pyQPanda.QCloudService, shot: int, chip_id: int = 2, is_amend: bool = True, is_mapping: bool = True, is_optimization: bool = True) -> str"""
    def build_get_state_tomography_density(self, shot: int, chip_id: int = ..., is_amend: bool = ..., is_mapping: bool = ..., is_optimization: bool = ...) -> str:
        """build_get_state_tomography_density(self: pyQPanda.QCloudService, shot: int, chip_id: int = 2, is_amend: bool = True, is_mapping: bool = True, is_optimization: bool = True) -> str"""
    @overload
    def build_init_object(self, prog: QProg, task_name: str = ..., task_from: int = ...) -> None:
        """build_init_object(*args, **kwargs)
        Overloaded function.

        1. build_init_object(self: pyQPanda.QCloudService, prog: pyQPanda.QProg, task_name: str = 'QPanda Experiment', task_from: int = 4) -> None

        2. build_init_object(self: pyQPanda.QCloudService, prog_str: str, task_name: str = 'QPanda Experiment', task_from: int = 4) -> None
        """
    @overload
    def build_init_object(self, prog_str: str, task_name: str = ..., task_from: int = ...) -> None:
        """build_init_object(*args, **kwargs)
        Overloaded function.

        1. build_init_object(self: pyQPanda.QCloudService, prog: pyQPanda.QProg, task_name: str = 'QPanda Experiment', task_from: int = 4) -> None

        2. build_init_object(self: pyQPanda.QCloudService, prog_str: str, task_name: str = 'QPanda Experiment', task_from: int = 4) -> None
        """
    def build_noise_measure(self, shots: int) -> str:
        """build_noise_measure(self: pyQPanda.QCloudService, shots: int) -> str"""
    def build_partial_amplitude_pmeasure(self, amplitudes: list[str]) -> str:
        """build_partial_amplitude_pmeasure(self: pyQPanda.QCloudService, amplitudes: List[str]) -> str"""
    def build_read_out_mitigation(self, shots: int, chip_id: int, expectations: list[str], noise_strength: list[float], qem_method: em_method) -> str:
        """build_read_out_mitigation(self: pyQPanda.QCloudService, shots: int, chip_id: int, expectations: List[str], noise_strength: List[float], qem_method: pyQPanda.em_method) -> str"""
    def build_real_chip_measure(self, shots: int, chip_id: int = ..., is_amend: bool = ..., is_mapping: bool = ..., is_optimization: bool = ...) -> str:
        """build_real_chip_measure(self: pyQPanda.QCloudService, shots: int, chip_id: int = 2, is_amend: bool = True, is_mapping: bool = True, is_optimization: bool = True) -> str"""
    @overload
    def build_real_chip_measure_batch(self, prog_list: list[str], shots: int, chip_id: int = ..., is_amend: bool = ..., is_mapping: bool = ..., is_optimization: bool = ..., enable_compress_check: bool = ..., batch_id: str = ..., task_from: int = ...) -> str:
        """build_real_chip_measure_batch(*args, **kwargs)
        Overloaded function.

        1. build_real_chip_measure_batch(self: pyQPanda.QCloudService, prog_list: List[str], shots: int, chip_id: int = 2, is_amend: bool = True, is_mapping: bool = True, is_optimization: bool = True, enable_compress_check: bool = False, batch_id: str = '', task_from: int = 4) -> str

        2. build_real_chip_measure_batch(self: pyQPanda.QCloudService, prog_list: List[pyQPanda.QProg], shots: int, chip_id: int = 2, is_amend: bool = True, is_mapping: bool = True, is_optimization: bool = True, enable_compress_check: bool = False, batch_id: str = '', task_from: int = 4) -> str
        """
    @overload
    def build_real_chip_measure_batch(self, prog_list: list[QProg], shots: int, chip_id: int = ..., is_amend: bool = ..., is_mapping: bool = ..., is_optimization: bool = ..., enable_compress_check: bool = ..., batch_id: str = ..., task_from: int = ...) -> str:
        """build_real_chip_measure_batch(*args, **kwargs)
        Overloaded function.

        1. build_real_chip_measure_batch(self: pyQPanda.QCloudService, prog_list: List[str], shots: int, chip_id: int = 2, is_amend: bool = True, is_mapping: bool = True, is_optimization: bool = True, enable_compress_check: bool = False, batch_id: str = '', task_from: int = 4) -> str

        2. build_real_chip_measure_batch(self: pyQPanda.QCloudService, prog_list: List[pyQPanda.QProg], shots: int, chip_id: int = 2, is_amend: bool = True, is_mapping: bool = True, is_optimization: bool = True, enable_compress_check: bool = False, batch_id: str = '', task_from: int = 4) -> str
        """
    def build_single_amplitude_pmeasure(self, amplitude: str) -> str:
        """build_single_amplitude_pmeasure(self: pyQPanda.QCloudService, amplitude: str) -> str"""
    def cyclic_query(self, arg0: str) -> tuple[bool, str]:
        """cyclic_query(self: pyQPanda.QCloudService, arg0: str) -> Tuple[bool, str]"""
    def enc_hybrid(self, pk_str: str, rdnum: str) -> tuple:
        """enc_hybrid(self: pyQPanda.QCloudService, pk_str: str, rdnum: str) -> tuple

        Perform hybrid encryption and return a tuple
        """
    def init(self, user_token: str, is_logged: bool = ...) -> None:
        """init(self: pyQPanda.QCloudService, user_token: str, is_logged: bool = False) -> None"""
    def parse_get_task_id(self, arg0: str) -> str:
        """parse_get_task_id(self: pyQPanda.QCloudService, arg0: str) -> str"""
    def query_comolex_result(self, arg0: str) -> complex:
        """query_comolex_result(self: pyQPanda.QCloudService, arg0: str) -> complex"""
    def query_prob_dict_result(self, arg0: str) -> dict[str, float]:
        """query_prob_dict_result(self: pyQPanda.QCloudService, arg0: str) -> Dict[str, float]"""
    def query_prob_dict_result_batch(self, arg0: list[str]) -> list[dict[str, float]]:
        """query_prob_dict_result_batch(self: pyQPanda.QCloudService, arg0: List[str]) -> List[Dict[str, float]]"""
    def query_prob_result(self, arg0: str) -> float:
        """query_prob_result(self: pyQPanda.QCloudService, arg0: str) -> float"""
    def query_prob_vec_result(self, arg0: str) -> list[float]:
        """query_prob_vec_result(self: pyQPanda.QCloudService, arg0: str) -> List[float]"""
    def query_qst_result(self, arg0: str) -> list[list[complex]]:
        """query_qst_result(self: pyQPanda.QCloudService, arg0: str) -> List[List[complex]]"""
    def query_state_dict_result(self, arg0: str) -> dict[str, complex]:
        """query_state_dict_result(self: pyQPanda.QCloudService, arg0: str) -> Dict[str, complex]"""
    def set_noise_model(self, arg0: NoiseModel, arg1: list[float], arg2: list[float]) -> None:
        """set_noise_model(self: pyQPanda.QCloudService, arg0: pyQPanda.NoiseModel, arg1: List[float], arg2: List[float]) -> None"""
    def set_qcloud_url(self, cloud_url: str) -> None:
        """set_qcloud_url(self: pyQPanda.QCloudService, cloud_url: str) -> None"""
    def sm4_decode(self, key: str, IV: str, enc_data: str) -> str:
        """sm4_decode(self: pyQPanda.QCloudService, key: str, IV: str, enc_data: str) -> str

        Decode data using SM4 algorithm
        """
    def sm4_encode(self, key: str, IV: str, data: str) -> str:
        """sm4_encode(self: pyQPanda.QCloudService, key: str, IV: str, data: str) -> str

        Encode data using SM4 algorithm
        """

class QCloudTaskConfig:
    chip_id: real_chip_type
    cloud_token: str
    open_amend: bool
    open_mapping: bool
    open_optimization: bool
    shots: int
    def __init__(self) -> None:
        """__init__(self: pyQPanda.QCloudTaskConfig) -> None"""

class QError:
    __members__: ClassVar[dict] = ...  # read-only
    UndefineError: ClassVar[QError] = ...
    __entries: ClassVar[dict] = ...
    destroyStateError: ClassVar[QError] = ...
    getQStateError: ClassVar[QError] = ...
    getResultError: ClassVar[QError] = ...
    initStateError: ClassVar[QError] = ...
    loadFileError: ClassVar[QError] = ...
    qErrorNone: ClassVar[QError] = ...
    qParameterError: ClassVar[QError] = ...
    qubitError: ClassVar[QError] = ...
    runProgramError: ClassVar[QError] = ...
    setComputeUnitError: ClassVar[QError] = ...
    def __init__(self, value: int) -> None:
        """__init__(self: pyQPanda.QError, value: int) -> None"""
    def __eq__(self, other: object) -> bool:
        """__eq__(self: object, other: object) -> bool"""
    def __hash__(self) -> int:
        """__hash__(self: object) -> int"""
    def __index__(self) -> int:
        """__index__(self: pyQPanda.QError) -> int"""
    def __int__(self) -> int:
        """__int__(self: pyQPanda.QError) -> int"""
    def __ne__(self, other: object) -> bool:
        """__ne__(self: object, other: object) -> bool"""
    @property
    def name(self) -> str: ...
    @property
    def value(self) -> int: ...

class QGate:
    def __init__(self, arg0: NodeIter) -> None:
        """__init__(self: pyQPanda.QGate, arg0: pyQPanda.NodeIter) -> None

        Initialize a QGate instance based on the provided iterator.
        This constructor checks the validity of the iterator and ensures it points to a valid gate node.

        Args:
             iter: A reference to a NodeIter that points to the node to be initialized.

        Returns:
             A QGate instance initialized from the gate node.

        """
    def control(self, control_qubits: QVec) -> QGate:
        """control(self: pyQPanda.QGate, control_qubits: pyQPanda.QVec) -> pyQPanda.QGate

        Get a controlled quantum gate based on the current QGate instance.
        This function creates a control version of the quantum gate using the specified control qubits.

        Args:
             control_qubits: A list of qubits that serve as control qubits for the gate.

        Returns:
             A new QGate instance representing the controlled gate.

        """
    @overload
    def dagger(self) -> QGate:
        """dagger(self: pyQPanda.QGate) -> pyQPanda.QGate

        Compute the dagger (Hermitian conjugate) of the QGate instance.
        This function returns the adjoint of the quantum gate, which is used in quantum mechanics for various calculations.

        Args:
             None

        Returns:
             A new QGate instance representing the dagger of the current gate.

        """
    @overload
    def dagger(self, Hermitianconjugate) -> Any:
        """dagger(self: pyQPanda.QGate) -> pyQPanda.QGate

        Compute the dagger (Hermitian conjugate) of the QGate instance.
        This function returns the adjoint of the quantum gate, which is used in quantum mechanics for various calculations.

        Args:
             None

        Returns:
             A new QGate instance representing the dagger of the current gate.

        """
    def gate_matrix(self) -> list[complex]:
        """gate_matrix(self: pyQPanda.QGate) -> List[complex]

        Get the matrix representation of the quantum gate.

        Args:
             qgate: The quantum gate instance.

        Returns:
             QStat: The matrix representation of the quantum gate.

        """
    def gate_type(self) -> int:
        """gate_type(self: pyQPanda.QGate) -> int

        Get the type of the quantum gate.

        Args:
             qgate: The quantum gate instance.

        Returns:
             The type of the quantum gate.

        """
    def get_control_qubit_num(self) -> int:
        """get_control_qubit_num(self: pyQPanda.QGate) -> int

        Retrieve the number of control qubits for the QGate instance.
        This function returns the count of qubits that act as control qubits for the gate.

        Args:
             None

        Returns:
             An integer representing the number of control qubits.

        """
    def get_control_qubits(self, control_qubits: QVec) -> int:
        """get_control_qubits(self: pyQPanda.QGate, control_qubits: pyQPanda.QVec) -> int

        Get the control vector from the current quantum gate node.

        Args:
             control_qubits: The control qubits output vector.

        Returns:
             int: Size of the control qubits.

        """
    def get_qubits(self, qubits: QVec) -> int:
        """get_qubits(self: pyQPanda.QGate, qubits: pyQPanda.QVec) -> int

        Get the qubit vector inside this quantum gate.

        Args:
             qubits: The qubits output vector.

        Returns:
             int: Size of the qubits.

        """
    def get_target_qubit_num(self) -> int:
        """get_target_qubit_num(self: pyQPanda.QGate) -> int

        Retrieve the number of target qubits for the QGate instance.
        This function returns the count of qubits that the quantum gate affects.

        Args:
             None

        Returns:
             An integer representing the number of target qubits.

        """
    def is_dagger(self) -> bool:
        """is_dagger(self: pyQPanda.QGate) -> bool

        Check if the QGate instance is a dagger (Hermitian conjugate) of another gate.
        This function determines whether the current gate is the adjoint of its corresponding gate.

        Args:
             None

        Returns:
             A boolean indicating whether the current gate is a dagger.

        """
    def set_control(self, arg0: QVec) -> bool:
        """set_control(self: pyQPanda.QGate, arg0: pyQPanda.QVec) -> bool

        Set the control qubits for the QGate instance.
        This function specifies which qubits will act as control qubits for the gate.

        Args:
             control_qubits: A list of qubits that will serve as control qubits.

        Returns:
             None

        """
    def set_dagger(self, arg0: bool) -> bool:
        """set_dagger(self: pyQPanda.QGate, arg0: bool) -> bool

        Set the QGate instance to be a dagger (Hermitian conjugate) of another gate.
        This function configures the current gate to represent its adjoint.

        Args:
             None

        Returns:
             None

        """

class QITE:
    def __init__(self) -> None:
        """__init__(self: pyQPanda.QITE) -> None"""
    def exec(self, is_optimization: bool = ...) -> int:
        """exec(self: pyQPanda.QITE, is_optimization: bool = True) -> int"""
    def get_all_exec_result(self, reverse: bool = ..., sort: bool = ...) -> list[list[tuple[int, float]]]:
        """get_all_exec_result(self: pyQPanda.QITE, reverse: bool = False, sort: bool = False) -> List[List[Tuple[int, float]]]"""
    def get_ansatz_list(self, *args, **kwargs):
        """get_ansatz_list(self: pyQPanda.QITE) -> List[List[QPanda::AnsatzGate]]"""
    def get_ansatz_theta_list(self) -> list[list[float]]:
        """get_ansatz_theta_list(self: pyQPanda.QITE) -> List[List[float]]"""
    def get_arbitary_cofficient(self, arg0: float) -> None:
        """get_arbitary_cofficient(self: pyQPanda.QITE, arg0: float) -> None"""
    def get_exec_result(self, reverse: bool = ..., sort: bool = ...) -> list[tuple[int, float]]:
        """get_exec_result(self: pyQPanda.QITE, reverse: bool = False, sort: bool = False) -> List[Tuple[int, float]]"""
    def get_result(self) -> list[tuple[int, float]]:
        """get_result(self: pyQPanda.QITE) -> List[Tuple[int, float]]"""
    def set_Hamiltonian(self, arg0) -> None:
        """set_Hamiltonian(self: pyQPanda.QITE, arg0: QPanda::PauliOp<std::complex<double> >) -> None"""
    def set_ansatz_gate(self, arg0) -> None:
        """set_ansatz_gate(self: pyQPanda.QITE, arg0: List[QPanda::AnsatzGate]) -> None"""
    def set_convergence_factor_Q(self, arg0: float) -> None:
        """set_convergence_factor_Q(self: pyQPanda.QITE, arg0: float) -> None"""
    def set_delta_tau(self, arg0: float) -> None:
        """set_delta_tau(self: pyQPanda.QITE, arg0: float) -> None"""
    def set_iter_num(self, arg0: int) -> None:
        """set_iter_num(self: pyQPanda.QITE, arg0: int) -> None"""
    def set_log_file(self, arg0: str) -> None:
        """set_log_file(self: pyQPanda.QITE, arg0: str) -> None"""
    def set_para_update_mode(self, arg0: UpdateMode) -> None:
        """set_para_update_mode(self: pyQPanda.QITE, arg0: pyQPanda.UpdateMode) -> None"""
    def set_pauli_matrix(self, arg0: QuantumMachine, arg1: numpy.ndarray[numpy.float64[m, n]]) -> None:
        """set_pauli_matrix(self: pyQPanda.QITE, arg0: pyQPanda.QuantumMachine, arg1: numpy.ndarray[numpy.float64[m, n]]) -> None"""
    def set_quantum_machine_type(self, arg0: QMachineType) -> None:
        """set_quantum_machine_type(self: pyQPanda.QITE, arg0: pyQPanda.QMachineType) -> None"""
    def set_upthrow_num(self, arg0: int) -> None:
        """set_upthrow_num(self: pyQPanda.QITE, arg0: int) -> None"""

class QIfProg:
    @overload
    def __init__(self, arg0: NodeIter) -> None:
        """__init__(*args, **kwargs)
        Overloaded function.

        1. __init__(self: pyQPanda.QIfProg, arg0: pyQPanda.NodeIter) -> None

        Constructor for QIfProg.

        Args:
             iter: An iterator to a node.

        Raises:
             runtime_error: If the iterator is null or the node type is incorrect.


        2. __init__(self: pyQPanda.QIfProg, classical_cond: pyQPanda.ClassicalCondition, true_branch_qprog: pyQPanda.QProg) -> None

        Constructor for initializing with a classical condition and a quantum program.

        Args:
             classical_cond: The classical condition to evaluate.

             true_branch_qprog: The quantum program to execute if the condition is true.


        3. __init__(self: pyQPanda.QIfProg, classical_cond: pyQPanda.ClassicalCondition, true_branch_qprog: pyQPanda.QProg, false_branch_qprog: pyQPanda.QProg) -> None

        Constructor for initializing with a classical condition and two quantum programs.

        Args:
             classical_cond: The classical condition to evaluate.

             true_branch_qprog: The quantum program to execute if the condition is true.

             false_branch_qprog: The quantum program to execute if the condition is false.

        """
    @overload
    def __init__(self, classical_cond: ClassicalCondition, true_branch_qprog: QProg) -> None:
        """__init__(*args, **kwargs)
        Overloaded function.

        1. __init__(self: pyQPanda.QIfProg, arg0: pyQPanda.NodeIter) -> None

        Constructor for QIfProg.

        Args:
             iter: An iterator to a node.

        Raises:
             runtime_error: If the iterator is null or the node type is incorrect.


        2. __init__(self: pyQPanda.QIfProg, classical_cond: pyQPanda.ClassicalCondition, true_branch_qprog: pyQPanda.QProg) -> None

        Constructor for initializing with a classical condition and a quantum program.

        Args:
             classical_cond: The classical condition to evaluate.

             true_branch_qprog: The quantum program to execute if the condition is true.


        3. __init__(self: pyQPanda.QIfProg, classical_cond: pyQPanda.ClassicalCondition, true_branch_qprog: pyQPanda.QProg, false_branch_qprog: pyQPanda.QProg) -> None

        Constructor for initializing with a classical condition and two quantum programs.

        Args:
             classical_cond: The classical condition to evaluate.

             true_branch_qprog: The quantum program to execute if the condition is true.

             false_branch_qprog: The quantum program to execute if the condition is false.

        """
    @overload
    def __init__(self, classical_cond: ClassicalCondition, true_branch_qprog: QProg, false_branch_qprog: QProg) -> None:
        """__init__(*args, **kwargs)
        Overloaded function.

        1. __init__(self: pyQPanda.QIfProg, arg0: pyQPanda.NodeIter) -> None

        Constructor for QIfProg.

        Args:
             iter: An iterator to a node.

        Raises:
             runtime_error: If the iterator is null or the node type is incorrect.


        2. __init__(self: pyQPanda.QIfProg, classical_cond: pyQPanda.ClassicalCondition, true_branch_qprog: pyQPanda.QProg) -> None

        Constructor for initializing with a classical condition and a quantum program.

        Args:
             classical_cond: The classical condition to evaluate.

             true_branch_qprog: The quantum program to execute if the condition is true.


        3. __init__(self: pyQPanda.QIfProg, classical_cond: pyQPanda.ClassicalCondition, true_branch_qprog: pyQPanda.QProg, false_branch_qprog: pyQPanda.QProg) -> None

        Constructor for initializing with a classical condition and two quantum programs.

        Args:
             classical_cond: The classical condition to evaluate.

             true_branch_qprog: The quantum program to execute if the condition is true.

             false_branch_qprog: The quantum program to execute if the condition is false.

        """
    def get_classical_condition(self) -> ClassicalCondition:
        """get_classical_condition(self: pyQPanda.QIfProg) -> pyQPanda.ClassicalCondition

        Retrieve the classical condition associated with the quantum if program.

        Returns:
             The classical condition object used in the if statement.

        """
    def get_false_branch(self) -> QProg:
        """get_false_branch(self: pyQPanda.QIfProg) -> pyQPanda.QProg

        Get the quantum program corresponding to the false branch.

        Returns:
             QProg: The quantum program for the false branch, or an empty QProg if null.

        Raises:
             runtime_error: If the false branch has an incorrect node type.

        """
    def get_true_branch(self) -> QProg:
        """get_true_branch(self: pyQPanda.QIfProg) -> pyQPanda.QProg

        Get the quantum program corresponding to the true branch.

        Returns:
             QProg: The quantum program for the true branch.

        Raises:
             runtime_error: If the true branch is null or has an incorrect node type.

        """

class QMachineType:
    __members__: ClassVar[dict] = ...  # read-only
    CPU: ClassVar[QMachineType] = ...
    CPU_SINGLE_THREAD: ClassVar[QMachineType] = ...
    GPU: ClassVar[QMachineType] = ...
    NOISE: ClassVar[QMachineType] = ...
    __entries: ClassVar[dict] = ...
    def __init__(self, value: int) -> None:
        """__init__(self: pyQPanda.QMachineType, value: int) -> None"""
    def __eq__(self, other: object) -> bool:
        """__eq__(self: object, other: object) -> bool"""
    def __hash__(self) -> int:
        """__hash__(self: object) -> int"""
    def __index__(self) -> int:
        """__index__(self: pyQPanda.QMachineType) -> int"""
    def __int__(self) -> int:
        """__int__(self: pyQPanda.QMachineType) -> int"""
    def __ne__(self, other: object) -> bool:
        """__ne__(self: object, other: object) -> bool"""
    @property
    def name(self) -> str: ...
    @property
    def value(self) -> int: ...

class QMeasure:
    def __init__(self, arg0: NodeIter) -> None:
        """__init__(self: pyQPanda.QMeasure, arg0: pyQPanda.NodeIter) -> None

        Initialize QMeasure from a node iterator.

        Args:
             iter (NodeIter&): The iterator pointing to the node.

        Returns:
             QMeasure: The initialized measurement object.

        Raises:
             runtime_error: If the iterator is null or the node type is incorrect.

        """

class QOperator:
    @overload
    def __init__(self) -> None:
        """__init__(*args, **kwargs)
        Overloaded function.

        1. __init__(self: pyQPanda.QOperator) -> None

        Initialize a new QOperator instance.

        Args:
             None

        Returns:
             A new QOperator instance.


        2. __init__(self: pyQPanda.QOperator, arg0: pyQPanda.QGate) -> None

        Initialize QOperator based on a quantum gate.

        Args:
             qgate: An instance of QGate.

        Returns:
             A new QOperator instance.


        3. __init__(self: pyQPanda.QOperator, arg0: pyQPanda.QCircuit) -> None

        Initialize QOperator based on a quantum circuit.

        Args:
             qcircuit: An instance of QCircuit.

        Returns:
             A new QOperator instance.

        """
    @overload
    def __init__(self, arg0: QGate) -> None:
        """__init__(*args, **kwargs)
        Overloaded function.

        1. __init__(self: pyQPanda.QOperator) -> None

        Initialize a new QOperator instance.

        Args:
             None

        Returns:
             A new QOperator instance.


        2. __init__(self: pyQPanda.QOperator, arg0: pyQPanda.QGate) -> None

        Initialize QOperator based on a quantum gate.

        Args:
             qgate: An instance of QGate.

        Returns:
             A new QOperator instance.


        3. __init__(self: pyQPanda.QOperator, arg0: pyQPanda.QCircuit) -> None

        Initialize QOperator based on a quantum circuit.

        Args:
             qcircuit: An instance of QCircuit.

        Returns:
             A new QOperator instance.

        """
    @overload
    def __init__(self, arg0: QCircuit) -> None:
        """__init__(*args, **kwargs)
        Overloaded function.

        1. __init__(self: pyQPanda.QOperator) -> None

        Initialize a new QOperator instance.

        Args:
             None

        Returns:
             A new QOperator instance.


        2. __init__(self: pyQPanda.QOperator, arg0: pyQPanda.QGate) -> None

        Initialize QOperator based on a quantum gate.

        Args:
             qgate: An instance of QGate.

        Returns:
             A new QOperator instance.


        3. __init__(self: pyQPanda.QOperator, arg0: pyQPanda.QCircuit) -> None

        Initialize QOperator based on a quantum circuit.

        Args:
             qcircuit: An instance of QCircuit.

        Returns:
             A new QOperator instance.

        """
    def get_matrix(self) -> list[complex]:
        """get_matrix(self: pyQPanda.QOperator) -> List[complex]

        Retrieve the matrix representation of the QOperator.

        Args:
             None

        Returns:
             The matrix representation of the QOperator.

        """
    def to_instruction(self, arg0: str) -> str:
        """to_instruction(self: pyQPanda.QOperator, arg0: str) -> str

        Convert the QOperator to an instruction representation.

        Args:
             None

        Returns:
             The instruction representation of the QOperator.

        """

class QOptimizationResult:
    fcalls: int
    fun_val: float
    iters: int
    key: str
    message: str
    para: list[float]
    def __init__(self, arg0: str, arg1: int, arg2: int, arg3: str, arg4: float, arg5: list[float]) -> None:
        """__init__(self: pyQPanda.QOptimizationResult, arg0: str, arg1: int, arg2: int, arg3: str, arg4: float, arg5: List[float]) -> None

        Initialize a QOptimizationResult instance.

        Args:
             message: A string containing the result message.

             iteration: A size_t representing the iteration count.

             total_iterations: A size_t representing the total iterations.

             status: A string indicating the optimization status.

             value: A double representing the optimization value.

             results: A vector of doubles for the optimization results.

        """

class QPilotMachine:
    def __init__(self) -> None:
        """__init__(self: pyQPanda.QPilotMachine) -> None"""
    def build_noise_params(self, nose_model_type: int, single_params: list[float], double_params: list[float]) -> PilotNoiseParams:
        """build_noise_params(self: pyQPanda.QPilotMachine, nose_model_type: int, single_params: List[float], double_params: List[float]) -> pyQPanda.PilotNoiseParams"""
    def execute_callback_full_amplitude_expectation(self, prog_str: str, hamiltonian: list[tuple[dict[int, str], float]], qubit_vec: list[int], cb_func: Callable[[ErrorCode, float], None], chip_id: str = ...) -> ErrorCode:
        """execute_callback_full_amplitude_expectation(self: pyQPanda.QPilotMachine, prog_str: str, hamiltonian: List[Tuple[Dict[int, str], float]], qubit_vec: List[int], cb_func: Callable[[pyQPanda.ErrorCode, float], None], chip_id: str = 'any_cluster_backend') -> pyQPanda.ErrorCode"""
    def execute_callback_full_amplitude_measure_task(self, prog_str: str, cb_func: Callable[[ErrorCode, dict[str, float]], None], chip_id: str = ..., shots: int = ...) -> ErrorCode:
        """execute_callback_full_amplitude_measure_task(self: pyQPanda.QPilotMachine, prog_str: str, cb_func: Callable[[pyQPanda.ErrorCode, Dict[str, float]], None], chip_id: str = 'any_cluster_backend', shots: int = 1000) -> pyQPanda.ErrorCode"""
    def execute_callback_full_amplitude_pmeasure_task(self, prog_str: str, qubit_vec: list[int], cb_func: Callable[[ErrorCode, dict[str, float]], None], chip_id: str = ...) -> ErrorCode:
        """execute_callback_full_amplitude_pmeasure_task(self: pyQPanda.QPilotMachine, prog_str: str, qubit_vec: List[int], cb_func: Callable[[pyQPanda.ErrorCode, Dict[str, float]], None], chip_id: str = 'any_cluster_backend') -> pyQPanda.ErrorCode"""
    def execute_callback_measure_task(self, prog_str: str, cb_func: Callable[[ErrorCode, dict[str, float]], None], chip_id: str = ..., b_mapping: bool = ..., b_optimization: bool = ..., shots: int = ..., specified_block: list[int] = ...) -> ErrorCode:
        """execute_callback_measure_task(self: pyQPanda.QPilotMachine, prog_str: str, cb_func: Callable[[pyQPanda.ErrorCode, Dict[str, float]], None], chip_id: str = 'any_quantum_chip', b_mapping: bool = True, b_optimization: bool = True, shots: int = 1000, specified_block: List[int] = []) -> pyQPanda.ErrorCode"""
    def execute_callback_noise_measure_task(self, prog_str: str, noise_params: PilotNoiseParams, cb_func: Callable[[ErrorCode, dict[str, float]], None], chip_id: str = ..., shots: int = ...) -> ErrorCode:
        """execute_callback_noise_measure_task(self: pyQPanda.QPilotMachine, prog_str: str, noise_params: pyQPanda.PilotNoiseParams, cb_func: Callable[[pyQPanda.ErrorCode, Dict[str, float]], None], chip_id: str = 'any_cluster_backend', shots: int = 1000) -> pyQPanda.ErrorCode"""
    def execute_callback_partial_amplitude_task(self, prog_str: str, target_amplitude_vec: list[str], cb_func: Callable[[ErrorCode, dict[str, complex]], None], chip_id: str = ...) -> ErrorCode:
        """execute_callback_partial_amplitude_task(self: pyQPanda.QPilotMachine, prog_str: str, target_amplitude_vec: List[str], cb_func: Callable[[pyQPanda.ErrorCode, Dict[str, complex]], None], chip_id: str = 'any_cluster_backend') -> pyQPanda.ErrorCode"""
    def execute_callback_single_amplitude_task(self, prog_str: str, target_amplitude: str, cb_func: Callable[[ErrorCode, complex], None], chip_id: str = ...) -> ErrorCode:
        """execute_callback_single_amplitude_task(self: pyQPanda.QPilotMachine, prog_str: str, target_amplitude: str, cb_func: Callable[[pyQPanda.ErrorCode, complex], None], chip_id: str = 'any_cluster_backend') -> pyQPanda.ErrorCode"""
    def execute_full_amplitude_expectation(self, prog_str: str, hamiltonian: list[tuple[dict[int, str], float]], qubit_vec: list[int], chip_id: str = ...) -> float:
        """execute_full_amplitude_expectation(self: pyQPanda.QPilotMachine, prog_str: str, hamiltonian: List[Tuple[Dict[int, str], float]], qubit_vec: List[int], chip_id: str = 'any_cluster_backend') -> float"""
    def execute_full_amplitude_measure_task(self, prog_str: str, chip_id: str = ..., shots: int = ...) -> dict[str, float]:
        """execute_full_amplitude_measure_task(self: pyQPanda.QPilotMachine, prog_str: str, chip_id: str = 'any_cluster_backend', shots: int = 1000) -> Dict[str, float]"""
    def execute_full_amplitude_pmeasure_task(self, prog_str: str, qubit_vec: list[int], chip_id: str = ...) -> dict[str, float]:
        """execute_full_amplitude_pmeasure_task(self: pyQPanda.QPilotMachine, prog_str: str, qubit_vec: List[int], chip_id: str = 'any_cluster_backend') -> Dict[str, float]"""
    def execute_measure_task(self, prog_str: str, chip_id: str = ..., b_mapping: bool = ..., b_optimization: bool = ..., shots: int = ..., specified_block: list[int] = ...) -> list[dict[str, float]]:
        """execute_measure_task(self: pyQPanda.QPilotMachine, prog_str: str, chip_id: str = 'any_quantum_chip', b_mapping: bool = True, b_optimization: bool = True, shots: int = 1000, specified_block: List[int] = []) -> List[Dict[str, float]]"""
    def execute_noise_measure_task(self, prog_str: str, noise_params: PilotNoiseParams, chip_id: str = ..., shots: int = ...) -> dict[str, float]:
        """execute_noise_measure_task(self: pyQPanda.QPilotMachine, prog_str: str, noise_params: pyQPanda.PilotNoiseParams, chip_id: str = 'any_cluster_backend', shots: int = 1000) -> Dict[str, float]"""
    def execute_partial_amplitude_task(self, prog_str: str, target_amplitude_vec: list[str], chip_id: str = ...) -> dict[str, complex]:
        """execute_partial_amplitude_task(self: pyQPanda.QPilotMachine, prog_str: str, target_amplitude_vec: List[str], chip_id: str = 'any_cluster_backend') -> Dict[str, complex]"""
    def execute_single_amplitude_task(self, prog_str: str, target_amplitude: str, chip_id: str = ...) -> complex:
        """execute_single_amplitude_task(self: pyQPanda.QPilotMachine, prog_str: str, target_amplitude: str, chip_id: str = 'any_cluster_backend') -> complex"""
    def init(self, pilot_url: str, log_cout: bool = ...) -> bool:
        """init(self: pyQPanda.QPilotMachine, pilot_url: str, log_cout: bool = False) -> bool"""
    def init_withconfig(self, config_path: str = ...) -> bool:
        """init_withconfig(self: pyQPanda.QPilotMachine, config_path: str = 'pilotmachine.conf') -> bool"""

class QPilotOSService(QuantumMachine):
    def __init__(self, machine_type: str = ...) -> None:
        """__init__(self: pyQPanda.QPilotOSService, machine_type: str = 'CPU') -> None"""
    def async_em_compute(self, parameter_json: str) -> str:
        """async_em_compute(self: pyQPanda.QPilotOSService, parameter_json: str) -> str"""
    def async_real_chip_expectation(self, prog: QProg, hamiltonian: str, qubits: list[int] = ..., shot: int = ..., chip_id: str = ..., is_amend: bool = ..., is_mapping: bool = ..., is_optimization: bool = ..., specified_block: list[int] = ..., describe: str = ...) -> str:
        """async_real_chip_expectation(self: pyQPanda.QPilotOSService, prog: pyQPanda.QProg, hamiltonian: str, qubits: List[int] = [], shot: int = 1000, chip_id: str = 'any_quantum_chip', is_amend: bool = True, is_mapping: bool = True, is_optimization: bool = True, specified_block: List[int] = [], describe: str = '') -> str

        deprecated, use request instead.
        """
    @overload
    def async_real_chip_measure(self, prog: QProg, shot: int = ..., chip_id: str = ..., is_amend: bool = ..., is_mapping: bool = ..., is_optimization: bool = ..., specified_block: list[int] = ..., is_prob_counts: bool = ..., describe: str = ..., point_lable: int = ...) -> str:
        """async_real_chip_measure(*args, **kwargs)
        Overloaded function.

        1. async_real_chip_measure(self: pyQPanda.QPilotOSService, prog: pyQPanda.QProg, shot: int = 1000, chip_id: str = 'any_quantum_chip', is_amend: bool = True, is_mapping: bool = True, is_optimization: bool = True, specified_block: List[int] = [], is_prob_counts: bool = True, describe: str = '', point_lable: int = 0) -> str

        2. async_real_chip_measure(self: pyQPanda.QPilotOSService, ir: str, shot: int = 1000, chip_id: str = 'any_quantum_chip', is_amend: bool = True, is_mapping: bool = True, is_optimization: bool = True, specified_block: List[int] = [], is_prob_counts: bool = True, describe: str = '', point_lable: int = 0) -> str

        3. async_real_chip_measure(self: pyQPanda.QPilotOSService, prog: List[pyQPanda.QProg], shot: int = 1000, chip_id: str = 'any_quantum_chip', is_amend: bool = True, is_mapping: bool = True, is_optimization: bool = True, specified_block: List[int] = [], is_prob_counts: bool = True, describe: str = '', point_lable: int = 0) -> str

        4. async_real_chip_measure(self: pyQPanda.QPilotOSService, ir: List[str], shot: int = 1000, chip_id: str = 'any_quantum_chip', is_amend: bool = True, is_mapping: bool = True, is_optimization: bool = True, specified_block: List[int] = [], is_prob_counts: bool = True, describe: str = '', point_lable: int = 0) -> str

        5. async_real_chip_measure(self: pyQPanda.QPilotOSService, prog: List[pyQPanda.QProg], config_str: str) -> str
        """
    @overload
    def async_real_chip_measure(self, ir: str, shot: int = ..., chip_id: str = ..., is_amend: bool = ..., is_mapping: bool = ..., is_optimization: bool = ..., specified_block: list[int] = ..., is_prob_counts: bool = ..., describe: str = ..., point_lable: int = ...) -> str:
        """async_real_chip_measure(*args, **kwargs)
        Overloaded function.

        1. async_real_chip_measure(self: pyQPanda.QPilotOSService, prog: pyQPanda.QProg, shot: int = 1000, chip_id: str = 'any_quantum_chip', is_amend: bool = True, is_mapping: bool = True, is_optimization: bool = True, specified_block: List[int] = [], is_prob_counts: bool = True, describe: str = '', point_lable: int = 0) -> str

        2. async_real_chip_measure(self: pyQPanda.QPilotOSService, ir: str, shot: int = 1000, chip_id: str = 'any_quantum_chip', is_amend: bool = True, is_mapping: bool = True, is_optimization: bool = True, specified_block: List[int] = [], is_prob_counts: bool = True, describe: str = '', point_lable: int = 0) -> str

        3. async_real_chip_measure(self: pyQPanda.QPilotOSService, prog: List[pyQPanda.QProg], shot: int = 1000, chip_id: str = 'any_quantum_chip', is_amend: bool = True, is_mapping: bool = True, is_optimization: bool = True, specified_block: List[int] = [], is_prob_counts: bool = True, describe: str = '', point_lable: int = 0) -> str

        4. async_real_chip_measure(self: pyQPanda.QPilotOSService, ir: List[str], shot: int = 1000, chip_id: str = 'any_quantum_chip', is_amend: bool = True, is_mapping: bool = True, is_optimization: bool = True, specified_block: List[int] = [], is_prob_counts: bool = True, describe: str = '', point_lable: int = 0) -> str

        5. async_real_chip_measure(self: pyQPanda.QPilotOSService, prog: List[pyQPanda.QProg], config_str: str) -> str
        """
    @overload
    def async_real_chip_measure(self, prog: list[QProg], shot: int = ..., chip_id: str = ..., is_amend: bool = ..., is_mapping: bool = ..., is_optimization: bool = ..., specified_block: list[int] = ..., is_prob_counts: bool = ..., describe: str = ..., point_lable: int = ...) -> str:
        """async_real_chip_measure(*args, **kwargs)
        Overloaded function.

        1. async_real_chip_measure(self: pyQPanda.QPilotOSService, prog: pyQPanda.QProg, shot: int = 1000, chip_id: str = 'any_quantum_chip', is_amend: bool = True, is_mapping: bool = True, is_optimization: bool = True, specified_block: List[int] = [], is_prob_counts: bool = True, describe: str = '', point_lable: int = 0) -> str

        2. async_real_chip_measure(self: pyQPanda.QPilotOSService, ir: str, shot: int = 1000, chip_id: str = 'any_quantum_chip', is_amend: bool = True, is_mapping: bool = True, is_optimization: bool = True, specified_block: List[int] = [], is_prob_counts: bool = True, describe: str = '', point_lable: int = 0) -> str

        3. async_real_chip_measure(self: pyQPanda.QPilotOSService, prog: List[pyQPanda.QProg], shot: int = 1000, chip_id: str = 'any_quantum_chip', is_amend: bool = True, is_mapping: bool = True, is_optimization: bool = True, specified_block: List[int] = [], is_prob_counts: bool = True, describe: str = '', point_lable: int = 0) -> str

        4. async_real_chip_measure(self: pyQPanda.QPilotOSService, ir: List[str], shot: int = 1000, chip_id: str = 'any_quantum_chip', is_amend: bool = True, is_mapping: bool = True, is_optimization: bool = True, specified_block: List[int] = [], is_prob_counts: bool = True, describe: str = '', point_lable: int = 0) -> str

        5. async_real_chip_measure(self: pyQPanda.QPilotOSService, prog: List[pyQPanda.QProg], config_str: str) -> str
        """
    @overload
    def async_real_chip_measure(self, ir: list[str], shot: int = ..., chip_id: str = ..., is_amend: bool = ..., is_mapping: bool = ..., is_optimization: bool = ..., specified_block: list[int] = ..., is_prob_counts: bool = ..., describe: str = ..., point_lable: int = ...) -> str:
        """async_real_chip_measure(*args, **kwargs)
        Overloaded function.

        1. async_real_chip_measure(self: pyQPanda.QPilotOSService, prog: pyQPanda.QProg, shot: int = 1000, chip_id: str = 'any_quantum_chip', is_amend: bool = True, is_mapping: bool = True, is_optimization: bool = True, specified_block: List[int] = [], is_prob_counts: bool = True, describe: str = '', point_lable: int = 0) -> str

        2. async_real_chip_measure(self: pyQPanda.QPilotOSService, ir: str, shot: int = 1000, chip_id: str = 'any_quantum_chip', is_amend: bool = True, is_mapping: bool = True, is_optimization: bool = True, specified_block: List[int] = [], is_prob_counts: bool = True, describe: str = '', point_lable: int = 0) -> str

        3. async_real_chip_measure(self: pyQPanda.QPilotOSService, prog: List[pyQPanda.QProg], shot: int = 1000, chip_id: str = 'any_quantum_chip', is_amend: bool = True, is_mapping: bool = True, is_optimization: bool = True, specified_block: List[int] = [], is_prob_counts: bool = True, describe: str = '', point_lable: int = 0) -> str

        4. async_real_chip_measure(self: pyQPanda.QPilotOSService, ir: List[str], shot: int = 1000, chip_id: str = 'any_quantum_chip', is_amend: bool = True, is_mapping: bool = True, is_optimization: bool = True, specified_block: List[int] = [], is_prob_counts: bool = True, describe: str = '', point_lable: int = 0) -> str

        5. async_real_chip_measure(self: pyQPanda.QPilotOSService, prog: List[pyQPanda.QProg], config_str: str) -> str
        """
    @overload
    def async_real_chip_measure(self, prog: list[QProg], config_str: str) -> str:
        """async_real_chip_measure(*args, **kwargs)
        Overloaded function.

        1. async_real_chip_measure(self: pyQPanda.QPilotOSService, prog: pyQPanda.QProg, shot: int = 1000, chip_id: str = 'any_quantum_chip', is_amend: bool = True, is_mapping: bool = True, is_optimization: bool = True, specified_block: List[int] = [], is_prob_counts: bool = True, describe: str = '', point_lable: int = 0) -> str

        2. async_real_chip_measure(self: pyQPanda.QPilotOSService, ir: str, shot: int = 1000, chip_id: str = 'any_quantum_chip', is_amend: bool = True, is_mapping: bool = True, is_optimization: bool = True, specified_block: List[int] = [], is_prob_counts: bool = True, describe: str = '', point_lable: int = 0) -> str

        3. async_real_chip_measure(self: pyQPanda.QPilotOSService, prog: List[pyQPanda.QProg], shot: int = 1000, chip_id: str = 'any_quantum_chip', is_amend: bool = True, is_mapping: bool = True, is_optimization: bool = True, specified_block: List[int] = [], is_prob_counts: bool = True, describe: str = '', point_lable: int = 0) -> str

        4. async_real_chip_measure(self: pyQPanda.QPilotOSService, ir: List[str], shot: int = 1000, chip_id: str = 'any_quantum_chip', is_amend: bool = True, is_mapping: bool = True, is_optimization: bool = True, specified_block: List[int] = [], is_prob_counts: bool = True, describe: str = '', point_lable: int = 0) -> str

        5. async_real_chip_measure(self: pyQPanda.QPilotOSService, prog: List[pyQPanda.QProg], config_str: str) -> str
        """
    @overload
    def async_real_chip_measure_vec(self, prog: list[QProg], shot: int = ..., chip_id: str = ..., is_amend: bool = ..., is_mapping: bool = ..., is_optimization: bool = ..., specified_block: list[int] = ..., is_prob_counts: bool = ..., describe: str = ..., point_lable: int = ...) -> str:
        """async_real_chip_measure_vec(*args, **kwargs)
        Overloaded function.

        1. async_real_chip_measure_vec(self: pyQPanda.QPilotOSService, prog: List[pyQPanda.QProg], shot: int = 1000, chip_id: str = 'any_quantum_chip', is_amend: bool = True, is_mapping: bool = True, is_optimization: bool = True, specified_block: List[int] = [], is_prob_counts: bool = True, describe: str = '', point_lable: int = 0) -> str

        2. async_real_chip_measure_vec(self: pyQPanda.QPilotOSService, ir: List[str], shot: int = 1000, chip_id: str = 'any_quantum_chip', is_amend: bool = True, is_mapping: bool = True, is_optimization: bool = True, specified_block: List[int] = [], is_prob_counts: bool = True, describe: str = '', point_lable: int = 0) -> str
        """
    @overload
    def async_real_chip_measure_vec(self, ir: list[str], shot: int = ..., chip_id: str = ..., is_amend: bool = ..., is_mapping: bool = ..., is_optimization: bool = ..., specified_block: list[int] = ..., is_prob_counts: bool = ..., describe: str = ..., point_lable: int = ...) -> str:
        """async_real_chip_measure_vec(*args, **kwargs)
        Overloaded function.

        1. async_real_chip_measure_vec(self: pyQPanda.QPilotOSService, prog: List[pyQPanda.QProg], shot: int = 1000, chip_id: str = 'any_quantum_chip', is_amend: bool = True, is_mapping: bool = True, is_optimization: bool = True, specified_block: List[int] = [], is_prob_counts: bool = True, describe: str = '', point_lable: int = 0) -> str

        2. async_real_chip_measure_vec(self: pyQPanda.QPilotOSService, ir: List[str], shot: int = 1000, chip_id: str = 'any_quantum_chip', is_amend: bool = True, is_mapping: bool = True, is_optimization: bool = True, specified_block: List[int] = [], is_prob_counts: bool = True, describe: str = '', point_lable: int = 0) -> str
        """
    def async_real_chip_qst(self, prog: QProg, shot: int = ..., chip_id: str = ..., is_amend: bool = ..., is_mapping: bool = ..., is_optimization: bool = ..., specified_block: list[int] = ..., describe: str = ..., point_lable: int = ...) -> str:
        """async_real_chip_qst(self: pyQPanda.QPilotOSService, prog: pyQPanda.QProg, shot: int = 1000, chip_id: str = 'any_quantum_chip', is_amend: bool = True, is_mapping: bool = True, is_optimization: bool = True, specified_block: List[int] = [], describe: str = '', point_lable: int = 0) -> str"""
    def async_real_chip_qst_density(self, prog: QProg, shot: int = ..., chip_id: str = ..., is_amend: bool = ..., is_mapping: bool = ..., is_optimization: bool = ..., specified_block: list[int] = ..., describe: str = ..., point_lable: int = ...) -> str:
        """async_real_chip_qst_density(self: pyQPanda.QPilotOSService, prog: pyQPanda.QProg, shot: int = 1000, chip_id: str = 'any_quantum_chip', is_amend: bool = True, is_mapping: bool = True, is_optimization: bool = True, specified_block: List[int] = [], describe: str = '', point_lable: int = 0) -> str"""
    def async_real_chip_qst_fidelity(self, prog: QProg, shot: int = ..., chip_id: str = ..., is_amend: bool = ..., is_mapping: bool = ..., is_optimization: bool = ..., specified_block: list[int] = ..., describe: str = ..., point_lable: int = ...) -> str:
        """async_real_chip_qst_fidelity(self: pyQPanda.QPilotOSService, prog: pyQPanda.QProg, shot: int = 1000, chip_id: str = 'any_quantum_chip', is_amend: bool = True, is_mapping: bool = True, is_optimization: bool = True, specified_block: List[int] = [], describe: str = '', point_lable: int = 0) -> str"""
    def build_expectation_task_msg(self, prog: QProg, hamiltonian: str, qubits: list[int] = ..., shot: int = ..., chip_id: str = ..., is_amend: bool = ..., is_mapping: bool = ..., is_optimization: bool = ..., specified_block: list[int] = ..., task_describe: str = ...) -> str:
        """build_expectation_task_msg(self: pyQPanda.QPilotOSService, prog: pyQPanda.QProg, hamiltonian: str, qubits: List[int] = [], shot: int = 1000, chip_id: str = 'any_quantum_chip', is_amend: bool = True, is_mapping: bool = True, is_optimization: bool = True, specified_block: List[int] = [], task_describe: str = '') -> str

        use C++ to build a expectation task body.
        """
    def build_init_msg(self, api_key: str) -> str:
        """build_init_msg(self: pyQPanda.QPilotOSService, api_key: str) -> str"""
    def build_qst_task_msg(self, prog: QProg, shot: int = ..., chip_id: str = ..., is_amend: bool = ..., is_mapping: bool = ..., is_optimization: bool = ..., specified_block: list[int] = ..., task_describe: str = ...) -> str:
        """build_qst_task_msg(self: pyQPanda.QPilotOSService, prog: pyQPanda.QProg, shot: int = 1000, chip_id: str = 'any_quantum_chip', is_amend: bool = True, is_mapping: bool = True, is_optimization: bool = True, specified_block: List[int] = [], task_describe: str = '') -> str

        use C++ to build ordinary qst task msg body
        """
    def build_query_msg(self, task_id: str) -> str:
        """build_query_msg(self: pyQPanda.QPilotOSService, task_id: str) -> str"""
    def build_task_msg(self, prog: list[QProg], shot: int, chip_id: str, is_amend: bool, is_mapping: bool, is_optimization: bool, specified_block: list[int], task_describe: str, point_lable: int, priority: int) -> str:
        """build_task_msg(self: pyQPanda.QPilotOSService, prog: List[pyQPanda.QProg], shot: int, chip_id: str, is_amend: bool, is_mapping: bool, is_optimization: bool, specified_block: List[int], task_describe: str, point_lable: int, priority: int) -> str

        use c++ to build real chip measure task msg body.
        """
    def build_task_proto_msg(self, shot: int, chip_id: str, is_amend: bool, is_mapping: bool, is_optimization: bool, specified_block: list[int], task_describe: str, point_lable: int, priority: int) -> str:
        """build_task_proto_msg(self: pyQPanda.QPilotOSService, shot: int, chip_id: str, is_amend: bool, is_mapping: bool, is_optimization: bool, specified_block: List[int], task_describe: str, point_lable: int, priority: int) -> str

        use c++ to build real chip measure task msg body.
        """
    @overload
    def cAlloc(self) -> ClassicalCondition:
        """cAlloc(*args, **kwargs)
        Overloaded function.

        1. cAlloc(self: pyQPanda.QPilotOSService) -> pyQPanda.ClassicalCondition

        Allocate a cbit

        2. cAlloc(self: pyQPanda.QPilotOSService, cbit: int) -> pyQPanda.ClassicalCondition

        Allocate a cbit
        """
    @overload
    def cAlloc(self, cbit: int) -> ClassicalCondition:
        """cAlloc(*args, **kwargs)
        Overloaded function.

        1. cAlloc(self: pyQPanda.QPilotOSService) -> pyQPanda.ClassicalCondition

        Allocate a cbit

        2. cAlloc(self: pyQPanda.QPilotOSService, cbit: int) -> pyQPanda.ClassicalCondition

        Allocate a cbit
        """
    def cAlloc_many(self, cbit_num: int) -> list[ClassicalCondition]:
        """cAlloc_many(self: pyQPanda.QPilotOSService, cbit_num: int) -> List[pyQPanda.ClassicalCondition]

        Allocate a list of cbits
        """
    def cFree(self, arg0: ClassicalCondition) -> None:
        """cFree(self: pyQPanda.QPilotOSService, arg0: pyQPanda.ClassicalCondition) -> None

        Free a cbit
        """
    @overload
    def cFree_all(self, cbit_list: list[ClassicalCondition]) -> None:
        """cFree_all(*args, **kwargs)
        Overloaded function.

        1. cFree_all(self: pyQPanda.QPilotOSService, cbit_list: List[pyQPanda.ClassicalCondition]) -> None

        Free a list of cbits

        2. cFree_all(self: pyQPanda.QPilotOSService) -> None

        Free all of cbits
        """
    @overload
    def cFree_all(self) -> None:
        """cFree_all(*args, **kwargs)
        Overloaded function.

        1. cFree_all(self: pyQPanda.QPilotOSService, cbit_list: List[pyQPanda.ClassicalCondition]) -> None

        Free a list of cbits

        2. cFree_all(self: pyQPanda.QPilotOSService) -> None

        Free all of cbits
        """
    def em_compute(self, parameter_json: str) -> list[float]:
        """em_compute(self: pyQPanda.QPilotOSService, parameter_json: str) -> List[float]"""
    def finalize(self) -> None:
        """finalize(self: pyQPanda.QPilotOSService) -> None

        finalize
        """
    def get_expectation_result(self, task_id: str) -> list:
        """get_expectation_result(self: pyQPanda.QPilotOSService, task_id: str) -> list"""
    @overload
    def get_measure_result(self, task_id: str) -> list:
        """get_measure_result(*args, **kwargs)
        Overloaded function.

        1. get_measure_result(self: pyQPanda.QPilotOSService, task_id: str) -> list

        2. get_measure_result(self: pyQPanda.QPilotOSService, task_id: str) -> list
        """
    @overload
    def get_measure_result(self, task_id: str) -> list:
        """get_measure_result(*args, **kwargs)
        Overloaded function.

        1. get_measure_result(self: pyQPanda.QPilotOSService, task_id: str) -> list

        2. get_measure_result(self: pyQPanda.QPilotOSService, task_id: str) -> list
        """
    def get_token(self, rep_json: str) -> ErrorCode:
        """get_token(self: pyQPanda.QPilotOSService, rep_json: str) -> pyQPanda.ErrorCode"""
    @overload
    def init(self) -> None:
        """init(*args, **kwargs)
        Overloaded function.

        1. init(self: pyQPanda.QPilotOSService) -> None

        2. init(self: pyQPanda.QPilotOSService, url: str, log_cout: bool = False, api_key: str = None) -> None

        3. init(self: pyQPanda.QPilotOSService, url: str, log_cout: bool = False, username: str = None, password: str = None) -> None
        """
    @overload
    def init(self, url: str, log_cout: bool = ..., api_key: str = ...) -> None:
        """init(*args, **kwargs)
        Overloaded function.

        1. init(self: pyQPanda.QPilotOSService) -> None

        2. init(self: pyQPanda.QPilotOSService, url: str, log_cout: bool = False, api_key: str = None) -> None

        3. init(self: pyQPanda.QPilotOSService, url: str, log_cout: bool = False, username: str = None, password: str = None) -> None
        """
    @overload
    def init(self, url: str, log_cout: bool = ..., username: str = ..., password: str = ...) -> None:
        """init(*args, **kwargs)
        Overloaded function.

        1. init(self: pyQPanda.QPilotOSService) -> None

        2. init(self: pyQPanda.QPilotOSService, url: str, log_cout: bool = False, api_key: str = None) -> None

        3. init(self: pyQPanda.QPilotOSService, url: str, log_cout: bool = False, username: str = None, password: str = None) -> None
        """
    def init_config(self, url: str, log_cout: bool) -> None:
        """init_config(self: pyQPanda.QPilotOSService, url: str, log_cout: bool) -> None"""
    def noise_learning(self, parameter_json: str = ...) -> str:
        """noise_learning(self: pyQPanda.QPilotOSService, parameter_json: str = True) -> str"""
    def output_version(self) -> str:
        """output_version(self: pyQPanda.QPilotOSService) -> str"""
    def pMeasureBinindex(self, prog: QProg, index: str, backendID: str = ...) -> float:
        """pMeasureBinindex(self: pyQPanda.QPilotOSService, prog: pyQPanda.QProg, index: str, backendID: str = 'any_cluster_backend') -> float"""
    def pMeasureDecindex(self, prog: QProg, index: str, backendID: str = ...) -> float:
        """pMeasureDecindex(self: pyQPanda.QPilotOSService, prog: pyQPanda.QProg, index: str, backendID: str = 'any_cluster_backend') -> float"""
    def parse_prob_counts_result(self, result_str: list[str]) -> list[dict[str, int]]:
        """parse_prob_counts_result(self: pyQPanda.QPilotOSService, result_str: List[str]) -> List[Dict[str, int]]

        Parse result str to map<string, double>
        Args:
            result_str: Taeget result string

        Returns:
            array: vector<map<string, double>>
        Raises:
            none

        """
    def parse_probability_result(self, result_str: list[str]) -> list[dict[str, float]]:
        """parse_probability_result(self: pyQPanda.QPilotOSService, result_str: List[str]) -> List[Dict[str, float]]

        Parse result str to map<string, double>
        Args:
            result_str: Taeget result string

        Returns:
            array: vector<map<string, double>>
        Raises:
            none

        """
    def parse_task_result(self, result_str: str) -> dict[str, float]:
        """parse_task_result(self: pyQPanda.QPilotOSService, result_str: str) -> Dict[str, float]

        Parse result str to map<string, double>
        Args:
            result_str: Taeget result string

        Returns:
            dict: map<string, double>
        Raises:
            none

        """
    def parser_expectation_result(self, json_str: str) -> list:
        """parser_expectation_result(self: pyQPanda.QPilotOSService, json_str: str) -> list

        deprecated, use Python's json lib.
        """
    def parser_sync_result(self, json_str: str) -> list[dict[str, float]]:
        """parser_sync_result(self: pyQPanda.QPilotOSService, json_str: str) -> List[Dict[str, float]]"""
    def pmeasure_subset(self, prog: QProg, amplitude: list[str], backendID: str = ...) -> dict[str, complex]:
        """pmeasure_subset(self: pyQPanda.QPilotOSService, prog: pyQPanda.QProg, amplitude: List[str], backendID: str = 'any_cluster_backend') -> Dict[str, complex]"""
    def probRunDict(self, prog: QProg, qubit_vec: list[int], backendID: str = ...) -> dict[str, float]:
        """probRunDict(self: pyQPanda.QPilotOSService, prog: pyQPanda.QProg, qubit_vec: List[int], backendID: str = 'any_cluster_backend') -> Dict[str, float]"""
    def qAlloc(self) -> Qubit:
        """qAlloc(self: pyQPanda.QPilotOSService) -> pyQPanda.Qubit

        Allocate a qubit
        """
    def qAlloc_many(self, qubit_num: int) -> list[Qubit]:
        """qAlloc_many(self: pyQPanda.QPilotOSService, qubit_num: int) -> List[pyQPanda.Qubit]

        Allocate a list of qubits
        """
    def qFree(self, qubit: Qubit) -> None:
        """qFree(self: pyQPanda.QPilotOSService, qubit: pyQPanda.Qubit) -> None

        Free a qubit
        """
    @overload
    def qFree_all(self, qubit_list: QVec) -> None:
        """qFree_all(*args, **kwargs)
        Overloaded function.

        1. qFree_all(self: pyQPanda.QPilotOSService, qubit_list: pyQPanda.QVec) -> None

        Free a list of qubits

        2. qFree_all(self: pyQPanda.QPilotOSService, arg0: pyQPanda.QVec) -> None

        Free all of qubits
        """
    @overload
    def qFree_all(self, arg0: QVec) -> None:
        """qFree_all(*args, **kwargs)
        Overloaded function.

        1. qFree_all(self: pyQPanda.QPilotOSService, qubit_list: pyQPanda.QVec) -> None

        Free a list of qubits

        2. qFree_all(self: pyQPanda.QPilotOSService, arg0: pyQPanda.QVec) -> None

        Free all of qubits
        """
    def query_compile_prog(self, task_id: str, without_compensate: bool = ...) -> list:
        """query_compile_prog(self: pyQPanda.QPilotOSService, task_id: str, without_compensate: bool = True) -> list

        Query Task compile prog by task_id
        Args:
            without_compensate: whether return the prog without angle compensate

        Returns:
            bool: whether find compile prog success
        Raises:
            none

        """
    @overload
    def query_task_state(self, task_id: str) -> list:
        """query_task_state(*args, **kwargs)
        Overloaded function.

        1. query_task_state(self: pyQPanda.QPilotOSService, task_id: str) -> list

        Query Task State by task_id
        Args:
            task_id: Taeget task id, got by async_real_chip_measure

        Returns:
            string: task state: 2: Running; 3: Finished; 4: Failed
            string: task result string
        Raises:
            none


        2. query_task_state(self: pyQPanda.QPilotOSService, task_id: str, is_save: bool, file_path: str = '') -> list
        """
    @overload
    def query_task_state(self, task_id: str, is_save: bool, file_path: str = ...) -> list:
        """query_task_state(*args, **kwargs)
        Overloaded function.

        1. query_task_state(self: pyQPanda.QPilotOSService, task_id: str) -> list

        Query Task State by task_id
        Args:
            task_id: Taeget task id, got by async_real_chip_measure

        Returns:
            string: task state: 2: Running; 3: Finished; 4: Failed
            string: task result string
        Raises:
            none


        2. query_task_state(self: pyQPanda.QPilotOSService, task_id: str, is_save: bool, file_path: str = '') -> list
        """
    def query_task_state_vec(self, task_id: str) -> list:
        """query_task_state_vec(self: pyQPanda.QPilotOSService, task_id: str) -> list

        Query Task State by task_id
        Args:
            task_id: Taeget task id, got by async_real_chip_measure

        Returns:
            string: task state: 2: Running; 3: Finished; 4: Failed
            array: task result array
        Raises:
            none

        """
    def real_chip_expectation(self, prog: QProg, hamiltonian: str, qubits: list[int] = ..., shot: int = ..., chip_id: str = ..., is_amend: bool = ..., is_mapping: bool = ..., is_optimization: bool = ..., specified_block: list[int] = ..., describe: str = ...) -> float:
        """real_chip_expectation(self: pyQPanda.QPilotOSService, prog: pyQPanda.QProg, hamiltonian: str, qubits: List[int] = [], shot: int = 1000, chip_id: str = 'any_quantum_chip', is_amend: bool = True, is_mapping: bool = True, is_optimization: bool = True, specified_block: List[int] = [], describe: str = '') -> float

        deprecated, use request instead.
        """
    @overload
    def real_chip_measure(self, prog: QProg, shot: int = ..., chip_id: str = ..., is_amend: bool = ..., is_mapping: bool = ..., is_optimization: bool = ..., specified_block: list[int] = ..., describe: str = ..., point_lable: int = ...) -> dict[str, float]:
        """real_chip_measure(*args, **kwargs)
        Overloaded function.

        1. real_chip_measure(self: pyQPanda.QPilotOSService, prog: pyQPanda.QProg, shot: int = 1000, chip_id: str = 'any_quantum_chip', is_amend: bool = True, is_mapping: bool = True, is_optimization: bool = True, specified_block: List[int] = [], describe: str = '', point_lable: int = 0) -> Dict[str, float]

        2. real_chip_measure(self: pyQPanda.QPilotOSService, ir: str, shot: int = 1000, chip_id: str = 'any_quantum_chip', is_amend: bool = True, is_mapping: bool = True, is_optimization: bool = True, specified_block: List[int] = [], describe: str = '', point_lable: int = 0) -> Dict[str, float]

        3. real_chip_measure(self: pyQPanda.QPilotOSService, prog: List[pyQPanda.QProg], shot: int = 1000, chip_id: str = 'any_quantum_chip', is_amend: bool = True, is_mapping: bool = True, is_optimization: bool = True, specified_block: List[int] = [], describe: str = '', point_lable: int = 0) -> List[Dict[str, float]]

        4. real_chip_measure(self: pyQPanda.QPilotOSService, ir: List[str], shot: int = 1000, chip_id: str = 'any_quantum_chip', is_amend: bool = True, is_mapping: bool = True, is_optimization: bool = True, specified_block: List[int] = [], describe: str = '', point_lable: int = 0) -> List[Dict[str, float]]

        5. real_chip_measure(self: pyQPanda.QPilotOSService, prog: List[pyQPanda.QProg], config_str: str) -> str
        """
    @overload
    def real_chip_measure(self, ir: str, shot: int = ..., chip_id: str = ..., is_amend: bool = ..., is_mapping: bool = ..., is_optimization: bool = ..., specified_block: list[int] = ..., describe: str = ..., point_lable: int = ...) -> dict[str, float]:
        """real_chip_measure(*args, **kwargs)
        Overloaded function.

        1. real_chip_measure(self: pyQPanda.QPilotOSService, prog: pyQPanda.QProg, shot: int = 1000, chip_id: str = 'any_quantum_chip', is_amend: bool = True, is_mapping: bool = True, is_optimization: bool = True, specified_block: List[int] = [], describe: str = '', point_lable: int = 0) -> Dict[str, float]

        2. real_chip_measure(self: pyQPanda.QPilotOSService, ir: str, shot: int = 1000, chip_id: str = 'any_quantum_chip', is_amend: bool = True, is_mapping: bool = True, is_optimization: bool = True, specified_block: List[int] = [], describe: str = '', point_lable: int = 0) -> Dict[str, float]

        3. real_chip_measure(self: pyQPanda.QPilotOSService, prog: List[pyQPanda.QProg], shot: int = 1000, chip_id: str = 'any_quantum_chip', is_amend: bool = True, is_mapping: bool = True, is_optimization: bool = True, specified_block: List[int] = [], describe: str = '', point_lable: int = 0) -> List[Dict[str, float]]

        4. real_chip_measure(self: pyQPanda.QPilotOSService, ir: List[str], shot: int = 1000, chip_id: str = 'any_quantum_chip', is_amend: bool = True, is_mapping: bool = True, is_optimization: bool = True, specified_block: List[int] = [], describe: str = '', point_lable: int = 0) -> List[Dict[str, float]]

        5. real_chip_measure(self: pyQPanda.QPilotOSService, prog: List[pyQPanda.QProg], config_str: str) -> str
        """
    @overload
    def real_chip_measure(self, prog: list[QProg], shot: int = ..., chip_id: str = ..., is_amend: bool = ..., is_mapping: bool = ..., is_optimization: bool = ..., specified_block: list[int] = ..., describe: str = ..., point_lable: int = ...) -> list[dict[str, float]]:
        """real_chip_measure(*args, **kwargs)
        Overloaded function.

        1. real_chip_measure(self: pyQPanda.QPilotOSService, prog: pyQPanda.QProg, shot: int = 1000, chip_id: str = 'any_quantum_chip', is_amend: bool = True, is_mapping: bool = True, is_optimization: bool = True, specified_block: List[int] = [], describe: str = '', point_lable: int = 0) -> Dict[str, float]

        2. real_chip_measure(self: pyQPanda.QPilotOSService, ir: str, shot: int = 1000, chip_id: str = 'any_quantum_chip', is_amend: bool = True, is_mapping: bool = True, is_optimization: bool = True, specified_block: List[int] = [], describe: str = '', point_lable: int = 0) -> Dict[str, float]

        3. real_chip_measure(self: pyQPanda.QPilotOSService, prog: List[pyQPanda.QProg], shot: int = 1000, chip_id: str = 'any_quantum_chip', is_amend: bool = True, is_mapping: bool = True, is_optimization: bool = True, specified_block: List[int] = [], describe: str = '', point_lable: int = 0) -> List[Dict[str, float]]

        4. real_chip_measure(self: pyQPanda.QPilotOSService, ir: List[str], shot: int = 1000, chip_id: str = 'any_quantum_chip', is_amend: bool = True, is_mapping: bool = True, is_optimization: bool = True, specified_block: List[int] = [], describe: str = '', point_lable: int = 0) -> List[Dict[str, float]]

        5. real_chip_measure(self: pyQPanda.QPilotOSService, prog: List[pyQPanda.QProg], config_str: str) -> str
        """
    @overload
    def real_chip_measure(self, ir: list[str], shot: int = ..., chip_id: str = ..., is_amend: bool = ..., is_mapping: bool = ..., is_optimization: bool = ..., specified_block: list[int] = ..., describe: str = ..., point_lable: int = ...) -> list[dict[str, float]]:
        """real_chip_measure(*args, **kwargs)
        Overloaded function.

        1. real_chip_measure(self: pyQPanda.QPilotOSService, prog: pyQPanda.QProg, shot: int = 1000, chip_id: str = 'any_quantum_chip', is_amend: bool = True, is_mapping: bool = True, is_optimization: bool = True, specified_block: List[int] = [], describe: str = '', point_lable: int = 0) -> Dict[str, float]

        2. real_chip_measure(self: pyQPanda.QPilotOSService, ir: str, shot: int = 1000, chip_id: str = 'any_quantum_chip', is_amend: bool = True, is_mapping: bool = True, is_optimization: bool = True, specified_block: List[int] = [], describe: str = '', point_lable: int = 0) -> Dict[str, float]

        3. real_chip_measure(self: pyQPanda.QPilotOSService, prog: List[pyQPanda.QProg], shot: int = 1000, chip_id: str = 'any_quantum_chip', is_amend: bool = True, is_mapping: bool = True, is_optimization: bool = True, specified_block: List[int] = [], describe: str = '', point_lable: int = 0) -> List[Dict[str, float]]

        4. real_chip_measure(self: pyQPanda.QPilotOSService, ir: List[str], shot: int = 1000, chip_id: str = 'any_quantum_chip', is_amend: bool = True, is_mapping: bool = True, is_optimization: bool = True, specified_block: List[int] = [], describe: str = '', point_lable: int = 0) -> List[Dict[str, float]]

        5. real_chip_measure(self: pyQPanda.QPilotOSService, prog: List[pyQPanda.QProg], config_str: str) -> str
        """
    @overload
    def real_chip_measure(self, prog: list[QProg], config_str: str) -> str:
        """real_chip_measure(*args, **kwargs)
        Overloaded function.

        1. real_chip_measure(self: pyQPanda.QPilotOSService, prog: pyQPanda.QProg, shot: int = 1000, chip_id: str = 'any_quantum_chip', is_amend: bool = True, is_mapping: bool = True, is_optimization: bool = True, specified_block: List[int] = [], describe: str = '', point_lable: int = 0) -> Dict[str, float]

        2. real_chip_measure(self: pyQPanda.QPilotOSService, ir: str, shot: int = 1000, chip_id: str = 'any_quantum_chip', is_amend: bool = True, is_mapping: bool = True, is_optimization: bool = True, specified_block: List[int] = [], describe: str = '', point_lable: int = 0) -> Dict[str, float]

        3. real_chip_measure(self: pyQPanda.QPilotOSService, prog: List[pyQPanda.QProg], shot: int = 1000, chip_id: str = 'any_quantum_chip', is_amend: bool = True, is_mapping: bool = True, is_optimization: bool = True, specified_block: List[int] = [], describe: str = '', point_lable: int = 0) -> List[Dict[str, float]]

        4. real_chip_measure(self: pyQPanda.QPilotOSService, ir: List[str], shot: int = 1000, chip_id: str = 'any_quantum_chip', is_amend: bool = True, is_mapping: bool = True, is_optimization: bool = True, specified_block: List[int] = [], describe: str = '', point_lable: int = 0) -> List[Dict[str, float]]

        5. real_chip_measure(self: pyQPanda.QPilotOSService, prog: List[pyQPanda.QProg], config_str: str) -> str
        """
    @overload
    def real_chip_measure_prob_count(self, ir: str, shot: int = ..., chip_id: str = ..., is_mapping: bool = ..., is_optimization: bool = ..., specified_block: list[int] = ..., describe: str = ..., point_lable: int = ...) -> dict[str, int]:
        """real_chip_measure_prob_count(*args, **kwargs)
        Overloaded function.

        1. real_chip_measure_prob_count(self: pyQPanda.QPilotOSService, ir: str, shot: int = 1000, chip_id: str = 'any_quantum_chip', is_mapping: bool = True, is_optimization: bool = True, specified_block: List[int] = [], describe: str = '', point_lable: int = 0) -> Dict[str, int]

        2. real_chip_measure_prob_count(self: pyQPanda.QPilotOSService, prog: pyQPanda.QProg, shot: int = 1000, chip_id: str = 'any_quantum_chip', is_mapping: bool = True, is_optimization: bool = True, specified_block: List[int] = [], describe: str = '', point_lable: int = 0) -> Dict[str, int]

        3. real_chip_measure_prob_count(self: pyQPanda.QPilotOSService, ir: List[str], shot: int = 1000, chip_id: str = 'any_quantum_chip', is_mapping: bool = True, is_optimization: bool = True, specified_block: List[int] = [], describe: str = '', point_lable: int = 0) -> List[Dict[str, int]]

        4. real_chip_measure_prob_count(self: pyQPanda.QPilotOSService, prog: List[pyQPanda.QProg], shot: int = 1000, chip_id: str = 'any_quantum_chip', is_mapping: bool = True, is_optimization: bool = True, specified_block: List[int] = [], describe: str = '', point_lable: int = 0) -> List[Dict[str, int]]
        """
    @overload
    def real_chip_measure_prob_count(self, prog: QProg, shot: int = ..., chip_id: str = ..., is_mapping: bool = ..., is_optimization: bool = ..., specified_block: list[int] = ..., describe: str = ..., point_lable: int = ...) -> dict[str, int]:
        """real_chip_measure_prob_count(*args, **kwargs)
        Overloaded function.

        1. real_chip_measure_prob_count(self: pyQPanda.QPilotOSService, ir: str, shot: int = 1000, chip_id: str = 'any_quantum_chip', is_mapping: bool = True, is_optimization: bool = True, specified_block: List[int] = [], describe: str = '', point_lable: int = 0) -> Dict[str, int]

        2. real_chip_measure_prob_count(self: pyQPanda.QPilotOSService, prog: pyQPanda.QProg, shot: int = 1000, chip_id: str = 'any_quantum_chip', is_mapping: bool = True, is_optimization: bool = True, specified_block: List[int] = [], describe: str = '', point_lable: int = 0) -> Dict[str, int]

        3. real_chip_measure_prob_count(self: pyQPanda.QPilotOSService, ir: List[str], shot: int = 1000, chip_id: str = 'any_quantum_chip', is_mapping: bool = True, is_optimization: bool = True, specified_block: List[int] = [], describe: str = '', point_lable: int = 0) -> List[Dict[str, int]]

        4. real_chip_measure_prob_count(self: pyQPanda.QPilotOSService, prog: List[pyQPanda.QProg], shot: int = 1000, chip_id: str = 'any_quantum_chip', is_mapping: bool = True, is_optimization: bool = True, specified_block: List[int] = [], describe: str = '', point_lable: int = 0) -> List[Dict[str, int]]
        """
    @overload
    def real_chip_measure_prob_count(self, ir: list[str], shot: int = ..., chip_id: str = ..., is_mapping: bool = ..., is_optimization: bool = ..., specified_block: list[int] = ..., describe: str = ..., point_lable: int = ...) -> list[dict[str, int]]:
        """real_chip_measure_prob_count(*args, **kwargs)
        Overloaded function.

        1. real_chip_measure_prob_count(self: pyQPanda.QPilotOSService, ir: str, shot: int = 1000, chip_id: str = 'any_quantum_chip', is_mapping: bool = True, is_optimization: bool = True, specified_block: List[int] = [], describe: str = '', point_lable: int = 0) -> Dict[str, int]

        2. real_chip_measure_prob_count(self: pyQPanda.QPilotOSService, prog: pyQPanda.QProg, shot: int = 1000, chip_id: str = 'any_quantum_chip', is_mapping: bool = True, is_optimization: bool = True, specified_block: List[int] = [], describe: str = '', point_lable: int = 0) -> Dict[str, int]

        3. real_chip_measure_prob_count(self: pyQPanda.QPilotOSService, ir: List[str], shot: int = 1000, chip_id: str = 'any_quantum_chip', is_mapping: bool = True, is_optimization: bool = True, specified_block: List[int] = [], describe: str = '', point_lable: int = 0) -> List[Dict[str, int]]

        4. real_chip_measure_prob_count(self: pyQPanda.QPilotOSService, prog: List[pyQPanda.QProg], shot: int = 1000, chip_id: str = 'any_quantum_chip', is_mapping: bool = True, is_optimization: bool = True, specified_block: List[int] = [], describe: str = '', point_lable: int = 0) -> List[Dict[str, int]]
        """
    @overload
    def real_chip_measure_prob_count(self, prog: list[QProg], shot: int = ..., chip_id: str = ..., is_mapping: bool = ..., is_optimization: bool = ..., specified_block: list[int] = ..., describe: str = ..., point_lable: int = ...) -> list[dict[str, int]]:
        """real_chip_measure_prob_count(*args, **kwargs)
        Overloaded function.

        1. real_chip_measure_prob_count(self: pyQPanda.QPilotOSService, ir: str, shot: int = 1000, chip_id: str = 'any_quantum_chip', is_mapping: bool = True, is_optimization: bool = True, specified_block: List[int] = [], describe: str = '', point_lable: int = 0) -> Dict[str, int]

        2. real_chip_measure_prob_count(self: pyQPanda.QPilotOSService, prog: pyQPanda.QProg, shot: int = 1000, chip_id: str = 'any_quantum_chip', is_mapping: bool = True, is_optimization: bool = True, specified_block: List[int] = [], describe: str = '', point_lable: int = 0) -> Dict[str, int]

        3. real_chip_measure_prob_count(self: pyQPanda.QPilotOSService, ir: List[str], shot: int = 1000, chip_id: str = 'any_quantum_chip', is_mapping: bool = True, is_optimization: bool = True, specified_block: List[int] = [], describe: str = '', point_lable: int = 0) -> List[Dict[str, int]]

        4. real_chip_measure_prob_count(self: pyQPanda.QPilotOSService, prog: List[pyQPanda.QProg], shot: int = 1000, chip_id: str = 'any_quantum_chip', is_mapping: bool = True, is_optimization: bool = True, specified_block: List[int] = [], describe: str = '', point_lable: int = 0) -> List[Dict[str, int]]
        """
    @overload
    def real_chip_measure_vec(self, prog: list[QProg], shot: int = ..., chip_id: str = ..., is_amend: bool = ..., is_mapping: bool = ..., is_optimization: bool = ..., specified_block: list[int] = ..., describe: str = ..., point_lable: int = ...) -> list[dict[str, float]]:
        """real_chip_measure_vec(*args, **kwargs)
        Overloaded function.

        1. real_chip_measure_vec(self: pyQPanda.QPilotOSService, prog: List[pyQPanda.QProg], shot: int = 1000, chip_id: str = 'any_quantum_chip', is_amend: bool = True, is_mapping: bool = True, is_optimization: bool = True, specified_block: List[int] = [], describe: str = '', point_lable: int = 0) -> List[Dict[str, float]]

        2. real_chip_measure_vec(self: pyQPanda.QPilotOSService, ir: List[str], shot: int = 1000, chip_id: str = 'any_quantum_chip', is_amend: bool = True, is_mapping: bool = True, is_optimization: bool = True, specified_block: List[int] = [], describe: str = '', point_lable: int = 0) -> List[Dict[str, float]]
        """
    @overload
    def real_chip_measure_vec(self, ir: list[str], shot: int = ..., chip_id: str = ..., is_amend: bool = ..., is_mapping: bool = ..., is_optimization: bool = ..., specified_block: list[int] = ..., describe: str = ..., point_lable: int = ...) -> list[dict[str, float]]:
        """real_chip_measure_vec(*args, **kwargs)
        Overloaded function.

        1. real_chip_measure_vec(self: pyQPanda.QPilotOSService, prog: List[pyQPanda.QProg], shot: int = 1000, chip_id: str = 'any_quantum_chip', is_amend: bool = True, is_mapping: bool = True, is_optimization: bool = True, specified_block: List[int] = [], describe: str = '', point_lable: int = 0) -> List[Dict[str, float]]

        2. real_chip_measure_vec(self: pyQPanda.QPilotOSService, ir: List[str], shot: int = 1000, chip_id: str = 'any_quantum_chip', is_amend: bool = True, is_mapping: bool = True, is_optimization: bool = True, specified_block: List[int] = [], describe: str = '', point_lable: int = 0) -> List[Dict[str, float]]
        """
    def runWithConfiguration(self, prog: QProg, shots: int = ..., backend_id: str = ..., noise_model: Noise = ...) -> dict[str, int]:
        """runWithConfiguration(self: pyQPanda.QPilotOSService, prog: pyQPanda.QProg, shots: int = 1000, backend_id: str = 'any_cluster_backend', noise_model: pyQPanda.Noise = <pyQPanda.Noise object at 0x000002856831C2F0>) -> Dict[str, int]"""
    def set_config(self, max_qubit: int, max_cbit: int) -> None:
        """set_config(self: pyQPanda.QPilotOSService, max_qubit: int, max_cbit: int) -> None

        set QVM max qubit and max cbit

        Args:
            max_qubit: quantum machine max qubit num 
            max_cbit: quantum machine max cbit num 

        Returns:
            none
        Raises:
            run_fail: An error occurred in set_configure

        """
    def tcp_recv(self, ip: str, port: int, task_id: str) -> list:
        """tcp_recv(self: pyQPanda.QPilotOSService, ip: str, port: int, task_id: str) -> list"""

class QProg:
    @overload
    def __init__(self) -> None:
        """__init__(*args, **kwargs)
        Overloaded function.

        1. __init__(self: pyQPanda.QProg) -> None

        2. __init__(self: pyQPanda.QProg, arg0: pyQPanda.QProg) -> None

        Construct a program node.

        Args:
             quantum_prog: The quantum program reference.

        Returns:
             A new program node.


        3. __init__(self: pyQPanda.QProg, arg0: pyQPanda.QCircuit) -> None

        Construct a program node from a QCircuit node.

        Args:
             qcircuit: The QCircuit reference.

        Returns:
             A new program node.


        4. __init__(self: pyQPanda.QProg, arg0: pyQPanda.QIfProg) -> None

        Construct a program node from a QIfProg node.

        Args:
             qifprog: The QIfProg reference.

        Returns:
             A new program node.


        5. __init__(self: pyQPanda.QProg, arg0: pyQPanda.QWhileProg) -> None

        Construct a program node from a QWhileProg node.

        Args:
             qwhileprog: The QWhileProg reference.

        Returns:
             A new program node.


        6. __init__(self: pyQPanda.QProg, arg0: pyQPanda.QGate) -> None

        Construct a program node from a QGate node.

        Args:
             qgate: The QGate reference.

        Returns:
             A new program node.


        7. __init__(self: pyQPanda.QProg, arg0: pyQPanda.QMeasure) -> None

        Construct a program node from a QMeasure node.

        Args:
             qmeasure: The QMeasure reference.

        Returns:
             A new program node.


        8. __init__(self: pyQPanda.QProg, arg0: pyQPanda.QReset) -> None

        Construct a program node from a QReset node.

        Args:
             qreset: The QReset reference.

        Returns:
             A new program node.


        9. __init__(self: pyQPanda.QProg, arg0: pyQPanda.ClassicalCondition) -> None

        Construct a program node from a ClassicalCondition node.

        Args:
             classical_condition: The ClassicalCondition reference.

        Returns:
             A new program node.


        10. __init__(self: pyQPanda.QProg, arg0: pyQPanda.NodeIter) -> None

        Construct a program node from a node iterator.

        Args:
             iter: The iterator for the node.

        Returns:
             A new program node.

        """
    @overload
    def __init__(self, arg0: QProg) -> None:
        """__init__(*args, **kwargs)
        Overloaded function.

        1. __init__(self: pyQPanda.QProg) -> None

        2. __init__(self: pyQPanda.QProg, arg0: pyQPanda.QProg) -> None

        Construct a program node.

        Args:
             quantum_prog: The quantum program reference.

        Returns:
             A new program node.


        3. __init__(self: pyQPanda.QProg, arg0: pyQPanda.QCircuit) -> None

        Construct a program node from a QCircuit node.

        Args:
             qcircuit: The QCircuit reference.

        Returns:
             A new program node.


        4. __init__(self: pyQPanda.QProg, arg0: pyQPanda.QIfProg) -> None

        Construct a program node from a QIfProg node.

        Args:
             qifprog: The QIfProg reference.

        Returns:
             A new program node.


        5. __init__(self: pyQPanda.QProg, arg0: pyQPanda.QWhileProg) -> None

        Construct a program node from a QWhileProg node.

        Args:
             qwhileprog: The QWhileProg reference.

        Returns:
             A new program node.


        6. __init__(self: pyQPanda.QProg, arg0: pyQPanda.QGate) -> None

        Construct a program node from a QGate node.

        Args:
             qgate: The QGate reference.

        Returns:
             A new program node.


        7. __init__(self: pyQPanda.QProg, arg0: pyQPanda.QMeasure) -> None

        Construct a program node from a QMeasure node.

        Args:
             qmeasure: The QMeasure reference.

        Returns:
             A new program node.


        8. __init__(self: pyQPanda.QProg, arg0: pyQPanda.QReset) -> None

        Construct a program node from a QReset node.

        Args:
             qreset: The QReset reference.

        Returns:
             A new program node.


        9. __init__(self: pyQPanda.QProg, arg0: pyQPanda.ClassicalCondition) -> None

        Construct a program node from a ClassicalCondition node.

        Args:
             classical_condition: The ClassicalCondition reference.

        Returns:
             A new program node.


        10. __init__(self: pyQPanda.QProg, arg0: pyQPanda.NodeIter) -> None

        Construct a program node from a node iterator.

        Args:
             iter: The iterator for the node.

        Returns:
             A new program node.

        """
    @overload
    def __init__(self, arg0: QCircuit) -> None:
        """__init__(*args, **kwargs)
        Overloaded function.

        1. __init__(self: pyQPanda.QProg) -> None

        2. __init__(self: pyQPanda.QProg, arg0: pyQPanda.QProg) -> None

        Construct a program node.

        Args:
             quantum_prog: The quantum program reference.

        Returns:
             A new program node.


        3. __init__(self: pyQPanda.QProg, arg0: pyQPanda.QCircuit) -> None

        Construct a program node from a QCircuit node.

        Args:
             qcircuit: The QCircuit reference.

        Returns:
             A new program node.


        4. __init__(self: pyQPanda.QProg, arg0: pyQPanda.QIfProg) -> None

        Construct a program node from a QIfProg node.

        Args:
             qifprog: The QIfProg reference.

        Returns:
             A new program node.


        5. __init__(self: pyQPanda.QProg, arg0: pyQPanda.QWhileProg) -> None

        Construct a program node from a QWhileProg node.

        Args:
             qwhileprog: The QWhileProg reference.

        Returns:
             A new program node.


        6. __init__(self: pyQPanda.QProg, arg0: pyQPanda.QGate) -> None

        Construct a program node from a QGate node.

        Args:
             qgate: The QGate reference.

        Returns:
             A new program node.


        7. __init__(self: pyQPanda.QProg, arg0: pyQPanda.QMeasure) -> None

        Construct a program node from a QMeasure node.

        Args:
             qmeasure: The QMeasure reference.

        Returns:
             A new program node.


        8. __init__(self: pyQPanda.QProg, arg0: pyQPanda.QReset) -> None

        Construct a program node from a QReset node.

        Args:
             qreset: The QReset reference.

        Returns:
             A new program node.


        9. __init__(self: pyQPanda.QProg, arg0: pyQPanda.ClassicalCondition) -> None

        Construct a program node from a ClassicalCondition node.

        Args:
             classical_condition: The ClassicalCondition reference.

        Returns:
             A new program node.


        10. __init__(self: pyQPanda.QProg, arg0: pyQPanda.NodeIter) -> None

        Construct a program node from a node iterator.

        Args:
             iter: The iterator for the node.

        Returns:
             A new program node.

        """
    @overload
    def __init__(self, arg0: QIfProg) -> None:
        """__init__(*args, **kwargs)
        Overloaded function.

        1. __init__(self: pyQPanda.QProg) -> None

        2. __init__(self: pyQPanda.QProg, arg0: pyQPanda.QProg) -> None

        Construct a program node.

        Args:
             quantum_prog: The quantum program reference.

        Returns:
             A new program node.


        3. __init__(self: pyQPanda.QProg, arg0: pyQPanda.QCircuit) -> None

        Construct a program node from a QCircuit node.

        Args:
             qcircuit: The QCircuit reference.

        Returns:
             A new program node.


        4. __init__(self: pyQPanda.QProg, arg0: pyQPanda.QIfProg) -> None

        Construct a program node from a QIfProg node.

        Args:
             qifprog: The QIfProg reference.

        Returns:
             A new program node.


        5. __init__(self: pyQPanda.QProg, arg0: pyQPanda.QWhileProg) -> None

        Construct a program node from a QWhileProg node.

        Args:
             qwhileprog: The QWhileProg reference.

        Returns:
             A new program node.


        6. __init__(self: pyQPanda.QProg, arg0: pyQPanda.QGate) -> None

        Construct a program node from a QGate node.

        Args:
             qgate: The QGate reference.

        Returns:
             A new program node.


        7. __init__(self: pyQPanda.QProg, arg0: pyQPanda.QMeasure) -> None

        Construct a program node from a QMeasure node.

        Args:
             qmeasure: The QMeasure reference.

        Returns:
             A new program node.


        8. __init__(self: pyQPanda.QProg, arg0: pyQPanda.QReset) -> None

        Construct a program node from a QReset node.

        Args:
             qreset: The QReset reference.

        Returns:
             A new program node.


        9. __init__(self: pyQPanda.QProg, arg0: pyQPanda.ClassicalCondition) -> None

        Construct a program node from a ClassicalCondition node.

        Args:
             classical_condition: The ClassicalCondition reference.

        Returns:
             A new program node.


        10. __init__(self: pyQPanda.QProg, arg0: pyQPanda.NodeIter) -> None

        Construct a program node from a node iterator.

        Args:
             iter: The iterator for the node.

        Returns:
             A new program node.

        """
    @overload
    def __init__(self, arg0: QWhileProg) -> None:
        """__init__(*args, **kwargs)
        Overloaded function.

        1. __init__(self: pyQPanda.QProg) -> None

        2. __init__(self: pyQPanda.QProg, arg0: pyQPanda.QProg) -> None

        Construct a program node.

        Args:
             quantum_prog: The quantum program reference.

        Returns:
             A new program node.


        3. __init__(self: pyQPanda.QProg, arg0: pyQPanda.QCircuit) -> None

        Construct a program node from a QCircuit node.

        Args:
             qcircuit: The QCircuit reference.

        Returns:
             A new program node.


        4. __init__(self: pyQPanda.QProg, arg0: pyQPanda.QIfProg) -> None

        Construct a program node from a QIfProg node.

        Args:
             qifprog: The QIfProg reference.

        Returns:
             A new program node.


        5. __init__(self: pyQPanda.QProg, arg0: pyQPanda.QWhileProg) -> None

        Construct a program node from a QWhileProg node.

        Args:
             qwhileprog: The QWhileProg reference.

        Returns:
             A new program node.


        6. __init__(self: pyQPanda.QProg, arg0: pyQPanda.QGate) -> None

        Construct a program node from a QGate node.

        Args:
             qgate: The QGate reference.

        Returns:
             A new program node.


        7. __init__(self: pyQPanda.QProg, arg0: pyQPanda.QMeasure) -> None

        Construct a program node from a QMeasure node.

        Args:
             qmeasure: The QMeasure reference.

        Returns:
             A new program node.


        8. __init__(self: pyQPanda.QProg, arg0: pyQPanda.QReset) -> None

        Construct a program node from a QReset node.

        Args:
             qreset: The QReset reference.

        Returns:
             A new program node.


        9. __init__(self: pyQPanda.QProg, arg0: pyQPanda.ClassicalCondition) -> None

        Construct a program node from a ClassicalCondition node.

        Args:
             classical_condition: The ClassicalCondition reference.

        Returns:
             A new program node.


        10. __init__(self: pyQPanda.QProg, arg0: pyQPanda.NodeIter) -> None

        Construct a program node from a node iterator.

        Args:
             iter: The iterator for the node.

        Returns:
             A new program node.

        """
    @overload
    def __init__(self, arg0: QGate) -> None:
        """__init__(*args, **kwargs)
        Overloaded function.

        1. __init__(self: pyQPanda.QProg) -> None

        2. __init__(self: pyQPanda.QProg, arg0: pyQPanda.QProg) -> None

        Construct a program node.

        Args:
             quantum_prog: The quantum program reference.

        Returns:
             A new program node.


        3. __init__(self: pyQPanda.QProg, arg0: pyQPanda.QCircuit) -> None

        Construct a program node from a QCircuit node.

        Args:
             qcircuit: The QCircuit reference.

        Returns:
             A new program node.


        4. __init__(self: pyQPanda.QProg, arg0: pyQPanda.QIfProg) -> None

        Construct a program node from a QIfProg node.

        Args:
             qifprog: The QIfProg reference.

        Returns:
             A new program node.


        5. __init__(self: pyQPanda.QProg, arg0: pyQPanda.QWhileProg) -> None

        Construct a program node from a QWhileProg node.

        Args:
             qwhileprog: The QWhileProg reference.

        Returns:
             A new program node.


        6. __init__(self: pyQPanda.QProg, arg0: pyQPanda.QGate) -> None

        Construct a program node from a QGate node.

        Args:
             qgate: The QGate reference.

        Returns:
             A new program node.


        7. __init__(self: pyQPanda.QProg, arg0: pyQPanda.QMeasure) -> None

        Construct a program node from a QMeasure node.

        Args:
             qmeasure: The QMeasure reference.

        Returns:
             A new program node.


        8. __init__(self: pyQPanda.QProg, arg0: pyQPanda.QReset) -> None

        Construct a program node from a QReset node.

        Args:
             qreset: The QReset reference.

        Returns:
             A new program node.


        9. __init__(self: pyQPanda.QProg, arg0: pyQPanda.ClassicalCondition) -> None

        Construct a program node from a ClassicalCondition node.

        Args:
             classical_condition: The ClassicalCondition reference.

        Returns:
             A new program node.


        10. __init__(self: pyQPanda.QProg, arg0: pyQPanda.NodeIter) -> None

        Construct a program node from a node iterator.

        Args:
             iter: The iterator for the node.

        Returns:
             A new program node.

        """
    @overload
    def __init__(self, arg0: QMeasure) -> None:
        """__init__(*args, **kwargs)
        Overloaded function.

        1. __init__(self: pyQPanda.QProg) -> None

        2. __init__(self: pyQPanda.QProg, arg0: pyQPanda.QProg) -> None

        Construct a program node.

        Args:
             quantum_prog: The quantum program reference.

        Returns:
             A new program node.


        3. __init__(self: pyQPanda.QProg, arg0: pyQPanda.QCircuit) -> None

        Construct a program node from a QCircuit node.

        Args:
             qcircuit: The QCircuit reference.

        Returns:
             A new program node.


        4. __init__(self: pyQPanda.QProg, arg0: pyQPanda.QIfProg) -> None

        Construct a program node from a QIfProg node.

        Args:
             qifprog: The QIfProg reference.

        Returns:
             A new program node.


        5. __init__(self: pyQPanda.QProg, arg0: pyQPanda.QWhileProg) -> None

        Construct a program node from a QWhileProg node.

        Args:
             qwhileprog: The QWhileProg reference.

        Returns:
             A new program node.


        6. __init__(self: pyQPanda.QProg, arg0: pyQPanda.QGate) -> None

        Construct a program node from a QGate node.

        Args:
             qgate: The QGate reference.

        Returns:
             A new program node.


        7. __init__(self: pyQPanda.QProg, arg0: pyQPanda.QMeasure) -> None

        Construct a program node from a QMeasure node.

        Args:
             qmeasure: The QMeasure reference.

        Returns:
             A new program node.


        8. __init__(self: pyQPanda.QProg, arg0: pyQPanda.QReset) -> None

        Construct a program node from a QReset node.

        Args:
             qreset: The QReset reference.

        Returns:
             A new program node.


        9. __init__(self: pyQPanda.QProg, arg0: pyQPanda.ClassicalCondition) -> None

        Construct a program node from a ClassicalCondition node.

        Args:
             classical_condition: The ClassicalCondition reference.

        Returns:
             A new program node.


        10. __init__(self: pyQPanda.QProg, arg0: pyQPanda.NodeIter) -> None

        Construct a program node from a node iterator.

        Args:
             iter: The iterator for the node.

        Returns:
             A new program node.

        """
    @overload
    def __init__(self, arg0: QReset) -> None:
        """__init__(*args, **kwargs)
        Overloaded function.

        1. __init__(self: pyQPanda.QProg) -> None

        2. __init__(self: pyQPanda.QProg, arg0: pyQPanda.QProg) -> None

        Construct a program node.

        Args:
             quantum_prog: The quantum program reference.

        Returns:
             A new program node.


        3. __init__(self: pyQPanda.QProg, arg0: pyQPanda.QCircuit) -> None

        Construct a program node from a QCircuit node.

        Args:
             qcircuit: The QCircuit reference.

        Returns:
             A new program node.


        4. __init__(self: pyQPanda.QProg, arg0: pyQPanda.QIfProg) -> None

        Construct a program node from a QIfProg node.

        Args:
             qifprog: The QIfProg reference.

        Returns:
             A new program node.


        5. __init__(self: pyQPanda.QProg, arg0: pyQPanda.QWhileProg) -> None

        Construct a program node from a QWhileProg node.

        Args:
             qwhileprog: The QWhileProg reference.

        Returns:
             A new program node.


        6. __init__(self: pyQPanda.QProg, arg0: pyQPanda.QGate) -> None

        Construct a program node from a QGate node.

        Args:
             qgate: The QGate reference.

        Returns:
             A new program node.


        7. __init__(self: pyQPanda.QProg, arg0: pyQPanda.QMeasure) -> None

        Construct a program node from a QMeasure node.

        Args:
             qmeasure: The QMeasure reference.

        Returns:
             A new program node.


        8. __init__(self: pyQPanda.QProg, arg0: pyQPanda.QReset) -> None

        Construct a program node from a QReset node.

        Args:
             qreset: The QReset reference.

        Returns:
             A new program node.


        9. __init__(self: pyQPanda.QProg, arg0: pyQPanda.ClassicalCondition) -> None

        Construct a program node from a ClassicalCondition node.

        Args:
             classical_condition: The ClassicalCondition reference.

        Returns:
             A new program node.


        10. __init__(self: pyQPanda.QProg, arg0: pyQPanda.NodeIter) -> None

        Construct a program node from a node iterator.

        Args:
             iter: The iterator for the node.

        Returns:
             A new program node.

        """
    @overload
    def __init__(self, arg0: ClassicalCondition) -> None:
        """__init__(*args, **kwargs)
        Overloaded function.

        1. __init__(self: pyQPanda.QProg) -> None

        2. __init__(self: pyQPanda.QProg, arg0: pyQPanda.QProg) -> None

        Construct a program node.

        Args:
             quantum_prog: The quantum program reference.

        Returns:
             A new program node.


        3. __init__(self: pyQPanda.QProg, arg0: pyQPanda.QCircuit) -> None

        Construct a program node from a QCircuit node.

        Args:
             qcircuit: The QCircuit reference.

        Returns:
             A new program node.


        4. __init__(self: pyQPanda.QProg, arg0: pyQPanda.QIfProg) -> None

        Construct a program node from a QIfProg node.

        Args:
             qifprog: The QIfProg reference.

        Returns:
             A new program node.


        5. __init__(self: pyQPanda.QProg, arg0: pyQPanda.QWhileProg) -> None

        Construct a program node from a QWhileProg node.

        Args:
             qwhileprog: The QWhileProg reference.

        Returns:
             A new program node.


        6. __init__(self: pyQPanda.QProg, arg0: pyQPanda.QGate) -> None

        Construct a program node from a QGate node.

        Args:
             qgate: The QGate reference.

        Returns:
             A new program node.


        7. __init__(self: pyQPanda.QProg, arg0: pyQPanda.QMeasure) -> None

        Construct a program node from a QMeasure node.

        Args:
             qmeasure: The QMeasure reference.

        Returns:
             A new program node.


        8. __init__(self: pyQPanda.QProg, arg0: pyQPanda.QReset) -> None

        Construct a program node from a QReset node.

        Args:
             qreset: The QReset reference.

        Returns:
             A new program node.


        9. __init__(self: pyQPanda.QProg, arg0: pyQPanda.ClassicalCondition) -> None

        Construct a program node from a ClassicalCondition node.

        Args:
             classical_condition: The ClassicalCondition reference.

        Returns:
             A new program node.


        10. __init__(self: pyQPanda.QProg, arg0: pyQPanda.NodeIter) -> None

        Construct a program node from a node iterator.

        Args:
             iter: The iterator for the node.

        Returns:
             A new program node.

        """
    @overload
    def __init__(self, arg0: NodeIter) -> None:
        """__init__(*args, **kwargs)
        Overloaded function.

        1. __init__(self: pyQPanda.QProg) -> None

        2. __init__(self: pyQPanda.QProg, arg0: pyQPanda.QProg) -> None

        Construct a program node.

        Args:
             quantum_prog: The quantum program reference.

        Returns:
             A new program node.


        3. __init__(self: pyQPanda.QProg, arg0: pyQPanda.QCircuit) -> None

        Construct a program node from a QCircuit node.

        Args:
             qcircuit: The QCircuit reference.

        Returns:
             A new program node.


        4. __init__(self: pyQPanda.QProg, arg0: pyQPanda.QIfProg) -> None

        Construct a program node from a QIfProg node.

        Args:
             qifprog: The QIfProg reference.

        Returns:
             A new program node.


        5. __init__(self: pyQPanda.QProg, arg0: pyQPanda.QWhileProg) -> None

        Construct a program node from a QWhileProg node.

        Args:
             qwhileprog: The QWhileProg reference.

        Returns:
             A new program node.


        6. __init__(self: pyQPanda.QProg, arg0: pyQPanda.QGate) -> None

        Construct a program node from a QGate node.

        Args:
             qgate: The QGate reference.

        Returns:
             A new program node.


        7. __init__(self: pyQPanda.QProg, arg0: pyQPanda.QMeasure) -> None

        Construct a program node from a QMeasure node.

        Args:
             qmeasure: The QMeasure reference.

        Returns:
             A new program node.


        8. __init__(self: pyQPanda.QProg, arg0: pyQPanda.QReset) -> None

        Construct a program node from a QReset node.

        Args:
             qreset: The QReset reference.

        Returns:
             A new program node.


        9. __init__(self: pyQPanda.QProg, arg0: pyQPanda.ClassicalCondition) -> None

        Construct a program node from a ClassicalCondition node.

        Args:
             classical_condition: The ClassicalCondition reference.

        Returns:
             A new program node.


        10. __init__(self: pyQPanda.QProg, arg0: pyQPanda.NodeIter) -> None

        Construct a program node from a node iterator.

        Args:
             iter: The iterator for the node.

        Returns:
             A new program node.

        """
    def begin(self) -> NodeIter:
        """begin(self: pyQPanda.QProg) -> pyQPanda.NodeIter

        Get an iterator to the first node in the program.

        Returns:
             A reference to the iterator pointing to the first node.

        """
    def end(self) -> NodeIter:
        """end(self: pyQPanda.QProg) -> pyQPanda.NodeIter

        Get an iterator to the end of the program.

        Returns:
             A reference to the iterator pointing past the last node.

        """
    def get_max_qubit_addr(self) -> int:
        """get_max_qubit_addr(self: pyQPanda.QProg) -> int"""
    def get_qgate_num(self) -> int:
        """get_qgate_num(self: pyQPanda.QProg) -> int"""
    def get_used_cbits(self, cbit_vector: list[ClassicalCondition]) -> list[ClassicalCondition]:
        """get_used_cbits(self: pyQPanda.QProg, cbit_vector: List[pyQPanda.ClassicalCondition]) -> List[pyQPanda.ClassicalCondition]

        Get a list of classical bits used in the program.

        Args:
             cbit_vector: The vector to store the used classical bits.

        Returns:
             A reference to the updated classical bit vector.

        """
    def get_used_qubits(self, qubit_vector: QVec) -> QVec:
        """get_used_qubits(self: pyQPanda.QProg, qubit_vector: pyQPanda.QVec) -> pyQPanda.QVec

        Get a list of qubits used in the program.

        Args:
             qubit_vector: The vector to store the used qubits.

        Returns:
             A reference to the updated qubit vector.

        """
    def head(self) -> NodeIter:
        """head(self: pyQPanda.QProg) -> pyQPanda.NodeIter

        Get an iterator to the head node of the program.

        Returns:
             A reference to the iterator pointing to the head node.

        """
    @overload
    def insert(self, arg0: QProg) -> QProg:
        """insert(*args, **kwargs)
        Overloaded function.

        1. insert(self: pyQPanda.QProg, arg0: pyQPanda.QProg) -> pyQPanda.QProg

        Insert a program into the current program.

        Args:
             program: The program to be inserted.

        Returns:
             A reference to the updated program after insertion.


        2. insert(self: pyQPanda.QProg, arg0: pyQPanda.QGate) -> pyQPanda.QProg

        Insert a gate into the current program.

        Args:
             gate: The gate to be inserted.

        Returns:
             A reference to the updated program after insertion.


        3. insert(self: pyQPanda.QProg, arg0: pyQPanda.QCircuit) -> pyQPanda.QProg

        Insert a circuit into the current program.

        Args:
             circuit: The circuit to be inserted.

        Returns:
             A reference to the updated program after insertion.


        4. insert(self: pyQPanda.QProg, arg0: pyQPanda.QIfProg) -> pyQPanda.QProg

        Insert an if program into the current program.

        Args:
             if_program: The if program to be inserted.

        Returns:
             A reference to the updated program after insertion.


        5. insert(self: pyQPanda.QProg, arg0: pyQPanda.QWhileProg) -> pyQPanda.QProg

        Insert a QWhileProg into the program.

        Returns:
             A reference to the updated program.


        6. insert(self: pyQPanda.QProg, arg0: pyQPanda.QMeasure) -> pyQPanda.QProg

        Insert a QMeasure into the program.

        Returns:
             A reference to the updated program.


        7. insert(self: pyQPanda.QProg, arg0: pyQPanda.QReset) -> pyQPanda.QProg

        Insert a QReset into the program.

        Args:
             reset_op: The reset operation to be inserted.

        Returns:
             A reference to the updated program.


        8. insert(self: pyQPanda.QProg, arg0: pyQPanda.ClassicalCondition) -> pyQPanda.QProg

        Insert a ClassicalCondition into the program.

        Args:
             condition_op: The classical condition operation to be inserted.

        Returns:
             A reference to the updated program.

        """
    @overload
    def insert(self, arg0: QGate) -> QProg:
        """insert(*args, **kwargs)
        Overloaded function.

        1. insert(self: pyQPanda.QProg, arg0: pyQPanda.QProg) -> pyQPanda.QProg

        Insert a program into the current program.

        Args:
             program: The program to be inserted.

        Returns:
             A reference to the updated program after insertion.


        2. insert(self: pyQPanda.QProg, arg0: pyQPanda.QGate) -> pyQPanda.QProg

        Insert a gate into the current program.

        Args:
             gate: The gate to be inserted.

        Returns:
             A reference to the updated program after insertion.


        3. insert(self: pyQPanda.QProg, arg0: pyQPanda.QCircuit) -> pyQPanda.QProg

        Insert a circuit into the current program.

        Args:
             circuit: The circuit to be inserted.

        Returns:
             A reference to the updated program after insertion.


        4. insert(self: pyQPanda.QProg, arg0: pyQPanda.QIfProg) -> pyQPanda.QProg

        Insert an if program into the current program.

        Args:
             if_program: The if program to be inserted.

        Returns:
             A reference to the updated program after insertion.


        5. insert(self: pyQPanda.QProg, arg0: pyQPanda.QWhileProg) -> pyQPanda.QProg

        Insert a QWhileProg into the program.

        Returns:
             A reference to the updated program.


        6. insert(self: pyQPanda.QProg, arg0: pyQPanda.QMeasure) -> pyQPanda.QProg

        Insert a QMeasure into the program.

        Returns:
             A reference to the updated program.


        7. insert(self: pyQPanda.QProg, arg0: pyQPanda.QReset) -> pyQPanda.QProg

        Insert a QReset into the program.

        Args:
             reset_op: The reset operation to be inserted.

        Returns:
             A reference to the updated program.


        8. insert(self: pyQPanda.QProg, arg0: pyQPanda.ClassicalCondition) -> pyQPanda.QProg

        Insert a ClassicalCondition into the program.

        Args:
             condition_op: The classical condition operation to be inserted.

        Returns:
             A reference to the updated program.

        """
    @overload
    def insert(self, arg0: QCircuit) -> QProg:
        """insert(*args, **kwargs)
        Overloaded function.

        1. insert(self: pyQPanda.QProg, arg0: pyQPanda.QProg) -> pyQPanda.QProg

        Insert a program into the current program.

        Args:
             program: The program to be inserted.

        Returns:
             A reference to the updated program after insertion.


        2. insert(self: pyQPanda.QProg, arg0: pyQPanda.QGate) -> pyQPanda.QProg

        Insert a gate into the current program.

        Args:
             gate: The gate to be inserted.

        Returns:
             A reference to the updated program after insertion.


        3. insert(self: pyQPanda.QProg, arg0: pyQPanda.QCircuit) -> pyQPanda.QProg

        Insert a circuit into the current program.

        Args:
             circuit: The circuit to be inserted.

        Returns:
             A reference to the updated program after insertion.


        4. insert(self: pyQPanda.QProg, arg0: pyQPanda.QIfProg) -> pyQPanda.QProg

        Insert an if program into the current program.

        Args:
             if_program: The if program to be inserted.

        Returns:
             A reference to the updated program after insertion.


        5. insert(self: pyQPanda.QProg, arg0: pyQPanda.QWhileProg) -> pyQPanda.QProg

        Insert a QWhileProg into the program.

        Returns:
             A reference to the updated program.


        6. insert(self: pyQPanda.QProg, arg0: pyQPanda.QMeasure) -> pyQPanda.QProg

        Insert a QMeasure into the program.

        Returns:
             A reference to the updated program.


        7. insert(self: pyQPanda.QProg, arg0: pyQPanda.QReset) -> pyQPanda.QProg

        Insert a QReset into the program.

        Args:
             reset_op: The reset operation to be inserted.

        Returns:
             A reference to the updated program.


        8. insert(self: pyQPanda.QProg, arg0: pyQPanda.ClassicalCondition) -> pyQPanda.QProg

        Insert a ClassicalCondition into the program.

        Args:
             condition_op: The classical condition operation to be inserted.

        Returns:
             A reference to the updated program.

        """
    @overload
    def insert(self, arg0: QIfProg) -> QProg:
        """insert(*args, **kwargs)
        Overloaded function.

        1. insert(self: pyQPanda.QProg, arg0: pyQPanda.QProg) -> pyQPanda.QProg

        Insert a program into the current program.

        Args:
             program: The program to be inserted.

        Returns:
             A reference to the updated program after insertion.


        2. insert(self: pyQPanda.QProg, arg0: pyQPanda.QGate) -> pyQPanda.QProg

        Insert a gate into the current program.

        Args:
             gate: The gate to be inserted.

        Returns:
             A reference to the updated program after insertion.


        3. insert(self: pyQPanda.QProg, arg0: pyQPanda.QCircuit) -> pyQPanda.QProg

        Insert a circuit into the current program.

        Args:
             circuit: The circuit to be inserted.

        Returns:
             A reference to the updated program after insertion.


        4. insert(self: pyQPanda.QProg, arg0: pyQPanda.QIfProg) -> pyQPanda.QProg

        Insert an if program into the current program.

        Args:
             if_program: The if program to be inserted.

        Returns:
             A reference to the updated program after insertion.


        5. insert(self: pyQPanda.QProg, arg0: pyQPanda.QWhileProg) -> pyQPanda.QProg

        Insert a QWhileProg into the program.

        Returns:
             A reference to the updated program.


        6. insert(self: pyQPanda.QProg, arg0: pyQPanda.QMeasure) -> pyQPanda.QProg

        Insert a QMeasure into the program.

        Returns:
             A reference to the updated program.


        7. insert(self: pyQPanda.QProg, arg0: pyQPanda.QReset) -> pyQPanda.QProg

        Insert a QReset into the program.

        Args:
             reset_op: The reset operation to be inserted.

        Returns:
             A reference to the updated program.


        8. insert(self: pyQPanda.QProg, arg0: pyQPanda.ClassicalCondition) -> pyQPanda.QProg

        Insert a ClassicalCondition into the program.

        Args:
             condition_op: The classical condition operation to be inserted.

        Returns:
             A reference to the updated program.

        """
    @overload
    def insert(self, arg0: QWhileProg) -> QProg:
        """insert(*args, **kwargs)
        Overloaded function.

        1. insert(self: pyQPanda.QProg, arg0: pyQPanda.QProg) -> pyQPanda.QProg

        Insert a program into the current program.

        Args:
             program: The program to be inserted.

        Returns:
             A reference to the updated program after insertion.


        2. insert(self: pyQPanda.QProg, arg0: pyQPanda.QGate) -> pyQPanda.QProg

        Insert a gate into the current program.

        Args:
             gate: The gate to be inserted.

        Returns:
             A reference to the updated program after insertion.


        3. insert(self: pyQPanda.QProg, arg0: pyQPanda.QCircuit) -> pyQPanda.QProg

        Insert a circuit into the current program.

        Args:
             circuit: The circuit to be inserted.

        Returns:
             A reference to the updated program after insertion.


        4. insert(self: pyQPanda.QProg, arg0: pyQPanda.QIfProg) -> pyQPanda.QProg

        Insert an if program into the current program.

        Args:
             if_program: The if program to be inserted.

        Returns:
             A reference to the updated program after insertion.


        5. insert(self: pyQPanda.QProg, arg0: pyQPanda.QWhileProg) -> pyQPanda.QProg

        Insert a QWhileProg into the program.

        Returns:
             A reference to the updated program.


        6. insert(self: pyQPanda.QProg, arg0: pyQPanda.QMeasure) -> pyQPanda.QProg

        Insert a QMeasure into the program.

        Returns:
             A reference to the updated program.


        7. insert(self: pyQPanda.QProg, arg0: pyQPanda.QReset) -> pyQPanda.QProg

        Insert a QReset into the program.

        Args:
             reset_op: The reset operation to be inserted.

        Returns:
             A reference to the updated program.


        8. insert(self: pyQPanda.QProg, arg0: pyQPanda.ClassicalCondition) -> pyQPanda.QProg

        Insert a ClassicalCondition into the program.

        Args:
             condition_op: The classical condition operation to be inserted.

        Returns:
             A reference to the updated program.

        """
    @overload
    def insert(self, arg0: QMeasure) -> QProg:
        """insert(*args, **kwargs)
        Overloaded function.

        1. insert(self: pyQPanda.QProg, arg0: pyQPanda.QProg) -> pyQPanda.QProg

        Insert a program into the current program.

        Args:
             program: The program to be inserted.

        Returns:
             A reference to the updated program after insertion.


        2. insert(self: pyQPanda.QProg, arg0: pyQPanda.QGate) -> pyQPanda.QProg

        Insert a gate into the current program.

        Args:
             gate: The gate to be inserted.

        Returns:
             A reference to the updated program after insertion.


        3. insert(self: pyQPanda.QProg, arg0: pyQPanda.QCircuit) -> pyQPanda.QProg

        Insert a circuit into the current program.

        Args:
             circuit: The circuit to be inserted.

        Returns:
             A reference to the updated program after insertion.


        4. insert(self: pyQPanda.QProg, arg0: pyQPanda.QIfProg) -> pyQPanda.QProg

        Insert an if program into the current program.

        Args:
             if_program: The if program to be inserted.

        Returns:
             A reference to the updated program after insertion.


        5. insert(self: pyQPanda.QProg, arg0: pyQPanda.QWhileProg) -> pyQPanda.QProg

        Insert a QWhileProg into the program.

        Returns:
             A reference to the updated program.


        6. insert(self: pyQPanda.QProg, arg0: pyQPanda.QMeasure) -> pyQPanda.QProg

        Insert a QMeasure into the program.

        Returns:
             A reference to the updated program.


        7. insert(self: pyQPanda.QProg, arg0: pyQPanda.QReset) -> pyQPanda.QProg

        Insert a QReset into the program.

        Args:
             reset_op: The reset operation to be inserted.

        Returns:
             A reference to the updated program.


        8. insert(self: pyQPanda.QProg, arg0: pyQPanda.ClassicalCondition) -> pyQPanda.QProg

        Insert a ClassicalCondition into the program.

        Args:
             condition_op: The classical condition operation to be inserted.

        Returns:
             A reference to the updated program.

        """
    @overload
    def insert(self, arg0: QReset) -> QProg:
        """insert(*args, **kwargs)
        Overloaded function.

        1. insert(self: pyQPanda.QProg, arg0: pyQPanda.QProg) -> pyQPanda.QProg

        Insert a program into the current program.

        Args:
             program: The program to be inserted.

        Returns:
             A reference to the updated program after insertion.


        2. insert(self: pyQPanda.QProg, arg0: pyQPanda.QGate) -> pyQPanda.QProg

        Insert a gate into the current program.

        Args:
             gate: The gate to be inserted.

        Returns:
             A reference to the updated program after insertion.


        3. insert(self: pyQPanda.QProg, arg0: pyQPanda.QCircuit) -> pyQPanda.QProg

        Insert a circuit into the current program.

        Args:
             circuit: The circuit to be inserted.

        Returns:
             A reference to the updated program after insertion.


        4. insert(self: pyQPanda.QProg, arg0: pyQPanda.QIfProg) -> pyQPanda.QProg

        Insert an if program into the current program.

        Args:
             if_program: The if program to be inserted.

        Returns:
             A reference to the updated program after insertion.


        5. insert(self: pyQPanda.QProg, arg0: pyQPanda.QWhileProg) -> pyQPanda.QProg

        Insert a QWhileProg into the program.

        Returns:
             A reference to the updated program.


        6. insert(self: pyQPanda.QProg, arg0: pyQPanda.QMeasure) -> pyQPanda.QProg

        Insert a QMeasure into the program.

        Returns:
             A reference to the updated program.


        7. insert(self: pyQPanda.QProg, arg0: pyQPanda.QReset) -> pyQPanda.QProg

        Insert a QReset into the program.

        Args:
             reset_op: The reset operation to be inserted.

        Returns:
             A reference to the updated program.


        8. insert(self: pyQPanda.QProg, arg0: pyQPanda.ClassicalCondition) -> pyQPanda.QProg

        Insert a ClassicalCondition into the program.

        Args:
             condition_op: The classical condition operation to be inserted.

        Returns:
             A reference to the updated program.

        """
    @overload
    def insert(self, arg0: ClassicalCondition) -> QProg:
        """insert(*args, **kwargs)
        Overloaded function.

        1. insert(self: pyQPanda.QProg, arg0: pyQPanda.QProg) -> pyQPanda.QProg

        Insert a program into the current program.

        Args:
             program: The program to be inserted.

        Returns:
             A reference to the updated program after insertion.


        2. insert(self: pyQPanda.QProg, arg0: pyQPanda.QGate) -> pyQPanda.QProg

        Insert a gate into the current program.

        Args:
             gate: The gate to be inserted.

        Returns:
             A reference to the updated program after insertion.


        3. insert(self: pyQPanda.QProg, arg0: pyQPanda.QCircuit) -> pyQPanda.QProg

        Insert a circuit into the current program.

        Args:
             circuit: The circuit to be inserted.

        Returns:
             A reference to the updated program after insertion.


        4. insert(self: pyQPanda.QProg, arg0: pyQPanda.QIfProg) -> pyQPanda.QProg

        Insert an if program into the current program.

        Args:
             if_program: The if program to be inserted.

        Returns:
             A reference to the updated program after insertion.


        5. insert(self: pyQPanda.QProg, arg0: pyQPanda.QWhileProg) -> pyQPanda.QProg

        Insert a QWhileProg into the program.

        Returns:
             A reference to the updated program.


        6. insert(self: pyQPanda.QProg, arg0: pyQPanda.QMeasure) -> pyQPanda.QProg

        Insert a QMeasure into the program.

        Returns:
             A reference to the updated program.


        7. insert(self: pyQPanda.QProg, arg0: pyQPanda.QReset) -> pyQPanda.QProg

        Insert a QReset into the program.

        Args:
             reset_op: The reset operation to be inserted.

        Returns:
             A reference to the updated program.


        8. insert(self: pyQPanda.QProg, arg0: pyQPanda.ClassicalCondition) -> pyQPanda.QProg

        Insert a ClassicalCondition into the program.

        Args:
             condition_op: The classical condition operation to be inserted.

        Returns:
             A reference to the updated program.

        """
    def is_empty(self) -> bool:
        """is_empty(self: pyQPanda.QProg) -> bool"""
    def is_measure_last_pos(self) -> bool:
        """is_measure_last_pos(self: pyQPanda.QProg) -> bool

        Check if the last operation in the program is a measurement.

        Returns:
             True if the last operation is a measurement, otherwise False.

        """
    def last(self) -> NodeIter:
        """last(self: pyQPanda.QProg) -> pyQPanda.NodeIter

        Get an iterator to the last node in the program.

        Args:
             None

        Returns:
             A reference to the iterator pointing to the last node.

        """
    @overload
    def __lshift__(self, arg0: QProg) -> QProg:
        """__lshift__(*args, **kwargs)
        Overloaded function.

        1. __lshift__(self: pyQPanda.QProg, arg0: pyQPanda.QProg) -> pyQPanda.QProg

        2. __lshift__(self: pyQPanda.QProg, arg0: pyQPanda.QGate) -> pyQPanda.QProg

        3. __lshift__(self: pyQPanda.QProg, arg0: pyQPanda.QCircuit) -> pyQPanda.QProg

        4. __lshift__(self: pyQPanda.QProg, arg0: pyQPanda.QIfProg) -> pyQPanda.QProg

        5. __lshift__(self: pyQPanda.QProg, arg0: pyQPanda.QWhileProg) -> pyQPanda.QProg

        6. __lshift__(self: pyQPanda.QProg, arg0: pyQPanda.QMeasure) -> pyQPanda.QProg

        7. __lshift__(self: pyQPanda.QProg, arg0: pyQPanda.QReset) -> pyQPanda.QProg

        8. __lshift__(self: pyQPanda.QProg, arg0: pyQPanda.ClassicalCondition) -> pyQPanda.QProg
        """
    @overload
    def __lshift__(self, arg0: QGate) -> QProg:
        """__lshift__(*args, **kwargs)
        Overloaded function.

        1. __lshift__(self: pyQPanda.QProg, arg0: pyQPanda.QProg) -> pyQPanda.QProg

        2. __lshift__(self: pyQPanda.QProg, arg0: pyQPanda.QGate) -> pyQPanda.QProg

        3. __lshift__(self: pyQPanda.QProg, arg0: pyQPanda.QCircuit) -> pyQPanda.QProg

        4. __lshift__(self: pyQPanda.QProg, arg0: pyQPanda.QIfProg) -> pyQPanda.QProg

        5. __lshift__(self: pyQPanda.QProg, arg0: pyQPanda.QWhileProg) -> pyQPanda.QProg

        6. __lshift__(self: pyQPanda.QProg, arg0: pyQPanda.QMeasure) -> pyQPanda.QProg

        7. __lshift__(self: pyQPanda.QProg, arg0: pyQPanda.QReset) -> pyQPanda.QProg

        8. __lshift__(self: pyQPanda.QProg, arg0: pyQPanda.ClassicalCondition) -> pyQPanda.QProg
        """
    @overload
    def __lshift__(self, arg0: QCircuit) -> QProg:
        """__lshift__(*args, **kwargs)
        Overloaded function.

        1. __lshift__(self: pyQPanda.QProg, arg0: pyQPanda.QProg) -> pyQPanda.QProg

        2. __lshift__(self: pyQPanda.QProg, arg0: pyQPanda.QGate) -> pyQPanda.QProg

        3. __lshift__(self: pyQPanda.QProg, arg0: pyQPanda.QCircuit) -> pyQPanda.QProg

        4. __lshift__(self: pyQPanda.QProg, arg0: pyQPanda.QIfProg) -> pyQPanda.QProg

        5. __lshift__(self: pyQPanda.QProg, arg0: pyQPanda.QWhileProg) -> pyQPanda.QProg

        6. __lshift__(self: pyQPanda.QProg, arg0: pyQPanda.QMeasure) -> pyQPanda.QProg

        7. __lshift__(self: pyQPanda.QProg, arg0: pyQPanda.QReset) -> pyQPanda.QProg

        8. __lshift__(self: pyQPanda.QProg, arg0: pyQPanda.ClassicalCondition) -> pyQPanda.QProg
        """
    @overload
    def __lshift__(self, arg0: QIfProg) -> QProg:
        """__lshift__(*args, **kwargs)
        Overloaded function.

        1. __lshift__(self: pyQPanda.QProg, arg0: pyQPanda.QProg) -> pyQPanda.QProg

        2. __lshift__(self: pyQPanda.QProg, arg0: pyQPanda.QGate) -> pyQPanda.QProg

        3. __lshift__(self: pyQPanda.QProg, arg0: pyQPanda.QCircuit) -> pyQPanda.QProg

        4. __lshift__(self: pyQPanda.QProg, arg0: pyQPanda.QIfProg) -> pyQPanda.QProg

        5. __lshift__(self: pyQPanda.QProg, arg0: pyQPanda.QWhileProg) -> pyQPanda.QProg

        6. __lshift__(self: pyQPanda.QProg, arg0: pyQPanda.QMeasure) -> pyQPanda.QProg

        7. __lshift__(self: pyQPanda.QProg, arg0: pyQPanda.QReset) -> pyQPanda.QProg

        8. __lshift__(self: pyQPanda.QProg, arg0: pyQPanda.ClassicalCondition) -> pyQPanda.QProg
        """
    @overload
    def __lshift__(self, arg0: QWhileProg) -> QProg:
        """__lshift__(*args, **kwargs)
        Overloaded function.

        1. __lshift__(self: pyQPanda.QProg, arg0: pyQPanda.QProg) -> pyQPanda.QProg

        2. __lshift__(self: pyQPanda.QProg, arg0: pyQPanda.QGate) -> pyQPanda.QProg

        3. __lshift__(self: pyQPanda.QProg, arg0: pyQPanda.QCircuit) -> pyQPanda.QProg

        4. __lshift__(self: pyQPanda.QProg, arg0: pyQPanda.QIfProg) -> pyQPanda.QProg

        5. __lshift__(self: pyQPanda.QProg, arg0: pyQPanda.QWhileProg) -> pyQPanda.QProg

        6. __lshift__(self: pyQPanda.QProg, arg0: pyQPanda.QMeasure) -> pyQPanda.QProg

        7. __lshift__(self: pyQPanda.QProg, arg0: pyQPanda.QReset) -> pyQPanda.QProg

        8. __lshift__(self: pyQPanda.QProg, arg0: pyQPanda.ClassicalCondition) -> pyQPanda.QProg
        """
    @overload
    def __lshift__(self, arg0: QMeasure) -> QProg:
        """__lshift__(*args, **kwargs)
        Overloaded function.

        1. __lshift__(self: pyQPanda.QProg, arg0: pyQPanda.QProg) -> pyQPanda.QProg

        2. __lshift__(self: pyQPanda.QProg, arg0: pyQPanda.QGate) -> pyQPanda.QProg

        3. __lshift__(self: pyQPanda.QProg, arg0: pyQPanda.QCircuit) -> pyQPanda.QProg

        4. __lshift__(self: pyQPanda.QProg, arg0: pyQPanda.QIfProg) -> pyQPanda.QProg

        5. __lshift__(self: pyQPanda.QProg, arg0: pyQPanda.QWhileProg) -> pyQPanda.QProg

        6. __lshift__(self: pyQPanda.QProg, arg0: pyQPanda.QMeasure) -> pyQPanda.QProg

        7. __lshift__(self: pyQPanda.QProg, arg0: pyQPanda.QReset) -> pyQPanda.QProg

        8. __lshift__(self: pyQPanda.QProg, arg0: pyQPanda.ClassicalCondition) -> pyQPanda.QProg
        """
    @overload
    def __lshift__(self, arg0: QReset) -> QProg:
        """__lshift__(*args, **kwargs)
        Overloaded function.

        1. __lshift__(self: pyQPanda.QProg, arg0: pyQPanda.QProg) -> pyQPanda.QProg

        2. __lshift__(self: pyQPanda.QProg, arg0: pyQPanda.QGate) -> pyQPanda.QProg

        3. __lshift__(self: pyQPanda.QProg, arg0: pyQPanda.QCircuit) -> pyQPanda.QProg

        4. __lshift__(self: pyQPanda.QProg, arg0: pyQPanda.QIfProg) -> pyQPanda.QProg

        5. __lshift__(self: pyQPanda.QProg, arg0: pyQPanda.QWhileProg) -> pyQPanda.QProg

        6. __lshift__(self: pyQPanda.QProg, arg0: pyQPanda.QMeasure) -> pyQPanda.QProg

        7. __lshift__(self: pyQPanda.QProg, arg0: pyQPanda.QReset) -> pyQPanda.QProg

        8. __lshift__(self: pyQPanda.QProg, arg0: pyQPanda.ClassicalCondition) -> pyQPanda.QProg
        """
    @overload
    def __lshift__(self, arg0: ClassicalCondition) -> QProg:
        """__lshift__(*args, **kwargs)
        Overloaded function.

        1. __lshift__(self: pyQPanda.QProg, arg0: pyQPanda.QProg) -> pyQPanda.QProg

        2. __lshift__(self: pyQPanda.QProg, arg0: pyQPanda.QGate) -> pyQPanda.QProg

        3. __lshift__(self: pyQPanda.QProg, arg0: pyQPanda.QCircuit) -> pyQPanda.QProg

        4. __lshift__(self: pyQPanda.QProg, arg0: pyQPanda.QIfProg) -> pyQPanda.QProg

        5. __lshift__(self: pyQPanda.QProg, arg0: pyQPanda.QWhileProg) -> pyQPanda.QProg

        6. __lshift__(self: pyQPanda.QProg, arg0: pyQPanda.QMeasure) -> pyQPanda.QProg

        7. __lshift__(self: pyQPanda.QProg, arg0: pyQPanda.QReset) -> pyQPanda.QProg

        8. __lshift__(self: pyQPanda.QProg, arg0: pyQPanda.ClassicalCondition) -> pyQPanda.QProg
        """

class QProgDAG:
    def __init__(self) -> None:
        """__init__(self: pyQPanda.QProgDAG) -> None"""
    def get_edges(self) -> list[QProgDAGEdge]:
        """get_edges(self: pyQPanda.QProgDAG) -> List[pyQPanda.QProgDAGEdge]

        Retrieve the set of edges in the quantum program DAG.

        Returns:
             List[QProgDAGEdge]: A list of edges in the DAG.

        """
    def get_target_vertex(self, vertice_num: int) -> QProgDAGVertex:
        """get_target_vertex(self: pyQPanda.QProgDAG, vertice_num: int) -> pyQPanda.QProgDAGVertex

        Retrieve a target vertex from the quantum program DAG.

        Args:
             vertice_num: The index of the vertex to retrieve.

        Returns:
             QVertex: A reference to the specified vertex in the DAG.

        """
    def get_vertex_set(self) -> list[QProgDAGVertex]:
        """get_vertex_set(self: pyQPanda.QProgDAG) -> List[pyQPanda.QProgDAGVertex]

        Retrieve the set of vertices in the quantum program DAG.

        Args:
             QVec: The set of vertices.

        Returns:
             QVec: A reference to the vector of vertices in the DAG.

        """

class QProgDAGEdge:
    m_from: int
    m_qubit: int
    m_to: int
    def __init__(self, from_arg: int, to_arg: int, qubit_arg: int) -> None:
        """__init__(self: pyQPanda.QProgDAGEdge, from_arg: int, to_arg: int, qubit_arg: int) -> None

        Initialize a quantum program DAG edge.

        Args:
             from_arg: The starting node of the edge.

             to_arg: The ending node of the edge.

             qubit_arg: The qubit associated with the edge.

        """

class QProgDAGVertex:
    m_id: int
    m_layer: int
    m_pre_node: list[int]
    m_succ_node: list[int]
    m_type: DAGNodeType
    def __init__(self) -> None:
        """__init__(self: pyQPanda.QProgDAGVertex) -> None

        Initialize a quantum program DAG vertex.
        This vertex represents a node in the quantum program's directed acyclic graph (DAG).
        """
    def get_control_vec(self) -> QVec:
        """get_control_vec(self: pyQPanda.QProgDAGVertex) -> pyQPanda.QVec

        Retrieve the vector of control qubits associated with the vertex.
        """
    def get_iter(self) -> NodeIter:
        """get_iter(self: pyQPanda.QProgDAGVertex) -> pyQPanda.NodeIter

        Retrieve the iterator associated with the vertex.
        """
    def get_qubits_vec(self) -> QVec:
        """get_qubits_vec(self: pyQPanda.QProgDAGVertex) -> pyQPanda.QVec

        Retrieve the vector of qubits associated with the vertex.
        """
    def is_dagger(self) -> bool:
        """is_dagger(self: pyQPanda.QProgDAGVertex) -> bool

        Check if the vertex is a dagger operation.
        """

class QReset:
    def __init__(self, arg0: NodeIter) -> None:
        """__init__(self: pyQPanda.QReset, arg0: pyQPanda.NodeIter) -> None

        Initialize QReset from a node iterator.

        Args:
             iter (NodeIter&): The iterator pointing to the node.

        Returns:
             QReset: The initialized reset object.

        Raises:
             runtime_error: If the iterator is null or the node type is incorrect.

        """

class QResult:
    def __init__(self, *args, **kwargs) -> None:
        """Initialize self.  See help(type(self)) for accurate signature."""
    def getResultMap(self) -> dict[str, bool]:
        """getResultMap(self: pyQPanda.QResult) -> Dict[str, bool]"""

class QVec:
    @overload
    def __init__(self) -> None:
        """__init__(*args, **kwargs)
        Overloaded function.

        1. __init__(self: pyQPanda.QVec) -> None

        Default constructor for QVec.

        Args:
             None

        Returns:
             A new instance of QVec.


        2. __init__(self: pyQPanda.QVec, qubit_list: List[pyQPanda.Qubit]) -> None

        Constructor that initializes QVec with a list of qubits.

        Args:
             qubit_list: A reference to a vector of pointers to Qubit instances.

        Returns:
             A new instance of QVec initialized with the provided qubits.


        3. __init__(self: pyQPanda.QVec, qvec: pyQPanda.QVec) -> None

        Copy constructor for QVec.

        Args:
             qvec: A reference to an existing QVec instance.

        Returns:
             A new instance of QVec that is a copy of the provided instance.


        4. __init__(self: pyQPanda.QVec, qubit: pyQPanda.Qubit) -> None

        Constructor that initializes QVec with a single qubit.

        Args:
             qubit: A pointer to a Qubit instance.

        Returns:
             A new instance of QVec initialized with the provided qubit.

        """
    @overload
    def __init__(self, qubit_list: list[Qubit]) -> None:
        """__init__(*args, **kwargs)
        Overloaded function.

        1. __init__(self: pyQPanda.QVec) -> None

        Default constructor for QVec.

        Args:
             None

        Returns:
             A new instance of QVec.


        2. __init__(self: pyQPanda.QVec, qubit_list: List[pyQPanda.Qubit]) -> None

        Constructor that initializes QVec with a list of qubits.

        Args:
             qubit_list: A reference to a vector of pointers to Qubit instances.

        Returns:
             A new instance of QVec initialized with the provided qubits.


        3. __init__(self: pyQPanda.QVec, qvec: pyQPanda.QVec) -> None

        Copy constructor for QVec.

        Args:
             qvec: A reference to an existing QVec instance.

        Returns:
             A new instance of QVec that is a copy of the provided instance.


        4. __init__(self: pyQPanda.QVec, qubit: pyQPanda.Qubit) -> None

        Constructor that initializes QVec with a single qubit.

        Args:
             qubit: A pointer to a Qubit instance.

        Returns:
             A new instance of QVec initialized with the provided qubit.

        """
    @overload
    def __init__(self, qvec: QVec) -> None:
        """__init__(*args, **kwargs)
        Overloaded function.

        1. __init__(self: pyQPanda.QVec) -> None

        Default constructor for QVec.

        Args:
             None

        Returns:
             A new instance of QVec.


        2. __init__(self: pyQPanda.QVec, qubit_list: List[pyQPanda.Qubit]) -> None

        Constructor that initializes QVec with a list of qubits.

        Args:
             qubit_list: A reference to a vector of pointers to Qubit instances.

        Returns:
             A new instance of QVec initialized with the provided qubits.


        3. __init__(self: pyQPanda.QVec, qvec: pyQPanda.QVec) -> None

        Copy constructor for QVec.

        Args:
             qvec: A reference to an existing QVec instance.

        Returns:
             A new instance of QVec that is a copy of the provided instance.


        4. __init__(self: pyQPanda.QVec, qubit: pyQPanda.Qubit) -> None

        Constructor that initializes QVec with a single qubit.

        Args:
             qubit: A pointer to a Qubit instance.

        Returns:
             A new instance of QVec initialized with the provided qubit.

        """
    @overload
    def __init__(self, qubit: Qubit) -> None:
        """__init__(*args, **kwargs)
        Overloaded function.

        1. __init__(self: pyQPanda.QVec) -> None

        Default constructor for QVec.

        Args:
             None

        Returns:
             A new instance of QVec.


        2. __init__(self: pyQPanda.QVec, qubit_list: List[pyQPanda.Qubit]) -> None

        Constructor that initializes QVec with a list of qubits.

        Args:
             qubit_list: A reference to a vector of pointers to Qubit instances.

        Returns:
             A new instance of QVec initialized with the provided qubits.


        3. __init__(self: pyQPanda.QVec, qvec: pyQPanda.QVec) -> None

        Copy constructor for QVec.

        Args:
             qvec: A reference to an existing QVec instance.

        Returns:
             A new instance of QVec that is a copy of the provided instance.


        4. __init__(self: pyQPanda.QVec, qubit: pyQPanda.Qubit) -> None

        Constructor that initializes QVec with a single qubit.

        Args:
             qubit: A pointer to a Qubit instance.

        Returns:
             A new instance of QVec initialized with the provided qubit.

        """
    def append(self, qubit: Qubit) -> None:
        """append(self: pyQPanda.QVec, qubit: pyQPanda.Qubit) -> None

        Add a qubit to the end of the QVec.

        Args:
             qubit: A pointer to the Qubit to be added.

        Returns:
             None.

        """
    def pop(self) -> None:
        """pop(self: pyQPanda.QVec) -> None

        Remove and return the last qubit from the QVec.

        Args:
             None

        Returns:
             A pointer to the removed Qubit.

        """
    def to_list(self) -> list[Qubit]:
        """to_list(self: pyQPanda.QVec) -> List[pyQPanda.Qubit]

        Convert the QVec to a standard vector of qubit pointers.

        Args:
             None

        Returns:
             A vector containing pointers to the qubits in the QVec.

        """
    @overload
    def __getitem__(self, qubit_addr: int) -> Qubit:
        """__getitem__(*args, **kwargs)
        Overloaded function.

        1. __getitem__(self: pyQPanda.QVec, qubit_addr: int) -> pyQPanda.Qubit

        Retrieve a qubit by its index.

        Args:
             qubit_addr: The index of the qubit to retrieve.

        Returns:
             A reference to the Qubit at the specified index.


        2. __getitem__(self: pyQPanda.QVec, classical_cond: QPanda::ClassicalCondition) -> pyQPanda.Qubit

        Retrieve a qubit based on a classical condition.

        Args:
             classical_cond: The classical condition to evaluate.

        Returns:
             A reference to the Qubit associated with the provided condition.

        """
    @overload
    def __getitem__(self, classical_cond) -> Qubit:
        """__getitem__(*args, **kwargs)
        Overloaded function.

        1. __getitem__(self: pyQPanda.QVec, qubit_addr: int) -> pyQPanda.Qubit

        Retrieve a qubit by its index.

        Args:
             qubit_addr: The index of the qubit to retrieve.

        Returns:
             A reference to the Qubit at the specified index.


        2. __getitem__(self: pyQPanda.QVec, classical_cond: QPanda::ClassicalCondition) -> pyQPanda.Qubit

        Retrieve a qubit based on a classical condition.

        Args:
             classical_cond: The classical condition to evaluate.

        Returns:
             A reference to the Qubit associated with the provided condition.

        """
    def __iter__(self) -> typing.Iterator[Qubit]:
        """def __iter__(self) -> typing.Iterator[Qubit]"""
    def __len__(self) -> int:
        """__len__(self: pyQPanda.QVec) -> int

        Get the number of qubits in the QVec.

        Args:
             None

        Returns:
             The count of qubits in the QVec.

        """

class QWhileProg:
    @overload
    def __init__(self, arg0: NodeIter) -> None:
        """__init__(*args, **kwargs)
        Overloaded function.

        1. __init__(self: pyQPanda.QWhileProg, arg0: pyQPanda.NodeIter) -> None

        Initialize QWhileProg from a node iterator.

        Args:
             iter (NodeIter&): The iterator pointing to the node.

        Raises:
             runtime_error: If the iterator is null or the node type is incorrect.


        2. __init__(self: pyQPanda.QWhileProg, arg0: pyQPanda.ClassicalCondition, arg1: pyQPanda.QProg) -> None
        """
    @overload
    def __init__(self, arg0: ClassicalCondition, arg1: QProg) -> None:
        """__init__(*args, **kwargs)
        Overloaded function.

        1. __init__(self: pyQPanda.QWhileProg, arg0: pyQPanda.NodeIter) -> None

        Initialize QWhileProg from a node iterator.

        Args:
             iter (NodeIter&): The iterator pointing to the node.

        Raises:
             runtime_error: If the iterator is null or the node type is incorrect.


        2. __init__(self: pyQPanda.QWhileProg, arg0: pyQPanda.ClassicalCondition, arg1: pyQPanda.QProg) -> None
        """
    def get_classical_condition(self) -> ClassicalCondition:
        """get_classical_condition(self: pyQPanda.QWhileProg) -> pyQPanda.ClassicalCondition

        Retrieve the classical condition associated with the while program.

        Returns:
             The classical condition object used in the while statement.

        """
    def get_true_branch(self) -> QProg:
        """get_true_branch(self: pyQPanda.QWhileProg) -> pyQPanda.QProg

        Retrieve the quantum program corresponding to the true branch.

        Returns:
             QProg: The quantum program for the true branch.

        Raises:
             runtime_error: If the true branch is null or has an incorrect node type.

        """

class QuantumMachine:
    def __init__(self, *args, **kwargs) -> None:
        """Initialize self.  See help(type(self)) for accurate signature."""
    def allocate_qubit_through_phy_address(self, address: int) -> Qubit:
        """allocate_qubit_through_phy_address(self: pyQPanda.QuantumMachine, address: int) -> pyQPanda.Qubit

        Allocate qubits through physical address.

        This function allocates qubits using the specified physical address.

        Args:
             address: The physical address of the qubit.

        Returns:
             The allocated qubit.

        """
    def allocate_qubit_through_vir_address(self, address: int) -> Qubit:
        """allocate_qubit_through_vir_address(self: pyQPanda.QuantumMachine, address: int) -> pyQPanda.Qubit

        Allocate a qubit using its physical address.

        This function allocates a qubit based on the specified physical address.

        Args:
             address: The physical address of the qubit to allocate.

        Returns:
             A reference to the allocated Qubit.

        """
    def async_run(self, qprog: QProg, noise_model: Noise = ...) -> None:
        """async_run(self: pyQPanda.QuantumMachine, qprog: pyQPanda.QProg, noise_model: pyQPanda.Noise = NoiseModel()) -> None

        Execute the quantum program asynchronously in the background.

        This function runs the specified quantum program without blocking the main thread.

        You can check the progress using get_processed_qgate_num(), determine if the process

        is finished with is_async_finished(), and retrieve results with get_async_result().

        Args:
             qprog: The quantum program to run.

             noise_model: (optional) The noise model to apply (default is NoiseModel()).

        Returns:
             A reference indicating the status of the asynchronous operation.

        """
    @overload
    def cAlloc(self) -> ClassicalCondition:
        """cAlloc(*args, **kwargs)
        Overloaded function.

        1. cAlloc(self: pyQPanda.QuantumMachine) -> pyQPanda.ClassicalCondition

        Allocate a classical bit (CBit).

        This function must be called after init().

        Args:
             None

        Returns:
             CBit: A reference to the allocated classical bit.


        2. cAlloc(self: pyQPanda.QuantumMachine, cbit: int) -> pyQPanda.ClassicalCondition

        Allocate a classical bit (CBit) in the QuantumMachine.

        This function allocates a CBit after the quantum machine has been initialized.

        Args:
             cbit_addr: The address of the CBit to allocate, which should be in the range [0, 29).

        Returns:
             Reference to the allocated CBit.

        """
    @overload
    def cAlloc(self, cbit: int) -> ClassicalCondition:
        """cAlloc(*args, **kwargs)
        Overloaded function.

        1. cAlloc(self: pyQPanda.QuantumMachine) -> pyQPanda.ClassicalCondition

        Allocate a classical bit (CBit).

        This function must be called after init().

        Args:
             None

        Returns:
             CBit: A reference to the allocated classical bit.


        2. cAlloc(self: pyQPanda.QuantumMachine, cbit: int) -> pyQPanda.ClassicalCondition

        Allocate a classical bit (CBit) in the QuantumMachine.

        This function allocates a CBit after the quantum machine has been initialized.

        Args:
             cbit_addr: The address of the CBit to allocate, which should be in the range [0, 29).

        Returns:
             Reference to the allocated CBit.

        """
    def cAlloc_many(self, cbit_num: int) -> list[ClassicalCondition]:
        """cAlloc_many(self: pyQPanda.QuantumMachine, cbit_num: int) -> List[pyQPanda.ClassicalCondition]

        Allocate multiple classical bits (CBits).

        This function must be called after init().

        Args:
             cbit_num: The number of classical bits to allocate.

        Returns:
             list[CBit]: A list of allocated classical bits.

        """
    def cFree(self, arg0: ClassicalCondition) -> None:
        """cFree(self: pyQPanda.QuantumMachine, arg0: pyQPanda.ClassicalCondition) -> None

        Free a classical bit (CBit).

        This function deallocates a previously allocated classical bit.

        Args:
             CBit: The classical bit to be freed.

        Returns:
             None: This function does not return a value.

        """
    @overload
    def cFree_all(self, cbit_list: list[ClassicalCondition]) -> None:
        """cFree_all(*args, **kwargs)
        Overloaded function.

        1. cFree_all(self: pyQPanda.QuantumMachine, cbit_list: List[pyQPanda.ClassicalCondition]) -> None

        Free all allocated classical bits (CBits).

        This function deallocates all classical bits provided in the input list.

        Args:
             None

        Returns:
             None: This function does not return a value.


        2. cFree_all(self: pyQPanda.QuantumMachine) -> None

        Free all classical bits (CBits).

        This function deallocates all classical bits that have been previously allocated.

        Args:
             None

        Returns:
             None: This function does not return a value.

        """
    @overload
    def cFree_all(self) -> None:
        """cFree_all(*args, **kwargs)
        Overloaded function.

        1. cFree_all(self: pyQPanda.QuantumMachine, cbit_list: List[pyQPanda.ClassicalCondition]) -> None

        Free all allocated classical bits (CBits).

        This function deallocates all classical bits provided in the input list.

        Args:
             None

        Returns:
             None: This function does not return a value.


        2. cFree_all(self: pyQPanda.QuantumMachine) -> None

        Free all classical bits (CBits).

        This function deallocates all classical bits that have been previously allocated.

        Args:
             None

        Returns:
             None: This function does not return a value.

        """
    def directly_run(self, qprog: QProg, noise_model: Noise = ...) -> dict[str, bool]:
        """directly_run(self: pyQPanda.QuantumMachine, qprog: pyQPanda.QProg, noise_model: pyQPanda.Noise = NoiseModel()) -> Dict[str, bool]

        Directly execute the quantum program.

        This function runs the specified quantum program immediately after the

        initialization (init()). It supports an optional noise model, which is

        currently only applicable to CPUQVM.

        Args:
             qprog: The quantum program to run.

             noise_model: (optional) The noise model to apply (default is no noise).

        Returns:
             A dictionary with the execution results:
                 The final qubit register state.
                 The measurement probabilities.

        """
    def finalize(self) -> None:
        """finalize(self: pyQPanda.QuantumMachine) -> None

        Finalize the quantum machine.

        Args:
             None

        Returns:
             None

        """
    def getAllocateCMem(self) -> int:
        """getAllocateCMem(self: pyQPanda.QuantumMachine) -> int

        Get the list of allocated classical bits (cbits) in the QuantumMachine.

        This function retrieves the cbits that have been allocated for use in the quantum machine.

        Args:
             None

        Returns:
             List of allocated cbits.

        """
    def getAllocateQubitNum(self) -> int:
        """getAllocateQubitNum(self: pyQPanda.QuantumMachine) -> int

        Get the list of allocated qubits in the QuantumMachine.

        This function retrieves the qubits that have been allocated for use in the quantum machine.

        Args:
             None

        Returns:
             List of allocated qubits.

        """
    def getStatus(self, *args, **kwargs):
        """getStatus(self: pyQPanda.QuantumMachine) -> QPanda::QMachineStatus

        Get the status of the Quantum machine.

        This function retrieves the current status of the Quantum machine.

        Args:
             None

        Returns:
             QMachineStatus: The status of the Quantum machine.

        """
    def get_allocate_cbits(self) -> list[ClassicalCondition]:
        """get_allocate_cbits(self: pyQPanda.QuantumMachine) -> List[pyQPanda.ClassicalCondition]

        Retrieve the list of allocated cbits in the QuantumMachine.

        This function returns a list of currently allocated cbits.

        Args:
             None

        Returns:
             A list of allocated cbits.

        """
    def get_allocate_cmem_num(self) -> int:
        """get_allocate_cmem_num(self: pyQPanda.QuantumMachine) -> int

        Retrieve the list of allocated cbits in the QuantumMachine.

        This function returns the currently allocated cbits.

        Args:
             None

        Returns:
             A list of allocated cbits.

        """
    def get_allocate_qubit_num(self) -> int:
        """get_allocate_qubit_num(self: pyQPanda.QuantumMachine) -> int

        Retrieve the list of allocated qubits in the QuantumMachine.

        This function returns the currently allocated qubits.

        Args:
             None

        Returns:
             A list of allocated qubits.

        """
    def get_allocate_qubits(self) -> list[Qubit]:
        """get_allocate_qubits(self: pyQPanda.QuantumMachine) -> List[pyQPanda.Qubit]

        Retrieve the list of allocated qubits in the QuantumMachine.

        This function returns a list of currently allocated qubits.

        Args:
             None

        Returns:
             A list of allocated qubits.

        """
    def get_async_result(self) -> dict[str, bool]:
        """get_async_result(self: pyQPanda.QuantumMachine) -> Dict[str, bool]

        Retrieve the result of the asynchronous quantum program execution.

        This function blocks the current code until the asynchronous process initiated

        by async_run() is complete, then returns the results.

        Returns:
             The result of the asynchronous execution.

        """
    @overload
    def get_expectation(self, qprog: QProg, hamiltonian: list[tuple[dict[int, str], float]], qubit_list: QVec) -> float:
        """get_expectation(*args, **kwargs)
        Overloaded function.

        1. get_expectation(self: pyQPanda.QuantumMachine, qprog: pyQPanda.QProg, hamiltonian: List[Tuple[Dict[int, str], float]], qubit_list: pyQPanda.QVec) -> float

        Calculate the expectation value of the given Hamiltonian.

        This function computes the expectation value based on the provided quantum program,

        Hamiltonian, and list of qubits to measure.

        Args:
             qprog: The quantum program to execute.

             hamiltonian: The Hamiltonian for which the expectation is calculated.

             qubit_list: A list of qubits to measure.

        Returns:
             A double representing the expectation value of the current Hamiltonian.


        2. get_expectation(self: pyQPanda.QuantumMachine, qprog: pyQPanda.QProg, hamiltonian: List[Tuple[Dict[int, str], float]], qubit_list: pyQPanda.QVec, shots: int) -> float

        Calculate the expectation value of the given Hamiltonian with specified measurement shots.

        This function computes the expectation value based on the provided quantum program,

        Hamiltonian, list of qubits to measure, and the number of measurement shots.

        Args:
             qprog: The quantum program to execute.

             hamiltonian: The Hamiltonian for which the expectation is calculated.

             qubit_list: A list of qubits to measure.

             shots: The number of measurement shots to perform.

        Returns:
             A double representing the expectation value of the current Hamiltonian.

        """
    @overload
    def get_expectation(self, qprog: QProg, hamiltonian: list[tuple[dict[int, str], float]], qubit_list: QVec, shots: int) -> float:
        """get_expectation(*args, **kwargs)
        Overloaded function.

        1. get_expectation(self: pyQPanda.QuantumMachine, qprog: pyQPanda.QProg, hamiltonian: List[Tuple[Dict[int, str], float]], qubit_list: pyQPanda.QVec) -> float

        Calculate the expectation value of the given Hamiltonian.

        This function computes the expectation value based on the provided quantum program,

        Hamiltonian, and list of qubits to measure.

        Args:
             qprog: The quantum program to execute.

             hamiltonian: The Hamiltonian for which the expectation is calculated.

             qubit_list: A list of qubits to measure.

        Returns:
             A double representing the expectation value of the current Hamiltonian.


        2. get_expectation(self: pyQPanda.QuantumMachine, qprog: pyQPanda.QProg, hamiltonian: List[Tuple[Dict[int, str], float]], qubit_list: pyQPanda.QVec, shots: int) -> float

        Calculate the expectation value of the given Hamiltonian with specified measurement shots.

        This function computes the expectation value based on the provided quantum program,

        Hamiltonian, list of qubits to measure, and the number of measurement shots.

        Args:
             qprog: The quantum program to execute.

             hamiltonian: The Hamiltonian for which the expectation is calculated.

             qubit_list: A list of qubits to measure.

             shots: The number of measurement shots to perform.

        Returns:
             A double representing the expectation value of the current Hamiltonian.

        """
    def get_gate_time_map(self) -> dict[GateType, int]:
        """get_gate_time_map(self: pyQPanda.QuantumMachine) -> Dict[pyQPanda.GateType, int]

        Retrieve the gate time mapping for the QuantumMachine.

        This function returns a map of gates to their corresponding execution times.

        Args:
             None

        Returns:
             A reference to the gate time map.

        """
    def get_processed_qgate_num(self) -> int:
        """get_processed_qgate_num(self: pyQPanda.QuantumMachine) -> int

        Retrieve the number of processed quantum gates.

        This function returns the total count of quantum gates that have been processed

        by the QuantumMachine.

        Returns:
             An integer representing the number of processed quantum gates.

        """
    def get_qstate(self) -> list[complex]:
        """get_qstate(self: pyQPanda.QuantumMachine) -> List[complex]

        Get the status of the quantum machine.

        Args:
             None

        Returns:
             QMachineStatus: The current status of the quantum machine.

        """
    def get_status(self, *args, **kwargs):
        """get_status(self: pyQPanda.QuantumMachine) -> QPanda::QMachineStatus

        Retrieve the status of the QuantumMachine.

        This function returns the current status of the quantum machine.

        Args:
             None

        Returns:
             The status of the Quantum machine, represented as a QMachineStatus.

        """
    def initQVM(self) -> None:
        """initQVM(self: pyQPanda.QuantumMachine) -> None

        Initialize the global unique quantum machine in the background.

        This function sets up the quantum machine based on the specified type.

        Args:
             machine_type: The type of quantum machine to initialize, as defined in pyQPanda.QMachineType.

        Returns:
             bool: True if the initialization is successful, otherwise false.

        """
    def init_qvm(self) -> None:
        """init_qvm(self: pyQPanda.QuantumMachine) -> None

        Initialize the global unique quantum machine in the background.

        This function sets up the quantum machine based on the specified type.

        Args:
             machine_type: The type of quantum machine to initialize, as defined in pyQPanda.QMachineType.

        Returns:
             bool: True if the initialization is successful, otherwise false.

        """
    def init_sparse_state(self, *args, **kwargs):
        """init_sparse_state(self: pyQPanda.QuantumMachine, state: Dict[str, complex] = std::map<std::string, qcomplex_t>(), qlist: pyQPanda.QVec = QVec()) -> None

        Initialize a sparse quantum state for the QuantumMachine.

        This function sets the initial sparse state of the quantum machine.

        Args:
             state: A map representing the sparse state, where keys are state identifiers and values are qcomplex_t. Defaults to an empty map.

             qlist: The list of qubits to which the sparse state will be applied, represented as a QVec object. Defaults to QVec().

        Returns:
             Reference to the updated quantum machine.

        """
    def init_state(self, state: list[complex] = ..., qlist: QVec = ...) -> None:
        """init_state(self: pyQPanda.QuantumMachine, state: List[complex] = QStat(), qlist: pyQPanda.QVec = QVec()) -> None

        Initialize the quantum state of the QuantumMachine.

        This function sets the initial state of the quantum machine.

        Args:
             state: The initial quantum state, represented as a QStat object. Defaults to QStat().

             qlist: The list of qubits to which the state will be applied, represented as a QVec object. Defaults to QVec().

        Returns:
             Reference to the updated quantum machine.

        """
    def is_async_finished(self) -> bool:
        """is_async_finished(self: pyQPanda.QuantumMachine) -> bool

        Check if the asynchronous quantum program execution is complete.

        This function returns a boolean indicating whether the asynchronous process

        initiated by async_run() has finished.

        Returns:
             True if the process is complete, False otherwise.

        """
    def qAlloc(self) -> Qubit:
        """qAlloc(self: pyQPanda.QuantumMachine) -> pyQPanda.Qubit

        Allocate a qubit.

        This function must be called after init().

        Args:
             qubit_addr: The physical address of the qubit, should be in the range [0, 29).

        """
    def qAlloc_many(self, qubit_num: int) -> list[Qubit]:
        """qAlloc_many(self: pyQPanda.QuantumMachine, qubit_num: int) -> List[pyQPanda.Qubit]

        Allocate multiple qubits.

        This function must be called after init().

        Args:
             qubit_num: The number of qubits to allocate.

        Returns:
             list[Qubit]: A list of allocated qubits.

        """
    def qFree(self, qubit: Qubit) -> None:
        """qFree(self: pyQPanda.QuantumMachine, qubit: pyQPanda.Qubit) -> None

        Free a qubit.

        This function deallocates a previously allocated qubit.

        Args:
             qubit: The Qubit to be freed.

        Returns:
             None: This function does not return a value.

        """
    @overload
    def qFree_all(self, qubit_list: QVec) -> None:
        """qFree_all(*args, **kwargs)
        Overloaded function.

        1. qFree_all(self: pyQPanda.QuantumMachine, qubit_list: pyQPanda.QVec) -> None

        Free all allocated qubits.

        This function deallocates all qubits that have been previously allocated.

        Args:
             None

        Returns:
             None: This function does not return a value.


        2. qFree_all(self: pyQPanda.QuantumMachine, arg0: pyQPanda.QVec) -> None

        Free all qubits.

        This function deallocates all qubits provided in the input vector.

        Args:
             None

        Returns:
             None: This function does not return a value.

        """
    @overload
    def qFree_all(self, arg0: QVec) -> None:
        """qFree_all(*args, **kwargs)
        Overloaded function.

        1. qFree_all(self: pyQPanda.QuantumMachine, qubit_list: pyQPanda.QVec) -> None

        Free all allocated qubits.

        This function deallocates all qubits that have been previously allocated.

        Args:
             None

        Returns:
             None: This function does not return a value.


        2. qFree_all(self: pyQPanda.QuantumMachine, arg0: pyQPanda.QVec) -> None

        Free all qubits.

        This function deallocates all qubits provided in the input vector.

        Args:
             None

        Returns:
             None: This function does not return a value.

        """
    @overload
    def run_with_configuration(self, qprog: QProg, cbit_list: list[ClassicalCondition], data: dict, noise_model: Noise = ...) -> dict[str, int]:
        """run_with_configuration(*args, **kwargs)
        Overloaded function.

        1. run_with_configuration(self: pyQPanda.QuantumMachine, qprog: pyQPanda.QProg, cbit_list: List[pyQPanda.ClassicalCondition], data: dict, noise_model: pyQPanda.Noise = NoiseModel()) -> Dict[str, int]

        Execute the quantum program with a specified configuration.

        This function runs the quantum program using the provided classical bits,

        parameters, and an optional noise model. It supports multiple shots for

        repeated execution.

        Args:
             qprog: The quantum program to execute.

             cbit_list: The list of classical bits.

             data: Parameters for the execution in dictionary form.

             noise_model: (optional) The noise model to apply (default is no noise).

        Returns:
             The execution results over the specified shots, including:
                 The final qubit register state.
                 The count of hits for each outcome.


        2. run_with_configuration(self: pyQPanda.QuantumMachine, qprog: pyQPanda.QProg, cbit_list: List[pyQPanda.ClassicalCondition], shot: int, noise_model: pyQPanda.Noise = NoiseModel()) -> Dict[str, int]

        Execute the quantum program with a specified configuration.

        This function runs the quantum program using the provided classical bits,

        parameters, and an optional noise model. It supports multiple shots for

        repeated execution.

        Args:
             qprog: The quantum program to execute.

             cbit_list: The list of classical bits.

             shot: The number of times to repeat the execution.

             noise_model: (optional) The noise model to apply (default is no noise).

        Note: Noise models currently only work on CPUQVM.

        Returns:
             A tuple containing the results of the quantum program execution:
                 The final qubit register state.
                 The count of hits for each outcome.


        3. run_with_configuration(self: pyQPanda.QuantumMachine, qprog: pyQPanda.QProg, shot: int, noise_model: pyQPanda.Noise = NoiseModel()) -> Dict[str, int]

        Execute the quantum program with a specified configuration.

        This function runs the quantum program using the provided quantum program,

        number of shots, and an optional noise model. It supports multiple shots

        for repeated execution.

        Args:
             qprog: The quantum program to execute.

             shot: The number of times to repeat the execution.

             noise_model: (optional) The noise model to apply (default is no noise).

        Returns:
             The execution results over the specified shots, including:
                 The final qubit register state.
                 The count of hits for each outcome.


        4. run_with_configuration(self: pyQPanda.QuantumMachine, qprog: pyQPanda.QProg, cbit_list: List[int], shot: int, noise_model: pyQPanda.Noise = NoiseModel()) -> Dict[str, int]

        Execute the quantum program with a specified configuration.

        This function runs the quantum program using the provided classical bits,

        the number of shots for repeated execution, and an optional noise model.

        Args:
             qprog: The quantum program to execute.

             cbit_list: The list of classical bits.

             shot: The number of times to repeat the execution.

             noise_model: (optional) The noise model to apply (default is no noise). Note: Noise models currently work only on CPUQVM.

        Returns:
             A tuple containing the execution results over the specified shots:
                 The final qubit register state.
                 The count of hits for each outcome.

        """
    @overload
    def run_with_configuration(self, qprog: QProg, cbit_list: list[ClassicalCondition], shot: int, noise_model: Noise = ...) -> dict[str, int]:
        """run_with_configuration(*args, **kwargs)
        Overloaded function.

        1. run_with_configuration(self: pyQPanda.QuantumMachine, qprog: pyQPanda.QProg, cbit_list: List[pyQPanda.ClassicalCondition], data: dict, noise_model: pyQPanda.Noise = NoiseModel()) -> Dict[str, int]

        Execute the quantum program with a specified configuration.

        This function runs the quantum program using the provided classical bits,

        parameters, and an optional noise model. It supports multiple shots for

        repeated execution.

        Args:
             qprog: The quantum program to execute.

             cbit_list: The list of classical bits.

             data: Parameters for the execution in dictionary form.

             noise_model: (optional) The noise model to apply (default is no noise).

        Returns:
             The execution results over the specified shots, including:
                 The final qubit register state.
                 The count of hits for each outcome.


        2. run_with_configuration(self: pyQPanda.QuantumMachine, qprog: pyQPanda.QProg, cbit_list: List[pyQPanda.ClassicalCondition], shot: int, noise_model: pyQPanda.Noise = NoiseModel()) -> Dict[str, int]

        Execute the quantum program with a specified configuration.

        This function runs the quantum program using the provided classical bits,

        parameters, and an optional noise model. It supports multiple shots for

        repeated execution.

        Args:
             qprog: The quantum program to execute.

             cbit_list: The list of classical bits.

             shot: The number of times to repeat the execution.

             noise_model: (optional) The noise model to apply (default is no noise).

        Note: Noise models currently only work on CPUQVM.

        Returns:
             A tuple containing the results of the quantum program execution:
                 The final qubit register state.
                 The count of hits for each outcome.


        3. run_with_configuration(self: pyQPanda.QuantumMachine, qprog: pyQPanda.QProg, shot: int, noise_model: pyQPanda.Noise = NoiseModel()) -> Dict[str, int]

        Execute the quantum program with a specified configuration.

        This function runs the quantum program using the provided quantum program,

        number of shots, and an optional noise model. It supports multiple shots

        for repeated execution.

        Args:
             qprog: The quantum program to execute.

             shot: The number of times to repeat the execution.

             noise_model: (optional) The noise model to apply (default is no noise).

        Returns:
             The execution results over the specified shots, including:
                 The final qubit register state.
                 The count of hits for each outcome.


        4. run_with_configuration(self: pyQPanda.QuantumMachine, qprog: pyQPanda.QProg, cbit_list: List[int], shot: int, noise_model: pyQPanda.Noise = NoiseModel()) -> Dict[str, int]

        Execute the quantum program with a specified configuration.

        This function runs the quantum program using the provided classical bits,

        the number of shots for repeated execution, and an optional noise model.

        Args:
             qprog: The quantum program to execute.

             cbit_list: The list of classical bits.

             shot: The number of times to repeat the execution.

             noise_model: (optional) The noise model to apply (default is no noise). Note: Noise models currently work only on CPUQVM.

        Returns:
             A tuple containing the execution results over the specified shots:
                 The final qubit register state.
                 The count of hits for each outcome.

        """
    @overload
    def run_with_configuration(self, qprog: QProg, shot: int, noise_model: Noise = ...) -> dict[str, int]:
        """run_with_configuration(*args, **kwargs)
        Overloaded function.

        1. run_with_configuration(self: pyQPanda.QuantumMachine, qprog: pyQPanda.QProg, cbit_list: List[pyQPanda.ClassicalCondition], data: dict, noise_model: pyQPanda.Noise = NoiseModel()) -> Dict[str, int]

        Execute the quantum program with a specified configuration.

        This function runs the quantum program using the provided classical bits,

        parameters, and an optional noise model. It supports multiple shots for

        repeated execution.

        Args:
             qprog: The quantum program to execute.

             cbit_list: The list of classical bits.

             data: Parameters for the execution in dictionary form.

             noise_model: (optional) The noise model to apply (default is no noise).

        Returns:
             The execution results over the specified shots, including:
                 The final qubit register state.
                 The count of hits for each outcome.


        2. run_with_configuration(self: pyQPanda.QuantumMachine, qprog: pyQPanda.QProg, cbit_list: List[pyQPanda.ClassicalCondition], shot: int, noise_model: pyQPanda.Noise = NoiseModel()) -> Dict[str, int]

        Execute the quantum program with a specified configuration.

        This function runs the quantum program using the provided classical bits,

        parameters, and an optional noise model. It supports multiple shots for

        repeated execution.

        Args:
             qprog: The quantum program to execute.

             cbit_list: The list of classical bits.

             shot: The number of times to repeat the execution.

             noise_model: (optional) The noise model to apply (default is no noise).

        Note: Noise models currently only work on CPUQVM.

        Returns:
             A tuple containing the results of the quantum program execution:
                 The final qubit register state.
                 The count of hits for each outcome.


        3. run_with_configuration(self: pyQPanda.QuantumMachine, qprog: pyQPanda.QProg, shot: int, noise_model: pyQPanda.Noise = NoiseModel()) -> Dict[str, int]

        Execute the quantum program with a specified configuration.

        This function runs the quantum program using the provided quantum program,

        number of shots, and an optional noise model. It supports multiple shots

        for repeated execution.

        Args:
             qprog: The quantum program to execute.

             shot: The number of times to repeat the execution.

             noise_model: (optional) The noise model to apply (default is no noise).

        Returns:
             The execution results over the specified shots, including:
                 The final qubit register state.
                 The count of hits for each outcome.


        4. run_with_configuration(self: pyQPanda.QuantumMachine, qprog: pyQPanda.QProg, cbit_list: List[int], shot: int, noise_model: pyQPanda.Noise = NoiseModel()) -> Dict[str, int]

        Execute the quantum program with a specified configuration.

        This function runs the quantum program using the provided classical bits,

        the number of shots for repeated execution, and an optional noise model.

        Args:
             qprog: The quantum program to execute.

             cbit_list: The list of classical bits.

             shot: The number of times to repeat the execution.

             noise_model: (optional) The noise model to apply (default is no noise). Note: Noise models currently work only on CPUQVM.

        Returns:
             A tuple containing the execution results over the specified shots:
                 The final qubit register state.
                 The count of hits for each outcome.

        """
    @overload
    def run_with_configuration(self, qprog: QProg, cbit_list: list[int], shot: int, noise_model: Noise = ...) -> dict[str, int]:
        """run_with_configuration(*args, **kwargs)
        Overloaded function.

        1. run_with_configuration(self: pyQPanda.QuantumMachine, qprog: pyQPanda.QProg, cbit_list: List[pyQPanda.ClassicalCondition], data: dict, noise_model: pyQPanda.Noise = NoiseModel()) -> Dict[str, int]

        Execute the quantum program with a specified configuration.

        This function runs the quantum program using the provided classical bits,

        parameters, and an optional noise model. It supports multiple shots for

        repeated execution.

        Args:
             qprog: The quantum program to execute.

             cbit_list: The list of classical bits.

             data: Parameters for the execution in dictionary form.

             noise_model: (optional) The noise model to apply (default is no noise).

        Returns:
             The execution results over the specified shots, including:
                 The final qubit register state.
                 The count of hits for each outcome.


        2. run_with_configuration(self: pyQPanda.QuantumMachine, qprog: pyQPanda.QProg, cbit_list: List[pyQPanda.ClassicalCondition], shot: int, noise_model: pyQPanda.Noise = NoiseModel()) -> Dict[str, int]

        Execute the quantum program with a specified configuration.

        This function runs the quantum program using the provided classical bits,

        parameters, and an optional noise model. It supports multiple shots for

        repeated execution.

        Args:
             qprog: The quantum program to execute.

             cbit_list: The list of classical bits.

             shot: The number of times to repeat the execution.

             noise_model: (optional) The noise model to apply (default is no noise).

        Note: Noise models currently only work on CPUQVM.

        Returns:
             A tuple containing the results of the quantum program execution:
                 The final qubit register state.
                 The count of hits for each outcome.


        3. run_with_configuration(self: pyQPanda.QuantumMachine, qprog: pyQPanda.QProg, shot: int, noise_model: pyQPanda.Noise = NoiseModel()) -> Dict[str, int]

        Execute the quantum program with a specified configuration.

        This function runs the quantum program using the provided quantum program,

        number of shots, and an optional noise model. It supports multiple shots

        for repeated execution.

        Args:
             qprog: The quantum program to execute.

             shot: The number of times to repeat the execution.

             noise_model: (optional) The noise model to apply (default is no noise).

        Returns:
             The execution results over the specified shots, including:
                 The final qubit register state.
                 The count of hits for each outcome.


        4. run_with_configuration(self: pyQPanda.QuantumMachine, qprog: pyQPanda.QProg, cbit_list: List[int], shot: int, noise_model: pyQPanda.Noise = NoiseModel()) -> Dict[str, int]

        Execute the quantum program with a specified configuration.

        This function runs the quantum program using the provided classical bits,

        the number of shots for repeated execution, and an optional noise model.

        Args:
             qprog: The quantum program to execute.

             cbit_list: The list of classical bits.

             shot: The number of times to repeat the execution.

             noise_model: (optional) The noise model to apply (default is no noise). Note: Noise models currently work only on CPUQVM.

        Returns:
             A tuple containing the execution results over the specified shots:
                 The final qubit register state.
                 The count of hits for each outcome.

        """
    def set_configure(self, max_qubit: int, max_cbit: int) -> None:
        """set_configure(self: pyQPanda.QuantumMachine, max_qubit: int, max_cbit: int) -> None

        Set the maximum qubit and cbit numbers for the QVM.

        Args:
             max_qubit: Maximum number of qubits in the quantum machine.

             max_cbit: Maximum number of cbits in the quantum machine.

        Returns:
             None

        """

class QuantumStateTomography:
    def __init__(self) -> None:
        """__init__(self: pyQPanda.QuantumStateTomography) -> None"""
    def caculate_tomography_density(self) -> list[list[complex]]:
        """caculate_tomography_density(self: pyQPanda.QuantumStateTomography) -> List[List[complex]]

        Calculate the tomography density.

        Returns:
             A reference to the calculated density matrix.

        """
    @overload
    def combine_qprogs(self, circuit: QProg, qlist: QVec) -> list[QProg]:
        """combine_qprogs(*args, **kwargs)
        Overloaded function.

        1. combine_qprogs(self: pyQPanda.QuantumStateTomography, circuit: pyQPanda.QProg, qlist: pyQPanda.QVec) -> List[pyQPanda.QProg]

        Return a list of quantum state tomography quantum programs.

        Args:
             circuit: The quantum circuit to be combined.

             qlist: The list of qubits involved.

        Returns:
             A reference to the combined quantum programs.


        2. combine_qprogs(self: pyQPanda.QuantumStateTomography, circuit: pyQPanda.QCircuit, qlist: pyQPanda.QVec) -> List[pyQPanda.QProg]

        Return a list of quantum state tomography quantum programs.

        Args:
             circuit: The quantum circuit to be combined.

             qlist: The list of qubits involved.

        Returns:
             A reference to the combined quantum programs.


        3. combine_qprogs(self: pyQPanda.QuantumStateTomography, circuit: pyQPanda.QProg, qlist: List[int]) -> List[pyQPanda.QProg]

        Return a list of quantum state tomography quantum programs.

        Args:
             circuit: The quantum circuit to be combined.

             qlist: A vector of indices representing the qubits involved.

        Returns:
             A reference to the combined quantum programs.


        4. combine_qprogs(self: pyQPanda.QuantumStateTomography, circuit: pyQPanda.QCircuit, qlist: List[int]) -> List[pyQPanda.QProg]

        Return a list of quantum state tomography quantum programs.

        Args:
             circuit: The quantum circuit to be combined.

             qlist: A vector of indices representing the qubits involved.

        Returns:
             A reference to the combined quantum programs.

        """
    @overload
    def combine_qprogs(self, circuit: QCircuit, qlist: QVec) -> list[QProg]:
        """combine_qprogs(*args, **kwargs)
        Overloaded function.

        1. combine_qprogs(self: pyQPanda.QuantumStateTomography, circuit: pyQPanda.QProg, qlist: pyQPanda.QVec) -> List[pyQPanda.QProg]

        Return a list of quantum state tomography quantum programs.

        Args:
             circuit: The quantum circuit to be combined.

             qlist: The list of qubits involved.

        Returns:
             A reference to the combined quantum programs.


        2. combine_qprogs(self: pyQPanda.QuantumStateTomography, circuit: pyQPanda.QCircuit, qlist: pyQPanda.QVec) -> List[pyQPanda.QProg]

        Return a list of quantum state tomography quantum programs.

        Args:
             circuit: The quantum circuit to be combined.

             qlist: The list of qubits involved.

        Returns:
             A reference to the combined quantum programs.


        3. combine_qprogs(self: pyQPanda.QuantumStateTomography, circuit: pyQPanda.QProg, qlist: List[int]) -> List[pyQPanda.QProg]

        Return a list of quantum state tomography quantum programs.

        Args:
             circuit: The quantum circuit to be combined.

             qlist: A vector of indices representing the qubits involved.

        Returns:
             A reference to the combined quantum programs.


        4. combine_qprogs(self: pyQPanda.QuantumStateTomography, circuit: pyQPanda.QCircuit, qlist: List[int]) -> List[pyQPanda.QProg]

        Return a list of quantum state tomography quantum programs.

        Args:
             circuit: The quantum circuit to be combined.

             qlist: A vector of indices representing the qubits involved.

        Returns:
             A reference to the combined quantum programs.

        """
    @overload
    def combine_qprogs(self, circuit: QProg, qlist: list[int]) -> list[QProg]:
        """combine_qprogs(*args, **kwargs)
        Overloaded function.

        1. combine_qprogs(self: pyQPanda.QuantumStateTomography, circuit: pyQPanda.QProg, qlist: pyQPanda.QVec) -> List[pyQPanda.QProg]

        Return a list of quantum state tomography quantum programs.

        Args:
             circuit: The quantum circuit to be combined.

             qlist: The list of qubits involved.

        Returns:
             A reference to the combined quantum programs.


        2. combine_qprogs(self: pyQPanda.QuantumStateTomography, circuit: pyQPanda.QCircuit, qlist: pyQPanda.QVec) -> List[pyQPanda.QProg]

        Return a list of quantum state tomography quantum programs.

        Args:
             circuit: The quantum circuit to be combined.

             qlist: The list of qubits involved.

        Returns:
             A reference to the combined quantum programs.


        3. combine_qprogs(self: pyQPanda.QuantumStateTomography, circuit: pyQPanda.QProg, qlist: List[int]) -> List[pyQPanda.QProg]

        Return a list of quantum state tomography quantum programs.

        Args:
             circuit: The quantum circuit to be combined.

             qlist: A vector of indices representing the qubits involved.

        Returns:
             A reference to the combined quantum programs.


        4. combine_qprogs(self: pyQPanda.QuantumStateTomography, circuit: pyQPanda.QCircuit, qlist: List[int]) -> List[pyQPanda.QProg]

        Return a list of quantum state tomography quantum programs.

        Args:
             circuit: The quantum circuit to be combined.

             qlist: A vector of indices representing the qubits involved.

        Returns:
             A reference to the combined quantum programs.

        """
    @overload
    def combine_qprogs(self, circuit: QCircuit, qlist: list[int]) -> list[QProg]:
        """combine_qprogs(*args, **kwargs)
        Overloaded function.

        1. combine_qprogs(self: pyQPanda.QuantumStateTomography, circuit: pyQPanda.QProg, qlist: pyQPanda.QVec) -> List[pyQPanda.QProg]

        Return a list of quantum state tomography quantum programs.

        Args:
             circuit: The quantum circuit to be combined.

             qlist: The list of qubits involved.

        Returns:
             A reference to the combined quantum programs.


        2. combine_qprogs(self: pyQPanda.QuantumStateTomography, circuit: pyQPanda.QCircuit, qlist: pyQPanda.QVec) -> List[pyQPanda.QProg]

        Return a list of quantum state tomography quantum programs.

        Args:
             circuit: The quantum circuit to be combined.

             qlist: The list of qubits involved.

        Returns:
             A reference to the combined quantum programs.


        3. combine_qprogs(self: pyQPanda.QuantumStateTomography, circuit: pyQPanda.QProg, qlist: List[int]) -> List[pyQPanda.QProg]

        Return a list of quantum state tomography quantum programs.

        Args:
             circuit: The quantum circuit to be combined.

             qlist: A vector of indices representing the qubits involved.

        Returns:
             A reference to the combined quantum programs.


        4. combine_qprogs(self: pyQPanda.QuantumStateTomography, circuit: pyQPanda.QCircuit, qlist: List[int]) -> List[pyQPanda.QProg]

        Return a list of quantum state tomography quantum programs.

        Args:
             circuit: The quantum circuit to be combined.

             qlist: A vector of indices representing the qubits involved.

        Returns:
             A reference to the combined quantum programs.

        """
    def exec(self, qm, shots: int) -> list[list[complex]]:
        """exec(self: pyQPanda.QuantumStateTomography, qm: QPanda::QuantumMachine, shots: int) -> List[List[complex]]

        Run state tomography quantum programs.

        Args:
             qm: The quantum machine to execute the programs on.

             shots: The number of shots for the execution.

        Returns:
             A reference to the execution results.

        """
    def set_qprog_results(self, qlist: int, results: list[dict[str, float]]) -> None:
        """set_qprog_results(self: pyQPanda.QuantumStateTomography, qlist: int, results: List[Dict[str, float]]) -> None

        Set the results of combined quantum programs.

        Args:
             qlist: The index of the qubit list.

             results: A vector of maps containing the result data.

        Returns:
             A reference to the updated state.

        Raises:
             run_fail: An error occurred while setting the results.

        """

class Qubit:
    def __init__(self, *args, **kwargs) -> None:
        """Initialize self.  See help(type(self)) for accurate signature."""
    def getPhysicalQubitPtr(self) -> PhysicalQubit:
        """getPhysicalQubitPtr(self: pyQPanda.Qubit) -> pyQPanda.PhysicalQubit

        Retrieve a pointer to the associated physical qubit.

        This function returns a pointer to the physical qubit that corresponds to this qubit instance.

        Args:
             None

        Returns:
             A pointer to the associated physical qubit.

        """
    def get_phy_addr(self) -> int:
        """get_phy_addr(self: pyQPanda.Qubit) -> int

        Retrieve the physical address of the qubit.

        This function returns the physical address associated with this qubit instance.

        Args:
             None

        Returns:
             The physical address of the qubit.

        """

class RMSPropOptimizer:
    def __init__(self, arg0: var, arg1: float, arg2: float, arg3: float) -> None:
        """__init__(self: pyQPanda.RMSPropOptimizer, arg0: pyQPanda.var, arg1: float, arg2: float, arg3: float) -> None"""
    def get_loss(self) -> float:
        """get_loss(self: pyQPanda.RMSPropOptimizer) -> float"""
    def get_variables(self) -> list[var]:
        """get_variables(self: pyQPanda.RMSPropOptimizer) -> List[pyQPanda.var]"""
    def minimize(self, arg0: float, arg1: float, arg2: float) -> Optimizer:
        """minimize(self: pyQPanda.var, arg0: float, arg1: float, arg2: float) -> pyQPanda.Optimizer"""
    def run(self, arg0: list[var], arg1: int) -> bool:
        """run(self: pyQPanda.RMSPropOptimizer, arg0: List[pyQPanda.var], arg1: int) -> bool"""

class SingleAmpQVM(QuantumMachine):
    def __init__(self) -> None:
        """__init__(self: pyQPanda.SingleAmpQVM) -> None"""
    @overload
    def get_prob_dict(self, arg0: QVec) -> dict[str, float]:
        """get_prob_dict(*args, **kwargs)
        Overloaded function.

        1. get_prob_dict(self: pyQPanda.SingleAmpQVM, arg0: pyQPanda.QVec) -> Dict[str, float]

        Get the pmeasure result as a dictionary.

        Args:
             qubit_list: A list of qubits for pmeasure.

        Returns:
             A dictionary containing the measurement results of the quantum machine.


        2. get_prob_dict(self: pyQPanda.SingleAmpQVM, arg0: List[int]) -> Dict[str, float]

        Get the pmeasure result as a dictionary.

        Args:
             qubit_list: A list of qubits for pmeasure.

        Returns:
             A dictionary containing the measurement results of the quantum machine.

        """
    @overload
    def get_prob_dict(self, arg0: list[int]) -> dict[str, float]:
        """get_prob_dict(*args, **kwargs)
        Overloaded function.

        1. get_prob_dict(self: pyQPanda.SingleAmpQVM, arg0: pyQPanda.QVec) -> Dict[str, float]

        Get the pmeasure result as a dictionary.

        Args:
             qubit_list: A list of qubits for pmeasure.

        Returns:
             A dictionary containing the measurement results of the quantum machine.


        2. get_prob_dict(self: pyQPanda.SingleAmpQVM, arg0: List[int]) -> Dict[str, float]

        Get the pmeasure result as a dictionary.

        Args:
             qubit_list: A list of qubits for pmeasure.

        Returns:
             A dictionary containing the measurement results of the quantum machine.

        """
    def get_quick_map_vertice(self, arg0: list[tuple[int, int]]) -> None:
        """get_quick_map_vertice(self: pyQPanda.SingleAmpQVM, arg0: List[Tuple[int, int]]) -> None

        Get the quick map vertices.

        Returns:
             A reference to the quick map vertices.

        """
    def get_sequence(self, arg0: list[int], arg1: list[list[tuple[int, bool]]]) -> int:
        """get_sequence(self: pyQPanda.SingleAmpQVM, arg0: List[int], arg1: List[List[Tuple[int, bool]]]) -> int

        Get the program sequence.

        Returns:
             A reference to the current program sequence.

        """
    def pmeasure_bin_amplitude(self, arg0: str) -> complex:
        """pmeasure_bin_amplitude(self: pyQPanda.SingleAmpQVM, arg0: str) -> complex

        Measure the bin amplitude of the quantum state.

        Args:
             bin_string: A string representing the bin.

        Returns:
             A complex number representing the bin amplitude.

        """
    def pmeasure_bin_index(self, arg0: str) -> float:
        """pmeasure_bin_index(self: pyQPanda.SingleAmpQVM, arg0: str) -> float

        Measure the bin index of the quantum state amplitude.

        Args:
             bin_string: A string representing the bin.

        Returns:
             A double representing the amplitude probability of the bin.

        """
    def pmeasure_dec_amplitude(self, arg0: str) -> complex:
        """pmeasure_dec_amplitude(self: pyQPanda.SingleAmpQVM, arg0: str) -> complex

        Measure the dec amplitude of the quantum state.

        Args:
             dec_string: A string representing the dec.

        Returns:
             A complex number representing the dec amplitude.

        """
    def pmeasure_dec_index(self, arg0: str) -> float:
        """pmeasure_dec_index(self: pyQPanda.SingleAmpQVM, arg0: str) -> float

        Measure the dec index of the quantum state amplitude.

        Args:
             dec_string: A string representing the dec.

        Returns:
             A double representing the amplitude probability of the dec.

        """
    @overload
    def prob_run_dict(self, arg0: QProg, arg1: QVec) -> dict[str, float]:
        """prob_run_dict(*args, **kwargs)
        Overloaded function.

        1. prob_run_dict(self: pyQPanda.SingleAmpQVM, arg0: pyQPanda.QProg, arg1: pyQPanda.QVec) -> Dict[str, float]

        Run the quantum program and get the pmeasure result as a dictionary.

        Args:
             qprog: The quantum program to run.

             qubit_list: A list of qubits for pmeasure.

        Returns:
             A dictionary containing the measurement results of the quantum machine.


        2. prob_run_dict(self: pyQPanda.SingleAmpQVM, arg0: pyQPanda.QProg, arg1: List[int]) -> Dict[str, float]

        Run the quantum program and get the pmeasure result as a dictionary.

        Args:
             qprog: The quantum program to run.

             qubit_list: A list of qubits for pmeasure.

        Returns:
             A dictionary containing the measurement results of the quantum machine.

        """
    @overload
    def prob_run_dict(self, arg0: QProg, arg1: list[int]) -> dict[str, float]:
        """prob_run_dict(*args, **kwargs)
        Overloaded function.

        1. prob_run_dict(self: pyQPanda.SingleAmpQVM, arg0: pyQPanda.QProg, arg1: pyQPanda.QVec) -> Dict[str, float]

        Run the quantum program and get the pmeasure result as a dictionary.

        Args:
             qprog: The quantum program to run.

             qubit_list: A list of qubits for pmeasure.

        Returns:
             A dictionary containing the measurement results of the quantum machine.


        2. prob_run_dict(self: pyQPanda.SingleAmpQVM, arg0: pyQPanda.QProg, arg1: List[int]) -> Dict[str, float]

        Run the quantum program and get the pmeasure result as a dictionary.

        Args:
             qprog: The quantum program to run.

             qubit_list: A list of qubits for pmeasure.

        Returns:
             A dictionary containing the measurement results of the quantum machine.

        """
    @overload
    def run(self, prog: QProg, qv: QVec, max_rank: int = ..., alloted_time: int = ...) -> None:
        """run(*args, **kwargs)
        Overloaded function.

        1. run(self: pyQPanda.SingleAmpQVM, prog: pyQPanda.QProg, qv: pyQPanda.QVec, max_rank: int = 30, alloted_time: int = 5) -> None

        Run the quantum program.

        Args:
             prog: A quantum program (QProg) to be executed.

             qv: A list of qubits (QVec) involved in the program.

             max_rank: The maximum rank to consider during execution (default is 30).

             alloted_time: The time allocated for execution (default is 5 seconds).

        Returns:
             None, as the function executes the program in place.


        2. run(self: pyQPanda.SingleAmpQVM, arg0: pyQPanda.QProg, arg1: pyQPanda.QVec, arg2: int, arg3: List[List[Tuple[int, bool]]]) -> None

        Run the quantum program.

        Args:
             prog: A quantum program (QProg) to be executed.

             qv: A list of qubits (QVec) involved in the program.

             max_rank: The maximum rank to consider during execution.

             sequences: A list of sequences (std::vector<qprog_sequence_t>).

        Returns:
             None, as the function executes the program in place.

        """
    @overload
    def run(self, arg0: QProg, arg1: QVec, arg2: int, arg3: list[list[tuple[int, bool]]]) -> None:
        """run(*args, **kwargs)
        Overloaded function.

        1. run(self: pyQPanda.SingleAmpQVM, prog: pyQPanda.QProg, qv: pyQPanda.QVec, max_rank: int = 30, alloted_time: int = 5) -> None

        Run the quantum program.

        Args:
             prog: A quantum program (QProg) to be executed.

             qv: A list of qubits (QVec) involved in the program.

             max_rank: The maximum rank to consider during execution (default is 30).

             alloted_time: The time allocated for execution (default is 5 seconds).

        Returns:
             None, as the function executes the program in place.


        2. run(self: pyQPanda.SingleAmpQVM, arg0: pyQPanda.QProg, arg1: pyQPanda.QVec, arg2: int, arg3: List[List[Tuple[int, bool]]]) -> None

        Run the quantum program.

        Args:
             prog: A quantum program (QProg) to be executed.

             qv: A list of qubits (QVec) involved in the program.

             max_rank: The maximum rank to consider during execution.

             sequences: A list of sequences (std::vector<qprog_sequence_t>).

        Returns:
             None, as the function executes the program in place.

        """

class SingleGateTransferType:
    __members__: ClassVar[dict] = ...  # read-only
    ARBITRARY_ROTATION: ClassVar[SingleGateTransferType] = ...
    DOUBLE_CONTINUOUS: ClassVar[SingleGateTransferType] = ...
    DOUBLE_DISCRETE: ClassVar[SingleGateTransferType] = ...
    SINGLE_CONTINUOUS_DISCRETE: ClassVar[SingleGateTransferType] = ...
    SINGLE_GATE_INVALID: ClassVar[SingleGateTransferType] = ...
    __entries: ClassVar[dict] = ...
    def __init__(self, value: int) -> None:
        """__init__(self: pyQPanda.SingleGateTransferType, value: int) -> None"""
    def __eq__(self, other: object) -> bool:
        """__eq__(self: object, other: object) -> bool"""
    def __hash__(self) -> int:
        """__hash__(self: object) -> int"""
    def __index__(self) -> int:
        """__index__(self: pyQPanda.SingleGateTransferType) -> int"""
    def __int__(self) -> int:
        """__int__(self: pyQPanda.SingleGateTransferType) -> int"""
    def __ne__(self, other: object) -> bool:
        """__ne__(self: object, other: object) -> bool"""
    @property
    def name(self) -> str: ...
    @property
    def value(self) -> int: ...

class SparseQVM(QuantumMachine):
    def __init__(self) -> None:
        """__init__(self: pyQPanda.SparseQVM) -> None"""
    def directlyRun(self, arg0: QProg) -> dict[str, bool]:
        """directlyRun(self: pyQPanda.SparseQVM, arg0: pyQPanda.QProg) -> Dict[str, bool]

        Run the quantum program and get the measurement results as a dictionary.

        Args:
             qprog: The quantum program to execute.

        Returns:
             Dict[str, bool]: The result of the quantum program execution in one shot.

        """
    def directly_run(self, arg0: QProg) -> dict[str, bool]:
        """directly_run(self: pyQPanda.SparseQVM, arg0: pyQPanda.QProg) -> Dict[str, bool]

        Run the quantum program and get the measurement results as a dictionary.

        Args:
             qprog: The quantum program to execute.

        Returns:
             The measurement results of the quantum machine.

        """
    def init_qvm(self) -> None:
        """init_qvm(self: pyQPanda.SparseQVM) -> None

        init quantum virtual machine
        """
    def prob_run_dict(self, arg0: QProg) -> dict[str, float]:
        """prob_run_dict(self: pyQPanda.SparseQVM, arg0: pyQPanda.QProg) -> Dict[str, float]

        Run the quantum program and get the measurement results as a dictionary.

        Args:
             qprog: The quantum program to execute.

        Returns:
             A dictionary containing the measurement results of the quantum machine.

        """
    def run_with_configuration(self, arg0: QProg, arg1: list[ClassicalCondition], arg2: int) -> dict[str, int]:
        """run_with_configuration(self: pyQPanda.SparseQVM, arg0: pyQPanda.QProg, arg1: List[pyQPanda.ClassicalCondition], arg2: int) -> Dict[str, int]

        Run the quantum program with the specified configuration and get the measurement results as a dictionary.

        Args:
             qprog: The quantum program to execute.

             cbits: The quantum classical bits.

             shots: The number of sample shots.

        Returns:
             The measurement results of the quantum machine.

        """

class Stabilizer(QuantumMachine):
    def __init__(self) -> None:
        """__init__(self: pyQPanda.Stabilizer) -> None"""
    def init_qvm(self) -> None:
        """init_qvm(self: pyQPanda.Stabilizer) -> None

        init quantum virtual machine
        """
    def prob_run_dict(self, qprog: QProg, qubits: QVec, select_max: int = ...) -> dict[str, float]:
        """prob_run_dict(self: pyQPanda.Stabilizer, qprog: pyQPanda.QProg, qubits: pyQPanda.QVec, select_max: int = -1) -> Dict[str, float]

        Run quantum program and get probabilities.

        Args:
             qprog: Quantum program to execute.

             qubits: Qubits to be measured for probabilities.

             select_max: Optional, selects the maximum number of probabilities to return.

        Returns:
             Probabilities result of the quantum program.

        """
    def run_with_configuration(self, qprog: QProg, shot: int) -> dict[str, int]:
        """run_with_configuration(self: pyQPanda.Stabilizer, qprog: pyQPanda.QProg, shot: int) -> Dict[str, int]

        Run quantum program and get shots result.

        Args:
             qprog: Quantum program to execute.

             shot: Number of measurement shots.

        Returns:
             Shots result of the quantum program.

        """
    @overload
    def set_noise_model(self, arg0: NoiseModel, arg1: GateType, arg2: float) -> None:
        """set_noise_model(*args, **kwargs)
        Overloaded function.

        1. set_noise_model(self: pyQPanda.Stabilizer, arg0: pyQPanda.NoiseModel, arg1: pyQPanda.GateType, arg2: float) -> None

        Set a noise model for the Stabilizer simulator with a specific gate type and probability.

        Args:
             noise_model: The noise model to apply (e.g., bit-flip, phase-flip, etc.).

             gate_type: The specific gate type associated with the noise model.

             probability: The probability of the noise occurring.

        Returns:
             None.


        2. set_noise_model(self: pyQPanda.Stabilizer, arg0: pyQPanda.NoiseModel, arg1: List[pyQPanda.GateType], arg2: float) -> None

        Set a noise model for the Stabilizer simulator with multiple gate types and a specified probability.

        Args:
             noise_model: The noise model to apply (e.g., bit-flip, phase-flip, etc.).

             gate_types: A vector of gate types associated with the noise model.

             probability: The probability of the noise occurring.

        Returns:
             None.


        3. set_noise_model(self: pyQPanda.Stabilizer, arg0: pyQPanda.NoiseModel, arg1: pyQPanda.GateType, arg2: float, arg3: pyQPanda.QVec) -> None

        Set a noise model for the Stabilizer simulator with a specific gate type, probability, and targeted qubits.

        Args:
             noise_model: The noise model to apply (e.g., bit-flip, phase-flip, etc.).

             gate_type: The specific gate type associated with the noise model.

             probability: The probability of the noise occurring.

             target_qubits: The qubits targeted by the noise model.

        Returns:
             None.


        4. set_noise_model(self: pyQPanda.Stabilizer, arg0: pyQPanda.NoiseModel, arg1: List[pyQPanda.GateType], arg2: float, arg3: pyQPanda.QVec) -> None

        Set a noise model for the Stabilizer simulator with multiple gate types, a specified probability, and targeted qubits.

        Args:
             noise_model: The noise model to apply (e.g., bit-flip, phase-flip, etc.).

             gate_types: A vector of gate types associated with the noise model.

             probability: The probability of the noise occurring.

             target_qubits: The qubits targeted by the noise model.

        Returns:
             None.


        5. set_noise_model(self: pyQPanda.Stabilizer, arg0: pyQPanda.NoiseModel, arg1: pyQPanda.GateType, arg2: float, arg3: List[pyQPanda.QVec]) -> None

        Set a noise model for the Stabilizer simulator with a specific gate type, probability, and multiple targeted qubits.

        Args:
             noise_model: The noise model to apply (e.g., bit-flip, phase-flip, etc.).

             gate_type: The specific gate type associated with the noise model.

             probability: The probability of the noise occurring.

             target_qubits: A vector of qubit vectors targeted by the noise model.

        Returns:
             None.

        """
    @overload
    def set_noise_model(self, arg0: NoiseModel, arg1: list[GateType], arg2: float) -> None:
        """set_noise_model(*args, **kwargs)
        Overloaded function.

        1. set_noise_model(self: pyQPanda.Stabilizer, arg0: pyQPanda.NoiseModel, arg1: pyQPanda.GateType, arg2: float) -> None

        Set a noise model for the Stabilizer simulator with a specific gate type and probability.

        Args:
             noise_model: The noise model to apply (e.g., bit-flip, phase-flip, etc.).

             gate_type: The specific gate type associated with the noise model.

             probability: The probability of the noise occurring.

        Returns:
             None.


        2. set_noise_model(self: pyQPanda.Stabilizer, arg0: pyQPanda.NoiseModel, arg1: List[pyQPanda.GateType], arg2: float) -> None

        Set a noise model for the Stabilizer simulator with multiple gate types and a specified probability.

        Args:
             noise_model: The noise model to apply (e.g., bit-flip, phase-flip, etc.).

             gate_types: A vector of gate types associated with the noise model.

             probability: The probability of the noise occurring.

        Returns:
             None.


        3. set_noise_model(self: pyQPanda.Stabilizer, arg0: pyQPanda.NoiseModel, arg1: pyQPanda.GateType, arg2: float, arg3: pyQPanda.QVec) -> None

        Set a noise model for the Stabilizer simulator with a specific gate type, probability, and targeted qubits.

        Args:
             noise_model: The noise model to apply (e.g., bit-flip, phase-flip, etc.).

             gate_type: The specific gate type associated with the noise model.

             probability: The probability of the noise occurring.

             target_qubits: The qubits targeted by the noise model.

        Returns:
             None.


        4. set_noise_model(self: pyQPanda.Stabilizer, arg0: pyQPanda.NoiseModel, arg1: List[pyQPanda.GateType], arg2: float, arg3: pyQPanda.QVec) -> None

        Set a noise model for the Stabilizer simulator with multiple gate types, a specified probability, and targeted qubits.

        Args:
             noise_model: The noise model to apply (e.g., bit-flip, phase-flip, etc.).

             gate_types: A vector of gate types associated with the noise model.

             probability: The probability of the noise occurring.

             target_qubits: The qubits targeted by the noise model.

        Returns:
             None.


        5. set_noise_model(self: pyQPanda.Stabilizer, arg0: pyQPanda.NoiseModel, arg1: pyQPanda.GateType, arg2: float, arg3: List[pyQPanda.QVec]) -> None

        Set a noise model for the Stabilizer simulator with a specific gate type, probability, and multiple targeted qubits.

        Args:
             noise_model: The noise model to apply (e.g., bit-flip, phase-flip, etc.).

             gate_type: The specific gate type associated with the noise model.

             probability: The probability of the noise occurring.

             target_qubits: A vector of qubit vectors targeted by the noise model.

        Returns:
             None.

        """
    @overload
    def set_noise_model(self, arg0: NoiseModel, arg1: GateType, arg2: float, arg3: QVec) -> None:
        """set_noise_model(*args, **kwargs)
        Overloaded function.

        1. set_noise_model(self: pyQPanda.Stabilizer, arg0: pyQPanda.NoiseModel, arg1: pyQPanda.GateType, arg2: float) -> None

        Set a noise model for the Stabilizer simulator with a specific gate type and probability.

        Args:
             noise_model: The noise model to apply (e.g., bit-flip, phase-flip, etc.).

             gate_type: The specific gate type associated with the noise model.

             probability: The probability of the noise occurring.

        Returns:
             None.


        2. set_noise_model(self: pyQPanda.Stabilizer, arg0: pyQPanda.NoiseModel, arg1: List[pyQPanda.GateType], arg2: float) -> None

        Set a noise model for the Stabilizer simulator with multiple gate types and a specified probability.

        Args:
             noise_model: The noise model to apply (e.g., bit-flip, phase-flip, etc.).

             gate_types: A vector of gate types associated with the noise model.

             probability: The probability of the noise occurring.

        Returns:
             None.


        3. set_noise_model(self: pyQPanda.Stabilizer, arg0: pyQPanda.NoiseModel, arg1: pyQPanda.GateType, arg2: float, arg3: pyQPanda.QVec) -> None

        Set a noise model for the Stabilizer simulator with a specific gate type, probability, and targeted qubits.

        Args:
             noise_model: The noise model to apply (e.g., bit-flip, phase-flip, etc.).

             gate_type: The specific gate type associated with the noise model.

             probability: The probability of the noise occurring.

             target_qubits: The qubits targeted by the noise model.

        Returns:
             None.


        4. set_noise_model(self: pyQPanda.Stabilizer, arg0: pyQPanda.NoiseModel, arg1: List[pyQPanda.GateType], arg2: float, arg3: pyQPanda.QVec) -> None

        Set a noise model for the Stabilizer simulator with multiple gate types, a specified probability, and targeted qubits.

        Args:
             noise_model: The noise model to apply (e.g., bit-flip, phase-flip, etc.).

             gate_types: A vector of gate types associated with the noise model.

             probability: The probability of the noise occurring.

             target_qubits: The qubits targeted by the noise model.

        Returns:
             None.


        5. set_noise_model(self: pyQPanda.Stabilizer, arg0: pyQPanda.NoiseModel, arg1: pyQPanda.GateType, arg2: float, arg3: List[pyQPanda.QVec]) -> None

        Set a noise model for the Stabilizer simulator with a specific gate type, probability, and multiple targeted qubits.

        Args:
             noise_model: The noise model to apply (e.g., bit-flip, phase-flip, etc.).

             gate_type: The specific gate type associated with the noise model.

             probability: The probability of the noise occurring.

             target_qubits: A vector of qubit vectors targeted by the noise model.

        Returns:
             None.

        """
    @overload
    def set_noise_model(self, arg0: NoiseModel, arg1: list[GateType], arg2: float, arg3: QVec) -> None:
        """set_noise_model(*args, **kwargs)
        Overloaded function.

        1. set_noise_model(self: pyQPanda.Stabilizer, arg0: pyQPanda.NoiseModel, arg1: pyQPanda.GateType, arg2: float) -> None

        Set a noise model for the Stabilizer simulator with a specific gate type and probability.

        Args:
             noise_model: The noise model to apply (e.g., bit-flip, phase-flip, etc.).

             gate_type: The specific gate type associated with the noise model.

             probability: The probability of the noise occurring.

        Returns:
             None.


        2. set_noise_model(self: pyQPanda.Stabilizer, arg0: pyQPanda.NoiseModel, arg1: List[pyQPanda.GateType], arg2: float) -> None

        Set a noise model for the Stabilizer simulator with multiple gate types and a specified probability.

        Args:
             noise_model: The noise model to apply (e.g., bit-flip, phase-flip, etc.).

             gate_types: A vector of gate types associated with the noise model.

             probability: The probability of the noise occurring.

        Returns:
             None.


        3. set_noise_model(self: pyQPanda.Stabilizer, arg0: pyQPanda.NoiseModel, arg1: pyQPanda.GateType, arg2: float, arg3: pyQPanda.QVec) -> None

        Set a noise model for the Stabilizer simulator with a specific gate type, probability, and targeted qubits.

        Args:
             noise_model: The noise model to apply (e.g., bit-flip, phase-flip, etc.).

             gate_type: The specific gate type associated with the noise model.

             probability: The probability of the noise occurring.

             target_qubits: The qubits targeted by the noise model.

        Returns:
             None.


        4. set_noise_model(self: pyQPanda.Stabilizer, arg0: pyQPanda.NoiseModel, arg1: List[pyQPanda.GateType], arg2: float, arg3: pyQPanda.QVec) -> None

        Set a noise model for the Stabilizer simulator with multiple gate types, a specified probability, and targeted qubits.

        Args:
             noise_model: The noise model to apply (e.g., bit-flip, phase-flip, etc.).

             gate_types: A vector of gate types associated with the noise model.

             probability: The probability of the noise occurring.

             target_qubits: The qubits targeted by the noise model.

        Returns:
             None.


        5. set_noise_model(self: pyQPanda.Stabilizer, arg0: pyQPanda.NoiseModel, arg1: pyQPanda.GateType, arg2: float, arg3: List[pyQPanda.QVec]) -> None

        Set a noise model for the Stabilizer simulator with a specific gate type, probability, and multiple targeted qubits.

        Args:
             noise_model: The noise model to apply (e.g., bit-flip, phase-flip, etc.).

             gate_type: The specific gate type associated with the noise model.

             probability: The probability of the noise occurring.

             target_qubits: A vector of qubit vectors targeted by the noise model.

        Returns:
             None.

        """
    @overload
    def set_noise_model(self, arg0: NoiseModel, arg1: GateType, arg2: float, arg3: list[QVec]) -> None:
        """set_noise_model(*args, **kwargs)
        Overloaded function.

        1. set_noise_model(self: pyQPanda.Stabilizer, arg0: pyQPanda.NoiseModel, arg1: pyQPanda.GateType, arg2: float) -> None

        Set a noise model for the Stabilizer simulator with a specific gate type and probability.

        Args:
             noise_model: The noise model to apply (e.g., bit-flip, phase-flip, etc.).

             gate_type: The specific gate type associated with the noise model.

             probability: The probability of the noise occurring.

        Returns:
             None.


        2. set_noise_model(self: pyQPanda.Stabilizer, arg0: pyQPanda.NoiseModel, arg1: List[pyQPanda.GateType], arg2: float) -> None

        Set a noise model for the Stabilizer simulator with multiple gate types and a specified probability.

        Args:
             noise_model: The noise model to apply (e.g., bit-flip, phase-flip, etc.).

             gate_types: A vector of gate types associated with the noise model.

             probability: The probability of the noise occurring.

        Returns:
             None.


        3. set_noise_model(self: pyQPanda.Stabilizer, arg0: pyQPanda.NoiseModel, arg1: pyQPanda.GateType, arg2: float, arg3: pyQPanda.QVec) -> None

        Set a noise model for the Stabilizer simulator with a specific gate type, probability, and targeted qubits.

        Args:
             noise_model: The noise model to apply (e.g., bit-flip, phase-flip, etc.).

             gate_type: The specific gate type associated with the noise model.

             probability: The probability of the noise occurring.

             target_qubits: The qubits targeted by the noise model.

        Returns:
             None.


        4. set_noise_model(self: pyQPanda.Stabilizer, arg0: pyQPanda.NoiseModel, arg1: List[pyQPanda.GateType], arg2: float, arg3: pyQPanda.QVec) -> None

        Set a noise model for the Stabilizer simulator with multiple gate types, a specified probability, and targeted qubits.

        Args:
             noise_model: The noise model to apply (e.g., bit-flip, phase-flip, etc.).

             gate_types: A vector of gate types associated with the noise model.

             probability: The probability of the noise occurring.

             target_qubits: The qubits targeted by the noise model.

        Returns:
             None.


        5. set_noise_model(self: pyQPanda.Stabilizer, arg0: pyQPanda.NoiseModel, arg1: pyQPanda.GateType, arg2: float, arg3: List[pyQPanda.QVec]) -> None

        Set a noise model for the Stabilizer simulator with a specific gate type, probability, and multiple targeted qubits.

        Args:
             noise_model: The noise model to apply (e.g., bit-flip, phase-flip, etc.).

             gate_type: The specific gate type associated with the noise model.

             probability: The probability of the noise occurring.

             target_qubits: A vector of qubit vectors targeted by the noise model.

        Returns:
             None.

        """

class UpdateMode:
    __members__: ClassVar[dict] = ...  # read-only
    GD_DIRECTION: ClassVar[UpdateMode] = ...
    GD_VALUE: ClassVar[UpdateMode] = ...
    __entries: ClassVar[dict] = ...
    def __init__(self, value: int) -> None:
        """__init__(self: pyQPanda.UpdateMode, value: int) -> None"""
    def __eq__(self, other: object) -> bool:
        """__eq__(self: object, other: object) -> bool"""
    def __ge__(self, other: object) -> bool:
        """__ge__(self: object, other: object) -> bool"""
    def __gt__(self, other: object) -> bool:
        """__gt__(self: object, other: object) -> bool"""
    def __hash__(self) -> int:
        """__hash__(self: object) -> int"""
    def __index__(self) -> int:
        """__index__(self: pyQPanda.UpdateMode) -> int"""
    def __int__(self) -> int:
        """__int__(self: pyQPanda.UpdateMode) -> int"""
    def __le__(self, other: object) -> bool:
        """__le__(self: object, other: object) -> bool"""
    def __lt__(self, other: object) -> bool:
        """__lt__(self: object, other: object) -> bool"""
    def __ne__(self, other: object) -> bool:
        """__ne__(self: object, other: object) -> bool"""
    @property
    def name(self) -> str: ...
    @property
    def value(self) -> int: ...

class VanillaGradientDescentOptimizer:
    def __init__(self, arg0: var, arg1: float, arg2: float, arg3: OptimizerMode) -> None:
        """__init__(self: pyQPanda.VanillaGradientDescentOptimizer, arg0: pyQPanda.var, arg1: float, arg2: float, arg3: pyQPanda.OptimizerMode) -> None"""
    def get_loss(self) -> float:
        """get_loss(self: pyQPanda.VanillaGradientDescentOptimizer) -> float"""
    def get_variables(self) -> list[var]:
        """get_variables(self: pyQPanda.VanillaGradientDescentOptimizer) -> List[pyQPanda.var]"""
    def minimize(self, arg0: float, arg1: float) -> Optimizer:
        """minimize(self: pyQPanda.var, arg0: float, arg1: float) -> pyQPanda.Optimizer"""
    def run(self, arg0: list[var], arg1: int) -> bool:
        """run(self: pyQPanda.VanillaGradientDescentOptimizer, arg0: List[pyQPanda.var], arg1: int) -> bool"""

class VariationalQuantumCircuit:
    @overload
    def __init__(self) -> None:
        """__init__(*args, **kwargs)
        Overloaded function.

        1. __init__(self: pyQPanda.VariationalQuantumCircuit) -> None

        2. __init__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.QCircuit) -> None
        """
    @overload
    def __init__(self, arg0: QCircuit) -> None:
        """__init__(*args, **kwargs)
        Overloaded function.

        1. __init__(self: pyQPanda.VariationalQuantumCircuit) -> None

        2. __init__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.QCircuit) -> None
        """
    def control(self, arg0: QVec) -> VariationalQuantumCircuit:
        """control(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.QVec) -> pyQPanda.VariationalQuantumCircuit"""
    def dagger(self) -> VariationalQuantumCircuit:
        """dagger(self: pyQPanda.VariationalQuantumCircuit) -> pyQPanda.VariationalQuantumCircuit"""
    @overload
    def feed(self) -> QCircuit:
        """feed(*args, **kwargs)
        Overloaded function.

        1. feed(self: pyQPanda.VariationalQuantumCircuit) -> pyQPanda.QCircuit

        2. feed(self: pyQPanda.VariationalQuantumCircuit, arg0: List[Tuple[std::weak_ptr<QPanda::Variational::VariationalQuantumGate>, int, float]]) -> pyQPanda.QCircuit
        """
    @overload
    def feed(self, arg0) -> QCircuit:
        """feed(*args, **kwargs)
        Overloaded function.

        1. feed(self: pyQPanda.VariationalQuantumCircuit) -> pyQPanda.QCircuit

        2. feed(self: pyQPanda.VariationalQuantumCircuit, arg0: List[Tuple[std::weak_ptr<QPanda::Variational::VariationalQuantumGate>, int, float]]) -> pyQPanda.QCircuit
        """
    def get_control_qubit(self) -> QVec:
        """get_control_qubit(self: pyQPanda.VariationalQuantumCircuit) -> pyQPanda.QVec"""
    @overload
    def insert(self, arg0: VariationalQuantumGate_I) -> VariationalQuantumCircuit:
        """insert(*args, **kwargs)
        Overloaded function.

        1. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_I) -> pyQPanda.VariationalQuantumCircuit

        2. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_H) -> pyQPanda.VariationalQuantumCircuit

        3. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_X) -> pyQPanda.VariationalQuantumCircuit

        4. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Y) -> pyQPanda.VariationalQuantumCircuit

        5. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_T) -> pyQPanda.VariationalQuantumCircuit

        6. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_S) -> pyQPanda.VariationalQuantumCircuit

        7. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Z) -> pyQPanda.VariationalQuantumCircuit

        8. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_X1) -> pyQPanda.VariationalQuantumCircuit

        9. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Y1) -> pyQPanda.VariationalQuantumCircuit

        10. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Z1) -> pyQPanda.VariationalQuantumCircuit

        11. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U1) -> pyQPanda.VariationalQuantumCircuit

        12. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U2) -> pyQPanda.VariationalQuantumCircuit

        13. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U3) -> pyQPanda.VariationalQuantumCircuit

        14. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U4) -> pyQPanda.VariationalQuantumCircuit

        15. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_RX) -> pyQPanda.VariationalQuantumCircuit

        16. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_RY) -> pyQPanda.VariationalQuantumCircuit

        17. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_RZ) -> pyQPanda.VariationalQuantumCircuit

        18. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CNOT) -> pyQPanda.VariationalQuantumCircuit

        19. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CR) -> pyQPanda.VariationalQuantumCircuit

        20. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CZ) -> pyQPanda.VariationalQuantumCircuit

        21. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CRX) -> pyQPanda.VariationalQuantumCircuit

        22. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CRY) -> pyQPanda.VariationalQuantumCircuit

        23. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CRZ) -> pyQPanda.VariationalQuantumCircuit

        24. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_SWAP) -> pyQPanda.VariationalQuantumCircuit

        25. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_iSWAP) -> pyQPanda.VariationalQuantumCircuit

        26. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_SqiSWAP) -> pyQPanda.VariationalQuantumCircuit

        27. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumCircuit) -> pyQPanda.VariationalQuantumCircuit

        28. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.QCircuit) -> pyQPanda.VariationalQuantumCircuit

        29. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.QGate) -> pyQPanda.VariationalQuantumCircuit
        """
    @overload
    def insert(self, arg0: VariationalQuantumGate_H) -> VariationalQuantumCircuit:
        """insert(*args, **kwargs)
        Overloaded function.

        1. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_I) -> pyQPanda.VariationalQuantumCircuit

        2. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_H) -> pyQPanda.VariationalQuantumCircuit

        3. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_X) -> pyQPanda.VariationalQuantumCircuit

        4. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Y) -> pyQPanda.VariationalQuantumCircuit

        5. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_T) -> pyQPanda.VariationalQuantumCircuit

        6. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_S) -> pyQPanda.VariationalQuantumCircuit

        7. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Z) -> pyQPanda.VariationalQuantumCircuit

        8. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_X1) -> pyQPanda.VariationalQuantumCircuit

        9. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Y1) -> pyQPanda.VariationalQuantumCircuit

        10. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Z1) -> pyQPanda.VariationalQuantumCircuit

        11. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U1) -> pyQPanda.VariationalQuantumCircuit

        12. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U2) -> pyQPanda.VariationalQuantumCircuit

        13. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U3) -> pyQPanda.VariationalQuantumCircuit

        14. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U4) -> pyQPanda.VariationalQuantumCircuit

        15. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_RX) -> pyQPanda.VariationalQuantumCircuit

        16. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_RY) -> pyQPanda.VariationalQuantumCircuit

        17. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_RZ) -> pyQPanda.VariationalQuantumCircuit

        18. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CNOT) -> pyQPanda.VariationalQuantumCircuit

        19. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CR) -> pyQPanda.VariationalQuantumCircuit

        20. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CZ) -> pyQPanda.VariationalQuantumCircuit

        21. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CRX) -> pyQPanda.VariationalQuantumCircuit

        22. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CRY) -> pyQPanda.VariationalQuantumCircuit

        23. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CRZ) -> pyQPanda.VariationalQuantumCircuit

        24. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_SWAP) -> pyQPanda.VariationalQuantumCircuit

        25. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_iSWAP) -> pyQPanda.VariationalQuantumCircuit

        26. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_SqiSWAP) -> pyQPanda.VariationalQuantumCircuit

        27. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumCircuit) -> pyQPanda.VariationalQuantumCircuit

        28. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.QCircuit) -> pyQPanda.VariationalQuantumCircuit

        29. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.QGate) -> pyQPanda.VariationalQuantumCircuit
        """
    @overload
    def insert(self, arg0: VariationalQuantumGate_X) -> VariationalQuantumCircuit:
        """insert(*args, **kwargs)
        Overloaded function.

        1. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_I) -> pyQPanda.VariationalQuantumCircuit

        2. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_H) -> pyQPanda.VariationalQuantumCircuit

        3. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_X) -> pyQPanda.VariationalQuantumCircuit

        4. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Y) -> pyQPanda.VariationalQuantumCircuit

        5. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_T) -> pyQPanda.VariationalQuantumCircuit

        6. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_S) -> pyQPanda.VariationalQuantumCircuit

        7. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Z) -> pyQPanda.VariationalQuantumCircuit

        8. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_X1) -> pyQPanda.VariationalQuantumCircuit

        9. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Y1) -> pyQPanda.VariationalQuantumCircuit

        10. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Z1) -> pyQPanda.VariationalQuantumCircuit

        11. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U1) -> pyQPanda.VariationalQuantumCircuit

        12. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U2) -> pyQPanda.VariationalQuantumCircuit

        13. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U3) -> pyQPanda.VariationalQuantumCircuit

        14. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U4) -> pyQPanda.VariationalQuantumCircuit

        15. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_RX) -> pyQPanda.VariationalQuantumCircuit

        16. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_RY) -> pyQPanda.VariationalQuantumCircuit

        17. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_RZ) -> pyQPanda.VariationalQuantumCircuit

        18. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CNOT) -> pyQPanda.VariationalQuantumCircuit

        19. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CR) -> pyQPanda.VariationalQuantumCircuit

        20. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CZ) -> pyQPanda.VariationalQuantumCircuit

        21. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CRX) -> pyQPanda.VariationalQuantumCircuit

        22. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CRY) -> pyQPanda.VariationalQuantumCircuit

        23. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CRZ) -> pyQPanda.VariationalQuantumCircuit

        24. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_SWAP) -> pyQPanda.VariationalQuantumCircuit

        25. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_iSWAP) -> pyQPanda.VariationalQuantumCircuit

        26. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_SqiSWAP) -> pyQPanda.VariationalQuantumCircuit

        27. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumCircuit) -> pyQPanda.VariationalQuantumCircuit

        28. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.QCircuit) -> pyQPanda.VariationalQuantumCircuit

        29. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.QGate) -> pyQPanda.VariationalQuantumCircuit
        """
    @overload
    def insert(self, arg0: VariationalQuantumGate_Y) -> VariationalQuantumCircuit:
        """insert(*args, **kwargs)
        Overloaded function.

        1. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_I) -> pyQPanda.VariationalQuantumCircuit

        2. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_H) -> pyQPanda.VariationalQuantumCircuit

        3. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_X) -> pyQPanda.VariationalQuantumCircuit

        4. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Y) -> pyQPanda.VariationalQuantumCircuit

        5. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_T) -> pyQPanda.VariationalQuantumCircuit

        6. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_S) -> pyQPanda.VariationalQuantumCircuit

        7. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Z) -> pyQPanda.VariationalQuantumCircuit

        8. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_X1) -> pyQPanda.VariationalQuantumCircuit

        9. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Y1) -> pyQPanda.VariationalQuantumCircuit

        10. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Z1) -> pyQPanda.VariationalQuantumCircuit

        11. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U1) -> pyQPanda.VariationalQuantumCircuit

        12. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U2) -> pyQPanda.VariationalQuantumCircuit

        13. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U3) -> pyQPanda.VariationalQuantumCircuit

        14. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U4) -> pyQPanda.VariationalQuantumCircuit

        15. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_RX) -> pyQPanda.VariationalQuantumCircuit

        16. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_RY) -> pyQPanda.VariationalQuantumCircuit

        17. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_RZ) -> pyQPanda.VariationalQuantumCircuit

        18. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CNOT) -> pyQPanda.VariationalQuantumCircuit

        19. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CR) -> pyQPanda.VariationalQuantumCircuit

        20. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CZ) -> pyQPanda.VariationalQuantumCircuit

        21. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CRX) -> pyQPanda.VariationalQuantumCircuit

        22. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CRY) -> pyQPanda.VariationalQuantumCircuit

        23. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CRZ) -> pyQPanda.VariationalQuantumCircuit

        24. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_SWAP) -> pyQPanda.VariationalQuantumCircuit

        25. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_iSWAP) -> pyQPanda.VariationalQuantumCircuit

        26. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_SqiSWAP) -> pyQPanda.VariationalQuantumCircuit

        27. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumCircuit) -> pyQPanda.VariationalQuantumCircuit

        28. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.QCircuit) -> pyQPanda.VariationalQuantumCircuit

        29. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.QGate) -> pyQPanda.VariationalQuantumCircuit
        """
    @overload
    def insert(self, arg0: VariationalQuantumGate_T) -> VariationalQuantumCircuit:
        """insert(*args, **kwargs)
        Overloaded function.

        1. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_I) -> pyQPanda.VariationalQuantumCircuit

        2. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_H) -> pyQPanda.VariationalQuantumCircuit

        3. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_X) -> pyQPanda.VariationalQuantumCircuit

        4. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Y) -> pyQPanda.VariationalQuantumCircuit

        5. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_T) -> pyQPanda.VariationalQuantumCircuit

        6. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_S) -> pyQPanda.VariationalQuantumCircuit

        7. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Z) -> pyQPanda.VariationalQuantumCircuit

        8. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_X1) -> pyQPanda.VariationalQuantumCircuit

        9. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Y1) -> pyQPanda.VariationalQuantumCircuit

        10. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Z1) -> pyQPanda.VariationalQuantumCircuit

        11. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U1) -> pyQPanda.VariationalQuantumCircuit

        12. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U2) -> pyQPanda.VariationalQuantumCircuit

        13. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U3) -> pyQPanda.VariationalQuantumCircuit

        14. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U4) -> pyQPanda.VariationalQuantumCircuit

        15. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_RX) -> pyQPanda.VariationalQuantumCircuit

        16. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_RY) -> pyQPanda.VariationalQuantumCircuit

        17. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_RZ) -> pyQPanda.VariationalQuantumCircuit

        18. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CNOT) -> pyQPanda.VariationalQuantumCircuit

        19. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CR) -> pyQPanda.VariationalQuantumCircuit

        20. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CZ) -> pyQPanda.VariationalQuantumCircuit

        21. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CRX) -> pyQPanda.VariationalQuantumCircuit

        22. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CRY) -> pyQPanda.VariationalQuantumCircuit

        23. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CRZ) -> pyQPanda.VariationalQuantumCircuit

        24. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_SWAP) -> pyQPanda.VariationalQuantumCircuit

        25. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_iSWAP) -> pyQPanda.VariationalQuantumCircuit

        26. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_SqiSWAP) -> pyQPanda.VariationalQuantumCircuit

        27. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumCircuit) -> pyQPanda.VariationalQuantumCircuit

        28. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.QCircuit) -> pyQPanda.VariationalQuantumCircuit

        29. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.QGate) -> pyQPanda.VariationalQuantumCircuit
        """
    @overload
    def insert(self, arg0: VariationalQuantumGate_S) -> VariationalQuantumCircuit:
        """insert(*args, **kwargs)
        Overloaded function.

        1. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_I) -> pyQPanda.VariationalQuantumCircuit

        2. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_H) -> pyQPanda.VariationalQuantumCircuit

        3. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_X) -> pyQPanda.VariationalQuantumCircuit

        4. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Y) -> pyQPanda.VariationalQuantumCircuit

        5. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_T) -> pyQPanda.VariationalQuantumCircuit

        6. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_S) -> pyQPanda.VariationalQuantumCircuit

        7. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Z) -> pyQPanda.VariationalQuantumCircuit

        8. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_X1) -> pyQPanda.VariationalQuantumCircuit

        9. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Y1) -> pyQPanda.VariationalQuantumCircuit

        10. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Z1) -> pyQPanda.VariationalQuantumCircuit

        11. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U1) -> pyQPanda.VariationalQuantumCircuit

        12. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U2) -> pyQPanda.VariationalQuantumCircuit

        13. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U3) -> pyQPanda.VariationalQuantumCircuit

        14. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U4) -> pyQPanda.VariationalQuantumCircuit

        15. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_RX) -> pyQPanda.VariationalQuantumCircuit

        16. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_RY) -> pyQPanda.VariationalQuantumCircuit

        17. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_RZ) -> pyQPanda.VariationalQuantumCircuit

        18. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CNOT) -> pyQPanda.VariationalQuantumCircuit

        19. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CR) -> pyQPanda.VariationalQuantumCircuit

        20. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CZ) -> pyQPanda.VariationalQuantumCircuit

        21. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CRX) -> pyQPanda.VariationalQuantumCircuit

        22. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CRY) -> pyQPanda.VariationalQuantumCircuit

        23. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CRZ) -> pyQPanda.VariationalQuantumCircuit

        24. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_SWAP) -> pyQPanda.VariationalQuantumCircuit

        25. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_iSWAP) -> pyQPanda.VariationalQuantumCircuit

        26. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_SqiSWAP) -> pyQPanda.VariationalQuantumCircuit

        27. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumCircuit) -> pyQPanda.VariationalQuantumCircuit

        28. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.QCircuit) -> pyQPanda.VariationalQuantumCircuit

        29. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.QGate) -> pyQPanda.VariationalQuantumCircuit
        """
    @overload
    def insert(self, arg0: VariationalQuantumGate_Z) -> VariationalQuantumCircuit:
        """insert(*args, **kwargs)
        Overloaded function.

        1. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_I) -> pyQPanda.VariationalQuantumCircuit

        2. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_H) -> pyQPanda.VariationalQuantumCircuit

        3. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_X) -> pyQPanda.VariationalQuantumCircuit

        4. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Y) -> pyQPanda.VariationalQuantumCircuit

        5. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_T) -> pyQPanda.VariationalQuantumCircuit

        6. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_S) -> pyQPanda.VariationalQuantumCircuit

        7. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Z) -> pyQPanda.VariationalQuantumCircuit

        8. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_X1) -> pyQPanda.VariationalQuantumCircuit

        9. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Y1) -> pyQPanda.VariationalQuantumCircuit

        10. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Z1) -> pyQPanda.VariationalQuantumCircuit

        11. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U1) -> pyQPanda.VariationalQuantumCircuit

        12. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U2) -> pyQPanda.VariationalQuantumCircuit

        13. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U3) -> pyQPanda.VariationalQuantumCircuit

        14. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U4) -> pyQPanda.VariationalQuantumCircuit

        15. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_RX) -> pyQPanda.VariationalQuantumCircuit

        16. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_RY) -> pyQPanda.VariationalQuantumCircuit

        17. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_RZ) -> pyQPanda.VariationalQuantumCircuit

        18. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CNOT) -> pyQPanda.VariationalQuantumCircuit

        19. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CR) -> pyQPanda.VariationalQuantumCircuit

        20. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CZ) -> pyQPanda.VariationalQuantumCircuit

        21. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CRX) -> pyQPanda.VariationalQuantumCircuit

        22. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CRY) -> pyQPanda.VariationalQuantumCircuit

        23. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CRZ) -> pyQPanda.VariationalQuantumCircuit

        24. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_SWAP) -> pyQPanda.VariationalQuantumCircuit

        25. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_iSWAP) -> pyQPanda.VariationalQuantumCircuit

        26. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_SqiSWAP) -> pyQPanda.VariationalQuantumCircuit

        27. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumCircuit) -> pyQPanda.VariationalQuantumCircuit

        28. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.QCircuit) -> pyQPanda.VariationalQuantumCircuit

        29. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.QGate) -> pyQPanda.VariationalQuantumCircuit
        """
    @overload
    def insert(self, arg0: VariationalQuantumGate_X1) -> VariationalQuantumCircuit:
        """insert(*args, **kwargs)
        Overloaded function.

        1. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_I) -> pyQPanda.VariationalQuantumCircuit

        2. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_H) -> pyQPanda.VariationalQuantumCircuit

        3. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_X) -> pyQPanda.VariationalQuantumCircuit

        4. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Y) -> pyQPanda.VariationalQuantumCircuit

        5. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_T) -> pyQPanda.VariationalQuantumCircuit

        6. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_S) -> pyQPanda.VariationalQuantumCircuit

        7. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Z) -> pyQPanda.VariationalQuantumCircuit

        8. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_X1) -> pyQPanda.VariationalQuantumCircuit

        9. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Y1) -> pyQPanda.VariationalQuantumCircuit

        10. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Z1) -> pyQPanda.VariationalQuantumCircuit

        11. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U1) -> pyQPanda.VariationalQuantumCircuit

        12. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U2) -> pyQPanda.VariationalQuantumCircuit

        13. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U3) -> pyQPanda.VariationalQuantumCircuit

        14. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U4) -> pyQPanda.VariationalQuantumCircuit

        15. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_RX) -> pyQPanda.VariationalQuantumCircuit

        16. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_RY) -> pyQPanda.VariationalQuantumCircuit

        17. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_RZ) -> pyQPanda.VariationalQuantumCircuit

        18. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CNOT) -> pyQPanda.VariationalQuantumCircuit

        19. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CR) -> pyQPanda.VariationalQuantumCircuit

        20. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CZ) -> pyQPanda.VariationalQuantumCircuit

        21. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CRX) -> pyQPanda.VariationalQuantumCircuit

        22. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CRY) -> pyQPanda.VariationalQuantumCircuit

        23. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CRZ) -> pyQPanda.VariationalQuantumCircuit

        24. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_SWAP) -> pyQPanda.VariationalQuantumCircuit

        25. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_iSWAP) -> pyQPanda.VariationalQuantumCircuit

        26. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_SqiSWAP) -> pyQPanda.VariationalQuantumCircuit

        27. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumCircuit) -> pyQPanda.VariationalQuantumCircuit

        28. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.QCircuit) -> pyQPanda.VariationalQuantumCircuit

        29. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.QGate) -> pyQPanda.VariationalQuantumCircuit
        """
    @overload
    def insert(self, arg0: VariationalQuantumGate_Y1) -> VariationalQuantumCircuit:
        """insert(*args, **kwargs)
        Overloaded function.

        1. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_I) -> pyQPanda.VariationalQuantumCircuit

        2. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_H) -> pyQPanda.VariationalQuantumCircuit

        3. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_X) -> pyQPanda.VariationalQuantumCircuit

        4. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Y) -> pyQPanda.VariationalQuantumCircuit

        5. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_T) -> pyQPanda.VariationalQuantumCircuit

        6. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_S) -> pyQPanda.VariationalQuantumCircuit

        7. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Z) -> pyQPanda.VariationalQuantumCircuit

        8. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_X1) -> pyQPanda.VariationalQuantumCircuit

        9. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Y1) -> pyQPanda.VariationalQuantumCircuit

        10. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Z1) -> pyQPanda.VariationalQuantumCircuit

        11. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U1) -> pyQPanda.VariationalQuantumCircuit

        12. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U2) -> pyQPanda.VariationalQuantumCircuit

        13. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U3) -> pyQPanda.VariationalQuantumCircuit

        14. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U4) -> pyQPanda.VariationalQuantumCircuit

        15. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_RX) -> pyQPanda.VariationalQuantumCircuit

        16. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_RY) -> pyQPanda.VariationalQuantumCircuit

        17. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_RZ) -> pyQPanda.VariationalQuantumCircuit

        18. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CNOT) -> pyQPanda.VariationalQuantumCircuit

        19. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CR) -> pyQPanda.VariationalQuantumCircuit

        20. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CZ) -> pyQPanda.VariationalQuantumCircuit

        21. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CRX) -> pyQPanda.VariationalQuantumCircuit

        22. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CRY) -> pyQPanda.VariationalQuantumCircuit

        23. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CRZ) -> pyQPanda.VariationalQuantumCircuit

        24. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_SWAP) -> pyQPanda.VariationalQuantumCircuit

        25. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_iSWAP) -> pyQPanda.VariationalQuantumCircuit

        26. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_SqiSWAP) -> pyQPanda.VariationalQuantumCircuit

        27. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumCircuit) -> pyQPanda.VariationalQuantumCircuit

        28. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.QCircuit) -> pyQPanda.VariationalQuantumCircuit

        29. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.QGate) -> pyQPanda.VariationalQuantumCircuit
        """
    @overload
    def insert(self, arg0: VariationalQuantumGate_Z1) -> VariationalQuantumCircuit:
        """insert(*args, **kwargs)
        Overloaded function.

        1. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_I) -> pyQPanda.VariationalQuantumCircuit

        2. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_H) -> pyQPanda.VariationalQuantumCircuit

        3. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_X) -> pyQPanda.VariationalQuantumCircuit

        4. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Y) -> pyQPanda.VariationalQuantumCircuit

        5. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_T) -> pyQPanda.VariationalQuantumCircuit

        6. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_S) -> pyQPanda.VariationalQuantumCircuit

        7. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Z) -> pyQPanda.VariationalQuantumCircuit

        8. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_X1) -> pyQPanda.VariationalQuantumCircuit

        9. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Y1) -> pyQPanda.VariationalQuantumCircuit

        10. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Z1) -> pyQPanda.VariationalQuantumCircuit

        11. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U1) -> pyQPanda.VariationalQuantumCircuit

        12. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U2) -> pyQPanda.VariationalQuantumCircuit

        13. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U3) -> pyQPanda.VariationalQuantumCircuit

        14. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U4) -> pyQPanda.VariationalQuantumCircuit

        15. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_RX) -> pyQPanda.VariationalQuantumCircuit

        16. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_RY) -> pyQPanda.VariationalQuantumCircuit

        17. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_RZ) -> pyQPanda.VariationalQuantumCircuit

        18. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CNOT) -> pyQPanda.VariationalQuantumCircuit

        19. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CR) -> pyQPanda.VariationalQuantumCircuit

        20. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CZ) -> pyQPanda.VariationalQuantumCircuit

        21. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CRX) -> pyQPanda.VariationalQuantumCircuit

        22. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CRY) -> pyQPanda.VariationalQuantumCircuit

        23. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CRZ) -> pyQPanda.VariationalQuantumCircuit

        24. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_SWAP) -> pyQPanda.VariationalQuantumCircuit

        25. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_iSWAP) -> pyQPanda.VariationalQuantumCircuit

        26. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_SqiSWAP) -> pyQPanda.VariationalQuantumCircuit

        27. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumCircuit) -> pyQPanda.VariationalQuantumCircuit

        28. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.QCircuit) -> pyQPanda.VariationalQuantumCircuit

        29. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.QGate) -> pyQPanda.VariationalQuantumCircuit
        """
    @overload
    def insert(self, arg0: VariationalQuantumGate_U1) -> VariationalQuantumCircuit:
        """insert(*args, **kwargs)
        Overloaded function.

        1. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_I) -> pyQPanda.VariationalQuantumCircuit

        2. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_H) -> pyQPanda.VariationalQuantumCircuit

        3. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_X) -> pyQPanda.VariationalQuantumCircuit

        4. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Y) -> pyQPanda.VariationalQuantumCircuit

        5. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_T) -> pyQPanda.VariationalQuantumCircuit

        6. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_S) -> pyQPanda.VariationalQuantumCircuit

        7. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Z) -> pyQPanda.VariationalQuantumCircuit

        8. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_X1) -> pyQPanda.VariationalQuantumCircuit

        9. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Y1) -> pyQPanda.VariationalQuantumCircuit

        10. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Z1) -> pyQPanda.VariationalQuantumCircuit

        11. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U1) -> pyQPanda.VariationalQuantumCircuit

        12. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U2) -> pyQPanda.VariationalQuantumCircuit

        13. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U3) -> pyQPanda.VariationalQuantumCircuit

        14. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U4) -> pyQPanda.VariationalQuantumCircuit

        15. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_RX) -> pyQPanda.VariationalQuantumCircuit

        16. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_RY) -> pyQPanda.VariationalQuantumCircuit

        17. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_RZ) -> pyQPanda.VariationalQuantumCircuit

        18. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CNOT) -> pyQPanda.VariationalQuantumCircuit

        19. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CR) -> pyQPanda.VariationalQuantumCircuit

        20. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CZ) -> pyQPanda.VariationalQuantumCircuit

        21. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CRX) -> pyQPanda.VariationalQuantumCircuit

        22. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CRY) -> pyQPanda.VariationalQuantumCircuit

        23. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CRZ) -> pyQPanda.VariationalQuantumCircuit

        24. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_SWAP) -> pyQPanda.VariationalQuantumCircuit

        25. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_iSWAP) -> pyQPanda.VariationalQuantumCircuit

        26. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_SqiSWAP) -> pyQPanda.VariationalQuantumCircuit

        27. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumCircuit) -> pyQPanda.VariationalQuantumCircuit

        28. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.QCircuit) -> pyQPanda.VariationalQuantumCircuit

        29. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.QGate) -> pyQPanda.VariationalQuantumCircuit
        """
    @overload
    def insert(self, arg0: VariationalQuantumGate_U2) -> VariationalQuantumCircuit:
        """insert(*args, **kwargs)
        Overloaded function.

        1. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_I) -> pyQPanda.VariationalQuantumCircuit

        2. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_H) -> pyQPanda.VariationalQuantumCircuit

        3. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_X) -> pyQPanda.VariationalQuantumCircuit

        4. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Y) -> pyQPanda.VariationalQuantumCircuit

        5. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_T) -> pyQPanda.VariationalQuantumCircuit

        6. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_S) -> pyQPanda.VariationalQuantumCircuit

        7. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Z) -> pyQPanda.VariationalQuantumCircuit

        8. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_X1) -> pyQPanda.VariationalQuantumCircuit

        9. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Y1) -> pyQPanda.VariationalQuantumCircuit

        10. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Z1) -> pyQPanda.VariationalQuantumCircuit

        11. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U1) -> pyQPanda.VariationalQuantumCircuit

        12. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U2) -> pyQPanda.VariationalQuantumCircuit

        13. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U3) -> pyQPanda.VariationalQuantumCircuit

        14. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U4) -> pyQPanda.VariationalQuantumCircuit

        15. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_RX) -> pyQPanda.VariationalQuantumCircuit

        16. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_RY) -> pyQPanda.VariationalQuantumCircuit

        17. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_RZ) -> pyQPanda.VariationalQuantumCircuit

        18. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CNOT) -> pyQPanda.VariationalQuantumCircuit

        19. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CR) -> pyQPanda.VariationalQuantumCircuit

        20. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CZ) -> pyQPanda.VariationalQuantumCircuit

        21. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CRX) -> pyQPanda.VariationalQuantumCircuit

        22. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CRY) -> pyQPanda.VariationalQuantumCircuit

        23. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CRZ) -> pyQPanda.VariationalQuantumCircuit

        24. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_SWAP) -> pyQPanda.VariationalQuantumCircuit

        25. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_iSWAP) -> pyQPanda.VariationalQuantumCircuit

        26. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_SqiSWAP) -> pyQPanda.VariationalQuantumCircuit

        27. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumCircuit) -> pyQPanda.VariationalQuantumCircuit

        28. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.QCircuit) -> pyQPanda.VariationalQuantumCircuit

        29. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.QGate) -> pyQPanda.VariationalQuantumCircuit
        """
    @overload
    def insert(self, arg0: VariationalQuantumGate_U3) -> VariationalQuantumCircuit:
        """insert(*args, **kwargs)
        Overloaded function.

        1. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_I) -> pyQPanda.VariationalQuantumCircuit

        2. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_H) -> pyQPanda.VariationalQuantumCircuit

        3. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_X) -> pyQPanda.VariationalQuantumCircuit

        4. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Y) -> pyQPanda.VariationalQuantumCircuit

        5. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_T) -> pyQPanda.VariationalQuantumCircuit

        6. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_S) -> pyQPanda.VariationalQuantumCircuit

        7. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Z) -> pyQPanda.VariationalQuantumCircuit

        8. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_X1) -> pyQPanda.VariationalQuantumCircuit

        9. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Y1) -> pyQPanda.VariationalQuantumCircuit

        10. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Z1) -> pyQPanda.VariationalQuantumCircuit

        11. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U1) -> pyQPanda.VariationalQuantumCircuit

        12. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U2) -> pyQPanda.VariationalQuantumCircuit

        13. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U3) -> pyQPanda.VariationalQuantumCircuit

        14. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U4) -> pyQPanda.VariationalQuantumCircuit

        15. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_RX) -> pyQPanda.VariationalQuantumCircuit

        16. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_RY) -> pyQPanda.VariationalQuantumCircuit

        17. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_RZ) -> pyQPanda.VariationalQuantumCircuit

        18. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CNOT) -> pyQPanda.VariationalQuantumCircuit

        19. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CR) -> pyQPanda.VariationalQuantumCircuit

        20. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CZ) -> pyQPanda.VariationalQuantumCircuit

        21. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CRX) -> pyQPanda.VariationalQuantumCircuit

        22. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CRY) -> pyQPanda.VariationalQuantumCircuit

        23. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CRZ) -> pyQPanda.VariationalQuantumCircuit

        24. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_SWAP) -> pyQPanda.VariationalQuantumCircuit

        25. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_iSWAP) -> pyQPanda.VariationalQuantumCircuit

        26. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_SqiSWAP) -> pyQPanda.VariationalQuantumCircuit

        27. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumCircuit) -> pyQPanda.VariationalQuantumCircuit

        28. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.QCircuit) -> pyQPanda.VariationalQuantumCircuit

        29. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.QGate) -> pyQPanda.VariationalQuantumCircuit
        """
    @overload
    def insert(self, arg0: VariationalQuantumGate_U4) -> VariationalQuantumCircuit:
        """insert(*args, **kwargs)
        Overloaded function.

        1. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_I) -> pyQPanda.VariationalQuantumCircuit

        2. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_H) -> pyQPanda.VariationalQuantumCircuit

        3. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_X) -> pyQPanda.VariationalQuantumCircuit

        4. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Y) -> pyQPanda.VariationalQuantumCircuit

        5. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_T) -> pyQPanda.VariationalQuantumCircuit

        6. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_S) -> pyQPanda.VariationalQuantumCircuit

        7. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Z) -> pyQPanda.VariationalQuantumCircuit

        8. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_X1) -> pyQPanda.VariationalQuantumCircuit

        9. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Y1) -> pyQPanda.VariationalQuantumCircuit

        10. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Z1) -> pyQPanda.VariationalQuantumCircuit

        11. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U1) -> pyQPanda.VariationalQuantumCircuit

        12. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U2) -> pyQPanda.VariationalQuantumCircuit

        13. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U3) -> pyQPanda.VariationalQuantumCircuit

        14. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U4) -> pyQPanda.VariationalQuantumCircuit

        15. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_RX) -> pyQPanda.VariationalQuantumCircuit

        16. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_RY) -> pyQPanda.VariationalQuantumCircuit

        17. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_RZ) -> pyQPanda.VariationalQuantumCircuit

        18. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CNOT) -> pyQPanda.VariationalQuantumCircuit

        19. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CR) -> pyQPanda.VariationalQuantumCircuit

        20. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CZ) -> pyQPanda.VariationalQuantumCircuit

        21. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CRX) -> pyQPanda.VariationalQuantumCircuit

        22. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CRY) -> pyQPanda.VariationalQuantumCircuit

        23. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CRZ) -> pyQPanda.VariationalQuantumCircuit

        24. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_SWAP) -> pyQPanda.VariationalQuantumCircuit

        25. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_iSWAP) -> pyQPanda.VariationalQuantumCircuit

        26. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_SqiSWAP) -> pyQPanda.VariationalQuantumCircuit

        27. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumCircuit) -> pyQPanda.VariationalQuantumCircuit

        28. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.QCircuit) -> pyQPanda.VariationalQuantumCircuit

        29. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.QGate) -> pyQPanda.VariationalQuantumCircuit
        """
    @overload
    def insert(self, arg0: VariationalQuantumGate_RX) -> VariationalQuantumCircuit:
        """insert(*args, **kwargs)
        Overloaded function.

        1. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_I) -> pyQPanda.VariationalQuantumCircuit

        2. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_H) -> pyQPanda.VariationalQuantumCircuit

        3. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_X) -> pyQPanda.VariationalQuantumCircuit

        4. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Y) -> pyQPanda.VariationalQuantumCircuit

        5. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_T) -> pyQPanda.VariationalQuantumCircuit

        6. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_S) -> pyQPanda.VariationalQuantumCircuit

        7. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Z) -> pyQPanda.VariationalQuantumCircuit

        8. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_X1) -> pyQPanda.VariationalQuantumCircuit

        9. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Y1) -> pyQPanda.VariationalQuantumCircuit

        10. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Z1) -> pyQPanda.VariationalQuantumCircuit

        11. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U1) -> pyQPanda.VariationalQuantumCircuit

        12. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U2) -> pyQPanda.VariationalQuantumCircuit

        13. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U3) -> pyQPanda.VariationalQuantumCircuit

        14. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U4) -> pyQPanda.VariationalQuantumCircuit

        15. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_RX) -> pyQPanda.VariationalQuantumCircuit

        16. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_RY) -> pyQPanda.VariationalQuantumCircuit

        17. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_RZ) -> pyQPanda.VariationalQuantumCircuit

        18. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CNOT) -> pyQPanda.VariationalQuantumCircuit

        19. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CR) -> pyQPanda.VariationalQuantumCircuit

        20. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CZ) -> pyQPanda.VariationalQuantumCircuit

        21. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CRX) -> pyQPanda.VariationalQuantumCircuit

        22. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CRY) -> pyQPanda.VariationalQuantumCircuit

        23. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CRZ) -> pyQPanda.VariationalQuantumCircuit

        24. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_SWAP) -> pyQPanda.VariationalQuantumCircuit

        25. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_iSWAP) -> pyQPanda.VariationalQuantumCircuit

        26. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_SqiSWAP) -> pyQPanda.VariationalQuantumCircuit

        27. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumCircuit) -> pyQPanda.VariationalQuantumCircuit

        28. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.QCircuit) -> pyQPanda.VariationalQuantumCircuit

        29. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.QGate) -> pyQPanda.VariationalQuantumCircuit
        """
    @overload
    def insert(self, arg0: VariationalQuantumGate_RY) -> VariationalQuantumCircuit:
        """insert(*args, **kwargs)
        Overloaded function.

        1. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_I) -> pyQPanda.VariationalQuantumCircuit

        2. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_H) -> pyQPanda.VariationalQuantumCircuit

        3. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_X) -> pyQPanda.VariationalQuantumCircuit

        4. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Y) -> pyQPanda.VariationalQuantumCircuit

        5. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_T) -> pyQPanda.VariationalQuantumCircuit

        6. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_S) -> pyQPanda.VariationalQuantumCircuit

        7. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Z) -> pyQPanda.VariationalQuantumCircuit

        8. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_X1) -> pyQPanda.VariationalQuantumCircuit

        9. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Y1) -> pyQPanda.VariationalQuantumCircuit

        10. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Z1) -> pyQPanda.VariationalQuantumCircuit

        11. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U1) -> pyQPanda.VariationalQuantumCircuit

        12. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U2) -> pyQPanda.VariationalQuantumCircuit

        13. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U3) -> pyQPanda.VariationalQuantumCircuit

        14. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U4) -> pyQPanda.VariationalQuantumCircuit

        15. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_RX) -> pyQPanda.VariationalQuantumCircuit

        16. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_RY) -> pyQPanda.VariationalQuantumCircuit

        17. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_RZ) -> pyQPanda.VariationalQuantumCircuit

        18. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CNOT) -> pyQPanda.VariationalQuantumCircuit

        19. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CR) -> pyQPanda.VariationalQuantumCircuit

        20. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CZ) -> pyQPanda.VariationalQuantumCircuit

        21. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CRX) -> pyQPanda.VariationalQuantumCircuit

        22. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CRY) -> pyQPanda.VariationalQuantumCircuit

        23. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CRZ) -> pyQPanda.VariationalQuantumCircuit

        24. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_SWAP) -> pyQPanda.VariationalQuantumCircuit

        25. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_iSWAP) -> pyQPanda.VariationalQuantumCircuit

        26. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_SqiSWAP) -> pyQPanda.VariationalQuantumCircuit

        27. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumCircuit) -> pyQPanda.VariationalQuantumCircuit

        28. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.QCircuit) -> pyQPanda.VariationalQuantumCircuit

        29. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.QGate) -> pyQPanda.VariationalQuantumCircuit
        """
    @overload
    def insert(self, arg0: VariationalQuantumGate_RZ) -> VariationalQuantumCircuit:
        """insert(*args, **kwargs)
        Overloaded function.

        1. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_I) -> pyQPanda.VariationalQuantumCircuit

        2. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_H) -> pyQPanda.VariationalQuantumCircuit

        3. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_X) -> pyQPanda.VariationalQuantumCircuit

        4. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Y) -> pyQPanda.VariationalQuantumCircuit

        5. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_T) -> pyQPanda.VariationalQuantumCircuit

        6. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_S) -> pyQPanda.VariationalQuantumCircuit

        7. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Z) -> pyQPanda.VariationalQuantumCircuit

        8. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_X1) -> pyQPanda.VariationalQuantumCircuit

        9. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Y1) -> pyQPanda.VariationalQuantumCircuit

        10. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Z1) -> pyQPanda.VariationalQuantumCircuit

        11. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U1) -> pyQPanda.VariationalQuantumCircuit

        12. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U2) -> pyQPanda.VariationalQuantumCircuit

        13. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U3) -> pyQPanda.VariationalQuantumCircuit

        14. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U4) -> pyQPanda.VariationalQuantumCircuit

        15. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_RX) -> pyQPanda.VariationalQuantumCircuit

        16. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_RY) -> pyQPanda.VariationalQuantumCircuit

        17. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_RZ) -> pyQPanda.VariationalQuantumCircuit

        18. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CNOT) -> pyQPanda.VariationalQuantumCircuit

        19. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CR) -> pyQPanda.VariationalQuantumCircuit

        20. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CZ) -> pyQPanda.VariationalQuantumCircuit

        21. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CRX) -> pyQPanda.VariationalQuantumCircuit

        22. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CRY) -> pyQPanda.VariationalQuantumCircuit

        23. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CRZ) -> pyQPanda.VariationalQuantumCircuit

        24. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_SWAP) -> pyQPanda.VariationalQuantumCircuit

        25. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_iSWAP) -> pyQPanda.VariationalQuantumCircuit

        26. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_SqiSWAP) -> pyQPanda.VariationalQuantumCircuit

        27. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumCircuit) -> pyQPanda.VariationalQuantumCircuit

        28. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.QCircuit) -> pyQPanda.VariationalQuantumCircuit

        29. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.QGate) -> pyQPanda.VariationalQuantumCircuit
        """
    @overload
    def insert(self, arg0: VariationalQuantumGate_CNOT) -> VariationalQuantumCircuit:
        """insert(*args, **kwargs)
        Overloaded function.

        1. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_I) -> pyQPanda.VariationalQuantumCircuit

        2. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_H) -> pyQPanda.VariationalQuantumCircuit

        3. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_X) -> pyQPanda.VariationalQuantumCircuit

        4. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Y) -> pyQPanda.VariationalQuantumCircuit

        5. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_T) -> pyQPanda.VariationalQuantumCircuit

        6. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_S) -> pyQPanda.VariationalQuantumCircuit

        7. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Z) -> pyQPanda.VariationalQuantumCircuit

        8. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_X1) -> pyQPanda.VariationalQuantumCircuit

        9. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Y1) -> pyQPanda.VariationalQuantumCircuit

        10. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Z1) -> pyQPanda.VariationalQuantumCircuit

        11. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U1) -> pyQPanda.VariationalQuantumCircuit

        12. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U2) -> pyQPanda.VariationalQuantumCircuit

        13. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U3) -> pyQPanda.VariationalQuantumCircuit

        14. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U4) -> pyQPanda.VariationalQuantumCircuit

        15. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_RX) -> pyQPanda.VariationalQuantumCircuit

        16. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_RY) -> pyQPanda.VariationalQuantumCircuit

        17. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_RZ) -> pyQPanda.VariationalQuantumCircuit

        18. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CNOT) -> pyQPanda.VariationalQuantumCircuit

        19. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CR) -> pyQPanda.VariationalQuantumCircuit

        20. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CZ) -> pyQPanda.VariationalQuantumCircuit

        21. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CRX) -> pyQPanda.VariationalQuantumCircuit

        22. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CRY) -> pyQPanda.VariationalQuantumCircuit

        23. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CRZ) -> pyQPanda.VariationalQuantumCircuit

        24. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_SWAP) -> pyQPanda.VariationalQuantumCircuit

        25. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_iSWAP) -> pyQPanda.VariationalQuantumCircuit

        26. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_SqiSWAP) -> pyQPanda.VariationalQuantumCircuit

        27. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumCircuit) -> pyQPanda.VariationalQuantumCircuit

        28. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.QCircuit) -> pyQPanda.VariationalQuantumCircuit

        29. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.QGate) -> pyQPanda.VariationalQuantumCircuit
        """
    @overload
    def insert(self, arg0: VariationalQuantumGate_CR) -> VariationalQuantumCircuit:
        """insert(*args, **kwargs)
        Overloaded function.

        1. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_I) -> pyQPanda.VariationalQuantumCircuit

        2. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_H) -> pyQPanda.VariationalQuantumCircuit

        3. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_X) -> pyQPanda.VariationalQuantumCircuit

        4. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Y) -> pyQPanda.VariationalQuantumCircuit

        5. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_T) -> pyQPanda.VariationalQuantumCircuit

        6. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_S) -> pyQPanda.VariationalQuantumCircuit

        7. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Z) -> pyQPanda.VariationalQuantumCircuit

        8. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_X1) -> pyQPanda.VariationalQuantumCircuit

        9. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Y1) -> pyQPanda.VariationalQuantumCircuit

        10. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Z1) -> pyQPanda.VariationalQuantumCircuit

        11. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U1) -> pyQPanda.VariationalQuantumCircuit

        12. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U2) -> pyQPanda.VariationalQuantumCircuit

        13. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U3) -> pyQPanda.VariationalQuantumCircuit

        14. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U4) -> pyQPanda.VariationalQuantumCircuit

        15. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_RX) -> pyQPanda.VariationalQuantumCircuit

        16. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_RY) -> pyQPanda.VariationalQuantumCircuit

        17. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_RZ) -> pyQPanda.VariationalQuantumCircuit

        18. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CNOT) -> pyQPanda.VariationalQuantumCircuit

        19. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CR) -> pyQPanda.VariationalQuantumCircuit

        20. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CZ) -> pyQPanda.VariationalQuantumCircuit

        21. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CRX) -> pyQPanda.VariationalQuantumCircuit

        22. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CRY) -> pyQPanda.VariationalQuantumCircuit

        23. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CRZ) -> pyQPanda.VariationalQuantumCircuit

        24. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_SWAP) -> pyQPanda.VariationalQuantumCircuit

        25. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_iSWAP) -> pyQPanda.VariationalQuantumCircuit

        26. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_SqiSWAP) -> pyQPanda.VariationalQuantumCircuit

        27. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumCircuit) -> pyQPanda.VariationalQuantumCircuit

        28. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.QCircuit) -> pyQPanda.VariationalQuantumCircuit

        29. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.QGate) -> pyQPanda.VariationalQuantumCircuit
        """
    @overload
    def insert(self, arg0: VariationalQuantumGate_CZ) -> VariationalQuantumCircuit:
        """insert(*args, **kwargs)
        Overloaded function.

        1. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_I) -> pyQPanda.VariationalQuantumCircuit

        2. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_H) -> pyQPanda.VariationalQuantumCircuit

        3. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_X) -> pyQPanda.VariationalQuantumCircuit

        4. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Y) -> pyQPanda.VariationalQuantumCircuit

        5. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_T) -> pyQPanda.VariationalQuantumCircuit

        6. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_S) -> pyQPanda.VariationalQuantumCircuit

        7. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Z) -> pyQPanda.VariationalQuantumCircuit

        8. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_X1) -> pyQPanda.VariationalQuantumCircuit

        9. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Y1) -> pyQPanda.VariationalQuantumCircuit

        10. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Z1) -> pyQPanda.VariationalQuantumCircuit

        11. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U1) -> pyQPanda.VariationalQuantumCircuit

        12. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U2) -> pyQPanda.VariationalQuantumCircuit

        13. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U3) -> pyQPanda.VariationalQuantumCircuit

        14. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U4) -> pyQPanda.VariationalQuantumCircuit

        15. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_RX) -> pyQPanda.VariationalQuantumCircuit

        16. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_RY) -> pyQPanda.VariationalQuantumCircuit

        17. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_RZ) -> pyQPanda.VariationalQuantumCircuit

        18. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CNOT) -> pyQPanda.VariationalQuantumCircuit

        19. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CR) -> pyQPanda.VariationalQuantumCircuit

        20. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CZ) -> pyQPanda.VariationalQuantumCircuit

        21. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CRX) -> pyQPanda.VariationalQuantumCircuit

        22. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CRY) -> pyQPanda.VariationalQuantumCircuit

        23. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CRZ) -> pyQPanda.VariationalQuantumCircuit

        24. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_SWAP) -> pyQPanda.VariationalQuantumCircuit

        25. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_iSWAP) -> pyQPanda.VariationalQuantumCircuit

        26. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_SqiSWAP) -> pyQPanda.VariationalQuantumCircuit

        27. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumCircuit) -> pyQPanda.VariationalQuantumCircuit

        28. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.QCircuit) -> pyQPanda.VariationalQuantumCircuit

        29. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.QGate) -> pyQPanda.VariationalQuantumCircuit
        """
    @overload
    def insert(self, arg0: VariationalQuantumGate_CRX) -> VariationalQuantumCircuit:
        """insert(*args, **kwargs)
        Overloaded function.

        1. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_I) -> pyQPanda.VariationalQuantumCircuit

        2. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_H) -> pyQPanda.VariationalQuantumCircuit

        3. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_X) -> pyQPanda.VariationalQuantumCircuit

        4. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Y) -> pyQPanda.VariationalQuantumCircuit

        5. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_T) -> pyQPanda.VariationalQuantumCircuit

        6. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_S) -> pyQPanda.VariationalQuantumCircuit

        7. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Z) -> pyQPanda.VariationalQuantumCircuit

        8. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_X1) -> pyQPanda.VariationalQuantumCircuit

        9. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Y1) -> pyQPanda.VariationalQuantumCircuit

        10. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Z1) -> pyQPanda.VariationalQuantumCircuit

        11. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U1) -> pyQPanda.VariationalQuantumCircuit

        12. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U2) -> pyQPanda.VariationalQuantumCircuit

        13. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U3) -> pyQPanda.VariationalQuantumCircuit

        14. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U4) -> pyQPanda.VariationalQuantumCircuit

        15. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_RX) -> pyQPanda.VariationalQuantumCircuit

        16. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_RY) -> pyQPanda.VariationalQuantumCircuit

        17. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_RZ) -> pyQPanda.VariationalQuantumCircuit

        18. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CNOT) -> pyQPanda.VariationalQuantumCircuit

        19. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CR) -> pyQPanda.VariationalQuantumCircuit

        20. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CZ) -> pyQPanda.VariationalQuantumCircuit

        21. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CRX) -> pyQPanda.VariationalQuantumCircuit

        22. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CRY) -> pyQPanda.VariationalQuantumCircuit

        23. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CRZ) -> pyQPanda.VariationalQuantumCircuit

        24. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_SWAP) -> pyQPanda.VariationalQuantumCircuit

        25. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_iSWAP) -> pyQPanda.VariationalQuantumCircuit

        26. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_SqiSWAP) -> pyQPanda.VariationalQuantumCircuit

        27. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumCircuit) -> pyQPanda.VariationalQuantumCircuit

        28. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.QCircuit) -> pyQPanda.VariationalQuantumCircuit

        29. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.QGate) -> pyQPanda.VariationalQuantumCircuit
        """
    @overload
    def insert(self, arg0: VariationalQuantumGate_CRY) -> VariationalQuantumCircuit:
        """insert(*args, **kwargs)
        Overloaded function.

        1. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_I) -> pyQPanda.VariationalQuantumCircuit

        2. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_H) -> pyQPanda.VariationalQuantumCircuit

        3. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_X) -> pyQPanda.VariationalQuantumCircuit

        4. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Y) -> pyQPanda.VariationalQuantumCircuit

        5. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_T) -> pyQPanda.VariationalQuantumCircuit

        6. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_S) -> pyQPanda.VariationalQuantumCircuit

        7. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Z) -> pyQPanda.VariationalQuantumCircuit

        8. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_X1) -> pyQPanda.VariationalQuantumCircuit

        9. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Y1) -> pyQPanda.VariationalQuantumCircuit

        10. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Z1) -> pyQPanda.VariationalQuantumCircuit

        11. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U1) -> pyQPanda.VariationalQuantumCircuit

        12. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U2) -> pyQPanda.VariationalQuantumCircuit

        13. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U3) -> pyQPanda.VariationalQuantumCircuit

        14. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U4) -> pyQPanda.VariationalQuantumCircuit

        15. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_RX) -> pyQPanda.VariationalQuantumCircuit

        16. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_RY) -> pyQPanda.VariationalQuantumCircuit

        17. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_RZ) -> pyQPanda.VariationalQuantumCircuit

        18. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CNOT) -> pyQPanda.VariationalQuantumCircuit

        19. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CR) -> pyQPanda.VariationalQuantumCircuit

        20. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CZ) -> pyQPanda.VariationalQuantumCircuit

        21. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CRX) -> pyQPanda.VariationalQuantumCircuit

        22. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CRY) -> pyQPanda.VariationalQuantumCircuit

        23. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CRZ) -> pyQPanda.VariationalQuantumCircuit

        24. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_SWAP) -> pyQPanda.VariationalQuantumCircuit

        25. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_iSWAP) -> pyQPanda.VariationalQuantumCircuit

        26. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_SqiSWAP) -> pyQPanda.VariationalQuantumCircuit

        27. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumCircuit) -> pyQPanda.VariationalQuantumCircuit

        28. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.QCircuit) -> pyQPanda.VariationalQuantumCircuit

        29. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.QGate) -> pyQPanda.VariationalQuantumCircuit
        """
    @overload
    def insert(self, arg0: VariationalQuantumGate_CRZ) -> VariationalQuantumCircuit:
        """insert(*args, **kwargs)
        Overloaded function.

        1. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_I) -> pyQPanda.VariationalQuantumCircuit

        2. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_H) -> pyQPanda.VariationalQuantumCircuit

        3. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_X) -> pyQPanda.VariationalQuantumCircuit

        4. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Y) -> pyQPanda.VariationalQuantumCircuit

        5. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_T) -> pyQPanda.VariationalQuantumCircuit

        6. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_S) -> pyQPanda.VariationalQuantumCircuit

        7. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Z) -> pyQPanda.VariationalQuantumCircuit

        8. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_X1) -> pyQPanda.VariationalQuantumCircuit

        9. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Y1) -> pyQPanda.VariationalQuantumCircuit

        10. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Z1) -> pyQPanda.VariationalQuantumCircuit

        11. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U1) -> pyQPanda.VariationalQuantumCircuit

        12. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U2) -> pyQPanda.VariationalQuantumCircuit

        13. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U3) -> pyQPanda.VariationalQuantumCircuit

        14. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U4) -> pyQPanda.VariationalQuantumCircuit

        15. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_RX) -> pyQPanda.VariationalQuantumCircuit

        16. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_RY) -> pyQPanda.VariationalQuantumCircuit

        17. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_RZ) -> pyQPanda.VariationalQuantumCircuit

        18. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CNOT) -> pyQPanda.VariationalQuantumCircuit

        19. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CR) -> pyQPanda.VariationalQuantumCircuit

        20. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CZ) -> pyQPanda.VariationalQuantumCircuit

        21. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CRX) -> pyQPanda.VariationalQuantumCircuit

        22. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CRY) -> pyQPanda.VariationalQuantumCircuit

        23. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CRZ) -> pyQPanda.VariationalQuantumCircuit

        24. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_SWAP) -> pyQPanda.VariationalQuantumCircuit

        25. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_iSWAP) -> pyQPanda.VariationalQuantumCircuit

        26. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_SqiSWAP) -> pyQPanda.VariationalQuantumCircuit

        27. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumCircuit) -> pyQPanda.VariationalQuantumCircuit

        28. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.QCircuit) -> pyQPanda.VariationalQuantumCircuit

        29. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.QGate) -> pyQPanda.VariationalQuantumCircuit
        """
    @overload
    def insert(self, arg0: VariationalQuantumGate_SWAP) -> VariationalQuantumCircuit:
        """insert(*args, **kwargs)
        Overloaded function.

        1. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_I) -> pyQPanda.VariationalQuantumCircuit

        2. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_H) -> pyQPanda.VariationalQuantumCircuit

        3. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_X) -> pyQPanda.VariationalQuantumCircuit

        4. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Y) -> pyQPanda.VariationalQuantumCircuit

        5. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_T) -> pyQPanda.VariationalQuantumCircuit

        6. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_S) -> pyQPanda.VariationalQuantumCircuit

        7. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Z) -> pyQPanda.VariationalQuantumCircuit

        8. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_X1) -> pyQPanda.VariationalQuantumCircuit

        9. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Y1) -> pyQPanda.VariationalQuantumCircuit

        10. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Z1) -> pyQPanda.VariationalQuantumCircuit

        11. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U1) -> pyQPanda.VariationalQuantumCircuit

        12. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U2) -> pyQPanda.VariationalQuantumCircuit

        13. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U3) -> pyQPanda.VariationalQuantumCircuit

        14. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U4) -> pyQPanda.VariationalQuantumCircuit

        15. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_RX) -> pyQPanda.VariationalQuantumCircuit

        16. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_RY) -> pyQPanda.VariationalQuantumCircuit

        17. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_RZ) -> pyQPanda.VariationalQuantumCircuit

        18. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CNOT) -> pyQPanda.VariationalQuantumCircuit

        19. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CR) -> pyQPanda.VariationalQuantumCircuit

        20. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CZ) -> pyQPanda.VariationalQuantumCircuit

        21. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CRX) -> pyQPanda.VariationalQuantumCircuit

        22. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CRY) -> pyQPanda.VariationalQuantumCircuit

        23. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CRZ) -> pyQPanda.VariationalQuantumCircuit

        24. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_SWAP) -> pyQPanda.VariationalQuantumCircuit

        25. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_iSWAP) -> pyQPanda.VariationalQuantumCircuit

        26. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_SqiSWAP) -> pyQPanda.VariationalQuantumCircuit

        27. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumCircuit) -> pyQPanda.VariationalQuantumCircuit

        28. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.QCircuit) -> pyQPanda.VariationalQuantumCircuit

        29. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.QGate) -> pyQPanda.VariationalQuantumCircuit
        """
    @overload
    def insert(self, arg0: VariationalQuantumGate_iSWAP) -> VariationalQuantumCircuit:
        """insert(*args, **kwargs)
        Overloaded function.

        1. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_I) -> pyQPanda.VariationalQuantumCircuit

        2. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_H) -> pyQPanda.VariationalQuantumCircuit

        3. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_X) -> pyQPanda.VariationalQuantumCircuit

        4. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Y) -> pyQPanda.VariationalQuantumCircuit

        5. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_T) -> pyQPanda.VariationalQuantumCircuit

        6. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_S) -> pyQPanda.VariationalQuantumCircuit

        7. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Z) -> pyQPanda.VariationalQuantumCircuit

        8. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_X1) -> pyQPanda.VariationalQuantumCircuit

        9. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Y1) -> pyQPanda.VariationalQuantumCircuit

        10. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Z1) -> pyQPanda.VariationalQuantumCircuit

        11. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U1) -> pyQPanda.VariationalQuantumCircuit

        12. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U2) -> pyQPanda.VariationalQuantumCircuit

        13. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U3) -> pyQPanda.VariationalQuantumCircuit

        14. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U4) -> pyQPanda.VariationalQuantumCircuit

        15. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_RX) -> pyQPanda.VariationalQuantumCircuit

        16. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_RY) -> pyQPanda.VariationalQuantumCircuit

        17. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_RZ) -> pyQPanda.VariationalQuantumCircuit

        18. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CNOT) -> pyQPanda.VariationalQuantumCircuit

        19. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CR) -> pyQPanda.VariationalQuantumCircuit

        20. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CZ) -> pyQPanda.VariationalQuantumCircuit

        21. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CRX) -> pyQPanda.VariationalQuantumCircuit

        22. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CRY) -> pyQPanda.VariationalQuantumCircuit

        23. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CRZ) -> pyQPanda.VariationalQuantumCircuit

        24. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_SWAP) -> pyQPanda.VariationalQuantumCircuit

        25. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_iSWAP) -> pyQPanda.VariationalQuantumCircuit

        26. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_SqiSWAP) -> pyQPanda.VariationalQuantumCircuit

        27. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumCircuit) -> pyQPanda.VariationalQuantumCircuit

        28. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.QCircuit) -> pyQPanda.VariationalQuantumCircuit

        29. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.QGate) -> pyQPanda.VariationalQuantumCircuit
        """
    @overload
    def insert(self, arg0: VariationalQuantumGate_SqiSWAP) -> VariationalQuantumCircuit:
        """insert(*args, **kwargs)
        Overloaded function.

        1. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_I) -> pyQPanda.VariationalQuantumCircuit

        2. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_H) -> pyQPanda.VariationalQuantumCircuit

        3. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_X) -> pyQPanda.VariationalQuantumCircuit

        4. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Y) -> pyQPanda.VariationalQuantumCircuit

        5. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_T) -> pyQPanda.VariationalQuantumCircuit

        6. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_S) -> pyQPanda.VariationalQuantumCircuit

        7. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Z) -> pyQPanda.VariationalQuantumCircuit

        8. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_X1) -> pyQPanda.VariationalQuantumCircuit

        9. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Y1) -> pyQPanda.VariationalQuantumCircuit

        10. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Z1) -> pyQPanda.VariationalQuantumCircuit

        11. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U1) -> pyQPanda.VariationalQuantumCircuit

        12. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U2) -> pyQPanda.VariationalQuantumCircuit

        13. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U3) -> pyQPanda.VariationalQuantumCircuit

        14. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U4) -> pyQPanda.VariationalQuantumCircuit

        15. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_RX) -> pyQPanda.VariationalQuantumCircuit

        16. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_RY) -> pyQPanda.VariationalQuantumCircuit

        17. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_RZ) -> pyQPanda.VariationalQuantumCircuit

        18. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CNOT) -> pyQPanda.VariationalQuantumCircuit

        19. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CR) -> pyQPanda.VariationalQuantumCircuit

        20. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CZ) -> pyQPanda.VariationalQuantumCircuit

        21. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CRX) -> pyQPanda.VariationalQuantumCircuit

        22. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CRY) -> pyQPanda.VariationalQuantumCircuit

        23. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CRZ) -> pyQPanda.VariationalQuantumCircuit

        24. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_SWAP) -> pyQPanda.VariationalQuantumCircuit

        25. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_iSWAP) -> pyQPanda.VariationalQuantumCircuit

        26. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_SqiSWAP) -> pyQPanda.VariationalQuantumCircuit

        27. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumCircuit) -> pyQPanda.VariationalQuantumCircuit

        28. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.QCircuit) -> pyQPanda.VariationalQuantumCircuit

        29. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.QGate) -> pyQPanda.VariationalQuantumCircuit
        """
    @overload
    def insert(self, arg0: VariationalQuantumCircuit) -> VariationalQuantumCircuit:
        """insert(*args, **kwargs)
        Overloaded function.

        1. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_I) -> pyQPanda.VariationalQuantumCircuit

        2. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_H) -> pyQPanda.VariationalQuantumCircuit

        3. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_X) -> pyQPanda.VariationalQuantumCircuit

        4. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Y) -> pyQPanda.VariationalQuantumCircuit

        5. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_T) -> pyQPanda.VariationalQuantumCircuit

        6. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_S) -> pyQPanda.VariationalQuantumCircuit

        7. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Z) -> pyQPanda.VariationalQuantumCircuit

        8. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_X1) -> pyQPanda.VariationalQuantumCircuit

        9. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Y1) -> pyQPanda.VariationalQuantumCircuit

        10. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Z1) -> pyQPanda.VariationalQuantumCircuit

        11. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U1) -> pyQPanda.VariationalQuantumCircuit

        12. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U2) -> pyQPanda.VariationalQuantumCircuit

        13. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U3) -> pyQPanda.VariationalQuantumCircuit

        14. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U4) -> pyQPanda.VariationalQuantumCircuit

        15. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_RX) -> pyQPanda.VariationalQuantumCircuit

        16. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_RY) -> pyQPanda.VariationalQuantumCircuit

        17. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_RZ) -> pyQPanda.VariationalQuantumCircuit

        18. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CNOT) -> pyQPanda.VariationalQuantumCircuit

        19. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CR) -> pyQPanda.VariationalQuantumCircuit

        20. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CZ) -> pyQPanda.VariationalQuantumCircuit

        21. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CRX) -> pyQPanda.VariationalQuantumCircuit

        22. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CRY) -> pyQPanda.VariationalQuantumCircuit

        23. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CRZ) -> pyQPanda.VariationalQuantumCircuit

        24. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_SWAP) -> pyQPanda.VariationalQuantumCircuit

        25. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_iSWAP) -> pyQPanda.VariationalQuantumCircuit

        26. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_SqiSWAP) -> pyQPanda.VariationalQuantumCircuit

        27. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumCircuit) -> pyQPanda.VariationalQuantumCircuit

        28. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.QCircuit) -> pyQPanda.VariationalQuantumCircuit

        29. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.QGate) -> pyQPanda.VariationalQuantumCircuit
        """
    @overload
    def insert(self, arg0: QCircuit) -> VariationalQuantumCircuit:
        """insert(*args, **kwargs)
        Overloaded function.

        1. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_I) -> pyQPanda.VariationalQuantumCircuit

        2. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_H) -> pyQPanda.VariationalQuantumCircuit

        3. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_X) -> pyQPanda.VariationalQuantumCircuit

        4. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Y) -> pyQPanda.VariationalQuantumCircuit

        5. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_T) -> pyQPanda.VariationalQuantumCircuit

        6. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_S) -> pyQPanda.VariationalQuantumCircuit

        7. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Z) -> pyQPanda.VariationalQuantumCircuit

        8. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_X1) -> pyQPanda.VariationalQuantumCircuit

        9. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Y1) -> pyQPanda.VariationalQuantumCircuit

        10. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Z1) -> pyQPanda.VariationalQuantumCircuit

        11. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U1) -> pyQPanda.VariationalQuantumCircuit

        12. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U2) -> pyQPanda.VariationalQuantumCircuit

        13. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U3) -> pyQPanda.VariationalQuantumCircuit

        14. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U4) -> pyQPanda.VariationalQuantumCircuit

        15. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_RX) -> pyQPanda.VariationalQuantumCircuit

        16. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_RY) -> pyQPanda.VariationalQuantumCircuit

        17. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_RZ) -> pyQPanda.VariationalQuantumCircuit

        18. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CNOT) -> pyQPanda.VariationalQuantumCircuit

        19. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CR) -> pyQPanda.VariationalQuantumCircuit

        20. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CZ) -> pyQPanda.VariationalQuantumCircuit

        21. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CRX) -> pyQPanda.VariationalQuantumCircuit

        22. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CRY) -> pyQPanda.VariationalQuantumCircuit

        23. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CRZ) -> pyQPanda.VariationalQuantumCircuit

        24. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_SWAP) -> pyQPanda.VariationalQuantumCircuit

        25. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_iSWAP) -> pyQPanda.VariationalQuantumCircuit

        26. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_SqiSWAP) -> pyQPanda.VariationalQuantumCircuit

        27. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumCircuit) -> pyQPanda.VariationalQuantumCircuit

        28. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.QCircuit) -> pyQPanda.VariationalQuantumCircuit

        29. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.QGate) -> pyQPanda.VariationalQuantumCircuit
        """
    @overload
    def insert(self, arg0: QGate) -> VariationalQuantumCircuit:
        """insert(*args, **kwargs)
        Overloaded function.

        1. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_I) -> pyQPanda.VariationalQuantumCircuit

        2. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_H) -> pyQPanda.VariationalQuantumCircuit

        3. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_X) -> pyQPanda.VariationalQuantumCircuit

        4. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Y) -> pyQPanda.VariationalQuantumCircuit

        5. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_T) -> pyQPanda.VariationalQuantumCircuit

        6. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_S) -> pyQPanda.VariationalQuantumCircuit

        7. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Z) -> pyQPanda.VariationalQuantumCircuit

        8. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_X1) -> pyQPanda.VariationalQuantumCircuit

        9. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Y1) -> pyQPanda.VariationalQuantumCircuit

        10. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Z1) -> pyQPanda.VariationalQuantumCircuit

        11. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U1) -> pyQPanda.VariationalQuantumCircuit

        12. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U2) -> pyQPanda.VariationalQuantumCircuit

        13. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U3) -> pyQPanda.VariationalQuantumCircuit

        14. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U4) -> pyQPanda.VariationalQuantumCircuit

        15. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_RX) -> pyQPanda.VariationalQuantumCircuit

        16. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_RY) -> pyQPanda.VariationalQuantumCircuit

        17. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_RZ) -> pyQPanda.VariationalQuantumCircuit

        18. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CNOT) -> pyQPanda.VariationalQuantumCircuit

        19. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CR) -> pyQPanda.VariationalQuantumCircuit

        20. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CZ) -> pyQPanda.VariationalQuantumCircuit

        21. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CRX) -> pyQPanda.VariationalQuantumCircuit

        22. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CRY) -> pyQPanda.VariationalQuantumCircuit

        23. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CRZ) -> pyQPanda.VariationalQuantumCircuit

        24. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_SWAP) -> pyQPanda.VariationalQuantumCircuit

        25. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_iSWAP) -> pyQPanda.VariationalQuantumCircuit

        26. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_SqiSWAP) -> pyQPanda.VariationalQuantumCircuit

        27. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumCircuit) -> pyQPanda.VariationalQuantumCircuit

        28. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.QCircuit) -> pyQPanda.VariationalQuantumCircuit

        29. insert(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.QGate) -> pyQPanda.VariationalQuantumCircuit
        """
    def is_dagger(self) -> bool:
        """is_dagger(self: pyQPanda.VariationalQuantumCircuit) -> bool"""
    def set_control(self, arg0: QVec) -> bool:
        """set_control(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.QVec) -> bool"""
    def set_dagger(self, arg0: bool) -> bool:
        """set_dagger(self: pyQPanda.VariationalQuantumCircuit, arg0: bool) -> bool"""
    @overload
    def __lshift__(self, arg0: VariationalQuantumGate_I) -> VariationalQuantumCircuit:
        """__lshift__(*args, **kwargs)
        Overloaded function.

        1. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_I) -> pyQPanda.VariationalQuantumCircuit

        2. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_H) -> pyQPanda.VariationalQuantumCircuit

        3. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_X) -> pyQPanda.VariationalQuantumCircuit

        4. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Y) -> pyQPanda.VariationalQuantumCircuit

        5. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_T) -> pyQPanda.VariationalQuantumCircuit

        6. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_S) -> pyQPanda.VariationalQuantumCircuit

        7. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Z) -> pyQPanda.VariationalQuantumCircuit

        8. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_X1) -> pyQPanda.VariationalQuantumCircuit

        9. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Y1) -> pyQPanda.VariationalQuantumCircuit

        10. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Z1) -> pyQPanda.VariationalQuantumCircuit

        11. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U1) -> pyQPanda.VariationalQuantumCircuit

        12. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U2) -> pyQPanda.VariationalQuantumCircuit

        13. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U3) -> pyQPanda.VariationalQuantumCircuit

        14. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U4) -> pyQPanda.VariationalQuantumCircuit

        15. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_RX) -> pyQPanda.VariationalQuantumCircuit

        16. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_RY) -> pyQPanda.VariationalQuantumCircuit

        17. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_RZ) -> pyQPanda.VariationalQuantumCircuit

        18. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CNOT) -> pyQPanda.VariationalQuantumCircuit

        19. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CR) -> pyQPanda.VariationalQuantumCircuit

        20. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CZ) -> pyQPanda.VariationalQuantumCircuit

        21. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CRX) -> pyQPanda.VariationalQuantumCircuit

        22. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CRY) -> pyQPanda.VariationalQuantumCircuit

        23. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CRZ) -> pyQPanda.VariationalQuantumCircuit

        24. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_SWAP) -> pyQPanda.VariationalQuantumCircuit

        25. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_iSWAP) -> pyQPanda.VariationalQuantumCircuit

        26. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_SqiSWAP) -> pyQPanda.VariationalQuantumCircuit

        27. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumCircuit) -> pyQPanda.VariationalQuantumCircuit

        28. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.QCircuit) -> pyQPanda.VariationalQuantumCircuit

        29. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.QGate) -> pyQPanda.VariationalQuantumCircuit
        """
    @overload
    def __lshift__(self, arg0: VariationalQuantumGate_H) -> VariationalQuantumCircuit:
        """__lshift__(*args, **kwargs)
        Overloaded function.

        1. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_I) -> pyQPanda.VariationalQuantumCircuit

        2. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_H) -> pyQPanda.VariationalQuantumCircuit

        3. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_X) -> pyQPanda.VariationalQuantumCircuit

        4. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Y) -> pyQPanda.VariationalQuantumCircuit

        5. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_T) -> pyQPanda.VariationalQuantumCircuit

        6. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_S) -> pyQPanda.VariationalQuantumCircuit

        7. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Z) -> pyQPanda.VariationalQuantumCircuit

        8. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_X1) -> pyQPanda.VariationalQuantumCircuit

        9. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Y1) -> pyQPanda.VariationalQuantumCircuit

        10. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Z1) -> pyQPanda.VariationalQuantumCircuit

        11. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U1) -> pyQPanda.VariationalQuantumCircuit

        12. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U2) -> pyQPanda.VariationalQuantumCircuit

        13. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U3) -> pyQPanda.VariationalQuantumCircuit

        14. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U4) -> pyQPanda.VariationalQuantumCircuit

        15. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_RX) -> pyQPanda.VariationalQuantumCircuit

        16. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_RY) -> pyQPanda.VariationalQuantumCircuit

        17. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_RZ) -> pyQPanda.VariationalQuantumCircuit

        18. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CNOT) -> pyQPanda.VariationalQuantumCircuit

        19. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CR) -> pyQPanda.VariationalQuantumCircuit

        20. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CZ) -> pyQPanda.VariationalQuantumCircuit

        21. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CRX) -> pyQPanda.VariationalQuantumCircuit

        22. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CRY) -> pyQPanda.VariationalQuantumCircuit

        23. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CRZ) -> pyQPanda.VariationalQuantumCircuit

        24. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_SWAP) -> pyQPanda.VariationalQuantumCircuit

        25. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_iSWAP) -> pyQPanda.VariationalQuantumCircuit

        26. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_SqiSWAP) -> pyQPanda.VariationalQuantumCircuit

        27. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumCircuit) -> pyQPanda.VariationalQuantumCircuit

        28. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.QCircuit) -> pyQPanda.VariationalQuantumCircuit

        29. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.QGate) -> pyQPanda.VariationalQuantumCircuit
        """
    @overload
    def __lshift__(self, arg0: VariationalQuantumGate_X) -> VariationalQuantumCircuit:
        """__lshift__(*args, **kwargs)
        Overloaded function.

        1. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_I) -> pyQPanda.VariationalQuantumCircuit

        2. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_H) -> pyQPanda.VariationalQuantumCircuit

        3. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_X) -> pyQPanda.VariationalQuantumCircuit

        4. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Y) -> pyQPanda.VariationalQuantumCircuit

        5. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_T) -> pyQPanda.VariationalQuantumCircuit

        6. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_S) -> pyQPanda.VariationalQuantumCircuit

        7. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Z) -> pyQPanda.VariationalQuantumCircuit

        8. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_X1) -> pyQPanda.VariationalQuantumCircuit

        9. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Y1) -> pyQPanda.VariationalQuantumCircuit

        10. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Z1) -> pyQPanda.VariationalQuantumCircuit

        11. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U1) -> pyQPanda.VariationalQuantumCircuit

        12. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U2) -> pyQPanda.VariationalQuantumCircuit

        13. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U3) -> pyQPanda.VariationalQuantumCircuit

        14. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U4) -> pyQPanda.VariationalQuantumCircuit

        15. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_RX) -> pyQPanda.VariationalQuantumCircuit

        16. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_RY) -> pyQPanda.VariationalQuantumCircuit

        17. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_RZ) -> pyQPanda.VariationalQuantumCircuit

        18. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CNOT) -> pyQPanda.VariationalQuantumCircuit

        19. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CR) -> pyQPanda.VariationalQuantumCircuit

        20. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CZ) -> pyQPanda.VariationalQuantumCircuit

        21. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CRX) -> pyQPanda.VariationalQuantumCircuit

        22. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CRY) -> pyQPanda.VariationalQuantumCircuit

        23. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CRZ) -> pyQPanda.VariationalQuantumCircuit

        24. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_SWAP) -> pyQPanda.VariationalQuantumCircuit

        25. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_iSWAP) -> pyQPanda.VariationalQuantumCircuit

        26. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_SqiSWAP) -> pyQPanda.VariationalQuantumCircuit

        27. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumCircuit) -> pyQPanda.VariationalQuantumCircuit

        28. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.QCircuit) -> pyQPanda.VariationalQuantumCircuit

        29. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.QGate) -> pyQPanda.VariationalQuantumCircuit
        """
    @overload
    def __lshift__(self, arg0: VariationalQuantumGate_Y) -> VariationalQuantumCircuit:
        """__lshift__(*args, **kwargs)
        Overloaded function.

        1. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_I) -> pyQPanda.VariationalQuantumCircuit

        2. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_H) -> pyQPanda.VariationalQuantumCircuit

        3. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_X) -> pyQPanda.VariationalQuantumCircuit

        4. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Y) -> pyQPanda.VariationalQuantumCircuit

        5. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_T) -> pyQPanda.VariationalQuantumCircuit

        6. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_S) -> pyQPanda.VariationalQuantumCircuit

        7. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Z) -> pyQPanda.VariationalQuantumCircuit

        8. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_X1) -> pyQPanda.VariationalQuantumCircuit

        9. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Y1) -> pyQPanda.VariationalQuantumCircuit

        10. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Z1) -> pyQPanda.VariationalQuantumCircuit

        11. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U1) -> pyQPanda.VariationalQuantumCircuit

        12. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U2) -> pyQPanda.VariationalQuantumCircuit

        13. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U3) -> pyQPanda.VariationalQuantumCircuit

        14. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U4) -> pyQPanda.VariationalQuantumCircuit

        15. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_RX) -> pyQPanda.VariationalQuantumCircuit

        16. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_RY) -> pyQPanda.VariationalQuantumCircuit

        17. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_RZ) -> pyQPanda.VariationalQuantumCircuit

        18. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CNOT) -> pyQPanda.VariationalQuantumCircuit

        19. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CR) -> pyQPanda.VariationalQuantumCircuit

        20. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CZ) -> pyQPanda.VariationalQuantumCircuit

        21. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CRX) -> pyQPanda.VariationalQuantumCircuit

        22. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CRY) -> pyQPanda.VariationalQuantumCircuit

        23. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CRZ) -> pyQPanda.VariationalQuantumCircuit

        24. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_SWAP) -> pyQPanda.VariationalQuantumCircuit

        25. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_iSWAP) -> pyQPanda.VariationalQuantumCircuit

        26. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_SqiSWAP) -> pyQPanda.VariationalQuantumCircuit

        27. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumCircuit) -> pyQPanda.VariationalQuantumCircuit

        28. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.QCircuit) -> pyQPanda.VariationalQuantumCircuit

        29. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.QGate) -> pyQPanda.VariationalQuantumCircuit
        """
    @overload
    def __lshift__(self, arg0: VariationalQuantumGate_T) -> VariationalQuantumCircuit:
        """__lshift__(*args, **kwargs)
        Overloaded function.

        1. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_I) -> pyQPanda.VariationalQuantumCircuit

        2. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_H) -> pyQPanda.VariationalQuantumCircuit

        3. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_X) -> pyQPanda.VariationalQuantumCircuit

        4. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Y) -> pyQPanda.VariationalQuantumCircuit

        5. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_T) -> pyQPanda.VariationalQuantumCircuit

        6. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_S) -> pyQPanda.VariationalQuantumCircuit

        7. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Z) -> pyQPanda.VariationalQuantumCircuit

        8. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_X1) -> pyQPanda.VariationalQuantumCircuit

        9. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Y1) -> pyQPanda.VariationalQuantumCircuit

        10. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Z1) -> pyQPanda.VariationalQuantumCircuit

        11. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U1) -> pyQPanda.VariationalQuantumCircuit

        12. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U2) -> pyQPanda.VariationalQuantumCircuit

        13. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U3) -> pyQPanda.VariationalQuantumCircuit

        14. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U4) -> pyQPanda.VariationalQuantumCircuit

        15. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_RX) -> pyQPanda.VariationalQuantumCircuit

        16. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_RY) -> pyQPanda.VariationalQuantumCircuit

        17. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_RZ) -> pyQPanda.VariationalQuantumCircuit

        18. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CNOT) -> pyQPanda.VariationalQuantumCircuit

        19. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CR) -> pyQPanda.VariationalQuantumCircuit

        20. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CZ) -> pyQPanda.VariationalQuantumCircuit

        21. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CRX) -> pyQPanda.VariationalQuantumCircuit

        22. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CRY) -> pyQPanda.VariationalQuantumCircuit

        23. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CRZ) -> pyQPanda.VariationalQuantumCircuit

        24. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_SWAP) -> pyQPanda.VariationalQuantumCircuit

        25. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_iSWAP) -> pyQPanda.VariationalQuantumCircuit

        26. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_SqiSWAP) -> pyQPanda.VariationalQuantumCircuit

        27. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumCircuit) -> pyQPanda.VariationalQuantumCircuit

        28. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.QCircuit) -> pyQPanda.VariationalQuantumCircuit

        29. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.QGate) -> pyQPanda.VariationalQuantumCircuit
        """
    @overload
    def __lshift__(self, arg0: VariationalQuantumGate_S) -> VariationalQuantumCircuit:
        """__lshift__(*args, **kwargs)
        Overloaded function.

        1. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_I) -> pyQPanda.VariationalQuantumCircuit

        2. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_H) -> pyQPanda.VariationalQuantumCircuit

        3. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_X) -> pyQPanda.VariationalQuantumCircuit

        4. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Y) -> pyQPanda.VariationalQuantumCircuit

        5. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_T) -> pyQPanda.VariationalQuantumCircuit

        6. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_S) -> pyQPanda.VariationalQuantumCircuit

        7. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Z) -> pyQPanda.VariationalQuantumCircuit

        8. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_X1) -> pyQPanda.VariationalQuantumCircuit

        9. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Y1) -> pyQPanda.VariationalQuantumCircuit

        10. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Z1) -> pyQPanda.VariationalQuantumCircuit

        11. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U1) -> pyQPanda.VariationalQuantumCircuit

        12. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U2) -> pyQPanda.VariationalQuantumCircuit

        13. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U3) -> pyQPanda.VariationalQuantumCircuit

        14. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U4) -> pyQPanda.VariationalQuantumCircuit

        15. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_RX) -> pyQPanda.VariationalQuantumCircuit

        16. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_RY) -> pyQPanda.VariationalQuantumCircuit

        17. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_RZ) -> pyQPanda.VariationalQuantumCircuit

        18. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CNOT) -> pyQPanda.VariationalQuantumCircuit

        19. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CR) -> pyQPanda.VariationalQuantumCircuit

        20. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CZ) -> pyQPanda.VariationalQuantumCircuit

        21. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CRX) -> pyQPanda.VariationalQuantumCircuit

        22. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CRY) -> pyQPanda.VariationalQuantumCircuit

        23. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CRZ) -> pyQPanda.VariationalQuantumCircuit

        24. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_SWAP) -> pyQPanda.VariationalQuantumCircuit

        25. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_iSWAP) -> pyQPanda.VariationalQuantumCircuit

        26. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_SqiSWAP) -> pyQPanda.VariationalQuantumCircuit

        27. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumCircuit) -> pyQPanda.VariationalQuantumCircuit

        28. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.QCircuit) -> pyQPanda.VariationalQuantumCircuit

        29. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.QGate) -> pyQPanda.VariationalQuantumCircuit
        """
    @overload
    def __lshift__(self, arg0: VariationalQuantumGate_Z) -> VariationalQuantumCircuit:
        """__lshift__(*args, **kwargs)
        Overloaded function.

        1. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_I) -> pyQPanda.VariationalQuantumCircuit

        2. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_H) -> pyQPanda.VariationalQuantumCircuit

        3. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_X) -> pyQPanda.VariationalQuantumCircuit

        4. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Y) -> pyQPanda.VariationalQuantumCircuit

        5. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_T) -> pyQPanda.VariationalQuantumCircuit

        6. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_S) -> pyQPanda.VariationalQuantumCircuit

        7. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Z) -> pyQPanda.VariationalQuantumCircuit

        8. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_X1) -> pyQPanda.VariationalQuantumCircuit

        9. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Y1) -> pyQPanda.VariationalQuantumCircuit

        10. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Z1) -> pyQPanda.VariationalQuantumCircuit

        11. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U1) -> pyQPanda.VariationalQuantumCircuit

        12. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U2) -> pyQPanda.VariationalQuantumCircuit

        13. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U3) -> pyQPanda.VariationalQuantumCircuit

        14. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U4) -> pyQPanda.VariationalQuantumCircuit

        15. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_RX) -> pyQPanda.VariationalQuantumCircuit

        16. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_RY) -> pyQPanda.VariationalQuantumCircuit

        17. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_RZ) -> pyQPanda.VariationalQuantumCircuit

        18. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CNOT) -> pyQPanda.VariationalQuantumCircuit

        19. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CR) -> pyQPanda.VariationalQuantumCircuit

        20. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CZ) -> pyQPanda.VariationalQuantumCircuit

        21. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CRX) -> pyQPanda.VariationalQuantumCircuit

        22. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CRY) -> pyQPanda.VariationalQuantumCircuit

        23. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CRZ) -> pyQPanda.VariationalQuantumCircuit

        24. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_SWAP) -> pyQPanda.VariationalQuantumCircuit

        25. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_iSWAP) -> pyQPanda.VariationalQuantumCircuit

        26. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_SqiSWAP) -> pyQPanda.VariationalQuantumCircuit

        27. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumCircuit) -> pyQPanda.VariationalQuantumCircuit

        28. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.QCircuit) -> pyQPanda.VariationalQuantumCircuit

        29. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.QGate) -> pyQPanda.VariationalQuantumCircuit
        """
    @overload
    def __lshift__(self, arg0: VariationalQuantumGate_X1) -> VariationalQuantumCircuit:
        """__lshift__(*args, **kwargs)
        Overloaded function.

        1. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_I) -> pyQPanda.VariationalQuantumCircuit

        2. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_H) -> pyQPanda.VariationalQuantumCircuit

        3. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_X) -> pyQPanda.VariationalQuantumCircuit

        4. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Y) -> pyQPanda.VariationalQuantumCircuit

        5. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_T) -> pyQPanda.VariationalQuantumCircuit

        6. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_S) -> pyQPanda.VariationalQuantumCircuit

        7. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Z) -> pyQPanda.VariationalQuantumCircuit

        8. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_X1) -> pyQPanda.VariationalQuantumCircuit

        9. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Y1) -> pyQPanda.VariationalQuantumCircuit

        10. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Z1) -> pyQPanda.VariationalQuantumCircuit

        11. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U1) -> pyQPanda.VariationalQuantumCircuit

        12. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U2) -> pyQPanda.VariationalQuantumCircuit

        13. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U3) -> pyQPanda.VariationalQuantumCircuit

        14. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U4) -> pyQPanda.VariationalQuantumCircuit

        15. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_RX) -> pyQPanda.VariationalQuantumCircuit

        16. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_RY) -> pyQPanda.VariationalQuantumCircuit

        17. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_RZ) -> pyQPanda.VariationalQuantumCircuit

        18. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CNOT) -> pyQPanda.VariationalQuantumCircuit

        19. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CR) -> pyQPanda.VariationalQuantumCircuit

        20. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CZ) -> pyQPanda.VariationalQuantumCircuit

        21. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CRX) -> pyQPanda.VariationalQuantumCircuit

        22. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CRY) -> pyQPanda.VariationalQuantumCircuit

        23. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CRZ) -> pyQPanda.VariationalQuantumCircuit

        24. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_SWAP) -> pyQPanda.VariationalQuantumCircuit

        25. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_iSWAP) -> pyQPanda.VariationalQuantumCircuit

        26. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_SqiSWAP) -> pyQPanda.VariationalQuantumCircuit

        27. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumCircuit) -> pyQPanda.VariationalQuantumCircuit

        28. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.QCircuit) -> pyQPanda.VariationalQuantumCircuit

        29. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.QGate) -> pyQPanda.VariationalQuantumCircuit
        """
    @overload
    def __lshift__(self, arg0: VariationalQuantumGate_Y1) -> VariationalQuantumCircuit:
        """__lshift__(*args, **kwargs)
        Overloaded function.

        1. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_I) -> pyQPanda.VariationalQuantumCircuit

        2. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_H) -> pyQPanda.VariationalQuantumCircuit

        3. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_X) -> pyQPanda.VariationalQuantumCircuit

        4. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Y) -> pyQPanda.VariationalQuantumCircuit

        5. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_T) -> pyQPanda.VariationalQuantumCircuit

        6. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_S) -> pyQPanda.VariationalQuantumCircuit

        7. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Z) -> pyQPanda.VariationalQuantumCircuit

        8. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_X1) -> pyQPanda.VariationalQuantumCircuit

        9. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Y1) -> pyQPanda.VariationalQuantumCircuit

        10. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Z1) -> pyQPanda.VariationalQuantumCircuit

        11. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U1) -> pyQPanda.VariationalQuantumCircuit

        12. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U2) -> pyQPanda.VariationalQuantumCircuit

        13. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U3) -> pyQPanda.VariationalQuantumCircuit

        14. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U4) -> pyQPanda.VariationalQuantumCircuit

        15. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_RX) -> pyQPanda.VariationalQuantumCircuit

        16. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_RY) -> pyQPanda.VariationalQuantumCircuit

        17. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_RZ) -> pyQPanda.VariationalQuantumCircuit

        18. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CNOT) -> pyQPanda.VariationalQuantumCircuit

        19. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CR) -> pyQPanda.VariationalQuantumCircuit

        20. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CZ) -> pyQPanda.VariationalQuantumCircuit

        21. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CRX) -> pyQPanda.VariationalQuantumCircuit

        22. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CRY) -> pyQPanda.VariationalQuantumCircuit

        23. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CRZ) -> pyQPanda.VariationalQuantumCircuit

        24. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_SWAP) -> pyQPanda.VariationalQuantumCircuit

        25. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_iSWAP) -> pyQPanda.VariationalQuantumCircuit

        26. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_SqiSWAP) -> pyQPanda.VariationalQuantumCircuit

        27. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumCircuit) -> pyQPanda.VariationalQuantumCircuit

        28. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.QCircuit) -> pyQPanda.VariationalQuantumCircuit

        29. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.QGate) -> pyQPanda.VariationalQuantumCircuit
        """
    @overload
    def __lshift__(self, arg0: VariationalQuantumGate_Z1) -> VariationalQuantumCircuit:
        """__lshift__(*args, **kwargs)
        Overloaded function.

        1. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_I) -> pyQPanda.VariationalQuantumCircuit

        2. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_H) -> pyQPanda.VariationalQuantumCircuit

        3. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_X) -> pyQPanda.VariationalQuantumCircuit

        4. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Y) -> pyQPanda.VariationalQuantumCircuit

        5. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_T) -> pyQPanda.VariationalQuantumCircuit

        6. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_S) -> pyQPanda.VariationalQuantumCircuit

        7. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Z) -> pyQPanda.VariationalQuantumCircuit

        8. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_X1) -> pyQPanda.VariationalQuantumCircuit

        9. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Y1) -> pyQPanda.VariationalQuantumCircuit

        10. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Z1) -> pyQPanda.VariationalQuantumCircuit

        11. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U1) -> pyQPanda.VariationalQuantumCircuit

        12. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U2) -> pyQPanda.VariationalQuantumCircuit

        13. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U3) -> pyQPanda.VariationalQuantumCircuit

        14. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U4) -> pyQPanda.VariationalQuantumCircuit

        15. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_RX) -> pyQPanda.VariationalQuantumCircuit

        16. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_RY) -> pyQPanda.VariationalQuantumCircuit

        17. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_RZ) -> pyQPanda.VariationalQuantumCircuit

        18. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CNOT) -> pyQPanda.VariationalQuantumCircuit

        19. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CR) -> pyQPanda.VariationalQuantumCircuit

        20. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CZ) -> pyQPanda.VariationalQuantumCircuit

        21. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CRX) -> pyQPanda.VariationalQuantumCircuit

        22. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CRY) -> pyQPanda.VariationalQuantumCircuit

        23. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CRZ) -> pyQPanda.VariationalQuantumCircuit

        24. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_SWAP) -> pyQPanda.VariationalQuantumCircuit

        25. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_iSWAP) -> pyQPanda.VariationalQuantumCircuit

        26. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_SqiSWAP) -> pyQPanda.VariationalQuantumCircuit

        27. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumCircuit) -> pyQPanda.VariationalQuantumCircuit

        28. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.QCircuit) -> pyQPanda.VariationalQuantumCircuit

        29. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.QGate) -> pyQPanda.VariationalQuantumCircuit
        """
    @overload
    def __lshift__(self, arg0: VariationalQuantumGate_U1) -> VariationalQuantumCircuit:
        """__lshift__(*args, **kwargs)
        Overloaded function.

        1. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_I) -> pyQPanda.VariationalQuantumCircuit

        2. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_H) -> pyQPanda.VariationalQuantumCircuit

        3. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_X) -> pyQPanda.VariationalQuantumCircuit

        4. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Y) -> pyQPanda.VariationalQuantumCircuit

        5. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_T) -> pyQPanda.VariationalQuantumCircuit

        6. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_S) -> pyQPanda.VariationalQuantumCircuit

        7. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Z) -> pyQPanda.VariationalQuantumCircuit

        8. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_X1) -> pyQPanda.VariationalQuantumCircuit

        9. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Y1) -> pyQPanda.VariationalQuantumCircuit

        10. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Z1) -> pyQPanda.VariationalQuantumCircuit

        11. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U1) -> pyQPanda.VariationalQuantumCircuit

        12. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U2) -> pyQPanda.VariationalQuantumCircuit

        13. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U3) -> pyQPanda.VariationalQuantumCircuit

        14. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U4) -> pyQPanda.VariationalQuantumCircuit

        15. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_RX) -> pyQPanda.VariationalQuantumCircuit

        16. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_RY) -> pyQPanda.VariationalQuantumCircuit

        17. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_RZ) -> pyQPanda.VariationalQuantumCircuit

        18. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CNOT) -> pyQPanda.VariationalQuantumCircuit

        19. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CR) -> pyQPanda.VariationalQuantumCircuit

        20. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CZ) -> pyQPanda.VariationalQuantumCircuit

        21. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CRX) -> pyQPanda.VariationalQuantumCircuit

        22. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CRY) -> pyQPanda.VariationalQuantumCircuit

        23. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CRZ) -> pyQPanda.VariationalQuantumCircuit

        24. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_SWAP) -> pyQPanda.VariationalQuantumCircuit

        25. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_iSWAP) -> pyQPanda.VariationalQuantumCircuit

        26. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_SqiSWAP) -> pyQPanda.VariationalQuantumCircuit

        27. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumCircuit) -> pyQPanda.VariationalQuantumCircuit

        28. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.QCircuit) -> pyQPanda.VariationalQuantumCircuit

        29. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.QGate) -> pyQPanda.VariationalQuantumCircuit
        """
    @overload
    def __lshift__(self, arg0: VariationalQuantumGate_U2) -> VariationalQuantumCircuit:
        """__lshift__(*args, **kwargs)
        Overloaded function.

        1. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_I) -> pyQPanda.VariationalQuantumCircuit

        2. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_H) -> pyQPanda.VariationalQuantumCircuit

        3. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_X) -> pyQPanda.VariationalQuantumCircuit

        4. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Y) -> pyQPanda.VariationalQuantumCircuit

        5. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_T) -> pyQPanda.VariationalQuantumCircuit

        6. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_S) -> pyQPanda.VariationalQuantumCircuit

        7. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Z) -> pyQPanda.VariationalQuantumCircuit

        8. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_X1) -> pyQPanda.VariationalQuantumCircuit

        9. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Y1) -> pyQPanda.VariationalQuantumCircuit

        10. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Z1) -> pyQPanda.VariationalQuantumCircuit

        11. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U1) -> pyQPanda.VariationalQuantumCircuit

        12. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U2) -> pyQPanda.VariationalQuantumCircuit

        13. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U3) -> pyQPanda.VariationalQuantumCircuit

        14. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U4) -> pyQPanda.VariationalQuantumCircuit

        15. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_RX) -> pyQPanda.VariationalQuantumCircuit

        16. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_RY) -> pyQPanda.VariationalQuantumCircuit

        17. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_RZ) -> pyQPanda.VariationalQuantumCircuit

        18. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CNOT) -> pyQPanda.VariationalQuantumCircuit

        19. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CR) -> pyQPanda.VariationalQuantumCircuit

        20. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CZ) -> pyQPanda.VariationalQuantumCircuit

        21. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CRX) -> pyQPanda.VariationalQuantumCircuit

        22. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CRY) -> pyQPanda.VariationalQuantumCircuit

        23. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CRZ) -> pyQPanda.VariationalQuantumCircuit

        24. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_SWAP) -> pyQPanda.VariationalQuantumCircuit

        25. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_iSWAP) -> pyQPanda.VariationalQuantumCircuit

        26. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_SqiSWAP) -> pyQPanda.VariationalQuantumCircuit

        27. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumCircuit) -> pyQPanda.VariationalQuantumCircuit

        28. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.QCircuit) -> pyQPanda.VariationalQuantumCircuit

        29. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.QGate) -> pyQPanda.VariationalQuantumCircuit
        """
    @overload
    def __lshift__(self, arg0: VariationalQuantumGate_U3) -> VariationalQuantumCircuit:
        """__lshift__(*args, **kwargs)
        Overloaded function.

        1. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_I) -> pyQPanda.VariationalQuantumCircuit

        2. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_H) -> pyQPanda.VariationalQuantumCircuit

        3. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_X) -> pyQPanda.VariationalQuantumCircuit

        4. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Y) -> pyQPanda.VariationalQuantumCircuit

        5. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_T) -> pyQPanda.VariationalQuantumCircuit

        6. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_S) -> pyQPanda.VariationalQuantumCircuit

        7. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Z) -> pyQPanda.VariationalQuantumCircuit

        8. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_X1) -> pyQPanda.VariationalQuantumCircuit

        9. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Y1) -> pyQPanda.VariationalQuantumCircuit

        10. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Z1) -> pyQPanda.VariationalQuantumCircuit

        11. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U1) -> pyQPanda.VariationalQuantumCircuit

        12. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U2) -> pyQPanda.VariationalQuantumCircuit

        13. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U3) -> pyQPanda.VariationalQuantumCircuit

        14. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U4) -> pyQPanda.VariationalQuantumCircuit

        15. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_RX) -> pyQPanda.VariationalQuantumCircuit

        16. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_RY) -> pyQPanda.VariationalQuantumCircuit

        17. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_RZ) -> pyQPanda.VariationalQuantumCircuit

        18. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CNOT) -> pyQPanda.VariationalQuantumCircuit

        19. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CR) -> pyQPanda.VariationalQuantumCircuit

        20. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CZ) -> pyQPanda.VariationalQuantumCircuit

        21. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CRX) -> pyQPanda.VariationalQuantumCircuit

        22. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CRY) -> pyQPanda.VariationalQuantumCircuit

        23. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CRZ) -> pyQPanda.VariationalQuantumCircuit

        24. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_SWAP) -> pyQPanda.VariationalQuantumCircuit

        25. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_iSWAP) -> pyQPanda.VariationalQuantumCircuit

        26. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_SqiSWAP) -> pyQPanda.VariationalQuantumCircuit

        27. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumCircuit) -> pyQPanda.VariationalQuantumCircuit

        28. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.QCircuit) -> pyQPanda.VariationalQuantumCircuit

        29. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.QGate) -> pyQPanda.VariationalQuantumCircuit
        """
    @overload
    def __lshift__(self, arg0: VariationalQuantumGate_U4) -> VariationalQuantumCircuit:
        """__lshift__(*args, **kwargs)
        Overloaded function.

        1. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_I) -> pyQPanda.VariationalQuantumCircuit

        2. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_H) -> pyQPanda.VariationalQuantumCircuit

        3. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_X) -> pyQPanda.VariationalQuantumCircuit

        4. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Y) -> pyQPanda.VariationalQuantumCircuit

        5. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_T) -> pyQPanda.VariationalQuantumCircuit

        6. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_S) -> pyQPanda.VariationalQuantumCircuit

        7. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Z) -> pyQPanda.VariationalQuantumCircuit

        8. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_X1) -> pyQPanda.VariationalQuantumCircuit

        9. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Y1) -> pyQPanda.VariationalQuantumCircuit

        10. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Z1) -> pyQPanda.VariationalQuantumCircuit

        11. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U1) -> pyQPanda.VariationalQuantumCircuit

        12. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U2) -> pyQPanda.VariationalQuantumCircuit

        13. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U3) -> pyQPanda.VariationalQuantumCircuit

        14. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U4) -> pyQPanda.VariationalQuantumCircuit

        15. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_RX) -> pyQPanda.VariationalQuantumCircuit

        16. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_RY) -> pyQPanda.VariationalQuantumCircuit

        17. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_RZ) -> pyQPanda.VariationalQuantumCircuit

        18. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CNOT) -> pyQPanda.VariationalQuantumCircuit

        19. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CR) -> pyQPanda.VariationalQuantumCircuit

        20. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CZ) -> pyQPanda.VariationalQuantumCircuit

        21. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CRX) -> pyQPanda.VariationalQuantumCircuit

        22. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CRY) -> pyQPanda.VariationalQuantumCircuit

        23. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CRZ) -> pyQPanda.VariationalQuantumCircuit

        24. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_SWAP) -> pyQPanda.VariationalQuantumCircuit

        25. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_iSWAP) -> pyQPanda.VariationalQuantumCircuit

        26. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_SqiSWAP) -> pyQPanda.VariationalQuantumCircuit

        27. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumCircuit) -> pyQPanda.VariationalQuantumCircuit

        28. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.QCircuit) -> pyQPanda.VariationalQuantumCircuit

        29. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.QGate) -> pyQPanda.VariationalQuantumCircuit
        """
    @overload
    def __lshift__(self, arg0: VariationalQuantumGate_RX) -> VariationalQuantumCircuit:
        """__lshift__(*args, **kwargs)
        Overloaded function.

        1. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_I) -> pyQPanda.VariationalQuantumCircuit

        2. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_H) -> pyQPanda.VariationalQuantumCircuit

        3. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_X) -> pyQPanda.VariationalQuantumCircuit

        4. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Y) -> pyQPanda.VariationalQuantumCircuit

        5. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_T) -> pyQPanda.VariationalQuantumCircuit

        6. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_S) -> pyQPanda.VariationalQuantumCircuit

        7. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Z) -> pyQPanda.VariationalQuantumCircuit

        8. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_X1) -> pyQPanda.VariationalQuantumCircuit

        9. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Y1) -> pyQPanda.VariationalQuantumCircuit

        10. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Z1) -> pyQPanda.VariationalQuantumCircuit

        11. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U1) -> pyQPanda.VariationalQuantumCircuit

        12. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U2) -> pyQPanda.VariationalQuantumCircuit

        13. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U3) -> pyQPanda.VariationalQuantumCircuit

        14. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U4) -> pyQPanda.VariationalQuantumCircuit

        15. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_RX) -> pyQPanda.VariationalQuantumCircuit

        16. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_RY) -> pyQPanda.VariationalQuantumCircuit

        17. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_RZ) -> pyQPanda.VariationalQuantumCircuit

        18. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CNOT) -> pyQPanda.VariationalQuantumCircuit

        19. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CR) -> pyQPanda.VariationalQuantumCircuit

        20. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CZ) -> pyQPanda.VariationalQuantumCircuit

        21. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CRX) -> pyQPanda.VariationalQuantumCircuit

        22. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CRY) -> pyQPanda.VariationalQuantumCircuit

        23. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CRZ) -> pyQPanda.VariationalQuantumCircuit

        24. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_SWAP) -> pyQPanda.VariationalQuantumCircuit

        25. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_iSWAP) -> pyQPanda.VariationalQuantumCircuit

        26. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_SqiSWAP) -> pyQPanda.VariationalQuantumCircuit

        27. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumCircuit) -> pyQPanda.VariationalQuantumCircuit

        28. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.QCircuit) -> pyQPanda.VariationalQuantumCircuit

        29. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.QGate) -> pyQPanda.VariationalQuantumCircuit
        """
    @overload
    def __lshift__(self, arg0: VariationalQuantumGate_RY) -> VariationalQuantumCircuit:
        """__lshift__(*args, **kwargs)
        Overloaded function.

        1. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_I) -> pyQPanda.VariationalQuantumCircuit

        2. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_H) -> pyQPanda.VariationalQuantumCircuit

        3. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_X) -> pyQPanda.VariationalQuantumCircuit

        4. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Y) -> pyQPanda.VariationalQuantumCircuit

        5. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_T) -> pyQPanda.VariationalQuantumCircuit

        6. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_S) -> pyQPanda.VariationalQuantumCircuit

        7. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Z) -> pyQPanda.VariationalQuantumCircuit

        8. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_X1) -> pyQPanda.VariationalQuantumCircuit

        9. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Y1) -> pyQPanda.VariationalQuantumCircuit

        10. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Z1) -> pyQPanda.VariationalQuantumCircuit

        11. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U1) -> pyQPanda.VariationalQuantumCircuit

        12. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U2) -> pyQPanda.VariationalQuantumCircuit

        13. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U3) -> pyQPanda.VariationalQuantumCircuit

        14. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U4) -> pyQPanda.VariationalQuantumCircuit

        15. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_RX) -> pyQPanda.VariationalQuantumCircuit

        16. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_RY) -> pyQPanda.VariationalQuantumCircuit

        17. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_RZ) -> pyQPanda.VariationalQuantumCircuit

        18. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CNOT) -> pyQPanda.VariationalQuantumCircuit

        19. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CR) -> pyQPanda.VariationalQuantumCircuit

        20. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CZ) -> pyQPanda.VariationalQuantumCircuit

        21. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CRX) -> pyQPanda.VariationalQuantumCircuit

        22. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CRY) -> pyQPanda.VariationalQuantumCircuit

        23. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CRZ) -> pyQPanda.VariationalQuantumCircuit

        24. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_SWAP) -> pyQPanda.VariationalQuantumCircuit

        25. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_iSWAP) -> pyQPanda.VariationalQuantumCircuit

        26. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_SqiSWAP) -> pyQPanda.VariationalQuantumCircuit

        27. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumCircuit) -> pyQPanda.VariationalQuantumCircuit

        28. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.QCircuit) -> pyQPanda.VariationalQuantumCircuit

        29. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.QGate) -> pyQPanda.VariationalQuantumCircuit
        """
    @overload
    def __lshift__(self, arg0: VariationalQuantumGate_RZ) -> VariationalQuantumCircuit:
        """__lshift__(*args, **kwargs)
        Overloaded function.

        1. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_I) -> pyQPanda.VariationalQuantumCircuit

        2. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_H) -> pyQPanda.VariationalQuantumCircuit

        3. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_X) -> pyQPanda.VariationalQuantumCircuit

        4. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Y) -> pyQPanda.VariationalQuantumCircuit

        5. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_T) -> pyQPanda.VariationalQuantumCircuit

        6. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_S) -> pyQPanda.VariationalQuantumCircuit

        7. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Z) -> pyQPanda.VariationalQuantumCircuit

        8. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_X1) -> pyQPanda.VariationalQuantumCircuit

        9. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Y1) -> pyQPanda.VariationalQuantumCircuit

        10. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Z1) -> pyQPanda.VariationalQuantumCircuit

        11. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U1) -> pyQPanda.VariationalQuantumCircuit

        12. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U2) -> pyQPanda.VariationalQuantumCircuit

        13. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U3) -> pyQPanda.VariationalQuantumCircuit

        14. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U4) -> pyQPanda.VariationalQuantumCircuit

        15. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_RX) -> pyQPanda.VariationalQuantumCircuit

        16. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_RY) -> pyQPanda.VariationalQuantumCircuit

        17. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_RZ) -> pyQPanda.VariationalQuantumCircuit

        18. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CNOT) -> pyQPanda.VariationalQuantumCircuit

        19. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CR) -> pyQPanda.VariationalQuantumCircuit

        20. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CZ) -> pyQPanda.VariationalQuantumCircuit

        21. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CRX) -> pyQPanda.VariationalQuantumCircuit

        22. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CRY) -> pyQPanda.VariationalQuantumCircuit

        23. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CRZ) -> pyQPanda.VariationalQuantumCircuit

        24. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_SWAP) -> pyQPanda.VariationalQuantumCircuit

        25. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_iSWAP) -> pyQPanda.VariationalQuantumCircuit

        26. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_SqiSWAP) -> pyQPanda.VariationalQuantumCircuit

        27. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumCircuit) -> pyQPanda.VariationalQuantumCircuit

        28. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.QCircuit) -> pyQPanda.VariationalQuantumCircuit

        29. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.QGate) -> pyQPanda.VariationalQuantumCircuit
        """
    @overload
    def __lshift__(self, arg0: VariationalQuantumGate_CNOT) -> VariationalQuantumCircuit:
        """__lshift__(*args, **kwargs)
        Overloaded function.

        1. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_I) -> pyQPanda.VariationalQuantumCircuit

        2. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_H) -> pyQPanda.VariationalQuantumCircuit

        3. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_X) -> pyQPanda.VariationalQuantumCircuit

        4. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Y) -> pyQPanda.VariationalQuantumCircuit

        5. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_T) -> pyQPanda.VariationalQuantumCircuit

        6. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_S) -> pyQPanda.VariationalQuantumCircuit

        7. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Z) -> pyQPanda.VariationalQuantumCircuit

        8. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_X1) -> pyQPanda.VariationalQuantumCircuit

        9. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Y1) -> pyQPanda.VariationalQuantumCircuit

        10. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Z1) -> pyQPanda.VariationalQuantumCircuit

        11. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U1) -> pyQPanda.VariationalQuantumCircuit

        12. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U2) -> pyQPanda.VariationalQuantumCircuit

        13. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U3) -> pyQPanda.VariationalQuantumCircuit

        14. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U4) -> pyQPanda.VariationalQuantumCircuit

        15. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_RX) -> pyQPanda.VariationalQuantumCircuit

        16. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_RY) -> pyQPanda.VariationalQuantumCircuit

        17. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_RZ) -> pyQPanda.VariationalQuantumCircuit

        18. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CNOT) -> pyQPanda.VariationalQuantumCircuit

        19. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CR) -> pyQPanda.VariationalQuantumCircuit

        20. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CZ) -> pyQPanda.VariationalQuantumCircuit

        21. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CRX) -> pyQPanda.VariationalQuantumCircuit

        22. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CRY) -> pyQPanda.VariationalQuantumCircuit

        23. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CRZ) -> pyQPanda.VariationalQuantumCircuit

        24. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_SWAP) -> pyQPanda.VariationalQuantumCircuit

        25. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_iSWAP) -> pyQPanda.VariationalQuantumCircuit

        26. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_SqiSWAP) -> pyQPanda.VariationalQuantumCircuit

        27. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumCircuit) -> pyQPanda.VariationalQuantumCircuit

        28. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.QCircuit) -> pyQPanda.VariationalQuantumCircuit

        29. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.QGate) -> pyQPanda.VariationalQuantumCircuit
        """
    @overload
    def __lshift__(self, arg0: VariationalQuantumGate_CR) -> VariationalQuantumCircuit:
        """__lshift__(*args, **kwargs)
        Overloaded function.

        1. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_I) -> pyQPanda.VariationalQuantumCircuit

        2. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_H) -> pyQPanda.VariationalQuantumCircuit

        3. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_X) -> pyQPanda.VariationalQuantumCircuit

        4. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Y) -> pyQPanda.VariationalQuantumCircuit

        5. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_T) -> pyQPanda.VariationalQuantumCircuit

        6. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_S) -> pyQPanda.VariationalQuantumCircuit

        7. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Z) -> pyQPanda.VariationalQuantumCircuit

        8. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_X1) -> pyQPanda.VariationalQuantumCircuit

        9. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Y1) -> pyQPanda.VariationalQuantumCircuit

        10. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Z1) -> pyQPanda.VariationalQuantumCircuit

        11. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U1) -> pyQPanda.VariationalQuantumCircuit

        12. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U2) -> pyQPanda.VariationalQuantumCircuit

        13. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U3) -> pyQPanda.VariationalQuantumCircuit

        14. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U4) -> pyQPanda.VariationalQuantumCircuit

        15. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_RX) -> pyQPanda.VariationalQuantumCircuit

        16. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_RY) -> pyQPanda.VariationalQuantumCircuit

        17. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_RZ) -> pyQPanda.VariationalQuantumCircuit

        18. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CNOT) -> pyQPanda.VariationalQuantumCircuit

        19. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CR) -> pyQPanda.VariationalQuantumCircuit

        20. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CZ) -> pyQPanda.VariationalQuantumCircuit

        21. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CRX) -> pyQPanda.VariationalQuantumCircuit

        22. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CRY) -> pyQPanda.VariationalQuantumCircuit

        23. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CRZ) -> pyQPanda.VariationalQuantumCircuit

        24. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_SWAP) -> pyQPanda.VariationalQuantumCircuit

        25. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_iSWAP) -> pyQPanda.VariationalQuantumCircuit

        26. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_SqiSWAP) -> pyQPanda.VariationalQuantumCircuit

        27. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumCircuit) -> pyQPanda.VariationalQuantumCircuit

        28. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.QCircuit) -> pyQPanda.VariationalQuantumCircuit

        29. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.QGate) -> pyQPanda.VariationalQuantumCircuit
        """
    @overload
    def __lshift__(self, arg0: VariationalQuantumGate_CZ) -> VariationalQuantumCircuit:
        """__lshift__(*args, **kwargs)
        Overloaded function.

        1. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_I) -> pyQPanda.VariationalQuantumCircuit

        2. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_H) -> pyQPanda.VariationalQuantumCircuit

        3. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_X) -> pyQPanda.VariationalQuantumCircuit

        4. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Y) -> pyQPanda.VariationalQuantumCircuit

        5. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_T) -> pyQPanda.VariationalQuantumCircuit

        6. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_S) -> pyQPanda.VariationalQuantumCircuit

        7. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Z) -> pyQPanda.VariationalQuantumCircuit

        8. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_X1) -> pyQPanda.VariationalQuantumCircuit

        9. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Y1) -> pyQPanda.VariationalQuantumCircuit

        10. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Z1) -> pyQPanda.VariationalQuantumCircuit

        11. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U1) -> pyQPanda.VariationalQuantumCircuit

        12. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U2) -> pyQPanda.VariationalQuantumCircuit

        13. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U3) -> pyQPanda.VariationalQuantumCircuit

        14. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U4) -> pyQPanda.VariationalQuantumCircuit

        15. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_RX) -> pyQPanda.VariationalQuantumCircuit

        16. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_RY) -> pyQPanda.VariationalQuantumCircuit

        17. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_RZ) -> pyQPanda.VariationalQuantumCircuit

        18. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CNOT) -> pyQPanda.VariationalQuantumCircuit

        19. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CR) -> pyQPanda.VariationalQuantumCircuit

        20. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CZ) -> pyQPanda.VariationalQuantumCircuit

        21. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CRX) -> pyQPanda.VariationalQuantumCircuit

        22. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CRY) -> pyQPanda.VariationalQuantumCircuit

        23. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CRZ) -> pyQPanda.VariationalQuantumCircuit

        24. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_SWAP) -> pyQPanda.VariationalQuantumCircuit

        25. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_iSWAP) -> pyQPanda.VariationalQuantumCircuit

        26. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_SqiSWAP) -> pyQPanda.VariationalQuantumCircuit

        27. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumCircuit) -> pyQPanda.VariationalQuantumCircuit

        28. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.QCircuit) -> pyQPanda.VariationalQuantumCircuit

        29. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.QGate) -> pyQPanda.VariationalQuantumCircuit
        """
    @overload
    def __lshift__(self, arg0: VariationalQuantumGate_CRX) -> VariationalQuantumCircuit:
        """__lshift__(*args, **kwargs)
        Overloaded function.

        1. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_I) -> pyQPanda.VariationalQuantumCircuit

        2. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_H) -> pyQPanda.VariationalQuantumCircuit

        3. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_X) -> pyQPanda.VariationalQuantumCircuit

        4. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Y) -> pyQPanda.VariationalQuantumCircuit

        5. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_T) -> pyQPanda.VariationalQuantumCircuit

        6. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_S) -> pyQPanda.VariationalQuantumCircuit

        7. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Z) -> pyQPanda.VariationalQuantumCircuit

        8. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_X1) -> pyQPanda.VariationalQuantumCircuit

        9. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Y1) -> pyQPanda.VariationalQuantumCircuit

        10. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Z1) -> pyQPanda.VariationalQuantumCircuit

        11. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U1) -> pyQPanda.VariationalQuantumCircuit

        12. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U2) -> pyQPanda.VariationalQuantumCircuit

        13. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U3) -> pyQPanda.VariationalQuantumCircuit

        14. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U4) -> pyQPanda.VariationalQuantumCircuit

        15. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_RX) -> pyQPanda.VariationalQuantumCircuit

        16. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_RY) -> pyQPanda.VariationalQuantumCircuit

        17. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_RZ) -> pyQPanda.VariationalQuantumCircuit

        18. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CNOT) -> pyQPanda.VariationalQuantumCircuit

        19. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CR) -> pyQPanda.VariationalQuantumCircuit

        20. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CZ) -> pyQPanda.VariationalQuantumCircuit

        21. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CRX) -> pyQPanda.VariationalQuantumCircuit

        22. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CRY) -> pyQPanda.VariationalQuantumCircuit

        23. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CRZ) -> pyQPanda.VariationalQuantumCircuit

        24. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_SWAP) -> pyQPanda.VariationalQuantumCircuit

        25. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_iSWAP) -> pyQPanda.VariationalQuantumCircuit

        26. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_SqiSWAP) -> pyQPanda.VariationalQuantumCircuit

        27. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumCircuit) -> pyQPanda.VariationalQuantumCircuit

        28. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.QCircuit) -> pyQPanda.VariationalQuantumCircuit

        29. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.QGate) -> pyQPanda.VariationalQuantumCircuit
        """
    @overload
    def __lshift__(self, arg0: VariationalQuantumGate_CRY) -> VariationalQuantumCircuit:
        """__lshift__(*args, **kwargs)
        Overloaded function.

        1. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_I) -> pyQPanda.VariationalQuantumCircuit

        2. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_H) -> pyQPanda.VariationalQuantumCircuit

        3. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_X) -> pyQPanda.VariationalQuantumCircuit

        4. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Y) -> pyQPanda.VariationalQuantumCircuit

        5. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_T) -> pyQPanda.VariationalQuantumCircuit

        6. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_S) -> pyQPanda.VariationalQuantumCircuit

        7. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Z) -> pyQPanda.VariationalQuantumCircuit

        8. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_X1) -> pyQPanda.VariationalQuantumCircuit

        9. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Y1) -> pyQPanda.VariationalQuantumCircuit

        10. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Z1) -> pyQPanda.VariationalQuantumCircuit

        11. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U1) -> pyQPanda.VariationalQuantumCircuit

        12. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U2) -> pyQPanda.VariationalQuantumCircuit

        13. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U3) -> pyQPanda.VariationalQuantumCircuit

        14. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U4) -> pyQPanda.VariationalQuantumCircuit

        15. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_RX) -> pyQPanda.VariationalQuantumCircuit

        16. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_RY) -> pyQPanda.VariationalQuantumCircuit

        17. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_RZ) -> pyQPanda.VariationalQuantumCircuit

        18. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CNOT) -> pyQPanda.VariationalQuantumCircuit

        19. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CR) -> pyQPanda.VariationalQuantumCircuit

        20. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CZ) -> pyQPanda.VariationalQuantumCircuit

        21. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CRX) -> pyQPanda.VariationalQuantumCircuit

        22. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CRY) -> pyQPanda.VariationalQuantumCircuit

        23. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CRZ) -> pyQPanda.VariationalQuantumCircuit

        24. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_SWAP) -> pyQPanda.VariationalQuantumCircuit

        25. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_iSWAP) -> pyQPanda.VariationalQuantumCircuit

        26. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_SqiSWAP) -> pyQPanda.VariationalQuantumCircuit

        27. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumCircuit) -> pyQPanda.VariationalQuantumCircuit

        28. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.QCircuit) -> pyQPanda.VariationalQuantumCircuit

        29. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.QGate) -> pyQPanda.VariationalQuantumCircuit
        """
    @overload
    def __lshift__(self, arg0: VariationalQuantumGate_CRZ) -> VariationalQuantumCircuit:
        """__lshift__(*args, **kwargs)
        Overloaded function.

        1. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_I) -> pyQPanda.VariationalQuantumCircuit

        2. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_H) -> pyQPanda.VariationalQuantumCircuit

        3. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_X) -> pyQPanda.VariationalQuantumCircuit

        4. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Y) -> pyQPanda.VariationalQuantumCircuit

        5. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_T) -> pyQPanda.VariationalQuantumCircuit

        6. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_S) -> pyQPanda.VariationalQuantumCircuit

        7. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Z) -> pyQPanda.VariationalQuantumCircuit

        8. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_X1) -> pyQPanda.VariationalQuantumCircuit

        9. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Y1) -> pyQPanda.VariationalQuantumCircuit

        10. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Z1) -> pyQPanda.VariationalQuantumCircuit

        11. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U1) -> pyQPanda.VariationalQuantumCircuit

        12. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U2) -> pyQPanda.VariationalQuantumCircuit

        13. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U3) -> pyQPanda.VariationalQuantumCircuit

        14. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U4) -> pyQPanda.VariationalQuantumCircuit

        15. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_RX) -> pyQPanda.VariationalQuantumCircuit

        16. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_RY) -> pyQPanda.VariationalQuantumCircuit

        17. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_RZ) -> pyQPanda.VariationalQuantumCircuit

        18. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CNOT) -> pyQPanda.VariationalQuantumCircuit

        19. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CR) -> pyQPanda.VariationalQuantumCircuit

        20. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CZ) -> pyQPanda.VariationalQuantumCircuit

        21. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CRX) -> pyQPanda.VariationalQuantumCircuit

        22. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CRY) -> pyQPanda.VariationalQuantumCircuit

        23. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CRZ) -> pyQPanda.VariationalQuantumCircuit

        24. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_SWAP) -> pyQPanda.VariationalQuantumCircuit

        25. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_iSWAP) -> pyQPanda.VariationalQuantumCircuit

        26. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_SqiSWAP) -> pyQPanda.VariationalQuantumCircuit

        27. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumCircuit) -> pyQPanda.VariationalQuantumCircuit

        28. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.QCircuit) -> pyQPanda.VariationalQuantumCircuit

        29. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.QGate) -> pyQPanda.VariationalQuantumCircuit
        """
    @overload
    def __lshift__(self, arg0: VariationalQuantumGate_SWAP) -> VariationalQuantumCircuit:
        """__lshift__(*args, **kwargs)
        Overloaded function.

        1. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_I) -> pyQPanda.VariationalQuantumCircuit

        2. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_H) -> pyQPanda.VariationalQuantumCircuit

        3. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_X) -> pyQPanda.VariationalQuantumCircuit

        4. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Y) -> pyQPanda.VariationalQuantumCircuit

        5. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_T) -> pyQPanda.VariationalQuantumCircuit

        6. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_S) -> pyQPanda.VariationalQuantumCircuit

        7. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Z) -> pyQPanda.VariationalQuantumCircuit

        8. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_X1) -> pyQPanda.VariationalQuantumCircuit

        9. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Y1) -> pyQPanda.VariationalQuantumCircuit

        10. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Z1) -> pyQPanda.VariationalQuantumCircuit

        11. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U1) -> pyQPanda.VariationalQuantumCircuit

        12. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U2) -> pyQPanda.VariationalQuantumCircuit

        13. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U3) -> pyQPanda.VariationalQuantumCircuit

        14. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U4) -> pyQPanda.VariationalQuantumCircuit

        15. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_RX) -> pyQPanda.VariationalQuantumCircuit

        16. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_RY) -> pyQPanda.VariationalQuantumCircuit

        17. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_RZ) -> pyQPanda.VariationalQuantumCircuit

        18. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CNOT) -> pyQPanda.VariationalQuantumCircuit

        19. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CR) -> pyQPanda.VariationalQuantumCircuit

        20. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CZ) -> pyQPanda.VariationalQuantumCircuit

        21. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CRX) -> pyQPanda.VariationalQuantumCircuit

        22. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CRY) -> pyQPanda.VariationalQuantumCircuit

        23. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CRZ) -> pyQPanda.VariationalQuantumCircuit

        24. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_SWAP) -> pyQPanda.VariationalQuantumCircuit

        25. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_iSWAP) -> pyQPanda.VariationalQuantumCircuit

        26. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_SqiSWAP) -> pyQPanda.VariationalQuantumCircuit

        27. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumCircuit) -> pyQPanda.VariationalQuantumCircuit

        28. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.QCircuit) -> pyQPanda.VariationalQuantumCircuit

        29. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.QGate) -> pyQPanda.VariationalQuantumCircuit
        """
    @overload
    def __lshift__(self, arg0: VariationalQuantumGate_iSWAP) -> VariationalQuantumCircuit:
        """__lshift__(*args, **kwargs)
        Overloaded function.

        1. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_I) -> pyQPanda.VariationalQuantumCircuit

        2. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_H) -> pyQPanda.VariationalQuantumCircuit

        3. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_X) -> pyQPanda.VariationalQuantumCircuit

        4. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Y) -> pyQPanda.VariationalQuantumCircuit

        5. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_T) -> pyQPanda.VariationalQuantumCircuit

        6. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_S) -> pyQPanda.VariationalQuantumCircuit

        7. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Z) -> pyQPanda.VariationalQuantumCircuit

        8. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_X1) -> pyQPanda.VariationalQuantumCircuit

        9. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Y1) -> pyQPanda.VariationalQuantumCircuit

        10. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Z1) -> pyQPanda.VariationalQuantumCircuit

        11. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U1) -> pyQPanda.VariationalQuantumCircuit

        12. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U2) -> pyQPanda.VariationalQuantumCircuit

        13. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U3) -> pyQPanda.VariationalQuantumCircuit

        14. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U4) -> pyQPanda.VariationalQuantumCircuit

        15. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_RX) -> pyQPanda.VariationalQuantumCircuit

        16. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_RY) -> pyQPanda.VariationalQuantumCircuit

        17. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_RZ) -> pyQPanda.VariationalQuantumCircuit

        18. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CNOT) -> pyQPanda.VariationalQuantumCircuit

        19. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CR) -> pyQPanda.VariationalQuantumCircuit

        20. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CZ) -> pyQPanda.VariationalQuantumCircuit

        21. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CRX) -> pyQPanda.VariationalQuantumCircuit

        22. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CRY) -> pyQPanda.VariationalQuantumCircuit

        23. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CRZ) -> pyQPanda.VariationalQuantumCircuit

        24. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_SWAP) -> pyQPanda.VariationalQuantumCircuit

        25. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_iSWAP) -> pyQPanda.VariationalQuantumCircuit

        26. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_SqiSWAP) -> pyQPanda.VariationalQuantumCircuit

        27. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumCircuit) -> pyQPanda.VariationalQuantumCircuit

        28. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.QCircuit) -> pyQPanda.VariationalQuantumCircuit

        29. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.QGate) -> pyQPanda.VariationalQuantumCircuit
        """
    @overload
    def __lshift__(self, arg0: VariationalQuantumGate_SqiSWAP) -> VariationalQuantumCircuit:
        """__lshift__(*args, **kwargs)
        Overloaded function.

        1. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_I) -> pyQPanda.VariationalQuantumCircuit

        2. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_H) -> pyQPanda.VariationalQuantumCircuit

        3. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_X) -> pyQPanda.VariationalQuantumCircuit

        4. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Y) -> pyQPanda.VariationalQuantumCircuit

        5. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_T) -> pyQPanda.VariationalQuantumCircuit

        6. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_S) -> pyQPanda.VariationalQuantumCircuit

        7. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Z) -> pyQPanda.VariationalQuantumCircuit

        8. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_X1) -> pyQPanda.VariationalQuantumCircuit

        9. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Y1) -> pyQPanda.VariationalQuantumCircuit

        10. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Z1) -> pyQPanda.VariationalQuantumCircuit

        11. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U1) -> pyQPanda.VariationalQuantumCircuit

        12. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U2) -> pyQPanda.VariationalQuantumCircuit

        13. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U3) -> pyQPanda.VariationalQuantumCircuit

        14. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U4) -> pyQPanda.VariationalQuantumCircuit

        15. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_RX) -> pyQPanda.VariationalQuantumCircuit

        16. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_RY) -> pyQPanda.VariationalQuantumCircuit

        17. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_RZ) -> pyQPanda.VariationalQuantumCircuit

        18. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CNOT) -> pyQPanda.VariationalQuantumCircuit

        19. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CR) -> pyQPanda.VariationalQuantumCircuit

        20. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CZ) -> pyQPanda.VariationalQuantumCircuit

        21. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CRX) -> pyQPanda.VariationalQuantumCircuit

        22. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CRY) -> pyQPanda.VariationalQuantumCircuit

        23. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CRZ) -> pyQPanda.VariationalQuantumCircuit

        24. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_SWAP) -> pyQPanda.VariationalQuantumCircuit

        25. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_iSWAP) -> pyQPanda.VariationalQuantumCircuit

        26. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_SqiSWAP) -> pyQPanda.VariationalQuantumCircuit

        27. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumCircuit) -> pyQPanda.VariationalQuantumCircuit

        28. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.QCircuit) -> pyQPanda.VariationalQuantumCircuit

        29. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.QGate) -> pyQPanda.VariationalQuantumCircuit
        """
    @overload
    def __lshift__(self, arg0: VariationalQuantumCircuit) -> VariationalQuantumCircuit:
        """__lshift__(*args, **kwargs)
        Overloaded function.

        1. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_I) -> pyQPanda.VariationalQuantumCircuit

        2. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_H) -> pyQPanda.VariationalQuantumCircuit

        3. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_X) -> pyQPanda.VariationalQuantumCircuit

        4. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Y) -> pyQPanda.VariationalQuantumCircuit

        5. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_T) -> pyQPanda.VariationalQuantumCircuit

        6. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_S) -> pyQPanda.VariationalQuantumCircuit

        7. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Z) -> pyQPanda.VariationalQuantumCircuit

        8. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_X1) -> pyQPanda.VariationalQuantumCircuit

        9. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Y1) -> pyQPanda.VariationalQuantumCircuit

        10. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Z1) -> pyQPanda.VariationalQuantumCircuit

        11. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U1) -> pyQPanda.VariationalQuantumCircuit

        12. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U2) -> pyQPanda.VariationalQuantumCircuit

        13. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U3) -> pyQPanda.VariationalQuantumCircuit

        14. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U4) -> pyQPanda.VariationalQuantumCircuit

        15. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_RX) -> pyQPanda.VariationalQuantumCircuit

        16. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_RY) -> pyQPanda.VariationalQuantumCircuit

        17. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_RZ) -> pyQPanda.VariationalQuantumCircuit

        18. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CNOT) -> pyQPanda.VariationalQuantumCircuit

        19. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CR) -> pyQPanda.VariationalQuantumCircuit

        20. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CZ) -> pyQPanda.VariationalQuantumCircuit

        21. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CRX) -> pyQPanda.VariationalQuantumCircuit

        22. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CRY) -> pyQPanda.VariationalQuantumCircuit

        23. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CRZ) -> pyQPanda.VariationalQuantumCircuit

        24. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_SWAP) -> pyQPanda.VariationalQuantumCircuit

        25. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_iSWAP) -> pyQPanda.VariationalQuantumCircuit

        26. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_SqiSWAP) -> pyQPanda.VariationalQuantumCircuit

        27. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumCircuit) -> pyQPanda.VariationalQuantumCircuit

        28. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.QCircuit) -> pyQPanda.VariationalQuantumCircuit

        29. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.QGate) -> pyQPanda.VariationalQuantumCircuit
        """
    @overload
    def __lshift__(self, arg0: QCircuit) -> VariationalQuantumCircuit:
        """__lshift__(*args, **kwargs)
        Overloaded function.

        1. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_I) -> pyQPanda.VariationalQuantumCircuit

        2. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_H) -> pyQPanda.VariationalQuantumCircuit

        3. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_X) -> pyQPanda.VariationalQuantumCircuit

        4. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Y) -> pyQPanda.VariationalQuantumCircuit

        5. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_T) -> pyQPanda.VariationalQuantumCircuit

        6. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_S) -> pyQPanda.VariationalQuantumCircuit

        7. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Z) -> pyQPanda.VariationalQuantumCircuit

        8. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_X1) -> pyQPanda.VariationalQuantumCircuit

        9. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Y1) -> pyQPanda.VariationalQuantumCircuit

        10. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Z1) -> pyQPanda.VariationalQuantumCircuit

        11. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U1) -> pyQPanda.VariationalQuantumCircuit

        12. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U2) -> pyQPanda.VariationalQuantumCircuit

        13. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U3) -> pyQPanda.VariationalQuantumCircuit

        14. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U4) -> pyQPanda.VariationalQuantumCircuit

        15. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_RX) -> pyQPanda.VariationalQuantumCircuit

        16. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_RY) -> pyQPanda.VariationalQuantumCircuit

        17. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_RZ) -> pyQPanda.VariationalQuantumCircuit

        18. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CNOT) -> pyQPanda.VariationalQuantumCircuit

        19. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CR) -> pyQPanda.VariationalQuantumCircuit

        20. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CZ) -> pyQPanda.VariationalQuantumCircuit

        21. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CRX) -> pyQPanda.VariationalQuantumCircuit

        22. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CRY) -> pyQPanda.VariationalQuantumCircuit

        23. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CRZ) -> pyQPanda.VariationalQuantumCircuit

        24. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_SWAP) -> pyQPanda.VariationalQuantumCircuit

        25. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_iSWAP) -> pyQPanda.VariationalQuantumCircuit

        26. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_SqiSWAP) -> pyQPanda.VariationalQuantumCircuit

        27. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumCircuit) -> pyQPanda.VariationalQuantumCircuit

        28. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.QCircuit) -> pyQPanda.VariationalQuantumCircuit

        29. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.QGate) -> pyQPanda.VariationalQuantumCircuit
        """
    @overload
    def __lshift__(self, arg0: QGate) -> VariationalQuantumCircuit:
        """__lshift__(*args, **kwargs)
        Overloaded function.

        1. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_I) -> pyQPanda.VariationalQuantumCircuit

        2. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_H) -> pyQPanda.VariationalQuantumCircuit

        3. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_X) -> pyQPanda.VariationalQuantumCircuit

        4. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Y) -> pyQPanda.VariationalQuantumCircuit

        5. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_T) -> pyQPanda.VariationalQuantumCircuit

        6. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_S) -> pyQPanda.VariationalQuantumCircuit

        7. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Z) -> pyQPanda.VariationalQuantumCircuit

        8. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_X1) -> pyQPanda.VariationalQuantumCircuit

        9. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Y1) -> pyQPanda.VariationalQuantumCircuit

        10. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_Z1) -> pyQPanda.VariationalQuantumCircuit

        11. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U1) -> pyQPanda.VariationalQuantumCircuit

        12. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U2) -> pyQPanda.VariationalQuantumCircuit

        13. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U3) -> pyQPanda.VariationalQuantumCircuit

        14. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_U4) -> pyQPanda.VariationalQuantumCircuit

        15. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_RX) -> pyQPanda.VariationalQuantumCircuit

        16. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_RY) -> pyQPanda.VariationalQuantumCircuit

        17. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_RZ) -> pyQPanda.VariationalQuantumCircuit

        18. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CNOT) -> pyQPanda.VariationalQuantumCircuit

        19. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CR) -> pyQPanda.VariationalQuantumCircuit

        20. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CZ) -> pyQPanda.VariationalQuantumCircuit

        21. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CRX) -> pyQPanda.VariationalQuantumCircuit

        22. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CRY) -> pyQPanda.VariationalQuantumCircuit

        23. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_CRZ) -> pyQPanda.VariationalQuantumCircuit

        24. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_SWAP) -> pyQPanda.VariationalQuantumCircuit

        25. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_iSWAP) -> pyQPanda.VariationalQuantumCircuit

        26. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumGate_SqiSWAP) -> pyQPanda.VariationalQuantumCircuit

        27. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.VariationalQuantumCircuit) -> pyQPanda.VariationalQuantumCircuit

        28. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.QCircuit) -> pyQPanda.VariationalQuantumCircuit

        29. __lshift__(self: pyQPanda.VariationalQuantumCircuit, arg0: pyQPanda.QGate) -> pyQPanda.VariationalQuantumCircuit
        """

class VariationalQuantumGate:
    def __init__(self, *args, **kwargs) -> None:
        """Initialize self.  See help(type(self)) for accurate signature."""
    def feed(self, arg0: dict[int, float]) -> QGate:
        """feed(self: pyQPanda.VariationalQuantumGate, arg0: Dict[int, float]) -> pyQPanda.QGate"""
    def get_constants(self) -> list[float]:
        """get_constants(self: pyQPanda.VariationalQuantumGate) -> List[float]"""
    def get_control_qubit(self) -> QVec:
        """get_control_qubit(self: pyQPanda.VariationalQuantumGate) -> pyQPanda.QVec"""
    def get_vars(self) -> list[var]:
        """get_vars(self: pyQPanda.VariationalQuantumGate) -> List[pyQPanda.var]"""
    def is_dagger(self) -> bool:
        """is_dagger(self: pyQPanda.VariationalQuantumGate) -> bool"""
    def set_control(self, arg0: QVec) -> bool:
        """set_control(self: pyQPanda.VariationalQuantumGate, arg0: pyQPanda.QVec) -> bool"""
    def set_dagger(self, arg0: bool) -> bool:
        """set_dagger(self: pyQPanda.VariationalQuantumGate, arg0: bool) -> bool"""

class VariationalQuantumGate_CNOT(VariationalQuantumGate):
    def __init__(self, arg0: Qubit, arg1: Qubit) -> None:
        """__init__(self: pyQPanda.VariationalQuantumGate_CNOT, arg0: pyQPanda.Qubit, arg1: pyQPanda.Qubit) -> None"""
    def control(self, arg0: QVec) -> VariationalQuantumGate_CNOT:
        """control(self: pyQPanda.VariationalQuantumGate_CNOT, arg0: pyQPanda.QVec) -> pyQPanda.VariationalQuantumGate_CNOT"""
    def dagger(self) -> VariationalQuantumGate_CNOT:
        """dagger(self: pyQPanda.VariationalQuantumGate_CNOT) -> pyQPanda.VariationalQuantumGate_CNOT"""

class VariationalQuantumGate_CR(VariationalQuantumGate):
    @overload
    def __init__(self, arg0: Qubit, arg1: Qubit, arg2: float) -> None:
        """__init__(*args, **kwargs)
        Overloaded function.

        1. __init__(self: pyQPanda.VariationalQuantumGate_CR, arg0: pyQPanda.Qubit, arg1: pyQPanda.Qubit, arg2: float) -> None

        2. __init__(self: pyQPanda.VariationalQuantumGate_CR, arg0: pyQPanda.Qubit, arg1: pyQPanda.Qubit, arg2: pyQPanda.var) -> None

        3. __init__(self: pyQPanda.VariationalQuantumGate_CR, arg0: pyQPanda.VariationalQuantumGate_CR) -> None
        """
    @overload
    def __init__(self, arg0: Qubit, arg1: Qubit, arg2: var) -> None:
        """__init__(*args, **kwargs)
        Overloaded function.

        1. __init__(self: pyQPanda.VariationalQuantumGate_CR, arg0: pyQPanda.Qubit, arg1: pyQPanda.Qubit, arg2: float) -> None

        2. __init__(self: pyQPanda.VariationalQuantumGate_CR, arg0: pyQPanda.Qubit, arg1: pyQPanda.Qubit, arg2: pyQPanda.var) -> None

        3. __init__(self: pyQPanda.VariationalQuantumGate_CR, arg0: pyQPanda.VariationalQuantumGate_CR) -> None
        """
    @overload
    def __init__(self, arg0: VariationalQuantumGate_CR) -> None:
        """__init__(*args, **kwargs)
        Overloaded function.

        1. __init__(self: pyQPanda.VariationalQuantumGate_CR, arg0: pyQPanda.Qubit, arg1: pyQPanda.Qubit, arg2: float) -> None

        2. __init__(self: pyQPanda.VariationalQuantumGate_CR, arg0: pyQPanda.Qubit, arg1: pyQPanda.Qubit, arg2: pyQPanda.var) -> None

        3. __init__(self: pyQPanda.VariationalQuantumGate_CR, arg0: pyQPanda.VariationalQuantumGate_CR) -> None
        """
    def control(self, arg0: QVec) -> VariationalQuantumGate_CR:
        """control(self: pyQPanda.VariationalQuantumGate_CR, arg0: pyQPanda.QVec) -> pyQPanda.VariationalQuantumGate_CR"""
    def dagger(self) -> VariationalQuantumGate_CR:
        """dagger(self: pyQPanda.VariationalQuantumGate_CR) -> pyQPanda.VariationalQuantumGate_CR"""

class VariationalQuantumGate_CRX(VariationalQuantumGate):
    @overload
    def __init__(self, arg0: Qubit, arg1: QVec, arg2: float) -> None:
        """__init__(*args, **kwargs)
        Overloaded function.

        1. __init__(self: pyQPanda.VariationalQuantumGate_CRX, arg0: pyQPanda.Qubit, arg1: pyQPanda.QVec, arg2: float) -> None

        2. __init__(self: pyQPanda.VariationalQuantumGate_CRX, arg0: pyQPanda.Qubit, arg1: pyQPanda.QVec, arg2: pyQPanda.var) -> None

        3. __init__(self: pyQPanda.VariationalQuantumGate_CRX, arg0: pyQPanda.VariationalQuantumGate_CRX) -> None
        """
    @overload
    def __init__(self, arg0: Qubit, arg1: QVec, arg2: var) -> None:
        """__init__(*args, **kwargs)
        Overloaded function.

        1. __init__(self: pyQPanda.VariationalQuantumGate_CRX, arg0: pyQPanda.Qubit, arg1: pyQPanda.QVec, arg2: float) -> None

        2. __init__(self: pyQPanda.VariationalQuantumGate_CRX, arg0: pyQPanda.Qubit, arg1: pyQPanda.QVec, arg2: pyQPanda.var) -> None

        3. __init__(self: pyQPanda.VariationalQuantumGate_CRX, arg0: pyQPanda.VariationalQuantumGate_CRX) -> None
        """
    @overload
    def __init__(self, arg0: VariationalQuantumGate_CRX) -> None:
        """__init__(*args, **kwargs)
        Overloaded function.

        1. __init__(self: pyQPanda.VariationalQuantumGate_CRX, arg0: pyQPanda.Qubit, arg1: pyQPanda.QVec, arg2: float) -> None

        2. __init__(self: pyQPanda.VariationalQuantumGate_CRX, arg0: pyQPanda.Qubit, arg1: pyQPanda.QVec, arg2: pyQPanda.var) -> None

        3. __init__(self: pyQPanda.VariationalQuantumGate_CRX, arg0: pyQPanda.VariationalQuantumGate_CRX) -> None
        """
    def control(self, arg0: QVec) -> VariationalQuantumGate_CRX:
        """control(self: pyQPanda.VariationalQuantumGate_CRX, arg0: pyQPanda.QVec) -> pyQPanda.VariationalQuantumGate_CRX"""
    def dagger(self) -> VariationalQuantumGate_CRX:
        """dagger(self: pyQPanda.VariationalQuantumGate_CRX) -> pyQPanda.VariationalQuantumGate_CRX"""

class VariationalQuantumGate_CRY(VariationalQuantumGate):
    @overload
    def __init__(self, arg0: Qubit, arg1: QVec, arg2: float) -> None:
        """__init__(*args, **kwargs)
        Overloaded function.

        1. __init__(self: pyQPanda.VariationalQuantumGate_CRY, arg0: pyQPanda.Qubit, arg1: pyQPanda.QVec, arg2: float) -> None

        2. __init__(self: pyQPanda.VariationalQuantumGate_CRY, arg0: pyQPanda.Qubit, arg1: pyQPanda.QVec, arg2: pyQPanda.var) -> None

        3. __init__(self: pyQPanda.VariationalQuantumGate_CRY, arg0: pyQPanda.VariationalQuantumGate_CRY) -> None
        """
    @overload
    def __init__(self, arg0: Qubit, arg1: QVec, arg2: var) -> None:
        """__init__(*args, **kwargs)
        Overloaded function.

        1. __init__(self: pyQPanda.VariationalQuantumGate_CRY, arg0: pyQPanda.Qubit, arg1: pyQPanda.QVec, arg2: float) -> None

        2. __init__(self: pyQPanda.VariationalQuantumGate_CRY, arg0: pyQPanda.Qubit, arg1: pyQPanda.QVec, arg2: pyQPanda.var) -> None

        3. __init__(self: pyQPanda.VariationalQuantumGate_CRY, arg0: pyQPanda.VariationalQuantumGate_CRY) -> None
        """
    @overload
    def __init__(self, arg0: VariationalQuantumGate_CRY) -> None:
        """__init__(*args, **kwargs)
        Overloaded function.

        1. __init__(self: pyQPanda.VariationalQuantumGate_CRY, arg0: pyQPanda.Qubit, arg1: pyQPanda.QVec, arg2: float) -> None

        2. __init__(self: pyQPanda.VariationalQuantumGate_CRY, arg0: pyQPanda.Qubit, arg1: pyQPanda.QVec, arg2: pyQPanda.var) -> None

        3. __init__(self: pyQPanda.VariationalQuantumGate_CRY, arg0: pyQPanda.VariationalQuantumGate_CRY) -> None
        """
    def control(self, arg0: QVec) -> VariationalQuantumGate_CRY:
        """control(self: pyQPanda.VariationalQuantumGate_CRY, arg0: pyQPanda.QVec) -> pyQPanda.VariationalQuantumGate_CRY"""
    def dagger(self) -> VariationalQuantumGate_CRY:
        """dagger(self: pyQPanda.VariationalQuantumGate_CRY) -> pyQPanda.VariationalQuantumGate_CRY"""

class VariationalQuantumGate_CRZ(VariationalQuantumGate):
    @overload
    def __init__(self, arg0: Qubit, arg1: QVec, arg2: float) -> None:
        """__init__(*args, **kwargs)
        Overloaded function.

        1. __init__(self: pyQPanda.VariationalQuantumGate_CRZ, arg0: pyQPanda.Qubit, arg1: pyQPanda.QVec, arg2: float) -> None

        2. __init__(self: pyQPanda.VariationalQuantumGate_CRZ, arg0: pyQPanda.Qubit, arg1: pyQPanda.QVec, arg2: pyQPanda.var) -> None

        3. __init__(self: pyQPanda.VariationalQuantumGate_CRZ, arg0: pyQPanda.VariationalQuantumGate_CRZ) -> None
        """
    @overload
    def __init__(self, arg0: Qubit, arg1: QVec, arg2: var) -> None:
        """__init__(*args, **kwargs)
        Overloaded function.

        1. __init__(self: pyQPanda.VariationalQuantumGate_CRZ, arg0: pyQPanda.Qubit, arg1: pyQPanda.QVec, arg2: float) -> None

        2. __init__(self: pyQPanda.VariationalQuantumGate_CRZ, arg0: pyQPanda.Qubit, arg1: pyQPanda.QVec, arg2: pyQPanda.var) -> None

        3. __init__(self: pyQPanda.VariationalQuantumGate_CRZ, arg0: pyQPanda.VariationalQuantumGate_CRZ) -> None
        """
    @overload
    def __init__(self, arg0: VariationalQuantumGate_CRZ) -> None:
        """__init__(*args, **kwargs)
        Overloaded function.

        1. __init__(self: pyQPanda.VariationalQuantumGate_CRZ, arg0: pyQPanda.Qubit, arg1: pyQPanda.QVec, arg2: float) -> None

        2. __init__(self: pyQPanda.VariationalQuantumGate_CRZ, arg0: pyQPanda.Qubit, arg1: pyQPanda.QVec, arg2: pyQPanda.var) -> None

        3. __init__(self: pyQPanda.VariationalQuantumGate_CRZ, arg0: pyQPanda.VariationalQuantumGate_CRZ) -> None
        """
    def control(self, arg0: QVec) -> VariationalQuantumGate_CRZ:
        """control(self: pyQPanda.VariationalQuantumGate_CRZ, arg0: pyQPanda.QVec) -> pyQPanda.VariationalQuantumGate_CRZ"""
    def dagger(self) -> VariationalQuantumGate_CRZ:
        """dagger(self: pyQPanda.VariationalQuantumGate_CRZ) -> pyQPanda.VariationalQuantumGate_CRZ"""

class VariationalQuantumGate_CU(VariationalQuantumGate):
    @overload
    def __init__(self, arg0: Qubit, arg1: Qubit, arg2: float, arg3: float, arg4: float, arg5: float) -> None:
        """__init__(*args, **kwargs)
        Overloaded function.

        1. __init__(self: pyQPanda.VariationalQuantumGate_CU, arg0: pyQPanda.Qubit, arg1: pyQPanda.Qubit, arg2: float, arg3: float, arg4: float, arg5: float) -> None

        2. __init__(self: pyQPanda.VariationalQuantumGate_CU, arg0: pyQPanda.Qubit, arg1: pyQPanda.Qubit, arg2: pyQPanda.var, arg3: pyQPanda.var, arg4: pyQPanda.var, arg5: pyQPanda.var) -> None

        3. __init__(self: pyQPanda.VariationalQuantumGate_CU, arg0: pyQPanda.VariationalQuantumGate_CU) -> None
        """
    @overload
    def __init__(self, arg0: Qubit, arg1: Qubit, arg2: var, arg3: var, arg4: var, arg5: var) -> None:
        """__init__(*args, **kwargs)
        Overloaded function.

        1. __init__(self: pyQPanda.VariationalQuantumGate_CU, arg0: pyQPanda.Qubit, arg1: pyQPanda.Qubit, arg2: float, arg3: float, arg4: float, arg5: float) -> None

        2. __init__(self: pyQPanda.VariationalQuantumGate_CU, arg0: pyQPanda.Qubit, arg1: pyQPanda.Qubit, arg2: pyQPanda.var, arg3: pyQPanda.var, arg4: pyQPanda.var, arg5: pyQPanda.var) -> None

        3. __init__(self: pyQPanda.VariationalQuantumGate_CU, arg0: pyQPanda.VariationalQuantumGate_CU) -> None
        """
    @overload
    def __init__(self, arg0: VariationalQuantumGate_CU) -> None:
        """__init__(*args, **kwargs)
        Overloaded function.

        1. __init__(self: pyQPanda.VariationalQuantumGate_CU, arg0: pyQPanda.Qubit, arg1: pyQPanda.Qubit, arg2: float, arg3: float, arg4: float, arg5: float) -> None

        2. __init__(self: pyQPanda.VariationalQuantumGate_CU, arg0: pyQPanda.Qubit, arg1: pyQPanda.Qubit, arg2: pyQPanda.var, arg3: pyQPanda.var, arg4: pyQPanda.var, arg5: pyQPanda.var) -> None

        3. __init__(self: pyQPanda.VariationalQuantumGate_CU, arg0: pyQPanda.VariationalQuantumGate_CU) -> None
        """
    def control(self, arg0: QVec) -> VariationalQuantumGate_CU:
        """control(self: pyQPanda.VariationalQuantumGate_CU, arg0: pyQPanda.QVec) -> pyQPanda.VariationalQuantumGate_CU"""
    def dagger(self) -> VariationalQuantumGate_CU:
        """dagger(self: pyQPanda.VariationalQuantumGate_CU) -> pyQPanda.VariationalQuantumGate_CU"""

class VariationalQuantumGate_CZ(VariationalQuantumGate):
    def __init__(self, arg0: Qubit, arg1: Qubit) -> None:
        """__init__(self: pyQPanda.VariationalQuantumGate_CZ, arg0: pyQPanda.Qubit, arg1: pyQPanda.Qubit) -> None"""
    def control(self, arg0: QVec) -> VariationalQuantumGate_CZ:
        """control(self: pyQPanda.VariationalQuantumGate_CZ, arg0: pyQPanda.QVec) -> pyQPanda.VariationalQuantumGate_CZ"""
    def dagger(self) -> VariationalQuantumGate_CZ:
        """dagger(self: pyQPanda.VariationalQuantumGate_CZ) -> pyQPanda.VariationalQuantumGate_CZ"""

class VariationalQuantumGate_H(VariationalQuantumGate):
    def __init__(self, arg0: Qubit) -> None:
        """__init__(self: pyQPanda.VariationalQuantumGate_H, arg0: pyQPanda.Qubit) -> None"""
    def control(self, arg0: QVec) -> VariationalQuantumGate_H:
        """control(self: pyQPanda.VariationalQuantumGate_H, arg0: pyQPanda.QVec) -> pyQPanda.VariationalQuantumGate_H"""
    def dagger(self) -> VariationalQuantumGate_H:
        """dagger(self: pyQPanda.VariationalQuantumGate_H) -> pyQPanda.VariationalQuantumGate_H"""

class VariationalQuantumGate_I(VariationalQuantumGate):
    def __init__(self, arg0: Qubit) -> None:
        """__init__(self: pyQPanda.VariationalQuantumGate_I, arg0: pyQPanda.Qubit) -> None"""
    def control(self, arg0: QVec) -> VariationalQuantumGate_I:
        """control(self: pyQPanda.VariationalQuantumGate_I, arg0: pyQPanda.QVec) -> pyQPanda.VariationalQuantumGate_I"""
    def dagger(self) -> VariationalQuantumGate_I:
        """dagger(self: pyQPanda.VariationalQuantumGate_I) -> pyQPanda.VariationalQuantumGate_I"""

class VariationalQuantumGate_RX(VariationalQuantumGate):
    @overload
    def __init__(self, arg0: Qubit, arg1: var) -> None:
        """__init__(*args, **kwargs)
        Overloaded function.

        1. __init__(self: pyQPanda.VariationalQuantumGate_RX, arg0: pyQPanda.Qubit, arg1: pyQPanda.var) -> None

        2. __init__(self: pyQPanda.VariationalQuantumGate_RX, arg0: pyQPanda.Qubit, arg1: float) -> None
        """
    @overload
    def __init__(self, arg0: Qubit, arg1: float) -> None:
        """__init__(*args, **kwargs)
        Overloaded function.

        1. __init__(self: pyQPanda.VariationalQuantumGate_RX, arg0: pyQPanda.Qubit, arg1: pyQPanda.var) -> None

        2. __init__(self: pyQPanda.VariationalQuantumGate_RX, arg0: pyQPanda.Qubit, arg1: float) -> None
        """
    def control(self, arg0: QVec) -> VariationalQuantumGate_RX:
        """control(self: pyQPanda.VariationalQuantumGate_RX, arg0: pyQPanda.QVec) -> pyQPanda.VariationalQuantumGate_RX"""
    def dagger(self) -> VariationalQuantumGate_RX:
        """dagger(self: pyQPanda.VariationalQuantumGate_RX) -> pyQPanda.VariationalQuantumGate_RX"""

class VariationalQuantumGate_RY(VariationalQuantumGate):
    @overload
    def __init__(self, arg0: Qubit, arg1: var) -> None:
        """__init__(*args, **kwargs)
        Overloaded function.

        1. __init__(self: pyQPanda.VariationalQuantumGate_RY, arg0: pyQPanda.Qubit, arg1: pyQPanda.var) -> None

        2. __init__(self: pyQPanda.VariationalQuantumGate_RY, arg0: pyQPanda.Qubit, arg1: float) -> None
        """
    @overload
    def __init__(self, arg0: Qubit, arg1: float) -> None:
        """__init__(*args, **kwargs)
        Overloaded function.

        1. __init__(self: pyQPanda.VariationalQuantumGate_RY, arg0: pyQPanda.Qubit, arg1: pyQPanda.var) -> None

        2. __init__(self: pyQPanda.VariationalQuantumGate_RY, arg0: pyQPanda.Qubit, arg1: float) -> None
        """
    def control(self, arg0: QVec) -> VariationalQuantumGate_RY:
        """control(self: pyQPanda.VariationalQuantumGate_RY, arg0: pyQPanda.QVec) -> pyQPanda.VariationalQuantumGate_RY"""
    def dagger(self) -> VariationalQuantumGate_RY:
        """dagger(self: pyQPanda.VariationalQuantumGate_RY) -> pyQPanda.VariationalQuantumGate_RY"""

class VariationalQuantumGate_RZ(VariationalQuantumGate):
    @overload
    def __init__(self, arg0: Qubit, arg1: var) -> None:
        """__init__(*args, **kwargs)
        Overloaded function.

        1. __init__(self: pyQPanda.VariationalQuantumGate_RZ, arg0: pyQPanda.Qubit, arg1: pyQPanda.var) -> None

        2. __init__(self: pyQPanda.VariationalQuantumGate_RZ, arg0: pyQPanda.Qubit, arg1: float) -> None
        """
    @overload
    def __init__(self, arg0: Qubit, arg1: float) -> None:
        """__init__(*args, **kwargs)
        Overloaded function.

        1. __init__(self: pyQPanda.VariationalQuantumGate_RZ, arg0: pyQPanda.Qubit, arg1: pyQPanda.var) -> None

        2. __init__(self: pyQPanda.VariationalQuantumGate_RZ, arg0: pyQPanda.Qubit, arg1: float) -> None
        """
    def control(self, arg0: QVec) -> VariationalQuantumGate_RZ:
        """control(self: pyQPanda.VariationalQuantumGate_RZ, arg0: pyQPanda.QVec) -> pyQPanda.VariationalQuantumGate_RZ"""
    def dagger(self) -> VariationalQuantumGate_RZ:
        """dagger(self: pyQPanda.VariationalQuantumGate_RZ) -> pyQPanda.VariationalQuantumGate_RZ"""

class VariationalQuantumGate_S(VariationalQuantumGate):
    def __init__(self, arg0: Qubit) -> None:
        """__init__(self: pyQPanda.VariationalQuantumGate_S, arg0: pyQPanda.Qubit) -> None"""
    def control(self, arg0: QVec) -> VariationalQuantumGate_S:
        """control(self: pyQPanda.VariationalQuantumGate_S, arg0: pyQPanda.QVec) -> pyQPanda.VariationalQuantumGate_S"""
    def dagger(self) -> VariationalQuantumGate_S:
        """dagger(self: pyQPanda.VariationalQuantumGate_S) -> pyQPanda.VariationalQuantumGate_S"""

class VariationalQuantumGate_SWAP(VariationalQuantumGate):
    def __init__(self, arg0: Qubit, arg1: Qubit) -> None:
        """__init__(self: pyQPanda.VariationalQuantumGate_SWAP, arg0: pyQPanda.Qubit, arg1: pyQPanda.Qubit) -> None"""
    def control(self, arg0: QVec) -> VariationalQuantumGate_SWAP:
        """control(self: pyQPanda.VariationalQuantumGate_SWAP, arg0: pyQPanda.QVec) -> pyQPanda.VariationalQuantumGate_SWAP"""
    def dagger(self) -> VariationalQuantumGate_SWAP:
        """dagger(self: pyQPanda.VariationalQuantumGate_SWAP) -> pyQPanda.VariationalQuantumGate_SWAP"""

class VariationalQuantumGate_SqiSWAP(VariationalQuantumGate):
    def __init__(self, arg0: Qubit, arg1: Qubit) -> None:
        """__init__(self: pyQPanda.VariationalQuantumGate_SqiSWAP, arg0: pyQPanda.Qubit, arg1: pyQPanda.Qubit) -> None"""
    def control(self, arg0: QVec) -> VariationalQuantumGate_SqiSWAP:
        """control(self: pyQPanda.VariationalQuantumGate_SqiSWAP, arg0: pyQPanda.QVec) -> pyQPanda.VariationalQuantumGate_SqiSWAP"""
    def dagger(self) -> VariationalQuantumGate_SqiSWAP:
        """dagger(self: pyQPanda.VariationalQuantumGate_SqiSWAP) -> pyQPanda.VariationalQuantumGate_SqiSWAP"""

class VariationalQuantumGate_T(VariationalQuantumGate):
    def __init__(self, arg0: Qubit) -> None:
        """__init__(self: pyQPanda.VariationalQuantumGate_T, arg0: pyQPanda.Qubit) -> None"""
    def control(self, arg0: QVec) -> VariationalQuantumGate_T:
        """control(self: pyQPanda.VariationalQuantumGate_T, arg0: pyQPanda.QVec) -> pyQPanda.VariationalQuantumGate_T"""
    def dagger(self) -> VariationalQuantumGate_T:
        """dagger(self: pyQPanda.VariationalQuantumGate_T) -> pyQPanda.VariationalQuantumGate_T"""

class VariationalQuantumGate_U1(VariationalQuantumGate):
    @overload
    def __init__(self, arg0: Qubit, arg1: var) -> None:
        """__init__(*args, **kwargs)
        Overloaded function.

        1. __init__(self: pyQPanda.VariationalQuantumGate_U1, arg0: pyQPanda.Qubit, arg1: pyQPanda.var) -> None

        2. __init__(self: pyQPanda.VariationalQuantumGate_U1, arg0: pyQPanda.Qubit, arg1: float) -> None
        """
    @overload
    def __init__(self, arg0: Qubit, arg1: float) -> None:
        """__init__(*args, **kwargs)
        Overloaded function.

        1. __init__(self: pyQPanda.VariationalQuantumGate_U1, arg0: pyQPanda.Qubit, arg1: pyQPanda.var) -> None

        2. __init__(self: pyQPanda.VariationalQuantumGate_U1, arg0: pyQPanda.Qubit, arg1: float) -> None
        """
    def control(self, arg0: QVec) -> VariationalQuantumGate_U1:
        """control(self: pyQPanda.VariationalQuantumGate_U1, arg0: pyQPanda.QVec) -> pyQPanda.VariationalQuantumGate_U1"""
    def dagger(self) -> VariationalQuantumGate_U1:
        """dagger(self: pyQPanda.VariationalQuantumGate_U1) -> pyQPanda.VariationalQuantumGate_U1"""

class VariationalQuantumGate_U2(VariationalQuantumGate):
    @overload
    def __init__(self, arg0: Qubit, arg1: var, arg2: var) -> None:
        """__init__(*args, **kwargs)
        Overloaded function.

        1. __init__(self: pyQPanda.VariationalQuantumGate_U2, arg0: pyQPanda.Qubit, arg1: pyQPanda.var, arg2: pyQPanda.var) -> None

        2. __init__(self: pyQPanda.VariationalQuantumGate_U2, arg0: pyQPanda.Qubit, arg1: float, arg2: float) -> None
        """
    @overload
    def __init__(self, arg0: Qubit, arg1: float, arg2: float) -> None:
        """__init__(*args, **kwargs)
        Overloaded function.

        1. __init__(self: pyQPanda.VariationalQuantumGate_U2, arg0: pyQPanda.Qubit, arg1: pyQPanda.var, arg2: pyQPanda.var) -> None

        2. __init__(self: pyQPanda.VariationalQuantumGate_U2, arg0: pyQPanda.Qubit, arg1: float, arg2: float) -> None
        """
    def control(self, arg0: QVec) -> VariationalQuantumGate_U2:
        """control(self: pyQPanda.VariationalQuantumGate_U2, arg0: pyQPanda.QVec) -> pyQPanda.VariationalQuantumGate_U2"""
    def dagger(self) -> VariationalQuantumGate_U2:
        """dagger(self: pyQPanda.VariationalQuantumGate_U2) -> pyQPanda.VariationalQuantumGate_U2"""

class VariationalQuantumGate_U3(VariationalQuantumGate):
    @overload
    def __init__(self, arg0: Qubit, arg1: var, arg2: var, arg3: var) -> None:
        """__init__(*args, **kwargs)
        Overloaded function.

        1. __init__(self: pyQPanda.VariationalQuantumGate_U3, arg0: pyQPanda.Qubit, arg1: pyQPanda.var, arg2: pyQPanda.var, arg3: pyQPanda.var) -> None

        2. __init__(self: pyQPanda.VariationalQuantumGate_U3, arg0: pyQPanda.Qubit, arg1: float, arg2: float, arg3: float) -> None
        """
    @overload
    def __init__(self, arg0: Qubit, arg1: float, arg2: float, arg3: float) -> None:
        """__init__(*args, **kwargs)
        Overloaded function.

        1. __init__(self: pyQPanda.VariationalQuantumGate_U3, arg0: pyQPanda.Qubit, arg1: pyQPanda.var, arg2: pyQPanda.var, arg3: pyQPanda.var) -> None

        2. __init__(self: pyQPanda.VariationalQuantumGate_U3, arg0: pyQPanda.Qubit, arg1: float, arg2: float, arg3: float) -> None
        """
    def control(self, arg0: QVec) -> VariationalQuantumGate_U3:
        """control(self: pyQPanda.VariationalQuantumGate_U3, arg0: pyQPanda.QVec) -> pyQPanda.VariationalQuantumGate_U3"""
    def dagger(self) -> VariationalQuantumGate_U3:
        """dagger(self: pyQPanda.VariationalQuantumGate_U3) -> pyQPanda.VariationalQuantumGate_U3"""

class VariationalQuantumGate_U4(VariationalQuantumGate):
    @overload
    def __init__(self, arg0: Qubit, arg1: var, arg2: var, arg3: var, arg4: var) -> None:
        """__init__(*args, **kwargs)
        Overloaded function.

        1. __init__(self: pyQPanda.VariationalQuantumGate_U4, arg0: pyQPanda.Qubit, arg1: pyQPanda.var, arg2: pyQPanda.var, arg3: pyQPanda.var, arg4: pyQPanda.var) -> None

        2. __init__(self: pyQPanda.VariationalQuantumGate_U4, arg0: pyQPanda.Qubit, arg1: float, arg2: float, arg3: float, arg4: float) -> None
        """
    @overload
    def __init__(self, arg0: Qubit, arg1: float, arg2: float, arg3: float, arg4: float) -> None:
        """__init__(*args, **kwargs)
        Overloaded function.

        1. __init__(self: pyQPanda.VariationalQuantumGate_U4, arg0: pyQPanda.Qubit, arg1: pyQPanda.var, arg2: pyQPanda.var, arg3: pyQPanda.var, arg4: pyQPanda.var) -> None

        2. __init__(self: pyQPanda.VariationalQuantumGate_U4, arg0: pyQPanda.Qubit, arg1: float, arg2: float, arg3: float, arg4: float) -> None
        """
    def control(self, arg0: QVec) -> VariationalQuantumGate_U4:
        """control(self: pyQPanda.VariationalQuantumGate_U4, arg0: pyQPanda.QVec) -> pyQPanda.VariationalQuantumGate_U4"""
    def dagger(self) -> VariationalQuantumGate_U4:
        """dagger(self: pyQPanda.VariationalQuantumGate_U4) -> pyQPanda.VariationalQuantumGate_U4"""

class VariationalQuantumGate_X(VariationalQuantumGate):
    def __init__(self, arg0: Qubit) -> None:
        """__init__(self: pyQPanda.VariationalQuantumGate_X, arg0: pyQPanda.Qubit) -> None"""
    def control(self, arg0: QVec) -> VariationalQuantumGate_X:
        """control(self: pyQPanda.VariationalQuantumGate_X, arg0: pyQPanda.QVec) -> pyQPanda.VariationalQuantumGate_X"""
    def dagger(self) -> VariationalQuantumGate_X:
        """dagger(self: pyQPanda.VariationalQuantumGate_X) -> pyQPanda.VariationalQuantumGate_X"""

class VariationalQuantumGate_X1(VariationalQuantumGate):
    def __init__(self, arg0: Qubit) -> None:
        """__init__(self: pyQPanda.VariationalQuantumGate_X1, arg0: pyQPanda.Qubit) -> None"""
    def control(self, arg0: QVec) -> VariationalQuantumGate_X1:
        """control(self: pyQPanda.VariationalQuantumGate_X1, arg0: pyQPanda.QVec) -> pyQPanda.VariationalQuantumGate_X1"""
    def dagger(self) -> VariationalQuantumGate_X1:
        """dagger(self: pyQPanda.VariationalQuantumGate_X1) -> pyQPanda.VariationalQuantumGate_X1"""

class VariationalQuantumGate_Y(VariationalQuantumGate):
    def __init__(self, arg0: Qubit) -> None:
        """__init__(self: pyQPanda.VariationalQuantumGate_Y, arg0: pyQPanda.Qubit) -> None"""
    def control(self, arg0: QVec) -> VariationalQuantumGate_Y:
        """control(self: pyQPanda.VariationalQuantumGate_Y, arg0: pyQPanda.QVec) -> pyQPanda.VariationalQuantumGate_Y"""
    def dagger(self) -> VariationalQuantumGate_Y:
        """dagger(self: pyQPanda.VariationalQuantumGate_Y) -> pyQPanda.VariationalQuantumGate_Y"""

class VariationalQuantumGate_Y1(VariationalQuantumGate):
    def __init__(self, arg0: Qubit) -> None:
        """__init__(self: pyQPanda.VariationalQuantumGate_Y1, arg0: pyQPanda.Qubit) -> None"""
    def control(self, arg0: QVec) -> VariationalQuantumGate_Y1:
        """control(self: pyQPanda.VariationalQuantumGate_Y1, arg0: pyQPanda.QVec) -> pyQPanda.VariationalQuantumGate_Y1"""
    def dagger(self) -> VariationalQuantumGate_Y1:
        """dagger(self: pyQPanda.VariationalQuantumGate_Y1) -> pyQPanda.VariationalQuantumGate_Y1"""

class VariationalQuantumGate_Z(VariationalQuantumGate):
    def __init__(self, arg0: Qubit) -> None:
        """__init__(self: pyQPanda.VariationalQuantumGate_Z, arg0: pyQPanda.Qubit) -> None"""
    def control(self, arg0: QVec) -> VariationalQuantumGate_Z:
        """control(self: pyQPanda.VariationalQuantumGate_Z, arg0: pyQPanda.QVec) -> pyQPanda.VariationalQuantumGate_Z"""
    def dagger(self) -> VariationalQuantumGate_Z:
        """dagger(self: pyQPanda.VariationalQuantumGate_Z) -> pyQPanda.VariationalQuantumGate_Z"""

class VariationalQuantumGate_Z1(VariationalQuantumGate):
    def __init__(self, arg0: Qubit) -> None:
        """__init__(self: pyQPanda.VariationalQuantumGate_Z1, arg0: pyQPanda.Qubit) -> None"""
    def control(self, arg0: QVec) -> VariationalQuantumGate_Z1:
        """control(self: pyQPanda.VariationalQuantumGate_Z1, arg0: pyQPanda.QVec) -> pyQPanda.VariationalQuantumGate_Z1"""
    def dagger(self) -> VariationalQuantumGate_Z1:
        """dagger(self: pyQPanda.VariationalQuantumGate_Z1) -> pyQPanda.VariationalQuantumGate_Z1"""

class VariationalQuantumGate_iSWAP(VariationalQuantumGate):
    def __init__(self, arg0: Qubit, arg1: Qubit) -> None:
        """__init__(self: pyQPanda.VariationalQuantumGate_iSWAP, arg0: pyQPanda.Qubit, arg1: pyQPanda.Qubit) -> None"""
    def control(self, arg0: QVec) -> VariationalQuantumGate_iSWAP:
        """control(self: pyQPanda.VariationalQuantumGate_iSWAP, arg0: pyQPanda.QVec) -> pyQPanda.VariationalQuantumGate_iSWAP"""
    def dagger(self) -> VariationalQuantumGate_iSWAP:
        """dagger(self: pyQPanda.VariationalQuantumGate_iSWAP) -> pyQPanda.VariationalQuantumGate_iSWAP"""

class em_method:
    __members__: ClassVar[dict] = ...  # read-only
    PEC: ClassVar[em_method] = ...
    READ_OUT: ClassVar[em_method] = ...
    ZNE: ClassVar[em_method] = ...
    __entries: ClassVar[dict] = ...
    def __init__(self, value: int) -> None:
        """__init__(self: pyQPanda.em_method, value: int) -> None"""
    def __eq__(self, other: object) -> bool:
        """__eq__(self: object, other: object) -> bool"""
    def __hash__(self) -> int:
        """__hash__(self: object) -> int"""
    def __index__(self) -> int:
        """__index__(self: pyQPanda.em_method) -> int"""
    def __int__(self) -> int:
        """__int__(self: pyQPanda.em_method) -> int"""
    def __ne__(self, other: object) -> bool:
        """__ne__(self: object, other: object) -> bool"""
    @property
    def name(self) -> str: ...
    @property
    def value(self) -> int: ...

class expression:
    def __init__(self, arg0: var) -> None:
        """__init__(self: pyQPanda.expression, arg0: pyQPanda.var) -> None"""
    @overload
    def backprop(self, arg0: dict[var, numpy.ndarray[numpy.float64[m, n]]]) -> None:
        """backprop(*args, **kwargs)
        Overloaded function.

        1. backprop(self: pyQPanda.expression, arg0: Dict[pyQPanda.var, numpy.ndarray[numpy.float64[m, n]]]) -> None

        2. backprop(self: pyQPanda.expression, arg0: Dict[pyQPanda.var, numpy.ndarray[numpy.float64[m, n]]], arg1: List[pyQPanda.var]) -> None
        """
    @overload
    def backprop(self, arg0: dict[var, numpy.ndarray[numpy.float64[m, n]]], arg1: list[var]) -> None:
        """backprop(*args, **kwargs)
        Overloaded function.

        1. backprop(self: pyQPanda.expression, arg0: Dict[pyQPanda.var, numpy.ndarray[numpy.float64[m, n]]]) -> None

        2. backprop(self: pyQPanda.expression, arg0: Dict[pyQPanda.var, numpy.ndarray[numpy.float64[m, n]]], arg1: List[pyQPanda.var]) -> None
        """
    def find_leaves(self) -> list[var]:
        """find_leaves(self: pyQPanda.expression) -> List[pyQPanda.var]"""
    def find_non_consts(self, arg0: list[var]) -> list[var]:
        """find_non_consts(self: pyQPanda.expression, arg0: List[pyQPanda.var]) -> List[pyQPanda.var]"""
    def get_root(self) -> var:
        """get_root(self: pyQPanda.expression) -> pyQPanda.var"""
    @overload
    def propagate(self) -> numpy.ndarray[numpy.float64[m, n]]:
        """propagate(*args, **kwargs)
        Overloaded function.

        1. propagate(self: pyQPanda.expression) -> numpy.ndarray[numpy.float64[m, n]]

        2. propagate(self: pyQPanda.expression, arg0: List[pyQPanda.var]) -> numpy.ndarray[numpy.float64[m, n]]
        """
    @overload
    def propagate(self, arg0: list[var]) -> numpy.ndarray[numpy.float64[m, n]]:
        """propagate(*args, **kwargs)
        Overloaded function.

        1. propagate(self: pyQPanda.expression) -> numpy.ndarray[numpy.float64[m, n]]

        2. propagate(self: pyQPanda.expression, arg0: List[pyQPanda.var]) -> numpy.ndarray[numpy.float64[m, n]]
        """

class hadamard_circuit(QCircuit):
    def __init__(self, arg0: QVec) -> None:
        """__init__(self: pyQPanda.hadamard_circuit, arg0: pyQPanda.QVec) -> None"""

class real_chip_type:
    __members__: ClassVar[dict] = ...  # read-only
    __entries: ClassVar[dict] = ...
    origin_72: ClassVar[real_chip_type] = ...
    origin_wuyuan_d3: ClassVar[real_chip_type] = ...
    origin_wuyuan_d4: ClassVar[real_chip_type] = ...
    origin_wuyuan_d5: ClassVar[real_chip_type] = ...
    def __init__(self, value: int) -> None:
        """__init__(self: pyQPanda.real_chip_type, value: int) -> None"""
    def __eq__(self, other: object) -> bool:
        """__eq__(self: object, other: object) -> bool"""
    def __hash__(self) -> int:
        """__hash__(self: object) -> int"""
    def __index__(self) -> int:
        """__index__(self: pyQPanda.real_chip_type) -> int"""
    def __int__(self) -> int:
        """__int__(self: pyQPanda.real_chip_type) -> int"""
    def __ne__(self, other: object) -> bool:
        """__ne__(self: object, other: object) -> bool"""
    @property
    def name(self) -> str: ...
    @property
    def value(self) -> int: ...

class var:
    @overload
    def __init__(self, arg0: float) -> None:
        """__init__(*args, **kwargs)
        Overloaded function.

        1. __init__(self: pyQPanda.var, arg0: float) -> None

        2. __init__(self: pyQPanda.var, arg0: numpy.ndarray[numpy.float64[m, n], flags.writeable]) -> None

        3. __init__(self: pyQPanda.var, arg0: float, arg1: bool) -> None

        4. __init__(self: pyQPanda.var, arg0: numpy.ndarray[numpy.float64[m, n], flags.writeable], arg1: bool) -> None
        """
    @overload
    def __init__(self, arg0: numpy.ndarray[numpy.float64[m, n], flags.writeable]) -> None:
        """__init__(*args, **kwargs)
        Overloaded function.

        1. __init__(self: pyQPanda.var, arg0: float) -> None

        2. __init__(self: pyQPanda.var, arg0: numpy.ndarray[numpy.float64[m, n], flags.writeable]) -> None

        3. __init__(self: pyQPanda.var, arg0: float, arg1: bool) -> None

        4. __init__(self: pyQPanda.var, arg0: numpy.ndarray[numpy.float64[m, n], flags.writeable], arg1: bool) -> None
        """
    @overload
    def __init__(self, arg0: float, arg1: bool) -> None:
        """__init__(*args, **kwargs)
        Overloaded function.

        1. __init__(self: pyQPanda.var, arg0: float) -> None

        2. __init__(self: pyQPanda.var, arg0: numpy.ndarray[numpy.float64[m, n], flags.writeable]) -> None

        3. __init__(self: pyQPanda.var, arg0: float, arg1: bool) -> None

        4. __init__(self: pyQPanda.var, arg0: numpy.ndarray[numpy.float64[m, n], flags.writeable], arg1: bool) -> None
        """
    @overload
    def __init__(self, arg0: numpy.ndarray[numpy.float64[m, n], flags.writeable], arg1: bool) -> None:
        """__init__(*args, **kwargs)
        Overloaded function.

        1. __init__(self: pyQPanda.var, arg0: float) -> None

        2. __init__(self: pyQPanda.var, arg0: numpy.ndarray[numpy.float64[m, n], flags.writeable]) -> None

        3. __init__(self: pyQPanda.var, arg0: float, arg1: bool) -> None

        4. __init__(self: pyQPanda.var, arg0: numpy.ndarray[numpy.float64[m, n], flags.writeable], arg1: bool) -> None
        """
    def clone(self) -> var:
        """clone(self: pyQPanda.var) -> pyQPanda.var"""
    def get_value(self) -> numpy.ndarray[numpy.float64[m, n]]:
        """get_value(self: pyQPanda.var) -> numpy.ndarray[numpy.float64[m, n]]"""
    @overload
    def set_value(self, arg0: numpy.ndarray[numpy.float64[m, n]]) -> None:
        """set_value(*args, **kwargs)
        Overloaded function.

        1. set_value(self: pyQPanda.var, arg0: numpy.ndarray[numpy.float64[m, n]]) -> None

        2. set_value(self: pyQPanda.var, arg0: float) -> None
        """
    @overload
    def set_value(self, arg0: float) -> None:
        """set_value(*args, **kwargs)
        Overloaded function.

        1. set_value(self: pyQPanda.var, arg0: numpy.ndarray[numpy.float64[m, n]]) -> None

        2. set_value(self: pyQPanda.var, arg0: float) -> None
        """
    @overload
    def __add__(self, arg0: var) -> var:
        """__add__(*args, **kwargs)
        Overloaded function.

        1. __add__(self: pyQPanda.var, arg0: pyQPanda.var) -> pyQPanda.var

        2. __add__(self: pyQPanda.var, arg0: float) -> pyQPanda.var
        """
    @overload
    def __add__(self, arg0: float) -> var:
        """__add__(*args, **kwargs)
        Overloaded function.

        1. __add__(self: pyQPanda.var, arg0: pyQPanda.var) -> pyQPanda.var

        2. __add__(self: pyQPanda.var, arg0: float) -> pyQPanda.var
        """
    @overload
    def __eq__(self, arg0: var) -> bool:
        """__eq__(*args, **kwargs)
        Overloaded function.

        1. __eq__(self: pyQPanda.var, arg0: pyQPanda.var) -> bool

        2. __eq__(self: pyQPanda.var, arg0: pyQPanda.var) -> bool
        """
    @overload
    def __eq__(self, arg0: var) -> bool:
        """__eq__(*args, **kwargs)
        Overloaded function.

        1. __eq__(self: pyQPanda.var, arg0: pyQPanda.var) -> bool

        2. __eq__(self: pyQPanda.var, arg0: pyQPanda.var) -> bool
        """
    def __getitem__(self, arg0: int) -> var:
        """__getitem__(self: pyQPanda.var, arg0: int) -> pyQPanda.var"""
    def __iter__(self) -> typing.Iterator[var]:
        """def __iter__(self) -> typing.Iterator[var]"""
    def __hash__(self) -> int:
        """__hash__(self: pyQPanda.var) -> int"""
    @overload
    def __mul__(self, arg0: var) -> var:
        """__mul__(*args, **kwargs)
        Overloaded function.

        1. __mul__(self: pyQPanda.var, arg0: pyQPanda.var) -> pyQPanda.var

        2. __mul__(self: pyQPanda.var, arg0: float) -> pyQPanda.var
        """
    @overload
    def __mul__(self, arg0: float) -> var:
        """__mul__(*args, **kwargs)
        Overloaded function.

        1. __mul__(self: pyQPanda.var, arg0: pyQPanda.var) -> pyQPanda.var

        2. __mul__(self: pyQPanda.var, arg0: float) -> pyQPanda.var
        """
    def __radd__(self, arg0: float) -> var:
        """__radd__(self: pyQPanda.var, arg0: float) -> pyQPanda.var"""
    def __rmul__(self, arg0: float) -> var:
        """__rmul__(self: pyQPanda.var, arg0: float) -> pyQPanda.var"""
    def __rsub__(self, arg0: float) -> var:
        """__rsub__(self: pyQPanda.var, arg0: float) -> pyQPanda.var"""
    def __rtruediv__(self, arg0: float) -> var:
        """__rtruediv__(self: pyQPanda.var, arg0: float) -> pyQPanda.var"""
    @overload
    def __sub__(self, arg0: var) -> var:
        """__sub__(*args, **kwargs)
        Overloaded function.

        1. __sub__(self: pyQPanda.var, arg0: pyQPanda.var) -> pyQPanda.var

        2. __sub__(self: pyQPanda.var, arg0: float) -> pyQPanda.var
        """
    @overload
    def __sub__(self, arg0: float) -> var:
        """__sub__(*args, **kwargs)
        Overloaded function.

        1. __sub__(self: pyQPanda.var, arg0: pyQPanda.var) -> pyQPanda.var

        2. __sub__(self: pyQPanda.var, arg0: float) -> pyQPanda.var
        """
    @overload
    def __truediv__(self, arg0: var) -> var:
        """__truediv__(*args, **kwargs)
        Overloaded function.

        1. __truediv__(self: pyQPanda.var, arg0: pyQPanda.var) -> pyQPanda.var

        2. __truediv__(self: pyQPanda.var, arg0: float) -> pyQPanda.var
        """
    @overload
    def __truediv__(self, arg0: float) -> var:
        """__truediv__(*args, **kwargs)
        Overloaded function.

        1. __truediv__(self: pyQPanda.var, arg0: pyQPanda.var) -> pyQPanda.var

        2. __truediv__(self: pyQPanda.var, arg0: float) -> pyQPanda.var
        """

@overload
def BARRIER(qubit: Qubit) -> QGate:
    """BARRIER(*args, **kwargs)
    Overloaded function.

    1. BARRIER(qubit: pyQPanda.Qubit) -> pyQPanda.QGate

    Create a BARRIER gate for a specified qubit.

    Args:
        qubit: the qubit to which the BARRIER will be applied.

    Returns:
        A BARRIER node representing the operation.


    2. BARRIER(qubit_list: int) -> pyQPanda.QGate

    Create a BARRIER gate for a list of qubits.

    Args:
        qubit_list: integer representing the qubits to which the BARRIER will be applied.

    Returns:
        A BARRIER node representing the operation.


    3. BARRIER(qubit_list: pyQPanda.QVec) -> pyQPanda.QGate

    Create a BARRIER gate for a list of qubits.

    Args:
        qubit_list: a list of qubits to which the BARRIER will be applied.

    Returns:
        A BARRIER node representing the operation.


    4. BARRIER(qubit_addr_list: List[int]) -> pyQPanda.QGate

    Create a BARRIER gate for a list of qubit addresses.

    Args:
        qubit_addr_list: a list of integers representing the addresses of the qubits.

    Returns:
        A BARRIER node representing the operation.

    """
@overload
def BARRIER(qubit_list: int) -> QGate:
    """BARRIER(*args, **kwargs)
    Overloaded function.

    1. BARRIER(qubit: pyQPanda.Qubit) -> pyQPanda.QGate

    Create a BARRIER gate for a specified qubit.

    Args:
        qubit: the qubit to which the BARRIER will be applied.

    Returns:
        A BARRIER node representing the operation.


    2. BARRIER(qubit_list: int) -> pyQPanda.QGate

    Create a BARRIER gate for a list of qubits.

    Args:
        qubit_list: integer representing the qubits to which the BARRIER will be applied.

    Returns:
        A BARRIER node representing the operation.


    3. BARRIER(qubit_list: pyQPanda.QVec) -> pyQPanda.QGate

    Create a BARRIER gate for a list of qubits.

    Args:
        qubit_list: a list of qubits to which the BARRIER will be applied.

    Returns:
        A BARRIER node representing the operation.


    4. BARRIER(qubit_addr_list: List[int]) -> pyQPanda.QGate

    Create a BARRIER gate for a list of qubit addresses.

    Args:
        qubit_addr_list: a list of integers representing the addresses of the qubits.

    Returns:
        A BARRIER node representing the operation.

    """
@overload
def BARRIER(qubit_list: QVec) -> QGate:
    """BARRIER(*args, **kwargs)
    Overloaded function.

    1. BARRIER(qubit: pyQPanda.Qubit) -> pyQPanda.QGate

    Create a BARRIER gate for a specified qubit.

    Args:
        qubit: the qubit to which the BARRIER will be applied.

    Returns:
        A BARRIER node representing the operation.


    2. BARRIER(qubit_list: int) -> pyQPanda.QGate

    Create a BARRIER gate for a list of qubits.

    Args:
        qubit_list: integer representing the qubits to which the BARRIER will be applied.

    Returns:
        A BARRIER node representing the operation.


    3. BARRIER(qubit_list: pyQPanda.QVec) -> pyQPanda.QGate

    Create a BARRIER gate for a list of qubits.

    Args:
        qubit_list: a list of qubits to which the BARRIER will be applied.

    Returns:
        A BARRIER node representing the operation.


    4. BARRIER(qubit_addr_list: List[int]) -> pyQPanda.QGate

    Create a BARRIER gate for a list of qubit addresses.

    Args:
        qubit_addr_list: a list of integers representing the addresses of the qubits.

    Returns:
        A BARRIER node representing the operation.

    """
@overload
def BARRIER(qubit_addr_list: list[int]) -> QGate:
    """BARRIER(*args, **kwargs)
    Overloaded function.

    1. BARRIER(qubit: pyQPanda.Qubit) -> pyQPanda.QGate

    Create a BARRIER gate for a specified qubit.

    Args:
        qubit: the qubit to which the BARRIER will be applied.

    Returns:
        A BARRIER node representing the operation.


    2. BARRIER(qubit_list: int) -> pyQPanda.QGate

    Create a BARRIER gate for a list of qubits.

    Args:
        qubit_list: integer representing the qubits to which the BARRIER will be applied.

    Returns:
        A BARRIER node representing the operation.


    3. BARRIER(qubit_list: pyQPanda.QVec) -> pyQPanda.QGate

    Create a BARRIER gate for a list of qubits.

    Args:
        qubit_list: a list of qubits to which the BARRIER will be applied.

    Returns:
        A BARRIER node representing the operation.


    4. BARRIER(qubit_addr_list: List[int]) -> pyQPanda.QGate

    Create a BARRIER gate for a list of qubit addresses.

    Args:
        qubit_addr_list: a list of integers representing the addresses of the qubits.

    Returns:
        A BARRIER node representing the operation.

    """
@overload
def CNOT(control_qubit: Qubit, target_qubit: Qubit) -> QGate:
    """CNOT(*args, **kwargs)
    Overloaded function.

    1. CNOT(control_qubit: pyQPanda.Qubit, target_qubit: pyQPanda.Qubit) -> pyQPanda.QGate

    Returns:
        a CNOT gate node
    Raises:
        run_fail: An error occurred in construct gate node


    2. CNOT(control_qubit_list: pyQPanda.QVec, target_qubit_list: pyQPanda.QVec) -> pyQPanda.QCircuit

    Returns:
        a CNOT gate node
    Raises:
        run_fail: An error occurred in construct gate node


    3. CNOT(control_qubit_addr: int, target_qubit_addr: int) -> pyQPanda.QGate

    4. CNOT(control_qubit_addr_list: List[int], target_qubit_addr_list: List[int]) -> pyQPanda.QCircuit

    Returns:
        a CNOT gate node
    Raises:
        run_fail: An error occurred in construct gate node

    """
@overload
def CNOT(control_qubit_list: QVec, target_qubit_list: QVec) -> QCircuit:
    """CNOT(*args, **kwargs)
    Overloaded function.

    1. CNOT(control_qubit: pyQPanda.Qubit, target_qubit: pyQPanda.Qubit) -> pyQPanda.QGate

    Returns:
        a CNOT gate node
    Raises:
        run_fail: An error occurred in construct gate node


    2. CNOT(control_qubit_list: pyQPanda.QVec, target_qubit_list: pyQPanda.QVec) -> pyQPanda.QCircuit

    Returns:
        a CNOT gate node
    Raises:
        run_fail: An error occurred in construct gate node


    3. CNOT(control_qubit_addr: int, target_qubit_addr: int) -> pyQPanda.QGate

    4. CNOT(control_qubit_addr_list: List[int], target_qubit_addr_list: List[int]) -> pyQPanda.QCircuit

    Returns:
        a CNOT gate node
    Raises:
        run_fail: An error occurred in construct gate node

    """
@overload
def CNOT(control_qubit_addr: int, target_qubit_addr: int) -> QGate:
    """CNOT(*args, **kwargs)
    Overloaded function.

    1. CNOT(control_qubit: pyQPanda.Qubit, target_qubit: pyQPanda.Qubit) -> pyQPanda.QGate

    Returns:
        a CNOT gate node
    Raises:
        run_fail: An error occurred in construct gate node


    2. CNOT(control_qubit_list: pyQPanda.QVec, target_qubit_list: pyQPanda.QVec) -> pyQPanda.QCircuit

    Returns:
        a CNOT gate node
    Raises:
        run_fail: An error occurred in construct gate node


    3. CNOT(control_qubit_addr: int, target_qubit_addr: int) -> pyQPanda.QGate

    4. CNOT(control_qubit_addr_list: List[int], target_qubit_addr_list: List[int]) -> pyQPanda.QCircuit

    Returns:
        a CNOT gate node
    Raises:
        run_fail: An error occurred in construct gate node

    """
@overload
def CNOT(control_qubit_addr_list: list[int], target_qubit_addr_list: list[int]) -> QCircuit:
    """CNOT(*args, **kwargs)
    Overloaded function.

    1. CNOT(control_qubit: pyQPanda.Qubit, target_qubit: pyQPanda.Qubit) -> pyQPanda.QGate

    Returns:
        a CNOT gate node
    Raises:
        run_fail: An error occurred in construct gate node


    2. CNOT(control_qubit_list: pyQPanda.QVec, target_qubit_list: pyQPanda.QVec) -> pyQPanda.QCircuit

    Returns:
        a CNOT gate node
    Raises:
        run_fail: An error occurred in construct gate node


    3. CNOT(control_qubit_addr: int, target_qubit_addr: int) -> pyQPanda.QGate

    4. CNOT(control_qubit_addr_list: List[int], target_qubit_addr_list: List[int]) -> pyQPanda.QCircuit

    Returns:
        a CNOT gate node
    Raises:
        run_fail: An error occurred in construct gate node

    """
@overload
def CP(control_qubit: Qubit, target_qubit: Qubit, theta_angle: float) -> QGate:
    """CP(*args, **kwargs)
    Overloaded function.

    1. CP(control_qubit: pyQPanda.Qubit, target_qubit: pyQPanda.Qubit, theta_angle: float) -> pyQPanda.QGate

    Returns:
        a CP gate node
    Raises:
        run_fail: An error occurred in construct gate node


    2. CP(control_qubit_list: pyQPanda.QVec, target_qubit_list: pyQPanda.QVec, theta_angle: float) -> pyQPanda.QCircuit

    Returns:
        a CP gate node
    Raises:
        run_fail: An error occurred in construct gate node


    3. CP(control_qubit_addr: int, target_qubit_addr: int, theta_angle: float) -> pyQPanda.QGate

    Returns:
        a CP gate node
    Raises:
        run_fail: An error occurred in construct gate node


    4. CP(control_qubit_addr_list: List[int], target_qubit_addr_list: List[int], theta_angle: float) -> pyQPanda.QCircuit

    Returns:
        a CP gate node
    Raises:
        run_fail: An error occurred in construct gate node

    """
@overload
def CP(control_qubit_list: QVec, target_qubit_list: QVec, theta_angle: float) -> QCircuit:
    """CP(*args, **kwargs)
    Overloaded function.

    1. CP(control_qubit: pyQPanda.Qubit, target_qubit: pyQPanda.Qubit, theta_angle: float) -> pyQPanda.QGate

    Returns:
        a CP gate node
    Raises:
        run_fail: An error occurred in construct gate node


    2. CP(control_qubit_list: pyQPanda.QVec, target_qubit_list: pyQPanda.QVec, theta_angle: float) -> pyQPanda.QCircuit

    Returns:
        a CP gate node
    Raises:
        run_fail: An error occurred in construct gate node


    3. CP(control_qubit_addr: int, target_qubit_addr: int, theta_angle: float) -> pyQPanda.QGate

    Returns:
        a CP gate node
    Raises:
        run_fail: An error occurred in construct gate node


    4. CP(control_qubit_addr_list: List[int], target_qubit_addr_list: List[int], theta_angle: float) -> pyQPanda.QCircuit

    Returns:
        a CP gate node
    Raises:
        run_fail: An error occurred in construct gate node

    """
@overload
def CP(control_qubit_addr: int, target_qubit_addr: int, theta_angle: float) -> QGate:
    """CP(*args, **kwargs)
    Overloaded function.

    1. CP(control_qubit: pyQPanda.Qubit, target_qubit: pyQPanda.Qubit, theta_angle: float) -> pyQPanda.QGate

    Returns:
        a CP gate node
    Raises:
        run_fail: An error occurred in construct gate node


    2. CP(control_qubit_list: pyQPanda.QVec, target_qubit_list: pyQPanda.QVec, theta_angle: float) -> pyQPanda.QCircuit

    Returns:
        a CP gate node
    Raises:
        run_fail: An error occurred in construct gate node


    3. CP(control_qubit_addr: int, target_qubit_addr: int, theta_angle: float) -> pyQPanda.QGate

    Returns:
        a CP gate node
    Raises:
        run_fail: An error occurred in construct gate node


    4. CP(control_qubit_addr_list: List[int], target_qubit_addr_list: List[int], theta_angle: float) -> pyQPanda.QCircuit

    Returns:
        a CP gate node
    Raises:
        run_fail: An error occurred in construct gate node

    """
@overload
def CP(control_qubit_addr_list: list[int], target_qubit_addr_list: list[int], theta_angle: float) -> QCircuit:
    """CP(*args, **kwargs)
    Overloaded function.

    1. CP(control_qubit: pyQPanda.Qubit, target_qubit: pyQPanda.Qubit, theta_angle: float) -> pyQPanda.QGate

    Returns:
        a CP gate node
    Raises:
        run_fail: An error occurred in construct gate node


    2. CP(control_qubit_list: pyQPanda.QVec, target_qubit_list: pyQPanda.QVec, theta_angle: float) -> pyQPanda.QCircuit

    Returns:
        a CP gate node
    Raises:
        run_fail: An error occurred in construct gate node


    3. CP(control_qubit_addr: int, target_qubit_addr: int, theta_angle: float) -> pyQPanda.QGate

    Returns:
        a CP gate node
    Raises:
        run_fail: An error occurred in construct gate node


    4. CP(control_qubit_addr_list: List[int], target_qubit_addr_list: List[int], theta_angle: float) -> pyQPanda.QCircuit

    Returns:
        a CP gate node
    Raises:
        run_fail: An error occurred in construct gate node

    """
@overload
def CR(control_qubit: Qubit, target_qubit: Qubit, theta_angle: float) -> QGate:
    """CR(*args, **kwargs)
    Overloaded function.

    1. CR(control_qubit: pyQPanda.Qubit, target_qubit: pyQPanda.Qubit, theta_angle: float) -> pyQPanda.QGate

    Returns:
        a CR gate node
    Raises:
        run_fail: An error occurred in construct gate node


    2. CR(control_qubit_list: pyQPanda.QVec, target_qubit_list: pyQPanda.QVec, theta_angle: float) -> pyQPanda.QCircuit

    Returns:
        a CR gate node
    Raises:
        run_fail: An error occurred in construct gate node


    3. CR(control_qubit_addr: int, target_qubit_addr: int, theta_angle: float) -> pyQPanda.QGate

    Returns:
        a CR gate node
    Raises:
        run_fail: An error occurred in construct gate node


    4. CR(control_qubit_addr_list: List[int], target_qubit_addr_list: List[int], theta_angle: float) -> pyQPanda.QCircuit

    Returns:
        a CR gate node
    Raises:
        run_fail: An error occurred in construct gate node

    """
@overload
def CR(control_qubit_list: QVec, target_qubit_list: QVec, theta_angle: float) -> QCircuit:
    """CR(*args, **kwargs)
    Overloaded function.

    1. CR(control_qubit: pyQPanda.Qubit, target_qubit: pyQPanda.Qubit, theta_angle: float) -> pyQPanda.QGate

    Returns:
        a CR gate node
    Raises:
        run_fail: An error occurred in construct gate node


    2. CR(control_qubit_list: pyQPanda.QVec, target_qubit_list: pyQPanda.QVec, theta_angle: float) -> pyQPanda.QCircuit

    Returns:
        a CR gate node
    Raises:
        run_fail: An error occurred in construct gate node


    3. CR(control_qubit_addr: int, target_qubit_addr: int, theta_angle: float) -> pyQPanda.QGate

    Returns:
        a CR gate node
    Raises:
        run_fail: An error occurred in construct gate node


    4. CR(control_qubit_addr_list: List[int], target_qubit_addr_list: List[int], theta_angle: float) -> pyQPanda.QCircuit

    Returns:
        a CR gate node
    Raises:
        run_fail: An error occurred in construct gate node

    """
@overload
def CR(control_qubit_addr: int, target_qubit_addr: int, theta_angle: float) -> QGate:
    """CR(*args, **kwargs)
    Overloaded function.

    1. CR(control_qubit: pyQPanda.Qubit, target_qubit: pyQPanda.Qubit, theta_angle: float) -> pyQPanda.QGate

    Returns:
        a CR gate node
    Raises:
        run_fail: An error occurred in construct gate node


    2. CR(control_qubit_list: pyQPanda.QVec, target_qubit_list: pyQPanda.QVec, theta_angle: float) -> pyQPanda.QCircuit

    Returns:
        a CR gate node
    Raises:
        run_fail: An error occurred in construct gate node


    3. CR(control_qubit_addr: int, target_qubit_addr: int, theta_angle: float) -> pyQPanda.QGate

    Returns:
        a CR gate node
    Raises:
        run_fail: An error occurred in construct gate node


    4. CR(control_qubit_addr_list: List[int], target_qubit_addr_list: List[int], theta_angle: float) -> pyQPanda.QCircuit

    Returns:
        a CR gate node
    Raises:
        run_fail: An error occurred in construct gate node

    """
@overload
def CR(control_qubit_addr_list: list[int], target_qubit_addr_list: list[int], theta_angle: float) -> QCircuit:
    """CR(*args, **kwargs)
    Overloaded function.

    1. CR(control_qubit: pyQPanda.Qubit, target_qubit: pyQPanda.Qubit, theta_angle: float) -> pyQPanda.QGate

    Returns:
        a CR gate node
    Raises:
        run_fail: An error occurred in construct gate node


    2. CR(control_qubit_list: pyQPanda.QVec, target_qubit_list: pyQPanda.QVec, theta_angle: float) -> pyQPanda.QCircuit

    Returns:
        a CR gate node
    Raises:
        run_fail: An error occurred in construct gate node


    3. CR(control_qubit_addr: int, target_qubit_addr: int, theta_angle: float) -> pyQPanda.QGate

    Returns:
        a CR gate node
    Raises:
        run_fail: An error occurred in construct gate node


    4. CR(control_qubit_addr_list: List[int], target_qubit_addr_list: List[int], theta_angle: float) -> pyQPanda.QCircuit

    Returns:
        a CR gate node
    Raises:
        run_fail: An error occurred in construct gate node

    """
@overload
def CU(alpha_angle: float, beta_angle: float, gamma_angle: float, delta_angle: float, control_qubit: Qubit, target_qubit: Qubit) -> QGate:
    """CU(*args, **kwargs)
    Overloaded function.

    1. CU(alpha_angle: float, beta_angle: float, gamma_angle: float, delta_angle: float, control_qubit: pyQPanda.Qubit, target_qubit: pyQPanda.Qubit) -> pyQPanda.QGate

    Create a CU gate.

    Args:
        alpha_angle: the alpha angle for the CU gate.

        beta_angle: the beta angle for the CU gate.

        gamma_angle: the gamma angle for the CU gate.

        delta_angle: the delta angle for the CU gate.

        control_qubit: the qubit that controls the operation.

        target_qubit: the qubit that is affected by the control qubit.

    Returns:
        A CU node representing the operation.

    2. CU(alpha_angle: float, beta_angle: float, gamma_angle: float, delta_angle: float, control_qubit_list: pyQPanda.QVec, target_qubi_list: pyQPanda.QVec) -> pyQPanda.QCircuit

    Create a CU gate.

    Args:
        alpha_angle (double): U4 gate alpha angle.

        beta_angle (double): U4 gate beta angle.

        gamma_angle (double): U4 gate gamma angle.

        delta_angle (double): U4 gate delta angle.

        control_qubit_list (const QVec &): List of control qubits.

        target_qubit_list (const QVec &): List of target qubits.

    Returns:
        A CU node representing the operation.


    3. CU(matrix: List[complex], control_qubit: pyQPanda.Qubit, target_qubit: pyQPanda.Qubit) -> pyQPanda.QGate

    Create a CU gate.

    Args:
        matrix (QStat &): The CU gate matrix.

        control_qubit (Qubit *): The control qubit.

        target_qubit (Qubit *): The target qubit.

    Returns:
        A CU node representing the operation.


    4. CU(matrix: List[complex], control_qubit_list: pyQPanda.QVec, target_qubit_list: pyQPanda.QVec) -> pyQPanda.QCircuit

    Create a CU gate.

    Args:
        matrix (QStat &): The CU gate matrix.

        control_qubit_list (const QVec &): List of control qubits.

        target_qubit_list (const QVec &): List of target qubits.

    Returns:
        A CU node representing the operation.


    5. CU(control_qubit: pyQPanda.Qubit, target_qubit: pyQPanda.Qubit, alpha_angle: float, beta_angle: float, gamma_angle: float, delta_angle: float) -> pyQPanda.QGate

    Create a CU gate.

    Args:
        control_qubit (Qubit *): The control qubit.

        target_qubit (Qubit *): The target qubit.

        alpha_angle (double): U4 gate alpha angle.

        beta_angle (double): U4 gate beta angle.

        gamma_angle (double): U4 gate gamma angle.

        delta_angle (double): U4 gate delta angle.

    Returns:
        A CU node representing the operation.


    6. CU(control_qubit_list: pyQPanda.QVec, target_qubit_list: pyQPanda.QVec, alpha_angle: float, beta_angle: float, gamma_angle: float, delta_angle: float) -> pyQPanda.QCircuit

    Create a CU gate.

    Args:
        control_qubit_list (QVec): Control qubit list.

        target_qubit_list (QVec): Target qubit list.

        alpha_angle (double): U4 gate alpha angle.

        beta_angle (double): U4 gate beta angle.

        gamma_angle (double): U4 gate gamma angle.

        delta_angle (double): U4 gate delta angle.

    Returns:
        CU node representing the operation.


    7. CU(control_qubit_addr: int, target_qubit_addr: int, alpha_angle: float, beta_angle: float, gamma_angle: float, delta_angle: float) -> pyQPanda.QGate

    Create a CU gate.

    Args:
        control_qubit_addr (int): Address of the control qubit.

        target_qubit_addr (int): Address of the target qubit.

        alpha_angle (double): U4 gate alpha angle.

        beta_angle (double): U4 gate beta angle.

        gamma_angle (double): U4 gate gamma angle.

        delta_angle (double): U4 gate delta angle.

    Returns:
        A CU node representing the operation.


    8. CU(control_qubit_addr_list: List[int], target_qubit_addr_list: List[int], alpha_angle: float, beta_angle: float, gamma_angle: float, delta_angle: float) -> pyQPanda.QCircuit

    Create a CU gate.

    Args:
        control_qubit_addr_list (std::vector<int>): List of control qubit addresses.

        target_qubit_addr_list (std::vector<int>): List of target qubit addresses.

        alpha_angle (double): U4 gate alpha angle.

        beta_angle (double): U4 gate beta angle.

        gamma_angle (double): U4 gate gamma angle.

        delta_angle (double): U4 gate delta angle.

    Returns:
        A CU node representing the operation.


    9. CU(control_qubit: pyQPanda.Qubit, target_qubit: pyQPanda.Qubit, matrix: List[complex]) -> pyQPanda.QGate

    Create a CU gate.

    Args:
        control_qubit (Qubit *): The control qubit.

        target_qubit (Qubit *): The target qubit.

        matrix (QStat &): The CU gate matrix.

    Returns:
        A CU node representing the operation.


    10. CU(control_qubit_list: pyQPanda.QVec, target_qubit_list: pyQPanda.QVec, matrix: List[complex]) -> pyQPanda.QCircuit

    Create a CU gate.

    Args:
        control_qubit_list (const QVec &): List of control qubits.

        target_qubit_list (const QVec &): List of target qubits.

        matrix (QStat &): The CU gate matrix.

    Returns:
        A CU node representing the operation.


    11. CU(control_qubit_addr: int, target_qubit_addr: int, matrix: List[complex]) -> pyQPanda.QGate

    Create a CU gate.

    Args:
        control_qubit_addr (int): Address of the control qubit.

        target_qubit_addr (int): Address of the target qubit.

        matrix (QStat &): The CU gate matrix.

    Returns:
        A CU node representing the operation.


    12. CU(control_qubit_addr_list: List[int], target_qubit_addr_list: List[int], matrix: List[complex]) -> pyQPanda.QCircuit

    Create a CU gate.

    Args:
        control_qubit_addr_list (const std::vector<int> &): List of control qubit addresses.

        target_qubit_addr_list (const std::vector<int> &): List of target qubit addresses.

        matrix (QStat &): The CU gate matrix.

    Returns:
        A CU node representing the operation.

    """
@overload
def CU(alpha_angle: float, beta_angle: float, gamma_angle: float, delta_angle: float, control_qubit_list: QVec, target_qubi_list: QVec) -> QCircuit:
    """CU(*args, **kwargs)
    Overloaded function.

    1. CU(alpha_angle: float, beta_angle: float, gamma_angle: float, delta_angle: float, control_qubit: pyQPanda.Qubit, target_qubit: pyQPanda.Qubit) -> pyQPanda.QGate

    Create a CU gate.

    Args:
        alpha_angle: the alpha angle for the CU gate.

        beta_angle: the beta angle for the CU gate.

        gamma_angle: the gamma angle for the CU gate.

        delta_angle: the delta angle for the CU gate.

        control_qubit: the qubit that controls the operation.

        target_qubit: the qubit that is affected by the control qubit.

    Returns:
        A CU node representing the operation.

    2. CU(alpha_angle: float, beta_angle: float, gamma_angle: float, delta_angle: float, control_qubit_list: pyQPanda.QVec, target_qubi_list: pyQPanda.QVec) -> pyQPanda.QCircuit

    Create a CU gate.

    Args:
        alpha_angle (double): U4 gate alpha angle.

        beta_angle (double): U4 gate beta angle.

        gamma_angle (double): U4 gate gamma angle.

        delta_angle (double): U4 gate delta angle.

        control_qubit_list (const QVec &): List of control qubits.

        target_qubit_list (const QVec &): List of target qubits.

    Returns:
        A CU node representing the operation.


    3. CU(matrix: List[complex], control_qubit: pyQPanda.Qubit, target_qubit: pyQPanda.Qubit) -> pyQPanda.QGate

    Create a CU gate.

    Args:
        matrix (QStat &): The CU gate matrix.

        control_qubit (Qubit *): The control qubit.

        target_qubit (Qubit *): The target qubit.

    Returns:
        A CU node representing the operation.


    4. CU(matrix: List[complex], control_qubit_list: pyQPanda.QVec, target_qubit_list: pyQPanda.QVec) -> pyQPanda.QCircuit

    Create a CU gate.

    Args:
        matrix (QStat &): The CU gate matrix.

        control_qubit_list (const QVec &): List of control qubits.

        target_qubit_list (const QVec &): List of target qubits.

    Returns:
        A CU node representing the operation.


    5. CU(control_qubit: pyQPanda.Qubit, target_qubit: pyQPanda.Qubit, alpha_angle: float, beta_angle: float, gamma_angle: float, delta_angle: float) -> pyQPanda.QGate

    Create a CU gate.

    Args:
        control_qubit (Qubit *): The control qubit.

        target_qubit (Qubit *): The target qubit.

        alpha_angle (double): U4 gate alpha angle.

        beta_angle (double): U4 gate beta angle.

        gamma_angle (double): U4 gate gamma angle.

        delta_angle (double): U4 gate delta angle.

    Returns:
        A CU node representing the operation.


    6. CU(control_qubit_list: pyQPanda.QVec, target_qubit_list: pyQPanda.QVec, alpha_angle: float, beta_angle: float, gamma_angle: float, delta_angle: float) -> pyQPanda.QCircuit

    Create a CU gate.

    Args:
        control_qubit_list (QVec): Control qubit list.

        target_qubit_list (QVec): Target qubit list.

        alpha_angle (double): U4 gate alpha angle.

        beta_angle (double): U4 gate beta angle.

        gamma_angle (double): U4 gate gamma angle.

        delta_angle (double): U4 gate delta angle.

    Returns:
        CU node representing the operation.


    7. CU(control_qubit_addr: int, target_qubit_addr: int, alpha_angle: float, beta_angle: float, gamma_angle: float, delta_angle: float) -> pyQPanda.QGate

    Create a CU gate.

    Args:
        control_qubit_addr (int): Address of the control qubit.

        target_qubit_addr (int): Address of the target qubit.

        alpha_angle (double): U4 gate alpha angle.

        beta_angle (double): U4 gate beta angle.

        gamma_angle (double): U4 gate gamma angle.

        delta_angle (double): U4 gate delta angle.

    Returns:
        A CU node representing the operation.


    8. CU(control_qubit_addr_list: List[int], target_qubit_addr_list: List[int], alpha_angle: float, beta_angle: float, gamma_angle: float, delta_angle: float) -> pyQPanda.QCircuit

    Create a CU gate.

    Args:
        control_qubit_addr_list (std::vector<int>): List of control qubit addresses.

        target_qubit_addr_list (std::vector<int>): List of target qubit addresses.

        alpha_angle (double): U4 gate alpha angle.

        beta_angle (double): U4 gate beta angle.

        gamma_angle (double): U4 gate gamma angle.

        delta_angle (double): U4 gate delta angle.

    Returns:
        A CU node representing the operation.


    9. CU(control_qubit: pyQPanda.Qubit, target_qubit: pyQPanda.Qubit, matrix: List[complex]) -> pyQPanda.QGate

    Create a CU gate.

    Args:
        control_qubit (Qubit *): The control qubit.

        target_qubit (Qubit *): The target qubit.

        matrix (QStat &): The CU gate matrix.

    Returns:
        A CU node representing the operation.


    10. CU(control_qubit_list: pyQPanda.QVec, target_qubit_list: pyQPanda.QVec, matrix: List[complex]) -> pyQPanda.QCircuit

    Create a CU gate.

    Args:
        control_qubit_list (const QVec &): List of control qubits.

        target_qubit_list (const QVec &): List of target qubits.

        matrix (QStat &): The CU gate matrix.

    Returns:
        A CU node representing the operation.


    11. CU(control_qubit_addr: int, target_qubit_addr: int, matrix: List[complex]) -> pyQPanda.QGate

    Create a CU gate.

    Args:
        control_qubit_addr (int): Address of the control qubit.

        target_qubit_addr (int): Address of the target qubit.

        matrix (QStat &): The CU gate matrix.

    Returns:
        A CU node representing the operation.


    12. CU(control_qubit_addr_list: List[int], target_qubit_addr_list: List[int], matrix: List[complex]) -> pyQPanda.QCircuit

    Create a CU gate.

    Args:
        control_qubit_addr_list (const std::vector<int> &): List of control qubit addresses.

        target_qubit_addr_list (const std::vector<int> &): List of target qubit addresses.

        matrix (QStat &): The CU gate matrix.

    Returns:
        A CU node representing the operation.

    """
@overload
def CU(matrix: list[complex], control_qubit: Qubit, target_qubit: Qubit) -> QGate:
    """CU(*args, **kwargs)
    Overloaded function.

    1. CU(alpha_angle: float, beta_angle: float, gamma_angle: float, delta_angle: float, control_qubit: pyQPanda.Qubit, target_qubit: pyQPanda.Qubit) -> pyQPanda.QGate

    Create a CU gate.

    Args:
        alpha_angle: the alpha angle for the CU gate.

        beta_angle: the beta angle for the CU gate.

        gamma_angle: the gamma angle for the CU gate.

        delta_angle: the delta angle for the CU gate.

        control_qubit: the qubit that controls the operation.

        target_qubit: the qubit that is affected by the control qubit.

    Returns:
        A CU node representing the operation.

    2. CU(alpha_angle: float, beta_angle: float, gamma_angle: float, delta_angle: float, control_qubit_list: pyQPanda.QVec, target_qubi_list: pyQPanda.QVec) -> pyQPanda.QCircuit

    Create a CU gate.

    Args:
        alpha_angle (double): U4 gate alpha angle.

        beta_angle (double): U4 gate beta angle.

        gamma_angle (double): U4 gate gamma angle.

        delta_angle (double): U4 gate delta angle.

        control_qubit_list (const QVec &): List of control qubits.

        target_qubit_list (const QVec &): List of target qubits.

    Returns:
        A CU node representing the operation.


    3. CU(matrix: List[complex], control_qubit: pyQPanda.Qubit, target_qubit: pyQPanda.Qubit) -> pyQPanda.QGate

    Create a CU gate.

    Args:
        matrix (QStat &): The CU gate matrix.

        control_qubit (Qubit *): The control qubit.

        target_qubit (Qubit *): The target qubit.

    Returns:
        A CU node representing the operation.


    4. CU(matrix: List[complex], control_qubit_list: pyQPanda.QVec, target_qubit_list: pyQPanda.QVec) -> pyQPanda.QCircuit

    Create a CU gate.

    Args:
        matrix (QStat &): The CU gate matrix.

        control_qubit_list (const QVec &): List of control qubits.

        target_qubit_list (const QVec &): List of target qubits.

    Returns:
        A CU node representing the operation.


    5. CU(control_qubit: pyQPanda.Qubit, target_qubit: pyQPanda.Qubit, alpha_angle: float, beta_angle: float, gamma_angle: float, delta_angle: float) -> pyQPanda.QGate

    Create a CU gate.

    Args:
        control_qubit (Qubit *): The control qubit.

        target_qubit (Qubit *): The target qubit.

        alpha_angle (double): U4 gate alpha angle.

        beta_angle (double): U4 gate beta angle.

        gamma_angle (double): U4 gate gamma angle.

        delta_angle (double): U4 gate delta angle.

    Returns:
        A CU node representing the operation.


    6. CU(control_qubit_list: pyQPanda.QVec, target_qubit_list: pyQPanda.QVec, alpha_angle: float, beta_angle: float, gamma_angle: float, delta_angle: float) -> pyQPanda.QCircuit

    Create a CU gate.

    Args:
        control_qubit_list (QVec): Control qubit list.

        target_qubit_list (QVec): Target qubit list.

        alpha_angle (double): U4 gate alpha angle.

        beta_angle (double): U4 gate beta angle.

        gamma_angle (double): U4 gate gamma angle.

        delta_angle (double): U4 gate delta angle.

    Returns:
        CU node representing the operation.


    7. CU(control_qubit_addr: int, target_qubit_addr: int, alpha_angle: float, beta_angle: float, gamma_angle: float, delta_angle: float) -> pyQPanda.QGate

    Create a CU gate.

    Args:
        control_qubit_addr (int): Address of the control qubit.

        target_qubit_addr (int): Address of the target qubit.

        alpha_angle (double): U4 gate alpha angle.

        beta_angle (double): U4 gate beta angle.

        gamma_angle (double): U4 gate gamma angle.

        delta_angle (double): U4 gate delta angle.

    Returns:
        A CU node representing the operation.


    8. CU(control_qubit_addr_list: List[int], target_qubit_addr_list: List[int], alpha_angle: float, beta_angle: float, gamma_angle: float, delta_angle: float) -> pyQPanda.QCircuit

    Create a CU gate.

    Args:
        control_qubit_addr_list (std::vector<int>): List of control qubit addresses.

        target_qubit_addr_list (std::vector<int>): List of target qubit addresses.

        alpha_angle (double): U4 gate alpha angle.

        beta_angle (double): U4 gate beta angle.

        gamma_angle (double): U4 gate gamma angle.

        delta_angle (double): U4 gate delta angle.

    Returns:
        A CU node representing the operation.


    9. CU(control_qubit: pyQPanda.Qubit, target_qubit: pyQPanda.Qubit, matrix: List[complex]) -> pyQPanda.QGate

    Create a CU gate.

    Args:
        control_qubit (Qubit *): The control qubit.

        target_qubit (Qubit *): The target qubit.

        matrix (QStat &): The CU gate matrix.

    Returns:
        A CU node representing the operation.


    10. CU(control_qubit_list: pyQPanda.QVec, target_qubit_list: pyQPanda.QVec, matrix: List[complex]) -> pyQPanda.QCircuit

    Create a CU gate.

    Args:
        control_qubit_list (const QVec &): List of control qubits.

        target_qubit_list (const QVec &): List of target qubits.

        matrix (QStat &): The CU gate matrix.

    Returns:
        A CU node representing the operation.


    11. CU(control_qubit_addr: int, target_qubit_addr: int, matrix: List[complex]) -> pyQPanda.QGate

    Create a CU gate.

    Args:
        control_qubit_addr (int): Address of the control qubit.

        target_qubit_addr (int): Address of the target qubit.

        matrix (QStat &): The CU gate matrix.

    Returns:
        A CU node representing the operation.


    12. CU(control_qubit_addr_list: List[int], target_qubit_addr_list: List[int], matrix: List[complex]) -> pyQPanda.QCircuit

    Create a CU gate.

    Args:
        control_qubit_addr_list (const std::vector<int> &): List of control qubit addresses.

        target_qubit_addr_list (const std::vector<int> &): List of target qubit addresses.

        matrix (QStat &): The CU gate matrix.

    Returns:
        A CU node representing the operation.

    """
@overload
def CU(matrix: list[complex], control_qubit_list: QVec, target_qubit_list: QVec) -> QCircuit:
    """CU(*args, **kwargs)
    Overloaded function.

    1. CU(alpha_angle: float, beta_angle: float, gamma_angle: float, delta_angle: float, control_qubit: pyQPanda.Qubit, target_qubit: pyQPanda.Qubit) -> pyQPanda.QGate

    Create a CU gate.

    Args:
        alpha_angle: the alpha angle for the CU gate.

        beta_angle: the beta angle for the CU gate.

        gamma_angle: the gamma angle for the CU gate.

        delta_angle: the delta angle for the CU gate.

        control_qubit: the qubit that controls the operation.

        target_qubit: the qubit that is affected by the control qubit.

    Returns:
        A CU node representing the operation.

    2. CU(alpha_angle: float, beta_angle: float, gamma_angle: float, delta_angle: float, control_qubit_list: pyQPanda.QVec, target_qubi_list: pyQPanda.QVec) -> pyQPanda.QCircuit

    Create a CU gate.

    Args:
        alpha_angle (double): U4 gate alpha angle.

        beta_angle (double): U4 gate beta angle.

        gamma_angle (double): U4 gate gamma angle.

        delta_angle (double): U4 gate delta angle.

        control_qubit_list (const QVec &): List of control qubits.

        target_qubit_list (const QVec &): List of target qubits.

    Returns:
        A CU node representing the operation.


    3. CU(matrix: List[complex], control_qubit: pyQPanda.Qubit, target_qubit: pyQPanda.Qubit) -> pyQPanda.QGate

    Create a CU gate.

    Args:
        matrix (QStat &): The CU gate matrix.

        control_qubit (Qubit *): The control qubit.

        target_qubit (Qubit *): The target qubit.

    Returns:
        A CU node representing the operation.


    4. CU(matrix: List[complex], control_qubit_list: pyQPanda.QVec, target_qubit_list: pyQPanda.QVec) -> pyQPanda.QCircuit

    Create a CU gate.

    Args:
        matrix (QStat &): The CU gate matrix.

        control_qubit_list (const QVec &): List of control qubits.

        target_qubit_list (const QVec &): List of target qubits.

    Returns:
        A CU node representing the operation.


    5. CU(control_qubit: pyQPanda.Qubit, target_qubit: pyQPanda.Qubit, alpha_angle: float, beta_angle: float, gamma_angle: float, delta_angle: float) -> pyQPanda.QGate

    Create a CU gate.

    Args:
        control_qubit (Qubit *): The control qubit.

        target_qubit (Qubit *): The target qubit.

        alpha_angle (double): U4 gate alpha angle.

        beta_angle (double): U4 gate beta angle.

        gamma_angle (double): U4 gate gamma angle.

        delta_angle (double): U4 gate delta angle.

    Returns:
        A CU node representing the operation.


    6. CU(control_qubit_list: pyQPanda.QVec, target_qubit_list: pyQPanda.QVec, alpha_angle: float, beta_angle: float, gamma_angle: float, delta_angle: float) -> pyQPanda.QCircuit

    Create a CU gate.

    Args:
        control_qubit_list (QVec): Control qubit list.

        target_qubit_list (QVec): Target qubit list.

        alpha_angle (double): U4 gate alpha angle.

        beta_angle (double): U4 gate beta angle.

        gamma_angle (double): U4 gate gamma angle.

        delta_angle (double): U4 gate delta angle.

    Returns:
        CU node representing the operation.


    7. CU(control_qubit_addr: int, target_qubit_addr: int, alpha_angle: float, beta_angle: float, gamma_angle: float, delta_angle: float) -> pyQPanda.QGate

    Create a CU gate.

    Args:
        control_qubit_addr (int): Address of the control qubit.

        target_qubit_addr (int): Address of the target qubit.

        alpha_angle (double): U4 gate alpha angle.

        beta_angle (double): U4 gate beta angle.

        gamma_angle (double): U4 gate gamma angle.

        delta_angle (double): U4 gate delta angle.

    Returns:
        A CU node representing the operation.


    8. CU(control_qubit_addr_list: List[int], target_qubit_addr_list: List[int], alpha_angle: float, beta_angle: float, gamma_angle: float, delta_angle: float) -> pyQPanda.QCircuit

    Create a CU gate.

    Args:
        control_qubit_addr_list (std::vector<int>): List of control qubit addresses.

        target_qubit_addr_list (std::vector<int>): List of target qubit addresses.

        alpha_angle (double): U4 gate alpha angle.

        beta_angle (double): U4 gate beta angle.

        gamma_angle (double): U4 gate gamma angle.

        delta_angle (double): U4 gate delta angle.

    Returns:
        A CU node representing the operation.


    9. CU(control_qubit: pyQPanda.Qubit, target_qubit: pyQPanda.Qubit, matrix: List[complex]) -> pyQPanda.QGate

    Create a CU gate.

    Args:
        control_qubit (Qubit *): The control qubit.

        target_qubit (Qubit *): The target qubit.

        matrix (QStat &): The CU gate matrix.

    Returns:
        A CU node representing the operation.


    10. CU(control_qubit_list: pyQPanda.QVec, target_qubit_list: pyQPanda.QVec, matrix: List[complex]) -> pyQPanda.QCircuit

    Create a CU gate.

    Args:
        control_qubit_list (const QVec &): List of control qubits.

        target_qubit_list (const QVec &): List of target qubits.

        matrix (QStat &): The CU gate matrix.

    Returns:
        A CU node representing the operation.


    11. CU(control_qubit_addr: int, target_qubit_addr: int, matrix: List[complex]) -> pyQPanda.QGate

    Create a CU gate.

    Args:
        control_qubit_addr (int): Address of the control qubit.

        target_qubit_addr (int): Address of the target qubit.

        matrix (QStat &): The CU gate matrix.

    Returns:
        A CU node representing the operation.


    12. CU(control_qubit_addr_list: List[int], target_qubit_addr_list: List[int], matrix: List[complex]) -> pyQPanda.QCircuit

    Create a CU gate.

    Args:
        control_qubit_addr_list (const std::vector<int> &): List of control qubit addresses.

        target_qubit_addr_list (const std::vector<int> &): List of target qubit addresses.

        matrix (QStat &): The CU gate matrix.

    Returns:
        A CU node representing the operation.

    """
@overload
def CU(control_qubit: Qubit, target_qubit: Qubit, alpha_angle: float, beta_angle: float, gamma_angle: float, delta_angle: float) -> QGate:
    """CU(*args, **kwargs)
    Overloaded function.

    1. CU(alpha_angle: float, beta_angle: float, gamma_angle: float, delta_angle: float, control_qubit: pyQPanda.Qubit, target_qubit: pyQPanda.Qubit) -> pyQPanda.QGate

    Create a CU gate.

    Args:
        alpha_angle: the alpha angle for the CU gate.

        beta_angle: the beta angle for the CU gate.

        gamma_angle: the gamma angle for the CU gate.

        delta_angle: the delta angle for the CU gate.

        control_qubit: the qubit that controls the operation.

        target_qubit: the qubit that is affected by the control qubit.

    Returns:
        A CU node representing the operation.

    2. CU(alpha_angle: float, beta_angle: float, gamma_angle: float, delta_angle: float, control_qubit_list: pyQPanda.QVec, target_qubi_list: pyQPanda.QVec) -> pyQPanda.QCircuit

    Create a CU gate.

    Args:
        alpha_angle (double): U4 gate alpha angle.

        beta_angle (double): U4 gate beta angle.

        gamma_angle (double): U4 gate gamma angle.

        delta_angle (double): U4 gate delta angle.

        control_qubit_list (const QVec &): List of control qubits.

        target_qubit_list (const QVec &): List of target qubits.

    Returns:
        A CU node representing the operation.


    3. CU(matrix: List[complex], control_qubit: pyQPanda.Qubit, target_qubit: pyQPanda.Qubit) -> pyQPanda.QGate

    Create a CU gate.

    Args:
        matrix (QStat &): The CU gate matrix.

        control_qubit (Qubit *): The control qubit.

        target_qubit (Qubit *): The target qubit.

    Returns:
        A CU node representing the operation.


    4. CU(matrix: List[complex], control_qubit_list: pyQPanda.QVec, target_qubit_list: pyQPanda.QVec) -> pyQPanda.QCircuit

    Create a CU gate.

    Args:
        matrix (QStat &): The CU gate matrix.

        control_qubit_list (const QVec &): List of control qubits.

        target_qubit_list (const QVec &): List of target qubits.

    Returns:
        A CU node representing the operation.


    5. CU(control_qubit: pyQPanda.Qubit, target_qubit: pyQPanda.Qubit, alpha_angle: float, beta_angle: float, gamma_angle: float, delta_angle: float) -> pyQPanda.QGate

    Create a CU gate.

    Args:
        control_qubit (Qubit *): The control qubit.

        target_qubit (Qubit *): The target qubit.

        alpha_angle (double): U4 gate alpha angle.

        beta_angle (double): U4 gate beta angle.

        gamma_angle (double): U4 gate gamma angle.

        delta_angle (double): U4 gate delta angle.

    Returns:
        A CU node representing the operation.


    6. CU(control_qubit_list: pyQPanda.QVec, target_qubit_list: pyQPanda.QVec, alpha_angle: float, beta_angle: float, gamma_angle: float, delta_angle: float) -> pyQPanda.QCircuit

    Create a CU gate.

    Args:
        control_qubit_list (QVec): Control qubit list.

        target_qubit_list (QVec): Target qubit list.

        alpha_angle (double): U4 gate alpha angle.

        beta_angle (double): U4 gate beta angle.

        gamma_angle (double): U4 gate gamma angle.

        delta_angle (double): U4 gate delta angle.

    Returns:
        CU node representing the operation.


    7. CU(control_qubit_addr: int, target_qubit_addr: int, alpha_angle: float, beta_angle: float, gamma_angle: float, delta_angle: float) -> pyQPanda.QGate

    Create a CU gate.

    Args:
        control_qubit_addr (int): Address of the control qubit.

        target_qubit_addr (int): Address of the target qubit.

        alpha_angle (double): U4 gate alpha angle.

        beta_angle (double): U4 gate beta angle.

        gamma_angle (double): U4 gate gamma angle.

        delta_angle (double): U4 gate delta angle.

    Returns:
        A CU node representing the operation.


    8. CU(control_qubit_addr_list: List[int], target_qubit_addr_list: List[int], alpha_angle: float, beta_angle: float, gamma_angle: float, delta_angle: float) -> pyQPanda.QCircuit

    Create a CU gate.

    Args:
        control_qubit_addr_list (std::vector<int>): List of control qubit addresses.

        target_qubit_addr_list (std::vector<int>): List of target qubit addresses.

        alpha_angle (double): U4 gate alpha angle.

        beta_angle (double): U4 gate beta angle.

        gamma_angle (double): U4 gate gamma angle.

        delta_angle (double): U4 gate delta angle.

    Returns:
        A CU node representing the operation.


    9. CU(control_qubit: pyQPanda.Qubit, target_qubit: pyQPanda.Qubit, matrix: List[complex]) -> pyQPanda.QGate

    Create a CU gate.

    Args:
        control_qubit (Qubit *): The control qubit.

        target_qubit (Qubit *): The target qubit.

        matrix (QStat &): The CU gate matrix.

    Returns:
        A CU node representing the operation.


    10. CU(control_qubit_list: pyQPanda.QVec, target_qubit_list: pyQPanda.QVec, matrix: List[complex]) -> pyQPanda.QCircuit

    Create a CU gate.

    Args:
        control_qubit_list (const QVec &): List of control qubits.

        target_qubit_list (const QVec &): List of target qubits.

        matrix (QStat &): The CU gate matrix.

    Returns:
        A CU node representing the operation.


    11. CU(control_qubit_addr: int, target_qubit_addr: int, matrix: List[complex]) -> pyQPanda.QGate

    Create a CU gate.

    Args:
        control_qubit_addr (int): Address of the control qubit.

        target_qubit_addr (int): Address of the target qubit.

        matrix (QStat &): The CU gate matrix.

    Returns:
        A CU node representing the operation.


    12. CU(control_qubit_addr_list: List[int], target_qubit_addr_list: List[int], matrix: List[complex]) -> pyQPanda.QCircuit

    Create a CU gate.

    Args:
        control_qubit_addr_list (const std::vector<int> &): List of control qubit addresses.

        target_qubit_addr_list (const std::vector<int> &): List of target qubit addresses.

        matrix (QStat &): The CU gate matrix.

    Returns:
        A CU node representing the operation.

    """
@overload
def CU(control_qubit_list: QVec, target_qubit_list: QVec, alpha_angle: float, beta_angle: float, gamma_angle: float, delta_angle: float) -> QCircuit:
    """CU(*args, **kwargs)
    Overloaded function.

    1. CU(alpha_angle: float, beta_angle: float, gamma_angle: float, delta_angle: float, control_qubit: pyQPanda.Qubit, target_qubit: pyQPanda.Qubit) -> pyQPanda.QGate

    Create a CU gate.

    Args:
        alpha_angle: the alpha angle for the CU gate.

        beta_angle: the beta angle for the CU gate.

        gamma_angle: the gamma angle for the CU gate.

        delta_angle: the delta angle for the CU gate.

        control_qubit: the qubit that controls the operation.

        target_qubit: the qubit that is affected by the control qubit.

    Returns:
        A CU node representing the operation.

    2. CU(alpha_angle: float, beta_angle: float, gamma_angle: float, delta_angle: float, control_qubit_list: pyQPanda.QVec, target_qubi_list: pyQPanda.QVec) -> pyQPanda.QCircuit

    Create a CU gate.

    Args:
        alpha_angle (double): U4 gate alpha angle.

        beta_angle (double): U4 gate beta angle.

        gamma_angle (double): U4 gate gamma angle.

        delta_angle (double): U4 gate delta angle.

        control_qubit_list (const QVec &): List of control qubits.

        target_qubit_list (const QVec &): List of target qubits.

    Returns:
        A CU node representing the operation.


    3. CU(matrix: List[complex], control_qubit: pyQPanda.Qubit, target_qubit: pyQPanda.Qubit) -> pyQPanda.QGate

    Create a CU gate.

    Args:
        matrix (QStat &): The CU gate matrix.

        control_qubit (Qubit *): The control qubit.

        target_qubit (Qubit *): The target qubit.

    Returns:
        A CU node representing the operation.


    4. CU(matrix: List[complex], control_qubit_list: pyQPanda.QVec, target_qubit_list: pyQPanda.QVec) -> pyQPanda.QCircuit

    Create a CU gate.

    Args:
        matrix (QStat &): The CU gate matrix.

        control_qubit_list (const QVec &): List of control qubits.

        target_qubit_list (const QVec &): List of target qubits.

    Returns:
        A CU node representing the operation.


    5. CU(control_qubit: pyQPanda.Qubit, target_qubit: pyQPanda.Qubit, alpha_angle: float, beta_angle: float, gamma_angle: float, delta_angle: float) -> pyQPanda.QGate

    Create a CU gate.

    Args:
        control_qubit (Qubit *): The control qubit.

        target_qubit (Qubit *): The target qubit.

        alpha_angle (double): U4 gate alpha angle.

        beta_angle (double): U4 gate beta angle.

        gamma_angle (double): U4 gate gamma angle.

        delta_angle (double): U4 gate delta angle.

    Returns:
        A CU node representing the operation.


    6. CU(control_qubit_list: pyQPanda.QVec, target_qubit_list: pyQPanda.QVec, alpha_angle: float, beta_angle: float, gamma_angle: float, delta_angle: float) -> pyQPanda.QCircuit

    Create a CU gate.

    Args:
        control_qubit_list (QVec): Control qubit list.

        target_qubit_list (QVec): Target qubit list.

        alpha_angle (double): U4 gate alpha angle.

        beta_angle (double): U4 gate beta angle.

        gamma_angle (double): U4 gate gamma angle.

        delta_angle (double): U4 gate delta angle.

    Returns:
        CU node representing the operation.


    7. CU(control_qubit_addr: int, target_qubit_addr: int, alpha_angle: float, beta_angle: float, gamma_angle: float, delta_angle: float) -> pyQPanda.QGate

    Create a CU gate.

    Args:
        control_qubit_addr (int): Address of the control qubit.

        target_qubit_addr (int): Address of the target qubit.

        alpha_angle (double): U4 gate alpha angle.

        beta_angle (double): U4 gate beta angle.

        gamma_angle (double): U4 gate gamma angle.

        delta_angle (double): U4 gate delta angle.

    Returns:
        A CU node representing the operation.


    8. CU(control_qubit_addr_list: List[int], target_qubit_addr_list: List[int], alpha_angle: float, beta_angle: float, gamma_angle: float, delta_angle: float) -> pyQPanda.QCircuit

    Create a CU gate.

    Args:
        control_qubit_addr_list (std::vector<int>): List of control qubit addresses.

        target_qubit_addr_list (std::vector<int>): List of target qubit addresses.

        alpha_angle (double): U4 gate alpha angle.

        beta_angle (double): U4 gate beta angle.

        gamma_angle (double): U4 gate gamma angle.

        delta_angle (double): U4 gate delta angle.

    Returns:
        A CU node representing the operation.


    9. CU(control_qubit: pyQPanda.Qubit, target_qubit: pyQPanda.Qubit, matrix: List[complex]) -> pyQPanda.QGate

    Create a CU gate.

    Args:
        control_qubit (Qubit *): The control qubit.

        target_qubit (Qubit *): The target qubit.

        matrix (QStat &): The CU gate matrix.

    Returns:
        A CU node representing the operation.


    10. CU(control_qubit_list: pyQPanda.QVec, target_qubit_list: pyQPanda.QVec, matrix: List[complex]) -> pyQPanda.QCircuit

    Create a CU gate.

    Args:
        control_qubit_list (const QVec &): List of control qubits.

        target_qubit_list (const QVec &): List of target qubits.

        matrix (QStat &): The CU gate matrix.

    Returns:
        A CU node representing the operation.


    11. CU(control_qubit_addr: int, target_qubit_addr: int, matrix: List[complex]) -> pyQPanda.QGate

    Create a CU gate.

    Args:
        control_qubit_addr (int): Address of the control qubit.

        target_qubit_addr (int): Address of the target qubit.

        matrix (QStat &): The CU gate matrix.

    Returns:
        A CU node representing the operation.


    12. CU(control_qubit_addr_list: List[int], target_qubit_addr_list: List[int], matrix: List[complex]) -> pyQPanda.QCircuit

    Create a CU gate.

    Args:
        control_qubit_addr_list (const std::vector<int> &): List of control qubit addresses.

        target_qubit_addr_list (const std::vector<int> &): List of target qubit addresses.

        matrix (QStat &): The CU gate matrix.

    Returns:
        A CU node representing the operation.

    """
@overload
def CU(control_qubit_addr: int, target_qubit_addr: int, alpha_angle: float, beta_angle: float, gamma_angle: float, delta_angle: float) -> QGate:
    """CU(*args, **kwargs)
    Overloaded function.

    1. CU(alpha_angle: float, beta_angle: float, gamma_angle: float, delta_angle: float, control_qubit: pyQPanda.Qubit, target_qubit: pyQPanda.Qubit) -> pyQPanda.QGate

    Create a CU gate.

    Args:
        alpha_angle: the alpha angle for the CU gate.

        beta_angle: the beta angle for the CU gate.

        gamma_angle: the gamma angle for the CU gate.

        delta_angle: the delta angle for the CU gate.

        control_qubit: the qubit that controls the operation.

        target_qubit: the qubit that is affected by the control qubit.

    Returns:
        A CU node representing the operation.

    2. CU(alpha_angle: float, beta_angle: float, gamma_angle: float, delta_angle: float, control_qubit_list: pyQPanda.QVec, target_qubi_list: pyQPanda.QVec) -> pyQPanda.QCircuit

    Create a CU gate.

    Args:
        alpha_angle (double): U4 gate alpha angle.

        beta_angle (double): U4 gate beta angle.

        gamma_angle (double): U4 gate gamma angle.

        delta_angle (double): U4 gate delta angle.

        control_qubit_list (const QVec &): List of control qubits.

        target_qubit_list (const QVec &): List of target qubits.

    Returns:
        A CU node representing the operation.


    3. CU(matrix: List[complex], control_qubit: pyQPanda.Qubit, target_qubit: pyQPanda.Qubit) -> pyQPanda.QGate

    Create a CU gate.

    Args:
        matrix (QStat &): The CU gate matrix.

        control_qubit (Qubit *): The control qubit.

        target_qubit (Qubit *): The target qubit.

    Returns:
        A CU node representing the operation.


    4. CU(matrix: List[complex], control_qubit_list: pyQPanda.QVec, target_qubit_list: pyQPanda.QVec) -> pyQPanda.QCircuit

    Create a CU gate.

    Args:
        matrix (QStat &): The CU gate matrix.

        control_qubit_list (const QVec &): List of control qubits.

        target_qubit_list (const QVec &): List of target qubits.

    Returns:
        A CU node representing the operation.


    5. CU(control_qubit: pyQPanda.Qubit, target_qubit: pyQPanda.Qubit, alpha_angle: float, beta_angle: float, gamma_angle: float, delta_angle: float) -> pyQPanda.QGate

    Create a CU gate.

    Args:
        control_qubit (Qubit *): The control qubit.

        target_qubit (Qubit *): The target qubit.

        alpha_angle (double): U4 gate alpha angle.

        beta_angle (double): U4 gate beta angle.

        gamma_angle (double): U4 gate gamma angle.

        delta_angle (double): U4 gate delta angle.

    Returns:
        A CU node representing the operation.


    6. CU(control_qubit_list: pyQPanda.QVec, target_qubit_list: pyQPanda.QVec, alpha_angle: float, beta_angle: float, gamma_angle: float, delta_angle: float) -> pyQPanda.QCircuit

    Create a CU gate.

    Args:
        control_qubit_list (QVec): Control qubit list.

        target_qubit_list (QVec): Target qubit list.

        alpha_angle (double): U4 gate alpha angle.

        beta_angle (double): U4 gate beta angle.

        gamma_angle (double): U4 gate gamma angle.

        delta_angle (double): U4 gate delta angle.

    Returns:
        CU node representing the operation.


    7. CU(control_qubit_addr: int, target_qubit_addr: int, alpha_angle: float, beta_angle: float, gamma_angle: float, delta_angle: float) -> pyQPanda.QGate

    Create a CU gate.

    Args:
        control_qubit_addr (int): Address of the control qubit.

        target_qubit_addr (int): Address of the target qubit.

        alpha_angle (double): U4 gate alpha angle.

        beta_angle (double): U4 gate beta angle.

        gamma_angle (double): U4 gate gamma angle.

        delta_angle (double): U4 gate delta angle.

    Returns:
        A CU node representing the operation.


    8. CU(control_qubit_addr_list: List[int], target_qubit_addr_list: List[int], alpha_angle: float, beta_angle: float, gamma_angle: float, delta_angle: float) -> pyQPanda.QCircuit

    Create a CU gate.

    Args:
        control_qubit_addr_list (std::vector<int>): List of control qubit addresses.

        target_qubit_addr_list (std::vector<int>): List of target qubit addresses.

        alpha_angle (double): U4 gate alpha angle.

        beta_angle (double): U4 gate beta angle.

        gamma_angle (double): U4 gate gamma angle.

        delta_angle (double): U4 gate delta angle.

    Returns:
        A CU node representing the operation.


    9. CU(control_qubit: pyQPanda.Qubit, target_qubit: pyQPanda.Qubit, matrix: List[complex]) -> pyQPanda.QGate

    Create a CU gate.

    Args:
        control_qubit (Qubit *): The control qubit.

        target_qubit (Qubit *): The target qubit.

        matrix (QStat &): The CU gate matrix.

    Returns:
        A CU node representing the operation.


    10. CU(control_qubit_list: pyQPanda.QVec, target_qubit_list: pyQPanda.QVec, matrix: List[complex]) -> pyQPanda.QCircuit

    Create a CU gate.

    Args:
        control_qubit_list (const QVec &): List of control qubits.

        target_qubit_list (const QVec &): List of target qubits.

        matrix (QStat &): The CU gate matrix.

    Returns:
        A CU node representing the operation.


    11. CU(control_qubit_addr: int, target_qubit_addr: int, matrix: List[complex]) -> pyQPanda.QGate

    Create a CU gate.

    Args:
        control_qubit_addr (int): Address of the control qubit.

        target_qubit_addr (int): Address of the target qubit.

        matrix (QStat &): The CU gate matrix.

    Returns:
        A CU node representing the operation.


    12. CU(control_qubit_addr_list: List[int], target_qubit_addr_list: List[int], matrix: List[complex]) -> pyQPanda.QCircuit

    Create a CU gate.

    Args:
        control_qubit_addr_list (const std::vector<int> &): List of control qubit addresses.

        target_qubit_addr_list (const std::vector<int> &): List of target qubit addresses.

        matrix (QStat &): The CU gate matrix.

    Returns:
        A CU node representing the operation.

    """
@overload
def CU(control_qubit_addr_list: list[int], target_qubit_addr_list: list[int], alpha_angle: float, beta_angle: float, gamma_angle: float, delta_angle: float) -> QCircuit:
    """CU(*args, **kwargs)
    Overloaded function.

    1. CU(alpha_angle: float, beta_angle: float, gamma_angle: float, delta_angle: float, control_qubit: pyQPanda.Qubit, target_qubit: pyQPanda.Qubit) -> pyQPanda.QGate

    Create a CU gate.

    Args:
        alpha_angle: the alpha angle for the CU gate.

        beta_angle: the beta angle for the CU gate.

        gamma_angle: the gamma angle for the CU gate.

        delta_angle: the delta angle for the CU gate.

        control_qubit: the qubit that controls the operation.

        target_qubit: the qubit that is affected by the control qubit.

    Returns:
        A CU node representing the operation.

    2. CU(alpha_angle: float, beta_angle: float, gamma_angle: float, delta_angle: float, control_qubit_list: pyQPanda.QVec, target_qubi_list: pyQPanda.QVec) -> pyQPanda.QCircuit

    Create a CU gate.

    Args:
        alpha_angle (double): U4 gate alpha angle.

        beta_angle (double): U4 gate beta angle.

        gamma_angle (double): U4 gate gamma angle.

        delta_angle (double): U4 gate delta angle.

        control_qubit_list (const QVec &): List of control qubits.

        target_qubit_list (const QVec &): List of target qubits.

    Returns:
        A CU node representing the operation.


    3. CU(matrix: List[complex], control_qubit: pyQPanda.Qubit, target_qubit: pyQPanda.Qubit) -> pyQPanda.QGate

    Create a CU gate.

    Args:
        matrix (QStat &): The CU gate matrix.

        control_qubit (Qubit *): The control qubit.

        target_qubit (Qubit *): The target qubit.

    Returns:
        A CU node representing the operation.


    4. CU(matrix: List[complex], control_qubit_list: pyQPanda.QVec, target_qubit_list: pyQPanda.QVec) -> pyQPanda.QCircuit

    Create a CU gate.

    Args:
        matrix (QStat &): The CU gate matrix.

        control_qubit_list (const QVec &): List of control qubits.

        target_qubit_list (const QVec &): List of target qubits.

    Returns:
        A CU node representing the operation.


    5. CU(control_qubit: pyQPanda.Qubit, target_qubit: pyQPanda.Qubit, alpha_angle: float, beta_angle: float, gamma_angle: float, delta_angle: float) -> pyQPanda.QGate

    Create a CU gate.

    Args:
        control_qubit (Qubit *): The control qubit.

        target_qubit (Qubit *): The target qubit.

        alpha_angle (double): U4 gate alpha angle.

        beta_angle (double): U4 gate beta angle.

        gamma_angle (double): U4 gate gamma angle.

        delta_angle (double): U4 gate delta angle.

    Returns:
        A CU node representing the operation.


    6. CU(control_qubit_list: pyQPanda.QVec, target_qubit_list: pyQPanda.QVec, alpha_angle: float, beta_angle: float, gamma_angle: float, delta_angle: float) -> pyQPanda.QCircuit

    Create a CU gate.

    Args:
        control_qubit_list (QVec): Control qubit list.

        target_qubit_list (QVec): Target qubit list.

        alpha_angle (double): U4 gate alpha angle.

        beta_angle (double): U4 gate beta angle.

        gamma_angle (double): U4 gate gamma angle.

        delta_angle (double): U4 gate delta angle.

    Returns:
        CU node representing the operation.


    7. CU(control_qubit_addr: int, target_qubit_addr: int, alpha_angle: float, beta_angle: float, gamma_angle: float, delta_angle: float) -> pyQPanda.QGate

    Create a CU gate.

    Args:
        control_qubit_addr (int): Address of the control qubit.

        target_qubit_addr (int): Address of the target qubit.

        alpha_angle (double): U4 gate alpha angle.

        beta_angle (double): U4 gate beta angle.

        gamma_angle (double): U4 gate gamma angle.

        delta_angle (double): U4 gate delta angle.

    Returns:
        A CU node representing the operation.


    8. CU(control_qubit_addr_list: List[int], target_qubit_addr_list: List[int], alpha_angle: float, beta_angle: float, gamma_angle: float, delta_angle: float) -> pyQPanda.QCircuit

    Create a CU gate.

    Args:
        control_qubit_addr_list (std::vector<int>): List of control qubit addresses.

        target_qubit_addr_list (std::vector<int>): List of target qubit addresses.

        alpha_angle (double): U4 gate alpha angle.

        beta_angle (double): U4 gate beta angle.

        gamma_angle (double): U4 gate gamma angle.

        delta_angle (double): U4 gate delta angle.

    Returns:
        A CU node representing the operation.


    9. CU(control_qubit: pyQPanda.Qubit, target_qubit: pyQPanda.Qubit, matrix: List[complex]) -> pyQPanda.QGate

    Create a CU gate.

    Args:
        control_qubit (Qubit *): The control qubit.

        target_qubit (Qubit *): The target qubit.

        matrix (QStat &): The CU gate matrix.

    Returns:
        A CU node representing the operation.


    10. CU(control_qubit_list: pyQPanda.QVec, target_qubit_list: pyQPanda.QVec, matrix: List[complex]) -> pyQPanda.QCircuit

    Create a CU gate.

    Args:
        control_qubit_list (const QVec &): List of control qubits.

        target_qubit_list (const QVec &): List of target qubits.

        matrix (QStat &): The CU gate matrix.

    Returns:
        A CU node representing the operation.


    11. CU(control_qubit_addr: int, target_qubit_addr: int, matrix: List[complex]) -> pyQPanda.QGate

    Create a CU gate.

    Args:
        control_qubit_addr (int): Address of the control qubit.

        target_qubit_addr (int): Address of the target qubit.

        matrix (QStat &): The CU gate matrix.

    Returns:
        A CU node representing the operation.


    12. CU(control_qubit_addr_list: List[int], target_qubit_addr_list: List[int], matrix: List[complex]) -> pyQPanda.QCircuit

    Create a CU gate.

    Args:
        control_qubit_addr_list (const std::vector<int> &): List of control qubit addresses.

        target_qubit_addr_list (const std::vector<int> &): List of target qubit addresses.

        matrix (QStat &): The CU gate matrix.

    Returns:
        A CU node representing the operation.

    """
@overload
def CU(control_qubit: Qubit, target_qubit: Qubit, matrix: list[complex]) -> QGate:
    """CU(*args, **kwargs)
    Overloaded function.

    1. CU(alpha_angle: float, beta_angle: float, gamma_angle: float, delta_angle: float, control_qubit: pyQPanda.Qubit, target_qubit: pyQPanda.Qubit) -> pyQPanda.QGate

    Create a CU gate.

    Args:
        alpha_angle: the alpha angle for the CU gate.

        beta_angle: the beta angle for the CU gate.

        gamma_angle: the gamma angle for the CU gate.

        delta_angle: the delta angle for the CU gate.

        control_qubit: the qubit that controls the operation.

        target_qubit: the qubit that is affected by the control qubit.

    Returns:
        A CU node representing the operation.

    2. CU(alpha_angle: float, beta_angle: float, gamma_angle: float, delta_angle: float, control_qubit_list: pyQPanda.QVec, target_qubi_list: pyQPanda.QVec) -> pyQPanda.QCircuit

    Create a CU gate.

    Args:
        alpha_angle (double): U4 gate alpha angle.

        beta_angle (double): U4 gate beta angle.

        gamma_angle (double): U4 gate gamma angle.

        delta_angle (double): U4 gate delta angle.

        control_qubit_list (const QVec &): List of control qubits.

        target_qubit_list (const QVec &): List of target qubits.

    Returns:
        A CU node representing the operation.


    3. CU(matrix: List[complex], control_qubit: pyQPanda.Qubit, target_qubit: pyQPanda.Qubit) -> pyQPanda.QGate

    Create a CU gate.

    Args:
        matrix (QStat &): The CU gate matrix.

        control_qubit (Qubit *): The control qubit.

        target_qubit (Qubit *): The target qubit.

    Returns:
        A CU node representing the operation.


    4. CU(matrix: List[complex], control_qubit_list: pyQPanda.QVec, target_qubit_list: pyQPanda.QVec) -> pyQPanda.QCircuit

    Create a CU gate.

    Args:
        matrix (QStat &): The CU gate matrix.

        control_qubit_list (const QVec &): List of control qubits.

        target_qubit_list (const QVec &): List of target qubits.

    Returns:
        A CU node representing the operation.


    5. CU(control_qubit: pyQPanda.Qubit, target_qubit: pyQPanda.Qubit, alpha_angle: float, beta_angle: float, gamma_angle: float, delta_angle: float) -> pyQPanda.QGate

    Create a CU gate.

    Args:
        control_qubit (Qubit *): The control qubit.

        target_qubit (Qubit *): The target qubit.

        alpha_angle (double): U4 gate alpha angle.

        beta_angle (double): U4 gate beta angle.

        gamma_angle (double): U4 gate gamma angle.

        delta_angle (double): U4 gate delta angle.

    Returns:
        A CU node representing the operation.


    6. CU(control_qubit_list: pyQPanda.QVec, target_qubit_list: pyQPanda.QVec, alpha_angle: float, beta_angle: float, gamma_angle: float, delta_angle: float) -> pyQPanda.QCircuit

    Create a CU gate.

    Args:
        control_qubit_list (QVec): Control qubit list.

        target_qubit_list (QVec): Target qubit list.

        alpha_angle (double): U4 gate alpha angle.

        beta_angle (double): U4 gate beta angle.

        gamma_angle (double): U4 gate gamma angle.

        delta_angle (double): U4 gate delta angle.

    Returns:
        CU node representing the operation.


    7. CU(control_qubit_addr: int, target_qubit_addr: int, alpha_angle: float, beta_angle: float, gamma_angle: float, delta_angle: float) -> pyQPanda.QGate

    Create a CU gate.

    Args:
        control_qubit_addr (int): Address of the control qubit.

        target_qubit_addr (int): Address of the target qubit.

        alpha_angle (double): U4 gate alpha angle.

        beta_angle (double): U4 gate beta angle.

        gamma_angle (double): U4 gate gamma angle.

        delta_angle (double): U4 gate delta angle.

    Returns:
        A CU node representing the operation.


    8. CU(control_qubit_addr_list: List[int], target_qubit_addr_list: List[int], alpha_angle: float, beta_angle: float, gamma_angle: float, delta_angle: float) -> pyQPanda.QCircuit

    Create a CU gate.

    Args:
        control_qubit_addr_list (std::vector<int>): List of control qubit addresses.

        target_qubit_addr_list (std::vector<int>): List of target qubit addresses.

        alpha_angle (double): U4 gate alpha angle.

        beta_angle (double): U4 gate beta angle.

        gamma_angle (double): U4 gate gamma angle.

        delta_angle (double): U4 gate delta angle.

    Returns:
        A CU node representing the operation.


    9. CU(control_qubit: pyQPanda.Qubit, target_qubit: pyQPanda.Qubit, matrix: List[complex]) -> pyQPanda.QGate

    Create a CU gate.

    Args:
        control_qubit (Qubit *): The control qubit.

        target_qubit (Qubit *): The target qubit.

        matrix (QStat &): The CU gate matrix.

    Returns:
        A CU node representing the operation.


    10. CU(control_qubit_list: pyQPanda.QVec, target_qubit_list: pyQPanda.QVec, matrix: List[complex]) -> pyQPanda.QCircuit

    Create a CU gate.

    Args:
        control_qubit_list (const QVec &): List of control qubits.

        target_qubit_list (const QVec &): List of target qubits.

        matrix (QStat &): The CU gate matrix.

    Returns:
        A CU node representing the operation.


    11. CU(control_qubit_addr: int, target_qubit_addr: int, matrix: List[complex]) -> pyQPanda.QGate

    Create a CU gate.

    Args:
        control_qubit_addr (int): Address of the control qubit.

        target_qubit_addr (int): Address of the target qubit.

        matrix (QStat &): The CU gate matrix.

    Returns:
        A CU node representing the operation.


    12. CU(control_qubit_addr_list: List[int], target_qubit_addr_list: List[int], matrix: List[complex]) -> pyQPanda.QCircuit

    Create a CU gate.

    Args:
        control_qubit_addr_list (const std::vector<int> &): List of control qubit addresses.

        target_qubit_addr_list (const std::vector<int> &): List of target qubit addresses.

        matrix (QStat &): The CU gate matrix.

    Returns:
        A CU node representing the operation.

    """
@overload
def CU(control_qubit_list: QVec, target_qubit_list: QVec, matrix: list[complex]) -> QCircuit:
    """CU(*args, **kwargs)
    Overloaded function.

    1. CU(alpha_angle: float, beta_angle: float, gamma_angle: float, delta_angle: float, control_qubit: pyQPanda.Qubit, target_qubit: pyQPanda.Qubit) -> pyQPanda.QGate

    Create a CU gate.

    Args:
        alpha_angle: the alpha angle for the CU gate.

        beta_angle: the beta angle for the CU gate.

        gamma_angle: the gamma angle for the CU gate.

        delta_angle: the delta angle for the CU gate.

        control_qubit: the qubit that controls the operation.

        target_qubit: the qubit that is affected by the control qubit.

    Returns:
        A CU node representing the operation.

    2. CU(alpha_angle: float, beta_angle: float, gamma_angle: float, delta_angle: float, control_qubit_list: pyQPanda.QVec, target_qubi_list: pyQPanda.QVec) -> pyQPanda.QCircuit

    Create a CU gate.

    Args:
        alpha_angle (double): U4 gate alpha angle.

        beta_angle (double): U4 gate beta angle.

        gamma_angle (double): U4 gate gamma angle.

        delta_angle (double): U4 gate delta angle.

        control_qubit_list (const QVec &): List of control qubits.

        target_qubit_list (const QVec &): List of target qubits.

    Returns:
        A CU node representing the operation.


    3. CU(matrix: List[complex], control_qubit: pyQPanda.Qubit, target_qubit: pyQPanda.Qubit) -> pyQPanda.QGate

    Create a CU gate.

    Args:
        matrix (QStat &): The CU gate matrix.

        control_qubit (Qubit *): The control qubit.

        target_qubit (Qubit *): The target qubit.

    Returns:
        A CU node representing the operation.


    4. CU(matrix: List[complex], control_qubit_list: pyQPanda.QVec, target_qubit_list: pyQPanda.QVec) -> pyQPanda.QCircuit

    Create a CU gate.

    Args:
        matrix (QStat &): The CU gate matrix.

        control_qubit_list (const QVec &): List of control qubits.

        target_qubit_list (const QVec &): List of target qubits.

    Returns:
        A CU node representing the operation.


    5. CU(control_qubit: pyQPanda.Qubit, target_qubit: pyQPanda.Qubit, alpha_angle: float, beta_angle: float, gamma_angle: float, delta_angle: float) -> pyQPanda.QGate

    Create a CU gate.

    Args:
        control_qubit (Qubit *): The control qubit.

        target_qubit (Qubit *): The target qubit.

        alpha_angle (double): U4 gate alpha angle.

        beta_angle (double): U4 gate beta angle.

        gamma_angle (double): U4 gate gamma angle.

        delta_angle (double): U4 gate delta angle.

    Returns:
        A CU node representing the operation.


    6. CU(control_qubit_list: pyQPanda.QVec, target_qubit_list: pyQPanda.QVec, alpha_angle: float, beta_angle: float, gamma_angle: float, delta_angle: float) -> pyQPanda.QCircuit

    Create a CU gate.

    Args:
        control_qubit_list (QVec): Control qubit list.

        target_qubit_list (QVec): Target qubit list.

        alpha_angle (double): U4 gate alpha angle.

        beta_angle (double): U4 gate beta angle.

        gamma_angle (double): U4 gate gamma angle.

        delta_angle (double): U4 gate delta angle.

    Returns:
        CU node representing the operation.


    7. CU(control_qubit_addr: int, target_qubit_addr: int, alpha_angle: float, beta_angle: float, gamma_angle: float, delta_angle: float) -> pyQPanda.QGate

    Create a CU gate.

    Args:
        control_qubit_addr (int): Address of the control qubit.

        target_qubit_addr (int): Address of the target qubit.

        alpha_angle (double): U4 gate alpha angle.

        beta_angle (double): U4 gate beta angle.

        gamma_angle (double): U4 gate gamma angle.

        delta_angle (double): U4 gate delta angle.

    Returns:
        A CU node representing the operation.


    8. CU(control_qubit_addr_list: List[int], target_qubit_addr_list: List[int], alpha_angle: float, beta_angle: float, gamma_angle: float, delta_angle: float) -> pyQPanda.QCircuit

    Create a CU gate.

    Args:
        control_qubit_addr_list (std::vector<int>): List of control qubit addresses.

        target_qubit_addr_list (std::vector<int>): List of target qubit addresses.

        alpha_angle (double): U4 gate alpha angle.

        beta_angle (double): U4 gate beta angle.

        gamma_angle (double): U4 gate gamma angle.

        delta_angle (double): U4 gate delta angle.

    Returns:
        A CU node representing the operation.


    9. CU(control_qubit: pyQPanda.Qubit, target_qubit: pyQPanda.Qubit, matrix: List[complex]) -> pyQPanda.QGate

    Create a CU gate.

    Args:
        control_qubit (Qubit *): The control qubit.

        target_qubit (Qubit *): The target qubit.

        matrix (QStat &): The CU gate matrix.

    Returns:
        A CU node representing the operation.


    10. CU(control_qubit_list: pyQPanda.QVec, target_qubit_list: pyQPanda.QVec, matrix: List[complex]) -> pyQPanda.QCircuit

    Create a CU gate.

    Args:
        control_qubit_list (const QVec &): List of control qubits.

        target_qubit_list (const QVec &): List of target qubits.

        matrix (QStat &): The CU gate matrix.

    Returns:
        A CU node representing the operation.


    11. CU(control_qubit_addr: int, target_qubit_addr: int, matrix: List[complex]) -> pyQPanda.QGate

    Create a CU gate.

    Args:
        control_qubit_addr (int): Address of the control qubit.

        target_qubit_addr (int): Address of the target qubit.

        matrix (QStat &): The CU gate matrix.

    Returns:
        A CU node representing the operation.


    12. CU(control_qubit_addr_list: List[int], target_qubit_addr_list: List[int], matrix: List[complex]) -> pyQPanda.QCircuit

    Create a CU gate.

    Args:
        control_qubit_addr_list (const std::vector<int> &): List of control qubit addresses.

        target_qubit_addr_list (const std::vector<int> &): List of target qubit addresses.

        matrix (QStat &): The CU gate matrix.

    Returns:
        A CU node representing the operation.

    """
@overload
def CU(control_qubit_addr: int, target_qubit_addr: int, matrix: list[complex]) -> QGate:
    """CU(*args, **kwargs)
    Overloaded function.

    1. CU(alpha_angle: float, beta_angle: float, gamma_angle: float, delta_angle: float, control_qubit: pyQPanda.Qubit, target_qubit: pyQPanda.Qubit) -> pyQPanda.QGate

    Create a CU gate.

    Args:
        alpha_angle: the alpha angle for the CU gate.

        beta_angle: the beta angle for the CU gate.

        gamma_angle: the gamma angle for the CU gate.

        delta_angle: the delta angle for the CU gate.

        control_qubit: the qubit that controls the operation.

        target_qubit: the qubit that is affected by the control qubit.

    Returns:
        A CU node representing the operation.

    2. CU(alpha_angle: float, beta_angle: float, gamma_angle: float, delta_angle: float, control_qubit_list: pyQPanda.QVec, target_qubi_list: pyQPanda.QVec) -> pyQPanda.QCircuit

    Create a CU gate.

    Args:
        alpha_angle (double): U4 gate alpha angle.

        beta_angle (double): U4 gate beta angle.

        gamma_angle (double): U4 gate gamma angle.

        delta_angle (double): U4 gate delta angle.

        control_qubit_list (const QVec &): List of control qubits.

        target_qubit_list (const QVec &): List of target qubits.

    Returns:
        A CU node representing the operation.


    3. CU(matrix: List[complex], control_qubit: pyQPanda.Qubit, target_qubit: pyQPanda.Qubit) -> pyQPanda.QGate

    Create a CU gate.

    Args:
        matrix (QStat &): The CU gate matrix.

        control_qubit (Qubit *): The control qubit.

        target_qubit (Qubit *): The target qubit.

    Returns:
        A CU node representing the operation.


    4. CU(matrix: List[complex], control_qubit_list: pyQPanda.QVec, target_qubit_list: pyQPanda.QVec) -> pyQPanda.QCircuit

    Create a CU gate.

    Args:
        matrix (QStat &): The CU gate matrix.

        control_qubit_list (const QVec &): List of control qubits.

        target_qubit_list (const QVec &): List of target qubits.

    Returns:
        A CU node representing the operation.


    5. CU(control_qubit: pyQPanda.Qubit, target_qubit: pyQPanda.Qubit, alpha_angle: float, beta_angle: float, gamma_angle: float, delta_angle: float) -> pyQPanda.QGate

    Create a CU gate.

    Args:
        control_qubit (Qubit *): The control qubit.

        target_qubit (Qubit *): The target qubit.

        alpha_angle (double): U4 gate alpha angle.

        beta_angle (double): U4 gate beta angle.

        gamma_angle (double): U4 gate gamma angle.

        delta_angle (double): U4 gate delta angle.

    Returns:
        A CU node representing the operation.


    6. CU(control_qubit_list: pyQPanda.QVec, target_qubit_list: pyQPanda.QVec, alpha_angle: float, beta_angle: float, gamma_angle: float, delta_angle: float) -> pyQPanda.QCircuit

    Create a CU gate.

    Args:
        control_qubit_list (QVec): Control qubit list.

        target_qubit_list (QVec): Target qubit list.

        alpha_angle (double): U4 gate alpha angle.

        beta_angle (double): U4 gate beta angle.

        gamma_angle (double): U4 gate gamma angle.

        delta_angle (double): U4 gate delta angle.

    Returns:
        CU node representing the operation.


    7. CU(control_qubit_addr: int, target_qubit_addr: int, alpha_angle: float, beta_angle: float, gamma_angle: float, delta_angle: float) -> pyQPanda.QGate

    Create a CU gate.

    Args:
        control_qubit_addr (int): Address of the control qubit.

        target_qubit_addr (int): Address of the target qubit.

        alpha_angle (double): U4 gate alpha angle.

        beta_angle (double): U4 gate beta angle.

        gamma_angle (double): U4 gate gamma angle.

        delta_angle (double): U4 gate delta angle.

    Returns:
        A CU node representing the operation.


    8. CU(control_qubit_addr_list: List[int], target_qubit_addr_list: List[int], alpha_angle: float, beta_angle: float, gamma_angle: float, delta_angle: float) -> pyQPanda.QCircuit

    Create a CU gate.

    Args:
        control_qubit_addr_list (std::vector<int>): List of control qubit addresses.

        target_qubit_addr_list (std::vector<int>): List of target qubit addresses.

        alpha_angle (double): U4 gate alpha angle.

        beta_angle (double): U4 gate beta angle.

        gamma_angle (double): U4 gate gamma angle.

        delta_angle (double): U4 gate delta angle.

    Returns:
        A CU node representing the operation.


    9. CU(control_qubit: pyQPanda.Qubit, target_qubit: pyQPanda.Qubit, matrix: List[complex]) -> pyQPanda.QGate

    Create a CU gate.

    Args:
        control_qubit (Qubit *): The control qubit.

        target_qubit (Qubit *): The target qubit.

        matrix (QStat &): The CU gate matrix.

    Returns:
        A CU node representing the operation.


    10. CU(control_qubit_list: pyQPanda.QVec, target_qubit_list: pyQPanda.QVec, matrix: List[complex]) -> pyQPanda.QCircuit

    Create a CU gate.

    Args:
        control_qubit_list (const QVec &): List of control qubits.

        target_qubit_list (const QVec &): List of target qubits.

        matrix (QStat &): The CU gate matrix.

    Returns:
        A CU node representing the operation.


    11. CU(control_qubit_addr: int, target_qubit_addr: int, matrix: List[complex]) -> pyQPanda.QGate

    Create a CU gate.

    Args:
        control_qubit_addr (int): Address of the control qubit.

        target_qubit_addr (int): Address of the target qubit.

        matrix (QStat &): The CU gate matrix.

    Returns:
        A CU node representing the operation.


    12. CU(control_qubit_addr_list: List[int], target_qubit_addr_list: List[int], matrix: List[complex]) -> pyQPanda.QCircuit

    Create a CU gate.

    Args:
        control_qubit_addr_list (const std::vector<int> &): List of control qubit addresses.

        target_qubit_addr_list (const std::vector<int> &): List of target qubit addresses.

        matrix (QStat &): The CU gate matrix.

    Returns:
        A CU node representing the operation.

    """
@overload
def CU(control_qubit_addr_list: list[int], target_qubit_addr_list: list[int], matrix: list[complex]) -> QCircuit:
    """CU(*args, **kwargs)
    Overloaded function.

    1. CU(alpha_angle: float, beta_angle: float, gamma_angle: float, delta_angle: float, control_qubit: pyQPanda.Qubit, target_qubit: pyQPanda.Qubit) -> pyQPanda.QGate

    Create a CU gate.

    Args:
        alpha_angle: the alpha angle for the CU gate.

        beta_angle: the beta angle for the CU gate.

        gamma_angle: the gamma angle for the CU gate.

        delta_angle: the delta angle for the CU gate.

        control_qubit: the qubit that controls the operation.

        target_qubit: the qubit that is affected by the control qubit.

    Returns:
        A CU node representing the operation.

    2. CU(alpha_angle: float, beta_angle: float, gamma_angle: float, delta_angle: float, control_qubit_list: pyQPanda.QVec, target_qubi_list: pyQPanda.QVec) -> pyQPanda.QCircuit

    Create a CU gate.

    Args:
        alpha_angle (double): U4 gate alpha angle.

        beta_angle (double): U4 gate beta angle.

        gamma_angle (double): U4 gate gamma angle.

        delta_angle (double): U4 gate delta angle.

        control_qubit_list (const QVec &): List of control qubits.

        target_qubit_list (const QVec &): List of target qubits.

    Returns:
        A CU node representing the operation.


    3. CU(matrix: List[complex], control_qubit: pyQPanda.Qubit, target_qubit: pyQPanda.Qubit) -> pyQPanda.QGate

    Create a CU gate.

    Args:
        matrix (QStat &): The CU gate matrix.

        control_qubit (Qubit *): The control qubit.

        target_qubit (Qubit *): The target qubit.

    Returns:
        A CU node representing the operation.


    4. CU(matrix: List[complex], control_qubit_list: pyQPanda.QVec, target_qubit_list: pyQPanda.QVec) -> pyQPanda.QCircuit

    Create a CU gate.

    Args:
        matrix (QStat &): The CU gate matrix.

        control_qubit_list (const QVec &): List of control qubits.

        target_qubit_list (const QVec &): List of target qubits.

    Returns:
        A CU node representing the operation.


    5. CU(control_qubit: pyQPanda.Qubit, target_qubit: pyQPanda.Qubit, alpha_angle: float, beta_angle: float, gamma_angle: float, delta_angle: float) -> pyQPanda.QGate

    Create a CU gate.

    Args:
        control_qubit (Qubit *): The control qubit.

        target_qubit (Qubit *): The target qubit.

        alpha_angle (double): U4 gate alpha angle.

        beta_angle (double): U4 gate beta angle.

        gamma_angle (double): U4 gate gamma angle.

        delta_angle (double): U4 gate delta angle.

    Returns:
        A CU node representing the operation.


    6. CU(control_qubit_list: pyQPanda.QVec, target_qubit_list: pyQPanda.QVec, alpha_angle: float, beta_angle: float, gamma_angle: float, delta_angle: float) -> pyQPanda.QCircuit

    Create a CU gate.

    Args:
        control_qubit_list (QVec): Control qubit list.

        target_qubit_list (QVec): Target qubit list.

        alpha_angle (double): U4 gate alpha angle.

        beta_angle (double): U4 gate beta angle.

        gamma_angle (double): U4 gate gamma angle.

        delta_angle (double): U4 gate delta angle.

    Returns:
        CU node representing the operation.


    7. CU(control_qubit_addr: int, target_qubit_addr: int, alpha_angle: float, beta_angle: float, gamma_angle: float, delta_angle: float) -> pyQPanda.QGate

    Create a CU gate.

    Args:
        control_qubit_addr (int): Address of the control qubit.

        target_qubit_addr (int): Address of the target qubit.

        alpha_angle (double): U4 gate alpha angle.

        beta_angle (double): U4 gate beta angle.

        gamma_angle (double): U4 gate gamma angle.

        delta_angle (double): U4 gate delta angle.

    Returns:
        A CU node representing the operation.


    8. CU(control_qubit_addr_list: List[int], target_qubit_addr_list: List[int], alpha_angle: float, beta_angle: float, gamma_angle: float, delta_angle: float) -> pyQPanda.QCircuit

    Create a CU gate.

    Args:
        control_qubit_addr_list (std::vector<int>): List of control qubit addresses.

        target_qubit_addr_list (std::vector<int>): List of target qubit addresses.

        alpha_angle (double): U4 gate alpha angle.

        beta_angle (double): U4 gate beta angle.

        gamma_angle (double): U4 gate gamma angle.

        delta_angle (double): U4 gate delta angle.

    Returns:
        A CU node representing the operation.


    9. CU(control_qubit: pyQPanda.Qubit, target_qubit: pyQPanda.Qubit, matrix: List[complex]) -> pyQPanda.QGate

    Create a CU gate.

    Args:
        control_qubit (Qubit *): The control qubit.

        target_qubit (Qubit *): The target qubit.

        matrix (QStat &): The CU gate matrix.

    Returns:
        A CU node representing the operation.


    10. CU(control_qubit_list: pyQPanda.QVec, target_qubit_list: pyQPanda.QVec, matrix: List[complex]) -> pyQPanda.QCircuit

    Create a CU gate.

    Args:
        control_qubit_list (const QVec &): List of control qubits.

        target_qubit_list (const QVec &): List of target qubits.

        matrix (QStat &): The CU gate matrix.

    Returns:
        A CU node representing the operation.


    11. CU(control_qubit_addr: int, target_qubit_addr: int, matrix: List[complex]) -> pyQPanda.QGate

    Create a CU gate.

    Args:
        control_qubit_addr (int): Address of the control qubit.

        target_qubit_addr (int): Address of the target qubit.

        matrix (QStat &): The CU gate matrix.

    Returns:
        A CU node representing the operation.


    12. CU(control_qubit_addr_list: List[int], target_qubit_addr_list: List[int], matrix: List[complex]) -> pyQPanda.QCircuit

    Create a CU gate.

    Args:
        control_qubit_addr_list (const std::vector<int> &): List of control qubit addresses.

        target_qubit_addr_list (const std::vector<int> &): List of target qubit addresses.

        matrix (QStat &): The CU gate matrix.

    Returns:
        A CU node representing the operation.

    """
@overload
def CZ(control_qubit: Qubit, target_qubit: Qubit) -> QGate:
    """CZ(*args, **kwargs)
    Overloaded function.

    1. CZ(control_qubit: pyQPanda.Qubit, target_qubit: pyQPanda.Qubit) -> pyQPanda.QGate

    Returns:
        a CZ gate node
    Raises:
        run_fail: An error occurred in construct gate node


    2. CZ(control_qubit_list: pyQPanda.QVec, target_qubit_list: pyQPanda.QVec) -> pyQPanda.QCircuit

    Returns:
        a CZ gate node
    Raises:
        run_fail: An error occurred in construct gate node


    3. CZ(control_qubit_addr: int, target_qubit_addr: int) -> pyQPanda.QGate

    4. CZ(control_qubit_addr_list: List[int], target_qubit_addr_list: List[int]) -> pyQPanda.QCircuit

    Returns:
        a CZ gate node
    Raises:
        run_fail: An error occurred in construct gate node

    """
@overload
def CZ(control_qubit_list: QVec, target_qubit_list: QVec) -> QCircuit:
    """CZ(*args, **kwargs)
    Overloaded function.

    1. CZ(control_qubit: pyQPanda.Qubit, target_qubit: pyQPanda.Qubit) -> pyQPanda.QGate

    Returns:
        a CZ gate node
    Raises:
        run_fail: An error occurred in construct gate node


    2. CZ(control_qubit_list: pyQPanda.QVec, target_qubit_list: pyQPanda.QVec) -> pyQPanda.QCircuit

    Returns:
        a CZ gate node
    Raises:
        run_fail: An error occurred in construct gate node


    3. CZ(control_qubit_addr: int, target_qubit_addr: int) -> pyQPanda.QGate

    4. CZ(control_qubit_addr_list: List[int], target_qubit_addr_list: List[int]) -> pyQPanda.QCircuit

    Returns:
        a CZ gate node
    Raises:
        run_fail: An error occurred in construct gate node

    """
@overload
def CZ(control_qubit_addr: int, target_qubit_addr: int) -> QGate:
    """CZ(*args, **kwargs)
    Overloaded function.

    1. CZ(control_qubit: pyQPanda.Qubit, target_qubit: pyQPanda.Qubit) -> pyQPanda.QGate

    Returns:
        a CZ gate node
    Raises:
        run_fail: An error occurred in construct gate node


    2. CZ(control_qubit_list: pyQPanda.QVec, target_qubit_list: pyQPanda.QVec) -> pyQPanda.QCircuit

    Returns:
        a CZ gate node
    Raises:
        run_fail: An error occurred in construct gate node


    3. CZ(control_qubit_addr: int, target_qubit_addr: int) -> pyQPanda.QGate

    4. CZ(control_qubit_addr_list: List[int], target_qubit_addr_list: List[int]) -> pyQPanda.QCircuit

    Returns:
        a CZ gate node
    Raises:
        run_fail: An error occurred in construct gate node

    """
@overload
def CZ(control_qubit_addr_list: list[int], target_qubit_addr_list: list[int]) -> QCircuit:
    """CZ(*args, **kwargs)
    Overloaded function.

    1. CZ(control_qubit: pyQPanda.Qubit, target_qubit: pyQPanda.Qubit) -> pyQPanda.QGate

    Returns:
        a CZ gate node
    Raises:
        run_fail: An error occurred in construct gate node


    2. CZ(control_qubit_list: pyQPanda.QVec, target_qubit_list: pyQPanda.QVec) -> pyQPanda.QCircuit

    Returns:
        a CZ gate node
    Raises:
        run_fail: An error occurred in construct gate node


    3. CZ(control_qubit_addr: int, target_qubit_addr: int) -> pyQPanda.QGate

    4. CZ(control_qubit_addr_list: List[int], target_qubit_addr_list: List[int]) -> pyQPanda.QCircuit

    Returns:
        a CZ gate node
    Raises:
        run_fail: An error occurred in construct gate node

    """
def CreateEmptyCircuit() -> QCircuit:
    """CreateEmptyCircuit() -> pyQPanda.QCircuit

    Create an empty QCircuit container.

    Args:
        none

    Returns:
        result: An empty QCircuit.

    """
def CreateEmptyQProg() -> QProg:
    """CreateEmptyQProg() -> pyQPanda.QProg

    Create an empty QProg container.

    Args:
        none

    Returns:
        an empty QProg.

    """
@overload
def CreateIfProg(classical_condition: ClassicalCondition, true_node: QProg) -> QIfProg:
    """CreateIfProg(*args, **kwargs)
    Overloaded function.

    1. CreateIfProg(classical_condition: pyQPanda.ClassicalCondition, true_node: pyQPanda.QProg) -> pyQPanda.QIfProg

    Create an IfProg that executes a quantum operation if a classical condition is true.

    Args:
        classical_condition: A classical condition representing the if condition.

        true_node: The quantum operations to execute if the condition is true.

    Returns:
        IfProg: The program that performs the specified operations if the condition is true.


    2. CreateIfProg(classical_condition: pyQPanda.ClassicalCondition, true_node: pyQPanda.QProg, false_node: pyQPanda.QProg) -> pyQPanda.QIfProg

    Create an IfProg that executes one of two quantum operations based on a classical condition.

    Args:
        classical_condition: A classical condition representing the if condition.

        true_node: The quantum operations to execute if the condition is true.

        false_node: The quantum operations to execute if the condition is false.

    Returns:
        IfProg: The program that performs the specified operations based on the condition.

    """
@overload
def CreateIfProg(classical_condition: ClassicalCondition, true_node: QProg, false_node: QProg) -> QIfProg:
    """CreateIfProg(*args, **kwargs)
    Overloaded function.

    1. CreateIfProg(classical_condition: pyQPanda.ClassicalCondition, true_node: pyQPanda.QProg) -> pyQPanda.QIfProg

    Create an IfProg that executes a quantum operation if a classical condition is true.

    Args:
        classical_condition: A classical condition representing the if condition.

        true_node: The quantum operations to execute if the condition is true.

    Returns:
        IfProg: The program that performs the specified operations if the condition is true.


    2. CreateIfProg(classical_condition: pyQPanda.ClassicalCondition, true_node: pyQPanda.QProg, false_node: pyQPanda.QProg) -> pyQPanda.QIfProg

    Create an IfProg that executes one of two quantum operations based on a classical condition.

    Args:
        classical_condition: A classical condition representing the if condition.

        true_node: The quantum operations to execute if the condition is true.

        false_node: The quantum operations to execute if the condition is false.

    Returns:
        IfProg: The program that performs the specified operations based on the condition.

    """
def CreateWhileProg(classical_condition: ClassicalCondition, true_node: QProg) -> QWhileProg:
    """CreateWhileProg(classical_condition: pyQPanda.ClassicalCondition, true_node: pyQPanda.QProg) -> pyQPanda.QWhileProg

    Create a WhileProg that executes while a classical condition is true.

    Args:
        classical_condition: A classical condition representing the while-loop condition.

        true_node: The quantum operations to execute while the condition is true.

    Returns:
        WhileProg: The program that performs the specified operations while the condition holds.

    """
def Grover(*args, **kwargs):
    """Grover(data: List[int], Classical_condition: pyQPanda.ClassicalCondition, QuantumMachine: pyQPanda.QuantumMachine, qlist: pyQPanda.QVec, data: int = 2) -> pyQPanda.QProg

    Quantum grover circuit

    Args:
        qvec: qubit list
        Classical_condition: quantum Classical condition
        QuantumMachine: quantum machine

    Returns:
        result : Grover circuit
    Raises:
        run_fail: An error occurred in Grover

    """
@overload
def Grover_search(list: list[int], Classical_condition: ClassicalCondition, QuantumMachine: QuantumMachine, repeat: int = ...) -> list:
    """Grover_search(*args, **kwargs)
    Overloaded function.

    1. Grover_search(list: List[int], Classical_condition: pyQPanda.ClassicalCondition, QuantumMachine: pyQPanda.QuantumMachine, repeat: int = 2) -> list

    Use Grover algorithm to search target data, return QProg and search_result

    Args:
        list: data list
        Classical_condition: quantum Classical condition
        QuantumMachine: quantum machine
        repeat: search repeat times

    Returns:
        result : Grover search result
    Raises:
        run_fail: An error occurred in Grover


    2. Grover_search(list: List[str], Classical_condition: str, QuantumMachine: pyQPanda.QuantumMachine, data: int = 2) -> list

    use Grover algorithm to search target data, return QProg and search_result
    """
@overload
def Grover_search(list: list[str], Classical_condition: str, QuantumMachine: QuantumMachine, data: int = ...) -> list:
    """Grover_search(*args, **kwargs)
    Overloaded function.

    1. Grover_search(list: List[int], Classical_condition: pyQPanda.ClassicalCondition, QuantumMachine: pyQPanda.QuantumMachine, repeat: int = 2) -> list

    Use Grover algorithm to search target data, return QProg and search_result

    Args:
        list: data list
        Classical_condition: quantum Classical condition
        QuantumMachine: quantum machine
        repeat: search repeat times

    Returns:
        result : Grover search result
    Raises:
        run_fail: An error occurred in Grover


    2. Grover_search(list: List[str], Classical_condition: str, QuantumMachine: pyQPanda.QuantumMachine, data: int = 2) -> list

    use Grover algorithm to search target data, return QProg and search_result
    """
@overload
def H(qubit: Qubit) -> QGate:
    """H(*args, **kwargs)
    Overloaded function.

    1. H(qubit: pyQPanda.Qubit) -> pyQPanda.QGate

    Create a H gate

    Args:
        qubit : quantum gate operate qubit

    Returns:
        a H gate node
    Raises:
        run_fail: An error occurred in construct gate node


    2. H(qubit_list: pyQPanda.QVec) -> pyQPanda.QCircuit

    Create a H gate

    Args:
        qubit_list: quantum gate operate qubits list

    Returns:
        a H gate node
    Raises:
        run_fail: An error occurred construct in gate node


    3. H(qubit_addr: int) -> pyQPanda.QGate

    Create a H gate

    Args:
        qubit_addr: quantum gate operate qubits addr

    Returns:
        a H gate node
    Raises:
        run_fail: An error occurred in construct gate node


    4. H(qubit_addr_list: List[int]) -> pyQPanda.QCircuit

    Create a H gate

    Args:
        qubit_list_addr: quantum gate  qubits list addr

    Returns:
        a H gate node
    Raises:
        run_fail: An error occurred in construct gate node

    """
@overload
def H(qubit_list: QVec) -> QCircuit:
    """H(*args, **kwargs)
    Overloaded function.

    1. H(qubit: pyQPanda.Qubit) -> pyQPanda.QGate

    Create a H gate

    Args:
        qubit : quantum gate operate qubit

    Returns:
        a H gate node
    Raises:
        run_fail: An error occurred in construct gate node


    2. H(qubit_list: pyQPanda.QVec) -> pyQPanda.QCircuit

    Create a H gate

    Args:
        qubit_list: quantum gate operate qubits list

    Returns:
        a H gate node
    Raises:
        run_fail: An error occurred construct in gate node


    3. H(qubit_addr: int) -> pyQPanda.QGate

    Create a H gate

    Args:
        qubit_addr: quantum gate operate qubits addr

    Returns:
        a H gate node
    Raises:
        run_fail: An error occurred in construct gate node


    4. H(qubit_addr_list: List[int]) -> pyQPanda.QCircuit

    Create a H gate

    Args:
        qubit_list_addr: quantum gate  qubits list addr

    Returns:
        a H gate node
    Raises:
        run_fail: An error occurred in construct gate node

    """
@overload
def H(qubit_addr: int) -> QGate:
    """H(*args, **kwargs)
    Overloaded function.

    1. H(qubit: pyQPanda.Qubit) -> pyQPanda.QGate

    Create a H gate

    Args:
        qubit : quantum gate operate qubit

    Returns:
        a H gate node
    Raises:
        run_fail: An error occurred in construct gate node


    2. H(qubit_list: pyQPanda.QVec) -> pyQPanda.QCircuit

    Create a H gate

    Args:
        qubit_list: quantum gate operate qubits list

    Returns:
        a H gate node
    Raises:
        run_fail: An error occurred construct in gate node


    3. H(qubit_addr: int) -> pyQPanda.QGate

    Create a H gate

    Args:
        qubit_addr: quantum gate operate qubits addr

    Returns:
        a H gate node
    Raises:
        run_fail: An error occurred in construct gate node


    4. H(qubit_addr_list: List[int]) -> pyQPanda.QCircuit

    Create a H gate

    Args:
        qubit_list_addr: quantum gate  qubits list addr

    Returns:
        a H gate node
    Raises:
        run_fail: An error occurred in construct gate node

    """
@overload
def H(qubit_addr_list: list[int]) -> QCircuit:
    """H(*args, **kwargs)
    Overloaded function.

    1. H(qubit: pyQPanda.Qubit) -> pyQPanda.QGate

    Create a H gate

    Args:
        qubit : quantum gate operate qubit

    Returns:
        a H gate node
    Raises:
        run_fail: An error occurred in construct gate node


    2. H(qubit_list: pyQPanda.QVec) -> pyQPanda.QCircuit

    Create a H gate

    Args:
        qubit_list: quantum gate operate qubits list

    Returns:
        a H gate node
    Raises:
        run_fail: An error occurred construct in gate node


    3. H(qubit_addr: int) -> pyQPanda.QGate

    Create a H gate

    Args:
        qubit_addr: quantum gate operate qubits addr

    Returns:
        a H gate node
    Raises:
        run_fail: An error occurred in construct gate node


    4. H(qubit_addr_list: List[int]) -> pyQPanda.QCircuit

    Create a H gate

    Args:
        qubit_list_addr: quantum gate  qubits list addr

    Returns:
        a H gate node
    Raises:
        run_fail: An error occurred in construct gate node

    """
def HHL_solve_linear_equations(matrix_A: list[complex], data_b: list[float], precision_cnt: int = ...) -> list[complex]:
    """HHL_solve_linear_equations(matrix_A: List[complex], data_b: List[float], precision_cnt: int = 0) -> List[complex]

    Use HHL algorithm to solve the target linear systems of equations : Ax = b

    Args:
        matrix_A: a unitary matrix or Hermitian N*N matrix with N = 2 ^ n

        data_b: a given vector

        precision_cnt: The count of digits after the decimal point
                       default is 0, indicates that there are only integer solutions.

    Returns:
        QStat The solution of equation, i.e.x for Ax = b

    Notes:
        The higher the precision is, the more qubit number and circuit - depth will be,
        for example: 1 - bit precision, 4 additional qubits are required,
        for 2 - bit precision, we need 7 additional qubits, and so on.
    """
@overload
def I(qubit: Qubit) -> QGate:
    """I(*args, **kwargs)
    Overloaded function.

    1. I(qubit: pyQPanda.Qubit) -> pyQPanda.QGate

    Create a I gate

    Args:
        qubit : quantum gate operate qubit

    Returns:
        a I gate node
    Raises:
        run_fail: An error occurred in construct gate node


    2. I(qubit_list: pyQPanda.QVec) -> pyQPanda.QCircuit

    Create a I gate

    Args:
        qubit_list: quantum gate operate qubits list

    Returns:
        a I gate node
    Raises:
        run_fail: An error occurred construct in gate node


    3. I(qubit_addr: int) -> pyQPanda.QGate

    Create a I gate

    Args:
        qubit_addr: quantum gate operate qubits addr

    Returns:
        a I gate node
    Raises:
        run_fail: An error occurred in construct gate node


    4. I(qubit_addr_list: List[int]) -> pyQPanda.QCircuit

    Create a I gate

    Args:
        qubit_list_addr: quantum gate  qubits list addr

    Returns:
        a I gate node
    Raises:
        run_fail: An error occurred in construct gate node

    """
@overload
def I(qubit_list: QVec) -> QCircuit:
    """I(*args, **kwargs)
    Overloaded function.

    1. I(qubit: pyQPanda.Qubit) -> pyQPanda.QGate

    Create a I gate

    Args:
        qubit : quantum gate operate qubit

    Returns:
        a I gate node
    Raises:
        run_fail: An error occurred in construct gate node


    2. I(qubit_list: pyQPanda.QVec) -> pyQPanda.QCircuit

    Create a I gate

    Args:
        qubit_list: quantum gate operate qubits list

    Returns:
        a I gate node
    Raises:
        run_fail: An error occurred construct in gate node


    3. I(qubit_addr: int) -> pyQPanda.QGate

    Create a I gate

    Args:
        qubit_addr: quantum gate operate qubits addr

    Returns:
        a I gate node
    Raises:
        run_fail: An error occurred in construct gate node


    4. I(qubit_addr_list: List[int]) -> pyQPanda.QCircuit

    Create a I gate

    Args:
        qubit_list_addr: quantum gate  qubits list addr

    Returns:
        a I gate node
    Raises:
        run_fail: An error occurred in construct gate node

    """
@overload
def I(qubit_addr: int) -> QGate:
    """I(*args, **kwargs)
    Overloaded function.

    1. I(qubit: pyQPanda.Qubit) -> pyQPanda.QGate

    Create a I gate

    Args:
        qubit : quantum gate operate qubit

    Returns:
        a I gate node
    Raises:
        run_fail: An error occurred in construct gate node


    2. I(qubit_list: pyQPanda.QVec) -> pyQPanda.QCircuit

    Create a I gate

    Args:
        qubit_list: quantum gate operate qubits list

    Returns:
        a I gate node
    Raises:
        run_fail: An error occurred construct in gate node


    3. I(qubit_addr: int) -> pyQPanda.QGate

    Create a I gate

    Args:
        qubit_addr: quantum gate operate qubits addr

    Returns:
        a I gate node
    Raises:
        run_fail: An error occurred in construct gate node


    4. I(qubit_addr_list: List[int]) -> pyQPanda.QCircuit

    Create a I gate

    Args:
        qubit_list_addr: quantum gate  qubits list addr

    Returns:
        a I gate node
    Raises:
        run_fail: An error occurred in construct gate node

    """
@overload
def I(qubit_addr_list: list[int]) -> QCircuit:
    """I(*args, **kwargs)
    Overloaded function.

    1. I(qubit: pyQPanda.Qubit) -> pyQPanda.QGate

    Create a I gate

    Args:
        qubit : quantum gate operate qubit

    Returns:
        a I gate node
    Raises:
        run_fail: An error occurred in construct gate node


    2. I(qubit_list: pyQPanda.QVec) -> pyQPanda.QCircuit

    Create a I gate

    Args:
        qubit_list: quantum gate operate qubits list

    Returns:
        a I gate node
    Raises:
        run_fail: An error occurred construct in gate node


    3. I(qubit_addr: int) -> pyQPanda.QGate

    Create a I gate

    Args:
        qubit_addr: quantum gate operate qubits addr

    Returns:
        a I gate node
    Raises:
        run_fail: An error occurred in construct gate node


    4. I(qubit_addr_list: List[int]) -> pyQPanda.QCircuit

    Create a I gate

    Args:
        qubit_list_addr: quantum gate  qubits list addr

    Returns:
        a I gate node
    Raises:
        run_fail: An error occurred in construct gate node

    """
def IDLE(qubit: Qubit, param: int) -> QGate:
    """IDLE(qubit: pyQPanda.Qubit, param: int) -> pyQPanda.QGate"""
def MAJ(arg0: Qubit, arg1: Qubit, arg2: Qubit) -> QCircuit:
    """MAJ(arg0: pyQPanda.Qubit, arg1: pyQPanda.Qubit, arg2: pyQPanda.Qubit) -> pyQPanda.QCircuit

    Quantum adder MAJ module
    """
def MAJ2(arg0: QVec, arg1: QVec, arg2: Qubit) -> QCircuit:
    """MAJ2(arg0: pyQPanda.QVec, arg1: pyQPanda.QVec, arg2: pyQPanda.Qubit) -> pyQPanda.QCircuit

    Quantum adder MAJ2 module
    """
@overload
def MS(first_qubit: Qubit, second_qubit: Qubit) -> QGate:
    """MS(*args, **kwargs)
    Overloaded function.

    1. MS(first_qubit: pyQPanda.Qubit, second_qubit: pyQPanda.Qubit) -> pyQPanda.QGate

    Returns:
        a MS gate node
    Raises:
        run_fail: An error occurred in construct gate node


    2. MS(first_qubit_list: pyQPanda.QVec, second_qubit_list: pyQPanda.QVec) -> pyQPanda.QCircuit

    Returns:
        a MS gate node
    Raises:
        run_fail: An error occurred in construct gate node


    3. MS(first_qubit_addr: int, second_qubit_addr: int) -> pyQPanda.QGate

    4. MS(first_qubit_addr_list: List[int], second_qubit_addr_list: List[int]) -> pyQPanda.QCircuit

    Returns:
        a MS gate node
    Raises:
        run_fail: An error occurred in construct gate node

    """
@overload
def MS(first_qubit_list: QVec, second_qubit_list: QVec) -> QCircuit:
    """MS(*args, **kwargs)
    Overloaded function.

    1. MS(first_qubit: pyQPanda.Qubit, second_qubit: pyQPanda.Qubit) -> pyQPanda.QGate

    Returns:
        a MS gate node
    Raises:
        run_fail: An error occurred in construct gate node


    2. MS(first_qubit_list: pyQPanda.QVec, second_qubit_list: pyQPanda.QVec) -> pyQPanda.QCircuit

    Returns:
        a MS gate node
    Raises:
        run_fail: An error occurred in construct gate node


    3. MS(first_qubit_addr: int, second_qubit_addr: int) -> pyQPanda.QGate

    4. MS(first_qubit_addr_list: List[int], second_qubit_addr_list: List[int]) -> pyQPanda.QCircuit

    Returns:
        a MS gate node
    Raises:
        run_fail: An error occurred in construct gate node

    """
@overload
def MS(first_qubit_addr: int, second_qubit_addr: int) -> QGate:
    """MS(*args, **kwargs)
    Overloaded function.

    1. MS(first_qubit: pyQPanda.Qubit, second_qubit: pyQPanda.Qubit) -> pyQPanda.QGate

    Returns:
        a MS gate node
    Raises:
        run_fail: An error occurred in construct gate node


    2. MS(first_qubit_list: pyQPanda.QVec, second_qubit_list: pyQPanda.QVec) -> pyQPanda.QCircuit

    Returns:
        a MS gate node
    Raises:
        run_fail: An error occurred in construct gate node


    3. MS(first_qubit_addr: int, second_qubit_addr: int) -> pyQPanda.QGate

    4. MS(first_qubit_addr_list: List[int], second_qubit_addr_list: List[int]) -> pyQPanda.QCircuit

    Returns:
        a MS gate node
    Raises:
        run_fail: An error occurred in construct gate node

    """
@overload
def MS(first_qubit_addr_list: list[int], second_qubit_addr_list: list[int]) -> QCircuit:
    """MS(*args, **kwargs)
    Overloaded function.

    1. MS(first_qubit: pyQPanda.Qubit, second_qubit: pyQPanda.Qubit) -> pyQPanda.QGate

    Returns:
        a MS gate node
    Raises:
        run_fail: An error occurred in construct gate node


    2. MS(first_qubit_list: pyQPanda.QVec, second_qubit_list: pyQPanda.QVec) -> pyQPanda.QCircuit

    Returns:
        a MS gate node
    Raises:
        run_fail: An error occurred in construct gate node


    3. MS(first_qubit_addr: int, second_qubit_addr: int) -> pyQPanda.QGate

    4. MS(first_qubit_addr_list: List[int], second_qubit_addr_list: List[int]) -> pyQPanda.QCircuit

    Returns:
        a MS gate node
    Raises:
        run_fail: An error occurred in construct gate node

    """
@overload
def Measure(qubit: Qubit, cbit: ClassicalCondition) -> QMeasure:
    """Measure(*args, **kwargs)
    Overloaded function.

    1. Measure(qubit: pyQPanda.Qubit, cbit: pyQPanda.ClassicalCondition) -> pyQPanda.QMeasure

    Create a measure node.

    Args:
        qubit: the qubit to be measured.

        cbit: classical bit that stores the quantum measurement result.

    Returns:
        a quantum measure node.


    2. Measure(qubit: pyQPanda.Qubit, cbit: pyQPanda.CBit) -> pyQPanda.QMeasure

    Create a measure node.

    Args:
        qubit: the qubit to be measured.

        cbit: classical bit that stores the quantum measurement result.

    Returns:
        a quantum measure node.


    3. Measure(qubit_addr: int, cbit_addr: int) -> pyQPanda.QMeasure

    Create a measure node.

    Args:
        qubit_addr: address of the qubit to be measured.

        cbit_addr: address of the classical bit that stores the quantum measurement result.

    Returns:
        a quantum measure node.

    """
@overload
def Measure(qubit: Qubit, cbit: CBit) -> QMeasure:
    """Measure(*args, **kwargs)
    Overloaded function.

    1. Measure(qubit: pyQPanda.Qubit, cbit: pyQPanda.ClassicalCondition) -> pyQPanda.QMeasure

    Create a measure node.

    Args:
        qubit: the qubit to be measured.

        cbit: classical bit that stores the quantum measurement result.

    Returns:
        a quantum measure node.


    2. Measure(qubit: pyQPanda.Qubit, cbit: pyQPanda.CBit) -> pyQPanda.QMeasure

    Create a measure node.

    Args:
        qubit: the qubit to be measured.

        cbit: classical bit that stores the quantum measurement result.

    Returns:
        a quantum measure node.


    3. Measure(qubit_addr: int, cbit_addr: int) -> pyQPanda.QMeasure

    Create a measure node.

    Args:
        qubit_addr: address of the qubit to be measured.

        cbit_addr: address of the classical bit that stores the quantum measurement result.

    Returns:
        a quantum measure node.

    """
@overload
def Measure(qubit_addr: int, cbit_addr: int) -> QMeasure:
    """Measure(*args, **kwargs)
    Overloaded function.

    1. Measure(qubit: pyQPanda.Qubit, cbit: pyQPanda.ClassicalCondition) -> pyQPanda.QMeasure

    Create a measure node.

    Args:
        qubit: the qubit to be measured.

        cbit: classical bit that stores the quantum measurement result.

    Returns:
        a quantum measure node.


    2. Measure(qubit: pyQPanda.Qubit, cbit: pyQPanda.CBit) -> pyQPanda.QMeasure

    Create a measure node.

    Args:
        qubit: the qubit to be measured.

        cbit: classical bit that stores the quantum measurement result.

    Returns:
        a quantum measure node.


    3. Measure(qubit_addr: int, cbit_addr: int) -> pyQPanda.QMeasure

    Create a measure node.

    Args:
        qubit_addr: address of the qubit to be measured.

        cbit_addr: address of the classical bit that stores the quantum measurement result.

    Returns:
        a quantum measure node.

    """
@overload
def OBMT_mapping(prog: QProg, quantum_machine: QuantumMachine, b_optimization: bool = ..., max_partial: int = ..., max_children: int = ..., config_data: str = ...) -> QProg:
    """OBMT_mapping(*args, **kwargs)
    Overloaded function.

    1. OBMT_mapping(prog: pyQPanda.QProg, quantum_machine: pyQPanda.QuantumMachine, b_optimization: bool = False, max_partial: int = 4294967295, max_children: int = 4294967295, config_data: str = 'QPandaConfig.json') -> pyQPanda.QProg

    OPT_BMT mapping

    Args:
        prog: the target prog

        quantum_machine: quantum machine

        b_optimization: whether open the optimization

        max_partial: Limits the max number of partial solutions per step, There is no limit by default

        max_children: Limits the max number of candidate - solutions per double gate, There is no limit by default

        config_data: config data, @See JsonConfigParam::load_config()

    Returns:
        mapped quantum program

    2. OBMT_mapping(prog: pyQPanda.QProg, quantum_machine: pyQPanda.QuantumMachine, b_optimization: bool, arch_matrix: numpy.ndarray[numpy.float64[m, n]]) -> pyQPanda.QProg

    OPT_BMT mapping

    Args:
        prog: the target prog

        quantum_machine: quantum machine

        b_optimization: whether open the optimization

        arch_matrix: arch graph matrix

    Returns:
        mapped quantum program
    """
@overload
def OBMT_mapping(prog: QProg, quantum_machine: QuantumMachine, b_optimization: bool, arch_matrix: numpy.ndarray[numpy.float64[m, n]]) -> QProg:
    """OBMT_mapping(*args, **kwargs)
    Overloaded function.

    1. OBMT_mapping(prog: pyQPanda.QProg, quantum_machine: pyQPanda.QuantumMachine, b_optimization: bool = False, max_partial: int = 4294967295, max_children: int = 4294967295, config_data: str = 'QPandaConfig.json') -> pyQPanda.QProg

    OPT_BMT mapping

    Args:
        prog: the target prog

        quantum_machine: quantum machine

        b_optimization: whether open the optimization

        max_partial: Limits the max number of partial solutions per step, There is no limit by default

        max_children: Limits the max number of candidate - solutions per double gate, There is no limit by default

        config_data: config data, @See JsonConfigParam::load_config()

    Returns:
        mapped quantum program

    2. OBMT_mapping(prog: pyQPanda.QProg, quantum_machine: pyQPanda.QuantumMachine, b_optimization: bool, arch_matrix: numpy.ndarray[numpy.float64[m, n]]) -> pyQPanda.QProg

    OPT_BMT mapping

    Args:
        prog: the target prog

        quantum_machine: quantum machine

        b_optimization: whether open the optimization

        arch_matrix: arch graph matrix

    Returns:
        mapped quantum program
    """
@overload
def P(qubit: Qubit, angle: float) -> QGate:
    """P(*args, **kwargs)
    Overloaded function.

    1. P(qubit: pyQPanda.Qubit, angle: float) -> pyQPanda.QGate

    Returns:
        a P gate node
    Raises:
        run_fail: An error occurred in construct gate node


    2. P(qubit_list: pyQPanda.QVec, angle: float) -> pyQPanda.QCircuit

    Create a P gate
    Args:
        qubit_list: quantum gate operate qubits list
        args : quantum gate angles

    Returns:
        a P gate node
    Raises:
        run_fail: An error occurred construct in gate node


    3. P(qubit_addr: int, angle: float) -> pyQPanda.QGate

    Create a P gate
    Args:
        qubit_addr: quantum gate operate qubits addr
        args : quantum gate angles

    Returns:
        a P gate node
    Raises:
        run_fail: An error occurred in construct gate node


    4. P(qubit_addr_list: List[int], angle: float) -> pyQPanda.QCircuit

    Create a P gate
    Args:
        qubit_list_addr: quantum gate  qubits list addr
        args : quantum gate angles

    Returns:
        a P gate node
    Raises:
        run_fail: An error occurred in construct gate node

    """
@overload
def P(qubit_list: QVec, angle: float) -> QCircuit:
    """P(*args, **kwargs)
    Overloaded function.

    1. P(qubit: pyQPanda.Qubit, angle: float) -> pyQPanda.QGate

    Returns:
        a P gate node
    Raises:
        run_fail: An error occurred in construct gate node


    2. P(qubit_list: pyQPanda.QVec, angle: float) -> pyQPanda.QCircuit

    Create a P gate
    Args:
        qubit_list: quantum gate operate qubits list
        args : quantum gate angles

    Returns:
        a P gate node
    Raises:
        run_fail: An error occurred construct in gate node


    3. P(qubit_addr: int, angle: float) -> pyQPanda.QGate

    Create a P gate
    Args:
        qubit_addr: quantum gate operate qubits addr
        args : quantum gate angles

    Returns:
        a P gate node
    Raises:
        run_fail: An error occurred in construct gate node


    4. P(qubit_addr_list: List[int], angle: float) -> pyQPanda.QCircuit

    Create a P gate
    Args:
        qubit_list_addr: quantum gate  qubits list addr
        args : quantum gate angles

    Returns:
        a P gate node
    Raises:
        run_fail: An error occurred in construct gate node

    """
@overload
def P(qubit_addr: int, angle: float) -> QGate:
    """P(*args, **kwargs)
    Overloaded function.

    1. P(qubit: pyQPanda.Qubit, angle: float) -> pyQPanda.QGate

    Returns:
        a P gate node
    Raises:
        run_fail: An error occurred in construct gate node


    2. P(qubit_list: pyQPanda.QVec, angle: float) -> pyQPanda.QCircuit

    Create a P gate
    Args:
        qubit_list: quantum gate operate qubits list
        args : quantum gate angles

    Returns:
        a P gate node
    Raises:
        run_fail: An error occurred construct in gate node


    3. P(qubit_addr: int, angle: float) -> pyQPanda.QGate

    Create a P gate
    Args:
        qubit_addr: quantum gate operate qubits addr
        args : quantum gate angles

    Returns:
        a P gate node
    Raises:
        run_fail: An error occurred in construct gate node


    4. P(qubit_addr_list: List[int], angle: float) -> pyQPanda.QCircuit

    Create a P gate
    Args:
        qubit_list_addr: quantum gate  qubits list addr
        args : quantum gate angles

    Returns:
        a P gate node
    Raises:
        run_fail: An error occurred in construct gate node

    """
@overload
def P(qubit_addr_list: list[int], angle: float) -> QCircuit:
    """P(*args, **kwargs)
    Overloaded function.

    1. P(qubit: pyQPanda.Qubit, angle: float) -> pyQPanda.QGate

    Returns:
        a P gate node
    Raises:
        run_fail: An error occurred in construct gate node


    2. P(qubit_list: pyQPanda.QVec, angle: float) -> pyQPanda.QCircuit

    Create a P gate
    Args:
        qubit_list: quantum gate operate qubits list
        args : quantum gate angles

    Returns:
        a P gate node
    Raises:
        run_fail: An error occurred construct in gate node


    3. P(qubit_addr: int, angle: float) -> pyQPanda.QGate

    Create a P gate
    Args:
        qubit_addr: quantum gate operate qubits addr
        args : quantum gate angles

    Returns:
        a P gate node
    Raises:
        run_fail: An error occurred in construct gate node


    4. P(qubit_addr_list: List[int], angle: float) -> pyQPanda.QCircuit

    Create a P gate
    Args:
        qubit_list_addr: quantum gate  qubits list addr
        args : quantum gate angles

    Returns:
        a P gate node
    Raises:
        run_fail: An error occurred in construct gate node

    """
def PMeasure(arg0: QVec, arg1: int) -> list[tuple[int, float]]:
    """PMeasure(arg0: pyQPanda.QVec, arg1: int) -> List[Tuple[int, float]]

    Deprecated, use pmeasure instead.

    Args:
        QVec: pmeasure qubits list.

        select_num: result select num.

    Returns:
        result: pmeasure qubits result.

    """
def PMeasure_no_index(arg0: QVec) -> list[float]:
    """PMeasure_no_index(arg0: pyQPanda.QVec) -> List[float]

    Deprecated, use pmeasure_no_index instead.

    Args:
        QVec: pmeasure qubits list.

    Returns:
        result: pmeasure qubits result.

    """
def QAdd(arg0: QVec, arg1: QVec, arg2: QVec) -> QCircuit:
    """QAdd(arg0: pyQPanda.QVec, arg1: pyQPanda.QVec, arg2: pyQPanda.QVec) -> pyQPanda.QCircuit

    Quantum adder that supports signed operations, but ignore carry
    """
def QAdder(arg0: QVec, arg1: QVec, arg2: Qubit, arg3: Qubit) -> QCircuit:
    """QAdder(arg0: pyQPanda.QVec, arg1: pyQPanda.QVec, arg2: pyQPanda.Qubit, arg3: pyQPanda.Qubit) -> pyQPanda.QCircuit

    Quantum adder with carry
    """
def QAdderIgnoreCarry(arg0: QVec, arg1: QVec, arg2: Qubit) -> QCircuit:
    """QAdderIgnoreCarry(arg0: pyQPanda.QVec, arg1: pyQPanda.QVec, arg2: pyQPanda.Qubit) -> pyQPanda.QCircuit


    Args:

        QVec: qubits list a
        QVec: qubits list b
        QVec: qubits list c
        Qubit: qubit

    Returns:

        result : circuit 

    Raises:
        run_fail: An error occurred in QAdderIgnoreCarry

    """
def QComplement(arg0: QVec, arg1: QVec) -> QCircuit:
    """QComplement(arg0: pyQPanda.QVec, arg1: pyQPanda.QVec) -> pyQPanda.QCircuit

    Convert quantum state to binary complement representation
    """
def QDiv(arg0: QVec, arg1: QVec, arg2: QVec, arg3: QVec, arg4: ClassicalCondition) -> QProg:
    """QDiv(arg0: pyQPanda.QVec, arg1: pyQPanda.QVec, arg2: pyQPanda.QVec, arg3: pyQPanda.QVec, arg4: pyQPanda.ClassicalCondition) -> pyQPanda.QProg

    Quantum division
    """
def QDivWithAccuracy(arg0: QVec, arg1: QVec, arg2: QVec, arg3: QVec, arg4: QVec, arg5: list[ClassicalCondition]) -> QProg:
    """QDivWithAccuracy(arg0: pyQPanda.QVec, arg1: pyQPanda.QVec, arg2: pyQPanda.QVec, arg3: pyQPanda.QVec, arg4: pyQPanda.QVec, arg5: List[pyQPanda.ClassicalCondition]) -> pyQPanda.QProg


    Args:

        QVec: qubits list a
        QVec: qubits list b
        QVec: qubits list c
        QVec: qubits list k
        QVec: qubits list f
        QVec: qubits list s
        list: ClassicalCondition list

    Returns:

        result : circuit 

    Raises:
        run_fail: An error occurred in QDivWithAccuracy

    """
def QDivider(a: QVec, b: QVec, c: QVec, k: QVec, t: ClassicalCondition) -> QProg:
    """QDivider(a: pyQPanda.QVec, b: pyQPanda.QVec, c: pyQPanda.QVec, k: pyQPanda.QVec, t: pyQPanda.ClassicalCondition) -> pyQPanda.QProg

    Quantum division, only supports positive division, and the highest position of a and b and c is sign bit
    """
def QDividerWithAccuracy(a: QVec, b: QVec, c: QVec, k: QVec, f: QVec, s: list[ClassicalCondition]) -> QProg:
    """QDividerWithAccuracy(a: pyQPanda.QVec, b: pyQPanda.QVec, c: pyQPanda.QVec, k: pyQPanda.QVec, f: pyQPanda.QVec, s: List[pyQPanda.ClassicalCondition]) -> pyQPanda.QProg


    Args:

        QVec: qubits list a
        QVec: qubits list b
        QVec: qubits list c
        QVec: qubits list k
        QVec: qubits list f
        QVec: qubits list s
        list: ClassicalCondition list

    Returns:

        result : circuit 

    Raises:
        run_fail: An error occurred in QDividerWithAccuracy

    """
@overload
def QDouble(first_qubit: Qubit, second_qubit: Qubit, matrix: list[complex]) -> QGate:
    """QDouble(*args, **kwargs)
    Overloaded function.

    1. QDouble(first_qubit: pyQPanda.Qubit, second_qubit: pyQPanda.Qubit, matrix: List[complex]) -> pyQPanda.QGate

    Returns:
        a QDouble gate node
    Raises:
        run_fail: An error occurred in construct gate node


    2. QDouble(first_qubit_list: pyQPanda.QVec, second_qubit_list: pyQPanda.QVec, matrix: List[complex]) -> pyQPanda.QCircuit

    Returns:
        a QDouble gate node
    Raises:
        run_fail: An error occurred in construct gate node


    3. QDouble(first_qubit_addr: int, second_qubit_addr: int, matrix: List[complex]) -> pyQPanda.QGate

    Returns:
        a QDouble gate node
    Raises:
        run_fail: An error occurred in construct gate node


    4. QDouble(first_qubit_addr_list: List[int], second_qubit_addr_list: List[int], matrix: List[complex]) -> pyQPanda.QCircuit

    Returns:
        a QDouble gate node
    Raises:
        run_fail: An error occurred in construct gate node

    """
@overload
def QDouble(first_qubit_list: QVec, second_qubit_list: QVec, matrix: list[complex]) -> QCircuit:
    """QDouble(*args, **kwargs)
    Overloaded function.

    1. QDouble(first_qubit: pyQPanda.Qubit, second_qubit: pyQPanda.Qubit, matrix: List[complex]) -> pyQPanda.QGate

    Returns:
        a QDouble gate node
    Raises:
        run_fail: An error occurred in construct gate node


    2. QDouble(first_qubit_list: pyQPanda.QVec, second_qubit_list: pyQPanda.QVec, matrix: List[complex]) -> pyQPanda.QCircuit

    Returns:
        a QDouble gate node
    Raises:
        run_fail: An error occurred in construct gate node


    3. QDouble(first_qubit_addr: int, second_qubit_addr: int, matrix: List[complex]) -> pyQPanda.QGate

    Returns:
        a QDouble gate node
    Raises:
        run_fail: An error occurred in construct gate node


    4. QDouble(first_qubit_addr_list: List[int], second_qubit_addr_list: List[int], matrix: List[complex]) -> pyQPanda.QCircuit

    Returns:
        a QDouble gate node
    Raises:
        run_fail: An error occurred in construct gate node

    """
@overload
def QDouble(first_qubit_addr: int, second_qubit_addr: int, matrix: list[complex]) -> QGate:
    """QDouble(*args, **kwargs)
    Overloaded function.

    1. QDouble(first_qubit: pyQPanda.Qubit, second_qubit: pyQPanda.Qubit, matrix: List[complex]) -> pyQPanda.QGate

    Returns:
        a QDouble gate node
    Raises:
        run_fail: An error occurred in construct gate node


    2. QDouble(first_qubit_list: pyQPanda.QVec, second_qubit_list: pyQPanda.QVec, matrix: List[complex]) -> pyQPanda.QCircuit

    Returns:
        a QDouble gate node
    Raises:
        run_fail: An error occurred in construct gate node


    3. QDouble(first_qubit_addr: int, second_qubit_addr: int, matrix: List[complex]) -> pyQPanda.QGate

    Returns:
        a QDouble gate node
    Raises:
        run_fail: An error occurred in construct gate node


    4. QDouble(first_qubit_addr_list: List[int], second_qubit_addr_list: List[int], matrix: List[complex]) -> pyQPanda.QCircuit

    Returns:
        a QDouble gate node
    Raises:
        run_fail: An error occurred in construct gate node

    """
@overload
def QDouble(first_qubit_addr_list: list[int], second_qubit_addr_list: list[int], matrix: list[complex]) -> QCircuit:
    """QDouble(*args, **kwargs)
    Overloaded function.

    1. QDouble(first_qubit: pyQPanda.Qubit, second_qubit: pyQPanda.Qubit, matrix: List[complex]) -> pyQPanda.QGate

    Returns:
        a QDouble gate node
    Raises:
        run_fail: An error occurred in construct gate node


    2. QDouble(first_qubit_list: pyQPanda.QVec, second_qubit_list: pyQPanda.QVec, matrix: List[complex]) -> pyQPanda.QCircuit

    Returns:
        a QDouble gate node
    Raises:
        run_fail: An error occurred in construct gate node


    3. QDouble(first_qubit_addr: int, second_qubit_addr: int, matrix: List[complex]) -> pyQPanda.QGate

    Returns:
        a QDouble gate node
    Raises:
        run_fail: An error occurred in construct gate node


    4. QDouble(first_qubit_addr_list: List[int], second_qubit_addr_list: List[int], matrix: List[complex]) -> pyQPanda.QCircuit

    Returns:
        a QDouble gate node
    Raises:
        run_fail: An error occurred in construct gate node

    """
def QFT(qubits: QVec) -> QCircuit:
    """QFT(qubits: pyQPanda.QVec) -> pyQPanda.QCircuit

    Build QFT quantum circuit

    Args:
        qvec: qubit list

    Returns:
        result : qft circuit
    Raises:
        run_fail: An error occurred in QFT

    """
def QMul(arg0: QVec, arg1: QVec, arg2: QVec, arg3: QVec) -> QCircuit:
    """QMul(arg0: pyQPanda.QVec, arg1: pyQPanda.QVec, arg2: pyQPanda.QVec, arg3: pyQPanda.QVec) -> pyQPanda.QCircuit

    Quantum multiplication
    """
def QMultiplier(arg0: QVec, arg1: QVec, arg2: QVec, arg3: QVec) -> QCircuit:
    """QMultiplier(arg0: pyQPanda.QVec, arg1: pyQPanda.QVec, arg2: pyQPanda.QVec, arg3: pyQPanda.QVec) -> pyQPanda.QCircuit

    Quantum multiplication, only supports positive multiplication
    """
def QOracle(qubit_list: QVec, matrix: numpy.ndarray[numpy.complex128[m, n]], tol: float = ...) -> QGate:
    """QOracle(qubit_list: pyQPanda.QVec, matrix: numpy.ndarray[numpy.complex128[m, n]], tol: float = 1e-10) -> pyQPanda.QGate

    Generate QOracle Gate.

    Args:
        qubit_list: gate in qubit list.

        matrix: gate operator matrix.

    Returns:
        Oracle gate.

    """
def QPE(control_qubits: QVec, target_qubits: QVec, matrix: list[complex], b_estimate_eigenvalue: bool = ...) -> QCircuit:
    """QPE(control_qubits: pyQPanda.QVec, target_qubits: pyQPanda.QVec, matrix: List[complex], b_estimate_eigenvalue: bool = False) -> pyQPanda.QCircuit

    Quantum phase estimation

    Args:
        control_qubits: control qubit list
        target_qubits: target qubit list
        matrix: matrix

    Returns:
        result : QPE circuit
    Raises:
        run_fail: An error occurred in QPE

    """
def QSub(arg0: QVec, arg1: QVec, arg2: QVec) -> QCircuit:
    """QSub(arg0: pyQPanda.QVec, arg1: pyQPanda.QVec, arg2: pyQPanda.QVec) -> pyQPanda.QCircuit

    Quantum subtraction
    """
@overload
def RX(qubit: Qubit, angle: float) -> QGate:
    """RX(*args, **kwargs)
    Overloaded function.

    1. RX(qubit: pyQPanda.Qubit, angle: float) -> pyQPanda.QGate

    Returns:
        a RX gate node
    Raises:
        run_fail: An error occurred in construct gate node


    2. RX(qubit_list: pyQPanda.QVec, angle: float) -> pyQPanda.QCircuit

    Create a RX gate
    Args:
        qubit_list: quantum gate operate qubits list
        args : quantum gate angles

    Returns:
        a RX gate node
    Raises:
        run_fail: An error occurred construct in gate node


    3. RX(qubit_addr: int, angle: float) -> pyQPanda.QGate

    Create a RX gate
    Args:
        qubit_addr: quantum gate operate qubits addr
        args : quantum gate angles

    Returns:
        a RX gate node
    Raises:
        run_fail: An error occurred in construct gate node


    4. RX(qubit_addr_list: List[int], angle: float) -> pyQPanda.QCircuit

    Create a RX gate
    Args:
        qubit_list_addr: quantum gate  qubits list addr
        args : quantum gate angles

    Returns:
        a RX gate node
    Raises:
        run_fail: An error occurred in construct gate node

    """
@overload
def RX(qubit_list: QVec, angle: float) -> QCircuit:
    """RX(*args, **kwargs)
    Overloaded function.

    1. RX(qubit: pyQPanda.Qubit, angle: float) -> pyQPanda.QGate

    Returns:
        a RX gate node
    Raises:
        run_fail: An error occurred in construct gate node


    2. RX(qubit_list: pyQPanda.QVec, angle: float) -> pyQPanda.QCircuit

    Create a RX gate
    Args:
        qubit_list: quantum gate operate qubits list
        args : quantum gate angles

    Returns:
        a RX gate node
    Raises:
        run_fail: An error occurred construct in gate node


    3. RX(qubit_addr: int, angle: float) -> pyQPanda.QGate

    Create a RX gate
    Args:
        qubit_addr: quantum gate operate qubits addr
        args : quantum gate angles

    Returns:
        a RX gate node
    Raises:
        run_fail: An error occurred in construct gate node


    4. RX(qubit_addr_list: List[int], angle: float) -> pyQPanda.QCircuit

    Create a RX gate
    Args:
        qubit_list_addr: quantum gate  qubits list addr
        args : quantum gate angles

    Returns:
        a RX gate node
    Raises:
        run_fail: An error occurred in construct gate node

    """
@overload
def RX(qubit_addr: int, angle: float) -> QGate:
    """RX(*args, **kwargs)
    Overloaded function.

    1. RX(qubit: pyQPanda.Qubit, angle: float) -> pyQPanda.QGate

    Returns:
        a RX gate node
    Raises:
        run_fail: An error occurred in construct gate node


    2. RX(qubit_list: pyQPanda.QVec, angle: float) -> pyQPanda.QCircuit

    Create a RX gate
    Args:
        qubit_list: quantum gate operate qubits list
        args : quantum gate angles

    Returns:
        a RX gate node
    Raises:
        run_fail: An error occurred construct in gate node


    3. RX(qubit_addr: int, angle: float) -> pyQPanda.QGate

    Create a RX gate
    Args:
        qubit_addr: quantum gate operate qubits addr
        args : quantum gate angles

    Returns:
        a RX gate node
    Raises:
        run_fail: An error occurred in construct gate node


    4. RX(qubit_addr_list: List[int], angle: float) -> pyQPanda.QCircuit

    Create a RX gate
    Args:
        qubit_list_addr: quantum gate  qubits list addr
        args : quantum gate angles

    Returns:
        a RX gate node
    Raises:
        run_fail: An error occurred in construct gate node

    """
@overload
def RX(qubit_addr_list: list[int], angle: float) -> QCircuit:
    """RX(*args, **kwargs)
    Overloaded function.

    1. RX(qubit: pyQPanda.Qubit, angle: float) -> pyQPanda.QGate

    Returns:
        a RX gate node
    Raises:
        run_fail: An error occurred in construct gate node


    2. RX(qubit_list: pyQPanda.QVec, angle: float) -> pyQPanda.QCircuit

    Create a RX gate
    Args:
        qubit_list: quantum gate operate qubits list
        args : quantum gate angles

    Returns:
        a RX gate node
    Raises:
        run_fail: An error occurred construct in gate node


    3. RX(qubit_addr: int, angle: float) -> pyQPanda.QGate

    Create a RX gate
    Args:
        qubit_addr: quantum gate operate qubits addr
        args : quantum gate angles

    Returns:
        a RX gate node
    Raises:
        run_fail: An error occurred in construct gate node


    4. RX(qubit_addr_list: List[int], angle: float) -> pyQPanda.QCircuit

    Create a RX gate
    Args:
        qubit_list_addr: quantum gate  qubits list addr
        args : quantum gate angles

    Returns:
        a RX gate node
    Raises:
        run_fail: An error occurred in construct gate node

    """
@overload
def RXX(control_qubit: Qubit, target_qubit: Qubit, alpha_angle: float) -> QGate:
    """RXX(*args, **kwargs)
    Overloaded function.

    1. RXX(control_qubit: pyQPanda.Qubit, target_qubit: pyQPanda.Qubit, alpha_angle: float) -> pyQPanda.QGate

    Create a RXX gate

    Args:
        Qubit : control qubit
        Qubit : target qubit
        double: gate rotation angle theta

    Returns:
        a RXX gate node
    Raises:
        run_fail: An error occurred in construct gate node


    2. RXX(control_qubit_list: pyQPanda.QVec, target_qubit_list: pyQPanda.QVec, alpha_angle: float) -> pyQPanda.QCircuit

    Create a RXX gate

    Args:
        control_qubit_list : control qubit list
        target_qubit_list : target qubit list
        double: gate rotation angle theta

    Returns:
        a RXX gate node
    Raises:
        run_fail: An error occurred in construct gate node


    3. RXX(control_qubit_addr: int, target_qubit_addr: int, alpha_angle: float) -> pyQPanda.QGate

    Create a RXX gate

    Args:
        qubit addr : control qubit addr 
        qubit addr : target qubit addr 
        double: gate rotation angle theta

    Returns:
        a RXX gate node
    Raises:
        run_fail: An error occurred in construct gate node


    4. RXX(control_qubit_addr_list: List[int], target_qubit_addr_list: List[int], alpha_angle: float) -> pyQPanda.QCircuit

    Create a RXX gate

    Args:
        qubit addr list : control qubit addr list
        qubit addr list : target qubit addr list
        double: gate rotation angle theta

    Returns:
        a RXX gate node
    Raises:
        run_fail: An error occurred in construct gate node

    """
@overload
def RXX(control_qubit_list: QVec, target_qubit_list: QVec, alpha_angle: float) -> QCircuit:
    """RXX(*args, **kwargs)
    Overloaded function.

    1. RXX(control_qubit: pyQPanda.Qubit, target_qubit: pyQPanda.Qubit, alpha_angle: float) -> pyQPanda.QGate

    Create a RXX gate

    Args:
        Qubit : control qubit
        Qubit : target qubit
        double: gate rotation angle theta

    Returns:
        a RXX gate node
    Raises:
        run_fail: An error occurred in construct gate node


    2. RXX(control_qubit_list: pyQPanda.QVec, target_qubit_list: pyQPanda.QVec, alpha_angle: float) -> pyQPanda.QCircuit

    Create a RXX gate

    Args:
        control_qubit_list : control qubit list
        target_qubit_list : target qubit list
        double: gate rotation angle theta

    Returns:
        a RXX gate node
    Raises:
        run_fail: An error occurred in construct gate node


    3. RXX(control_qubit_addr: int, target_qubit_addr: int, alpha_angle: float) -> pyQPanda.QGate

    Create a RXX gate

    Args:
        qubit addr : control qubit addr 
        qubit addr : target qubit addr 
        double: gate rotation angle theta

    Returns:
        a RXX gate node
    Raises:
        run_fail: An error occurred in construct gate node


    4. RXX(control_qubit_addr_list: List[int], target_qubit_addr_list: List[int], alpha_angle: float) -> pyQPanda.QCircuit

    Create a RXX gate

    Args:
        qubit addr list : control qubit addr list
        qubit addr list : target qubit addr list
        double: gate rotation angle theta

    Returns:
        a RXX gate node
    Raises:
        run_fail: An error occurred in construct gate node

    """
@overload
def RXX(control_qubit_addr: int, target_qubit_addr: int, alpha_angle: float) -> QGate:
    """RXX(*args, **kwargs)
    Overloaded function.

    1. RXX(control_qubit: pyQPanda.Qubit, target_qubit: pyQPanda.Qubit, alpha_angle: float) -> pyQPanda.QGate

    Create a RXX gate

    Args:
        Qubit : control qubit
        Qubit : target qubit
        double: gate rotation angle theta

    Returns:
        a RXX gate node
    Raises:
        run_fail: An error occurred in construct gate node


    2. RXX(control_qubit_list: pyQPanda.QVec, target_qubit_list: pyQPanda.QVec, alpha_angle: float) -> pyQPanda.QCircuit

    Create a RXX gate

    Args:
        control_qubit_list : control qubit list
        target_qubit_list : target qubit list
        double: gate rotation angle theta

    Returns:
        a RXX gate node
    Raises:
        run_fail: An error occurred in construct gate node


    3. RXX(control_qubit_addr: int, target_qubit_addr: int, alpha_angle: float) -> pyQPanda.QGate

    Create a RXX gate

    Args:
        qubit addr : control qubit addr 
        qubit addr : target qubit addr 
        double: gate rotation angle theta

    Returns:
        a RXX gate node
    Raises:
        run_fail: An error occurred in construct gate node


    4. RXX(control_qubit_addr_list: List[int], target_qubit_addr_list: List[int], alpha_angle: float) -> pyQPanda.QCircuit

    Create a RXX gate

    Args:
        qubit addr list : control qubit addr list
        qubit addr list : target qubit addr list
        double: gate rotation angle theta

    Returns:
        a RXX gate node
    Raises:
        run_fail: An error occurred in construct gate node

    """
@overload
def RXX(control_qubit_addr_list: list[int], target_qubit_addr_list: list[int], alpha_angle: float) -> QCircuit:
    """RXX(*args, **kwargs)
    Overloaded function.

    1. RXX(control_qubit: pyQPanda.Qubit, target_qubit: pyQPanda.Qubit, alpha_angle: float) -> pyQPanda.QGate

    Create a RXX gate

    Args:
        Qubit : control qubit
        Qubit : target qubit
        double: gate rotation angle theta

    Returns:
        a RXX gate node
    Raises:
        run_fail: An error occurred in construct gate node


    2. RXX(control_qubit_list: pyQPanda.QVec, target_qubit_list: pyQPanda.QVec, alpha_angle: float) -> pyQPanda.QCircuit

    Create a RXX gate

    Args:
        control_qubit_list : control qubit list
        target_qubit_list : target qubit list
        double: gate rotation angle theta

    Returns:
        a RXX gate node
    Raises:
        run_fail: An error occurred in construct gate node


    3. RXX(control_qubit_addr: int, target_qubit_addr: int, alpha_angle: float) -> pyQPanda.QGate

    Create a RXX gate

    Args:
        qubit addr : control qubit addr 
        qubit addr : target qubit addr 
        double: gate rotation angle theta

    Returns:
        a RXX gate node
    Raises:
        run_fail: An error occurred in construct gate node


    4. RXX(control_qubit_addr_list: List[int], target_qubit_addr_list: List[int], alpha_angle: float) -> pyQPanda.QCircuit

    Create a RXX gate

    Args:
        qubit addr list : control qubit addr list
        qubit addr list : target qubit addr list
        double: gate rotation angle theta

    Returns:
        a RXX gate node
    Raises:
        run_fail: An error occurred in construct gate node

    """
@overload
def RY(qubit: Qubit, angle: float) -> QGate:
    """RY(*args, **kwargs)
    Overloaded function.

    1. RY(qubit: pyQPanda.Qubit, angle: float) -> pyQPanda.QGate

    Returns:
        a RY gate node
    Raises:
        run_fail: An error occurred in construct gate node


    2. RY(qubit_list: pyQPanda.QVec, angle: float) -> pyQPanda.QCircuit

    Create a RY gate
    Args:
        qubit_list: quantum gate operate qubits list
        args : quantum gate angles

    Returns:
        a RY gate node
    Raises:
        run_fail: An error occurred construct in gate node


    3. RY(qubit_addr: int, angle: float) -> pyQPanda.QGate

    Create a RY gate
    Args:
        qubit_addr: quantum gate operate qubits addr
        args : quantum gate angles

    Returns:
        a RY gate node
    Raises:
        run_fail: An error occurred in construct gate node


    4. RY(qubit_addr_list: List[int], angle: float) -> pyQPanda.QCircuit

    Create a RY gate
    Args:
        qubit_list_addr: quantum gate  qubits list addr
        args : quantum gate angles

    Returns:
        a RY gate node
    Raises:
        run_fail: An error occurred in construct gate node

    """
@overload
def RY(qubit_list: QVec, angle: float) -> QCircuit:
    """RY(*args, **kwargs)
    Overloaded function.

    1. RY(qubit: pyQPanda.Qubit, angle: float) -> pyQPanda.QGate

    Returns:
        a RY gate node
    Raises:
        run_fail: An error occurred in construct gate node


    2. RY(qubit_list: pyQPanda.QVec, angle: float) -> pyQPanda.QCircuit

    Create a RY gate
    Args:
        qubit_list: quantum gate operate qubits list
        args : quantum gate angles

    Returns:
        a RY gate node
    Raises:
        run_fail: An error occurred construct in gate node


    3. RY(qubit_addr: int, angle: float) -> pyQPanda.QGate

    Create a RY gate
    Args:
        qubit_addr: quantum gate operate qubits addr
        args : quantum gate angles

    Returns:
        a RY gate node
    Raises:
        run_fail: An error occurred in construct gate node


    4. RY(qubit_addr_list: List[int], angle: float) -> pyQPanda.QCircuit

    Create a RY gate
    Args:
        qubit_list_addr: quantum gate  qubits list addr
        args : quantum gate angles

    Returns:
        a RY gate node
    Raises:
        run_fail: An error occurred in construct gate node

    """
@overload
def RY(qubit_addr: int, angle: float) -> QGate:
    """RY(*args, **kwargs)
    Overloaded function.

    1. RY(qubit: pyQPanda.Qubit, angle: float) -> pyQPanda.QGate

    Returns:
        a RY gate node
    Raises:
        run_fail: An error occurred in construct gate node


    2. RY(qubit_list: pyQPanda.QVec, angle: float) -> pyQPanda.QCircuit

    Create a RY gate
    Args:
        qubit_list: quantum gate operate qubits list
        args : quantum gate angles

    Returns:
        a RY gate node
    Raises:
        run_fail: An error occurred construct in gate node


    3. RY(qubit_addr: int, angle: float) -> pyQPanda.QGate

    Create a RY gate
    Args:
        qubit_addr: quantum gate operate qubits addr
        args : quantum gate angles

    Returns:
        a RY gate node
    Raises:
        run_fail: An error occurred in construct gate node


    4. RY(qubit_addr_list: List[int], angle: float) -> pyQPanda.QCircuit

    Create a RY gate
    Args:
        qubit_list_addr: quantum gate  qubits list addr
        args : quantum gate angles

    Returns:
        a RY gate node
    Raises:
        run_fail: An error occurred in construct gate node

    """
@overload
def RY(qubit_addr_list: list[int], angle: float) -> QCircuit:
    """RY(*args, **kwargs)
    Overloaded function.

    1. RY(qubit: pyQPanda.Qubit, angle: float) -> pyQPanda.QGate

    Returns:
        a RY gate node
    Raises:
        run_fail: An error occurred in construct gate node


    2. RY(qubit_list: pyQPanda.QVec, angle: float) -> pyQPanda.QCircuit

    Create a RY gate
    Args:
        qubit_list: quantum gate operate qubits list
        args : quantum gate angles

    Returns:
        a RY gate node
    Raises:
        run_fail: An error occurred construct in gate node


    3. RY(qubit_addr: int, angle: float) -> pyQPanda.QGate

    Create a RY gate
    Args:
        qubit_addr: quantum gate operate qubits addr
        args : quantum gate angles

    Returns:
        a RY gate node
    Raises:
        run_fail: An error occurred in construct gate node


    4. RY(qubit_addr_list: List[int], angle: float) -> pyQPanda.QCircuit

    Create a RY gate
    Args:
        qubit_list_addr: quantum gate  qubits list addr
        args : quantum gate angles

    Returns:
        a RY gate node
    Raises:
        run_fail: An error occurred in construct gate node

    """
@overload
def RYY(control_qubit: Qubit, target_qubit: Qubit, alpha_angle: float) -> QGate:
    """RYY(*args, **kwargs)
    Overloaded function.

    1. RYY(control_qubit: pyQPanda.Qubit, target_qubit: pyQPanda.Qubit, alpha_angle: float) -> pyQPanda.QGate

    Create a RYY gate

    Args:
        Qubit : control qubit
        Qubit : target qubit
        double: gate rotation angle theta

    Returns:
        a RYY gate node
    Raises:
        run_fail: An error occurred in construct gate node


    2. RYY(control_qubit_list: pyQPanda.QVec, target_qubit_list: pyQPanda.QVec, alpha_angle: float) -> pyQPanda.QCircuit

    Create a RYY gate

    Args:
        control_qubit_list : control qubit list
        target_qubit_list : target qubit list
        double: gate rotation angle theta

    Returns:
        a RYY gate node
    Raises:
        run_fail: An error occurred in construct gate node


    3. RYY(control_qubit_addr: int, target_qubit_addr: int, alpha_angle: float) -> pyQPanda.QGate

    Create a RYY gate

    Args:
        qubit addr : control qubit addr 
        qubit addr : target qubit addr 
        double: gate rotation angle theta

    Returns:
        a RYY gate node
    Raises:
        run_fail: An error occurred in construct gate node


    4. RYY(control_qubit_addr_list: List[int], target_qubit_addr_list: List[int], alpha_angle: float) -> pyQPanda.QCircuit

    Create a RYY gate

    Args:
        qubit addr list : control qubit addr list
        qubit addr list : target qubit addr list
        double: gate rotation angle theta

    Returns:
        a RYY gate node
    Raises:
        run_fail: An error occurred in construct gate node

    """
@overload
def RYY(control_qubit_list: QVec, target_qubit_list: QVec, alpha_angle: float) -> QCircuit:
    """RYY(*args, **kwargs)
    Overloaded function.

    1. RYY(control_qubit: pyQPanda.Qubit, target_qubit: pyQPanda.Qubit, alpha_angle: float) -> pyQPanda.QGate

    Create a RYY gate

    Args:
        Qubit : control qubit
        Qubit : target qubit
        double: gate rotation angle theta

    Returns:
        a RYY gate node
    Raises:
        run_fail: An error occurred in construct gate node


    2. RYY(control_qubit_list: pyQPanda.QVec, target_qubit_list: pyQPanda.QVec, alpha_angle: float) -> pyQPanda.QCircuit

    Create a RYY gate

    Args:
        control_qubit_list : control qubit list
        target_qubit_list : target qubit list
        double: gate rotation angle theta

    Returns:
        a RYY gate node
    Raises:
        run_fail: An error occurred in construct gate node


    3. RYY(control_qubit_addr: int, target_qubit_addr: int, alpha_angle: float) -> pyQPanda.QGate

    Create a RYY gate

    Args:
        qubit addr : control qubit addr 
        qubit addr : target qubit addr 
        double: gate rotation angle theta

    Returns:
        a RYY gate node
    Raises:
        run_fail: An error occurred in construct gate node


    4. RYY(control_qubit_addr_list: List[int], target_qubit_addr_list: List[int], alpha_angle: float) -> pyQPanda.QCircuit

    Create a RYY gate

    Args:
        qubit addr list : control qubit addr list
        qubit addr list : target qubit addr list
        double: gate rotation angle theta

    Returns:
        a RYY gate node
    Raises:
        run_fail: An error occurred in construct gate node

    """
@overload
def RYY(control_qubit_addr: int, target_qubit_addr: int, alpha_angle: float) -> QGate:
    """RYY(*args, **kwargs)
    Overloaded function.

    1. RYY(control_qubit: pyQPanda.Qubit, target_qubit: pyQPanda.Qubit, alpha_angle: float) -> pyQPanda.QGate

    Create a RYY gate

    Args:
        Qubit : control qubit
        Qubit : target qubit
        double: gate rotation angle theta

    Returns:
        a RYY gate node
    Raises:
        run_fail: An error occurred in construct gate node


    2. RYY(control_qubit_list: pyQPanda.QVec, target_qubit_list: pyQPanda.QVec, alpha_angle: float) -> pyQPanda.QCircuit

    Create a RYY gate

    Args:
        control_qubit_list : control qubit list
        target_qubit_list : target qubit list
        double: gate rotation angle theta

    Returns:
        a RYY gate node
    Raises:
        run_fail: An error occurred in construct gate node


    3. RYY(control_qubit_addr: int, target_qubit_addr: int, alpha_angle: float) -> pyQPanda.QGate

    Create a RYY gate

    Args:
        qubit addr : control qubit addr 
        qubit addr : target qubit addr 
        double: gate rotation angle theta

    Returns:
        a RYY gate node
    Raises:
        run_fail: An error occurred in construct gate node


    4. RYY(control_qubit_addr_list: List[int], target_qubit_addr_list: List[int], alpha_angle: float) -> pyQPanda.QCircuit

    Create a RYY gate

    Args:
        qubit addr list : control qubit addr list
        qubit addr list : target qubit addr list
        double: gate rotation angle theta

    Returns:
        a RYY gate node
    Raises:
        run_fail: An error occurred in construct gate node

    """
@overload
def RYY(control_qubit_addr_list: list[int], target_qubit_addr_list: list[int], alpha_angle: float) -> QCircuit:
    """RYY(*args, **kwargs)
    Overloaded function.

    1. RYY(control_qubit: pyQPanda.Qubit, target_qubit: pyQPanda.Qubit, alpha_angle: float) -> pyQPanda.QGate

    Create a RYY gate

    Args:
        Qubit : control qubit
        Qubit : target qubit
        double: gate rotation angle theta

    Returns:
        a RYY gate node
    Raises:
        run_fail: An error occurred in construct gate node


    2. RYY(control_qubit_list: pyQPanda.QVec, target_qubit_list: pyQPanda.QVec, alpha_angle: float) -> pyQPanda.QCircuit

    Create a RYY gate

    Args:
        control_qubit_list : control qubit list
        target_qubit_list : target qubit list
        double: gate rotation angle theta

    Returns:
        a RYY gate node
    Raises:
        run_fail: An error occurred in construct gate node


    3. RYY(control_qubit_addr: int, target_qubit_addr: int, alpha_angle: float) -> pyQPanda.QGate

    Create a RYY gate

    Args:
        qubit addr : control qubit addr 
        qubit addr : target qubit addr 
        double: gate rotation angle theta

    Returns:
        a RYY gate node
    Raises:
        run_fail: An error occurred in construct gate node


    4. RYY(control_qubit_addr_list: List[int], target_qubit_addr_list: List[int], alpha_angle: float) -> pyQPanda.QCircuit

    Create a RYY gate

    Args:
        qubit addr list : control qubit addr list
        qubit addr list : target qubit addr list
        double: gate rotation angle theta

    Returns:
        a RYY gate node
    Raises:
        run_fail: An error occurred in construct gate node

    """
@overload
def RZ(qubit: Qubit, angle: float) -> QGate:
    """RZ(*args, **kwargs)
    Overloaded function.

    1. RZ(qubit: pyQPanda.Qubit, angle: float) -> pyQPanda.QGate

    Returns:
        a RZ gate node
    Raises:
        run_fail: An error occurred in construct gate node


    2. RZ(qubit_list: pyQPanda.QVec, angle: float) -> pyQPanda.QCircuit

    Create a RZ gate
    Args:
        qubit_list: quantum gate operate qubits list
        args : quantum gate angles

    Returns:
        a RZ gate node
    Raises:
        run_fail: An error occurred construct in gate node


    3. RZ(qubit_addr: int, angle: float) -> pyQPanda.QGate

    Create a RZ gate
    Args:
        qubit_addr: quantum gate operate qubits addr
        args : quantum gate angles

    Returns:
        a RZ gate node
    Raises:
        run_fail: An error occurred in construct gate node


    4. RZ(qubit_addr_list: List[int], angle: float) -> pyQPanda.QCircuit

    Create a RZ gate
    Args:
        qubit_list_addr: quantum gate  qubits list addr
        args : quantum gate angles

    Returns:
        a RZ gate node
    Raises:
        run_fail: An error occurred in construct gate node

    """
@overload
def RZ(qubit_list: QVec, angle: float) -> QCircuit:
    """RZ(*args, **kwargs)
    Overloaded function.

    1. RZ(qubit: pyQPanda.Qubit, angle: float) -> pyQPanda.QGate

    Returns:
        a RZ gate node
    Raises:
        run_fail: An error occurred in construct gate node


    2. RZ(qubit_list: pyQPanda.QVec, angle: float) -> pyQPanda.QCircuit

    Create a RZ gate
    Args:
        qubit_list: quantum gate operate qubits list
        args : quantum gate angles

    Returns:
        a RZ gate node
    Raises:
        run_fail: An error occurred construct in gate node


    3. RZ(qubit_addr: int, angle: float) -> pyQPanda.QGate

    Create a RZ gate
    Args:
        qubit_addr: quantum gate operate qubits addr
        args : quantum gate angles

    Returns:
        a RZ gate node
    Raises:
        run_fail: An error occurred in construct gate node


    4. RZ(qubit_addr_list: List[int], angle: float) -> pyQPanda.QCircuit

    Create a RZ gate
    Args:
        qubit_list_addr: quantum gate  qubits list addr
        args : quantum gate angles

    Returns:
        a RZ gate node
    Raises:
        run_fail: An error occurred in construct gate node

    """
@overload
def RZ(qubit_addr: int, angle: float) -> QGate:
    """RZ(*args, **kwargs)
    Overloaded function.

    1. RZ(qubit: pyQPanda.Qubit, angle: float) -> pyQPanda.QGate

    Returns:
        a RZ gate node
    Raises:
        run_fail: An error occurred in construct gate node


    2. RZ(qubit_list: pyQPanda.QVec, angle: float) -> pyQPanda.QCircuit

    Create a RZ gate
    Args:
        qubit_list: quantum gate operate qubits list
        args : quantum gate angles

    Returns:
        a RZ gate node
    Raises:
        run_fail: An error occurred construct in gate node


    3. RZ(qubit_addr: int, angle: float) -> pyQPanda.QGate

    Create a RZ gate
    Args:
        qubit_addr: quantum gate operate qubits addr
        args : quantum gate angles

    Returns:
        a RZ gate node
    Raises:
        run_fail: An error occurred in construct gate node


    4. RZ(qubit_addr_list: List[int], angle: float) -> pyQPanda.QCircuit

    Create a RZ gate
    Args:
        qubit_list_addr: quantum gate  qubits list addr
        args : quantum gate angles

    Returns:
        a RZ gate node
    Raises:
        run_fail: An error occurred in construct gate node

    """
@overload
def RZ(qubit_addr_list: list[int], angle: float) -> QCircuit:
    """RZ(*args, **kwargs)
    Overloaded function.

    1. RZ(qubit: pyQPanda.Qubit, angle: float) -> pyQPanda.QGate

    Returns:
        a RZ gate node
    Raises:
        run_fail: An error occurred in construct gate node


    2. RZ(qubit_list: pyQPanda.QVec, angle: float) -> pyQPanda.QCircuit

    Create a RZ gate
    Args:
        qubit_list: quantum gate operate qubits list
        args : quantum gate angles

    Returns:
        a RZ gate node
    Raises:
        run_fail: An error occurred construct in gate node


    3. RZ(qubit_addr: int, angle: float) -> pyQPanda.QGate

    Create a RZ gate
    Args:
        qubit_addr: quantum gate operate qubits addr
        args : quantum gate angles

    Returns:
        a RZ gate node
    Raises:
        run_fail: An error occurred in construct gate node


    4. RZ(qubit_addr_list: List[int], angle: float) -> pyQPanda.QCircuit

    Create a RZ gate
    Args:
        qubit_list_addr: quantum gate  qubits list addr
        args : quantum gate angles

    Returns:
        a RZ gate node
    Raises:
        run_fail: An error occurred in construct gate node

    """
@overload
def RZX(control_qubit: Qubit, target_qubit: Qubit, alpha_angle: float) -> QGate:
    """RZX(*args, **kwargs)
    Overloaded function.

    1. RZX(control_qubit: pyQPanda.Qubit, target_qubit: pyQPanda.Qubit, alpha_angle: float) -> pyQPanda.QGate

    Create a RZX gate

    Args:
        Qubit : control qubit
        Qubit : target qubit
        double: gate rotation angle theta

    Returns:
        a RZX gate node
    Raises:
        run_fail: An error occurred in construct gate node


    2. RZX(control_qubit_list: pyQPanda.QVec, target_qubit_list: pyQPanda.QVec, alpha_angle: float) -> pyQPanda.QCircuit

    Create a RZX gate

    Args:
        control_qubit_list : control qubit list
        target_qubit_list : target qubit list
        double: gate rotation angle theta

    Returns:
        a RZX gate node
    Raises:
        run_fail: An error occurred in construct gate node


    3. RZX(control_qubit_addr: int, target_qubit_addr: int, alpha_angle: float) -> pyQPanda.QGate

    Create a RZX gate

    Args:
        qubit addr : control qubit addr 
        qubit addr : target qubit addr 
        double: gate rotation angle theta

    Returns:
        a RZX gate node
    Raises:
        run_fail: An error occurred in construct gate node


    4. RZX(control_qubit_addr_list: List[int], target_qubit_addr_list: List[int], alpha_angle: float) -> pyQPanda.QCircuit

    Create a RZX gate

    Args:
        qubit addr list : control qubit addr list
        qubit addr list : target qubit addr list
        double: gate rotation angle theta

    Returns:
        a RZX gate node
    Raises:
        run_fail: An error occurred in construct gate node

    """
@overload
def RZX(control_qubit_list: QVec, target_qubit_list: QVec, alpha_angle: float) -> QCircuit:
    """RZX(*args, **kwargs)
    Overloaded function.

    1. RZX(control_qubit: pyQPanda.Qubit, target_qubit: pyQPanda.Qubit, alpha_angle: float) -> pyQPanda.QGate

    Create a RZX gate

    Args:
        Qubit : control qubit
        Qubit : target qubit
        double: gate rotation angle theta

    Returns:
        a RZX gate node
    Raises:
        run_fail: An error occurred in construct gate node


    2. RZX(control_qubit_list: pyQPanda.QVec, target_qubit_list: pyQPanda.QVec, alpha_angle: float) -> pyQPanda.QCircuit

    Create a RZX gate

    Args:
        control_qubit_list : control qubit list
        target_qubit_list : target qubit list
        double: gate rotation angle theta

    Returns:
        a RZX gate node
    Raises:
        run_fail: An error occurred in construct gate node


    3. RZX(control_qubit_addr: int, target_qubit_addr: int, alpha_angle: float) -> pyQPanda.QGate

    Create a RZX gate

    Args:
        qubit addr : control qubit addr 
        qubit addr : target qubit addr 
        double: gate rotation angle theta

    Returns:
        a RZX gate node
    Raises:
        run_fail: An error occurred in construct gate node


    4. RZX(control_qubit_addr_list: List[int], target_qubit_addr_list: List[int], alpha_angle: float) -> pyQPanda.QCircuit

    Create a RZX gate

    Args:
        qubit addr list : control qubit addr list
        qubit addr list : target qubit addr list
        double: gate rotation angle theta

    Returns:
        a RZX gate node
    Raises:
        run_fail: An error occurred in construct gate node

    """
@overload
def RZX(control_qubit_addr: int, target_qubit_addr: int, alpha_angle: float) -> QGate:
    """RZX(*args, **kwargs)
    Overloaded function.

    1. RZX(control_qubit: pyQPanda.Qubit, target_qubit: pyQPanda.Qubit, alpha_angle: float) -> pyQPanda.QGate

    Create a RZX gate

    Args:
        Qubit : control qubit
        Qubit : target qubit
        double: gate rotation angle theta

    Returns:
        a RZX gate node
    Raises:
        run_fail: An error occurred in construct gate node


    2. RZX(control_qubit_list: pyQPanda.QVec, target_qubit_list: pyQPanda.QVec, alpha_angle: float) -> pyQPanda.QCircuit

    Create a RZX gate

    Args:
        control_qubit_list : control qubit list
        target_qubit_list : target qubit list
        double: gate rotation angle theta

    Returns:
        a RZX gate node
    Raises:
        run_fail: An error occurred in construct gate node


    3. RZX(control_qubit_addr: int, target_qubit_addr: int, alpha_angle: float) -> pyQPanda.QGate

    Create a RZX gate

    Args:
        qubit addr : control qubit addr 
        qubit addr : target qubit addr 
        double: gate rotation angle theta

    Returns:
        a RZX gate node
    Raises:
        run_fail: An error occurred in construct gate node


    4. RZX(control_qubit_addr_list: List[int], target_qubit_addr_list: List[int], alpha_angle: float) -> pyQPanda.QCircuit

    Create a RZX gate

    Args:
        qubit addr list : control qubit addr list
        qubit addr list : target qubit addr list
        double: gate rotation angle theta

    Returns:
        a RZX gate node
    Raises:
        run_fail: An error occurred in construct gate node

    """
@overload
def RZX(control_qubit_addr_list: list[int], target_qubit_addr_list: list[int], alpha_angle: float) -> QCircuit:
    """RZX(*args, **kwargs)
    Overloaded function.

    1. RZX(control_qubit: pyQPanda.Qubit, target_qubit: pyQPanda.Qubit, alpha_angle: float) -> pyQPanda.QGate

    Create a RZX gate

    Args:
        Qubit : control qubit
        Qubit : target qubit
        double: gate rotation angle theta

    Returns:
        a RZX gate node
    Raises:
        run_fail: An error occurred in construct gate node


    2. RZX(control_qubit_list: pyQPanda.QVec, target_qubit_list: pyQPanda.QVec, alpha_angle: float) -> pyQPanda.QCircuit

    Create a RZX gate

    Args:
        control_qubit_list : control qubit list
        target_qubit_list : target qubit list
        double: gate rotation angle theta

    Returns:
        a RZX gate node
    Raises:
        run_fail: An error occurred in construct gate node


    3. RZX(control_qubit_addr: int, target_qubit_addr: int, alpha_angle: float) -> pyQPanda.QGate

    Create a RZX gate

    Args:
        qubit addr : control qubit addr 
        qubit addr : target qubit addr 
        double: gate rotation angle theta

    Returns:
        a RZX gate node
    Raises:
        run_fail: An error occurred in construct gate node


    4. RZX(control_qubit_addr_list: List[int], target_qubit_addr_list: List[int], alpha_angle: float) -> pyQPanda.QCircuit

    Create a RZX gate

    Args:
        qubit addr list : control qubit addr list
        qubit addr list : target qubit addr list
        double: gate rotation angle theta

    Returns:
        a RZX gate node
    Raises:
        run_fail: An error occurred in construct gate node

    """
@overload
def RZZ(control_qubit: Qubit, target_qubit: Qubit, alpha_angle: float) -> QGate:
    """RZZ(*args, **kwargs)
    Overloaded function.

    1. RZZ(control_qubit: pyQPanda.Qubit, target_qubit: pyQPanda.Qubit, alpha_angle: float) -> pyQPanda.QGate

    Create a RZZ gate

    Args:
        Qubit : control qubit
        Qubit : target qubit
        double: gate rotation angle theta

    Returns:
        a RZZ gate node
    Raises:
        run_fail: An error occurred in construct gate node


    2. RZZ(control_qubit_list: pyQPanda.QVec, target_qubit_list: pyQPanda.QVec, alpha_angle: float) -> pyQPanda.QCircuit

    Create a RZZ gate

    Args:
        control_qubit_list : control qubit list
        target_qubit_list : target qubit list
        double: gate rotation angle theta

    Returns:
        a RZZ gate node
    Raises:
        run_fail: An error occurred in construct gate node


    3. RZZ(control_qubit_addr: int, target_qubit_addr: int, alpha_angle: float) -> pyQPanda.QGate

    Create a RZZ gate

    Args:
        qubit addr : control qubit addr 
        qubit addr : target qubit addr 
        double: gate rotation angle theta

    Returns:
        a RZZ gate node
    Raises:
        run_fail: An error occurred in construct gate node


    4. RZZ(control_qubit_addr_list: List[int], target_qubit_addr_list: List[int], alpha_angle: float) -> pyQPanda.QCircuit

    Create a RZZ gate

    Args:
        qubit addr list : control qubit addr list
        qubit addr list : target qubit addr list
        double: gate rotation angle theta

    Returns:
        a RZZ gate node
    Raises:
        run_fail: An error occurred in construct gate node

    """
@overload
def RZZ(control_qubit_list: QVec, target_qubit_list: QVec, alpha_angle: float) -> QCircuit:
    """RZZ(*args, **kwargs)
    Overloaded function.

    1. RZZ(control_qubit: pyQPanda.Qubit, target_qubit: pyQPanda.Qubit, alpha_angle: float) -> pyQPanda.QGate

    Create a RZZ gate

    Args:
        Qubit : control qubit
        Qubit : target qubit
        double: gate rotation angle theta

    Returns:
        a RZZ gate node
    Raises:
        run_fail: An error occurred in construct gate node


    2. RZZ(control_qubit_list: pyQPanda.QVec, target_qubit_list: pyQPanda.QVec, alpha_angle: float) -> pyQPanda.QCircuit

    Create a RZZ gate

    Args:
        control_qubit_list : control qubit list
        target_qubit_list : target qubit list
        double: gate rotation angle theta

    Returns:
        a RZZ gate node
    Raises:
        run_fail: An error occurred in construct gate node


    3. RZZ(control_qubit_addr: int, target_qubit_addr: int, alpha_angle: float) -> pyQPanda.QGate

    Create a RZZ gate

    Args:
        qubit addr : control qubit addr 
        qubit addr : target qubit addr 
        double: gate rotation angle theta

    Returns:
        a RZZ gate node
    Raises:
        run_fail: An error occurred in construct gate node


    4. RZZ(control_qubit_addr_list: List[int], target_qubit_addr_list: List[int], alpha_angle: float) -> pyQPanda.QCircuit

    Create a RZZ gate

    Args:
        qubit addr list : control qubit addr list
        qubit addr list : target qubit addr list
        double: gate rotation angle theta

    Returns:
        a RZZ gate node
    Raises:
        run_fail: An error occurred in construct gate node

    """
@overload
def RZZ(control_qubit_addr: int, target_qubit_addr: int, alpha_angle: float) -> QGate:
    """RZZ(*args, **kwargs)
    Overloaded function.

    1. RZZ(control_qubit: pyQPanda.Qubit, target_qubit: pyQPanda.Qubit, alpha_angle: float) -> pyQPanda.QGate

    Create a RZZ gate

    Args:
        Qubit : control qubit
        Qubit : target qubit
        double: gate rotation angle theta

    Returns:
        a RZZ gate node
    Raises:
        run_fail: An error occurred in construct gate node


    2. RZZ(control_qubit_list: pyQPanda.QVec, target_qubit_list: pyQPanda.QVec, alpha_angle: float) -> pyQPanda.QCircuit

    Create a RZZ gate

    Args:
        control_qubit_list : control qubit list
        target_qubit_list : target qubit list
        double: gate rotation angle theta

    Returns:
        a RZZ gate node
    Raises:
        run_fail: An error occurred in construct gate node


    3. RZZ(control_qubit_addr: int, target_qubit_addr: int, alpha_angle: float) -> pyQPanda.QGate

    Create a RZZ gate

    Args:
        qubit addr : control qubit addr 
        qubit addr : target qubit addr 
        double: gate rotation angle theta

    Returns:
        a RZZ gate node
    Raises:
        run_fail: An error occurred in construct gate node


    4. RZZ(control_qubit_addr_list: List[int], target_qubit_addr_list: List[int], alpha_angle: float) -> pyQPanda.QCircuit

    Create a RZZ gate

    Args:
        qubit addr list : control qubit addr list
        qubit addr list : target qubit addr list
        double: gate rotation angle theta

    Returns:
        a RZZ gate node
    Raises:
        run_fail: An error occurred in construct gate node

    """
@overload
def RZZ(control_qubit_addr_list: list[int], target_qubit_addr_list: list[int], alpha_angle: float) -> QCircuit:
    """RZZ(*args, **kwargs)
    Overloaded function.

    1. RZZ(control_qubit: pyQPanda.Qubit, target_qubit: pyQPanda.Qubit, alpha_angle: float) -> pyQPanda.QGate

    Create a RZZ gate

    Args:
        Qubit : control qubit
        Qubit : target qubit
        double: gate rotation angle theta

    Returns:
        a RZZ gate node
    Raises:
        run_fail: An error occurred in construct gate node


    2. RZZ(control_qubit_list: pyQPanda.QVec, target_qubit_list: pyQPanda.QVec, alpha_angle: float) -> pyQPanda.QCircuit

    Create a RZZ gate

    Args:
        control_qubit_list : control qubit list
        target_qubit_list : target qubit list
        double: gate rotation angle theta

    Returns:
        a RZZ gate node
    Raises:
        run_fail: An error occurred in construct gate node


    3. RZZ(control_qubit_addr: int, target_qubit_addr: int, alpha_angle: float) -> pyQPanda.QGate

    Create a RZZ gate

    Args:
        qubit addr : control qubit addr 
        qubit addr : target qubit addr 
        double: gate rotation angle theta

    Returns:
        a RZZ gate node
    Raises:
        run_fail: An error occurred in construct gate node


    4. RZZ(control_qubit_addr_list: List[int], target_qubit_addr_list: List[int], alpha_angle: float) -> pyQPanda.QCircuit

    Create a RZZ gate

    Args:
        qubit addr list : control qubit addr list
        qubit addr list : target qubit addr list
        double: gate rotation angle theta

    Returns:
        a RZZ gate node
    Raises:
        run_fail: An error occurred in construct gate node

    """
@overload
def Reset(qubit: Qubit) -> QReset:
    """Reset(*args, **kwargs)
    Overloaded function.

    1. Reset(qubit: pyQPanda.Qubit) -> pyQPanda.QReset

    Create a Reset node.

    Args:
        qubit (Qubit *): The qubit to be reset.

    Returns:
        A Reset node representing the operation.


    2. Reset(qubit_addr: int) -> pyQPanda.QReset

    Create a Reset node.

    Args:
        qubit_addr (int): Address of the qubit to be reset.

    Returns:
        A Reset node representing the operation.

    """
@overload
def Reset(qubit_addr: int) -> QReset:
    """Reset(*args, **kwargs)
    Overloaded function.

    1. Reset(qubit: pyQPanda.Qubit) -> pyQPanda.QReset

    Create a Reset node.

    Args:
        qubit (Qubit *): The qubit to be reset.

    Returns:
        A Reset node representing the operation.


    2. Reset(qubit_addr: int) -> pyQPanda.QReset

    Create a Reset node.

    Args:
        qubit_addr (int): Address of the qubit to be reset.

    Returns:
        A Reset node representing the operation.

    """
@overload
def S(qubit: Qubit) -> QGate:
    """S(*args, **kwargs)
    Overloaded function.

    1. S(qubit: pyQPanda.Qubit) -> pyQPanda.QGate

    Create a S gate

    Args:
        qubit : quantum gate operate qubit

    Returns:
        a S gate node
    Raises:
        run_fail: An error occurred in construct gate node


    2. S(qubit_list: pyQPanda.QVec) -> pyQPanda.QCircuit

    Create a S gate

    Args:
        qubit_list: quantum gate operate qubits list

    Returns:
        a S gate node
    Raises:
        run_fail: An error occurred construct in gate node


    3. S(qubit_addr: int) -> pyQPanda.QGate

    Create a S gate

    Args:
        qubit_addr: quantum gate operate qubits addr

    Returns:
        a S gate node
    Raises:
        run_fail: An error occurred in construct gate node


    4. S(qubit_addr_list: List[int]) -> pyQPanda.QCircuit

    Create a S gate

    Args:
        qubit_list_addr: quantum gate  qubits list addr

    Returns:
        a S gate node
    Raises:
        run_fail: An error occurred in construct gate node

    """
@overload
def S(qubit_list: QVec) -> QCircuit:
    """S(*args, **kwargs)
    Overloaded function.

    1. S(qubit: pyQPanda.Qubit) -> pyQPanda.QGate

    Create a S gate

    Args:
        qubit : quantum gate operate qubit

    Returns:
        a S gate node
    Raises:
        run_fail: An error occurred in construct gate node


    2. S(qubit_list: pyQPanda.QVec) -> pyQPanda.QCircuit

    Create a S gate

    Args:
        qubit_list: quantum gate operate qubits list

    Returns:
        a S gate node
    Raises:
        run_fail: An error occurred construct in gate node


    3. S(qubit_addr: int) -> pyQPanda.QGate

    Create a S gate

    Args:
        qubit_addr: quantum gate operate qubits addr

    Returns:
        a S gate node
    Raises:
        run_fail: An error occurred in construct gate node


    4. S(qubit_addr_list: List[int]) -> pyQPanda.QCircuit

    Create a S gate

    Args:
        qubit_list_addr: quantum gate  qubits list addr

    Returns:
        a S gate node
    Raises:
        run_fail: An error occurred in construct gate node

    """
@overload
def S(qubit_addr: int) -> QGate:
    """S(*args, **kwargs)
    Overloaded function.

    1. S(qubit: pyQPanda.Qubit) -> pyQPanda.QGate

    Create a S gate

    Args:
        qubit : quantum gate operate qubit

    Returns:
        a S gate node
    Raises:
        run_fail: An error occurred in construct gate node


    2. S(qubit_list: pyQPanda.QVec) -> pyQPanda.QCircuit

    Create a S gate

    Args:
        qubit_list: quantum gate operate qubits list

    Returns:
        a S gate node
    Raises:
        run_fail: An error occurred construct in gate node


    3. S(qubit_addr: int) -> pyQPanda.QGate

    Create a S gate

    Args:
        qubit_addr: quantum gate operate qubits addr

    Returns:
        a S gate node
    Raises:
        run_fail: An error occurred in construct gate node


    4. S(qubit_addr_list: List[int]) -> pyQPanda.QCircuit

    Create a S gate

    Args:
        qubit_list_addr: quantum gate  qubits list addr

    Returns:
        a S gate node
    Raises:
        run_fail: An error occurred in construct gate node

    """
@overload
def S(qubit_addr_list: list[int]) -> QCircuit:
    """S(*args, **kwargs)
    Overloaded function.

    1. S(qubit: pyQPanda.Qubit) -> pyQPanda.QGate

    Create a S gate

    Args:
        qubit : quantum gate operate qubit

    Returns:
        a S gate node
    Raises:
        run_fail: An error occurred in construct gate node


    2. S(qubit_list: pyQPanda.QVec) -> pyQPanda.QCircuit

    Create a S gate

    Args:
        qubit_list: quantum gate operate qubits list

    Returns:
        a S gate node
    Raises:
        run_fail: An error occurred construct in gate node


    3. S(qubit_addr: int) -> pyQPanda.QGate

    Create a S gate

    Args:
        qubit_addr: quantum gate operate qubits addr

    Returns:
        a S gate node
    Raises:
        run_fail: An error occurred in construct gate node


    4. S(qubit_addr_list: List[int]) -> pyQPanda.QCircuit

    Create a S gate

    Args:
        qubit_list_addr: quantum gate  qubits list addr

    Returns:
        a S gate node
    Raises:
        run_fail: An error occurred in construct gate node

    """
@overload
def SWAP(first_qubit: Qubit, second_qubit: Qubit) -> QGate:
    """SWAP(*args, **kwargs)
    Overloaded function.

    1. SWAP(first_qubit: pyQPanda.Qubit, second_qubit: pyQPanda.Qubit) -> pyQPanda.QGate

    Returns:
        a SWAP gate node
    Raises:
        run_fail: An error occurred in construct gate node


    2. SWAP(first_qubit_list: pyQPanda.QVec, second_qubit_list: pyQPanda.QVec) -> pyQPanda.QCircuit

    Returns:
        a SWAP gate node
    Raises:
        run_fail: An error occurred in construct gate node


    3. SWAP(first_qubit_addr: int, second_qubit_addr: int) -> pyQPanda.QGate

    4. SWAP(first_qubit_addr_list: List[int], second_qubit_addr_list: List[int]) -> pyQPanda.QCircuit

    Returns:
        a SWAP gate node
    Raises:
        run_fail: An error occurred in construct gate node

    """
@overload
def SWAP(first_qubit_list: QVec, second_qubit_list: QVec) -> QCircuit:
    """SWAP(*args, **kwargs)
    Overloaded function.

    1. SWAP(first_qubit: pyQPanda.Qubit, second_qubit: pyQPanda.Qubit) -> pyQPanda.QGate

    Returns:
        a SWAP gate node
    Raises:
        run_fail: An error occurred in construct gate node


    2. SWAP(first_qubit_list: pyQPanda.QVec, second_qubit_list: pyQPanda.QVec) -> pyQPanda.QCircuit

    Returns:
        a SWAP gate node
    Raises:
        run_fail: An error occurred in construct gate node


    3. SWAP(first_qubit_addr: int, second_qubit_addr: int) -> pyQPanda.QGate

    4. SWAP(first_qubit_addr_list: List[int], second_qubit_addr_list: List[int]) -> pyQPanda.QCircuit

    Returns:
        a SWAP gate node
    Raises:
        run_fail: An error occurred in construct gate node

    """
@overload
def SWAP(first_qubit_addr: int, second_qubit_addr: int) -> QGate:
    """SWAP(*args, **kwargs)
    Overloaded function.

    1. SWAP(first_qubit: pyQPanda.Qubit, second_qubit: pyQPanda.Qubit) -> pyQPanda.QGate

    Returns:
        a SWAP gate node
    Raises:
        run_fail: An error occurred in construct gate node


    2. SWAP(first_qubit_list: pyQPanda.QVec, second_qubit_list: pyQPanda.QVec) -> pyQPanda.QCircuit

    Returns:
        a SWAP gate node
    Raises:
        run_fail: An error occurred in construct gate node


    3. SWAP(first_qubit_addr: int, second_qubit_addr: int) -> pyQPanda.QGate

    4. SWAP(first_qubit_addr_list: List[int], second_qubit_addr_list: List[int]) -> pyQPanda.QCircuit

    Returns:
        a SWAP gate node
    Raises:
        run_fail: An error occurred in construct gate node

    """
@overload
def SWAP(first_qubit_addr_list: list[int], second_qubit_addr_list: list[int]) -> QCircuit:
    """SWAP(*args, **kwargs)
    Overloaded function.

    1. SWAP(first_qubit: pyQPanda.Qubit, second_qubit: pyQPanda.Qubit) -> pyQPanda.QGate

    Returns:
        a SWAP gate node
    Raises:
        run_fail: An error occurred in construct gate node


    2. SWAP(first_qubit_list: pyQPanda.QVec, second_qubit_list: pyQPanda.QVec) -> pyQPanda.QCircuit

    Returns:
        a SWAP gate node
    Raises:
        run_fail: An error occurred in construct gate node


    3. SWAP(first_qubit_addr: int, second_qubit_addr: int) -> pyQPanda.QGate

    4. SWAP(first_qubit_addr_list: List[int], second_qubit_addr_list: List[int]) -> pyQPanda.QCircuit

    Returns:
        a SWAP gate node
    Raises:
        run_fail: An error occurred in construct gate node

    """
def Shor_factorization(arg0: int) -> tuple[bool, tuple[int, int]]:
    """Shor_factorization(arg0: int) -> Tuple[bool, Tuple[int, int]]

    Use Shor factorize integer num

    Args:
        int: target integer num
        result: Shor result

    Returns:
        result : Shor_factorization search result
    Raises:
        run_fail: An error occurred in Shor_factorization

    """
@overload
def SqiSWAP(first_qubit: Qubit, second_qubit: Qubit) -> QGate:
    """SqiSWAP(*args, **kwargs)
    Overloaded function.

    1. SqiSWAP(first_qubit: pyQPanda.Qubit, second_qubit: pyQPanda.Qubit) -> pyQPanda.QGate

    Returns:
        a SqiSWAP gate node
    Raises:
        run_fail: An error occurred in construct gate node


    2. SqiSWAP(first_qubit_list: pyQPanda.QVec, second_qubit_list: pyQPanda.QVec) -> pyQPanda.QCircuit

    Returns:
        a SqiSWAP gate node
    Raises:
        run_fail: An error occurred in construct gate node


    3. SqiSWAP(first_qubit_addr: int, second_qubit_addr: int) -> pyQPanda.QGate

    4. SqiSWAP(first_qubit_addr_list: List[int], second_qubit_addr_list: List[int]) -> pyQPanda.QCircuit

    Returns:
        a SqiSWAP gate node
    Raises:
        run_fail: An error occurred in construct gate node

    """
@overload
def SqiSWAP(first_qubit_list: QVec, second_qubit_list: QVec) -> QCircuit:
    """SqiSWAP(*args, **kwargs)
    Overloaded function.

    1. SqiSWAP(first_qubit: pyQPanda.Qubit, second_qubit: pyQPanda.Qubit) -> pyQPanda.QGate

    Returns:
        a SqiSWAP gate node
    Raises:
        run_fail: An error occurred in construct gate node


    2. SqiSWAP(first_qubit_list: pyQPanda.QVec, second_qubit_list: pyQPanda.QVec) -> pyQPanda.QCircuit

    Returns:
        a SqiSWAP gate node
    Raises:
        run_fail: An error occurred in construct gate node


    3. SqiSWAP(first_qubit_addr: int, second_qubit_addr: int) -> pyQPanda.QGate

    4. SqiSWAP(first_qubit_addr_list: List[int], second_qubit_addr_list: List[int]) -> pyQPanda.QCircuit

    Returns:
        a SqiSWAP gate node
    Raises:
        run_fail: An error occurred in construct gate node

    """
@overload
def SqiSWAP(first_qubit_addr: int, second_qubit_addr: int) -> QGate:
    """SqiSWAP(*args, **kwargs)
    Overloaded function.

    1. SqiSWAP(first_qubit: pyQPanda.Qubit, second_qubit: pyQPanda.Qubit) -> pyQPanda.QGate

    Returns:
        a SqiSWAP gate node
    Raises:
        run_fail: An error occurred in construct gate node


    2. SqiSWAP(first_qubit_list: pyQPanda.QVec, second_qubit_list: pyQPanda.QVec) -> pyQPanda.QCircuit

    Returns:
        a SqiSWAP gate node
    Raises:
        run_fail: An error occurred in construct gate node


    3. SqiSWAP(first_qubit_addr: int, second_qubit_addr: int) -> pyQPanda.QGate

    4. SqiSWAP(first_qubit_addr_list: List[int], second_qubit_addr_list: List[int]) -> pyQPanda.QCircuit

    Returns:
        a SqiSWAP gate node
    Raises:
        run_fail: An error occurred in construct gate node

    """
@overload
def SqiSWAP(first_qubit_addr_list: list[int], second_qubit_addr_list: list[int]) -> QCircuit:
    """SqiSWAP(*args, **kwargs)
    Overloaded function.

    1. SqiSWAP(first_qubit: pyQPanda.Qubit, second_qubit: pyQPanda.Qubit) -> pyQPanda.QGate

    Returns:
        a SqiSWAP gate node
    Raises:
        run_fail: An error occurred in construct gate node


    2. SqiSWAP(first_qubit_list: pyQPanda.QVec, second_qubit_list: pyQPanda.QVec) -> pyQPanda.QCircuit

    Returns:
        a SqiSWAP gate node
    Raises:
        run_fail: An error occurred in construct gate node


    3. SqiSWAP(first_qubit_addr: int, second_qubit_addr: int) -> pyQPanda.QGate

    4. SqiSWAP(first_qubit_addr_list: List[int], second_qubit_addr_list: List[int]) -> pyQPanda.QCircuit

    Returns:
        a SqiSWAP gate node
    Raises:
        run_fail: An error occurred in construct gate node

    """
@overload
def T(qubit: Qubit) -> QGate:
    """T(*args, **kwargs)
    Overloaded function.

    1. T(qubit: pyQPanda.Qubit) -> pyQPanda.QGate

    Create a T gate

    Args:
        qubit : quantum gate operate qubit

    Returns:
        a T gate node
    Raises:
        run_fail: An error occurred in construct gate node


    2. T(qubit_list: pyQPanda.QVec) -> pyQPanda.QCircuit

    Create a T gate

    Args:
        qubit_list: quantum gate operate qubits list

    Returns:
        a T gate node
    Raises:
        run_fail: An error occurred construct in gate node


    3. T(qubit_addr: int) -> pyQPanda.QGate

    Create a T gate

    Args:
        qubit_addr: quantum gate operate qubits addr

    Returns:
        a T gate node
    Raises:
        run_fail: An error occurred in construct gate node


    4. T(qubit_addr_list: List[int]) -> pyQPanda.QCircuit

    Create a T gate

    Args:
        qubit_list_addr: quantum gate  qubits list addr

    Returns:
        a T gate node
    Raises:
        run_fail: An error occurred in construct gate node

    """
@overload
def T(qubit_list: QVec) -> QCircuit:
    """T(*args, **kwargs)
    Overloaded function.

    1. T(qubit: pyQPanda.Qubit) -> pyQPanda.QGate

    Create a T gate

    Args:
        qubit : quantum gate operate qubit

    Returns:
        a T gate node
    Raises:
        run_fail: An error occurred in construct gate node


    2. T(qubit_list: pyQPanda.QVec) -> pyQPanda.QCircuit

    Create a T gate

    Args:
        qubit_list: quantum gate operate qubits list

    Returns:
        a T gate node
    Raises:
        run_fail: An error occurred construct in gate node


    3. T(qubit_addr: int) -> pyQPanda.QGate

    Create a T gate

    Args:
        qubit_addr: quantum gate operate qubits addr

    Returns:
        a T gate node
    Raises:
        run_fail: An error occurred in construct gate node


    4. T(qubit_addr_list: List[int]) -> pyQPanda.QCircuit

    Create a T gate

    Args:
        qubit_list_addr: quantum gate  qubits list addr

    Returns:
        a T gate node
    Raises:
        run_fail: An error occurred in construct gate node

    """
@overload
def T(qubit_addr: int) -> QGate:
    """T(*args, **kwargs)
    Overloaded function.

    1. T(qubit: pyQPanda.Qubit) -> pyQPanda.QGate

    Create a T gate

    Args:
        qubit : quantum gate operate qubit

    Returns:
        a T gate node
    Raises:
        run_fail: An error occurred in construct gate node


    2. T(qubit_list: pyQPanda.QVec) -> pyQPanda.QCircuit

    Create a T gate

    Args:
        qubit_list: quantum gate operate qubits list

    Returns:
        a T gate node
    Raises:
        run_fail: An error occurred construct in gate node


    3. T(qubit_addr: int) -> pyQPanda.QGate

    Create a T gate

    Args:
        qubit_addr: quantum gate operate qubits addr

    Returns:
        a T gate node
    Raises:
        run_fail: An error occurred in construct gate node


    4. T(qubit_addr_list: List[int]) -> pyQPanda.QCircuit

    Create a T gate

    Args:
        qubit_list_addr: quantum gate  qubits list addr

    Returns:
        a T gate node
    Raises:
        run_fail: An error occurred in construct gate node

    """
@overload
def T(qubit_addr_list: list[int]) -> QCircuit:
    """T(*args, **kwargs)
    Overloaded function.

    1. T(qubit: pyQPanda.Qubit) -> pyQPanda.QGate

    Create a T gate

    Args:
        qubit : quantum gate operate qubit

    Returns:
        a T gate node
    Raises:
        run_fail: An error occurred in construct gate node


    2. T(qubit_list: pyQPanda.QVec) -> pyQPanda.QCircuit

    Create a T gate

    Args:
        qubit_list: quantum gate operate qubits list

    Returns:
        a T gate node
    Raises:
        run_fail: An error occurred construct in gate node


    3. T(qubit_addr: int) -> pyQPanda.QGate

    Create a T gate

    Args:
        qubit_addr: quantum gate operate qubits addr

    Returns:
        a T gate node
    Raises:
        run_fail: An error occurred in construct gate node


    4. T(qubit_addr_list: List[int]) -> pyQPanda.QCircuit

    Create a T gate

    Args:
        qubit_list_addr: quantum gate  qubits list addr

    Returns:
        a T gate node
    Raises:
        run_fail: An error occurred in construct gate node

    """
@overload
def Toffoli(control_qubit_first: Qubit, control_qubit_second: Qubit, target_qubit: Qubit) -> QGate:
    """Toffoli(*args, **kwargs)
    Overloaded function.

    1. Toffoli(control_qubit_first: pyQPanda.Qubit, control_qubit_second: pyQPanda.Qubit, target_qubit: pyQPanda.Qubit) -> pyQPanda.QGate

    Create a Toffoli gate.

    Args:
        control_qubit_first (Qubit *): First control qubit.

        control_qubit_second (Qubit *): Second control qubit.

        target_qubit (Qubit *): Target qubit.

    Returns:
        A Toffoli node representing the operation.


    2. Toffoli(control_qubit_addr_first: int, control_qubit_addr_second: int, target_qubit_addr: int) -> pyQPanda.QGate

    Create a Toffoli gate.

    Args:
        control_qubit_addr_first (int): Address of the first control qubit.

        control_qubit_addr_second (int): Address of the second control qubit.

        target_qubit_addr (int): Address of the target qubit.

    Returns:
        A Toffoli node representing the operation.

    """
@overload
def Toffoli(control_qubit_addr_first: int, control_qubit_addr_second: int, target_qubit_addr: int) -> QGate:
    """Toffoli(*args, **kwargs)
    Overloaded function.

    1. Toffoli(control_qubit_first: pyQPanda.Qubit, control_qubit_second: pyQPanda.Qubit, target_qubit: pyQPanda.Qubit) -> pyQPanda.QGate

    Create a Toffoli gate.

    Args:
        control_qubit_first (Qubit *): First control qubit.

        control_qubit_second (Qubit *): Second control qubit.

        target_qubit (Qubit *): Target qubit.

    Returns:
        A Toffoli node representing the operation.


    2. Toffoli(control_qubit_addr_first: int, control_qubit_addr_second: int, target_qubit_addr: int) -> pyQPanda.QGate

    Create a Toffoli gate.

    Args:
        control_qubit_addr_first (int): Address of the first control qubit.

        control_qubit_addr_second (int): Address of the second control qubit.

        target_qubit_addr (int): Address of the target qubit.

    Returns:
        A Toffoli node representing the operation.

    """
@overload
def U1(qubit: Qubit, angle: float) -> QGate:
    """U1(*args, **kwargs)
    Overloaded function.

    1. U1(qubit: pyQPanda.Qubit, angle: float) -> pyQPanda.QGate

    Returns:
        a U1 gate node
    Raises:
        run_fail: An error occurred in construct gate node


    2. U1(qubit_list: pyQPanda.QVec, angle: float) -> pyQPanda.QCircuit

    Create a U1 gate
    Args:
        qubit_list: quantum gate operate qubits list
        args : quantum gate angles

    Returns:
        a U1 gate node
    Raises:
        run_fail: An error occurred construct in gate node


    3. U1(qubit_addr: int, angle: float) -> pyQPanda.QGate

    Create a U1 gate
    Args:
        qubit_addr: quantum gate operate qubits addr
        args : quantum gate angles

    Returns:
        a U1 gate node
    Raises:
        run_fail: An error occurred in construct gate node


    4. U1(qubit_addr_list: List[int], angle: float) -> pyQPanda.QCircuit

    Create a U1 gate
    Args:
        qubit_list_addr: quantum gate  qubits list addr
        args : quantum gate angles

    Returns:
        a U1 gate node
    Raises:
        run_fail: An error occurred in construct gate node

    """
@overload
def U1(qubit_list: QVec, angle: float) -> QCircuit:
    """U1(*args, **kwargs)
    Overloaded function.

    1. U1(qubit: pyQPanda.Qubit, angle: float) -> pyQPanda.QGate

    Returns:
        a U1 gate node
    Raises:
        run_fail: An error occurred in construct gate node


    2. U1(qubit_list: pyQPanda.QVec, angle: float) -> pyQPanda.QCircuit

    Create a U1 gate
    Args:
        qubit_list: quantum gate operate qubits list
        args : quantum gate angles

    Returns:
        a U1 gate node
    Raises:
        run_fail: An error occurred construct in gate node


    3. U1(qubit_addr: int, angle: float) -> pyQPanda.QGate

    Create a U1 gate
    Args:
        qubit_addr: quantum gate operate qubits addr
        args : quantum gate angles

    Returns:
        a U1 gate node
    Raises:
        run_fail: An error occurred in construct gate node


    4. U1(qubit_addr_list: List[int], angle: float) -> pyQPanda.QCircuit

    Create a U1 gate
    Args:
        qubit_list_addr: quantum gate  qubits list addr
        args : quantum gate angles

    Returns:
        a U1 gate node
    Raises:
        run_fail: An error occurred in construct gate node

    """
@overload
def U1(qubit_addr: int, angle: float) -> QGate:
    """U1(*args, **kwargs)
    Overloaded function.

    1. U1(qubit: pyQPanda.Qubit, angle: float) -> pyQPanda.QGate

    Returns:
        a U1 gate node
    Raises:
        run_fail: An error occurred in construct gate node


    2. U1(qubit_list: pyQPanda.QVec, angle: float) -> pyQPanda.QCircuit

    Create a U1 gate
    Args:
        qubit_list: quantum gate operate qubits list
        args : quantum gate angles

    Returns:
        a U1 gate node
    Raises:
        run_fail: An error occurred construct in gate node


    3. U1(qubit_addr: int, angle: float) -> pyQPanda.QGate

    Create a U1 gate
    Args:
        qubit_addr: quantum gate operate qubits addr
        args : quantum gate angles

    Returns:
        a U1 gate node
    Raises:
        run_fail: An error occurred in construct gate node


    4. U1(qubit_addr_list: List[int], angle: float) -> pyQPanda.QCircuit

    Create a U1 gate
    Args:
        qubit_list_addr: quantum gate  qubits list addr
        args : quantum gate angles

    Returns:
        a U1 gate node
    Raises:
        run_fail: An error occurred in construct gate node

    """
@overload
def U1(qubit_addr_list: list[int], angle: float) -> QCircuit:
    """U1(*args, **kwargs)
    Overloaded function.

    1. U1(qubit: pyQPanda.Qubit, angle: float) -> pyQPanda.QGate

    Returns:
        a U1 gate node
    Raises:
        run_fail: An error occurred in construct gate node


    2. U1(qubit_list: pyQPanda.QVec, angle: float) -> pyQPanda.QCircuit

    Create a U1 gate
    Args:
        qubit_list: quantum gate operate qubits list
        args : quantum gate angles

    Returns:
        a U1 gate node
    Raises:
        run_fail: An error occurred construct in gate node


    3. U1(qubit_addr: int, angle: float) -> pyQPanda.QGate

    Create a U1 gate
    Args:
        qubit_addr: quantum gate operate qubits addr
        args : quantum gate angles

    Returns:
        a U1 gate node
    Raises:
        run_fail: An error occurred in construct gate node


    4. U1(qubit_addr_list: List[int], angle: float) -> pyQPanda.QCircuit

    Create a U1 gate
    Args:
        qubit_list_addr: quantum gate  qubits list addr
        args : quantum gate angles

    Returns:
        a U1 gate node
    Raises:
        run_fail: An error occurred in construct gate node

    """
@overload
def U2(qubit: Qubit, phi_angle: float, lambda_angle: float) -> QGate:
    """U2(*args, **kwargs)
    Overloaded function.

    1. U2(qubit: pyQPanda.Qubit, phi_angle: float, lambda_angle: float) -> pyQPanda.QGate

    Returns:
        a U2 gate node
    Raises:
        run_fail: An error occurred in construct gate node


    2. U2(qubit_list: pyQPanda.QVec, phi_angle: float, lambda_angle: float) -> pyQPanda.QCircuit

    Create a U2 gate
    Args:
        qubit_list: quantum gate operate qubits list
        args : quantum gate angles

    Returns:
        a U2 gate node
    Raises:
        run_fail: An error occurred construct in gate node


    3. U2(qubit_addr: int, phi_angle: float, lambda_angle: float) -> pyQPanda.QGate

    Create a U2 gate
    Args:
        qubit_addr: quantum gate operate qubits addr
        args : quantum gate angles

    Returns:
        a U2 gate node
    Raises:
        run_fail: An error occurred in construct gate node


    4. U2(qubit_addr_list: List[int], phi_angle: float, lambda_angle: float) -> pyQPanda.QCircuit

    Create a U2 gate
    Args:
        qubit_list_addr: quantum gate  qubits list addr
        args : quantum gate angles

    Returns:
        a U2 gate node
    Raises:
        run_fail: An error occurred in construct gate node

    """
@overload
def U2(qubit_list: QVec, phi_angle: float, lambda_angle: float) -> QCircuit:
    """U2(*args, **kwargs)
    Overloaded function.

    1. U2(qubit: pyQPanda.Qubit, phi_angle: float, lambda_angle: float) -> pyQPanda.QGate

    Returns:
        a U2 gate node
    Raises:
        run_fail: An error occurred in construct gate node


    2. U2(qubit_list: pyQPanda.QVec, phi_angle: float, lambda_angle: float) -> pyQPanda.QCircuit

    Create a U2 gate
    Args:
        qubit_list: quantum gate operate qubits list
        args : quantum gate angles

    Returns:
        a U2 gate node
    Raises:
        run_fail: An error occurred construct in gate node


    3. U2(qubit_addr: int, phi_angle: float, lambda_angle: float) -> pyQPanda.QGate

    Create a U2 gate
    Args:
        qubit_addr: quantum gate operate qubits addr
        args : quantum gate angles

    Returns:
        a U2 gate node
    Raises:
        run_fail: An error occurred in construct gate node


    4. U2(qubit_addr_list: List[int], phi_angle: float, lambda_angle: float) -> pyQPanda.QCircuit

    Create a U2 gate
    Args:
        qubit_list_addr: quantum gate  qubits list addr
        args : quantum gate angles

    Returns:
        a U2 gate node
    Raises:
        run_fail: An error occurred in construct gate node

    """
@overload
def U2(qubit_addr: int, phi_angle: float, lambda_angle: float) -> QGate:
    """U2(*args, **kwargs)
    Overloaded function.

    1. U2(qubit: pyQPanda.Qubit, phi_angle: float, lambda_angle: float) -> pyQPanda.QGate

    Returns:
        a U2 gate node
    Raises:
        run_fail: An error occurred in construct gate node


    2. U2(qubit_list: pyQPanda.QVec, phi_angle: float, lambda_angle: float) -> pyQPanda.QCircuit

    Create a U2 gate
    Args:
        qubit_list: quantum gate operate qubits list
        args : quantum gate angles

    Returns:
        a U2 gate node
    Raises:
        run_fail: An error occurred construct in gate node


    3. U2(qubit_addr: int, phi_angle: float, lambda_angle: float) -> pyQPanda.QGate

    Create a U2 gate
    Args:
        qubit_addr: quantum gate operate qubits addr
        args : quantum gate angles

    Returns:
        a U2 gate node
    Raises:
        run_fail: An error occurred in construct gate node


    4. U2(qubit_addr_list: List[int], phi_angle: float, lambda_angle: float) -> pyQPanda.QCircuit

    Create a U2 gate
    Args:
        qubit_list_addr: quantum gate  qubits list addr
        args : quantum gate angles

    Returns:
        a U2 gate node
    Raises:
        run_fail: An error occurred in construct gate node

    """
@overload
def U2(qubit_addr_list: list[int], phi_angle: float, lambda_angle: float) -> QCircuit:
    """U2(*args, **kwargs)
    Overloaded function.

    1. U2(qubit: pyQPanda.Qubit, phi_angle: float, lambda_angle: float) -> pyQPanda.QGate

    Returns:
        a U2 gate node
    Raises:
        run_fail: An error occurred in construct gate node


    2. U2(qubit_list: pyQPanda.QVec, phi_angle: float, lambda_angle: float) -> pyQPanda.QCircuit

    Create a U2 gate
    Args:
        qubit_list: quantum gate operate qubits list
        args : quantum gate angles

    Returns:
        a U2 gate node
    Raises:
        run_fail: An error occurred construct in gate node


    3. U2(qubit_addr: int, phi_angle: float, lambda_angle: float) -> pyQPanda.QGate

    Create a U2 gate
    Args:
        qubit_addr: quantum gate operate qubits addr
        args : quantum gate angles

    Returns:
        a U2 gate node
    Raises:
        run_fail: An error occurred in construct gate node


    4. U2(qubit_addr_list: List[int], phi_angle: float, lambda_angle: float) -> pyQPanda.QCircuit

    Create a U2 gate
    Args:
        qubit_list_addr: quantum gate  qubits list addr
        args : quantum gate angles

    Returns:
        a U2 gate node
    Raises:
        run_fail: An error occurred in construct gate node

    """
@overload
def U3(qubit: Qubit, theta_angle: float, phi_angle: float, lambda_angle: float) -> QGate:
    """U3(*args, **kwargs)
    Overloaded function.

    1. U3(qubit: pyQPanda.Qubit, theta_angle: float, phi_angle: float, lambda_angle: float) -> pyQPanda.QGate

    Returns:
        a U3 gate node
    Raises:
        run_fail: An error occurred in construct gate node


    2. U3(qubit_list: pyQPanda.QVec, theta_angle: float, phi_angle: float, lambda_angle: float) -> pyQPanda.QCircuit

    Create a U3 gate
    Args:
        qubit_list: quantum gate operate qubits list
        args : quantum gate angles

    Returns:
        a U3 gate node
    Raises:
        run_fail: An error occurred construct in gate node


    3. U3(qubit_addr: int, theta_angle: float, phi_angle: float, lambda_angle: float) -> pyQPanda.QGate

    Create a U3 gate
    Args:
        qubit_addr: quantum gate operate qubits addr
        args : quantum gate angles

    Returns:
        a U3 gate node
    Raises:
        run_fail: An error occurred in construct gate node


    4. U3(qubit_addr_list: List[int], theta_angle: float, phi_angle: float, lambda_angle: float) -> pyQPanda.QCircuit

    Create a U3 gate
    Args:
        qubit_list_addr: quantum gate  qubits list addr
        args : quantum gate angles

    Returns:
        a U3 gate node
    Raises:
        run_fail: An error occurred in construct gate node

    """
@overload
def U3(qubit_list: QVec, theta_angle: float, phi_angle: float, lambda_angle: float) -> QCircuit:
    """U3(*args, **kwargs)
    Overloaded function.

    1. U3(qubit: pyQPanda.Qubit, theta_angle: float, phi_angle: float, lambda_angle: float) -> pyQPanda.QGate

    Returns:
        a U3 gate node
    Raises:
        run_fail: An error occurred in construct gate node


    2. U3(qubit_list: pyQPanda.QVec, theta_angle: float, phi_angle: float, lambda_angle: float) -> pyQPanda.QCircuit

    Create a U3 gate
    Args:
        qubit_list: quantum gate operate qubits list
        args : quantum gate angles

    Returns:
        a U3 gate node
    Raises:
        run_fail: An error occurred construct in gate node


    3. U3(qubit_addr: int, theta_angle: float, phi_angle: float, lambda_angle: float) -> pyQPanda.QGate

    Create a U3 gate
    Args:
        qubit_addr: quantum gate operate qubits addr
        args : quantum gate angles

    Returns:
        a U3 gate node
    Raises:
        run_fail: An error occurred in construct gate node


    4. U3(qubit_addr_list: List[int], theta_angle: float, phi_angle: float, lambda_angle: float) -> pyQPanda.QCircuit

    Create a U3 gate
    Args:
        qubit_list_addr: quantum gate  qubits list addr
        args : quantum gate angles

    Returns:
        a U3 gate node
    Raises:
        run_fail: An error occurred in construct gate node

    """
@overload
def U3(qubit_addr: int, theta_angle: float, phi_angle: float, lambda_angle: float) -> QGate:
    """U3(*args, **kwargs)
    Overloaded function.

    1. U3(qubit: pyQPanda.Qubit, theta_angle: float, phi_angle: float, lambda_angle: float) -> pyQPanda.QGate

    Returns:
        a U3 gate node
    Raises:
        run_fail: An error occurred in construct gate node


    2. U3(qubit_list: pyQPanda.QVec, theta_angle: float, phi_angle: float, lambda_angle: float) -> pyQPanda.QCircuit

    Create a U3 gate
    Args:
        qubit_list: quantum gate operate qubits list
        args : quantum gate angles

    Returns:
        a U3 gate node
    Raises:
        run_fail: An error occurred construct in gate node


    3. U3(qubit_addr: int, theta_angle: float, phi_angle: float, lambda_angle: float) -> pyQPanda.QGate

    Create a U3 gate
    Args:
        qubit_addr: quantum gate operate qubits addr
        args : quantum gate angles

    Returns:
        a U3 gate node
    Raises:
        run_fail: An error occurred in construct gate node


    4. U3(qubit_addr_list: List[int], theta_angle: float, phi_angle: float, lambda_angle: float) -> pyQPanda.QCircuit

    Create a U3 gate
    Args:
        qubit_list_addr: quantum gate  qubits list addr
        args : quantum gate angles

    Returns:
        a U3 gate node
    Raises:
        run_fail: An error occurred in construct gate node

    """
@overload
def U3(qubit_addr_list: list[int], theta_angle: float, phi_angle: float, lambda_angle: float) -> QCircuit:
    """U3(*args, **kwargs)
    Overloaded function.

    1. U3(qubit: pyQPanda.Qubit, theta_angle: float, phi_angle: float, lambda_angle: float) -> pyQPanda.QGate

    Returns:
        a U3 gate node
    Raises:
        run_fail: An error occurred in construct gate node


    2. U3(qubit_list: pyQPanda.QVec, theta_angle: float, phi_angle: float, lambda_angle: float) -> pyQPanda.QCircuit

    Create a U3 gate
    Args:
        qubit_list: quantum gate operate qubits list
        args : quantum gate angles

    Returns:
        a U3 gate node
    Raises:
        run_fail: An error occurred construct in gate node


    3. U3(qubit_addr: int, theta_angle: float, phi_angle: float, lambda_angle: float) -> pyQPanda.QGate

    Create a U3 gate
    Args:
        qubit_addr: quantum gate operate qubits addr
        args : quantum gate angles

    Returns:
        a U3 gate node
    Raises:
        run_fail: An error occurred in construct gate node


    4. U3(qubit_addr_list: List[int], theta_angle: float, phi_angle: float, lambda_angle: float) -> pyQPanda.QCircuit

    Create a U3 gate
    Args:
        qubit_list_addr: quantum gate  qubits list addr
        args : quantum gate angles

    Returns:
        a U3 gate node
    Raises:
        run_fail: An error occurred in construct gate node

    """
@overload
def U4(matrix: list[complex], qubit: Qubit) -> QGate:
    """U4(*args, **kwargs)
    Overloaded function.

    1. U4(matrix: List[complex], qubit: pyQPanda.Qubit) -> pyQPanda.QGate

    Create a U4 gate.

    Args:
        matrix: the U4 gate matrix to be applied.

        qubit: the target qubit for the U4 gate.

    Returns:
        A U4 node representing the operation.


    2. U4(alpha_angle: float, beta_angle: float, gamma_angle: float, delta_angle: float, qubit: pyQPanda.Qubit) -> pyQPanda.QGate

    Create a U4 gate.

    Args:
        alpha_angle: the alpha angle for the U4 gate.

        beta_angle: the beta angle for the U4 gate.

        gamma_angle: the gamma angle for the U4 gate.

        delta_angle: the delta angle for the U4 gate.

        qubit: the target qubit for the U4 gate.

    Returns:
        A U4 node representing the operation.


    3. U4(qubit: pyQPanda.Qubit, matrix: List[complex]) -> pyQPanda.QGate

    Create a U4 gate.

    Args:
        qubit: the target qubit for the U4 gate.

        matrix: the U4 gate matrix to be applied.

    Returns:
        A U4 node representing the operation.


    4. U4(qubit_list: pyQPanda.QVec, matrix: List[complex]) -> pyQPanda.QCircuit

    Create a U4 gate.

    Args:
        qubit_list: the list of target qubits for the U4 gate.

        matrix: the U4 gate matrix to be applied.

    Returns:
        A U4 node representing the operation.


    5. U4(qubit_addr: int, matrix: List[complex]) -> pyQPanda.QGate

    Create a U4 gate.

    Args:
        qubit_addr: the address of the target qubit for the U4 gate.

        matrix: the U4 gate matrix to be applied.

    Returns:
        A U4 node representing the operation.


    6. U4(qubit_addr_list: List[int], matrix: List[complex]) -> pyQPanda.QCircuit

    Create a U4 gate.

    Args:
        qubit_addr_list: the list of addresses for the target qubits of the U4 gate.

        matrix: the U4 gate matrix to be applied.

    Returns:
        A U4 node representing the operation.


    7. U4(qubit: pyQPanda.Qubit, alpha_anlge: float, beta_anlge: float, gamma_anlge: float, delta_anlge: float) -> pyQPanda.QGate

    Create a U4 gate.

    Args:
        qubit: the target qubit for the U4 gate.

        alpha_angle: the alpha angle for the U4 gate.

        beta_angle: the beta angle for the U4 gate.

        gamma_angle: the gamma angle for the U4 gate.

        delta_angle: the delta angle for the U4 gate.

    Returns:
        A U4 node representing the operation.


    8. U4(qubit_list: pyQPanda.QVec, alpha_angle: float, beta_angle: float, gamma_angle: float, delta_angle: float) -> pyQPanda.QCircuit

    Create a U4 gate.


    Args:
        qubit_list: the list of target qubits for the U4 gate.

        alpha_angle: the alpha angle for the U4 gate.

        beta_angle: the beta angle for the U4 gate.

        gamma_angle: the gamma angle for the U4 gate.

        delta_angle: the delta angle for the U4 gate.

    Returns:
        A U4 node representing the operation.

    9. U4(qubit_addr: int, alpha_anlge: float, beta_anlge: float, gamma_anlge: float, delta_anlge: float) -> pyQPanda.QGate

    Create a U4 gate.

    Args:
        qubit_addr: the address of the target qubit for the U4 gate.

        alpha_angle: the alpha angle for the U4 gate.

        beta_angle: the beta angle for the U4 gate.

        gamma_angle: the gamma angle for the U4 gate.

        delta_angle: the delta angle for the U4 gate.

    Returns:
        A U4 node representing the operation.

    10. U4(qubit_addr_list: List[int], alpha_anlge: float, beta_anlge: float, gamma_anlge: float, delta_anlge: float) -> pyQPanda.QCircuit

    Create a U4 gate.

    Args:
        qubit_addr_list: the list of addresses of target qubits for the U4 gate.

        alpha_angle: the alpha angle for the U4 gate.

        beta_angle: the beta angle for the U4 gate.

        gamma_angle: the gamma angle for the U4 gate.

        delta_angle: the delta angle for the U4 gate.

    Returns:
        A U4 node representing the operation.
    """
@overload
def U4(alpha_angle: float, beta_angle: float, gamma_angle: float, delta_angle: float, qubit: Qubit) -> QGate:
    """U4(*args, **kwargs)
    Overloaded function.

    1. U4(matrix: List[complex], qubit: pyQPanda.Qubit) -> pyQPanda.QGate

    Create a U4 gate.

    Args:
        matrix: the U4 gate matrix to be applied.

        qubit: the target qubit for the U4 gate.

    Returns:
        A U4 node representing the operation.


    2. U4(alpha_angle: float, beta_angle: float, gamma_angle: float, delta_angle: float, qubit: pyQPanda.Qubit) -> pyQPanda.QGate

    Create a U4 gate.

    Args:
        alpha_angle: the alpha angle for the U4 gate.

        beta_angle: the beta angle for the U4 gate.

        gamma_angle: the gamma angle for the U4 gate.

        delta_angle: the delta angle for the U4 gate.

        qubit: the target qubit for the U4 gate.

    Returns:
        A U4 node representing the operation.


    3. U4(qubit: pyQPanda.Qubit, matrix: List[complex]) -> pyQPanda.QGate

    Create a U4 gate.

    Args:
        qubit: the target qubit for the U4 gate.

        matrix: the U4 gate matrix to be applied.

    Returns:
        A U4 node representing the operation.


    4. U4(qubit_list: pyQPanda.QVec, matrix: List[complex]) -> pyQPanda.QCircuit

    Create a U4 gate.

    Args:
        qubit_list: the list of target qubits for the U4 gate.

        matrix: the U4 gate matrix to be applied.

    Returns:
        A U4 node representing the operation.


    5. U4(qubit_addr: int, matrix: List[complex]) -> pyQPanda.QGate

    Create a U4 gate.

    Args:
        qubit_addr: the address of the target qubit for the U4 gate.

        matrix: the U4 gate matrix to be applied.

    Returns:
        A U4 node representing the operation.


    6. U4(qubit_addr_list: List[int], matrix: List[complex]) -> pyQPanda.QCircuit

    Create a U4 gate.

    Args:
        qubit_addr_list: the list of addresses for the target qubits of the U4 gate.

        matrix: the U4 gate matrix to be applied.

    Returns:
        A U4 node representing the operation.


    7. U4(qubit: pyQPanda.Qubit, alpha_anlge: float, beta_anlge: float, gamma_anlge: float, delta_anlge: float) -> pyQPanda.QGate

    Create a U4 gate.

    Args:
        qubit: the target qubit for the U4 gate.

        alpha_angle: the alpha angle for the U4 gate.

        beta_angle: the beta angle for the U4 gate.

        gamma_angle: the gamma angle for the U4 gate.

        delta_angle: the delta angle for the U4 gate.

    Returns:
        A U4 node representing the operation.


    8. U4(qubit_list: pyQPanda.QVec, alpha_angle: float, beta_angle: float, gamma_angle: float, delta_angle: float) -> pyQPanda.QCircuit

    Create a U4 gate.


    Args:
        qubit_list: the list of target qubits for the U4 gate.

        alpha_angle: the alpha angle for the U4 gate.

        beta_angle: the beta angle for the U4 gate.

        gamma_angle: the gamma angle for the U4 gate.

        delta_angle: the delta angle for the U4 gate.

    Returns:
        A U4 node representing the operation.

    9. U4(qubit_addr: int, alpha_anlge: float, beta_anlge: float, gamma_anlge: float, delta_anlge: float) -> pyQPanda.QGate

    Create a U4 gate.

    Args:
        qubit_addr: the address of the target qubit for the U4 gate.

        alpha_angle: the alpha angle for the U4 gate.

        beta_angle: the beta angle for the U4 gate.

        gamma_angle: the gamma angle for the U4 gate.

        delta_angle: the delta angle for the U4 gate.

    Returns:
        A U4 node representing the operation.

    10. U4(qubit_addr_list: List[int], alpha_anlge: float, beta_anlge: float, gamma_anlge: float, delta_anlge: float) -> pyQPanda.QCircuit

    Create a U4 gate.

    Args:
        qubit_addr_list: the list of addresses of target qubits for the U4 gate.

        alpha_angle: the alpha angle for the U4 gate.

        beta_angle: the beta angle for the U4 gate.

        gamma_angle: the gamma angle for the U4 gate.

        delta_angle: the delta angle for the U4 gate.

    Returns:
        A U4 node representing the operation.
    """
@overload
def U4(qubit: Qubit, matrix: list[complex]) -> QGate:
    """U4(*args, **kwargs)
    Overloaded function.

    1. U4(matrix: List[complex], qubit: pyQPanda.Qubit) -> pyQPanda.QGate

    Create a U4 gate.

    Args:
        matrix: the U4 gate matrix to be applied.

        qubit: the target qubit for the U4 gate.

    Returns:
        A U4 node representing the operation.


    2. U4(alpha_angle: float, beta_angle: float, gamma_angle: float, delta_angle: float, qubit: pyQPanda.Qubit) -> pyQPanda.QGate

    Create a U4 gate.

    Args:
        alpha_angle: the alpha angle for the U4 gate.

        beta_angle: the beta angle for the U4 gate.

        gamma_angle: the gamma angle for the U4 gate.

        delta_angle: the delta angle for the U4 gate.

        qubit: the target qubit for the U4 gate.

    Returns:
        A U4 node representing the operation.


    3. U4(qubit: pyQPanda.Qubit, matrix: List[complex]) -> pyQPanda.QGate

    Create a U4 gate.

    Args:
        qubit: the target qubit for the U4 gate.

        matrix: the U4 gate matrix to be applied.

    Returns:
        A U4 node representing the operation.


    4. U4(qubit_list: pyQPanda.QVec, matrix: List[complex]) -> pyQPanda.QCircuit

    Create a U4 gate.

    Args:
        qubit_list: the list of target qubits for the U4 gate.

        matrix: the U4 gate matrix to be applied.

    Returns:
        A U4 node representing the operation.


    5. U4(qubit_addr: int, matrix: List[complex]) -> pyQPanda.QGate

    Create a U4 gate.

    Args:
        qubit_addr: the address of the target qubit for the U4 gate.

        matrix: the U4 gate matrix to be applied.

    Returns:
        A U4 node representing the operation.


    6. U4(qubit_addr_list: List[int], matrix: List[complex]) -> pyQPanda.QCircuit

    Create a U4 gate.

    Args:
        qubit_addr_list: the list of addresses for the target qubits of the U4 gate.

        matrix: the U4 gate matrix to be applied.

    Returns:
        A U4 node representing the operation.


    7. U4(qubit: pyQPanda.Qubit, alpha_anlge: float, beta_anlge: float, gamma_anlge: float, delta_anlge: float) -> pyQPanda.QGate

    Create a U4 gate.

    Args:
        qubit: the target qubit for the U4 gate.

        alpha_angle: the alpha angle for the U4 gate.

        beta_angle: the beta angle for the U4 gate.

        gamma_angle: the gamma angle for the U4 gate.

        delta_angle: the delta angle for the U4 gate.

    Returns:
        A U4 node representing the operation.


    8. U4(qubit_list: pyQPanda.QVec, alpha_angle: float, beta_angle: float, gamma_angle: float, delta_angle: float) -> pyQPanda.QCircuit

    Create a U4 gate.


    Args:
        qubit_list: the list of target qubits for the U4 gate.

        alpha_angle: the alpha angle for the U4 gate.

        beta_angle: the beta angle for the U4 gate.

        gamma_angle: the gamma angle for the U4 gate.

        delta_angle: the delta angle for the U4 gate.

    Returns:
        A U4 node representing the operation.

    9. U4(qubit_addr: int, alpha_anlge: float, beta_anlge: float, gamma_anlge: float, delta_anlge: float) -> pyQPanda.QGate

    Create a U4 gate.

    Args:
        qubit_addr: the address of the target qubit for the U4 gate.

        alpha_angle: the alpha angle for the U4 gate.

        beta_angle: the beta angle for the U4 gate.

        gamma_angle: the gamma angle for the U4 gate.

        delta_angle: the delta angle for the U4 gate.

    Returns:
        A U4 node representing the operation.

    10. U4(qubit_addr_list: List[int], alpha_anlge: float, beta_anlge: float, gamma_anlge: float, delta_anlge: float) -> pyQPanda.QCircuit

    Create a U4 gate.

    Args:
        qubit_addr_list: the list of addresses of target qubits for the U4 gate.

        alpha_angle: the alpha angle for the U4 gate.

        beta_angle: the beta angle for the U4 gate.

        gamma_angle: the gamma angle for the U4 gate.

        delta_angle: the delta angle for the U4 gate.

    Returns:
        A U4 node representing the operation.
    """
@overload
def U4(qubit_list: QVec, matrix: list[complex]) -> QCircuit:
    """U4(*args, **kwargs)
    Overloaded function.

    1. U4(matrix: List[complex], qubit: pyQPanda.Qubit) -> pyQPanda.QGate

    Create a U4 gate.

    Args:
        matrix: the U4 gate matrix to be applied.

        qubit: the target qubit for the U4 gate.

    Returns:
        A U4 node representing the operation.


    2. U4(alpha_angle: float, beta_angle: float, gamma_angle: float, delta_angle: float, qubit: pyQPanda.Qubit) -> pyQPanda.QGate

    Create a U4 gate.

    Args:
        alpha_angle: the alpha angle for the U4 gate.

        beta_angle: the beta angle for the U4 gate.

        gamma_angle: the gamma angle for the U4 gate.

        delta_angle: the delta angle for the U4 gate.

        qubit: the target qubit for the U4 gate.

    Returns:
        A U4 node representing the operation.


    3. U4(qubit: pyQPanda.Qubit, matrix: List[complex]) -> pyQPanda.QGate

    Create a U4 gate.

    Args:
        qubit: the target qubit for the U4 gate.

        matrix: the U4 gate matrix to be applied.

    Returns:
        A U4 node representing the operation.


    4. U4(qubit_list: pyQPanda.QVec, matrix: List[complex]) -> pyQPanda.QCircuit

    Create a U4 gate.

    Args:
        qubit_list: the list of target qubits for the U4 gate.

        matrix: the U4 gate matrix to be applied.

    Returns:
        A U4 node representing the operation.


    5. U4(qubit_addr: int, matrix: List[complex]) -> pyQPanda.QGate

    Create a U4 gate.

    Args:
        qubit_addr: the address of the target qubit for the U4 gate.

        matrix: the U4 gate matrix to be applied.

    Returns:
        A U4 node representing the operation.


    6. U4(qubit_addr_list: List[int], matrix: List[complex]) -> pyQPanda.QCircuit

    Create a U4 gate.

    Args:
        qubit_addr_list: the list of addresses for the target qubits of the U4 gate.

        matrix: the U4 gate matrix to be applied.

    Returns:
        A U4 node representing the operation.


    7. U4(qubit: pyQPanda.Qubit, alpha_anlge: float, beta_anlge: float, gamma_anlge: float, delta_anlge: float) -> pyQPanda.QGate

    Create a U4 gate.

    Args:
        qubit: the target qubit for the U4 gate.

        alpha_angle: the alpha angle for the U4 gate.

        beta_angle: the beta angle for the U4 gate.

        gamma_angle: the gamma angle for the U4 gate.

        delta_angle: the delta angle for the U4 gate.

    Returns:
        A U4 node representing the operation.


    8. U4(qubit_list: pyQPanda.QVec, alpha_angle: float, beta_angle: float, gamma_angle: float, delta_angle: float) -> pyQPanda.QCircuit

    Create a U4 gate.


    Args:
        qubit_list: the list of target qubits for the U4 gate.

        alpha_angle: the alpha angle for the U4 gate.

        beta_angle: the beta angle for the U4 gate.

        gamma_angle: the gamma angle for the U4 gate.

        delta_angle: the delta angle for the U4 gate.

    Returns:
        A U4 node representing the operation.

    9. U4(qubit_addr: int, alpha_anlge: float, beta_anlge: float, gamma_anlge: float, delta_anlge: float) -> pyQPanda.QGate

    Create a U4 gate.

    Args:
        qubit_addr: the address of the target qubit for the U4 gate.

        alpha_angle: the alpha angle for the U4 gate.

        beta_angle: the beta angle for the U4 gate.

        gamma_angle: the gamma angle for the U4 gate.

        delta_angle: the delta angle for the U4 gate.

    Returns:
        A U4 node representing the operation.

    10. U4(qubit_addr_list: List[int], alpha_anlge: float, beta_anlge: float, gamma_anlge: float, delta_anlge: float) -> pyQPanda.QCircuit

    Create a U4 gate.

    Args:
        qubit_addr_list: the list of addresses of target qubits for the U4 gate.

        alpha_angle: the alpha angle for the U4 gate.

        beta_angle: the beta angle for the U4 gate.

        gamma_angle: the gamma angle for the U4 gate.

        delta_angle: the delta angle for the U4 gate.

    Returns:
        A U4 node representing the operation.
    """
@overload
def U4(qubit_addr: int, matrix: list[complex]) -> QGate:
    """U4(*args, **kwargs)
    Overloaded function.

    1. U4(matrix: List[complex], qubit: pyQPanda.Qubit) -> pyQPanda.QGate

    Create a U4 gate.

    Args:
        matrix: the U4 gate matrix to be applied.

        qubit: the target qubit for the U4 gate.

    Returns:
        A U4 node representing the operation.


    2. U4(alpha_angle: float, beta_angle: float, gamma_angle: float, delta_angle: float, qubit: pyQPanda.Qubit) -> pyQPanda.QGate

    Create a U4 gate.

    Args:
        alpha_angle: the alpha angle for the U4 gate.

        beta_angle: the beta angle for the U4 gate.

        gamma_angle: the gamma angle for the U4 gate.

        delta_angle: the delta angle for the U4 gate.

        qubit: the target qubit for the U4 gate.

    Returns:
        A U4 node representing the operation.


    3. U4(qubit: pyQPanda.Qubit, matrix: List[complex]) -> pyQPanda.QGate

    Create a U4 gate.

    Args:
        qubit: the target qubit for the U4 gate.

        matrix: the U4 gate matrix to be applied.

    Returns:
        A U4 node representing the operation.


    4. U4(qubit_list: pyQPanda.QVec, matrix: List[complex]) -> pyQPanda.QCircuit

    Create a U4 gate.

    Args:
        qubit_list: the list of target qubits for the U4 gate.

        matrix: the U4 gate matrix to be applied.

    Returns:
        A U4 node representing the operation.


    5. U4(qubit_addr: int, matrix: List[complex]) -> pyQPanda.QGate

    Create a U4 gate.

    Args:
        qubit_addr: the address of the target qubit for the U4 gate.

        matrix: the U4 gate matrix to be applied.

    Returns:
        A U4 node representing the operation.


    6. U4(qubit_addr_list: List[int], matrix: List[complex]) -> pyQPanda.QCircuit

    Create a U4 gate.

    Args:
        qubit_addr_list: the list of addresses for the target qubits of the U4 gate.

        matrix: the U4 gate matrix to be applied.

    Returns:
        A U4 node representing the operation.


    7. U4(qubit: pyQPanda.Qubit, alpha_anlge: float, beta_anlge: float, gamma_anlge: float, delta_anlge: float) -> pyQPanda.QGate

    Create a U4 gate.

    Args:
        qubit: the target qubit for the U4 gate.

        alpha_angle: the alpha angle for the U4 gate.

        beta_angle: the beta angle for the U4 gate.

        gamma_angle: the gamma angle for the U4 gate.

        delta_angle: the delta angle for the U4 gate.

    Returns:
        A U4 node representing the operation.


    8. U4(qubit_list: pyQPanda.QVec, alpha_angle: float, beta_angle: float, gamma_angle: float, delta_angle: float) -> pyQPanda.QCircuit

    Create a U4 gate.


    Args:
        qubit_list: the list of target qubits for the U4 gate.

        alpha_angle: the alpha angle for the U4 gate.

        beta_angle: the beta angle for the U4 gate.

        gamma_angle: the gamma angle for the U4 gate.

        delta_angle: the delta angle for the U4 gate.

    Returns:
        A U4 node representing the operation.

    9. U4(qubit_addr: int, alpha_anlge: float, beta_anlge: float, gamma_anlge: float, delta_anlge: float) -> pyQPanda.QGate

    Create a U4 gate.

    Args:
        qubit_addr: the address of the target qubit for the U4 gate.

        alpha_angle: the alpha angle for the U4 gate.

        beta_angle: the beta angle for the U4 gate.

        gamma_angle: the gamma angle for the U4 gate.

        delta_angle: the delta angle for the U4 gate.

    Returns:
        A U4 node representing the operation.

    10. U4(qubit_addr_list: List[int], alpha_anlge: float, beta_anlge: float, gamma_anlge: float, delta_anlge: float) -> pyQPanda.QCircuit

    Create a U4 gate.

    Args:
        qubit_addr_list: the list of addresses of target qubits for the U4 gate.

        alpha_angle: the alpha angle for the U4 gate.

        beta_angle: the beta angle for the U4 gate.

        gamma_angle: the gamma angle for the U4 gate.

        delta_angle: the delta angle for the U4 gate.

    Returns:
        A U4 node representing the operation.
    """
@overload
def U4(qubit_addr_list: list[int], matrix: list[complex]) -> QCircuit:
    """U4(*args, **kwargs)
    Overloaded function.

    1. U4(matrix: List[complex], qubit: pyQPanda.Qubit) -> pyQPanda.QGate

    Create a U4 gate.

    Args:
        matrix: the U4 gate matrix to be applied.

        qubit: the target qubit for the U4 gate.

    Returns:
        A U4 node representing the operation.


    2. U4(alpha_angle: float, beta_angle: float, gamma_angle: float, delta_angle: float, qubit: pyQPanda.Qubit) -> pyQPanda.QGate

    Create a U4 gate.

    Args:
        alpha_angle: the alpha angle for the U4 gate.

        beta_angle: the beta angle for the U4 gate.

        gamma_angle: the gamma angle for the U4 gate.

        delta_angle: the delta angle for the U4 gate.

        qubit: the target qubit for the U4 gate.

    Returns:
        A U4 node representing the operation.


    3. U4(qubit: pyQPanda.Qubit, matrix: List[complex]) -> pyQPanda.QGate

    Create a U4 gate.

    Args:
        qubit: the target qubit for the U4 gate.

        matrix: the U4 gate matrix to be applied.

    Returns:
        A U4 node representing the operation.


    4. U4(qubit_list: pyQPanda.QVec, matrix: List[complex]) -> pyQPanda.QCircuit

    Create a U4 gate.

    Args:
        qubit_list: the list of target qubits for the U4 gate.

        matrix: the U4 gate matrix to be applied.

    Returns:
        A U4 node representing the operation.


    5. U4(qubit_addr: int, matrix: List[complex]) -> pyQPanda.QGate

    Create a U4 gate.

    Args:
        qubit_addr: the address of the target qubit for the U4 gate.

        matrix: the U4 gate matrix to be applied.

    Returns:
        A U4 node representing the operation.


    6. U4(qubit_addr_list: List[int], matrix: List[complex]) -> pyQPanda.QCircuit

    Create a U4 gate.

    Args:
        qubit_addr_list: the list of addresses for the target qubits of the U4 gate.

        matrix: the U4 gate matrix to be applied.

    Returns:
        A U4 node representing the operation.


    7. U4(qubit: pyQPanda.Qubit, alpha_anlge: float, beta_anlge: float, gamma_anlge: float, delta_anlge: float) -> pyQPanda.QGate

    Create a U4 gate.

    Args:
        qubit: the target qubit for the U4 gate.

        alpha_angle: the alpha angle for the U4 gate.

        beta_angle: the beta angle for the U4 gate.

        gamma_angle: the gamma angle for the U4 gate.

        delta_angle: the delta angle for the U4 gate.

    Returns:
        A U4 node representing the operation.


    8. U4(qubit_list: pyQPanda.QVec, alpha_angle: float, beta_angle: float, gamma_angle: float, delta_angle: float) -> pyQPanda.QCircuit

    Create a U4 gate.


    Args:
        qubit_list: the list of target qubits for the U4 gate.

        alpha_angle: the alpha angle for the U4 gate.

        beta_angle: the beta angle for the U4 gate.

        gamma_angle: the gamma angle for the U4 gate.

        delta_angle: the delta angle for the U4 gate.

    Returns:
        A U4 node representing the operation.

    9. U4(qubit_addr: int, alpha_anlge: float, beta_anlge: float, gamma_anlge: float, delta_anlge: float) -> pyQPanda.QGate

    Create a U4 gate.

    Args:
        qubit_addr: the address of the target qubit for the U4 gate.

        alpha_angle: the alpha angle for the U4 gate.

        beta_angle: the beta angle for the U4 gate.

        gamma_angle: the gamma angle for the U4 gate.

        delta_angle: the delta angle for the U4 gate.

    Returns:
        A U4 node representing the operation.

    10. U4(qubit_addr_list: List[int], alpha_anlge: float, beta_anlge: float, gamma_anlge: float, delta_anlge: float) -> pyQPanda.QCircuit

    Create a U4 gate.

    Args:
        qubit_addr_list: the list of addresses of target qubits for the U4 gate.

        alpha_angle: the alpha angle for the U4 gate.

        beta_angle: the beta angle for the U4 gate.

        gamma_angle: the gamma angle for the U4 gate.

        delta_angle: the delta angle for the U4 gate.

    Returns:
        A U4 node representing the operation.
    """
@overload
def U4(qubit: Qubit, alpha_anlge: float, beta_anlge: float, gamma_anlge: float, delta_anlge: float) -> QGate:
    """U4(*args, **kwargs)
    Overloaded function.

    1. U4(matrix: List[complex], qubit: pyQPanda.Qubit) -> pyQPanda.QGate

    Create a U4 gate.

    Args:
        matrix: the U4 gate matrix to be applied.

        qubit: the target qubit for the U4 gate.

    Returns:
        A U4 node representing the operation.


    2. U4(alpha_angle: float, beta_angle: float, gamma_angle: float, delta_angle: float, qubit: pyQPanda.Qubit) -> pyQPanda.QGate

    Create a U4 gate.

    Args:
        alpha_angle: the alpha angle for the U4 gate.

        beta_angle: the beta angle for the U4 gate.

        gamma_angle: the gamma angle for the U4 gate.

        delta_angle: the delta angle for the U4 gate.

        qubit: the target qubit for the U4 gate.

    Returns:
        A U4 node representing the operation.


    3. U4(qubit: pyQPanda.Qubit, matrix: List[complex]) -> pyQPanda.QGate

    Create a U4 gate.

    Args:
        qubit: the target qubit for the U4 gate.

        matrix: the U4 gate matrix to be applied.

    Returns:
        A U4 node representing the operation.


    4. U4(qubit_list: pyQPanda.QVec, matrix: List[complex]) -> pyQPanda.QCircuit

    Create a U4 gate.

    Args:
        qubit_list: the list of target qubits for the U4 gate.

        matrix: the U4 gate matrix to be applied.

    Returns:
        A U4 node representing the operation.


    5. U4(qubit_addr: int, matrix: List[complex]) -> pyQPanda.QGate

    Create a U4 gate.

    Args:
        qubit_addr: the address of the target qubit for the U4 gate.

        matrix: the U4 gate matrix to be applied.

    Returns:
        A U4 node representing the operation.


    6. U4(qubit_addr_list: List[int], matrix: List[complex]) -> pyQPanda.QCircuit

    Create a U4 gate.

    Args:
        qubit_addr_list: the list of addresses for the target qubits of the U4 gate.

        matrix: the U4 gate matrix to be applied.

    Returns:
        A U4 node representing the operation.


    7. U4(qubit: pyQPanda.Qubit, alpha_anlge: float, beta_anlge: float, gamma_anlge: float, delta_anlge: float) -> pyQPanda.QGate

    Create a U4 gate.

    Args:
        qubit: the target qubit for the U4 gate.

        alpha_angle: the alpha angle for the U4 gate.

        beta_angle: the beta angle for the U4 gate.

        gamma_angle: the gamma angle for the U4 gate.

        delta_angle: the delta angle for the U4 gate.

    Returns:
        A U4 node representing the operation.


    8. U4(qubit_list: pyQPanda.QVec, alpha_angle: float, beta_angle: float, gamma_angle: float, delta_angle: float) -> pyQPanda.QCircuit

    Create a U4 gate.


    Args:
        qubit_list: the list of target qubits for the U4 gate.

        alpha_angle: the alpha angle for the U4 gate.

        beta_angle: the beta angle for the U4 gate.

        gamma_angle: the gamma angle for the U4 gate.

        delta_angle: the delta angle for the U4 gate.

    Returns:
        A U4 node representing the operation.

    9. U4(qubit_addr: int, alpha_anlge: float, beta_anlge: float, gamma_anlge: float, delta_anlge: float) -> pyQPanda.QGate

    Create a U4 gate.

    Args:
        qubit_addr: the address of the target qubit for the U4 gate.

        alpha_angle: the alpha angle for the U4 gate.

        beta_angle: the beta angle for the U4 gate.

        gamma_angle: the gamma angle for the U4 gate.

        delta_angle: the delta angle for the U4 gate.

    Returns:
        A U4 node representing the operation.

    10. U4(qubit_addr_list: List[int], alpha_anlge: float, beta_anlge: float, gamma_anlge: float, delta_anlge: float) -> pyQPanda.QCircuit

    Create a U4 gate.

    Args:
        qubit_addr_list: the list of addresses of target qubits for the U4 gate.

        alpha_angle: the alpha angle for the U4 gate.

        beta_angle: the beta angle for the U4 gate.

        gamma_angle: the gamma angle for the U4 gate.

        delta_angle: the delta angle for the U4 gate.

    Returns:
        A U4 node representing the operation.
    """
@overload
def U4(qubit_list: QVec, alpha_angle: float, beta_angle: float, gamma_angle: float, delta_angle: float) -> QCircuit:
    """U4(*args, **kwargs)
    Overloaded function.

    1. U4(matrix: List[complex], qubit: pyQPanda.Qubit) -> pyQPanda.QGate

    Create a U4 gate.

    Args:
        matrix: the U4 gate matrix to be applied.

        qubit: the target qubit for the U4 gate.

    Returns:
        A U4 node representing the operation.


    2. U4(alpha_angle: float, beta_angle: float, gamma_angle: float, delta_angle: float, qubit: pyQPanda.Qubit) -> pyQPanda.QGate

    Create a U4 gate.

    Args:
        alpha_angle: the alpha angle for the U4 gate.

        beta_angle: the beta angle for the U4 gate.

        gamma_angle: the gamma angle for the U4 gate.

        delta_angle: the delta angle for the U4 gate.

        qubit: the target qubit for the U4 gate.

    Returns:
        A U4 node representing the operation.


    3. U4(qubit: pyQPanda.Qubit, matrix: List[complex]) -> pyQPanda.QGate

    Create a U4 gate.

    Args:
        qubit: the target qubit for the U4 gate.

        matrix: the U4 gate matrix to be applied.

    Returns:
        A U4 node representing the operation.


    4. U4(qubit_list: pyQPanda.QVec, matrix: List[complex]) -> pyQPanda.QCircuit

    Create a U4 gate.

    Args:
        qubit_list: the list of target qubits for the U4 gate.

        matrix: the U4 gate matrix to be applied.

    Returns:
        A U4 node representing the operation.


    5. U4(qubit_addr: int, matrix: List[complex]) -> pyQPanda.QGate

    Create a U4 gate.

    Args:
        qubit_addr: the address of the target qubit for the U4 gate.

        matrix: the U4 gate matrix to be applied.

    Returns:
        A U4 node representing the operation.


    6. U4(qubit_addr_list: List[int], matrix: List[complex]) -> pyQPanda.QCircuit

    Create a U4 gate.

    Args:
        qubit_addr_list: the list of addresses for the target qubits of the U4 gate.

        matrix: the U4 gate matrix to be applied.

    Returns:
        A U4 node representing the operation.


    7. U4(qubit: pyQPanda.Qubit, alpha_anlge: float, beta_anlge: float, gamma_anlge: float, delta_anlge: float) -> pyQPanda.QGate

    Create a U4 gate.

    Args:
        qubit: the target qubit for the U4 gate.

        alpha_angle: the alpha angle for the U4 gate.

        beta_angle: the beta angle for the U4 gate.

        gamma_angle: the gamma angle for the U4 gate.

        delta_angle: the delta angle for the U4 gate.

    Returns:
        A U4 node representing the operation.


    8. U4(qubit_list: pyQPanda.QVec, alpha_angle: float, beta_angle: float, gamma_angle: float, delta_angle: float) -> pyQPanda.QCircuit

    Create a U4 gate.


    Args:
        qubit_list: the list of target qubits for the U4 gate.

        alpha_angle: the alpha angle for the U4 gate.

        beta_angle: the beta angle for the U4 gate.

        gamma_angle: the gamma angle for the U4 gate.

        delta_angle: the delta angle for the U4 gate.

    Returns:
        A U4 node representing the operation.

    9. U4(qubit_addr: int, alpha_anlge: float, beta_anlge: float, gamma_anlge: float, delta_anlge: float) -> pyQPanda.QGate

    Create a U4 gate.

    Args:
        qubit_addr: the address of the target qubit for the U4 gate.

        alpha_angle: the alpha angle for the U4 gate.

        beta_angle: the beta angle for the U4 gate.

        gamma_angle: the gamma angle for the U4 gate.

        delta_angle: the delta angle for the U4 gate.

    Returns:
        A U4 node representing the operation.

    10. U4(qubit_addr_list: List[int], alpha_anlge: float, beta_anlge: float, gamma_anlge: float, delta_anlge: float) -> pyQPanda.QCircuit

    Create a U4 gate.

    Args:
        qubit_addr_list: the list of addresses of target qubits for the U4 gate.

        alpha_angle: the alpha angle for the U4 gate.

        beta_angle: the beta angle for the U4 gate.

        gamma_angle: the gamma angle for the U4 gate.

        delta_angle: the delta angle for the U4 gate.

    Returns:
        A U4 node representing the operation.
    """
@overload
def U4(qubit_addr: int, alpha_anlge: float, beta_anlge: float, gamma_anlge: float, delta_anlge: float) -> QGate:
    """U4(*args, **kwargs)
    Overloaded function.

    1. U4(matrix: List[complex], qubit: pyQPanda.Qubit) -> pyQPanda.QGate

    Create a U4 gate.

    Args:
        matrix: the U4 gate matrix to be applied.

        qubit: the target qubit for the U4 gate.

    Returns:
        A U4 node representing the operation.


    2. U4(alpha_angle: float, beta_angle: float, gamma_angle: float, delta_angle: float, qubit: pyQPanda.Qubit) -> pyQPanda.QGate

    Create a U4 gate.

    Args:
        alpha_angle: the alpha angle for the U4 gate.

        beta_angle: the beta angle for the U4 gate.

        gamma_angle: the gamma angle for the U4 gate.

        delta_angle: the delta angle for the U4 gate.

        qubit: the target qubit for the U4 gate.

    Returns:
        A U4 node representing the operation.


    3. U4(qubit: pyQPanda.Qubit, matrix: List[complex]) -> pyQPanda.QGate

    Create a U4 gate.

    Args:
        qubit: the target qubit for the U4 gate.

        matrix: the U4 gate matrix to be applied.

    Returns:
        A U4 node representing the operation.


    4. U4(qubit_list: pyQPanda.QVec, matrix: List[complex]) -> pyQPanda.QCircuit

    Create a U4 gate.

    Args:
        qubit_list: the list of target qubits for the U4 gate.

        matrix: the U4 gate matrix to be applied.

    Returns:
        A U4 node representing the operation.


    5. U4(qubit_addr: int, matrix: List[complex]) -> pyQPanda.QGate

    Create a U4 gate.

    Args:
        qubit_addr: the address of the target qubit for the U4 gate.

        matrix: the U4 gate matrix to be applied.

    Returns:
        A U4 node representing the operation.


    6. U4(qubit_addr_list: List[int], matrix: List[complex]) -> pyQPanda.QCircuit

    Create a U4 gate.

    Args:
        qubit_addr_list: the list of addresses for the target qubits of the U4 gate.

        matrix: the U4 gate matrix to be applied.

    Returns:
        A U4 node representing the operation.


    7. U4(qubit: pyQPanda.Qubit, alpha_anlge: float, beta_anlge: float, gamma_anlge: float, delta_anlge: float) -> pyQPanda.QGate

    Create a U4 gate.

    Args:
        qubit: the target qubit for the U4 gate.

        alpha_angle: the alpha angle for the U4 gate.

        beta_angle: the beta angle for the U4 gate.

        gamma_angle: the gamma angle for the U4 gate.

        delta_angle: the delta angle for the U4 gate.

    Returns:
        A U4 node representing the operation.


    8. U4(qubit_list: pyQPanda.QVec, alpha_angle: float, beta_angle: float, gamma_angle: float, delta_angle: float) -> pyQPanda.QCircuit

    Create a U4 gate.


    Args:
        qubit_list: the list of target qubits for the U4 gate.

        alpha_angle: the alpha angle for the U4 gate.

        beta_angle: the beta angle for the U4 gate.

        gamma_angle: the gamma angle for the U4 gate.

        delta_angle: the delta angle for the U4 gate.

    Returns:
        A U4 node representing the operation.

    9. U4(qubit_addr: int, alpha_anlge: float, beta_anlge: float, gamma_anlge: float, delta_anlge: float) -> pyQPanda.QGate

    Create a U4 gate.

    Args:
        qubit_addr: the address of the target qubit for the U4 gate.

        alpha_angle: the alpha angle for the U4 gate.

        beta_angle: the beta angle for the U4 gate.

        gamma_angle: the gamma angle for the U4 gate.

        delta_angle: the delta angle for the U4 gate.

    Returns:
        A U4 node representing the operation.

    10. U4(qubit_addr_list: List[int], alpha_anlge: float, beta_anlge: float, gamma_anlge: float, delta_anlge: float) -> pyQPanda.QCircuit

    Create a U4 gate.

    Args:
        qubit_addr_list: the list of addresses of target qubits for the U4 gate.

        alpha_angle: the alpha angle for the U4 gate.

        beta_angle: the beta angle for the U4 gate.

        gamma_angle: the gamma angle for the U4 gate.

        delta_angle: the delta angle for the U4 gate.

    Returns:
        A U4 node representing the operation.
    """
@overload
def U4(qubit_addr_list: list[int], alpha_anlge: float, beta_anlge: float, gamma_anlge: float, delta_anlge: float) -> QCircuit:
    """U4(*args, **kwargs)
    Overloaded function.

    1. U4(matrix: List[complex], qubit: pyQPanda.Qubit) -> pyQPanda.QGate

    Create a U4 gate.

    Args:
        matrix: the U4 gate matrix to be applied.

        qubit: the target qubit for the U4 gate.

    Returns:
        A U4 node representing the operation.


    2. U4(alpha_angle: float, beta_angle: float, gamma_angle: float, delta_angle: float, qubit: pyQPanda.Qubit) -> pyQPanda.QGate

    Create a U4 gate.

    Args:
        alpha_angle: the alpha angle for the U4 gate.

        beta_angle: the beta angle for the U4 gate.

        gamma_angle: the gamma angle for the U4 gate.

        delta_angle: the delta angle for the U4 gate.

        qubit: the target qubit for the U4 gate.

    Returns:
        A U4 node representing the operation.


    3. U4(qubit: pyQPanda.Qubit, matrix: List[complex]) -> pyQPanda.QGate

    Create a U4 gate.

    Args:
        qubit: the target qubit for the U4 gate.

        matrix: the U4 gate matrix to be applied.

    Returns:
        A U4 node representing the operation.


    4. U4(qubit_list: pyQPanda.QVec, matrix: List[complex]) -> pyQPanda.QCircuit

    Create a U4 gate.

    Args:
        qubit_list: the list of target qubits for the U4 gate.

        matrix: the U4 gate matrix to be applied.

    Returns:
        A U4 node representing the operation.


    5. U4(qubit_addr: int, matrix: List[complex]) -> pyQPanda.QGate

    Create a U4 gate.

    Args:
        qubit_addr: the address of the target qubit for the U4 gate.

        matrix: the U4 gate matrix to be applied.

    Returns:
        A U4 node representing the operation.


    6. U4(qubit_addr_list: List[int], matrix: List[complex]) -> pyQPanda.QCircuit

    Create a U4 gate.

    Args:
        qubit_addr_list: the list of addresses for the target qubits of the U4 gate.

        matrix: the U4 gate matrix to be applied.

    Returns:
        A U4 node representing the operation.


    7. U4(qubit: pyQPanda.Qubit, alpha_anlge: float, beta_anlge: float, gamma_anlge: float, delta_anlge: float) -> pyQPanda.QGate

    Create a U4 gate.

    Args:
        qubit: the target qubit for the U4 gate.

        alpha_angle: the alpha angle for the U4 gate.

        beta_angle: the beta angle for the U4 gate.

        gamma_angle: the gamma angle for the U4 gate.

        delta_angle: the delta angle for the U4 gate.

    Returns:
        A U4 node representing the operation.


    8. U4(qubit_list: pyQPanda.QVec, alpha_angle: float, beta_angle: float, gamma_angle: float, delta_angle: float) -> pyQPanda.QCircuit

    Create a U4 gate.


    Args:
        qubit_list: the list of target qubits for the U4 gate.

        alpha_angle: the alpha angle for the U4 gate.

        beta_angle: the beta angle for the U4 gate.

        gamma_angle: the gamma angle for the U4 gate.

        delta_angle: the delta angle for the U4 gate.

    Returns:
        A U4 node representing the operation.

    9. U4(qubit_addr: int, alpha_anlge: float, beta_anlge: float, gamma_anlge: float, delta_anlge: float) -> pyQPanda.QGate

    Create a U4 gate.

    Args:
        qubit_addr: the address of the target qubit for the U4 gate.

        alpha_angle: the alpha angle for the U4 gate.

        beta_angle: the beta angle for the U4 gate.

        gamma_angle: the gamma angle for the U4 gate.

        delta_angle: the delta angle for the U4 gate.

    Returns:
        A U4 node representing the operation.

    10. U4(qubit_addr_list: List[int], alpha_anlge: float, beta_anlge: float, gamma_anlge: float, delta_anlge: float) -> pyQPanda.QCircuit

    Create a U4 gate.

    Args:
        qubit_addr_list: the list of addresses of target qubits for the U4 gate.

        alpha_angle: the alpha angle for the U4 gate.

        beta_angle: the beta angle for the U4 gate.

        gamma_angle: the gamma angle for the U4 gate.

        delta_angle: the delta angle for the U4 gate.

    Returns:
        A U4 node representing the operation.
    """
def UMA(arg0: Qubit, arg1: Qubit, arg2: Qubit) -> QCircuit:
    """UMA(arg0: pyQPanda.Qubit, arg1: pyQPanda.Qubit, arg2: pyQPanda.Qubit) -> pyQPanda.QCircuit

    Quantum adder UMA module
    """
def VQG_CNOT_batch(*args, **kwargs):
    """VQG_CNOT_batch(arg0: pyQPanda.QVec, arg1: pyQPanda.QVec) -> QPanda::Variational::VariationalQuantumCircuit

    variational quantum CNOT batch gates
    """
def VQG_CU_batch(*args, **kwargs):
    """VQG_CU_batch(arg0: pyQPanda.QVec, arg1: pyQPanda.QVec, arg2: QPanda::Variational::var, arg3: QPanda::Variational::var, arg4: QPanda::Variational::var, arg5: QPanda::Variational::var) -> QPanda::Variational::VariationalQuantumCircuit

    variational quantum CU batch gates
    """
def VQG_CZ_batch(*args, **kwargs):
    """VQG_CZ_batch(arg0: pyQPanda.QVec, arg1: pyQPanda.QVec) -> QPanda::Variational::VariationalQuantumCircuit

    variational quantum CZ batch gates
    """
def VQG_H_batch(*args, **kwargs):
    """VQG_H_batch(arg0: pyQPanda.QVec) -> QPanda::Variational::VariationalQuantumCircuit

    variational quantum H batch gates
    """
def VQG_I_batch(*args, **kwargs):
    """VQG_I_batch(arg0: pyQPanda.QVec) -> QPanda::Variational::VariationalQuantumCircuit

    variational quantum I batch gates
    """
def VQG_SWAP_batch(*args, **kwargs):
    """VQG_SWAP_batch(arg0: pyQPanda.QVec, arg1: pyQPanda.QVec) -> QPanda::Variational::VariationalQuantumCircuit

    variational quantum SWAP batch gates
    """
def VQG_S_batch(*args, **kwargs):
    """VQG_S_batch(arg0: pyQPanda.QVec) -> QPanda::Variational::VariationalQuantumCircuit

    variational quantum S batch gates
    """
def VQG_SqiSWAP_batch(*args, **kwargs):
    """VQG_SqiSWAP_batch(arg0: pyQPanda.QVec, arg1: pyQPanda.QVec) -> QPanda::Variational::VariationalQuantumCircuit

    variational quantum SqiSWAP batch gates
    """
def VQG_T_batch(*args, **kwargs):
    """VQG_T_batch(arg0: pyQPanda.QVec) -> QPanda::Variational::VariationalQuantumCircuit

    variational quantum T batch gates
    """
def VQG_U1_batch(*args, **kwargs):
    """VQG_U1_batch(arg0: pyQPanda.QVec, arg1: QPanda::Variational::var) -> QPanda::Variational::VariationalQuantumCircuit

    variational quantum U1 batch gates
    """
def VQG_U2_batch(*args, **kwargs):
    """VQG_U2_batch(arg0: pyQPanda.QVec, arg1: QPanda::Variational::var, arg2: QPanda::Variational::var) -> QPanda::Variational::VariationalQuantumCircuit

    variational quantum U2 batch gates
    """
def VQG_U3_batch(*args, **kwargs):
    """VQG_U3_batch(arg0: pyQPanda.QVec, arg1: QPanda::Variational::var, arg2: QPanda::Variational::var, arg3: QPanda::Variational::var) -> QPanda::Variational::VariationalQuantumCircuit

    variational quantum U3 batch gates
    """
def VQG_U4_batch(*args, **kwargs):
    """VQG_U4_batch(arg0: pyQPanda.QVec, arg1: QPanda::Variational::var, arg2: QPanda::Variational::var, arg3: QPanda::Variational::var, arg4: QPanda::Variational::var) -> QPanda::Variational::VariationalQuantumCircuit

    variational quantum U4 batch gates
    """
def VQG_X1_batch(*args, **kwargs):
    """VQG_X1_batch(arg0: pyQPanda.QVec) -> QPanda::Variational::VariationalQuantumCircuit

    variational quantum X1 batch gates
    """
def VQG_X_batch(*args, **kwargs):
    """VQG_X_batch(arg0: pyQPanda.QVec) -> QPanda::Variational::VariationalQuantumCircuit

    variational quantum X batch gates
    """
def VQG_Y1_batch(*args, **kwargs):
    """VQG_Y1_batch(arg0: pyQPanda.QVec) -> QPanda::Variational::VariationalQuantumCircuit

    variational quantum Y1 batch gates
    """
def VQG_Y_batch(*args, **kwargs):
    """VQG_Y_batch(arg0: pyQPanda.QVec) -> QPanda::Variational::VariationalQuantumCircuit

    variational quantum Y batch gates
    """
def VQG_Z1_batch(*args, **kwargs):
    """VQG_Z1_batch(arg0: pyQPanda.QVec) -> QPanda::Variational::VariationalQuantumCircuit

    variational quantum Z1 batch gates
    """
def VQG_Z_batch(*args, **kwargs):
    """VQG_Z_batch(arg0: pyQPanda.QVec) -> QPanda::Variational::VariationalQuantumCircuit

    variational quantum Z batch gates
    """
def VQG_iSWAP_batch(*args, **kwargs):
    """VQG_iSWAP_batch(arg0: pyQPanda.QVec, arg1: pyQPanda.QVec) -> QPanda::Variational::VariationalQuantumCircuit

    variational quantum iSWAP batch gates
    """
@overload
def X(qubit: Qubit) -> QGate:
    """X(*args, **kwargs)
    Overloaded function.

    1. X(qubit: pyQPanda.Qubit) -> pyQPanda.QGate

    Create a X gate

    Args:
        qubit : quantum gate operate qubit

    Returns:
        a X gate node
    Raises:
        run_fail: An error occurred in construct gate node


    2. X(qubit_list: pyQPanda.QVec) -> pyQPanda.QCircuit

    Create a X gate

    Args:
        qubit_list: quantum gate operate qubits list

    Returns:
        a X gate node
    Raises:
        run_fail: An error occurred construct in gate node


    3. X(qubit_addr: int) -> pyQPanda.QGate

    Create a X gate

    Args:
        qubit_addr: quantum gate operate qubits addr

    Returns:
        a X gate node
    Raises:
        run_fail: An error occurred in construct gate node


    4. X(qubit_addr_list: List[int]) -> pyQPanda.QCircuit

    Create a X gate

    Args:
        qubit_list_addr: quantum gate  qubits list addr

    Returns:
        a X gate node
    Raises:
        run_fail: An error occurred in construct gate node

    """
@overload
def X(qubit_list: QVec) -> QCircuit:
    """X(*args, **kwargs)
    Overloaded function.

    1. X(qubit: pyQPanda.Qubit) -> pyQPanda.QGate

    Create a X gate

    Args:
        qubit : quantum gate operate qubit

    Returns:
        a X gate node
    Raises:
        run_fail: An error occurred in construct gate node


    2. X(qubit_list: pyQPanda.QVec) -> pyQPanda.QCircuit

    Create a X gate

    Args:
        qubit_list: quantum gate operate qubits list

    Returns:
        a X gate node
    Raises:
        run_fail: An error occurred construct in gate node


    3. X(qubit_addr: int) -> pyQPanda.QGate

    Create a X gate

    Args:
        qubit_addr: quantum gate operate qubits addr

    Returns:
        a X gate node
    Raises:
        run_fail: An error occurred in construct gate node


    4. X(qubit_addr_list: List[int]) -> pyQPanda.QCircuit

    Create a X gate

    Args:
        qubit_list_addr: quantum gate  qubits list addr

    Returns:
        a X gate node
    Raises:
        run_fail: An error occurred in construct gate node

    """
@overload
def X(qubit_addr: int) -> QGate:
    """X(*args, **kwargs)
    Overloaded function.

    1. X(qubit: pyQPanda.Qubit) -> pyQPanda.QGate

    Create a X gate

    Args:
        qubit : quantum gate operate qubit

    Returns:
        a X gate node
    Raises:
        run_fail: An error occurred in construct gate node


    2. X(qubit_list: pyQPanda.QVec) -> pyQPanda.QCircuit

    Create a X gate

    Args:
        qubit_list: quantum gate operate qubits list

    Returns:
        a X gate node
    Raises:
        run_fail: An error occurred construct in gate node


    3. X(qubit_addr: int) -> pyQPanda.QGate

    Create a X gate

    Args:
        qubit_addr: quantum gate operate qubits addr

    Returns:
        a X gate node
    Raises:
        run_fail: An error occurred in construct gate node


    4. X(qubit_addr_list: List[int]) -> pyQPanda.QCircuit

    Create a X gate

    Args:
        qubit_list_addr: quantum gate  qubits list addr

    Returns:
        a X gate node
    Raises:
        run_fail: An error occurred in construct gate node

    """
@overload
def X(qubit_addr_list: list[int]) -> QCircuit:
    """X(*args, **kwargs)
    Overloaded function.

    1. X(qubit: pyQPanda.Qubit) -> pyQPanda.QGate

    Create a X gate

    Args:
        qubit : quantum gate operate qubit

    Returns:
        a X gate node
    Raises:
        run_fail: An error occurred in construct gate node


    2. X(qubit_list: pyQPanda.QVec) -> pyQPanda.QCircuit

    Create a X gate

    Args:
        qubit_list: quantum gate operate qubits list

    Returns:
        a X gate node
    Raises:
        run_fail: An error occurred construct in gate node


    3. X(qubit_addr: int) -> pyQPanda.QGate

    Create a X gate

    Args:
        qubit_addr: quantum gate operate qubits addr

    Returns:
        a X gate node
    Raises:
        run_fail: An error occurred in construct gate node


    4. X(qubit_addr_list: List[int]) -> pyQPanda.QCircuit

    Create a X gate

    Args:
        qubit_list_addr: quantum gate  qubits list addr

    Returns:
        a X gate node
    Raises:
        run_fail: An error occurred in construct gate node

    """
@overload
def X1(qubit: Qubit) -> QGate:
    """X1(*args, **kwargs)
    Overloaded function.

    1. X1(qubit: pyQPanda.Qubit) -> pyQPanda.QGate

    Create a X1 gate

    Args:
        qubit : quantum gate operate qubit

    Returns:
        a X1 gate node
    Raises:
        run_fail: An error occurred in construct gate node


    2. X1(qubit_list: pyQPanda.QVec) -> pyQPanda.QCircuit

    Create a X1 gate

    Args:
        qubit_list: quantum gate operate qubits list

    Returns:
        a X1 gate node
    Raises:
        run_fail: An error occurred construct in gate node


    3. X1(qubit_addr: int) -> pyQPanda.QGate

    Create a X1 gate

    Args:
        qubit_addr: quantum gate operate qubits addr

    Returns:
        a X1 gate node
    Raises:
        run_fail: An error occurred in construct gate node


    4. X1(qubit_addr_list: List[int]) -> pyQPanda.QCircuit

    Create a X1 gate

    Args:
        qubit_list_addr: quantum gate  qubits list addr

    Returns:
        a X1 gate node
    Raises:
        run_fail: An error occurred in construct gate node

    """
@overload
def X1(qubit_list: QVec) -> QCircuit:
    """X1(*args, **kwargs)
    Overloaded function.

    1. X1(qubit: pyQPanda.Qubit) -> pyQPanda.QGate

    Create a X1 gate

    Args:
        qubit : quantum gate operate qubit

    Returns:
        a X1 gate node
    Raises:
        run_fail: An error occurred in construct gate node


    2. X1(qubit_list: pyQPanda.QVec) -> pyQPanda.QCircuit

    Create a X1 gate

    Args:
        qubit_list: quantum gate operate qubits list

    Returns:
        a X1 gate node
    Raises:
        run_fail: An error occurred construct in gate node


    3. X1(qubit_addr: int) -> pyQPanda.QGate

    Create a X1 gate

    Args:
        qubit_addr: quantum gate operate qubits addr

    Returns:
        a X1 gate node
    Raises:
        run_fail: An error occurred in construct gate node


    4. X1(qubit_addr_list: List[int]) -> pyQPanda.QCircuit

    Create a X1 gate

    Args:
        qubit_list_addr: quantum gate  qubits list addr

    Returns:
        a X1 gate node
    Raises:
        run_fail: An error occurred in construct gate node

    """
@overload
def X1(qubit_addr: int) -> QGate:
    """X1(*args, **kwargs)
    Overloaded function.

    1. X1(qubit: pyQPanda.Qubit) -> pyQPanda.QGate

    Create a X1 gate

    Args:
        qubit : quantum gate operate qubit

    Returns:
        a X1 gate node
    Raises:
        run_fail: An error occurred in construct gate node


    2. X1(qubit_list: pyQPanda.QVec) -> pyQPanda.QCircuit

    Create a X1 gate

    Args:
        qubit_list: quantum gate operate qubits list

    Returns:
        a X1 gate node
    Raises:
        run_fail: An error occurred construct in gate node


    3. X1(qubit_addr: int) -> pyQPanda.QGate

    Create a X1 gate

    Args:
        qubit_addr: quantum gate operate qubits addr

    Returns:
        a X1 gate node
    Raises:
        run_fail: An error occurred in construct gate node


    4. X1(qubit_addr_list: List[int]) -> pyQPanda.QCircuit

    Create a X1 gate

    Args:
        qubit_list_addr: quantum gate  qubits list addr

    Returns:
        a X1 gate node
    Raises:
        run_fail: An error occurred in construct gate node

    """
@overload
def X1(qubit_addr_list: list[int]) -> QCircuit:
    """X1(*args, **kwargs)
    Overloaded function.

    1. X1(qubit: pyQPanda.Qubit) -> pyQPanda.QGate

    Create a X1 gate

    Args:
        qubit : quantum gate operate qubit

    Returns:
        a X1 gate node
    Raises:
        run_fail: An error occurred in construct gate node


    2. X1(qubit_list: pyQPanda.QVec) -> pyQPanda.QCircuit

    Create a X1 gate

    Args:
        qubit_list: quantum gate operate qubits list

    Returns:
        a X1 gate node
    Raises:
        run_fail: An error occurred construct in gate node


    3. X1(qubit_addr: int) -> pyQPanda.QGate

    Create a X1 gate

    Args:
        qubit_addr: quantum gate operate qubits addr

    Returns:
        a X1 gate node
    Raises:
        run_fail: An error occurred in construct gate node


    4. X1(qubit_addr_list: List[int]) -> pyQPanda.QCircuit

    Create a X1 gate

    Args:
        qubit_list_addr: quantum gate  qubits list addr

    Returns:
        a X1 gate node
    Raises:
        run_fail: An error occurred in construct gate node

    """
@overload
def Y(qubit: Qubit) -> QGate:
    """Y(*args, **kwargs)
    Overloaded function.

    1. Y(qubit: pyQPanda.Qubit) -> pyQPanda.QGate

    Create a Y gate

    Args:
        qubit : quantum gate operate qubit

    Returns:
        a Y gate node
    Raises:
        run_fail: An error occurred in construct gate node


    2. Y(qubit_list: pyQPanda.QVec) -> pyQPanda.QCircuit

    Create a Y gate

    Args:
        qubit_list: quantum gate operate qubits list

    Returns:
        a Y gate node
    Raises:
        run_fail: An error occurred construct in gate node


    3. Y(qubit_addr: int) -> pyQPanda.QGate

    Create a Y gate

    Args:
        qubit_addr: quantum gate operate qubits addr

    Returns:
        a Y gate node
    Raises:
        run_fail: An error occurred in construct gate node


    4. Y(qubit_addr_list: List[int]) -> pyQPanda.QCircuit

    Create a Y gate

    Args:
        qubit_list_addr: quantum gate  qubits list addr

    Returns:
        a Y gate node
    Raises:
        run_fail: An error occurred in construct gate node

    """
@overload
def Y(qubit_list: QVec) -> QCircuit:
    """Y(*args, **kwargs)
    Overloaded function.

    1. Y(qubit: pyQPanda.Qubit) -> pyQPanda.QGate

    Create a Y gate

    Args:
        qubit : quantum gate operate qubit

    Returns:
        a Y gate node
    Raises:
        run_fail: An error occurred in construct gate node


    2. Y(qubit_list: pyQPanda.QVec) -> pyQPanda.QCircuit

    Create a Y gate

    Args:
        qubit_list: quantum gate operate qubits list

    Returns:
        a Y gate node
    Raises:
        run_fail: An error occurred construct in gate node


    3. Y(qubit_addr: int) -> pyQPanda.QGate

    Create a Y gate

    Args:
        qubit_addr: quantum gate operate qubits addr

    Returns:
        a Y gate node
    Raises:
        run_fail: An error occurred in construct gate node


    4. Y(qubit_addr_list: List[int]) -> pyQPanda.QCircuit

    Create a Y gate

    Args:
        qubit_list_addr: quantum gate  qubits list addr

    Returns:
        a Y gate node
    Raises:
        run_fail: An error occurred in construct gate node

    """
@overload
def Y(qubit_addr: int) -> QGate:
    """Y(*args, **kwargs)
    Overloaded function.

    1. Y(qubit: pyQPanda.Qubit) -> pyQPanda.QGate

    Create a Y gate

    Args:
        qubit : quantum gate operate qubit

    Returns:
        a Y gate node
    Raises:
        run_fail: An error occurred in construct gate node


    2. Y(qubit_list: pyQPanda.QVec) -> pyQPanda.QCircuit

    Create a Y gate

    Args:
        qubit_list: quantum gate operate qubits list

    Returns:
        a Y gate node
    Raises:
        run_fail: An error occurred construct in gate node


    3. Y(qubit_addr: int) -> pyQPanda.QGate

    Create a Y gate

    Args:
        qubit_addr: quantum gate operate qubits addr

    Returns:
        a Y gate node
    Raises:
        run_fail: An error occurred in construct gate node


    4. Y(qubit_addr_list: List[int]) -> pyQPanda.QCircuit

    Create a Y gate

    Args:
        qubit_list_addr: quantum gate  qubits list addr

    Returns:
        a Y gate node
    Raises:
        run_fail: An error occurred in construct gate node

    """
@overload
def Y(qubit_addr_list: list[int]) -> QCircuit:
    """Y(*args, **kwargs)
    Overloaded function.

    1. Y(qubit: pyQPanda.Qubit) -> pyQPanda.QGate

    Create a Y gate

    Args:
        qubit : quantum gate operate qubit

    Returns:
        a Y gate node
    Raises:
        run_fail: An error occurred in construct gate node


    2. Y(qubit_list: pyQPanda.QVec) -> pyQPanda.QCircuit

    Create a Y gate

    Args:
        qubit_list: quantum gate operate qubits list

    Returns:
        a Y gate node
    Raises:
        run_fail: An error occurred construct in gate node


    3. Y(qubit_addr: int) -> pyQPanda.QGate

    Create a Y gate

    Args:
        qubit_addr: quantum gate operate qubits addr

    Returns:
        a Y gate node
    Raises:
        run_fail: An error occurred in construct gate node


    4. Y(qubit_addr_list: List[int]) -> pyQPanda.QCircuit

    Create a Y gate

    Args:
        qubit_list_addr: quantum gate  qubits list addr

    Returns:
        a Y gate node
    Raises:
        run_fail: An error occurred in construct gate node

    """
@overload
def Y1(qubit: Qubit) -> QGate:
    """Y1(*args, **kwargs)
    Overloaded function.

    1. Y1(qubit: pyQPanda.Qubit) -> pyQPanda.QGate

    Create a Y1 gate

    Args:
        qubit : quantum gate operate qubit

    Returns:
        a Y1 gate node
    Raises:
        run_fail: An error occurred in construct gate node


    2. Y1(qubit_list: pyQPanda.QVec) -> pyQPanda.QCircuit

    Create a Y1 gate

    Args:
        qubit_list: quantum gate operate qubits list

    Returns:
        a Y1 gate node
    Raises:
        run_fail: An error occurred construct in gate node


    3. Y1(qubit_addr: int) -> pyQPanda.QGate

    Create a Y1 gate

    Args:
        qubit_addr: quantum gate operate qubits addr

    Returns:
        a Y1 gate node
    Raises:
        run_fail: An error occurred in construct gate node


    4. Y1(qubit_addr_list: List[int]) -> pyQPanda.QCircuit

    Create a Y1 gate

    Args:
        qubit_list_addr: quantum gate  qubits list addr

    Returns:
        a Y1 gate node
    Raises:
        run_fail: An error occurred in construct gate node

    """
@overload
def Y1(qubit_list: QVec) -> QCircuit:
    """Y1(*args, **kwargs)
    Overloaded function.

    1. Y1(qubit: pyQPanda.Qubit) -> pyQPanda.QGate

    Create a Y1 gate

    Args:
        qubit : quantum gate operate qubit

    Returns:
        a Y1 gate node
    Raises:
        run_fail: An error occurred in construct gate node


    2. Y1(qubit_list: pyQPanda.QVec) -> pyQPanda.QCircuit

    Create a Y1 gate

    Args:
        qubit_list: quantum gate operate qubits list

    Returns:
        a Y1 gate node
    Raises:
        run_fail: An error occurred construct in gate node


    3. Y1(qubit_addr: int) -> pyQPanda.QGate

    Create a Y1 gate

    Args:
        qubit_addr: quantum gate operate qubits addr

    Returns:
        a Y1 gate node
    Raises:
        run_fail: An error occurred in construct gate node


    4. Y1(qubit_addr_list: List[int]) -> pyQPanda.QCircuit

    Create a Y1 gate

    Args:
        qubit_list_addr: quantum gate  qubits list addr

    Returns:
        a Y1 gate node
    Raises:
        run_fail: An error occurred in construct gate node

    """
@overload
def Y1(qubit_addr: int) -> QGate:
    """Y1(*args, **kwargs)
    Overloaded function.

    1. Y1(qubit: pyQPanda.Qubit) -> pyQPanda.QGate

    Create a Y1 gate

    Args:
        qubit : quantum gate operate qubit

    Returns:
        a Y1 gate node
    Raises:
        run_fail: An error occurred in construct gate node


    2. Y1(qubit_list: pyQPanda.QVec) -> pyQPanda.QCircuit

    Create a Y1 gate

    Args:
        qubit_list: quantum gate operate qubits list

    Returns:
        a Y1 gate node
    Raises:
        run_fail: An error occurred construct in gate node


    3. Y1(qubit_addr: int) -> pyQPanda.QGate

    Create a Y1 gate

    Args:
        qubit_addr: quantum gate operate qubits addr

    Returns:
        a Y1 gate node
    Raises:
        run_fail: An error occurred in construct gate node


    4. Y1(qubit_addr_list: List[int]) -> pyQPanda.QCircuit

    Create a Y1 gate

    Args:
        qubit_list_addr: quantum gate  qubits list addr

    Returns:
        a Y1 gate node
    Raises:
        run_fail: An error occurred in construct gate node

    """
@overload
def Y1(qubit_addr_list: list[int]) -> QCircuit:
    """Y1(*args, **kwargs)
    Overloaded function.

    1. Y1(qubit: pyQPanda.Qubit) -> pyQPanda.QGate

    Create a Y1 gate

    Args:
        qubit : quantum gate operate qubit

    Returns:
        a Y1 gate node
    Raises:
        run_fail: An error occurred in construct gate node


    2. Y1(qubit_list: pyQPanda.QVec) -> pyQPanda.QCircuit

    Create a Y1 gate

    Args:
        qubit_list: quantum gate operate qubits list

    Returns:
        a Y1 gate node
    Raises:
        run_fail: An error occurred construct in gate node


    3. Y1(qubit_addr: int) -> pyQPanda.QGate

    Create a Y1 gate

    Args:
        qubit_addr: quantum gate operate qubits addr

    Returns:
        a Y1 gate node
    Raises:
        run_fail: An error occurred in construct gate node


    4. Y1(qubit_addr_list: List[int]) -> pyQPanda.QCircuit

    Create a Y1 gate

    Args:
        qubit_list_addr: quantum gate  qubits list addr

    Returns:
        a Y1 gate node
    Raises:
        run_fail: An error occurred in construct gate node

    """
@overload
def Z(qubit: Qubit) -> QGate:
    """Z(*args, **kwargs)
    Overloaded function.

    1. Z(qubit: pyQPanda.Qubit) -> pyQPanda.QGate

    Create a Z gate

    Args:
        qubit : quantum gate operate qubit

    Returns:
        a Z gate node
    Raises:
        run_fail: An error occurred in construct gate node


    2. Z(qubit_list: pyQPanda.QVec) -> pyQPanda.QCircuit

    Create a Z gate

    Args:
        qubit_list: quantum gate operate qubits list

    Returns:
        a Z gate node
    Raises:
        run_fail: An error occurred construct in gate node


    3. Z(qubit_addr: int) -> pyQPanda.QGate

    Create a Z gate

    Args:
        qubit_addr: quantum gate operate qubits addr

    Returns:
        a Z gate node
    Raises:
        run_fail: An error occurred in construct gate node


    4. Z(qubit_addr_list: List[int]) -> pyQPanda.QCircuit

    Create a Z gate

    Args:
        qubit_list_addr: quantum gate  qubits list addr

    Returns:
        a Z gate node
    Raises:
        run_fail: An error occurred in construct gate node

    """
@overload
def Z(qubit_list: QVec) -> QCircuit:
    """Z(*args, **kwargs)
    Overloaded function.

    1. Z(qubit: pyQPanda.Qubit) -> pyQPanda.QGate

    Create a Z gate

    Args:
        qubit : quantum gate operate qubit

    Returns:
        a Z gate node
    Raises:
        run_fail: An error occurred in construct gate node


    2. Z(qubit_list: pyQPanda.QVec) -> pyQPanda.QCircuit

    Create a Z gate

    Args:
        qubit_list: quantum gate operate qubits list

    Returns:
        a Z gate node
    Raises:
        run_fail: An error occurred construct in gate node


    3. Z(qubit_addr: int) -> pyQPanda.QGate

    Create a Z gate

    Args:
        qubit_addr: quantum gate operate qubits addr

    Returns:
        a Z gate node
    Raises:
        run_fail: An error occurred in construct gate node


    4. Z(qubit_addr_list: List[int]) -> pyQPanda.QCircuit

    Create a Z gate

    Args:
        qubit_list_addr: quantum gate  qubits list addr

    Returns:
        a Z gate node
    Raises:
        run_fail: An error occurred in construct gate node

    """
@overload
def Z(qubit_addr: int) -> QGate:
    """Z(*args, **kwargs)
    Overloaded function.

    1. Z(qubit: pyQPanda.Qubit) -> pyQPanda.QGate

    Create a Z gate

    Args:
        qubit : quantum gate operate qubit

    Returns:
        a Z gate node
    Raises:
        run_fail: An error occurred in construct gate node


    2. Z(qubit_list: pyQPanda.QVec) -> pyQPanda.QCircuit

    Create a Z gate

    Args:
        qubit_list: quantum gate operate qubits list

    Returns:
        a Z gate node
    Raises:
        run_fail: An error occurred construct in gate node


    3. Z(qubit_addr: int) -> pyQPanda.QGate

    Create a Z gate

    Args:
        qubit_addr: quantum gate operate qubits addr

    Returns:
        a Z gate node
    Raises:
        run_fail: An error occurred in construct gate node


    4. Z(qubit_addr_list: List[int]) -> pyQPanda.QCircuit

    Create a Z gate

    Args:
        qubit_list_addr: quantum gate  qubits list addr

    Returns:
        a Z gate node
    Raises:
        run_fail: An error occurred in construct gate node

    """
@overload
def Z(qubit_addr_list: list[int]) -> QCircuit:
    """Z(*args, **kwargs)
    Overloaded function.

    1. Z(qubit: pyQPanda.Qubit) -> pyQPanda.QGate

    Create a Z gate

    Args:
        qubit : quantum gate operate qubit

    Returns:
        a Z gate node
    Raises:
        run_fail: An error occurred in construct gate node


    2. Z(qubit_list: pyQPanda.QVec) -> pyQPanda.QCircuit

    Create a Z gate

    Args:
        qubit_list: quantum gate operate qubits list

    Returns:
        a Z gate node
    Raises:
        run_fail: An error occurred construct in gate node


    3. Z(qubit_addr: int) -> pyQPanda.QGate

    Create a Z gate

    Args:
        qubit_addr: quantum gate operate qubits addr

    Returns:
        a Z gate node
    Raises:
        run_fail: An error occurred in construct gate node


    4. Z(qubit_addr_list: List[int]) -> pyQPanda.QCircuit

    Create a Z gate

    Args:
        qubit_list_addr: quantum gate  qubits list addr

    Returns:
        a Z gate node
    Raises:
        run_fail: An error occurred in construct gate node

    """
@overload
def Z1(qubit: Qubit) -> QGate:
    """Z1(*args, **kwargs)
    Overloaded function.

    1. Z1(qubit: pyQPanda.Qubit) -> pyQPanda.QGate

    Create a Z1 gate

    Args:
        qubit : quantum gate operate qubit

    Returns:
        a Z1 gate node
    Raises:
        run_fail: An error occurred in construct gate node


    2. Z1(qubit_list: pyQPanda.QVec) -> pyQPanda.QCircuit

    Create a Z1 gate

    Args:
        qubit_list: quantum gate operate qubits list

    Returns:
        a Z1 gate node
    Raises:
        run_fail: An error occurred construct in gate node


    3. Z1(qubit_addr: int) -> pyQPanda.QGate

    Create a Z1 gate

    Args:
        qubit_addr: quantum gate operate qubits addr

    Returns:
        a Z1 gate node
    Raises:
        run_fail: An error occurred in construct gate node


    4. Z1(qubit_addr_list: List[int]) -> pyQPanda.QCircuit

    Create a Z1 gate

    Args:
        qubit_list_addr: quantum gate  qubits list addr

    Returns:
        a Z1 gate node
    Raises:
        run_fail: An error occurred in construct gate node

    """
@overload
def Z1(qubit_list: QVec) -> QCircuit:
    """Z1(*args, **kwargs)
    Overloaded function.

    1. Z1(qubit: pyQPanda.Qubit) -> pyQPanda.QGate

    Create a Z1 gate

    Args:
        qubit : quantum gate operate qubit

    Returns:
        a Z1 gate node
    Raises:
        run_fail: An error occurred in construct gate node


    2. Z1(qubit_list: pyQPanda.QVec) -> pyQPanda.QCircuit

    Create a Z1 gate

    Args:
        qubit_list: quantum gate operate qubits list

    Returns:
        a Z1 gate node
    Raises:
        run_fail: An error occurred construct in gate node


    3. Z1(qubit_addr: int) -> pyQPanda.QGate

    Create a Z1 gate

    Args:
        qubit_addr: quantum gate operate qubits addr

    Returns:
        a Z1 gate node
    Raises:
        run_fail: An error occurred in construct gate node


    4. Z1(qubit_addr_list: List[int]) -> pyQPanda.QCircuit

    Create a Z1 gate

    Args:
        qubit_list_addr: quantum gate  qubits list addr

    Returns:
        a Z1 gate node
    Raises:
        run_fail: An error occurred in construct gate node

    """
@overload
def Z1(qubit_addr: int) -> QGate:
    """Z1(*args, **kwargs)
    Overloaded function.

    1. Z1(qubit: pyQPanda.Qubit) -> pyQPanda.QGate

    Create a Z1 gate

    Args:
        qubit : quantum gate operate qubit

    Returns:
        a Z1 gate node
    Raises:
        run_fail: An error occurred in construct gate node


    2. Z1(qubit_list: pyQPanda.QVec) -> pyQPanda.QCircuit

    Create a Z1 gate

    Args:
        qubit_list: quantum gate operate qubits list

    Returns:
        a Z1 gate node
    Raises:
        run_fail: An error occurred construct in gate node


    3. Z1(qubit_addr: int) -> pyQPanda.QGate

    Create a Z1 gate

    Args:
        qubit_addr: quantum gate operate qubits addr

    Returns:
        a Z1 gate node
    Raises:
        run_fail: An error occurred in construct gate node


    4. Z1(qubit_addr_list: List[int]) -> pyQPanda.QCircuit

    Create a Z1 gate

    Args:
        qubit_list_addr: quantum gate  qubits list addr

    Returns:
        a Z1 gate node
    Raises:
        run_fail: An error occurred in construct gate node

    """
@overload
def Z1(qubit_addr_list: list[int]) -> QCircuit:
    """Z1(*args, **kwargs)
    Overloaded function.

    1. Z1(qubit: pyQPanda.Qubit) -> pyQPanda.QGate

    Create a Z1 gate

    Args:
        qubit : quantum gate operate qubit

    Returns:
        a Z1 gate node
    Raises:
        run_fail: An error occurred in construct gate node


    2. Z1(qubit_list: pyQPanda.QVec) -> pyQPanda.QCircuit

    Create a Z1 gate

    Args:
        qubit_list: quantum gate operate qubits list

    Returns:
        a Z1 gate node
    Raises:
        run_fail: An error occurred construct in gate node


    3. Z1(qubit_addr: int) -> pyQPanda.QGate

    Create a Z1 gate

    Args:
        qubit_addr: quantum gate operate qubits addr

    Returns:
        a Z1 gate node
    Raises:
        run_fail: An error occurred in construct gate node


    4. Z1(qubit_addr_list: List[int]) -> pyQPanda.QCircuit

    Create a Z1 gate

    Args:
        qubit_list_addr: quantum gate  qubits list addr

    Returns:
        a Z1 gate node
    Raises:
        run_fail: An error occurred in construct gate node

    """
def accumulateProbability(probability_list: list[float]) -> list[float]:
    """accumulateProbability(probability_list: List[float]) -> List[float]

    Accumulate the probability from a probability list.

    Args:
        probability_list: measured result in probability list form.

    Returns:
        accumulated_result: accumulated result.

    """
def accumulate_probabilities(probability_list: list[float]) -> list[float]:
    """accumulate_probabilities(probability_list: List[float]) -> List[float]

    Accumulate the probability from a probability list.

    Args:
        probability_list: measured result in probability list form.

    Returns:
        accumulated_result: accumulated result.

    """
def accumulate_probability(probability_list: list[float]) -> list[float]:
    """accumulate_probability(probability_list: List[float]) -> List[float]

    Accumulate the probability from a probability list.

    Args:
        probability_list: measured result in probability list form.

    Returns:
        accumulated_result: accumulated result.

    """
def acos(arg0: var) -> var:
    """acos(arg0: pyQPanda.var) -> pyQPanda.var"""
@overload
def add(arg0: ClassicalCondition, arg1: ClassicalCondition) -> ClassicalCondition:
    """add(*args, **kwargs)
    Overloaded function.

    1. add(arg0: pyQPanda.ClassicalCondition, arg1: pyQPanda.ClassicalCondition) -> pyQPanda.ClassicalCondition

    Add two ClassicalCondition objects.

    Args:
        a: The first ClassicalCondition.

        b: The second ClassicalCondition.

    Returns:
        ClassicalCondition: The sum of the two conditions.


    2. add(arg0: pyQPanda.ClassicalCondition, arg1: int) -> pyQPanda.ClassicalCondition

    Add a ClassicalCondition and a bit size.

    Args:
        a: The ClassicalCondition to which the bit size will be added.

        b: The bit size to be added to the ClassicalCondition.

    Returns:
        ClassicalCondition: The resulting ClassicalCondition after addition.


    3. add(arg0: int, arg1: pyQPanda.ClassicalCondition) -> pyQPanda.ClassicalCondition

    Add a bit size and a ClassicalCondition.

    Args:
        a: The bit size to be added.

        b: The ClassicalCondition to which the bit size will be added.

    Returns:
        ClassicalCondition: The resulting ClassicalCondition after addition.

    """
@overload
def add(arg0: ClassicalCondition, arg1: int) -> ClassicalCondition:
    """add(*args, **kwargs)
    Overloaded function.

    1. add(arg0: pyQPanda.ClassicalCondition, arg1: pyQPanda.ClassicalCondition) -> pyQPanda.ClassicalCondition

    Add two ClassicalCondition objects.

    Args:
        a: The first ClassicalCondition.

        b: The second ClassicalCondition.

    Returns:
        ClassicalCondition: The sum of the two conditions.


    2. add(arg0: pyQPanda.ClassicalCondition, arg1: int) -> pyQPanda.ClassicalCondition

    Add a ClassicalCondition and a bit size.

    Args:
        a: The ClassicalCondition to which the bit size will be added.

        b: The bit size to be added to the ClassicalCondition.

    Returns:
        ClassicalCondition: The resulting ClassicalCondition after addition.


    3. add(arg0: int, arg1: pyQPanda.ClassicalCondition) -> pyQPanda.ClassicalCondition

    Add a bit size and a ClassicalCondition.

    Args:
        a: The bit size to be added.

        b: The ClassicalCondition to which the bit size will be added.

    Returns:
        ClassicalCondition: The resulting ClassicalCondition after addition.

    """
@overload
def add(arg0: int, arg1: ClassicalCondition) -> ClassicalCondition:
    """add(*args, **kwargs)
    Overloaded function.

    1. add(arg0: pyQPanda.ClassicalCondition, arg1: pyQPanda.ClassicalCondition) -> pyQPanda.ClassicalCondition

    Add two ClassicalCondition objects.

    Args:
        a: The first ClassicalCondition.

        b: The second ClassicalCondition.

    Returns:
        ClassicalCondition: The sum of the two conditions.


    2. add(arg0: pyQPanda.ClassicalCondition, arg1: int) -> pyQPanda.ClassicalCondition

    Add a ClassicalCondition and a bit size.

    Args:
        a: The ClassicalCondition to which the bit size will be added.

        b: The bit size to be added to the ClassicalCondition.

    Returns:
        ClassicalCondition: The resulting ClassicalCondition after addition.


    3. add(arg0: int, arg1: pyQPanda.ClassicalCondition) -> pyQPanda.ClassicalCondition

    Add a bit size and a ClassicalCondition.

    Args:
        a: The bit size to be added.

        b: The ClassicalCondition to which the bit size will be added.

    Returns:
        ClassicalCondition: The resulting ClassicalCondition after addition.

    """
def all_cut_of_graph(adjacent_matrix: list[list[float]], all_cut_list: list[float], target_value_list: list[int]) -> float:
    """all_cut_of_graph(adjacent_matrix: List[List[float]], all_cut_list: List[float], target_value_list: List[int]) -> float

    Generate a graph representation for the max cut problem.

    Args:
        adjacent_matrix: The adjacency matrix for the quantum program.

        all_cut_list: A list of all cut graphs in the quantum program.

        target_value_list: A list of target cut values.

    Returns:
        max value: The maximum value found from the cuts.

    """
@overload
def amplitude_encode(qubit: QVec, data: list[float], b_need_check_normalization: bool = ...) -> QCircuit:
    """amplitude_encode(*args, **kwargs)
    Overloaded function.

    1. amplitude_encode(qubit: pyQPanda.QVec, data: List[float], b_need_check_normalization: bool = True) -> pyQPanda.QCircuit

    Encode the input double data to the amplitude of qubits

    Args:
        qubit: quantum program qubits
        data: double data list
        b_need_check_normalization: is need to check normalization

    Returns:
        result circuit 
    Raises:
        run_fail: An error occurred in amplitude_encode


    2. amplitude_encode(qubit: pyQPanda.QVec, data: List[complex]) -> pyQPanda.QCircuit

    Encode the input double data to the amplitude of qubits

    Args:
        qubit: quantum program qubits
        data: double data list

    Returns:
        result circuit 
    Raises:
        run_fail: An error occurred in amplitude_encode

    """
@overload
def amplitude_encode(qubit: QVec, data: list[complex]) -> QCircuit:
    """amplitude_encode(*args, **kwargs)
    Overloaded function.

    1. amplitude_encode(qubit: pyQPanda.QVec, data: List[float], b_need_check_normalization: bool = True) -> pyQPanda.QCircuit

    Encode the input double data to the amplitude of qubits

    Args:
        qubit: quantum program qubits
        data: double data list
        b_need_check_normalization: is need to check normalization

    Returns:
        result circuit 
    Raises:
        run_fail: An error occurred in amplitude_encode


    2. amplitude_encode(qubit: pyQPanda.QVec, data: List[complex]) -> pyQPanda.QCircuit

    Encode the input double data to the amplitude of qubits

    Args:
        qubit: quantum program qubits
        data: double data list

    Returns:
        result circuit 
    Raises:
        run_fail: An error occurred in amplitude_encode

    """
@overload
def apply_QGate(qubit_list: QVec, func_obj: Callable[[Qubit], QGate]) -> QCircuit:
    """apply_QGate(*args, **kwargs)
    Overloaded function.

    1. apply_QGate(qubit_list: pyQPanda.QVec, func_obj: Callable[[pyQPanda.Qubit], pyQPanda.QGate]) -> pyQPanda.QCircuit

    Apply a quantum gate operation to a list of qubits.


    Args:
        qubit_list: List of qubits to which the gate will be applied.

        func_obj: A function object that takes a Qubit and returns a QGate.


    Returns:
        QCircuit: The resulting circuit containing the QGate operations on all qubits.


    2. apply_QGate(qubit_addr_list: List[int], func_obj: Callable[[int], pyQPanda.QGate]) -> pyQPanda.QCircuit

    Apply a quantum gate operation to a list of qubit addresses.


    Args:
        qubit_addr_list: List of qubit addresses to which the gate will be applied.

        func_obj: A function object that takes a qubit address (int) and returns a QGate.


    Returns:
        QCircuit: The resulting circuit containing the QGate operations on all qubits.

    """
@overload
def apply_QGate(qubit_addr_list: list[int], func_obj: Callable[[int], QGate]) -> QCircuit:
    """apply_QGate(*args, **kwargs)
    Overloaded function.

    1. apply_QGate(qubit_list: pyQPanda.QVec, func_obj: Callable[[pyQPanda.Qubit], pyQPanda.QGate]) -> pyQPanda.QCircuit

    Apply a quantum gate operation to a list of qubits.


    Args:
        qubit_list: List of qubits to which the gate will be applied.

        func_obj: A function object that takes a Qubit and returns a QGate.


    Returns:
        QCircuit: The resulting circuit containing the QGate operations on all qubits.


    2. apply_QGate(qubit_addr_list: List[int], func_obj: Callable[[int], pyQPanda.QGate]) -> pyQPanda.QCircuit

    Apply a quantum gate operation to a list of qubit addresses.


    Args:
        qubit_addr_list: List of qubit addresses to which the gate will be applied.

        func_obj: A function object that takes a qubit address (int) and returns a QGate.


    Returns:
        QCircuit: The resulting circuit containing the QGate operations on all qubits.

    """
def asin(arg0: var) -> var:
    """asin(arg0: pyQPanda.var) -> pyQPanda.var"""
@overload
def assign(arg0: ClassicalCondition, arg1: ClassicalCondition) -> ClassicalCondition:
    """assign(*args, **kwargs)
    Overloaded function.

    1. assign(arg0: pyQPanda.ClassicalCondition, arg1: pyQPanda.ClassicalCondition) -> pyQPanda.ClassicalCondition

    Assign the value of one ClassicalCondition to another.

    Args:
        a: The ClassicalCondition to be assigned to (passed by reference).

        b: The ClassicalCondition to assign from.

    Returns:
        ClassicalCondition: The updated value of the first ClassicalCondition.


    2. assign(arg0: pyQPanda.ClassicalCondition, arg1: int) -> pyQPanda.ClassicalCondition

    Assign a bit size value to a ClassicalCondition.

    Args:
        a: The ClassicalCondition to be updated (passed by reference).

        b: The bit size value to assign.

    Returns:
        ClassicalCondition: The updated ClassicalCondition after assignment.

    """
@overload
def assign(arg0: ClassicalCondition, arg1: int) -> ClassicalCondition:
    """assign(*args, **kwargs)
    Overloaded function.

    1. assign(arg0: pyQPanda.ClassicalCondition, arg1: pyQPanda.ClassicalCondition) -> pyQPanda.ClassicalCondition

    Assign the value of one ClassicalCondition to another.

    Args:
        a: The ClassicalCondition to be assigned to (passed by reference).

        b: The ClassicalCondition to assign from.

    Returns:
        ClassicalCondition: The updated value of the first ClassicalCondition.


    2. assign(arg0: pyQPanda.ClassicalCondition, arg1: int) -> pyQPanda.ClassicalCondition

    Assign a bit size value to a ClassicalCondition.

    Args:
        a: The ClassicalCondition to be updated (passed by reference).

        b: The bit size value to assign.

    Returns:
        ClassicalCondition: The updated ClassicalCondition after assignment.

    """
def atan(arg0: var) -> var:
    """atan(arg0: pyQPanda.var) -> pyQPanda.var"""
@overload
def average_gate_fidelity(state1: numpy.ndarray[numpy.complex128[m, n]], state2: list[complex]) -> float:
    """average_gate_fidelity(*args, **kwargs)
    Overloaded function.

    1. average_gate_fidelity(state1: numpy.ndarray[numpy.complex128[m, n]], state2: List[complex]) -> float

    Calculate the average gate fidelity between a quantum operation and a quantum state.

    Args:
        matrix: a quantum operation represented as a matrix.

        state: a single quantum state represented as a list.

    Returns:
        The average gate fidelity, a value in the range [0, 1].


    2. average_gate_fidelity(state1: numpy.ndarray[numpy.complex128[m, n]], state2: numpy.ndarray[numpy.complex128[m, n]]) -> float

    Calculate the average gate fidelity between two quantum operation matrices.

    Args:
        matrix1: the first quantum operation represented as a matrix.

        matrix2: the second quantum operation represented as a matrix.

    Returns:
        The average gate fidelity, a value in the range [0, 1].

    """
@overload
def average_gate_fidelity(state1: numpy.ndarray[numpy.complex128[m, n]], state2: numpy.ndarray[numpy.complex128[m, n]]) -> float:
    """average_gate_fidelity(*args, **kwargs)
    Overloaded function.

    1. average_gate_fidelity(state1: numpy.ndarray[numpy.complex128[m, n]], state2: List[complex]) -> float

    Calculate the average gate fidelity between a quantum operation and a quantum state.

    Args:
        matrix: a quantum operation represented as a matrix.

        state: a single quantum state represented as a list.

    Returns:
        The average gate fidelity, a value in the range [0, 1].


    2. average_gate_fidelity(state1: numpy.ndarray[numpy.complex128[m, n]], state2: numpy.ndarray[numpy.complex128[m, n]]) -> float

    Calculate the average gate fidelity between two quantum operation matrices.

    Args:
        matrix1: the first quantum operation represented as a matrix.

        matrix2: the second quantum operation represented as a matrix.

    Returns:
        The average gate fidelity, a value in the range [0, 1].

    """
def bin_to_prog(bin_data: list[int], qubit_list: QVec, cbit_list: list[ClassicalCondition], qprog: QProg) -> bool:
    """bin_to_prog(bin_data: List[int], qubit_list: pyQPanda.QVec, cbit_list: List[pyQPanda.ClassicalCondition], qprog: pyQPanda.QProg) -> bool

    Parse binary data to transform into a quantum program.

    Args:
        bin_data: binary data that stores quantum program information.

        qubit_list: list of quantum qubits.

        cbit_list: list of classical bits.

        qprog: quantum program.

    Returns:
        prog: the parsed quantum program.

    """
def bind_data(arg0: int, arg1: QVec) -> QCircuit:
    """bind_data(arg0: int, arg1: pyQPanda.QVec) -> pyQPanda.QCircuit


    Args:
        int: classical data
        QVec: qubits list

    Returns:
        result : circuit 
    Raises:
        run_fail: An error occurred in bind_data

    """
def bind_nonnegative_data(arg0: int, arg1: QVec) -> QCircuit:
    """bind_nonnegative_data(arg0: int, arg1: pyQPanda.QVec) -> pyQPanda.QCircuit


    Args:
        int: classical data
        QVec: qubits list

    Returns:
        result : circuit 
    Raises:
        run_fail: An error occurred in bind_nonnegative_data

    """
def build_HHL_circuit(matrix_A: list[complex], data_b: list[float], qvm: QuantumMachine, precision_cnt: int = ...) -> QCircuit:
    """build_HHL_circuit(matrix_A: List[complex], data_b: List[float], qvm: pyQPanda.QuantumMachine, precision_cnt: int = 0) -> pyQPanda.QCircuit

    build the quantum circuit for HHL algorithm to solve the target linear systems of equations : Ax = b

    Args:
        matrix_A: a unitary matrix or Hermitian N*N matrix with N = 2 ^ n

        data_b: a given vector

        qvm: quantum machine

        precision_cnt: The count of digits after the decimal point,
                       default is 0, indicates that there are only integer solutions

    Returns:
        QCircuit The whole quantum circuit for HHL algorithm

    Notes:
        The higher the precision is, the more qubit number and circuit - depth will be,
        for example: 1 - bit precision, 4 additional qubits are required,
        for 2 - bit precision, we need 7 additional qubits, and so on.
        The final solution = (HHL result) * (normalization factor for b) * (1 << ceil(log2(pow(10, precision_cnt))))
    """
@overload
def cAlloc() -> ClassicalCondition:
    """cAlloc(*args, **kwargs)
    Overloaded function.

    1. cAlloc() -> pyQPanda.ClassicalCondition

    Allocate a CBit
    After init()

    Args:
        none

    Returns:
        classic result cbit


    2. cAlloc(cbit_addr: int) -> pyQPanda.ClassicalCondition

    Allocate a CBit
    After init()

    Args:
        cbit_addr: cbit address, should be in [0,29).

    Returns:
        classic result cbit

    """
@overload
def cAlloc(cbit_addr: int) -> ClassicalCondition:
    """cAlloc(*args, **kwargs)
    Overloaded function.

    1. cAlloc() -> pyQPanda.ClassicalCondition

    Allocate a CBit
    After init()

    Args:
        none

    Returns:
        classic result cbit


    2. cAlloc(cbit_addr: int) -> pyQPanda.ClassicalCondition

    Allocate a CBit
    After init()

    Args:
        cbit_addr: cbit address, should be in [0,29).

    Returns:
        classic result cbit

    """
def cAlloc_many(cbit_num: int) -> list[ClassicalCondition]:
    """cAlloc_many(cbit_num: int) -> List[pyQPanda.ClassicalCondition]

    Allocate several CBits
    After init()

    Args:
        cbit_num: numbers of cbit want to be created.

    Returns:
        list of cbit

    """
def cFree(cbit: ClassicalCondition) -> None:
    """cFree(cbit: pyQPanda.ClassicalCondition) -> None

    Free a CBit

    Args:
        CBit: The CBit to be freed.

    Returns:
        none

    """
@overload
def cFree_all() -> None:
    """cFree_all(*args, **kwargs)
    Overloaded function.

    1. cFree_all() -> None

    Free all CBits

    Args:
        none

    Returns:
        none


    2. cFree_all(cbit_list: List[pyQPanda.ClassicalCondition]) -> None

    Free all CBits

    Args:
        cbit_list: a list of cbits.

    Returns:
        none

    """
@overload
def cFree_all(cbit_list: list[ClassicalCondition]) -> None:
    """cFree_all(*args, **kwargs)
    Overloaded function.

    1. cFree_all() -> None

    Free all CBits

    Args:
        none

    Returns:
        none


    2. cFree_all(cbit_list: List[pyQPanda.ClassicalCondition]) -> None

    Free all CBits

    Args:
        cbit_list: a list of cbits.

    Returns:
        none

    """
@overload
def calculate_quantum_volume(noise_qvm: NoiseQVM, qubit_list: list[list[int]], ntrials: int, shots: int = ...) -> int:
    """calculate_quantum_volume(*args, **kwargs)
    Overloaded function.

    1. calculate_quantum_volume(noise_qvm: pyQPanda.NoiseQVM, qubit_list: List[List[int]], ntrials: int, shots: int = 1000) -> int

    calculate quantum volume

    Args:
        noise_qvm: noise quantum machine
        qubit_list: qubit list 
        ntrials: ntrials
        shots: measure shots

    Returns:
        result data dict
    Raises:
        run_fail: An error occurred in calculate_quantum_volume


    2. calculate_quantum_volume(cloud_qvm: QPanda::QCloudMachine, qubit_list: List[List[int]], ntrials: int, shots: int = 1000) -> int

    calculate quantum volume

    Args:
        noise_qvm: noise quantum machine
        qubit_list: qubit list 
        ntrials: ntrials
        shots: measure shots

    Returns:
        result data dict
    Raises:
        run_fail: An error occurred in calculate_quantum_volume


    3. calculate_quantum_volume(config: pyQPanda.QCloudTaskConfig, qubit_list: List[List[int]], ntrials: int) -> int

    calculate quantum volume

    Args:
        config: QCloudTaskConfig
        qubit_list: qubit list 
        ntrials: ntrials

    Returns:
        result data dict
    Raises:
        run_fail: An error occurred in calculate_quantum_volume

    """
@overload
def calculate_quantum_volume(cloud_qvm, qubit_list: list[list[int]], ntrials: int, shots: int = ...) -> int:
    """calculate_quantum_volume(*args, **kwargs)
    Overloaded function.

    1. calculate_quantum_volume(noise_qvm: pyQPanda.NoiseQVM, qubit_list: List[List[int]], ntrials: int, shots: int = 1000) -> int

    calculate quantum volume

    Args:
        noise_qvm: noise quantum machine
        qubit_list: qubit list 
        ntrials: ntrials
        shots: measure shots

    Returns:
        result data dict
    Raises:
        run_fail: An error occurred in calculate_quantum_volume


    2. calculate_quantum_volume(cloud_qvm: QPanda::QCloudMachine, qubit_list: List[List[int]], ntrials: int, shots: int = 1000) -> int

    calculate quantum volume

    Args:
        noise_qvm: noise quantum machine
        qubit_list: qubit list 
        ntrials: ntrials
        shots: measure shots

    Returns:
        result data dict
    Raises:
        run_fail: An error occurred in calculate_quantum_volume


    3. calculate_quantum_volume(config: pyQPanda.QCloudTaskConfig, qubit_list: List[List[int]], ntrials: int) -> int

    calculate quantum volume

    Args:
        config: QCloudTaskConfig
        qubit_list: qubit list 
        ntrials: ntrials

    Returns:
        result data dict
    Raises:
        run_fail: An error occurred in calculate_quantum_volume

    """
@overload
def calculate_quantum_volume(config: QCloudTaskConfig, qubit_list: list[list[int]], ntrials: int) -> int:
    """calculate_quantum_volume(*args, **kwargs)
    Overloaded function.

    1. calculate_quantum_volume(noise_qvm: pyQPanda.NoiseQVM, qubit_list: List[List[int]], ntrials: int, shots: int = 1000) -> int

    calculate quantum volume

    Args:
        noise_qvm: noise quantum machine
        qubit_list: qubit list 
        ntrials: ntrials
        shots: measure shots

    Returns:
        result data dict
    Raises:
        run_fail: An error occurred in calculate_quantum_volume


    2. calculate_quantum_volume(cloud_qvm: QPanda::QCloudMachine, qubit_list: List[List[int]], ntrials: int, shots: int = 1000) -> int

    calculate quantum volume

    Args:
        noise_qvm: noise quantum machine
        qubit_list: qubit list 
        ntrials: ntrials
        shots: measure shots

    Returns:
        result data dict
    Raises:
        run_fail: An error occurred in calculate_quantum_volume


    3. calculate_quantum_volume(config: pyQPanda.QCloudTaskConfig, qubit_list: List[List[int]], ntrials: int) -> int

    calculate quantum volume

    Args:
        config: QCloudTaskConfig
        qubit_list: qubit list 
        ntrials: ntrials

    Returns:
        result data dict
    Raises:
        run_fail: An error occurred in calculate_quantum_volume

    """
def cast_qprog_qcircuit(qprog: QProg) -> QCircuit:
    """cast_qprog_qcircuit(qprog: pyQPanda.QProg) -> pyQPanda.QCircuit

    Cast a quantum program into a quantum circuit.

    Args:
        qprog: The quantum program to be cast.

    Returns:
        QCircuit: The resulting quantum circuit.

    """
def cast_qprog_qgate(qprog: QProg) -> QGate:
    """cast_qprog_qgate(qprog: pyQPanda.QProg) -> pyQPanda.QGate

    Cast a quantum program into a quantum gate.

    Args:
        qprog: The quantum program to be cast.

    Returns:
        None: This function does not return a value.

    """
def cast_qprog_qmeasure(qprog: QProg) -> QMeasure:
    """cast_qprog_qmeasure(qprog: pyQPanda.QProg) -> pyQPanda.QMeasure

    Cast a quantum program into a quantum measurement.

    Args:
        qprog: The quantum program to be cast.

    Returns:
        None: This function does not return a value.

    """
def circuit_layer(qprog: QProg) -> list:
    """circuit_layer(qprog: pyQPanda.QProg) -> list

    Quantum circuit layering.

    Args:
        QProg: Quantum program.

    Returns:
        A list containing layer information and qubits/cbits in use.

    """
def circuit_optimizer(qprog: QProg, optimizer_cir_vec: list[tuple[QCircuit, QCircuit]] = ..., mode_list: list[QCircuitOPtimizerMode] = ...) -> QProg:
    """circuit_optimizer(qprog: pyQPanda.QProg, optimizer_cir_vec: List[Tuple[pyQPanda.QCircuit, pyQPanda.QCircuit]] = [], mode_list: List[pyQPanda.QCircuitOPtimizerMode] = []) -> pyQPanda.QProg

    Optimize a quantum circuit.

    Args:
        qprog: the quantum program to optimize.

        optimizer_cir_vec: a list of quantum circuits for optimization.

        mode_list: a list of optimization modes.

    Returns:
        the updated quantum program after optimization.

    """
def circuit_optimizer_by_config(qprog: QProg, config_file: str = ..., mode_list: list[QCircuitOPtimizerMode] = ...) -> QProg:
    """circuit_optimizer_by_config(qprog: pyQPanda.QProg, config_file: str = 'QPandaConfig.json', mode_list: List[pyQPanda.QCircuitOPtimizerMode] = []) -> pyQPanda.QProg

    Optimize a quantum circuit using configuration data.

    Args:
        qprog: the quantum program to optimize.

        config_file: configuration data for optimization.

        mode_list: a list of optimization modes.

    Returns:
        the updated quantum program after optimization.

    """
def comm_protocol_decode(encode_data: bytes, machine: QuantumMachine) -> tuple[list[QProg], CommProtocolConfig]:
    """comm_protocol_decode(encode_data: bytes, machine: pyQPanda.QuantumMachine) -> Tuple[List[pyQPanda.QProg], pyQPanda.CommProtocolConfig]

    Decode binary data into a list of quantum programs using the communication protocol.

    Args:
        encode_data: The encoded binary data representing quantum programs.

        machine: A pointer to the QuantumMachine used for decoding.

    Returns:
        tuple: A tuple containing the decoded program list and the communication protocol configuration.

    """
@overload
def comm_protocol_encode(prog: QProg, config: CommProtocolConfig = ...) -> bytes:
    """comm_protocol_encode(*args, **kwargs)
    Overloaded function.

    1. comm_protocol_encode(prog: pyQPanda.QProg, config: pyQPanda.CommProtocolConfig = <pyQPanda.CommProtocolConfig object at 0x00000285683432B0>) -> bytes

    Encode the communication protocol data into binary format.

    Args:
        prog: The quantum program to be encoded.

        config: The configuration for the communication protocol. Defaults to an empty configuration.

    Returns:
        bytes: The encoded binary data representing the communication protocol.


    2. comm_protocol_encode(prog_list: List[pyQPanda.QProg], config: pyQPanda.CommProtocolConfig = <pyQPanda.CommProtocolConfig object at 0x00000285682950B0>) -> bytes

    Encode a list of quantum programs into binary communication protocol data.

    Args:
        prog_list: A list of quantum programs to be encoded.

        config: The configuration for the communication protocol. Defaults to an empty configuration.

    Returns:
        bytes: The encoded binary data representing the communication protocol.

    """
@overload
def comm_protocol_encode(prog_list: list[QProg], config: CommProtocolConfig = ...) -> bytes:
    """comm_protocol_encode(*args, **kwargs)
    Overloaded function.

    1. comm_protocol_encode(prog: pyQPanda.QProg, config: pyQPanda.CommProtocolConfig = <pyQPanda.CommProtocolConfig object at 0x00000285683432B0>) -> bytes

    Encode the communication protocol data into binary format.

    Args:
        prog: The quantum program to be encoded.

        config: The configuration for the communication protocol. Defaults to an empty configuration.

    Returns:
        bytes: The encoded binary data representing the communication protocol.


    2. comm_protocol_encode(prog_list: List[pyQPanda.QProg], config: pyQPanda.CommProtocolConfig = <pyQPanda.CommProtocolConfig object at 0x00000285682950B0>) -> bytes

    Encode a list of quantum programs into binary communication protocol data.

    Args:
        prog_list: A list of quantum programs to be encoded.

        config: The configuration for the communication protocol. Defaults to an empty configuration.

    Returns:
        bytes: The encoded binary data representing the communication protocol.

    """
def constModAdd(arg0: QVec, arg1: int, arg2: int, arg3: QVec, arg4: QVec) -> QCircuit:
    """constModAdd(arg0: pyQPanda.QVec, arg1: int, arg2: int, arg3: pyQPanda.QVec, arg4: pyQPanda.QVec) -> pyQPanda.QCircuit


    Args:
       QVec qvec
       int base
       int module_Num
       QVec qvec1
       QVec qvec2

    Returns:
        result circuit 
    Raises:
        run_fail: An error occurred in constModAdd

    """
def constModExp(arg0: QVec, arg1: QVec, arg2: int, arg3: int, arg4: QVec, arg5: QVec, arg6: QVec) -> QCircuit:
    """constModExp(arg0: pyQPanda.QVec, arg1: pyQPanda.QVec, arg2: int, arg3: int, arg4: pyQPanda.QVec, arg5: pyQPanda.QVec, arg6: pyQPanda.QVec) -> pyQPanda.QCircuit


    Args:
       QVec qvec
       int base
       int module_Num
       QVec qvec1
       QVec qvec2

    Returns:
        result circuit 
    Raises:
        run_fail: An error occurred in constModExp

    """
def constModMul(arg0: QVec, arg1: int, arg2: int, arg3: QVec, arg4: QVec, arg5: QVec) -> QCircuit:
    """constModMul(arg0: pyQPanda.QVec, arg1: int, arg2: int, arg3: pyQPanda.QVec, arg4: pyQPanda.QVec, arg5: pyQPanda.QVec) -> pyQPanda.QCircuit


    Args:
       QVec qvec
       int base
       int module_Num
       QVec qvec1
       QVec qvec2

    Returns:
        result circuit 
    Raises:
        run_fail: An error occurred in constModMul

    """
def convert_binary_data_to_qprog(machine: QuantumMachine, data: list[int]) -> QProg:
    """convert_binary_data_to_qprog(machine: pyQPanda.QuantumMachine, data: List[int]) -> pyQPanda.QProg

    Parse binary data into a quantum program.

    Args:
        machine: The quantum machine used for execution.

        data: The binary data representing the quantum program.

    Returns:
        QProg: The generated quantum program.

    """
def convert_originir_str_to_qprog(originir_str: str, machine: QuantumMachine) -> list:
    """convert_originir_str_to_qprog(originir_str: str, machine: pyQPanda.QuantumMachine) -> list

    Transform OriginIR string into QProg.

    Args:
        originir_str: OriginIR string.

        machine: initialized quantum machine.

    Returns:
        A list containing QProg, qubit_list, and cbit_list.

    """
def convert_originir_to_qprog(file_path: str, machine: QuantumMachine) -> list:
    """convert_originir_to_qprog(file_path: str, machine: pyQPanda.QuantumMachine) -> list

    Read an OriginIR file and transform it into QProg.

    Args:
        file_path: OriginIR file path.

        machine: initialized quantum machine.

    Returns:
        A list containing QProg, qubit_list, and cbit_list.

    """
def convert_pyquil_file_to_originir(file_path: str) -> str:
    """convert_pyquil_file_to_originir(file_path: str) -> str

    Read pyquil file and trans to OriginIR string

    Args:
        file_path: pyquil file path

    Returns:
        str originir_strRaises:
        run_fail: An error occurred in convert_pyquil_to_qprog

    """
def convert_pyquil_file_to_qprog(file_path: str, machine: QuantumMachine) -> list:
    """convert_pyquil_file_to_qprog(file_path: str, machine: pyQPanda.QuantumMachine) -> list

    Read pyquil file and trans to QProg

    Args:
        file_path: pyquil file path
        machine: initialized quantum machine

    Returns:
        list cotains QProg, qubit_list, cbit_listRaises:
        run_fail: An error occurred in convert_pyquil_to_qprog

    """
def convert_pyquil_string_to_originir(pyquil_str: str) -> str:
    """convert_pyquil_string_to_originir(pyquil_str: str) -> str

    Trans pyquil to OriginIR string

    Args:
        pyquil_str: pyquil string

    Returns:
        str originir_str
    """
def convert_pyquil_string_to_qprog(pyquil_str: str, machine: QuantumMachine) -> list:
    """convert_pyquil_string_to_qprog(pyquil_str: str, machine: pyQPanda.QuantumMachine) -> list

    Trans pyquil to QProg

    Args:
        pyquil_str: pyquil string
        machine: initialized quantum machine

    Returns:
        list cotains QProg, qubit_list, cbit_list
    """
def convert_qasm_string_to_originir(qasm_str: str) -> str:
    """convert_qasm_string_to_originir(qasm_str: str) -> str

    Trans QASM to OriginIR string

    Args:
        qasm_str: QASM string

    Returns:
        str originir_str
    """
def convert_qasm_string_to_qprog(qasm_str: str, machine: QuantumMachine) -> list:
    """convert_qasm_string_to_qprog(qasm_str: str, machine: pyQPanda.QuantumMachine) -> list

    Trans QASM to QProg

    Args:
        qasm_str: QASM string
        machine: initialized quantum machine

    Returns:
        list cotains QProg, qubit_list, cbit_list
    """
def convert_qasm_to_originir(file_path: str) -> str:
    """convert_qasm_to_originir(file_path: str) -> str

    Read QASM file and trans to OriginIR string

    Args:
        file_path: QASM file path

    Returns:
        str originir_strRaises:
        run_fail: An error occurred in convert_qasm_to_originir

    """
def convert_qasm_to_qprog(file_path: str, machine: QuantumMachine) -> list:
    """convert_qasm_to_qprog(file_path: str, machine: pyQPanda.QuantumMachine) -> list

    Read QASM file and trans to QProg

    Args:
        file_path: QASM file path
        machine: initialized quantum machine

    Returns:
        list cotains QProg, qubit_list, cbit_listRaises:
        run_fail: An error occurred in convert_qasm_to_qprog

    """
@overload
def convert_qprog_to_binary(qprog: QProg, machine: QuantumMachine) -> list[int]:
    """convert_qprog_to_binary(*args, **kwargs)
    Overloaded function.

    1. convert_qprog_to_binary(qprog: pyQPanda.QProg, machine: pyQPanda.QuantumMachine) -> List[int]

    Convert a quantum program into binary data.

    Args:
        qprog: quantum program.

        machine: quantum machine.

    Returns:
        string: binary data representation of the quantum program.


    2. convert_qprog_to_binary(qprog: pyQPanda.QProg, machine: pyQPanda.QuantumMachine, fname: str) -> None

    Store the quantum program in a binary file.

    Args:
        qprog: quantum program.

        machine: quantum machine.

        fname: name of the binary data file.

    Returns:
        none: This function does not return a value.

    """
@overload
def convert_qprog_to_binary(qprog: QProg, machine: QuantumMachine, fname: str) -> None:
    """convert_qprog_to_binary(*args, **kwargs)
    Overloaded function.

    1. convert_qprog_to_binary(qprog: pyQPanda.QProg, machine: pyQPanda.QuantumMachine) -> List[int]

    Convert a quantum program into binary data.

    Args:
        qprog: quantum program.

        machine: quantum machine.

    Returns:
        string: binary data representation of the quantum program.


    2. convert_qprog_to_binary(qprog: pyQPanda.QProg, machine: pyQPanda.QuantumMachine, fname: str) -> None

    Store the quantum program in a binary file.

    Args:
        qprog: quantum program.

        machine: quantum machine.

        fname: name of the binary data file.

    Returns:
        none: This function does not return a value.

    """
def convert_qprog_to_originir(*args, **kwargs):
    """convert_qprog_to_originir(qprog: pyQPanda.QProg, machine: pyQPanda.QuantumMachine) -> str

    Convert QProg to OriginIR string.

    Args:
        qprog: quantum program (QProg&).

        machine: quantum machine (QuantumMachine*).

    Returns:
        originir: OriginIR string. For more information, see the OriginIR introduction:

      https://pyqpanda-toturial.readthedocs.io/zh/latest

    """
def convert_qprog_to_qasm(qprog: QProg, machine: QuantumMachine) -> str:
    """convert_qprog_to_qasm(qprog: pyQPanda.QProg, machine: pyQPanda.QuantumMachine) -> str

    Convert a quantum program to a QASM instruction string.

    Args:
        qprog: The quantum program to be converted.

        machine: The quantum machine used for execution.

    Returns:
        str: A QASM string representing the quantum program.

    """
def convert_qprog_to_quil(qprog: QProg, machine: QuantumMachine) -> str:
    """convert_qprog_to_quil(qprog: pyQPanda.QProg, machine: pyQPanda.QuantumMachine) -> str

    Convert QProg to Quil instruction.

    Args:
        qprog: quantum program (QProg).

        machine: quantum machine (QuantumMachine*).

    Returns:
        Quil instruction string.

    """
def cos(arg0: var) -> var:
    """cos(arg0: pyQPanda.var) -> pyQPanda.var"""
@overload
def count_gate(quantum_prog: QProg) -> int:
    """count_gate(*args, **kwargs)
    Overloaded function.

    1. count_gate(quantum_prog: pyQPanda.QProg) -> int

    Count quantum gate number in the quantum program.

    Args:
        quantum_prog: quantum program.

    Returns:
        result: gate count.


    2. count_gate(quantum_circuit: pyQPanda.QCircuit) -> int

    Count quantum gate number in the quantum circuit.

    Args:
        quantum_circuit: quantum circuit.

    Returns:
        result: gate count.

    """
@overload
def count_gate(quantum_circuit: QCircuit) -> int:
    """count_gate(*args, **kwargs)
    Overloaded function.

    1. count_gate(quantum_prog: pyQPanda.QProg) -> int

    Count quantum gate number in the quantum program.

    Args:
        quantum_prog: quantum program.

    Returns:
        result: gate count.


    2. count_gate(quantum_circuit: pyQPanda.QCircuit) -> int

    Count quantum gate number in the quantum circuit.

    Args:
        quantum_circuit: quantum circuit.

    Returns:
        result: gate count.

    """
@overload
def count_prog_info(node: QProg, selected_types: list[GateType] = ...) -> ProgCount:
    """count_prog_info(*args, **kwargs)
    Overloaded function.

    1. count_prog_info(node: pyQPanda.QProg, selected_types: List[pyQPanda.GateType] = []) -> pyQPanda.ProgCount

    Count quantum program information.

    Args:
        node: quantum program (QProg).

        selected_types: vector of selected GateType (default is empty).

    Returns:
        ProgCount struct.


    2. count_prog_info(node: pyQPanda.QCircuit, selected_types: List[pyQPanda.GateType] = []) -> pyQPanda.ProgCount

    Count quantum program information.

    Args:
        node: quantum circuit (QCircuit).

        selected_types: vector of selected GateType (default is empty).

    Returns:
        ProgCount struct.

    """
@overload
def count_prog_info(node: QCircuit, selected_types: list[GateType] = ...) -> ProgCount:
    """count_prog_info(*args, **kwargs)
    Overloaded function.

    1. count_prog_info(node: pyQPanda.QProg, selected_types: List[pyQPanda.GateType] = []) -> pyQPanda.ProgCount

    Count quantum program information.

    Args:
        node: quantum program (QProg).

        selected_types: vector of selected GateType (default is empty).

    Returns:
        ProgCount struct.


    2. count_prog_info(node: pyQPanda.QCircuit, selected_types: List[pyQPanda.GateType] = []) -> pyQPanda.ProgCount

    Count quantum program information.

    Args:
        node: quantum circuit (QCircuit).

        selected_types: vector of selected GateType (default is empty).

    Returns:
        ProgCount struct.

    """
@overload
def count_qgate_num(prog: QProg, gate_type: int = ...) -> int:
    """count_qgate_num(*args, **kwargs)
    Overloaded function.

    1. count_qgate_num(prog: pyQPanda.QProg, gate_type: int = -1) -> int

    Count quantum gate number in the quantum program.

    Args:
        prog: quantum program (QProg&).

        gate_type: type of gate to count (const GateType).

    Returns:
        result: number of quantum gates of the specified GateType.


    2. count_qgate_num(circuit: pyQPanda.QCircuit, gate_type: int = -1) -> int

    Count quantum gate number in the quantum circuit.

    Args:
        circuit: quantum circuit (QCircuit&).

        gate_type: type of gate to count (const GateType).

    Returns:
        result: number of quantum gates of the specified GateType.

    """
@overload
def count_qgate_num(circuit: QCircuit, gate_type: int = ...) -> int:
    """count_qgate_num(*args, **kwargs)
    Overloaded function.

    1. count_qgate_num(prog: pyQPanda.QProg, gate_type: int = -1) -> int

    Count quantum gate number in the quantum program.

    Args:
        prog: quantum program (QProg&).

        gate_type: type of gate to count (const GateType).

    Returns:
        result: number of quantum gates of the specified GateType.


    2. count_qgate_num(circuit: pyQPanda.QCircuit, gate_type: int = -1) -> int

    Count quantum gate number in the quantum circuit.

    Args:
        circuit: quantum circuit (QCircuit&).

        gate_type: type of gate to count (const GateType).

    Returns:
        result: number of quantum gates of the specified GateType.

    """
def create_empty_circuit() -> QCircuit:
    """create_empty_circuit() -> pyQPanda.QCircuit

    Create an empty QCircuit container.

    Args:
        none

    Returns:
        result: An empty QCircuit.

    """
def create_empty_qprog() -> QProg:
    """create_empty_qprog() -> pyQPanda.QProg

    Create an empty QProg container.

    Args:
        none.

    Returns:
        an empty QProg.

    """
@overload
def create_if_prog(classical_condition: ClassicalCondition, true_node: QProg) -> QIfProg:
    """create_if_prog(*args, **kwargs)
    Overloaded function.

    1. create_if_prog(classical_condition: pyQPanda.ClassicalCondition, true_node: pyQPanda.QProg) -> pyQPanda.QIfProg

    Create a classical quantum IfProg.

    Args:
        classical_condition: A quantum cbit representing the condition.

        true_node: A quantum IfProg node that defines the operation to execute if the condition is true.

    Returns:
        result: A classical quantum IfProg that executes based on the specified condition.


    2. create_if_prog(classical_condition: pyQPanda.ClassicalCondition, true_node: pyQPanda.QProg, false_node: pyQPanda.QProg) -> pyQPanda.QIfProg

    Create a classical quantum IfProg.

    Args:
        classical_condition: A quantum cbit representing the condition.

        true_node: A quantum IfProg node that defines the operation to execute if the condition is true.

        false_node: A quantum IfProg node that defines the operation to execute if the condition is false.

    Returns:
        result: A classical quantum IfProg that executes based on the specified condition.

    """
@overload
def create_if_prog(classical_condition: ClassicalCondition, true_node: QProg, false_node: QProg) -> QIfProg:
    """create_if_prog(*args, **kwargs)
    Overloaded function.

    1. create_if_prog(classical_condition: pyQPanda.ClassicalCondition, true_node: pyQPanda.QProg) -> pyQPanda.QIfProg

    Create a classical quantum IfProg.

    Args:
        classical_condition: A quantum cbit representing the condition.

        true_node: A quantum IfProg node that defines the operation to execute if the condition is true.

    Returns:
        result: A classical quantum IfProg that executes based on the specified condition.


    2. create_if_prog(classical_condition: pyQPanda.ClassicalCondition, true_node: pyQPanda.QProg, false_node: pyQPanda.QProg) -> pyQPanda.QIfProg

    Create a classical quantum IfProg.

    Args:
        classical_condition: A quantum cbit representing the condition.

        true_node: A quantum IfProg node that defines the operation to execute if the condition is true.

        false_node: A quantum IfProg node that defines the operation to execute if the condition is false.

    Returns:
        result: A classical quantum IfProg that executes based on the specified condition.

    """
def create_while_prog(classical_condition: ClassicalCondition, true_node: QProg) -> QWhileProg:
    """create_while_prog(classical_condition: pyQPanda.ClassicalCondition, true_node: pyQPanda.QProg) -> pyQPanda.QWhileProg

    Create a WhileProg.

    Args:
        classical_condition: A quantum cbit representing the condition.

        true_node: A quantum QWhile node that defines the operation to execute while the condition is true.

    Returns:
        result: A WhileProg that executes the specified operations based on the condition.

    """
def crossEntropy(arg0: var, arg1: var) -> var:
    """crossEntropy(arg0: pyQPanda.var, arg1: pyQPanda.var) -> pyQPanda.var"""
@overload
def decompose_multiple_control_qgate(qprog: QProg, machine: QuantumMachine, config_file: str = ...) -> QProg:
    """decompose_multiple_control_qgate(*args, **kwargs)
    Overloaded function.

    1. decompose_multiple_control_qgate(qprog: pyQPanda.QProg, machine: pyQPanda.QuantumMachine, config_file: str = 'QPandaConfig.json') -> pyQPanda.QProg

    Decompose a multiple control quantum gate.

    Args:
        qprog: the quantum program containing the gate to be decomposed.

        machine: the quantum machine used for decomposition.

        config_file: path to the configuration file (default is CONFIG_PATH).

    Returns:
        the updated quantum program after the decomposition.


    2. decompose_multiple_control_qgate(qprog: pyQPanda.QProg, machine: pyQPanda.QuantumMachine, convert_single_gates: List[str], convert_double_gates: List[str], b_transform_to_base_qgate: bool = True) -> pyQPanda.QProg

    Decompose multiple control QGate.

    Args:
        qprog: Quantum program.

        machine: Quantum machine.

        convert_single_gates: Sets of quantum single gates.

        convert_double_gates: Sets of quantum double gates.

        b_transform_to_base_qgate: Transform to base QGate sets.

    Returns:
        A new program after decomposition.

    """
@overload
def decompose_multiple_control_qgate(qprog: QProg, machine: QuantumMachine, convert_single_gates: list[str], convert_double_gates: list[str], b_transform_to_base_qgate: bool = ...) -> QProg:
    """decompose_multiple_control_qgate(*args, **kwargs)
    Overloaded function.

    1. decompose_multiple_control_qgate(qprog: pyQPanda.QProg, machine: pyQPanda.QuantumMachine, config_file: str = 'QPandaConfig.json') -> pyQPanda.QProg

    Decompose a multiple control quantum gate.

    Args:
        qprog: the quantum program containing the gate to be decomposed.

        machine: the quantum machine used for decomposition.

        config_file: path to the configuration file (default is CONFIG_PATH).

    Returns:
        the updated quantum program after the decomposition.


    2. decompose_multiple_control_qgate(qprog: pyQPanda.QProg, machine: pyQPanda.QuantumMachine, convert_single_gates: List[str], convert_double_gates: List[str], b_transform_to_base_qgate: bool = True) -> pyQPanda.QProg

    Decompose multiple control QGate.

    Args:
        qprog: Quantum program.

        machine: Quantum machine.

        convert_single_gates: Sets of quantum single gates.

        convert_double_gates: Sets of quantum double gates.

        b_transform_to_base_qgate: Transform to base QGate sets.

    Returns:
        A new program after decomposition.

    """
@overload
def deep_copy(node: QProg) -> QProg:
    """deep_copy(*args, **kwargs)
    Overloaded function.

    1. deep_copy(node: pyQPanda.QProg) -> pyQPanda.QProg

    Create a deep copy of the given quantum program node.

    Args:
        node: The quantum program node to copy.

    Returns:
        A deep copy of the quantum program node.


    2. deep_copy(node: pyQPanda.QCircuit) -> pyQPanda.QCircuit

    Create a deep copy of the given quantum program node.

    Args:
        node: The quantum program node to copy.

    Returns:
        A deep copy of the quantum program node.


    3. deep_copy(node: pyQPanda.QGate) -> pyQPanda.QGate

    Create a deep copy of the given quantum program node.

    Args:
        node: The quantum program node to copy.

    Returns:
        A deep copy of the quantum program node.


    4. deep_copy(node: pyQPanda.QMeasure) -> pyQPanda.QMeasure

    Create a deep copy of the given quantum program node.

    Args:
        node: The quantum program node to copy.

    Returns:
        A deep copy of the quantum program node.


    5. deep_copy(node: pyQPanda.ClassicalProg) -> pyQPanda.ClassicalProg

    Create a deep copy of the given quantum program node.

    Args:
        node: The quantum program node to copy.

    Returns:
        A deep copy of the quantum program node.


    6. deep_copy(node: pyQPanda.QIfProg) -> pyQPanda.QIfProg

    Create a deep copy of the given quantum program node.

    Args:
        node: The quantum program node to copy.

    Returns:
        A deep copy of the quantum program node.


    7. deep_copy(node: pyQPanda.QWhileProg) -> pyQPanda.QWhileProg

    Create a deep copy of the given quantum program node.

    Args:
        node: The quantum program node to copy.

    Returns:
        A deep copy of the quantum program node.

    """
@overload
def deep_copy(node: QCircuit) -> QCircuit:
    """deep_copy(*args, **kwargs)
    Overloaded function.

    1. deep_copy(node: pyQPanda.QProg) -> pyQPanda.QProg

    Create a deep copy of the given quantum program node.

    Args:
        node: The quantum program node to copy.

    Returns:
        A deep copy of the quantum program node.


    2. deep_copy(node: pyQPanda.QCircuit) -> pyQPanda.QCircuit

    Create a deep copy of the given quantum program node.

    Args:
        node: The quantum program node to copy.

    Returns:
        A deep copy of the quantum program node.


    3. deep_copy(node: pyQPanda.QGate) -> pyQPanda.QGate

    Create a deep copy of the given quantum program node.

    Args:
        node: The quantum program node to copy.

    Returns:
        A deep copy of the quantum program node.


    4. deep_copy(node: pyQPanda.QMeasure) -> pyQPanda.QMeasure

    Create a deep copy of the given quantum program node.

    Args:
        node: The quantum program node to copy.

    Returns:
        A deep copy of the quantum program node.


    5. deep_copy(node: pyQPanda.ClassicalProg) -> pyQPanda.ClassicalProg

    Create a deep copy of the given quantum program node.

    Args:
        node: The quantum program node to copy.

    Returns:
        A deep copy of the quantum program node.


    6. deep_copy(node: pyQPanda.QIfProg) -> pyQPanda.QIfProg

    Create a deep copy of the given quantum program node.

    Args:
        node: The quantum program node to copy.

    Returns:
        A deep copy of the quantum program node.


    7. deep_copy(node: pyQPanda.QWhileProg) -> pyQPanda.QWhileProg

    Create a deep copy of the given quantum program node.

    Args:
        node: The quantum program node to copy.

    Returns:
        A deep copy of the quantum program node.

    """
@overload
def deep_copy(node: QGate) -> QGate:
    """deep_copy(*args, **kwargs)
    Overloaded function.

    1. deep_copy(node: pyQPanda.QProg) -> pyQPanda.QProg

    Create a deep copy of the given quantum program node.

    Args:
        node: The quantum program node to copy.

    Returns:
        A deep copy of the quantum program node.


    2. deep_copy(node: pyQPanda.QCircuit) -> pyQPanda.QCircuit

    Create a deep copy of the given quantum program node.

    Args:
        node: The quantum program node to copy.

    Returns:
        A deep copy of the quantum program node.


    3. deep_copy(node: pyQPanda.QGate) -> pyQPanda.QGate

    Create a deep copy of the given quantum program node.

    Args:
        node: The quantum program node to copy.

    Returns:
        A deep copy of the quantum program node.


    4. deep_copy(node: pyQPanda.QMeasure) -> pyQPanda.QMeasure

    Create a deep copy of the given quantum program node.

    Args:
        node: The quantum program node to copy.

    Returns:
        A deep copy of the quantum program node.


    5. deep_copy(node: pyQPanda.ClassicalProg) -> pyQPanda.ClassicalProg

    Create a deep copy of the given quantum program node.

    Args:
        node: The quantum program node to copy.

    Returns:
        A deep copy of the quantum program node.


    6. deep_copy(node: pyQPanda.QIfProg) -> pyQPanda.QIfProg

    Create a deep copy of the given quantum program node.

    Args:
        node: The quantum program node to copy.

    Returns:
        A deep copy of the quantum program node.


    7. deep_copy(node: pyQPanda.QWhileProg) -> pyQPanda.QWhileProg

    Create a deep copy of the given quantum program node.

    Args:
        node: The quantum program node to copy.

    Returns:
        A deep copy of the quantum program node.

    """
@overload
def deep_copy(node: QMeasure) -> QMeasure:
    """deep_copy(*args, **kwargs)
    Overloaded function.

    1. deep_copy(node: pyQPanda.QProg) -> pyQPanda.QProg

    Create a deep copy of the given quantum program node.

    Args:
        node: The quantum program node to copy.

    Returns:
        A deep copy of the quantum program node.


    2. deep_copy(node: pyQPanda.QCircuit) -> pyQPanda.QCircuit

    Create a deep copy of the given quantum program node.

    Args:
        node: The quantum program node to copy.

    Returns:
        A deep copy of the quantum program node.


    3. deep_copy(node: pyQPanda.QGate) -> pyQPanda.QGate

    Create a deep copy of the given quantum program node.

    Args:
        node: The quantum program node to copy.

    Returns:
        A deep copy of the quantum program node.


    4. deep_copy(node: pyQPanda.QMeasure) -> pyQPanda.QMeasure

    Create a deep copy of the given quantum program node.

    Args:
        node: The quantum program node to copy.

    Returns:
        A deep copy of the quantum program node.


    5. deep_copy(node: pyQPanda.ClassicalProg) -> pyQPanda.ClassicalProg

    Create a deep copy of the given quantum program node.

    Args:
        node: The quantum program node to copy.

    Returns:
        A deep copy of the quantum program node.


    6. deep_copy(node: pyQPanda.QIfProg) -> pyQPanda.QIfProg

    Create a deep copy of the given quantum program node.

    Args:
        node: The quantum program node to copy.

    Returns:
        A deep copy of the quantum program node.


    7. deep_copy(node: pyQPanda.QWhileProg) -> pyQPanda.QWhileProg

    Create a deep copy of the given quantum program node.

    Args:
        node: The quantum program node to copy.

    Returns:
        A deep copy of the quantum program node.

    """
@overload
def deep_copy(node: ClassicalProg) -> ClassicalProg:
    """deep_copy(*args, **kwargs)
    Overloaded function.

    1. deep_copy(node: pyQPanda.QProg) -> pyQPanda.QProg

    Create a deep copy of the given quantum program node.

    Args:
        node: The quantum program node to copy.

    Returns:
        A deep copy of the quantum program node.


    2. deep_copy(node: pyQPanda.QCircuit) -> pyQPanda.QCircuit

    Create a deep copy of the given quantum program node.

    Args:
        node: The quantum program node to copy.

    Returns:
        A deep copy of the quantum program node.


    3. deep_copy(node: pyQPanda.QGate) -> pyQPanda.QGate

    Create a deep copy of the given quantum program node.

    Args:
        node: The quantum program node to copy.

    Returns:
        A deep copy of the quantum program node.


    4. deep_copy(node: pyQPanda.QMeasure) -> pyQPanda.QMeasure

    Create a deep copy of the given quantum program node.

    Args:
        node: The quantum program node to copy.

    Returns:
        A deep copy of the quantum program node.


    5. deep_copy(node: pyQPanda.ClassicalProg) -> pyQPanda.ClassicalProg

    Create a deep copy of the given quantum program node.

    Args:
        node: The quantum program node to copy.

    Returns:
        A deep copy of the quantum program node.


    6. deep_copy(node: pyQPanda.QIfProg) -> pyQPanda.QIfProg

    Create a deep copy of the given quantum program node.

    Args:
        node: The quantum program node to copy.

    Returns:
        A deep copy of the quantum program node.


    7. deep_copy(node: pyQPanda.QWhileProg) -> pyQPanda.QWhileProg

    Create a deep copy of the given quantum program node.

    Args:
        node: The quantum program node to copy.

    Returns:
        A deep copy of the quantum program node.

    """
@overload
def deep_copy(node: QIfProg) -> QIfProg:
    """deep_copy(*args, **kwargs)
    Overloaded function.

    1. deep_copy(node: pyQPanda.QProg) -> pyQPanda.QProg

    Create a deep copy of the given quantum program node.

    Args:
        node: The quantum program node to copy.

    Returns:
        A deep copy of the quantum program node.


    2. deep_copy(node: pyQPanda.QCircuit) -> pyQPanda.QCircuit

    Create a deep copy of the given quantum program node.

    Args:
        node: The quantum program node to copy.

    Returns:
        A deep copy of the quantum program node.


    3. deep_copy(node: pyQPanda.QGate) -> pyQPanda.QGate

    Create a deep copy of the given quantum program node.

    Args:
        node: The quantum program node to copy.

    Returns:
        A deep copy of the quantum program node.


    4. deep_copy(node: pyQPanda.QMeasure) -> pyQPanda.QMeasure

    Create a deep copy of the given quantum program node.

    Args:
        node: The quantum program node to copy.

    Returns:
        A deep copy of the quantum program node.


    5. deep_copy(node: pyQPanda.ClassicalProg) -> pyQPanda.ClassicalProg

    Create a deep copy of the given quantum program node.

    Args:
        node: The quantum program node to copy.

    Returns:
        A deep copy of the quantum program node.


    6. deep_copy(node: pyQPanda.QIfProg) -> pyQPanda.QIfProg

    Create a deep copy of the given quantum program node.

    Args:
        node: The quantum program node to copy.

    Returns:
        A deep copy of the quantum program node.


    7. deep_copy(node: pyQPanda.QWhileProg) -> pyQPanda.QWhileProg

    Create a deep copy of the given quantum program node.

    Args:
        node: The quantum program node to copy.

    Returns:
        A deep copy of the quantum program node.

    """
@overload
def deep_copy(node: QWhileProg) -> QWhileProg:
    """deep_copy(*args, **kwargs)
    Overloaded function.

    1. deep_copy(node: pyQPanda.QProg) -> pyQPanda.QProg

    Create a deep copy of the given quantum program node.

    Args:
        node: The quantum program node to copy.

    Returns:
        A deep copy of the quantum program node.


    2. deep_copy(node: pyQPanda.QCircuit) -> pyQPanda.QCircuit

    Create a deep copy of the given quantum program node.

    Args:
        node: The quantum program node to copy.

    Returns:
        A deep copy of the quantum program node.


    3. deep_copy(node: pyQPanda.QGate) -> pyQPanda.QGate

    Create a deep copy of the given quantum program node.

    Args:
        node: The quantum program node to copy.

    Returns:
        A deep copy of the quantum program node.


    4. deep_copy(node: pyQPanda.QMeasure) -> pyQPanda.QMeasure

    Create a deep copy of the given quantum program node.

    Args:
        node: The quantum program node to copy.

    Returns:
        A deep copy of the quantum program node.


    5. deep_copy(node: pyQPanda.ClassicalProg) -> pyQPanda.ClassicalProg

    Create a deep copy of the given quantum program node.

    Args:
        node: The quantum program node to copy.

    Returns:
        A deep copy of the quantum program node.


    6. deep_copy(node: pyQPanda.QIfProg) -> pyQPanda.QIfProg

    Create a deep copy of the given quantum program node.

    Args:
        node: The quantum program node to copy.

    Returns:
        A deep copy of the quantum program node.


    7. deep_copy(node: pyQPanda.QWhileProg) -> pyQPanda.QWhileProg

    Create a deep copy of the given quantum program node.

    Args:
        node: The quantum program node to copy.

    Returns:
        A deep copy of the quantum program node.

    """
def del_weak_edge(topo_data: list[list[int]]) -> None:
    """del_weak_edge(topo_data: List[List[int]]) -> None

    Delete weakly connected edges from the quantum program topology.

    Args:
        topo_data: The topology data of the quantum program.

    Returns:
        None.

    """
def del_weak_edge2(topo_data: list[list[int]], max_connect_degree: int, sub_graph_set: list[int]) -> list:
    """del_weak_edge2(topo_data: List[List[int]], max_connect_degree: int, sub_graph_set: List[int]) -> list

    Delete weakly connected edges from the quantum program topology.

    Args:
        topo_data: The topology data of the quantum program.

        max_connect_degree: The maximum allowable connection degree.

        sub_graph_set: A list of subgraph identifiers.

    Returns:
        A list containing the updated topology data, intermediary points, and candidate edges.

    """
def del_weak_edge3(topo_data: list[list[int]], sub_graph_set: list[int], max_connect_degree: int, lamda1: float, lamda2: float, lamda3: float) -> list:
    """del_weak_edge3(topo_data: List[List[int]], sub_graph_set: List[int], max_connect_degree: int, lamda1: float, lamda2: float, lamda3: float) -> list

    Delete weakly connected edges based on specified parameters.

    Args:
        topo_data: The topology data of the quantum program.

        sub_graph_set: A list of subgraph identifiers.

        max_connect_degree: The maximum allowable connection degree.

        lamda1: Weight parameter for edge evaluation.

        lamda2: Weight parameter for edge evaluation.

        lamda3: Weight parameter for edge evaluation.

    Returns:
        A list containing the updated topology data and intermediary points.

    """
def destroy_quantum_machine(machine: QuantumMachine) -> None:
    """destroy_quantum_machine(machine: pyQPanda.QuantumMachine) -> None

    Destroy a quantum machine.

    Args:
        machine: type should be one of CPUQVM, CPUSingleThreadQVM, GPUQVM, NoiseQVM.

    Returns:
        None.

    """
def directly_run(qprog: QProg, noise_model: Noise = ...) -> dict[str, bool]:
    """directly_run(qprog: pyQPanda.QProg, noise_model: pyQPanda.Noise = NoiseModel()) -> Dict[str, bool]

    Directly run a quantum program
    After init()

    Args:
        qprog: The quantum program to be executed.

        noise_model: The noise model to be used, default is no noise. The noise model only works on CPUQVM currently.

    Returns:
        Dict[str, bool]: Result of the quantum program execution in one shot.
                        The first element is the final qubit register state,
                        and the second is its measurement probability.

    """
@overload
def div(arg0: ClassicalCondition, arg1: ClassicalCondition) -> ClassicalCondition:
    """div(*args, **kwargs)
    Overloaded function.

    1. div(arg0: pyQPanda.ClassicalCondition, arg1: pyQPanda.ClassicalCondition) -> pyQPanda.ClassicalCondition

    Divide one ClassicalCondition by another.

    Args:
        a: The numerator ClassicalCondition.

        b: The denominator ClassicalCondition.

    Returns:
        ClassicalCondition: The result of the division.


    2. div(arg0: pyQPanda.ClassicalCondition, arg1: int) -> pyQPanda.ClassicalCondition

    Divide a ClassicalCondition by a bit size.

    Args:
        a: The ClassicalCondition (numerator).

        b: The bit size (denominator).

    Returns:
        ClassicalCondition: The result of the division.


    3. div(arg0: int, arg1: pyQPanda.ClassicalCondition) -> pyQPanda.ClassicalCondition

    Divide a bit size by a ClassicalCondition.

    Args:
        a: The bit size (numerator).

        b: The ClassicalCondition (denominator).

    Returns:
        ClassicalCondition: The result of the division.

    """
@overload
def div(arg0: ClassicalCondition, arg1: int) -> ClassicalCondition:
    """div(*args, **kwargs)
    Overloaded function.

    1. div(arg0: pyQPanda.ClassicalCondition, arg1: pyQPanda.ClassicalCondition) -> pyQPanda.ClassicalCondition

    Divide one ClassicalCondition by another.

    Args:
        a: The numerator ClassicalCondition.

        b: The denominator ClassicalCondition.

    Returns:
        ClassicalCondition: The result of the division.


    2. div(arg0: pyQPanda.ClassicalCondition, arg1: int) -> pyQPanda.ClassicalCondition

    Divide a ClassicalCondition by a bit size.

    Args:
        a: The ClassicalCondition (numerator).

        b: The bit size (denominator).

    Returns:
        ClassicalCondition: The result of the division.


    3. div(arg0: int, arg1: pyQPanda.ClassicalCondition) -> pyQPanda.ClassicalCondition

    Divide a bit size by a ClassicalCondition.

    Args:
        a: The bit size (numerator).

        b: The ClassicalCondition (denominator).

    Returns:
        ClassicalCondition: The result of the division.

    """
@overload
def div(arg0: int, arg1: ClassicalCondition) -> ClassicalCondition:
    """div(*args, **kwargs)
    Overloaded function.

    1. div(arg0: pyQPanda.ClassicalCondition, arg1: pyQPanda.ClassicalCondition) -> pyQPanda.ClassicalCondition

    Divide one ClassicalCondition by another.

    Args:
        a: The numerator ClassicalCondition.

        b: The denominator ClassicalCondition.

    Returns:
        ClassicalCondition: The result of the division.


    2. div(arg0: pyQPanda.ClassicalCondition, arg1: int) -> pyQPanda.ClassicalCondition

    Divide a ClassicalCondition by a bit size.

    Args:
        a: The ClassicalCondition (numerator).

        b: The bit size (denominator).

    Returns:
        ClassicalCondition: The result of the division.


    3. div(arg0: int, arg1: pyQPanda.ClassicalCondition) -> pyQPanda.ClassicalCondition

    Divide a bit size by a ClassicalCondition.

    Args:
        a: The bit size (numerator).

        b: The ClassicalCondition (denominator).

    Returns:
        ClassicalCondition: The result of the division.

    """
def dot(arg0: var, arg1: var) -> var:
    """dot(arg0: pyQPanda.var, arg1: pyQPanda.var) -> pyQPanda.var"""
@overload
def double_gate_xeb(config: QCloudTaskConfig, qubit0: int, qubit1: int, clifford_range: list[int], num_circuits: int, gate_type: GateType = ...) -> dict[int, float]:
    """double_gate_xeb(*args, **kwargs)
    Overloaded function.

    1. double_gate_xeb(config: pyQPanda.QCloudTaskConfig, qubit0: int, qubit1: int, clifford_range: List[int], num_circuits: int, gate_type: pyQPanda.GateType = GateType.CZ_GATE) -> Dict[int, float]

    double gate xeb

    Args:
        qvm: quantum machine
        qubit0: double qubit 0
        qubit1: double qubit 1
        clifford_range: clifford range list
        num_circuits: the num of circuits
        interleaved_gates: interleaved gates list

    Returns:
        result data dict
    Raises:
        run_fail: An error occurred in double_gate_xeb


    2. double_gate_xeb(qvm: pyQPanda.QuantumMachine, qubit0: pyQPanda.Qubit, qubit1: pyQPanda.Qubit, clifford_range: List[int], num_circuits: int, shots: int, chip_id: int = 2, gate_type: pyQPanda.GateType = GateType.CZ_GATE) -> Dict[int, float]

    double gate xeb

    Args:
        qvm: quantum machine
        qubit0: double qubit 0
        qubit1: double qubit 1
        clifford_range: clifford range list
        num_circuits: the num of circuits
        shots: measure shots
        chip type: RealChipType
        interleaved_gates: interleaved gates list

    Returns:
        result data dict
    Raises:
        run_fail: An error occurred in double_gate_xeb

    """
@overload
def double_gate_xeb(qvm: QuantumMachine, qubit0: Qubit, qubit1: Qubit, clifford_range: list[int], num_circuits: int, shots: int, chip_id: int = ..., gate_type: GateType = ...) -> dict[int, float]:
    """double_gate_xeb(*args, **kwargs)
    Overloaded function.

    1. double_gate_xeb(config: pyQPanda.QCloudTaskConfig, qubit0: int, qubit1: int, clifford_range: List[int], num_circuits: int, gate_type: pyQPanda.GateType = GateType.CZ_GATE) -> Dict[int, float]

    double gate xeb

    Args:
        qvm: quantum machine
        qubit0: double qubit 0
        qubit1: double qubit 1
        clifford_range: clifford range list
        num_circuits: the num of circuits
        interleaved_gates: interleaved gates list

    Returns:
        result data dict
    Raises:
        run_fail: An error occurred in double_gate_xeb


    2. double_gate_xeb(qvm: pyQPanda.QuantumMachine, qubit0: pyQPanda.Qubit, qubit1: pyQPanda.Qubit, clifford_range: List[int], num_circuits: int, shots: int, chip_id: int = 2, gate_type: pyQPanda.GateType = GateType.CZ_GATE) -> Dict[int, float]

    double gate xeb

    Args:
        qvm: quantum machine
        qubit0: double qubit 0
        qubit1: double qubit 1
        clifford_range: clifford range list
        num_circuits: the num of circuits
        shots: measure shots
        chip type: RealChipType
        interleaved_gates: interleaved gates list

    Returns:
        result data dict
    Raises:
        run_fail: An error occurred in double_gate_xeb

    """
@overload
def double_qubit_rb(qvm: QuantumMachine, qubit0: Qubit, qubit1: Qubit, clifford_range: list[int], num_circuits: int, shots: int, chip_id: int = ..., interleaved_gates: list[QGate] = ...) -> dict[int, float]:
    """double_qubit_rb(*args, **kwargs)
    Overloaded function.

    1. double_qubit_rb(qvm: pyQPanda.QuantumMachine, qubit0: pyQPanda.Qubit, qubit1: pyQPanda.Qubit, clifford_range: List[int], num_circuits: int, shots: int, chip_id: int = 2, interleaved_gates: List[pyQPanda.QGate] = []) -> Dict[int, float]

    double qubit rb with WU YUAN chip
    Args:
        qvm: quantum machine
        qubit0: double qubit 0
        qubit1: double qubit 1
        clifford_range: clifford range list
        num_circuits: the num of circuits
        shots: measure shots
        chip type: RealChipType
        interleaved_gates: interleaved gates list

    Returns:
        result data dict
    Raises:
        run_fail: An error occurred in double_qubit_rb


    2. double_qubit_rb(config: pyQPanda.QCloudTaskConfig, qubit0: int, qubit1: int, clifford_range: List[int], num_circuits: int, interleaved_gates: List[pyQPanda.QGate] = []) -> Dict[int, float]

    double qubit rb with origin chip
    Args:
        config: QCloudTaskConfig
        qubit0: double qubit 0
        qubit1: double qubit 1
        clifford_range: clifford range list
        num_circuits: the num of circuits
        interleaved_gates: interleaved gates list

    Returns:
        result data dict
    Raises:
        run_fail: An error occurred in double_qubit_rb

    """
@overload
def double_qubit_rb(config: QCloudTaskConfig, qubit0: int, qubit1: int, clifford_range: list[int], num_circuits: int, interleaved_gates: list[QGate] = ...) -> dict[int, float]:
    """double_qubit_rb(*args, **kwargs)
    Overloaded function.

    1. double_qubit_rb(qvm: pyQPanda.QuantumMachine, qubit0: pyQPanda.Qubit, qubit1: pyQPanda.Qubit, clifford_range: List[int], num_circuits: int, shots: int, chip_id: int = 2, interleaved_gates: List[pyQPanda.QGate] = []) -> Dict[int, float]

    double qubit rb with WU YUAN chip
    Args:
        qvm: quantum machine
        qubit0: double qubit 0
        qubit1: double qubit 1
        clifford_range: clifford range list
        num_circuits: the num of circuits
        shots: measure shots
        chip type: RealChipType
        interleaved_gates: interleaved gates list

    Returns:
        result data dict
    Raises:
        run_fail: An error occurred in double_qubit_rb


    2. double_qubit_rb(config: pyQPanda.QCloudTaskConfig, qubit0: int, qubit1: int, clifford_range: List[int], num_circuits: int, interleaved_gates: List[pyQPanda.QGate] = []) -> Dict[int, float]

    double qubit rb with origin chip
    Args:
        config: QCloudTaskConfig
        qubit0: double qubit 0
        qubit1: double qubit 1
        clifford_range: clifford range list
        num_circuits: the num of circuits
        interleaved_gates: interleaved gates list

    Returns:
        result data dict
    Raises:
        run_fail: An error occurred in double_qubit_rb

    """
def draw_qprog_latex(prog: QProg, auto_wrap_len: int = ..., output_file: str = ..., with_logo: bool = ..., itr_start: NodeIter = ..., itr_end: NodeIter = ..., b_with_gate_params: bool) -> str:
    """draw_qprog_latex(prog: pyQPanda.QProg, auto_wrap_len: int = 100, output_file: str = 'QCircuit.tex', with_logo: bool = False, itr_start: pyQPanda.NodeIter = NodeIter(), itr_end: pyQPanda.NodeIter = NodeIter(), b_with_gate_params: bool) -> str

    Convert a quantum prog/circuit to latex source code, and save the source code to file in current path with name QCircuit.tex
    Args:
        QProg: quantum prog 
        auto_wrap_len: defaut is 100 
        output_file: result output file name 
        itr_start: nodeiter start 
        itr_end: nodeiter end 
        b_with_gate_params: if b_with_gate_params is True, draw gate with gate's params.if b_with_gate_params is False, draw gate without gate's params.

    Returns:
        result data tuple contains prog info
    Raises:
        run_fail: An error occurred in get draw_qprog_text

    """
def draw_qprog_latex_with_clock(prog: QProg, config_data: str = ..., auto_wrap_len: bool = ..., output_file: int = ..., with_logo: str = ..., itr_start: NodeIter = ..., itr_end: NodeIter = ..., b_with_gate_params: bool) -> str:
    """draw_qprog_latex_with_clock(prog: pyQPanda.QProg, config_data: str = 'QPandaConfig.json', auto_wrap_len: bool = 100, output_file: int = 'QCircuit.tex', with_logo: str = False, itr_start: pyQPanda.NodeIter = NodeIter(), itr_end: pyQPanda.NodeIter = NodeIter(), b_with_gate_params: bool) -> str

    Convert a quantum prog/circuit to latex source code with time sequence, and save the source code to file in current path with name QCircuit.tex
    Args:
        QProg: quantum prog 
        config_data: default config file is QPandaConfig.json 
        auto_wrap_len: defaut is 100 
        output_file: result output file name 
        itr_start: nodeiter start 
        itr_end: nodeiter end 
        b_with_gate_params: if b_with_gate_params is True, draw gate with gate's params.if b_with_gate_params is False, draw gate without gate's params.

    Returns:
        result data tuple contains prog info
    Raises:
        run_fail: An error occurred in get draw_qprog_text

    """
def draw_qprog_text(qprog: QProg, auto_wrap_len: int = ..., output_file: str = ..., itr_start: NodeIter = ..., itr_end: NodeIter = ..., b_with_gate_params: bool) -> str:
    """draw_qprog_text(qprog: pyQPanda.QProg, auto_wrap_len: int = 100, output_file: str = 'QCircuitTextPic.txt', itr_start: pyQPanda.NodeIter = NodeIter(), itr_end: pyQPanda.NodeIter = NodeIter(), b_with_gate_params: bool) -> str

    Convert a quantum prog/circuit to text-pic(UTF-8 code),
    and will save the text-pic in file named QCircuitTextPic.txt in the same time in current path
    Args:
        QProg: quantum prog 
        auto_wrap_len: defaut is 100 
        output_file: result output file name 
        itr_start: nodeiter start 
        itr_end: nodeiter end 
        b_with_gate_params: if b_with_gate_params is True, draw gate with gate's params.if b_with_gate_params is False, draw gate without gate's params.

    Returns:
        result data tuple contains prog info
    Raises:
        run_fail: An error occurred in get draw_qprog_text

    """
def draw_qprog_text_with_clock(prog: QProg, config_data: str = ..., auto_wrap_len: int = ..., output_file: str = ..., itr_start: NodeIter = ..., itr_end: NodeIter = ..., b_with_gate_params: bool) -> str:
    """draw_qprog_text_with_clock(prog: pyQPanda.QProg, config_data: str = 'QPandaConfig.json', auto_wrap_len: int = 100, output_file: str = 'QCircuitTextPic.txt', itr_start: pyQPanda.NodeIter = NodeIter(), itr_end: pyQPanda.NodeIter = NodeIter(), b_with_gate_params: bool) -> str

    Convert a quantum prog/circuit to text-pic(UTF-8 code) with time sequence,
    and will save the text-pic in file named QCircuitTextPic.txt in the same time in current path
    Args:
        QProg: quantum prog 
        auto_wrap_len: defaut is 100 
        output_file: result output file name 
        itr_start: nodeiter start 
        itr_end: nodeiter end 
        b_with_gate_params: if b_with_gate_params is True, draw gate with gate's params.if b_with_gate_params is False, draw gate without gate's params.

    Returns:
        result data tuple contains prog info
    Raises:
        run_fail: An error occurred in get draw_qprog_text

    """
def dropout(arg0: var, arg1: var) -> var:
    """dropout(arg0: pyQPanda.var, arg1: pyQPanda.var) -> pyQPanda.var"""
@overload
def equal(arg0: ClassicalCondition, arg1: ClassicalCondition) -> ClassicalCondition:
    """equal(*args, **kwargs)
    Overloaded function.

    1. equal(arg0: pyQPanda.ClassicalCondition, arg1: pyQPanda.ClassicalCondition) -> pyQPanda.ClassicalCondition

    Check if two ClassicalConditions are equal.

    Args:
        a: The first ClassicalCondition.

        b: The second ClassicalCondition.

    Returns:
        bool: True if both ClassicalConditions are equal, otherwise False.


    2. equal(arg0: pyQPanda.ClassicalCondition, arg1: int) -> pyQPanda.ClassicalCondition

    Check if a ClassicalCondition is equal to a bit size.

    Args:
        a: The ClassicalCondition to compare.

        b: The bit size to compare against.

    Returns:
        bool: True if the ClassicalCondition is equal to the bit size, otherwise False.


    3. equal(arg0: int, arg1: pyQPanda.ClassicalCondition) -> pyQPanda.ClassicalCondition

    Check if a bit size is equal to a ClassicalCondition.

    Args:
        a: The bit size to compare.

        b: The ClassicalCondition to compare against.

    Returns:
        bool: True if the bit size is equal to the ClassicalCondition, otherwise False.

    """
@overload
def equal(arg0: ClassicalCondition, arg1: int) -> ClassicalCondition:
    """equal(*args, **kwargs)
    Overloaded function.

    1. equal(arg0: pyQPanda.ClassicalCondition, arg1: pyQPanda.ClassicalCondition) -> pyQPanda.ClassicalCondition

    Check if two ClassicalConditions are equal.

    Args:
        a: The first ClassicalCondition.

        b: The second ClassicalCondition.

    Returns:
        bool: True if both ClassicalConditions are equal, otherwise False.


    2. equal(arg0: pyQPanda.ClassicalCondition, arg1: int) -> pyQPanda.ClassicalCondition

    Check if a ClassicalCondition is equal to a bit size.

    Args:
        a: The ClassicalCondition to compare.

        b: The bit size to compare against.

    Returns:
        bool: True if the ClassicalCondition is equal to the bit size, otherwise False.


    3. equal(arg0: int, arg1: pyQPanda.ClassicalCondition) -> pyQPanda.ClassicalCondition

    Check if a bit size is equal to a ClassicalCondition.

    Args:
        a: The bit size to compare.

        b: The ClassicalCondition to compare against.

    Returns:
        bool: True if the bit size is equal to the ClassicalCondition, otherwise False.

    """
@overload
def equal(arg0: int, arg1: ClassicalCondition) -> ClassicalCondition:
    """equal(*args, **kwargs)
    Overloaded function.

    1. equal(arg0: pyQPanda.ClassicalCondition, arg1: pyQPanda.ClassicalCondition) -> pyQPanda.ClassicalCondition

    Check if two ClassicalConditions are equal.

    Args:
        a: The first ClassicalCondition.

        b: The second ClassicalCondition.

    Returns:
        bool: True if both ClassicalConditions are equal, otherwise False.


    2. equal(arg0: pyQPanda.ClassicalCondition, arg1: int) -> pyQPanda.ClassicalCondition

    Check if a ClassicalCondition is equal to a bit size.

    Args:
        a: The ClassicalCondition to compare.

        b: The bit size to compare against.

    Returns:
        bool: True if the ClassicalCondition is equal to the bit size, otherwise False.


    3. equal(arg0: int, arg1: pyQPanda.ClassicalCondition) -> pyQPanda.ClassicalCondition

    Check if a bit size is equal to a ClassicalCondition.

    Args:
        a: The bit size to compare.

        b: The ClassicalCondition to compare against.

    Returns:
        bool: True if the bit size is equal to the ClassicalCondition, otherwise False.

    """
def estimate_topology(topo_data: list[list[int]]) -> float:
    """estimate_topology(topo_data: List[List[int]]) -> float

    Evaluate topology performance.

    Args:
        topo_data: Quantum program topology data.

    Returns:
        Result data.

    """
@overload
def eval(arg0: var, arg1: bool) -> numpy.ndarray[numpy.float64[m, n]]:
    """eval(*args, **kwargs)
    Overloaded function.

    1. eval(arg0: pyQPanda.var, arg1: bool) -> numpy.ndarray[numpy.float64[m, n]]

    2. eval(arg0: pyQPanda.var) -> numpy.ndarray[numpy.float64[m, n]]
    """
@overload
def eval(arg0: var) -> numpy.ndarray[numpy.float64[m, n]]:
    """eval(*args, **kwargs)
    Overloaded function.

    1. eval(arg0: pyQPanda.var, arg1: bool) -> numpy.ndarray[numpy.float64[m, n]]

    2. eval(arg0: pyQPanda.var) -> numpy.ndarray[numpy.float64[m, n]]
    """
def exp(arg0: var) -> var:
    """exp(arg0: pyQPanda.var) -> pyQPanda.var"""
def expMat(arg0: complex, arg1: numpy.ndarray[numpy.complex128[m, n]], arg2: float) -> numpy.ndarray[numpy.complex128[m, n]]:
    """expMat(arg0: complex, arg1: numpy.ndarray[numpy.complex128[m, n]], arg2: float) -> numpy.ndarray[numpy.complex128[m, n]]

    Calculate the matrix power of e.

    This function returns the power of matrix e.

    Args:
         None

    Returns:
         The computed matrix.

    """
@overload
def expand_linear_equations(matrix: list[complex], list: list[float]) -> list:
    """expand_linear_equations(*args, **kwargs)
    Overloaded function.

    1. expand_linear_equations(matrix: List[complex], list: List[float]) -> list

    Extending linear equations to N dimension, N = 2 ^ n

    Args:
        matrix: the source matrix, which will be extend to N*N, N = 2 ^ n

        list: the source vector b, which will be extend to 2 ^ n

    2. expand_linear_equations(matrix: List[complex], list: List[float]) -> list

    Extending linear equations to N dimension, N = 2 ^ n

    Args:
        matrix: the source matrix, which will be extend to N*N, N = 2 ^ n

        list: the source vector b, which will be extend to 2 ^ n
    """
@overload
def expand_linear_equations(matrix: list[complex], list: list[float]) -> list:
    """expand_linear_equations(*args, **kwargs)
    Overloaded function.

    1. expand_linear_equations(matrix: List[complex], list: List[float]) -> list

    Extending linear equations to N dimension, N = 2 ^ n

    Args:
        matrix: the source matrix, which will be extend to N*N, N = 2 ^ n

        list: the source vector b, which will be extend to 2 ^ n

    2. expand_linear_equations(matrix: List[complex], list: List[float]) -> list

    Extending linear equations to N dimension, N = 2 ^ n

    Args:
        matrix: the source matrix, which will be extend to N*N, N = 2 ^ n

        list: the source vector b, which will be extend to 2 ^ n
    """
def fill_qprog_by_I(qprog: QProg) -> QProg:
    """fill_qprog_by_I(qprog: pyQPanda.QProg) -> pyQPanda.QProg

    Fill the input quantum program with I gates and return a new quantum program.

    Args:
        qprog: the input quantum program.

    Returns:
        a new quantum program filled with I gates.

    """
def finalize() -> None:
    """finalize() -> None

    Finalize the environment and destroy global unique quantum machine.

    Args:
        none

    Returns:
        none

    """
def fit_to_gbk(utf8_str: str) -> str:
    """fit_to_gbk(utf8_str: str) -> str

    Special character conversion.

    Args:
        utf8_str: string using utf-8 encoding.

    Returns:
        result: converted string.

    """
@overload
def flatten(qprog: QProg) -> None:
    """flatten(*args, **kwargs)
    Overloaded function.

    1. flatten(qprog: pyQPanda.QProg) -> None

    Flatten a quantum program in place.

    Args:
        qprog: The quantum program to be flattened.

    Returns:
        None: The function modifies the quantum program directly.


    2. flatten(qcircuit: pyQPanda.QCircuit) -> None

    Flatten a quantum circuit in place.

    Args:
        qcircuit: The quantum circuit to be flattened.

    Returns:
        None: The function modifies the circuit directly.

    """
@overload
def flatten(qcircuit: QCircuit) -> None:
    """flatten(*args, **kwargs)
    Overloaded function.

    1. flatten(qprog: pyQPanda.QProg) -> None

    Flatten a quantum program in place.

    Args:
        qprog: The quantum program to be flattened.

    Returns:
        None: The function modifies the quantum program directly.


    2. flatten(qcircuit: pyQPanda.QCircuit) -> None

    Flatten a quantum circuit in place.

    Args:
        qcircuit: The quantum circuit to be flattened.

    Returns:
        None: The function modifies the circuit directly.

    """
def getAllocateCMem() -> int:
    """getAllocateCMem() -> int

    Deprecated, use get_allocate_cmem_num instead.

    Args:
        none

    Returns:
        allocate qubit num.

    """
def getAllocateQubitNum() -> int:
    """getAllocateQubitNum() -> int

    Deprecated, use get_allocate_qubit_num instead.

    Args:
        none

    Returns:
        allocate cbit num.

    """
def get_adjacent_qgate_type(qprog: QProg, node_iter: NodeIter) -> list[NodeInfo]:
    """get_adjacent_qgate_type(qprog: pyQPanda.QProg, node_iter: pyQPanda.NodeIter) -> List[pyQPanda.NodeInfo]

    Get the adjacent quantum gates' (the front one and the back one) type info from QProg.

    Args:
        qprog: Target quantum program.

        node_iter: Gate node iterator in qprog.

    Returns:
        The front and back node info of node_iter in qprog.

    """
def get_all_used_qubits(qprog: QProg) -> list[Qubit]:
    """get_all_used_qubits(qprog: pyQPanda.QProg) -> List[pyQPanda.Qubit]

    Get all the quantum bits used in the input program.

    Args:
        qprog: A quantum program.

    Returns:
        result: A list of all used qubits.

    """
def get_all_used_qubits_to_int(qprog: QProg) -> list[int]:
    """get_all_used_qubits_to_int(qprog: pyQPanda.QProg) -> List[int]

    Get the addresses of all used quantum bits in the input program.

    Args:
        qprog: A quantum program.

    Returns:
        result: A list of addresses of all used qubits.

    """
def get_allocate_cbits() -> list[ClassicalCondition]:
    """get_allocate_cbits() -> List[pyQPanda.ClassicalCondition]

    Get allocated cbits of QuantumMachine

    Args:
        None

    Returns:
        A list of allocated cbits.

    """
def get_allocate_cmem_num() -> int:
    """get_allocate_cmem_num() -> int

    Get allocate cmem num.

    Args:
        none.

    Returns:
        cbit_num: allocate cbit num.

    """
def get_allocate_qubit_num() -> int:
    """get_allocate_qubit_num() -> int

    Get allocate qubit num.

    Args:
        none.

    Returns:
        qubit_num: allocate qubit num.

    """
def get_allocate_qubits() -> list[Qubit]:
    """get_allocate_qubits() -> List[pyQPanda.Qubit]

    Get allocated qubits of QuantumMachine

    Args:
        None

    Returns:
        A list of allocated qubits.

    """
def get_bin_data(qprog: QProg) -> list[int]:
    """get_bin_data(qprog: pyQPanda.QProg) -> List[int]

    Get quantum program binary data.

    Args:
        qprog: quantum program (QProg).

    Returns:
        binary data as a list.

    """
def get_bin_str(qprog: QProg, machine: QuantumMachine) -> str:
    """get_bin_str(qprog: pyQPanda.QProg, machine: pyQPanda.QuantumMachine) -> str

    Transform a quantum program into a string representation.

    Args:
        qprog: quantum program.

        machine: quantum machine.

    Returns:
        string: base64-encoded string of the binary representation.

    """
def get_circuit_optimal_topology(qprog: QProg, machine: QuantumMachine, max_connect_degree: int, config_file: str = ...) -> list[list[int]]:
    """get_circuit_optimal_topology(qprog: pyQPanda.QProg, machine: pyQPanda.QuantumMachine, max_connect_degree: int, config_file: str = 'QPandaConfig.json') -> List[List[int]]

    Retrieve the optimal topology of the input quantum circuit.

    Args:
        qprog: The quantum program for which to determine the topology.

        machine: The quantum machine used for execution.

        max_connect_degree: The maximum allowable connection degree.

        config_file: Path to the configuration file (default is CONFIG_PATH).

    Returns:
        The topology program data.

    """
def get_clock_cycle(qpog: QProg) -> int:
    """get_clock_cycle(qpog: pyQPanda.QProg) -> int

    Get quantum program clock cycle.

    Args:
        qprog: quantum program (QProg).

    Returns:
        clock_cycle.

    """
def get_complex_points(topo_data: list[list[int]], max_connect_degree: int) -> list[int]:
    """get_complex_points(topo_data: List[List[int]], max_connect_degree: int) -> List[int]

    Retrieve complex points from the given topology data.

    Args:
        topo_data: The topology data of the quantum program.

        max_connect_degree: The maximum allowable connection degree.

    Returns:
        A list of complex points extracted from the topology data.

    """
def get_double_gate_block_topology(qprog: QProg) -> list[list[int]]:
    """get_double_gate_block_topology(qprog: pyQPanda.QProg) -> List[List[int]]

    Retrieve the double gate block topology from the input quantum program.

    Args:
        qprog: The quantum program for which to extract the double gate block topology.

    Returns:
        The topology program data.

    """
def get_matrix(*args, **kwargs):
    """get_matrix(qprog: pyQPanda.QProg, positive_seq: bool = False, nodeitr_start: pyQPanda.NodeIter = NodeIter(), nodeitr_end: pyQPanda.NodeIter = NodeIter()) -> List[complex]

    Get the target matrix between the input two NodeIters.

    Args:
        qprog: Quantum program.

        positive_seq: Qubit order of output matrix; true for positive sequence (q0q1q2),
      false for inverted order (q2q1q0), default is false.

        nodeitr_start: The start NodeIter.

        nodeitr_end: The end NodeIter.

    Returns:
        The target matrix including all the QGate's matrices (multiplied).

    """
def get_prob_dict(qubit_list: QVec, select_max: int = ...) -> dict[str, float]:
    """get_prob_dict(qubit_list: pyQPanda.QVec, select_max: int = -1) -> Dict[str, float]

    Get pmeasure result as dict

    Args:
        qubit_list: pmeasure qubits list.

        select_max: max returned element num in returned tuple, should be in [-1, 1<<len(qubit_list)],

        default is -1, meaning no limit.

    Returns:
        Measure result of quantum machine.

    """
def get_prob_list(qubit_list: QVec, select_max: int = ...) -> list[float]:
    """get_prob_list(qubit_list: pyQPanda.QVec, select_max: int = -1) -> List[float]

    Get pmeasure result as list

    Args:
        qubit_list: pmeasure qubits list.

        select_max: max returned element num in returned tuple, should be in [-1, 1<<len(qubit_list)],

        default is -1, meaning no limit.

    Returns:
        Measure result of quantum machine.

    """
@overload
def get_qgate_num(quantum_prog: QProg) -> int:
    """get_qgate_num(*args, **kwargs)
    Overloaded function.

    1. get_qgate_num(quantum_prog: pyQPanda.QProg) -> int

    Count quantum gate number in the quantum program.

    Args:
        quantum_prog: quantum program.

    Returns:
        result: gate count.


    2. get_qgate_num(quantum_circuit: pyQPanda.QCircuit) -> int

    Count quantum gate number in the quantum circuit.

    Args:
        quantum_circuit: quantum circuit.

    Returns:
        result: gate count.


    3. get_qgate_num(qprog: pyQPanda.QProg) -> int

    Count the number of quantum gates in a quantum program.

    Args:
        qprog: The quantum program to analyze.

    Returns:
        int: The number of quantum gates in the quantum program.

    """
@overload
def get_qgate_num(quantum_circuit: QCircuit) -> int:
    """get_qgate_num(*args, **kwargs)
    Overloaded function.

    1. get_qgate_num(quantum_prog: pyQPanda.QProg) -> int

    Count quantum gate number in the quantum program.

    Args:
        quantum_prog: quantum program.

    Returns:
        result: gate count.


    2. get_qgate_num(quantum_circuit: pyQPanda.QCircuit) -> int

    Count quantum gate number in the quantum circuit.

    Args:
        quantum_circuit: quantum circuit.

    Returns:
        result: gate count.


    3. get_qgate_num(qprog: pyQPanda.QProg) -> int

    Count the number of quantum gates in a quantum program.

    Args:
        qprog: The quantum program to analyze.

    Returns:
        int: The number of quantum gates in the quantum program.

    """
@overload
def get_qgate_num(qprog: QProg) -> int:
    """get_qgate_num(*args, **kwargs)
    Overloaded function.

    1. get_qgate_num(quantum_prog: pyQPanda.QProg) -> int

    Count quantum gate number in the quantum program.

    Args:
        quantum_prog: quantum program.

    Returns:
        result: gate count.


    2. get_qgate_num(quantum_circuit: pyQPanda.QCircuit) -> int

    Count quantum gate number in the quantum circuit.

    Args:
        quantum_circuit: quantum circuit.

    Returns:
        result: gate count.


    3. get_qgate_num(qprog: pyQPanda.QProg) -> int

    Count the number of quantum gates in a quantum program.

    Args:
        qprog: The quantum program to analyze.

    Returns:
        int: The number of quantum gates in the quantum program.

    """
def get_qprog_clock_cycle(qprog: QProg, machine: QuantumMachine, optimize: bool = ...) -> int:
    """get_qprog_clock_cycle(qprog: pyQPanda.QProg, machine: pyQPanda.QuantumMachine, optimize: bool = False) -> int

    Get Quantum Program Clock Cycle.

    Args:
        qprog: quantum program (QProg).

        machine: quantum machine (QuantumMachine*).

        optimize: whether to optimize qprog (default is false).

    Returns:
        QProg time consumed, no unit, not in seconds.

    """
@overload
def get_qstate() -> list[complex]:
    """get_qstate() -> List[complex]

    Get quantum machine state vector.

    Args:
        none: no parameters required.

    Returns:
        list: state vector list result.

    Examples:
        >>> print(machine.get_qstate())
        [0.707+0j, 0.707+0j, 0, 0]

    """
@overload
def get_qstate() -> Any:
    """get_qstate() -> List[complex]

    Get quantum machine state vector.

    Args:
        none: no parameters required.

    Returns:
        list: state vector list result.

    Examples:
        >>> print(machine.get_qstate())
        [0.707+0j, 0.707+0j, 0, 0]

    """
def get_sub_graph(topo_data: list[list[int]]) -> list[int]:
    """get_sub_graph(topo_data: List[List[int]]) -> List[int]

    Retrieve a subgraph from the provided topology data.

    Args:
        topo_data: The topology data of the quantum program.

    Returns:
        sub graph: The extracted subgraph from the provided topology.

    """
def get_tuple_list(qubit_list: QVec, select_max: int = ...) -> list[tuple[int, float]]:
    """get_tuple_list(qubit_list: pyQPanda.QVec, select_max: int = -1) -> List[Tuple[int, float]]

    Get pmeasure result as tuple list

    Args:
        qubit_list: pmeasure qubits list.

        select_max: max returned element num in returned tuple, should be in [-1, 1<<len(qubit_list)],

        default is -1, meaning no limit.

    Returns:
        Measure result of quantum machine.

    """
def get_unitary(*args, **kwargs):
    """get_unitary(qprog: pyQPanda.QProg, positive_seq: bool = False, nodeitr_start: pyQPanda.NodeIter = NodeIter(), nodeitr_end: pyQPanda.NodeIter = NodeIter()) -> List[complex]

    Get the target unitary matrix between the input two NodeIters.

    Args:
        qprog: Quantum program.

        positive_seq: Qubit order of output matrix; true for positive sequence (q0q1q2),
      false for inverted order (q2q1q0), default is false.

        nodeitr_start: The start NodeIter.

        nodeitr_end: The end NodeIter.

    Returns:
        The target unitary matrix including all the QGate's matrices (multiplied).

    """
def get_unsupport_qgate_num(qprog: QProg, support_gates: list[list[str]]) -> int:
    """get_unsupport_qgate_num(qprog: pyQPanda.QProg, support_gates: List[List[str]]) -> int

    Count the number of unsupported gates in a quantum program.

    Args:
        qprog: The quantum program to analyze.

        support_gates: A list of supported gates.

    Returns:
        int: The number of unsupported gates in the quantum program.

    """
def getstat(*args, **kwargs):
    """getstat() -> QPanda::QMachineStatus

    Get the status of the Quantum machine

    Args:
        None

    Returns:
        The status of the Quantum machine, see QMachineStatus.

    """
@overload
def iSWAP(first_qubit: Qubit, second_qubit: Qubit) -> QGate:
    """iSWAP(*args, **kwargs)
    Overloaded function.

    1. iSWAP(first_qubit: pyQPanda.Qubit, second_qubit: pyQPanda.Qubit) -> pyQPanda.QGate

    Returns:
        a iSWAP gate node
    Raises:
        run_fail: An error occurred in construct gate node


    2. iSWAP(first_qubit_list: pyQPanda.QVec, second_qubit_list: pyQPanda.QVec) -> pyQPanda.QCircuit

    Returns:
        a iSWAP gate node
    Raises:
        run_fail: An error occurred in construct gate node


    3. iSWAP(first_qubit_addr: int, second_qubit_addr: int) -> pyQPanda.QGate

    4. iSWAP(first_qubit_addr_list: List[int], second_qubit_addr_list: List[int]) -> pyQPanda.QCircuit

    Returns:
        a iSWAP gate node
    Raises:
        run_fail: An error occurred in construct gate node


    5. iSWAP(first_qubit: pyQPanda.Qubit, second_qubit: pyQPanda.Qubit, theta_angle: float) -> pyQPanda.QGate

    Returns:
        a iSWAP gate node
    Raises:
        run_fail: An error occurred in construct gate node


    6. iSWAP(first_qubit_list: pyQPanda.QVec, second_qubit_list: pyQPanda.QVec, theta_angle: float) -> pyQPanda.QCircuit

    Returns:
        a iSWAP gate node
    Raises:
        run_fail: An error occurred in construct gate node


    7. iSWAP(first_qubit_addr: int, second_qubit_addr: int, theta_angle: float) -> pyQPanda.QGate

    Returns:
        a iSWAP gate node
    Raises:
        run_fail: An error occurred in construct gate node


    8. iSWAP(first_qubit_addr_list: List[int], second_qubit_addr_list: List[int], theta_angle: float) -> pyQPanda.QCircuit

    Returns:
        a iSWAP gate node
    Raises:
        run_fail: An error occurred in construct gate node

    """
@overload
def iSWAP(first_qubit_list: QVec, second_qubit_list: QVec) -> QCircuit:
    """iSWAP(*args, **kwargs)
    Overloaded function.

    1. iSWAP(first_qubit: pyQPanda.Qubit, second_qubit: pyQPanda.Qubit) -> pyQPanda.QGate

    Returns:
        a iSWAP gate node
    Raises:
        run_fail: An error occurred in construct gate node


    2. iSWAP(first_qubit_list: pyQPanda.QVec, second_qubit_list: pyQPanda.QVec) -> pyQPanda.QCircuit

    Returns:
        a iSWAP gate node
    Raises:
        run_fail: An error occurred in construct gate node


    3. iSWAP(first_qubit_addr: int, second_qubit_addr: int) -> pyQPanda.QGate

    4. iSWAP(first_qubit_addr_list: List[int], second_qubit_addr_list: List[int]) -> pyQPanda.QCircuit

    Returns:
        a iSWAP gate node
    Raises:
        run_fail: An error occurred in construct gate node


    5. iSWAP(first_qubit: pyQPanda.Qubit, second_qubit: pyQPanda.Qubit, theta_angle: float) -> pyQPanda.QGate

    Returns:
        a iSWAP gate node
    Raises:
        run_fail: An error occurred in construct gate node


    6. iSWAP(first_qubit_list: pyQPanda.QVec, second_qubit_list: pyQPanda.QVec, theta_angle: float) -> pyQPanda.QCircuit

    Returns:
        a iSWAP gate node
    Raises:
        run_fail: An error occurred in construct gate node


    7. iSWAP(first_qubit_addr: int, second_qubit_addr: int, theta_angle: float) -> pyQPanda.QGate

    Returns:
        a iSWAP gate node
    Raises:
        run_fail: An error occurred in construct gate node


    8. iSWAP(first_qubit_addr_list: List[int], second_qubit_addr_list: List[int], theta_angle: float) -> pyQPanda.QCircuit

    Returns:
        a iSWAP gate node
    Raises:
        run_fail: An error occurred in construct gate node

    """
@overload
def iSWAP(first_qubit_addr: int, second_qubit_addr: int) -> QGate:
    """iSWAP(*args, **kwargs)
    Overloaded function.

    1. iSWAP(first_qubit: pyQPanda.Qubit, second_qubit: pyQPanda.Qubit) -> pyQPanda.QGate

    Returns:
        a iSWAP gate node
    Raises:
        run_fail: An error occurred in construct gate node


    2. iSWAP(first_qubit_list: pyQPanda.QVec, second_qubit_list: pyQPanda.QVec) -> pyQPanda.QCircuit

    Returns:
        a iSWAP gate node
    Raises:
        run_fail: An error occurred in construct gate node


    3. iSWAP(first_qubit_addr: int, second_qubit_addr: int) -> pyQPanda.QGate

    4. iSWAP(first_qubit_addr_list: List[int], second_qubit_addr_list: List[int]) -> pyQPanda.QCircuit

    Returns:
        a iSWAP gate node
    Raises:
        run_fail: An error occurred in construct gate node


    5. iSWAP(first_qubit: pyQPanda.Qubit, second_qubit: pyQPanda.Qubit, theta_angle: float) -> pyQPanda.QGate

    Returns:
        a iSWAP gate node
    Raises:
        run_fail: An error occurred in construct gate node


    6. iSWAP(first_qubit_list: pyQPanda.QVec, second_qubit_list: pyQPanda.QVec, theta_angle: float) -> pyQPanda.QCircuit

    Returns:
        a iSWAP gate node
    Raises:
        run_fail: An error occurred in construct gate node


    7. iSWAP(first_qubit_addr: int, second_qubit_addr: int, theta_angle: float) -> pyQPanda.QGate

    Returns:
        a iSWAP gate node
    Raises:
        run_fail: An error occurred in construct gate node


    8. iSWAP(first_qubit_addr_list: List[int], second_qubit_addr_list: List[int], theta_angle: float) -> pyQPanda.QCircuit

    Returns:
        a iSWAP gate node
    Raises:
        run_fail: An error occurred in construct gate node

    """
@overload
def iSWAP(first_qubit_addr_list: list[int], second_qubit_addr_list: list[int]) -> QCircuit:
    """iSWAP(*args, **kwargs)
    Overloaded function.

    1. iSWAP(first_qubit: pyQPanda.Qubit, second_qubit: pyQPanda.Qubit) -> pyQPanda.QGate

    Returns:
        a iSWAP gate node
    Raises:
        run_fail: An error occurred in construct gate node


    2. iSWAP(first_qubit_list: pyQPanda.QVec, second_qubit_list: pyQPanda.QVec) -> pyQPanda.QCircuit

    Returns:
        a iSWAP gate node
    Raises:
        run_fail: An error occurred in construct gate node


    3. iSWAP(first_qubit_addr: int, second_qubit_addr: int) -> pyQPanda.QGate

    4. iSWAP(first_qubit_addr_list: List[int], second_qubit_addr_list: List[int]) -> pyQPanda.QCircuit

    Returns:
        a iSWAP gate node
    Raises:
        run_fail: An error occurred in construct gate node


    5. iSWAP(first_qubit: pyQPanda.Qubit, second_qubit: pyQPanda.Qubit, theta_angle: float) -> pyQPanda.QGate

    Returns:
        a iSWAP gate node
    Raises:
        run_fail: An error occurred in construct gate node


    6. iSWAP(first_qubit_list: pyQPanda.QVec, second_qubit_list: pyQPanda.QVec, theta_angle: float) -> pyQPanda.QCircuit

    Returns:
        a iSWAP gate node
    Raises:
        run_fail: An error occurred in construct gate node


    7. iSWAP(first_qubit_addr: int, second_qubit_addr: int, theta_angle: float) -> pyQPanda.QGate

    Returns:
        a iSWAP gate node
    Raises:
        run_fail: An error occurred in construct gate node


    8. iSWAP(first_qubit_addr_list: List[int], second_qubit_addr_list: List[int], theta_angle: float) -> pyQPanda.QCircuit

    Returns:
        a iSWAP gate node
    Raises:
        run_fail: An error occurred in construct gate node

    """
@overload
def iSWAP(first_qubit: Qubit, second_qubit: Qubit, theta_angle: float) -> QGate:
    """iSWAP(*args, **kwargs)
    Overloaded function.

    1. iSWAP(first_qubit: pyQPanda.Qubit, second_qubit: pyQPanda.Qubit) -> pyQPanda.QGate

    Returns:
        a iSWAP gate node
    Raises:
        run_fail: An error occurred in construct gate node


    2. iSWAP(first_qubit_list: pyQPanda.QVec, second_qubit_list: pyQPanda.QVec) -> pyQPanda.QCircuit

    Returns:
        a iSWAP gate node
    Raises:
        run_fail: An error occurred in construct gate node


    3. iSWAP(first_qubit_addr: int, second_qubit_addr: int) -> pyQPanda.QGate

    4. iSWAP(first_qubit_addr_list: List[int], second_qubit_addr_list: List[int]) -> pyQPanda.QCircuit

    Returns:
        a iSWAP gate node
    Raises:
        run_fail: An error occurred in construct gate node


    5. iSWAP(first_qubit: pyQPanda.Qubit, second_qubit: pyQPanda.Qubit, theta_angle: float) -> pyQPanda.QGate

    Returns:
        a iSWAP gate node
    Raises:
        run_fail: An error occurred in construct gate node


    6. iSWAP(first_qubit_list: pyQPanda.QVec, second_qubit_list: pyQPanda.QVec, theta_angle: float) -> pyQPanda.QCircuit

    Returns:
        a iSWAP gate node
    Raises:
        run_fail: An error occurred in construct gate node


    7. iSWAP(first_qubit_addr: int, second_qubit_addr: int, theta_angle: float) -> pyQPanda.QGate

    Returns:
        a iSWAP gate node
    Raises:
        run_fail: An error occurred in construct gate node


    8. iSWAP(first_qubit_addr_list: List[int], second_qubit_addr_list: List[int], theta_angle: float) -> pyQPanda.QCircuit

    Returns:
        a iSWAP gate node
    Raises:
        run_fail: An error occurred in construct gate node

    """
@overload
def iSWAP(first_qubit_list: QVec, second_qubit_list: QVec, theta_angle: float) -> QCircuit:
    """iSWAP(*args, **kwargs)
    Overloaded function.

    1. iSWAP(first_qubit: pyQPanda.Qubit, second_qubit: pyQPanda.Qubit) -> pyQPanda.QGate

    Returns:
        a iSWAP gate node
    Raises:
        run_fail: An error occurred in construct gate node


    2. iSWAP(first_qubit_list: pyQPanda.QVec, second_qubit_list: pyQPanda.QVec) -> pyQPanda.QCircuit

    Returns:
        a iSWAP gate node
    Raises:
        run_fail: An error occurred in construct gate node


    3. iSWAP(first_qubit_addr: int, second_qubit_addr: int) -> pyQPanda.QGate

    4. iSWAP(first_qubit_addr_list: List[int], second_qubit_addr_list: List[int]) -> pyQPanda.QCircuit

    Returns:
        a iSWAP gate node
    Raises:
        run_fail: An error occurred in construct gate node


    5. iSWAP(first_qubit: pyQPanda.Qubit, second_qubit: pyQPanda.Qubit, theta_angle: float) -> pyQPanda.QGate

    Returns:
        a iSWAP gate node
    Raises:
        run_fail: An error occurred in construct gate node


    6. iSWAP(first_qubit_list: pyQPanda.QVec, second_qubit_list: pyQPanda.QVec, theta_angle: float) -> pyQPanda.QCircuit

    Returns:
        a iSWAP gate node
    Raises:
        run_fail: An error occurred in construct gate node


    7. iSWAP(first_qubit_addr: int, second_qubit_addr: int, theta_angle: float) -> pyQPanda.QGate

    Returns:
        a iSWAP gate node
    Raises:
        run_fail: An error occurred in construct gate node


    8. iSWAP(first_qubit_addr_list: List[int], second_qubit_addr_list: List[int], theta_angle: float) -> pyQPanda.QCircuit

    Returns:
        a iSWAP gate node
    Raises:
        run_fail: An error occurred in construct gate node

    """
@overload
def iSWAP(first_qubit_addr: int, second_qubit_addr: int, theta_angle: float) -> QGate:
    """iSWAP(*args, **kwargs)
    Overloaded function.

    1. iSWAP(first_qubit: pyQPanda.Qubit, second_qubit: pyQPanda.Qubit) -> pyQPanda.QGate

    Returns:
        a iSWAP gate node
    Raises:
        run_fail: An error occurred in construct gate node


    2. iSWAP(first_qubit_list: pyQPanda.QVec, second_qubit_list: pyQPanda.QVec) -> pyQPanda.QCircuit

    Returns:
        a iSWAP gate node
    Raises:
        run_fail: An error occurred in construct gate node


    3. iSWAP(first_qubit_addr: int, second_qubit_addr: int) -> pyQPanda.QGate

    4. iSWAP(first_qubit_addr_list: List[int], second_qubit_addr_list: List[int]) -> pyQPanda.QCircuit

    Returns:
        a iSWAP gate node
    Raises:
        run_fail: An error occurred in construct gate node


    5. iSWAP(first_qubit: pyQPanda.Qubit, second_qubit: pyQPanda.Qubit, theta_angle: float) -> pyQPanda.QGate

    Returns:
        a iSWAP gate node
    Raises:
        run_fail: An error occurred in construct gate node


    6. iSWAP(first_qubit_list: pyQPanda.QVec, second_qubit_list: pyQPanda.QVec, theta_angle: float) -> pyQPanda.QCircuit

    Returns:
        a iSWAP gate node
    Raises:
        run_fail: An error occurred in construct gate node


    7. iSWAP(first_qubit_addr: int, second_qubit_addr: int, theta_angle: float) -> pyQPanda.QGate

    Returns:
        a iSWAP gate node
    Raises:
        run_fail: An error occurred in construct gate node


    8. iSWAP(first_qubit_addr_list: List[int], second_qubit_addr_list: List[int], theta_angle: float) -> pyQPanda.QCircuit

    Returns:
        a iSWAP gate node
    Raises:
        run_fail: An error occurred in construct gate node

    """
@overload
def iSWAP(first_qubit_addr_list: list[int], second_qubit_addr_list: list[int], theta_angle: float) -> QCircuit:
    """iSWAP(*args, **kwargs)
    Overloaded function.

    1. iSWAP(first_qubit: pyQPanda.Qubit, second_qubit: pyQPanda.Qubit) -> pyQPanda.QGate

    Returns:
        a iSWAP gate node
    Raises:
        run_fail: An error occurred in construct gate node


    2. iSWAP(first_qubit_list: pyQPanda.QVec, second_qubit_list: pyQPanda.QVec) -> pyQPanda.QCircuit

    Returns:
        a iSWAP gate node
    Raises:
        run_fail: An error occurred in construct gate node


    3. iSWAP(first_qubit_addr: int, second_qubit_addr: int) -> pyQPanda.QGate

    4. iSWAP(first_qubit_addr_list: List[int], second_qubit_addr_list: List[int]) -> pyQPanda.QCircuit

    Returns:
        a iSWAP gate node
    Raises:
        run_fail: An error occurred in construct gate node


    5. iSWAP(first_qubit: pyQPanda.Qubit, second_qubit: pyQPanda.Qubit, theta_angle: float) -> pyQPanda.QGate

    Returns:
        a iSWAP gate node
    Raises:
        run_fail: An error occurred in construct gate node


    6. iSWAP(first_qubit_list: pyQPanda.QVec, second_qubit_list: pyQPanda.QVec, theta_angle: float) -> pyQPanda.QCircuit

    Returns:
        a iSWAP gate node
    Raises:
        run_fail: An error occurred in construct gate node


    7. iSWAP(first_qubit_addr: int, second_qubit_addr: int, theta_angle: float) -> pyQPanda.QGate

    Returns:
        a iSWAP gate node
    Raises:
        run_fail: An error occurred in construct gate node


    8. iSWAP(first_qubit_addr_list: List[int], second_qubit_addr_list: List[int], theta_angle: float) -> pyQPanda.QCircuit

    Returns:
        a iSWAP gate node
    Raises:
        run_fail: An error occurred in construct gate node

    """
def init(machine_type: QMachineType = ...) -> bool:
    """init(machine_type: pyQPanda.QMachineType = QMachineType.CPU) -> bool

    Init the global unique quantum machine at background.

    Args:
        machine_type: quantum machine type, see pyQPanda.QMachineType

    Returns:
        bool: true if initialization success

    """
def init_quantum_machine(machine_type: QMachineType = ...) -> QuantumMachine:
    """init_quantum_machine(machine_type: pyQPanda.QMachineType = QMachineType.CPU,) -> pyQPanda.QuantumMachine

    Create and initialize a new quantum machine, and let it be a globally unique quantum machine.

    Args:
        machine_type: quantum machine type, see pyQPanda.QMachineType.

    Returns:
        object: the quantum machine, type depends on machine_type:

            QMachineType.CPU               --> pyQPanda.CPUQVM

            QMachineType.CPU_SINGLE_THREAD --> pyQPanda.CPUSingleThreadQVM

            QMachineType.GPU               --> pyQPanda.GPUQVM (if pyQPanda is built with GPU)

            QMachineType.NOISE             --> pyQPanda.NoiseQVM

            return None if initialization fails.

    """
def inverse(arg0: var) -> var:
    """inverse(arg0: pyQPanda.var) -> pyQPanda.var"""
def isCarry(arg0: QVec, arg1: QVec, arg2: Qubit, arg3: Qubit) -> QCircuit:
    """isCarry(arg0: pyQPanda.QVec, arg1: pyQPanda.QVec, arg2: pyQPanda.Qubit, arg3: pyQPanda.Qubit) -> pyQPanda.QCircuit

    Construct a circuit to determine if there is a carry
    """
def is_match_topology(gate: QGate, topo: list[list[float]]) -> bool:
    """is_match_topology(gate: pyQPanda.QGate, topo: List[List[float]]) -> bool

    Judge if the QGate matches the target topologic structure of the quantum circuit.

    Args:
        gate (QGate): The quantum gate to evaluate.

        topo: The target topologic structure of the quantum circuit.

    Returns:
        bool: True if it matches, otherwise false.

    """
def is_supported_qgate_type(nodeitr: NodeIter) -> bool:
    """is_supported_qgate_type(nodeitr: pyQPanda.NodeIter) -> bool

    Judge if the target node is a QGate type.

    Args:
        nodeitr: Node iterator in the quantum program.

    Returns:
        bool: True if the target node is a QGate type, otherwise false.

    """
def is_swappable(prog: QProg, nodeitr_1: NodeIter, nodeitr_2: NodeIter) -> bool:
    """is_swappable(prog: pyQPanda.QProg, nodeitr_1: pyQPanda.NodeIter, nodeitr_2: pyQPanda.NodeIter) -> bool

    Judge whether the specified two NodeIters in the quantum program can be exchanged.

    Args:
        prog: Target quantum program.

        nodeitr_1: Node iterator 1 in the quantum program.

        nodeitr_2: Node iterator 2 in the quantum program.

    Returns:
        bool: True if the two NodeIters can be exchanged, otherwise false.

    """
def iterative_amplitude_estimation(arg0: QCircuit, arg1: QVec, arg2: float, arg3: float) -> float:
    """iterative_amplitude_estimation(arg0: pyQPanda.QCircuit, arg1: pyQPanda.QVec, arg2: float, arg3: float) -> float

    estimate the probability corresponding to the ground state |1> of the last bit
    Args:
        QCircuit: quantum circuit
        qvec: qubit list
        double: epsilon
        double: confidence

    Returns:
        result iterative amplitude
    Raises:
        run_fail: An error occurred in iterative_amplitude_estimation

    """
def ldd_decompose(qprog: QProg) -> QProg:
    """ldd_decompose(qprog: pyQPanda.QProg) -> pyQPanda.QProg

    Decompose a multiple control quantum gate using LDD.

    Args:
        qprog: the quantum program to be decomposed.

    Returns:
        the updated quantum program after decomposition.

    """
def log(arg0: var) -> var:
    """log(arg0: pyQPanda.var) -> pyQPanda.var"""
@overload
def matrix_decompose(qubits: QVec, matrix: numpy.ndarray[numpy.complex128[m, n]], mode: DecompositionMode = ..., b_positive_seq: bool = ...) -> QCircuit:
    """matrix_decompose(*args, **kwargs)
    Overloaded function.

    1. matrix_decompose(qubits: pyQPanda.QVec, matrix: numpy.ndarray[numpy.complex128[m, n]], mode: pyQPanda.DecompositionMode = DecompositionMode.QSD, b_positive_seq: bool = True) -> pyQPanda.QCircuit

    Matrix decomposition

    Args:
        qubits: the used qubits

        matrix: The target matrix

        mode: DecompositionMode decomposition mode, default is QSD

        b_positive_seq: true for positive sequence(q0q1q2), false for inverted order(q2q1q0), default is true

    Returns:
        QCircuit The quantum circuit for target matrix

    2. matrix_decompose(qubits: pyQPanda.QVec, matrix: List[complex], mode: pyQPanda.DecompositionMode = DecompositionMode.QSD, b_positive_seq: bool = True) -> pyQPanda.QCircuit

    Matrix decomposition

    Args:
        qubits: the used qubits

        matrix: The target matrix

        mode: DecompositionMode decomposition mode, default is QSD

        b_positive_seq: true for positive sequence(q0q1q2), false for inverted order(q2q1q0), default is true

    Returns:
        QCircuit The quantum circuit for target matrix
    """
@overload
def matrix_decompose(qubits: QVec, matrix: list[complex], mode: DecompositionMode = ..., b_positive_seq: bool = ...) -> QCircuit:
    """matrix_decompose(*args, **kwargs)
    Overloaded function.

    1. matrix_decompose(qubits: pyQPanda.QVec, matrix: numpy.ndarray[numpy.complex128[m, n]], mode: pyQPanda.DecompositionMode = DecompositionMode.QSD, b_positive_seq: bool = True) -> pyQPanda.QCircuit

    Matrix decomposition

    Args:
        qubits: the used qubits

        matrix: The target matrix

        mode: DecompositionMode decomposition mode, default is QSD

        b_positive_seq: true for positive sequence(q0q1q2), false for inverted order(q2q1q0), default is true

    Returns:
        QCircuit The quantum circuit for target matrix

    2. matrix_decompose(qubits: pyQPanda.QVec, matrix: List[complex], mode: pyQPanda.DecompositionMode = DecompositionMode.QSD, b_positive_seq: bool = True) -> pyQPanda.QCircuit

    Matrix decomposition

    Args:
        qubits: the used qubits

        matrix: The target matrix

        mode: DecompositionMode decomposition mode, default is QSD

        b_positive_seq: true for positive sequence(q0q1q2), false for inverted order(q2q1q0), default is true

    Returns:
        QCircuit The quantum circuit for target matrix
    """
@overload
def matrix_decompose_paulis(arg0: QuantumMachine, arg1: numpy.ndarray[numpy.float64[m, n]]) -> list[tuple[float, QCircuit]]:
    """matrix_decompose_paulis(*args, **kwargs)
    Overloaded function.

    1. matrix_decompose_paulis(arg0: pyQPanda.QuantumMachine, arg1: numpy.ndarray[numpy.float64[m, n]]) -> List[Tuple[float, pyQPanda.QCircuit]]

    decompose matrix into paulis combination

    Args:
        quantum_machine: quantum machine

        matrix: 2^N *2^N double matrix 

    Returns:
        result : linearcom contains pauli circuit

    2. matrix_decompose_paulis(arg0: pyQPanda.QVec, arg1: numpy.ndarray[numpy.float64[m, n]]) -> List[Tuple[float, pyQPanda.QCircuit]]

    decompose matrix into paulis combination

    Args:
        quantum_machine: quantum machine
        matrix: 2^N *2^N double matrix 

    Returns:
        result : linearcom contains pauli circuit
    """
@overload
def matrix_decompose_paulis(arg0: QVec, arg1: numpy.ndarray[numpy.float64[m, n]]) -> list[tuple[float, QCircuit]]:
    """matrix_decompose_paulis(*args, **kwargs)
    Overloaded function.

    1. matrix_decompose_paulis(arg0: pyQPanda.QuantumMachine, arg1: numpy.ndarray[numpy.float64[m, n]]) -> List[Tuple[float, pyQPanda.QCircuit]]

    decompose matrix into paulis combination

    Args:
        quantum_machine: quantum machine

        matrix: 2^N *2^N double matrix 

    Returns:
        result : linearcom contains pauli circuit

    2. matrix_decompose_paulis(arg0: pyQPanda.QVec, arg1: numpy.ndarray[numpy.float64[m, n]]) -> List[Tuple[float, pyQPanda.QCircuit]]

    decompose matrix into paulis combination

    Args:
        quantum_machine: quantum machine
        matrix: 2^N *2^N double matrix 

    Returns:
        result : linearcom contains pauli circuit
    """
@overload
def measure_all(qubit_list: QVec, cbit_list: list[ClassicalCondition]) -> QProg:
    """measure_all(*args, **kwargs)
    Overloaded function.

    1. measure_all(qubit_list: pyQPanda.QVec, cbit_list: List[pyQPanda.ClassicalCondition]) -> pyQPanda.QProg

    Create a list of measure nodes.

    Args:
        qubit_list: the qubits to be measured.

        cbit_list: classical bits that store the quantum measurement results.

    Returns:
        a list of measure nodes.


    2. measure_all(qubit_addr_list: List[int], cbit_addr_list: List[int]) -> pyQPanda.QProg

    Create a list of measure nodes.

    Args:
        qubit_addr_list: list of addresses of the qubits to be measured.

        cbit_addr_list: list of addresses of the classical bits that store the quantum measurement results.

    Returns:
        a list of measure nodes.

    """
@overload
def measure_all(qubit_addr_list: list[int], cbit_addr_list: list[int]) -> QProg:
    """measure_all(*args, **kwargs)
    Overloaded function.

    1. measure_all(qubit_list: pyQPanda.QVec, cbit_list: List[pyQPanda.ClassicalCondition]) -> pyQPanda.QProg

    Create a list of measure nodes.

    Args:
        qubit_list: the qubits to be measured.

        cbit_list: classical bits that store the quantum measurement results.

    Returns:
        a list of measure nodes.


    2. measure_all(qubit_addr_list: List[int], cbit_addr_list: List[int]) -> pyQPanda.QProg

    Create a list of measure nodes.

    Args:
        qubit_addr_list: list of addresses of the qubits to be measured.

        cbit_addr_list: list of addresses of the classical bits that store the quantum measurement results.

    Returns:
        a list of measure nodes.

    """
@overload
def mul(arg0: ClassicalCondition, arg1: ClassicalCondition) -> ClassicalCondition:
    """mul(*args, **kwargs)
    Overloaded function.

    1. mul(arg0: pyQPanda.ClassicalCondition, arg1: pyQPanda.ClassicalCondition) -> pyQPanda.ClassicalCondition

    Multiply two ClassicalConditions.

    Args:
        a: The first ClassicalCondition.

        b: The second ClassicalCondition to multiply with.

    Returns:
        ClassicalCondition: The result of the multiplication.


    2. mul(arg0: pyQPanda.ClassicalCondition, arg1: int) -> pyQPanda.ClassicalCondition

    Multiply a ClassicalCondition by a bit size.

    Args:
        a: The ClassicalCondition to be multiplied.

        b: The bit size to multiply with the ClassicalCondition.

    Returns:
        ClassicalCondition: The resulting ClassicalCondition after multiplication.


    3. mul(arg0: int, arg1: pyQPanda.ClassicalCondition) -> pyQPanda.ClassicalCondition

    Multiply a bit size by a ClassicalCondition.

    Args:
        a: The bit size to be multiplied.

        b: The ClassicalCondition to multiply with the bit size.

    Returns:
        ClassicalCondition: The resulting ClassicalCondition after multiplication.

    """
@overload
def mul(arg0: ClassicalCondition, arg1: int) -> ClassicalCondition:
    """mul(*args, **kwargs)
    Overloaded function.

    1. mul(arg0: pyQPanda.ClassicalCondition, arg1: pyQPanda.ClassicalCondition) -> pyQPanda.ClassicalCondition

    Multiply two ClassicalConditions.

    Args:
        a: The first ClassicalCondition.

        b: The second ClassicalCondition to multiply with.

    Returns:
        ClassicalCondition: The result of the multiplication.


    2. mul(arg0: pyQPanda.ClassicalCondition, arg1: int) -> pyQPanda.ClassicalCondition

    Multiply a ClassicalCondition by a bit size.

    Args:
        a: The ClassicalCondition to be multiplied.

        b: The bit size to multiply with the ClassicalCondition.

    Returns:
        ClassicalCondition: The resulting ClassicalCondition after multiplication.


    3. mul(arg0: int, arg1: pyQPanda.ClassicalCondition) -> pyQPanda.ClassicalCondition

    Multiply a bit size by a ClassicalCondition.

    Args:
        a: The bit size to be multiplied.

        b: The ClassicalCondition to multiply with the bit size.

    Returns:
        ClassicalCondition: The resulting ClassicalCondition after multiplication.

    """
@overload
def mul(arg0: int, arg1: ClassicalCondition) -> ClassicalCondition:
    """mul(*args, **kwargs)
    Overloaded function.

    1. mul(arg0: pyQPanda.ClassicalCondition, arg1: pyQPanda.ClassicalCondition) -> pyQPanda.ClassicalCondition

    Multiply two ClassicalConditions.

    Args:
        a: The first ClassicalCondition.

        b: The second ClassicalCondition to multiply with.

    Returns:
        ClassicalCondition: The result of the multiplication.


    2. mul(arg0: pyQPanda.ClassicalCondition, arg1: int) -> pyQPanda.ClassicalCondition

    Multiply a ClassicalCondition by a bit size.

    Args:
        a: The ClassicalCondition to be multiplied.

        b: The bit size to multiply with the ClassicalCondition.

    Returns:
        ClassicalCondition: The resulting ClassicalCondition after multiplication.


    3. mul(arg0: int, arg1: pyQPanda.ClassicalCondition) -> pyQPanda.ClassicalCondition

    Multiply a bit size by a ClassicalCondition.

    Args:
        a: The bit size to be multiplied.

        b: The ClassicalCondition to multiply with the bit size.

    Returns:
        ClassicalCondition: The resulting ClassicalCondition after multiplication.

    """
def originir_to_qprog(file_path: str, machine: QuantumMachine) -> QProg:
    """originir_to_qprog(file_path: str, machine: pyQPanda.QuantumMachine) -> pyQPanda.QProg

    Read an OriginIR file and transform it into QProg.

    Args:
        file_path: OriginIR file path.

        machine: initialized quantum machine.

    Returns:
        Transformed QProg.

    """
def pauli_combination_replace(arg0: list[tuple[float, QCircuit]], arg1: QuantumMachine, arg2: str, arg3: str) -> list[tuple[float, QCircuit]]:
    """pauli_combination_replace(arg0: List[Tuple[float, pyQPanda.QCircuit]], arg1: pyQPanda.QuantumMachine, arg2: str, arg3: str) -> List[Tuple[float, pyQPanda.QCircuit]]"""
def planarity_testing(topo_data: list[list[int]]) -> bool:
    """planarity_testing(topo_data: List[List[int]]) -> bool

    Perform planarity testing.

    Args:
        topo_data: Quantum program topology data.

    Returns:
        Result data.

    """
def pmeasure(qubit_list: QVec, select_max: int) -> list[tuple[int, float]]:
    """pmeasure(qubit_list: pyQPanda.QVec, select_max: int) -> List[Tuple[int, float]]

    Get the probability distribution over qubits.

    Args:
        qubit_list: qubit list to measure.

        select_max: max returned element num in returned tuple, should be in [-1, 1<<len(qubit_list)];

        default is -1, means no limit.

    Returns:
        Measure result of quantum machine in tuple form.

    """
def pmeasure_no_index(qubit_list: QVec) -> list[float]:
    """pmeasure_no_index(qubit_list: pyQPanda.QVec) -> List[float]

    Get the probability distribution over qubits.

    Args:
        qubit_list: qubit list to measure.

    Returns:
        Measure result of quantum machine in list form.

    """
def poly(arg0: var, arg1: var) -> var:
    """poly(arg0: pyQPanda.var, arg1: pyQPanda.var) -> pyQPanda.var"""
def print_matrix(matrix: list[complex], precision: int = ...) -> str:
    """print_matrix(matrix: List[complex], precision: int = 8) -> str

    Print matrix elements.

    Args:
        matrix (QStat): The matrix to print.

        precision (int, optional): Double value to string cutoff precision (default is 8).

    Returns:
        A string representation of the matrix.

    """
def prob_run_dict(qprog: QProg, qubit_list: QVec, select_max: int = ...) -> dict[str, float]:
    """prob_run_dict(qprog: pyQPanda.QProg, qubit_list: pyQPanda.QVec, select_max: int = -1) -> Dict[str, float]

    Run quantum program and get pmeasure result as dict

    Args:
        qprog: quantum program.

        qubit_list: pmeasure qubits list.

        select_max: max returned element num in returned tuple, should be in [-1, 1<<len(qubit_list)],

        default is -1, meaning no limit.

    Returns:
        Measure result of quantum machine.

    """
def prob_run_list(qprog: QProg, qubit_list: QVec, select_max: int = ...) -> list[float]:
    """prob_run_list(qprog: pyQPanda.QProg, qubit_list: pyQPanda.QVec, select_max: int = -1) -> List[float]

    Run quantum program and get pmeasure result as list

    Args:
        qprog: quantum program.

        qubit_list: pmeasure qubits list.

        select_max: max returned element num in returned tuple, should be in [-1, 1<<len(qubit_list)],

        default is -1, meaning no limit.

    Returns:
        Measure result of quantum machine.

    """
def prob_run_tuple_list(qptog: QProg, qubit_list: QVec, select_max: int = ...) -> list[tuple[int, float]]:
    """prob_run_tuple_list(qptog: pyQPanda.QProg, qubit_list: pyQPanda.QVec, select_max: int = -1) -> List[Tuple[int, float]]

    Run quantum program and get pmeasure result as tuple list

    Args:
        qprog: quantum program.

        qubit_list: pmeasure qubits list.

        select_max: max returned element num in returned tuple, should be in [-1, 1<<len(qubit_list)],

        default is -1, meaning no limit.

    Returns:
        Measure result of quantum machine.

    """
def prog_layer(*args, **kwargs):
    """prog_layer(prog: pyQPanda.QProg) -> QPanda::TopologSequence<std::shared_ptr<QPanda::OptimizerNodeInfo> >

    Process the given quantum program layer.

    Args:
        prog: The quantum program to be processed.

    Returns:
        Processed quantum program layer.

    """
def prog_to_dag(prog: QProg) -> QProgDAG:
    """prog_to_dag(prog: pyQPanda.QProg) -> pyQPanda.QProgDAG

    Convert a quantum program into a directed acyclic graph (DAG).

    Args:
         prog: The quantum program to be converted.

    Returns:
         QProgDAG: A reference to the resulting DAG.

    """
@overload
def qAlloc() -> Qubit:
    """qAlloc(*args, **kwargs)
    Overloaded function.

    1. qAlloc() -> pyQPanda.Qubit

    Create a qubit
    After init()

    Args:
        None: No parameters are required for this function.

    Returns:
        Qubit: A new qubit.

        None: If the quantum machine has created the maximum number of qubits, which is 29.


    2. qAlloc(qubit_addr: int) -> pyQPanda.Qubit

    Allocate a qubit
    After init()

    Args:
        qubit_addr: The physical address of the qubit, should be in [0, 29).

    Returns:
        Qubit: A new qubit.

        None: If qubit_addr is invalid or if the maximum number of allowed qubits has been reached.

    """
@overload
def qAlloc(qubit_addr: int) -> Qubit:
    """qAlloc(*args, **kwargs)
    Overloaded function.

    1. qAlloc() -> pyQPanda.Qubit

    Create a qubit
    After init()

    Args:
        None: No parameters are required for this function.

    Returns:
        Qubit: A new qubit.

        None: If the quantum machine has created the maximum number of qubits, which is 29.


    2. qAlloc(qubit_addr: int) -> pyQPanda.Qubit

    Allocate a qubit
    After init()

    Args:
        qubit_addr: The physical address of the qubit, should be in [0, 29).

    Returns:
        Qubit: A new qubit.

        None: If qubit_addr is invalid or if the maximum number of allowed qubits has been reached.

    """
def qAlloc_many(qubit_num: int) -> list[Qubit]:
    """qAlloc_many(qubit_num: int) -> List[pyQPanda.Qubit]

    Allocate several qubits
    After init()

    Args:
        qubit_num: The number of qubits to be created.

    Returns:
        list[pyQPanda.Qubit]: list of qubit.

    """
def qFree(qubit: Qubit) -> None:
    """qFree(qubit: pyQPanda.Qubit) -> None

    Free a qubit

    Args:
        qubit: a qubit

    Returns:
        None

    """
@overload
def qFree_all() -> None:
    """qFree_all(*args, **kwargs)
    Overloaded function.

    1. qFree_all() -> None

    Free all qubits

    Args:
        None

    Returns:
        None


    2. qFree_all(qubit_list: pyQPanda.QVec) -> None

    Free a list of qubits

    Args:
        qubit_list: A list of qubits to be freed.

    Returns:
        None

    """
@overload
def qFree_all(qubit_list: QVec) -> None:
    """qFree_all(*args, **kwargs)
    Overloaded function.

    1. qFree_all() -> None

    Free all qubits

    Args:
        None

    Returns:
        None


    2. qFree_all(qubit_list: pyQPanda.QVec) -> None

    Free a list of qubits

    Args:
        qubit_list: A list of qubits to be freed.

    Returns:
        None

    """
@overload
def qop(VariationalQuantumCircuit: VariationalQuantumCircuit, Hamiltonian, QuantumMachine: QuantumMachine, qubitList: list[Qubit]) -> var:
    """qop(*args, **kwargs)
    Overloaded function.

    1. qop(VariationalQuantumCircuit: pyQPanda.VariationalQuantumCircuit, Hamiltonian: QPanda::PauliOp<std::complex<double> >, QuantumMachine: pyQPanda.QuantumMachine, qubitList: List[pyQPanda.Qubit]) -> pyQPanda.var

    2. qop(VariationalQuantumCircuit: pyQPanda.VariationalQuantumCircuit, Hamiltonian: QPanda::PauliOp<std::complex<double> >, QuantumMachine: pyQPanda.QuantumMachine, qubitList: Dict[int, pyQPanda.Qubit]) -> pyQPanda.var
    """
@overload
def qop(VariationalQuantumCircuit: VariationalQuantumCircuit, Hamiltonian, QuantumMachine: QuantumMachine, qubitList: dict[int, Qubit]) -> var:
    """qop(*args, **kwargs)
    Overloaded function.

    1. qop(VariationalQuantumCircuit: pyQPanda.VariationalQuantumCircuit, Hamiltonian: QPanda::PauliOp<std::complex<double> >, QuantumMachine: pyQPanda.QuantumMachine, qubitList: List[pyQPanda.Qubit]) -> pyQPanda.var

    2. qop(VariationalQuantumCircuit: pyQPanda.VariationalQuantumCircuit, Hamiltonian: QPanda::PauliOp<std::complex<double> >, QuantumMachine: pyQPanda.QuantumMachine, qubitList: Dict[int, pyQPanda.Qubit]) -> pyQPanda.var
    """
def qop_pmeasure(arg0: VariationalQuantumCircuit, arg1: list[int], arg2: QuantumMachine, arg3: list[Qubit]) -> var:
    """qop_pmeasure(arg0: pyQPanda.VariationalQuantumCircuit, arg1: List[int], arg2: pyQPanda.QuantumMachine, arg3: List[pyQPanda.Qubit]) -> pyQPanda.var"""
def quantum_chip_adapter(qprog: QProg, machine: QuantumMachine, mapping: bool = ..., config_file: str = ...) -> list:
    """quantum_chip_adapter(qprog: pyQPanda.QProg, machine: pyQPanda.QuantumMachine, mapping: bool = True, config_file: str = 'QPandaConfig.json') -> list

    Perform adaptive conversion for the quantum chip.

    Args:
        qprog: the quantum program.

        machine: the quantum machine to be used.

        mapping: whether to perform the mapping operation (default is true).

        config_file: configuration file path (default is CONFIG_PATH).

    Returns:
        a list containing the quantum program and the list of qubits after mapping; if mapping is false, the qubit list may be misoperated.

    """
def quantum_walk_alg(*args, **kwargs):
    """quantum_walk_alg(data: List[int], Classical_condition: pyQPanda.ClassicalCondition, QuantumMachine: pyQPanda.QuantumMachine, qlist: pyQPanda.QVec, data: int = 2) -> pyQPanda.QProg

    Build quantum-walk algorithm quantum circuit
    """
def quantum_walk_search(list: list[int], Classical_condition: ClassicalCondition, QuantumMachine: QuantumMachine, data: int = ...) -> list:
    """quantum_walk_search(list: List[int], Classical_condition: pyQPanda.ClassicalCondition, QuantumMachine: pyQPanda.QuantumMachine, data: int = 2) -> list

    Use Quantum-walk Algorithm to search target data, return QProg and search_result

    Args:
        list: data list
        Classical_condition: quantum Classical condition
        QuantumMachine: quantum machine
        repeat: search repeat times

    Returns:
        result : Quantum-walk search result
    Raises:
        run_fail: An error occurred in Quantum-walk

    """
def quick_measure(qubit_list: QVec, shots: int) -> dict[str, int]:
    """quick_measure(qubit_list: pyQPanda.QVec, shots: int) -> Dict[str, int]

    Quick measure.

    Args:
        qubit_list: qubit list to measure.

        shots: the repeat number of measure operations.

    Returns:
        result: result of quantum program execution.

    """
def random_qcircuit(qvec: QVec, depth: int = ..., gate_type: list[str] = ...) -> QCircuit:
    """random_qcircuit(qvec: pyQPanda.QVec, depth: int = 100, gate_type: List[str] = []) -> pyQPanda.QCircuit

    Generate a random quantum circuit.

    Args:
        qvec: Output circuits for the random circuit.

        depth: Circuit depth (default is 100).

        gate_type: Types of gates to use (default is an empty vector).

    Returns:
        A random quantum circuit.

    """
def random_qprog(qubit_row: int, qubit_col: int, depth: int, qvm: QuantumMachine, qvec: QVec) -> QProg:
    """random_qprog(qubit_row: int, qubit_col: int, depth: int, qvm: pyQPanda.QuantumMachine, qvec: pyQPanda.QVec) -> pyQPanda.QProg

    Generate a random quantum program.

    Args:
        qubit_row: Circuit qubit row value.

        qubit_col: Circuit qubit column value.

        depth: Circuit depth.

        qvm: Quantum machine.

        qvec: Output circuits for the random quantum program.

    Returns:
        A random quantum program.

    """
def recover_edges(topo_data: list[list[int]], max_connect_degree: int, candidate_edges: list[tuple[int, list[int]]]) -> list[list[int]]:
    """recover_edges(topo_data: List[List[int]], max_connect_degree: int, candidate_edges: List[Tuple[int, List[int]]]) -> List[List[int]]

    Recover edges using the specified candidate edges.

    Args:
        topo_data: The topology data of the quantum program.

        max_connect_degree: The maximum allowed connection degree.

        candidate_edges: A list of edges to consider for recovery.

    Returns:
        The updated topology data after recovery.

    """
def remap(prog: QProg, target_qlist: QVec, target_clist: list[ClassicalCondition] = ...) -> QProg:
    """remap(prog: pyQPanda.QProg, target_qlist: pyQPanda.QVec, target_clist: List[pyQPanda.ClassicalCondition] = []) -> pyQPanda.QProg

    Map the source quantum program to the target qubits.

    Args:
        prog: Source quantum program.

        target_qlist: Target qubits.

        target_clist: Target classical bits (default is an empty vector).

    Returns:
        The target quantum program.

    """
def replace_complex_points(src_topo_data: list[list[int]], max_connect_degree: int, sub_topo_vec: list[tuple[int, list[list[int]]]]) -> None:
    """replace_complex_points(src_topo_data: List[List[int]], max_connect_degree: int, sub_topo_vec: List[Tuple[int, List[List[int]]]]) -> None

    Replace complex points in the source topology with subgraphs.

    Args:
        src_topo_data: The source topology data of the quantum program.

        max_connect_degree: The maximum allowable connection degree.

        sub_topo_vec: A list of sub-topologies to replace the complex points.

    Returns:
        None: This function modifies the source topology in place.

    """
def run_with_configuration(*args, **kwargs):
    """run_with_configuration(*args, **kwargs)
    Overloaded function.

    1. run_with_configuration(program: pyQPanda.QProg, cbit_list: List[pyQPanda.ClassicalCondition], shots: int, noise_model: pyQPanda.Noise = NoiseModel()) -> Dict[str, int]

    Run quantum program with configuration

    Args:
        program: quantum program.

        cbit_list: classical cbits list.

        shots: number of times to repeat the quantum program.

        noise_model: noise model; defaults to no noise. Noise model only works on CPUQVM now.

    Returns:
        Result of quantum program execution in shots.
      First is the final qubit register state, second is its hit count.


    2. run_with_configuration(program: pyQPanda.QProg, shots: int, noise_model: pyQPanda.Noise = NoiseModel()) -> Dict[str, int]

    Run quantum program with configuration.

    Args:
        program: quantum program.

        shots: repeat run quantum program times.

        noise_model: noise model, default is no noise. Noise model only works on CPUQVM now.

    Returns:
        tuple: result of quantum program execution in shots.

        First is the final qubit register state, second is its hit shot.

    """
@overload
def sabre_mapping(prog: QProg, quantum_machine: QuantumMachine, init_map: QVec, max_look_ahead: int = ..., max_iterations: int = ..., config_data: str = ..., m_max_random_mappings: int = ..., hops: int = ...) -> QProg:
    """sabre_mapping(*args, **kwargs)
    Overloaded function.

    1. sabre_mapping(prog: pyQPanda.QProg, quantum_machine: pyQPanda.QuantumMachine, init_map: pyQPanda.QVec, max_look_ahead: int = 20, max_iterations: int = 10, config_data: str = 'QPandaConfig.json', m_max_random_mappings: int = 10000, hops: int = 1) -> pyQPanda.QProg

    sabre mapping

    Args:
        prog: the target prog

        quantum_machine: quantum machine

        init_map: init map

        max_look_ahead: sabre_mapping max_look_ahead

        max_iterations: sabre_mapping max_iterations

        config_data: config data, @See JsonConfigParam::load_config()
        m_max_random_mappings: m_max_random_mappings data
        hops: hops data

    Returns:
        mapped quantum program

    2. sabre_mapping(prog: pyQPanda.QProg, quantum_machine: pyQPanda.QuantumMachine, max_look_ahead: int = 20, max_iterations: int = 10, config_data: str = 'QPandaConfig.json') -> pyQPanda.QProg

    sabre mapping

    Args:
        prog: the target prog

        quantum_machine: quantum machine

        max_look_ahead: sabre_mapping max_look_ahead

        max_iterations: sabre_mapping max_iterations

        config_data: config data, @See JsonConfigParam::load_config()

    Returns:
        mapped quantum program

    3. sabre_mapping(prog: pyQPanda.QProg, quantum_machine: pyQPanda.QuantumMachine, max_look_ahead: int, max_iterations: int, arch_matrix: numpy.ndarray[numpy.float64[m, n]]) -> pyQPanda.QProg

    sabre mapping

    Args:
        prog: the target prog

        quantum_machine: quantum machine

        max_look_ahead: sabre_mapping max_look_ahead, default is 20 

        max_iterations: sabre_mapping max_iterations, default is 10 

        arch_matrix: arch matrix

    Returns:
        mapped quantum program
    """
@overload
def sabre_mapping(prog: QProg, quantum_machine: QuantumMachine, max_look_ahead: int = ..., max_iterations: int = ..., config_data: str = ...) -> QProg:
    """sabre_mapping(*args, **kwargs)
    Overloaded function.

    1. sabre_mapping(prog: pyQPanda.QProg, quantum_machine: pyQPanda.QuantumMachine, init_map: pyQPanda.QVec, max_look_ahead: int = 20, max_iterations: int = 10, config_data: str = 'QPandaConfig.json', m_max_random_mappings: int = 10000, hops: int = 1) -> pyQPanda.QProg

    sabre mapping

    Args:
        prog: the target prog

        quantum_machine: quantum machine

        init_map: init map

        max_look_ahead: sabre_mapping max_look_ahead

        max_iterations: sabre_mapping max_iterations

        config_data: config data, @See JsonConfigParam::load_config()
        m_max_random_mappings: m_max_random_mappings data
        hops: hops data

    Returns:
        mapped quantum program

    2. sabre_mapping(prog: pyQPanda.QProg, quantum_machine: pyQPanda.QuantumMachine, max_look_ahead: int = 20, max_iterations: int = 10, config_data: str = 'QPandaConfig.json') -> pyQPanda.QProg

    sabre mapping

    Args:
        prog: the target prog

        quantum_machine: quantum machine

        max_look_ahead: sabre_mapping max_look_ahead

        max_iterations: sabre_mapping max_iterations

        config_data: config data, @See JsonConfigParam::load_config()

    Returns:
        mapped quantum program

    3. sabre_mapping(prog: pyQPanda.QProg, quantum_machine: pyQPanda.QuantumMachine, max_look_ahead: int, max_iterations: int, arch_matrix: numpy.ndarray[numpy.float64[m, n]]) -> pyQPanda.QProg

    sabre mapping

    Args:
        prog: the target prog

        quantum_machine: quantum machine

        max_look_ahead: sabre_mapping max_look_ahead, default is 20 

        max_iterations: sabre_mapping max_iterations, default is 10 

        arch_matrix: arch matrix

    Returns:
        mapped quantum program
    """
@overload
def sabre_mapping(prog: QProg, quantum_machine: QuantumMachine, max_look_ahead: int, max_iterations: int, arch_matrix: numpy.ndarray[numpy.float64[m, n]]) -> QProg:
    """sabre_mapping(*args, **kwargs)
    Overloaded function.

    1. sabre_mapping(prog: pyQPanda.QProg, quantum_machine: pyQPanda.QuantumMachine, init_map: pyQPanda.QVec, max_look_ahead: int = 20, max_iterations: int = 10, config_data: str = 'QPandaConfig.json', m_max_random_mappings: int = 10000, hops: int = 1) -> pyQPanda.QProg

    sabre mapping

    Args:
        prog: the target prog

        quantum_machine: quantum machine

        init_map: init map

        max_look_ahead: sabre_mapping max_look_ahead

        max_iterations: sabre_mapping max_iterations

        config_data: config data, @See JsonConfigParam::load_config()
        m_max_random_mappings: m_max_random_mappings data
        hops: hops data

    Returns:
        mapped quantum program

    2. sabre_mapping(prog: pyQPanda.QProg, quantum_machine: pyQPanda.QuantumMachine, max_look_ahead: int = 20, max_iterations: int = 10, config_data: str = 'QPandaConfig.json') -> pyQPanda.QProg

    sabre mapping

    Args:
        prog: the target prog

        quantum_machine: quantum machine

        max_look_ahead: sabre_mapping max_look_ahead

        max_iterations: sabre_mapping max_iterations

        config_data: config data, @See JsonConfigParam::load_config()

    Returns:
        mapped quantum program

    3. sabre_mapping(prog: pyQPanda.QProg, quantum_machine: pyQPanda.QuantumMachine, max_look_ahead: int, max_iterations: int, arch_matrix: numpy.ndarray[numpy.float64[m, n]]) -> pyQPanda.QProg

    sabre mapping

    Args:
        prog: the target prog

        quantum_machine: quantum machine

        max_look_ahead: sabre_mapping max_look_ahead, default is 20 

        max_iterations: sabre_mapping max_iterations, default is 10 

        arch_matrix: arch matrix

    Returns:
        mapped quantum program
    """
def sigmoid(arg0: var) -> var:
    """sigmoid(arg0: pyQPanda.var) -> pyQPanda.var"""
def sin(arg0: var) -> var:
    """sin(arg0: pyQPanda.var) -> pyQPanda.var"""
@overload
def single_qubit_rb(qvm: QuantumMachine, qubit: Qubit, clifford_range: list[int], num_circuits: int, shots: int, chip_id: int = ..., interleaved_gates: list[QGate] = ...) -> dict[int, float]:
    """single_qubit_rb(*args, **kwargs)
    Overloaded function.

    1. single_qubit_rb(qvm: pyQPanda.QuantumMachine, qubit: pyQPanda.Qubit, clifford_range: List[int], num_circuits: int, shots: int, chip_id: int = 2, interleaved_gates: List[pyQPanda.QGate] = []) -> Dict[int, float]

    Single qubit rb with WU YUAN chip

    Args:
        qvm: quantum machine
        qubit: single qubit
        clifford_range: clifford range list
        num_circuits: the num of circuits
        shots: measure shots
        chip type: RealChipType
        interleaved_gates: interleaved gates list

    Returns:
        result data dict
    Raises:
        run_fail: An error occurred in single_qubit_rb


    2. single_qubit_rb(config: pyQPanda.QCloudTaskConfig, qubit: int, clifford_range: List[int], num_circuits: int, interleaved_gates: List[pyQPanda.QGate] = []) -> Dict[int, float]

    Single qubit rb with origin chip

    Args:
        config: quantum QCloudTaskConfig
        qubit: single qubit
        clifford_range: clifford range list
        num_circuits: the num of circuits
        interleaved_gates: interleaved gates list

    Returns:
        result data dict
    Raises:
        run_fail: An error occurred in single_qubit_rb

    """
@overload
def single_qubit_rb(config: QCloudTaskConfig, qubit: int, clifford_range: list[int], num_circuits: int, interleaved_gates: list[QGate] = ...) -> dict[int, float]:
    """single_qubit_rb(*args, **kwargs)
    Overloaded function.

    1. single_qubit_rb(qvm: pyQPanda.QuantumMachine, qubit: pyQPanda.Qubit, clifford_range: List[int], num_circuits: int, shots: int, chip_id: int = 2, interleaved_gates: List[pyQPanda.QGate] = []) -> Dict[int, float]

    Single qubit rb with WU YUAN chip

    Args:
        qvm: quantum machine
        qubit: single qubit
        clifford_range: clifford range list
        num_circuits: the num of circuits
        shots: measure shots
        chip type: RealChipType
        interleaved_gates: interleaved gates list

    Returns:
        result data dict
    Raises:
        run_fail: An error occurred in single_qubit_rb


    2. single_qubit_rb(config: pyQPanda.QCloudTaskConfig, qubit: int, clifford_range: List[int], num_circuits: int, interleaved_gates: List[pyQPanda.QGate] = []) -> Dict[int, float]

    Single qubit rb with origin chip

    Args:
        config: quantum QCloudTaskConfig
        qubit: single qubit
        clifford_range: clifford range list
        num_circuits: the num of circuits
        interleaved_gates: interleaved gates list

    Returns:
        result data dict
    Raises:
        run_fail: An error occurred in single_qubit_rb

    """
def softmax(arg0: var) -> var:
    """softmax(arg0: pyQPanda.var) -> pyQPanda.var"""
def split_complex_points(complex_points: list[int], max_connect_degree: int, topo_data: list[list[int]], split_method: ComplexVertexSplitMethod = ...) -> list[tuple[int, list[list[int]]]]:
    """split_complex_points(complex_points: List[int], max_connect_degree: int, topo_data: List[List[int]], split_method: pyQPanda.ComplexVertexSplitMethod = ComplexVertexSplitMethod.LINEAR) -> List[Tuple[int, List[List[int]]]]

    Split complex points into multiple discrete points.

    Args:
        complex_points: A list of complex points to be split.

        max_connect_degree: The maximum allowable connection degree.

        topo_data: The topology data of the quantum program.

        split_method: Method for splitting, as defined in ComplexVertexSplitMethod. Defaults to ComplexVertexSplitMethod.LINEAR.

    Returns:
        None: The function modifies the input data in place.

    """
def stack(arg0: int, *args) -> var:
    """stack(arg0: int, *args) -> pyQPanda.var"""
@overload
def state_fidelity(state1: list[complex], state2: list[complex]) -> float:
    """state_fidelity(*args, **kwargs)
    Overloaded function.

    1. state_fidelity(state1: List[complex], state2: List[complex]) -> float

    Compare two quantum states and calculate their fidelity.

    Args:
        state1: first quantum state represented as a list.

        state2: second quantum state represented as a list.

    Returns:
        The fidelity between the two states, a value in the range [0, 1].


    2. state_fidelity(matrix1: List[List[complex]], matrix2: List[List[complex]]) -> float

    Compare two quantum state matrices and calculate their fidelity.

       Args:
           matrix1: first quantum state matrix.

           matrix2: second quantum state matrix.

       Returns:
           The fidelity between the two matrices, a value in the range [0, 1].


    3. state_fidelity(state1: List[complex], state2: List[List[complex]]) -> float

    Compare a quantum state with a state matrix and calculate their fidelity.

    Args:
        state: a single quantum state represented as a list.

        matrix: a quantum state matrix.

    Returns:
        The fidelity between the state and the matrix, a value in the range [0, 1].


    4. state_fidelity(state1: List[List[complex]], state2: List[complex]) -> float

    Compare a quantum state matrix with a quantum state and calculate their fidelity.

    Args:
        matrix: a quantum state matrix.

        state: a single quantum state represented as a list.

    Returns:
        The fidelity between the matrix and the state, a value in the range [0, 1].

    """
@overload
def state_fidelity(matrix1: list[list[complex]], matrix2: list[list[complex]]) -> float:
    """state_fidelity(*args, **kwargs)
    Overloaded function.

    1. state_fidelity(state1: List[complex], state2: List[complex]) -> float

    Compare two quantum states and calculate their fidelity.

    Args:
        state1: first quantum state represented as a list.

        state2: second quantum state represented as a list.

    Returns:
        The fidelity between the two states, a value in the range [0, 1].


    2. state_fidelity(matrix1: List[List[complex]], matrix2: List[List[complex]]) -> float

    Compare two quantum state matrices and calculate their fidelity.

       Args:
           matrix1: first quantum state matrix.

           matrix2: second quantum state matrix.

       Returns:
           The fidelity between the two matrices, a value in the range [0, 1].


    3. state_fidelity(state1: List[complex], state2: List[List[complex]]) -> float

    Compare a quantum state with a state matrix and calculate their fidelity.

    Args:
        state: a single quantum state represented as a list.

        matrix: a quantum state matrix.

    Returns:
        The fidelity between the state and the matrix, a value in the range [0, 1].


    4. state_fidelity(state1: List[List[complex]], state2: List[complex]) -> float

    Compare a quantum state matrix with a quantum state and calculate their fidelity.

    Args:
        matrix: a quantum state matrix.

        state: a single quantum state represented as a list.

    Returns:
        The fidelity between the matrix and the state, a value in the range [0, 1].

    """
@overload
def state_fidelity(state1: list[complex], state2: list[list[complex]]) -> float:
    """state_fidelity(*args, **kwargs)
    Overloaded function.

    1. state_fidelity(state1: List[complex], state2: List[complex]) -> float

    Compare two quantum states and calculate their fidelity.

    Args:
        state1: first quantum state represented as a list.

        state2: second quantum state represented as a list.

    Returns:
        The fidelity between the two states, a value in the range [0, 1].


    2. state_fidelity(matrix1: List[List[complex]], matrix2: List[List[complex]]) -> float

    Compare two quantum state matrices and calculate their fidelity.

       Args:
           matrix1: first quantum state matrix.

           matrix2: second quantum state matrix.

       Returns:
           The fidelity between the two matrices, a value in the range [0, 1].


    3. state_fidelity(state1: List[complex], state2: List[List[complex]]) -> float

    Compare a quantum state with a state matrix and calculate their fidelity.

    Args:
        state: a single quantum state represented as a list.

        matrix: a quantum state matrix.

    Returns:
        The fidelity between the state and the matrix, a value in the range [0, 1].


    4. state_fidelity(state1: List[List[complex]], state2: List[complex]) -> float

    Compare a quantum state matrix with a quantum state and calculate their fidelity.

    Args:
        matrix: a quantum state matrix.

        state: a single quantum state represented as a list.

    Returns:
        The fidelity between the matrix and the state, a value in the range [0, 1].

    """
@overload
def state_fidelity(state1: list[list[complex]], state2: list[complex]) -> float:
    """state_fidelity(*args, **kwargs)
    Overloaded function.

    1. state_fidelity(state1: List[complex], state2: List[complex]) -> float

    Compare two quantum states and calculate their fidelity.

    Args:
        state1: first quantum state represented as a list.

        state2: second quantum state represented as a list.

    Returns:
        The fidelity between the two states, a value in the range [0, 1].


    2. state_fidelity(matrix1: List[List[complex]], matrix2: List[List[complex]]) -> float

    Compare two quantum state matrices and calculate their fidelity.

       Args:
           matrix1: first quantum state matrix.

           matrix2: second quantum state matrix.

       Returns:
           The fidelity between the two matrices, a value in the range [0, 1].


    3. state_fidelity(state1: List[complex], state2: List[List[complex]]) -> float

    Compare a quantum state with a state matrix and calculate their fidelity.

    Args:
        state: a single quantum state represented as a list.

        matrix: a quantum state matrix.

    Returns:
        The fidelity between the state and the matrix, a value in the range [0, 1].


    4. state_fidelity(state1: List[List[complex]], state2: List[complex]) -> float

    Compare a quantum state matrix with a quantum state and calculate their fidelity.

    Args:
        matrix: a quantum state matrix.

        state: a single quantum state represented as a list.

    Returns:
        The fidelity between the matrix and the state, a value in the range [0, 1].

    """
@overload
def sub(arg0: ClassicalCondition, arg1: ClassicalCondition) -> ClassicalCondition:
    """sub(*args, **kwargs)
    Overloaded function.

    1. sub(arg0: pyQPanda.ClassicalCondition, arg1: pyQPanda.ClassicalCondition) -> pyQPanda.ClassicalCondition

    Subtract one ClassicalCondition from another.

    Args:
        a: The ClassicalCondition to subtract from.

        b: The ClassicalCondition to subtract.

    Returns:
        ClassicalCondition: The result of the subtraction.


    2. sub(arg0: pyQPanda.ClassicalCondition, arg1: int) -> pyQPanda.ClassicalCondition

    Subtract a bit size from a ClassicalCondition.

    Args:
        a: The ClassicalCondition from which the bit size will be subtracted.

        b: The bit size to subtract from the ClassicalCondition.

    Returns:
        ClassicalCondition: The resulting ClassicalCondition after subtraction.


    3. sub(arg0: int, arg1: pyQPanda.ClassicalCondition) -> pyQPanda.ClassicalCondition

    Subtract a ClassicalCondition from a bit size.

    Args:
        a: The bit size to subtract from.

        b: The ClassicalCondition to be subtracted.

    Returns:
        ClassicalCondition: The resulting ClassicalCondition after subtraction.

    """
@overload
def sub(arg0: ClassicalCondition, arg1: int) -> ClassicalCondition:
    """sub(*args, **kwargs)
    Overloaded function.

    1. sub(arg0: pyQPanda.ClassicalCondition, arg1: pyQPanda.ClassicalCondition) -> pyQPanda.ClassicalCondition

    Subtract one ClassicalCondition from another.

    Args:
        a: The ClassicalCondition to subtract from.

        b: The ClassicalCondition to subtract.

    Returns:
        ClassicalCondition: The result of the subtraction.


    2. sub(arg0: pyQPanda.ClassicalCondition, arg1: int) -> pyQPanda.ClassicalCondition

    Subtract a bit size from a ClassicalCondition.

    Args:
        a: The ClassicalCondition from which the bit size will be subtracted.

        b: The bit size to subtract from the ClassicalCondition.

    Returns:
        ClassicalCondition: The resulting ClassicalCondition after subtraction.


    3. sub(arg0: int, arg1: pyQPanda.ClassicalCondition) -> pyQPanda.ClassicalCondition

    Subtract a ClassicalCondition from a bit size.

    Args:
        a: The bit size to subtract from.

        b: The ClassicalCondition to be subtracted.

    Returns:
        ClassicalCondition: The resulting ClassicalCondition after subtraction.

    """
@overload
def sub(arg0: int, arg1: ClassicalCondition) -> ClassicalCondition:
    """sub(*args, **kwargs)
    Overloaded function.

    1. sub(arg0: pyQPanda.ClassicalCondition, arg1: pyQPanda.ClassicalCondition) -> pyQPanda.ClassicalCondition

    Subtract one ClassicalCondition from another.

    Args:
        a: The ClassicalCondition to subtract from.

        b: The ClassicalCondition to subtract.

    Returns:
        ClassicalCondition: The result of the subtraction.


    2. sub(arg0: pyQPanda.ClassicalCondition, arg1: int) -> pyQPanda.ClassicalCondition

    Subtract a bit size from a ClassicalCondition.

    Args:
        a: The ClassicalCondition from which the bit size will be subtracted.

        b: The bit size to subtract from the ClassicalCondition.

    Returns:
        ClassicalCondition: The resulting ClassicalCondition after subtraction.


    3. sub(arg0: int, arg1: pyQPanda.ClassicalCondition) -> pyQPanda.ClassicalCondition

    Subtract a ClassicalCondition from a bit size.

    Args:
        a: The bit size to subtract from.

        b: The ClassicalCondition to be subtracted.

    Returns:
        ClassicalCondition: The resulting ClassicalCondition after subtraction.

    """
def sum(arg0: var) -> var:
    """sum(arg0: pyQPanda.var) -> pyQPanda.var"""
def tan(arg0: var) -> var:
    """tan(arg0: pyQPanda.var) -> pyQPanda.var"""
def to_Quil(qprog: QProg, machine: QuantumMachine) -> str:
    """to_Quil(qprog: pyQPanda.QProg, machine: pyQPanda.QuantumMachine) -> str

    Transform QProg to Quil instruction.

    Args:
        qprog: quantum program (QProg).

        machine: quantum machine (QuantumMachine*).

    Returns:
        Quil instruction string.

    """
@overload
def to_originir(qprog: QProg, machine: QuantumMachine) -> str:
    """to_originir(*args, **kwargs)
    Overloaded function.

    1. to_originir(qprog: pyQPanda.QProg, machine: pyQPanda.QuantumMachine) -> str

    Transform QProg to OriginIR string.

    Args:
         qprog: T, the quantum program or circuit to transform.

         machine: QuantumMachine*, the quantum machine being used.

    Returns:
         OriginIR string, the transformed representation of the quantum program.


    2. to_originir(qprog: pyQPanda.QCircuit, machine: pyQPanda.QuantumMachine) -> str

    Transform QProg to OriginIR string.

    Args:
         qprog: T, the quantum program or circuit to transform.

         machine: QuantumMachine*, the quantum machine being used.

    Returns:
         OriginIR string, the transformed representation of the quantum program.


    3. to_originir(qprog: pyQPanda.QGate, machine: pyQPanda.QuantumMachine) -> str

    Transform QProg to OriginIR string.

    Args:
         qprog: T, the quantum program or circuit to transform.

         machine: QuantumMachine*, the quantum machine being used.

    Returns:
         OriginIR string, the transformed representation of the quantum program.


    4. to_originir(qprog: pyQPanda.QIfProg, machine: pyQPanda.QuantumMachine) -> str

    Transform QProg to OriginIR string.

    Args:
         qprog: T, the quantum program or circuit to transform.

         machine: QuantumMachine*, the quantum machine being used.

    Returns:
         OriginIR string, the transformed representation of the quantum program.


    5. to_originir(qprog: pyQPanda.QWhileProg, machine: pyQPanda.QuantumMachine) -> str

    Transform QProg to OriginIR string.

    Args:
         qprog: T, the quantum program or circuit to transform.

         machine: QuantumMachine*, the quantum machine being used.

    Returns:
         OriginIR string, the transformed representation of the quantum program.


    6. to_originir(qprog: pyQPanda.QMeasure, machine: pyQPanda.QuantumMachine) -> str

    Transform QProg to OriginIR string.

    Args:
         qprog: T, the quantum program or circuit to transform.

         machine: QuantumMachine*, the quantum machine being used.

    Returns:
         OriginIR string, the transformed representation of the quantum program.

    """
@overload
def to_originir(qprog: QCircuit, machine: QuantumMachine) -> str:
    """to_originir(*args, **kwargs)
    Overloaded function.

    1. to_originir(qprog: pyQPanda.QProg, machine: pyQPanda.QuantumMachine) -> str

    Transform QProg to OriginIR string.

    Args:
         qprog: T, the quantum program or circuit to transform.

         machine: QuantumMachine*, the quantum machine being used.

    Returns:
         OriginIR string, the transformed representation of the quantum program.


    2. to_originir(qprog: pyQPanda.QCircuit, machine: pyQPanda.QuantumMachine) -> str

    Transform QProg to OriginIR string.

    Args:
         qprog: T, the quantum program or circuit to transform.

         machine: QuantumMachine*, the quantum machine being used.

    Returns:
         OriginIR string, the transformed representation of the quantum program.


    3. to_originir(qprog: pyQPanda.QGate, machine: pyQPanda.QuantumMachine) -> str

    Transform QProg to OriginIR string.

    Args:
         qprog: T, the quantum program or circuit to transform.

         machine: QuantumMachine*, the quantum machine being used.

    Returns:
         OriginIR string, the transformed representation of the quantum program.


    4. to_originir(qprog: pyQPanda.QIfProg, machine: pyQPanda.QuantumMachine) -> str

    Transform QProg to OriginIR string.

    Args:
         qprog: T, the quantum program or circuit to transform.

         machine: QuantumMachine*, the quantum machine being used.

    Returns:
         OriginIR string, the transformed representation of the quantum program.


    5. to_originir(qprog: pyQPanda.QWhileProg, machine: pyQPanda.QuantumMachine) -> str

    Transform QProg to OriginIR string.

    Args:
         qprog: T, the quantum program or circuit to transform.

         machine: QuantumMachine*, the quantum machine being used.

    Returns:
         OriginIR string, the transformed representation of the quantum program.


    6. to_originir(qprog: pyQPanda.QMeasure, machine: pyQPanda.QuantumMachine) -> str

    Transform QProg to OriginIR string.

    Args:
         qprog: T, the quantum program or circuit to transform.

         machine: QuantumMachine*, the quantum machine being used.

    Returns:
         OriginIR string, the transformed representation of the quantum program.

    """
@overload
def to_originir(qprog: QGate, machine: QuantumMachine) -> str:
    """to_originir(*args, **kwargs)
    Overloaded function.

    1. to_originir(qprog: pyQPanda.QProg, machine: pyQPanda.QuantumMachine) -> str

    Transform QProg to OriginIR string.

    Args:
         qprog: T, the quantum program or circuit to transform.

         machine: QuantumMachine*, the quantum machine being used.

    Returns:
         OriginIR string, the transformed representation of the quantum program.


    2. to_originir(qprog: pyQPanda.QCircuit, machine: pyQPanda.QuantumMachine) -> str

    Transform QProg to OriginIR string.

    Args:
         qprog: T, the quantum program or circuit to transform.

         machine: QuantumMachine*, the quantum machine being used.

    Returns:
         OriginIR string, the transformed representation of the quantum program.


    3. to_originir(qprog: pyQPanda.QGate, machine: pyQPanda.QuantumMachine) -> str

    Transform QProg to OriginIR string.

    Args:
         qprog: T, the quantum program or circuit to transform.

         machine: QuantumMachine*, the quantum machine being used.

    Returns:
         OriginIR string, the transformed representation of the quantum program.


    4. to_originir(qprog: pyQPanda.QIfProg, machine: pyQPanda.QuantumMachine) -> str

    Transform QProg to OriginIR string.

    Args:
         qprog: T, the quantum program or circuit to transform.

         machine: QuantumMachine*, the quantum machine being used.

    Returns:
         OriginIR string, the transformed representation of the quantum program.


    5. to_originir(qprog: pyQPanda.QWhileProg, machine: pyQPanda.QuantumMachine) -> str

    Transform QProg to OriginIR string.

    Args:
         qprog: T, the quantum program or circuit to transform.

         machine: QuantumMachine*, the quantum machine being used.

    Returns:
         OriginIR string, the transformed representation of the quantum program.


    6. to_originir(qprog: pyQPanda.QMeasure, machine: pyQPanda.QuantumMachine) -> str

    Transform QProg to OriginIR string.

    Args:
         qprog: T, the quantum program or circuit to transform.

         machine: QuantumMachine*, the quantum machine being used.

    Returns:
         OriginIR string, the transformed representation of the quantum program.

    """
@overload
def to_originir(qprog: QIfProg, machine: QuantumMachine) -> str:
    """to_originir(*args, **kwargs)
    Overloaded function.

    1. to_originir(qprog: pyQPanda.QProg, machine: pyQPanda.QuantumMachine) -> str

    Transform QProg to OriginIR string.

    Args:
         qprog: T, the quantum program or circuit to transform.

         machine: QuantumMachine*, the quantum machine being used.

    Returns:
         OriginIR string, the transformed representation of the quantum program.


    2. to_originir(qprog: pyQPanda.QCircuit, machine: pyQPanda.QuantumMachine) -> str

    Transform QProg to OriginIR string.

    Args:
         qprog: T, the quantum program or circuit to transform.

         machine: QuantumMachine*, the quantum machine being used.

    Returns:
         OriginIR string, the transformed representation of the quantum program.


    3. to_originir(qprog: pyQPanda.QGate, machine: pyQPanda.QuantumMachine) -> str

    Transform QProg to OriginIR string.

    Args:
         qprog: T, the quantum program or circuit to transform.

         machine: QuantumMachine*, the quantum machine being used.

    Returns:
         OriginIR string, the transformed representation of the quantum program.


    4. to_originir(qprog: pyQPanda.QIfProg, machine: pyQPanda.QuantumMachine) -> str

    Transform QProg to OriginIR string.

    Args:
         qprog: T, the quantum program or circuit to transform.

         machine: QuantumMachine*, the quantum machine being used.

    Returns:
         OriginIR string, the transformed representation of the quantum program.


    5. to_originir(qprog: pyQPanda.QWhileProg, machine: pyQPanda.QuantumMachine) -> str

    Transform QProg to OriginIR string.

    Args:
         qprog: T, the quantum program or circuit to transform.

         machine: QuantumMachine*, the quantum machine being used.

    Returns:
         OriginIR string, the transformed representation of the quantum program.


    6. to_originir(qprog: pyQPanda.QMeasure, machine: pyQPanda.QuantumMachine) -> str

    Transform QProg to OriginIR string.

    Args:
         qprog: T, the quantum program or circuit to transform.

         machine: QuantumMachine*, the quantum machine being used.

    Returns:
         OriginIR string, the transformed representation of the quantum program.

    """
@overload
def to_originir(qprog: QWhileProg, machine: QuantumMachine) -> str:
    """to_originir(*args, **kwargs)
    Overloaded function.

    1. to_originir(qprog: pyQPanda.QProg, machine: pyQPanda.QuantumMachine) -> str

    Transform QProg to OriginIR string.

    Args:
         qprog: T, the quantum program or circuit to transform.

         machine: QuantumMachine*, the quantum machine being used.

    Returns:
         OriginIR string, the transformed representation of the quantum program.


    2. to_originir(qprog: pyQPanda.QCircuit, machine: pyQPanda.QuantumMachine) -> str

    Transform QProg to OriginIR string.

    Args:
         qprog: T, the quantum program or circuit to transform.

         machine: QuantumMachine*, the quantum machine being used.

    Returns:
         OriginIR string, the transformed representation of the quantum program.


    3. to_originir(qprog: pyQPanda.QGate, machine: pyQPanda.QuantumMachine) -> str

    Transform QProg to OriginIR string.

    Args:
         qprog: T, the quantum program or circuit to transform.

         machine: QuantumMachine*, the quantum machine being used.

    Returns:
         OriginIR string, the transformed representation of the quantum program.


    4. to_originir(qprog: pyQPanda.QIfProg, machine: pyQPanda.QuantumMachine) -> str

    Transform QProg to OriginIR string.

    Args:
         qprog: T, the quantum program or circuit to transform.

         machine: QuantumMachine*, the quantum machine being used.

    Returns:
         OriginIR string, the transformed representation of the quantum program.


    5. to_originir(qprog: pyQPanda.QWhileProg, machine: pyQPanda.QuantumMachine) -> str

    Transform QProg to OriginIR string.

    Args:
         qprog: T, the quantum program or circuit to transform.

         machine: QuantumMachine*, the quantum machine being used.

    Returns:
         OriginIR string, the transformed representation of the quantum program.


    6. to_originir(qprog: pyQPanda.QMeasure, machine: pyQPanda.QuantumMachine) -> str

    Transform QProg to OriginIR string.

    Args:
         qprog: T, the quantum program or circuit to transform.

         machine: QuantumMachine*, the quantum machine being used.

    Returns:
         OriginIR string, the transformed representation of the quantum program.

    """
@overload
def to_originir(qprog: QMeasure, machine: QuantumMachine) -> str:
    """to_originir(*args, **kwargs)
    Overloaded function.

    1. to_originir(qprog: pyQPanda.QProg, machine: pyQPanda.QuantumMachine) -> str

    Transform QProg to OriginIR string.

    Args:
         qprog: T, the quantum program or circuit to transform.

         machine: QuantumMachine*, the quantum machine being used.

    Returns:
         OriginIR string, the transformed representation of the quantum program.


    2. to_originir(qprog: pyQPanda.QCircuit, machine: pyQPanda.QuantumMachine) -> str

    Transform QProg to OriginIR string.

    Args:
         qprog: T, the quantum program or circuit to transform.

         machine: QuantumMachine*, the quantum machine being used.

    Returns:
         OriginIR string, the transformed representation of the quantum program.


    3. to_originir(qprog: pyQPanda.QGate, machine: pyQPanda.QuantumMachine) -> str

    Transform QProg to OriginIR string.

    Args:
         qprog: T, the quantum program or circuit to transform.

         machine: QuantumMachine*, the quantum machine being used.

    Returns:
         OriginIR string, the transformed representation of the quantum program.


    4. to_originir(qprog: pyQPanda.QIfProg, machine: pyQPanda.QuantumMachine) -> str

    Transform QProg to OriginIR string.

    Args:
         qprog: T, the quantum program or circuit to transform.

         machine: QuantumMachine*, the quantum machine being used.

    Returns:
         OriginIR string, the transformed representation of the quantum program.


    5. to_originir(qprog: pyQPanda.QWhileProg, machine: pyQPanda.QuantumMachine) -> str

    Transform QProg to OriginIR string.

    Args:
         qprog: T, the quantum program or circuit to transform.

         machine: QuantumMachine*, the quantum machine being used.

    Returns:
         OriginIR string, the transformed representation of the quantum program.


    6. to_originir(qprog: pyQPanda.QMeasure, machine: pyQPanda.QuantumMachine) -> str

    Transform QProg to OriginIR string.

    Args:
         qprog: T, the quantum program or circuit to transform.

         machine: QuantumMachine*, the quantum machine being used.

    Returns:
         OriginIR string, the transformed representation of the quantum program.

    """
def topology_match(qprog: QProg, qubit_list: QVec, machine: QuantumMachine, confing_file: str = ...) -> list:
    """topology_match(qprog: pyQPanda.QProg, qubit_list: pyQPanda.QVec, machine: pyQPanda.QuantumMachine, confing_file: str = 'QPandaConfig.json') -> list

    Judge whether a quantum program matches the topology of the physical qubits.

    Args:
        qprog: The quantum program to be evaluated.

        qubit_list: The list of qubits in the quantum program.

        machine: The quantum machine used for execution.

        confing_file: The configuration file path for matching (default: QPandaConfig.json).

    Returns:
        list: Contains the resulting quantum program and the qubit list.

    """
def transform_binary_data_to_qprog(machine: QuantumMachine, data: list[int]) -> QProg:
    """transform_binary_data_to_qprog(machine: pyQPanda.QuantumMachine, data: List[int]) -> pyQPanda.QProg

    Parse binary data to transform it into a quantum program.

    Args:
        machine: quantum machine.

        data: list containing binary data from transform_qprog_to_binary().

    Returns:
        QProg: the resulting quantum program.

    """
def transform_originir_to_qprog(fname: str, machine: QuantumMachine) -> QProg:
    """transform_originir_to_qprog(fname: str, machine: pyQPanda.QuantumMachine) -> pyQPanda.QProg

    Transform OriginIR instruction from a file into a QProg.

    Args:
        fname: file containing the OriginIR instructions.

        machine: the quantum machine.

    Returns:
        QProg: the resulting quantum program.

    """
@overload
def transform_qprog_to_binary(qprog: QProg, machine: QuantumMachine) -> list[int]:
    """transform_qprog_to_binary(*args, **kwargs)
    Overloaded function.

    1. transform_qprog_to_binary(qprog: pyQPanda.QProg, machine: pyQPanda.QuantumMachine) -> List[int]

    Transform quantum program to binary data.

    Args:
        qprog: quantum program (QProg).

        machine: quantum machine.

    Returns:
        binary data as a list.


    2. transform_qprog_to_binary(qprog: pyQPanda.QProg, machine: pyQPanda.QuantumMachine, fname: str) -> None

    Save quantum program to file as binary data.

    Args:
        qprog: quantum program (QProg).

        machine: quantum machine.

        fname: name of the file to save to.

    """
@overload
def transform_qprog_to_binary(qprog: QProg, machine: QuantumMachine, fname: str) -> None:
    """transform_qprog_to_binary(*args, **kwargs)
    Overloaded function.

    1. transform_qprog_to_binary(qprog: pyQPanda.QProg, machine: pyQPanda.QuantumMachine) -> List[int]

    Transform quantum program to binary data.

    Args:
        qprog: quantum program (QProg).

        machine: quantum machine.

    Returns:
        binary data as a list.


    2. transform_qprog_to_binary(qprog: pyQPanda.QProg, machine: pyQPanda.QuantumMachine, fname: str) -> None

    Save quantum program to file as binary data.

    Args:
        qprog: quantum program (QProg).

        machine: quantum machine.

        fname: name of the file to save to.

    """
def transform_qprog_to_originir(qprog: QProg, machine: QuantumMachine) -> str:
    """transform_qprog_to_originir(qprog: pyQPanda.QProg, machine: pyQPanda.QuantumMachine) -> str

    Transform a quantum program into an OriginIR instruction string.

    Args:
        qprog: the quantum program (QProg).

        machine: the quantum machine.

    Returns:
        string: the resulting OriginIR instruction string.

    """
def transform_qprog_to_quil(qprog: QProg, machine: QuantumMachine) -> str:
    """transform_qprog_to_quil(qprog: pyQPanda.QProg, machine: pyQPanda.QuantumMachine) -> str

    Transform QProg to Quil instruction.

    Args:
        qprog: quantum program (QProg).

        machine: quantum machine (QuantumMachine*).

    Returns:
        Quil instruction string.

    """
@overload
def transform_to_base_qgate(qprog: QProg, machine: QuantumMachine, config_file: str = ...) -> QProg:
    """transform_to_base_qgate(*args, **kwargs)
    Overloaded function.

    1. transform_to_base_qgate(qprog: pyQPanda.QProg, machine: pyQPanda.QuantumMachine, config_file: str = 'QPandaConfig.json') -> pyQPanda.QProg

    Convert quantum gates to basic quantum gates.

    Args:
        qprog: the quantum program to be transformed.

        machine: the quantum machine used for the transformation.

        config_file: path to the configuration file (default is CONFIG_PATH).

    Returns:
        the updated quantum program after transformation.


    2. transform_to_base_qgate(qprog: pyQPanda.QProg, machine: pyQPanda.QuantumMachine, convert_single_gates: List[str], convert_double_gates: List[str]) -> pyQPanda.QProg

    Convert quantum gates to basic gates.

    Args:
        qprog: the quantum program to transform.

        machine: the quantum machine for the transformation.

        convert_single_gates: a set of quantum single gates to convert.

        convert_double_gates: a set of quantum double gates to convert.

    Returns:
        the updated quantum program after the conversion.

    """
@overload
def transform_to_base_qgate(qprog: QProg, machine: QuantumMachine, convert_single_gates: list[str], convert_double_gates: list[str]) -> QProg:
    """transform_to_base_qgate(*args, **kwargs)
    Overloaded function.

    1. transform_to_base_qgate(qprog: pyQPanda.QProg, machine: pyQPanda.QuantumMachine, config_file: str = 'QPandaConfig.json') -> pyQPanda.QProg

    Convert quantum gates to basic quantum gates.

    Args:
        qprog: the quantum program to be transformed.

        machine: the quantum machine used for the transformation.

        config_file: path to the configuration file (default is CONFIG_PATH).

    Returns:
        the updated quantum program after transformation.


    2. transform_to_base_qgate(qprog: pyQPanda.QProg, machine: pyQPanda.QuantumMachine, convert_single_gates: List[str], convert_double_gates: List[str]) -> pyQPanda.QProg

    Convert quantum gates to basic gates.

    Args:
        qprog: the quantum program to transform.

        machine: the quantum machine for the transformation.

        convert_single_gates: a set of quantum single gates to convert.

        convert_double_gates: a set of quantum double gates to convert.

    Returns:
        the updated quantum program after the conversion.

    """
def transfrom_pauli_operator_to_matrix(arg0) -> list[complex]:
    """transfrom_pauli_operator_to_matrix(arg0: QPanda::PauliOp<std::complex<double> >) -> List[complex]

    transfrom pauli operator to matrix

    Args:
        matrix: 2^N *2^N double matrix 

    Returns:
        result : hamiltonian
    """
def transpose(arg0: var) -> var:
    """transpose(arg0: pyQPanda.var) -> pyQPanda.var"""
def validate_double_qgate_type(gate_str_list: list[str]) -> list:
    """validate_double_qgate_type(gate_str_list: List[str]) -> list

    Get valid QGates and valid double bit QGate type.

    Args:
        double_gates: A list of double gate strings.

    Returns:
        result: A list containing the validated gate type and valid double gates.

    """
def validate_single_qgate_type(gate_str_list: list[str]) -> list:
    """validate_single_qgate_type(gate_str_list: List[str]) -> list

    Get valid QGates and valid single bit QGate type.

    Args:
        single_gates: A list of single gate strings.

    Returns:
        result: A list containing the validated gate type and valid single gates.

    """
def vector_dot(x: list[float], y: list[float]) -> float:
    """vector_dot(x: List[float], y: List[float]) -> float

    Compute the inner product of two vectors.

    Args:
        x: A list representing the first vector.

        y: A list representing the second vector.

    Returns:
        dot result: The dot product of vectors x and y.

    """
def virtual_z_transform(prog: QProg, quantum_machine: QuantumMachine, b_del_rz_gate: bool = ..., config_data: str = ...) -> QProg:
    """virtual_z_transform(prog: pyQPanda.QProg, quantum_machine: pyQPanda.QuantumMachine, b_del_rz_gate: bool = False, config_data: str = 'QPandaConfig.json') -> pyQPanda.QProg

    virtual z transform

    Args:
        prog: the target prog

        quantum_machine: quantum machine

        b_del_rz_gate: whether delete the rz gate 

        config_data: config data, @See JsonConfigParam::load_config()

    Returns:
        mapped quantum program
    """
